/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <command.h>
#include <exports.h>
#include <errno.h>
#include <vsprintf.h> /* for panic() */
#include "ta_rpmb_impl.h"
#include "bootenv_impl.h"
#include "storage_offset_impl.h"
#include "ce_ops_impl.h"

int ce_ops_init(struct ce_ops *ops)
{
	if (!ops)
		return -ENOMEM;

	memset(ops, 0, sizeof(struct ce_ops));

#ifdef CE_STORAGE_USE_UBOOTENV
	ops->cu_read = ce_bootenv_dump_console_unlock_bit;
	ops->cu_write = ce_bootenv_set_console_unlock_bit;
	ops->le_read = ce_bootenv_dump_log_enable_bit;
	ops->le_write = ce_bootenv_set_log_enable_bit;
	ops->read_all = ce_bootenv_dump_all;
	ops->get_nonce = ce_bootenv_get_nonce;
#elif defined CE_STORAGE_USE_TA_RPMB
	ops->cu_read = ce_rpmb_dump_console_unlock_bit;
	ops->cu_write = ce_rpmb_set_console_unlock_bit;
	ops->le_read = ce_rpmb_dump_log_enable_bit;
	ops->le_write = ce_rpmb_set_log_enable_bit;
	ops->read_all = ce_rpmb_dump_all;
	ops->get_nonce = ce_rpmb_get_nonce;
#elif defined CE_STORAGE_USE_OFFSET
	ops->cu_read = ce_offset_dump_console_unlock_bit;
	ops->cu_write = ce_offset_set_console_unlock_bit;
	ops->le_read = ce_offset_dump_log_enable_bit;
	ops->le_write = ce_offset_set_log_enable_bit;
	ops->read_all = ce_offset_dump_all;
	ops->get_nonce = ce_offset_get_nonce;
#else
#error CE storage method is not SET !
#endif
	return 0;
}

int ce_ops_cu_read(struct ce_ops *ops)
{
	if (!ops || !ops->cu_read)
		panic("%s: invaild usage!\n", __func__);

	return ops->cu_read();
}

int ce_ops_cu_write(struct ce_ops *ops, bool enable)
{
	if (!ops || !ops->cu_write)
		panic("%s: invaild usage!\n", __func__);

	return ops->cu_write(enable);
}

int ce_ops_le_read(struct ce_ops *ops)
{
	if (!ops || !ops->le_read)
		panic("%s: invaild usage!\n", __func__);

	return ops->le_read();
}

int ce_ops_le_write(struct ce_ops *ops, bool enable)
{
	if (!ops || !ops->le_write)
		panic("%s: invaild usage!\n", __func__);

	return ops->le_write(enable);
}

int ce_ops_read_all(struct ce_ops *ops)
{
	if (!ops || !ops->read_all)
		panic("%s: invaild usage!\n", __func__);

	return ops->read_all();
}

int ce_ops_get_nonce(struct ce_ops *ops, void *out, size_t len)
{
	if (!ops || !ops->get_nonce)
		panic("%s: invaild usage!\n", __func__);

	return ops->get_nonce(out, len);
}

