/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <linux/stddef.h>
#include <environment.h>
#include <utility.h>
#include "bootenv_impl.h"

static inline int _check_env(const char *env)
{
	char *res = env_get(env);

	if (!res || res[0] == '0')
		return 0;

	return 1;
}

static inline int _set_env(const char *env, bool en)
{
	if (en)
		env_set(env, "1");
	else
		env_set(env, "0");
	env_save();

	return 0;
}

int ce_bootenv_dump_console_unlock_bit(void)
{
	return _check_env("console_unlock");
}

int ce_bootenv_dump_log_enable_bit(void)
{
	return _check_env("log_enable");
}

int ce_bootenv_set_console_unlock_bit(bool enable)
{
	printf("CE bootenv: Set CONSOLE UNLOCK bit to %s\n",
			enable ? "enable" : "disable");

	return _set_env("console_unlock", enable);
}

int ce_bootenv_set_log_enable_bit(bool enable)
{
	printf("CE bootenv: Set LOG ENABLE bit to %s\n",
			enable ? "enable" : "disable");

	return _set_env("log_enable", enable);
}

int ce_bootenv_dump_all(void)
{
	int res;

	res = ce_bootenv_dump_console_unlock_bit();
	printf("CE bootenv: Get CONSOLE UNLOCK bit %s\n",
			res ? "enable" : "disable");

	res = ce_bootenv_dump_log_enable_bit();
	printf("CE bootenv: Get LOG ENABLE bit %s\n",
			res ? "enable" : "disable");

	return 0;
}

int ce_bootenv_get_nonce(void *out, size_t len)
{
	return generate_random_number(out, len);
}

