/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <command.h>
#include <errno.h>
#include <dm/uclass.h>
#include <dm/uclass-id.h>
#include <dm/pinctrl.h>
#include <debug_impl.h>

int do_console_log(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = 0;
	struct udevice *dev = NULL;

	UBOOT_TRACE("IN\n");
	ret = uclass_get_device(UCLASS_PINCTRL, 1, &dev);
	if (ret) {
		UBOOT_ERROR("uclass_get_device failed: %d\n", ret);
		return ret;
	}

	if (!dev) {
		UBOOT_ERROR("get serial device failed\n");
		return -ENODEV;
	}

	if (argc != 2) {
		cmd_usage(cmdtp);
	} else {
		if(strncmp(argv[1], "hdmi_b", 6) == 0) {
#ifdef CONFIG_PINCTRL
			ret = pinctrl_select_state(dev, "hdmib_pm_uart0");
			if (ret) {
				UBOOT_ERROR("pinctrl select hdmib_pm_uart0 failed: %d\n", ret);
				return ret;
			}
			UBOOT_INFO("pinctrl select hdmib_pm_uart0\n");
#else
			UBOOT_ERROR("pinctrl not supported\n");
#endif
		} else {
			cmd_usage(cmdtp);
		}
	}

	UBOOT_TRACE("OK\n");
	return ret;
}

U_BOOT_CMD(
	console_log, 2, 1, do_console_log,
	"console_log  - print console log from hdmi\n",
	"command: console_log <hdmi_b>\n"
	" hdmi_b : selec hdmi port b to print console log\n"
);
