/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2021]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <command.h>

#include "debug_impl.h"

#if defined(CONFIG_REMOTEPROC_MTK_PMU_CORTEX_M4)
#include "cm4_impl.h"
#define PMU_DEVNODE "/pmu_rproc"
#endif
#if defined(CONFIG_REMOTEPROC_MTK_PMU_MCS_51)
#include "pmu_ini_impl.h"
#include "pmu_mcs51_impl.h"
#endif

int do_boot_pmu(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = 0;
#if defined(CONFIG_REMOTEPROC_MTK_PMU_CORTEX_M4) || defined(CONFIG_REMOTEPROC_MTK_PMU_MCS_51)
	unsigned int start = 0, end = 0;
#endif

	UBOOT_TRACE("do command: boot_pmu.\n");
#if defined(CONFIG_REMOTEPROC_MTK_PMU_CORTEX_M4)
	start = get_timer(0);
	ret = cm4_loader(PMU_DEVNODE);
	end = get_timer(0);
    UBOOT_BOOTTIME("[boot_pmu][cm4][ret=%d][start:%u][end:%u][total time:%u].\n",
					ret, start, end, end - start);
#endif
#if defined(CONFIG_REMOTEPROC_MTK_PMU_MCS_51)
	start = get_timer(0);
	pmu_ini_loader();
	ret = pmu_mcs51_loader();
	end = get_timer(0);
    UBOOT_BOOTTIME("[boot_pmu][mcs51][ret=%d][start:%u][end:%u][total time:%u].\n",
					ret, start, end, end - start);
#endif
	return ret;
}

U_BOOT_CMD(
	boot_pmu, 2, 1, do_boot_pmu,
	"boot coprocessor with pmu\n",
	NULL
);

int do_consult_pmu(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int ret = 0;
#if defined(CONFIG_REMOTEPROC_MTK_PMU_CORTEX_M4) || defined(CONFIG_REMOTEPROC_MTK_PMU_MCS_51)
    unsigned int start = 0, end = 0;
#endif

    UBOOT_TRACE("do command: consult_pmu.\n");
#if defined(CONFIG_REMOTEPROC_MTK_PMU_CORTEX_M4)
    start = get_timer(0);
    ret = cm4_consulter(PMU_DEVNODE);
    end = get_timer(0);
    UBOOT_BOOTTIME("[consult_pmu][cm4][ret=%d][start:%u][end:%u][total time:%u].\n",
                    ret, start, end, end - start);
#endif
#if defined(CONFIG_REMOTEPROC_MTK_PMU_MCS_51)
    start = get_timer(0);
    ret = pmu_mcs51_consulter();
    end = get_timer(0);
    UBOOT_BOOTTIME("[consult_pmu][mcs51][ret=%d][start:%u][end:%u][total time:%u].\n",
                    ret, start, end, end - start);
#endif
    return ret;
}

U_BOOT_CMD(
    consult_pmu, 2, 1, do_consult_pmu,
    "consult coprocessor with pmu\n",
    NULL
);
