/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include <stdio.h>
#include <common.h>
#include <MsTypes.h>
#include <apiPNL.h>

#include <debug_impl.h>
#include <drvMIU.h>
#include "halSIW.h"
#include <ms_utils.h>
#include "MDrvSIW.h"
#include "apiDemura.h"


MS_BOOL MApi_SIW_Enable(MS_PHYADDR phy_addr,MS_BOOL bEnable)
{
    if (bEnable == TRUE)
    {
        MDrv_SIW_Demura_AutoDownload_Setup(phy_addr, TRUE);
    }
    else
    {
        MDrv_SIW_Demura_AutoDownload_Setup(phy_addr, FALSE);
    }

    DEBUG_SIW_DEMURA("phy_addr : 0x%x\n", (uint)phy_addr);
    return TRUE;
}


MS_BOOL MApi_SIW_Init(MS_PHYADDR pBuf)
{
    MS_U64 virt_addr;
    MS_U64 data_addr;

    virt_addr = PA2BA(pBuf);

    DEBUG_SIW_DEMURA("MApi_SIW_Init Go! phy_addr 0x%lx, virt_addr 0x%lx\n", pBuf, virt_addr);

    if (init_spi_flash() != TRUE)
    {
        UBOOT_ERROR("init_spi_flash error!\n");
        MDrv_SIW_Demura_RD_Done(FALSE);
        return FALSE;
    }
    // Read flash to demura buffer
    if (read_spi_flash((MS_U8 *)virt_addr, 0, SIW_DATA_SIZE) != TRUE)
    {
        UBOOT_ERROR("read_spi_flash error!\n");
        MDrv_SIW_Demura_RD_Done(FALSE);
        return FALSE;
    }

    // lut checksum
    data_addr = virt_addr + SIW_LUT_OFFSET;
    DEBUG_SIW_DEMURA("lut checksum data_addr 0x%lX\n", data_addr);
    if (TRUE != MDrv_SIW_Demura_Checksum_CRC16((MS_U8 *)data_addr, SIW_LUT_SIZE))
    {
        UBOOT_ERROR("MDrv_SIW_Checksum_CRC16 LUT error\n");
        MDrv_SIW_Demura_RD_Done(FALSE);
        return FALSE;
    }

    // reg setting checksum
    data_addr += SIW_LUT_SIZE + SIW_LUT_CRC_SIZE;
    DEBUG_SIW_DEMURA("reg checksum data_addr 0x%lX\n", data_addr);
    if (TRUE != MDrv_SIW_Demura_Checksum_CRC16((MS_U8 *)data_addr, SIW_SPOC_REG_SIZE))
    {
        UBOOT_ERROR("MDrv_SIW_Checksum_CRC16 REG error\n");
        MDrv_SIW_Demura_RD_Done(FALSE);
        return FALSE;
    }

    MDrv_SIW_Demura_RD_Done(TRUE);

    MDrv_SIW_Demura_SPOC_Reg((MS_U8 *)data_addr);

    return TRUE;
}


