/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#include <stdio.h>
#include <common.h>
#include <MsTypes.h>
//#include <MsSystem.h>
#include <drvMIU.h>
#include <debug_impl.h>
#include <drvMMIO.h>
#include <halRegOp.h>
#include "MDrvSIW.h"
#include "halSIW.h"
#include <common.h>

void MDrv_SIW_Demura_AutoDownload_Setup(MS_PHYADDR phy_addr, MS_BOOL bEnable)
{
    DEBUG_SIW_DEMURA("MDrv_SIW_Demura_AutoDownload_Setup IN\n");
#if(TEST_SIW_DEMURA == 1)
    if (bEnable == TRUE)
    {
        HAL_SIW_Demura_Bypass(FALSE);
    }
    else
    {
        HAL_SIW_Demura_Bypass(TRUE);
    }
#endif

    if (bEnable == TRUE)
    {
        HAL_SIW_Demura_SetDL_BaseAddr(phy_addr + SIW_LUT_OFFSET);
        /* SIW Demura HW will set depth/length dynamically */
        //HAL_SIW_DEMURA_SetDL_Depth();
        //HAL_SIW_DEMURA_SetDL_ReqLenth();
        HAL_SIW_Demura_Enable_DL(TRUE);
        HAL_SIW_Demura_Big_Endian(TRUE);
    }
    else
    {
        HAL_SIW_Demura_Enable_DL(FALSE);
    }
    DEBUG_SIW_DEMURA("MDrv_SIW_Demura_AutoDownload_Setup OK\n");
}

void MDrv_SIW_Demura_SPOC_Reg(MS_U8 *pReg)
{
    HAL_SIW_Demura_SPOC_Reg(pReg);
}

MS_BOOL MDrv_SIW_Demura_Checksum_CRC16(MS_U8 *pHdr, MS_U32 lut_len)
{
    MS_U32 i, j;
    MS_U16 crc = 0xFFFF;
    MS_U16 checksum;
    MS_U8 u8_data;

    DEBUG_SIW_DEMURA("IN len %d\n", lut_len);
    // golden
    checksum = *(pHdr + lut_len); // high byte
    checksum = (checksum << 8) | (*(pHdr + lut_len + 1)); // high | low byte

    for(j = 0; j < lut_len; j++)
    {
        u8_data = pHdr[j];
        crc ^= (MS_U16)(u8_data) << 8;

        for(i = 0; i < 8; i++)
        {
            if(crc & 0x8000)
            {
                crc = (crc << 1) ^ 0x8005;
            }
            else
            {
                crc <<= 1;
            }
        }
    }
    DEBUG_SIW_DEMURA("crc = 0x%04X checksum = 0x%04X checksum addr 0x%X\n", crc, checksum, (pHdr + lut_len));
    if (crc != checksum)
    {
        UBOOT_ERROR("CRC failed!\n");
        return FALSE;
    }
    DEBUG_SIW_DEMURA("OK\n");
    return TRUE;

}


void MDrv_SIW_Demura_RD_Done(MS_BOOL bDone)
{
    HAL_SIW_Demura_RD_Done(bDone);
}

