/* SPDX-License-Identifier: GPL-2.0-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2 of the GNU General Public License ("GPLv2 License")
 * or BSD License.
 *
 * GPLv2 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See http://www.gnu.org/licenses/gpl-2.0.html for more details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
#ifndef _MS_ARRAY_H_
#define _MS_ARRAY_H_

struct array_dim2
{
    int    item_size;
    int    dim1_num;
    int    dim2_num;
    void **dim1_buf;
    void  *dim2_buf;
};

struct array_dim4
{
    int      item_size;
    int      dim1_num;
    int      dim2_num;
    int      dim3_num;
    int      dim4_num;
    void ****dim1_buf;
    void  ***dim2_buf;
    void   **dim3_buf;
    void    *dim4_buf;
};


MS_BOOL  malloc_array_dim2(struct array_dim2 *p, int item_size, int dim1_num, int dim2_num);
void free_array_dim2(struct array_dim2 *p);

MS_BOOL  malloc_array_dim4(struct array_dim4 *p, int item_size, int dim1, int dim2, int dim3, int dim4);
void free_array_dim4(struct array_dim4 *p);


#define DEFINE_ARRAY_DIM2(struct_name, isize, dim1_num, dim2_num) ({ \
    malloc_array_dim2(&struct_name, isize, dim1_num, dim2_num); \
    }) 

#define FREE_ARRAY_DIM2(struct_name)    free_array_dim2(&struct_name)
#define PTR_ARRAY_DIM2(struct_name)     (struct_name.dim1_buf)


#define DEFINE_ARRAY_DIM4(struct_name, isize, dim1, dim2, dim3, dim4) ({ \
    malloc_array_dim4(&struct_name, isize, dim1, dim2, dim3, dim4);  \
    })

#define FREE_ARRAY_DIM4(struct_name)    free_array_dim4(&struct_name)
#define PTR_ARRAY_DIM4(struct_name)     (struct_name.dim1_buf)

#endif

