//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////
#define  _PQ_UTILITY_C_

#if !defined(MSOS_TYPE_LINUX_KERNEL)
#include "string.h"
#include "stdio.h"
#include <sys/stat.h>
#define do_div(x,y) ((x)/=(y))
#else
#include <linux/string.h>
#include <asm/div64.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/err.h>
#include <asm/uaccess.h>
#include <linux/slab.h>
#endif

#include "MsCommon.h"
//#include "MsVersion.h"
#include "MsOS.h"
#include "MsTypes.h"
#include "utopia.h"
#include "utopia_dapi.h"

#include "hwreg_utility2.h"
#include "PQ_utility.h"

// Create List
PqList * createPqList(void)
{
#ifdef MSOS_TYPE_LINUX_KERNEL
    PqList* re = (PqList *) vmalloc(sizeof(PqList));
#else
    PqList* re = (PqList *) malloc(sizeof(PqList));
#endif
    re->count = 0;
    re->first = NULL;
    re->last = NULL;
    re->equal = NULL;
    return re;
}

// Create List with params
PqList * createMySearchList(MS_BOOL(*equal)(void * a, void * b))
{
    PqList* re = createPqList();
    re->equal = equal;
    return re;
}

// Free list
void freePqList(PqList * list)
{
    PqNode *p;
#ifdef MSOS_TYPE_LINUX_KERNEL
    while(list->first)
    {

        p = list->first->next;
        vfree(list->first);
        list->first = p;
    }
    vfree(list);
#else
    while(list->first)
    {

        p = list->first->next;
        free(list->first);
        list->first = p;
    }
    free(list);
#endif
}

//Insert data in tail
void pqListInsertDataAtLast(PqList* const list, void* const data)
{
    if((data == NULL) || (list == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return;
    }
#ifdef MSOS_TYPE_LINUX_KERNEL
    PqNode *node = (PqNode *) vmalloc(sizeof(PqNode));
#else
    PqNode *node = (PqNode *) malloc(sizeof(PqNode));
#endif
    node->data = data;
    node->next = NULL;
    if(list->count)
    {
        list->last->next = node;
        list->last = node;
    }
    else
    {
        list->first = node;
        list->last = node;
    }
    (list->count)++;
}

//Insert data in head
void pqListInsertDataAtFirst(PqList * const list, void* const data)
{
    if((data == NULL) || (list == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return;
    }
#ifdef MSOS_TYPE_LINUX_KERNEL
    PqNode *node = (PqNode *) vmalloc(sizeof(PqNode));
#else
    PqNode *node = (PqNode *) malloc(sizeof(PqNode));
#endif
    node->data = data;
    node->next = NULL;
    if(list->count)
    {
        node->next = list->first;
        list->first = node;
    }
    else
    {
        list->first = node;
        list->last = node;
    }
    (list->count)++;
}

//Insert data before index
void pqListInsertDataAt(PqList * const list, void* const data, MS_U16 index)
{
    if((data == NULL) || (list == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return;
    }
    if(index == 0)
    {
        pqListInsertDataAtFirst(list, data);
        return;
    }
    if(index == list->count)
    {
        pqListInsertDataAtLast(list, data);
        return;
    }
#ifdef MSOS_TYPE_LINUX_KERNEL
    PqNode *node = (PqNode *) vmalloc(sizeof(PqNode));
#else
    PqNode *node = (PqNode *) malloc(sizeof(PqNode));
#endif
    node->data = data;
    node->next = NULL;
    PqNode* p = list->first;
    MS_U16 i = 0;
    for (; i < index -1; i++)
    {
        p = p->next;
    }
    node->next = p->next;
    p->next = node;
    (list->count)++;

}

//Remove Data at last
void* pqListRemoveDataAtLast(PqList* const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    if(list->count == 1)
    {
        return pqListRemoveDataAtFirst(list);
    }
    PqNode *p = list->first;
    while(p->next != list->last)
    {
        p = p->next;
    }
    void *re = list->last->data;
#ifdef MSOS_TYPE_LINUX_KERNEL
    vfree(list->last);
#else
    free(list->last);
#endif
    p->next = NULL;
    list->last = p;
    (list->count)--;
    return re;
}

//Remove Data at first
void* pqListRemoveDataAtFirst(PqList * const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    PqNode *p = list->first;
    list->first = p->next;
    void *re = p->data;
#ifdef MSOS_TYPE_LINUX_KERNEL
    vfree(p);
#else
    free(p);
#endif
    (list->count)--;
    if(list->count == 0)
    {
        list->last = NULL;
    }
    return re;
}

// Remove data at index
void* pqListRemoveDataAt(PqList* const list, MS_U16 index)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    if(index > list->count)
    {
        printf("Warnning! index is overflow!!!\n");
        return NULL;
    }
    if(index == 0)
    {
        return pqListRemoveDataAtFirst(list);
    }
    if(index == list->count)
    {
        return pqListRemoveDataAtLast(list);
    }
    PqNode *p = list->first;
    MS_U16 i = 0;
    for (; i < index -1; i++)
    {
        p = p->next;
    }
    PqNode *tp = p->next;
    p->next = p->next->next;
    void *re = tp->data;
#ifdef MSOS_TYPE_LINUX_KERNEL
    vfree(p);
#else
    free(tp);
#endif
    (list->count)--;
    return re;
}

// Get list length
MS_U16 pqListGetSize(const PqList * const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return 0;
    }
    return list->count;
}

// Print list data
void pqListOutput(const PqList * const list, void(*pt)(const void * const))
{
    if ((list == NULL) || (pt == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return;
    }
    PqNode *p = list->first;
    while(p)
    {
        (*pt)(p->data);
        p = p->next;
    }
}

//Get data at index
void* pqListGetDataAt(const PqList * const list, MS_U16 index)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    if(index > list->count)
    {
        printf("Warnning! index is overflow!!!\n");
        return NULL;
    }
    if(index == list->count -1)
    {
        return pqListGetDataAtLast(list);
    }
    PqNode *p = list->first;
    MS_U16 i = 0;
    for(; i < index; i++)
    {
        p = p->next;
    }
    return p->data;
}

//Get data at first
void* pqListGetDataAtFirst(const PqList * const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    return list->first->data;
}

//Get data at last
void* pqListGetDataAtLast(const PqList * const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    return list->last->data;
}

// Find data potion
MS_U16 pqListFindDataIndex(const PqList * const list, void * data)
{
    if ((list == NULL) || (data == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return 0;
    }
    PqNode *p = list->first;
    MS_U16 re = 0;
    if(list->equal)
    {
        while(p)
        {
            if((p->data == data) || (*(list->equal))(p->data, data))
            {
                return re;
            }
            re++;
            p = p->next;
        }
    }
    else
    {
        while(p)
        {
            if(p->data == data)
            {
                return re;
            }
            re++;
            p = p->next;
        }
    }
    return -1;
}

//Create list iterator
PqListIterator* createPqListIterator(const PqList * const list)
{
    if (list == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
#ifdef MSOS_TYPE_LINUX_KERNEL
    PqListIterator *re = (PqListIterator *) vmalloc(sizeof(PqListIterator));
#else
    PqListIterator *re = (PqListIterator *) malloc(sizeof(PqListIterator));
#endif
    re->p = list->first;
    re->allSize = list->count;
    re->count = 0;
    return re;
}

// Free List iterator
void freePqListIterator(PqListIterator* iterator)
{
    if (iterator == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return;
    }
#ifdef MSOS_TYPE_LINUX_KERNEL
    vfree(iterator);
#else
    free(iterator);
#endif
}

 // Check Iterator has next element or not
MS_BOOL pqListIteratorHasNext(const PqListIterator* const iterator)
{
    if (iterator == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return FALSE;
    }
    return iterator->count < iterator->allSize;
}

void * pqListIteratorNext(PqListIterator* const iterator)
{
    if (iterator == NULL)
    {
        printf("Warnning! NULL  pointer!!!\n");
        return NULL;
    }
    void * re = iterator->p->data;
    iterator->p = iterator->p->next;
    (iterator->count)++;
    return re;
}

 // Remode object
MS_BOOL pqListRemoveDataObject(PqList* const list, void * data)
{
    if((data == NULL) || (list == NULL))
    {
        printf("Warnning! NULL  pointer!!!\n");
        return FALSE;
    }
    PqListIterator * it = createPqListIterator(list);
    MS_BOOL ret = FALSE;
    while(pqListIteratorHasNext(it))
    {
        void * ld = pqListIteratorNext(it);
        if(data == ld || (list->equal != NULL && (*(list->equal))(ld, data)))
        {
 #ifdef  MSOS_TYPE_LINUX_KERNEL
            vfree(ld);
 #else
            free(ld);
 #endif
            ret = TRUE;
            break;
        }
    }
    if(ret)
    {
        pqListRemoveDataAt(list, it->count - 1);
    }
    freePqListIterator(it);
    return ret;
}

