//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// file    drvSYS.c
/// @brief  System Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////


//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
#if defined (MSOS_TYPE_LINUX)
#include <errno.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#endif

#ifndef MSOS_TYPE_LINUX_KERNEL
#include <string.h>
#else
#include <linux/slab.h>
#endif
#ifdef MSOS_TYPE_LINUX_KERNEL
#include "MsFS.h"
#endif
#include "MsCommon.h"
#include "MsVersion.h"
#include "regCHIP.h"
#include "halCHIP.h"
#include "drvSYS.h"
#include "drvMMIO.h"
#include "halSYS.h"
#include "utopia_dapi.h"
#include "drvSYS_priv.h"
#include "utopia.h"
#include "label.h"
#include "drvMIU.h"
#include "drvMBX.h"
#include "apiMBX.h"
#include "drvBDMA.h"
#include "drvSEM.h"
#include "ULog.h"
#include "drvOPTEE_priv.h"
//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define TAG_SYS                      "SYS"
#define HDCP_KEY_SIZE                289UL

//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
SYS_IO_PROC                         g_SysIoProc =
                                    {
                                        .SysSwI2CReadBytes = 0,
                                        .SysSwI2CWriteBytes = 0,
                                    };

#ifdef CONFIG_MSTAR_DVFS_ENABLE
MS_U32                              g_bDvfsInitFlag = 0;
#endif

extern MS_U8                        _u8SysDbgLevel;

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
static MSIF_Version                 _sysVersion =
                                    {
                                        .DDI = { SYS_DRV_VERSION, },
                                    };

static SYS_Status                   _sysStatus =
                                    {
                                        .WDT_Active = FALSE,
                                    };

#if defined (MSOS_TYPE_LINUX)
static char                         utopia_mode[10] = {""};
#endif
static char                         tee_mode[10] = {""};
MS_U32                              Input_Type = E_SYS_INPUT_TYPE_DOLBY_HDMI;

static MS_BOOL                      _bEfuseInVirtualMode = FALSE;
static MS_BOOL                      _bEfuseInVirtualInit = FALSE;
//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------
#define SYS_DBG_FUNC()              if (_u8SysDbgLevel >= E_SYS_DBGLV_ALL) \
                                        {MS_DEBUG_MSG(ULOGD(TAG_SYS, "\t====   %s   ====\n", __FUNCTION__));}
#define SYS_DBG_INFO(x, args...)    if (_u8SysDbgLevel >= E_SYS_DBGLV_INFO ) \
                                        {MS_DEBUG_MSG(ULOGI(TAG_SYS, x, ##args));}
#define SYS_DBG_ERR(x, args...)     if (_u8SysDbgLevel >= E_SYS_DBGLV_ERR_ONLY) \
                                        {MS_DEBUG_MSG(ULOGE(TAG_SYS, x, ##args));}
//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Global Functions
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
// SYS Init
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// DEBUG
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  INFO
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
/// Get system information
/// @return Chip Version ID
//-------------------------------------------------------------------------------------------------
MS_U8 MDrv_SYS_GetChipVer(void)
{
    return SYS_GetChipVer();
}

//-------------------------------------------------------------------------------------------------
/// Get system information
/// @return Chip Revision ID
//-------------------------------------------------------------------------------------------------
MS_U8 MDrv_SYS_GetChipRev(void)
{
    return SYS_GetChipRev();
}

//-------------------------------------------------------------------------------------------------
/// Get system information
/// @return Chip Device ID
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_SYS_GetChipID(void)
{
    return SYS_GetChipID();
}

//-------------------------------------------------------------------------------------------------
/// Get system information
/// @return @ref SYS_Info
//-------------------------------------------------------------------------------------------------
const SYS_Info* MDrv_SYS_GetInfo(void)
{
    return SYS_GetInfo();
}

const MSIF_Version* MDrv_SYS_GetLibVer(void)
{
    return (const MSIF_Version*)(&_sysVersion);
}

MS_BOOL MDrv_SYS_GetStatus(SYS_Status *pStatus)
{
    memcpy(pStatus, &_sysStatus, sizeof(SYS_Status));
    return TRUE;
}

void MDrv_SYS_GetSoftwareVersion(MS_SW_VERSION_INFO *pSoftwareVersionInfo)
{
    memset(pSoftwareVersionInfo, '\0', sizeof(MS_SW_VERSION_INFO));

    strncpy(
        pSoftwareVersionInfo->MajorVersion,
        (char *) _sysVersion.DDI.version,
        sizeof(_sysVersion.DDI.version)
        );

    strncpy(
        pSoftwareVersionInfo->MinorVersion,
        (char *) _sysVersion.DDI.build,
        sizeof(_sysVersion.DDI.build)
        );

#ifdef UTOPIA_BSP_VERSION
    snprintf(
        pSoftwareVersionInfo->UtopiaBspVersion,
        sizeof(pSoftwareVersionInfo->UtopiaBspVersion),
        "%d",
        UTOPIA_BSP_VERSION
        );
#endif

#ifdef SW_VERSION_API_SYS
    snprintf(
        pSoftwareVersionInfo->ChangeList_API,
        sizeof(pSoftwareVersionInfo->ChangeList_API),
        "%d",
        SW_VERSION_API_SYS
        );
#endif

#ifdef SW_VERSION_DRV_SYS
    snprintf(
        pSoftwareVersionInfo->ChangeList_DRV,
        sizeof(pSoftwareVersionInfo->ChangeList_DRV),
        "%d",
        SW_VERSION_DRV_SYS
        );
#endif

#ifdef SW_VERSION_HAL_SYS
    snprintf(
        pSoftwareVersionInfo->ChangeList_HAL,
        sizeof(pSoftwareVersionInfo->ChangeList_HAL),
        "%d",
        SW_VERSION_HAL_SYS
        );
#endif
}

//-------------------------------------------------------------------------------------------------
// CHIP TYPE
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
/// @brief \b Function  \b Name: MDrv_SYS_GetChipType
/// @brief \b Function  \b Description: get the TV/STB chip type
/// @param eLevel       \b IN: E_SYS_DBGLV_NONE/E_SYS_DBGLV_ERR_ONLY/E_SYS_DBGLV_INFO/E_SYS_DBGLV_ALL
/// @return             \b E_SYS_CHIP_TYPE  E_SYS_CHIP_TV:TV chip set / E_SYS_CHIP_STB : STB chip set
//-------------------------------------------------------------------------------------------------
E_SYS_CHIP_TYPE MDrv_SYS_GetChipType(void)
{
    return HAL_SYS_GetChipType();
}

void MDrv_SYS_SetChipType(E_SYS_CHIP_TYPE Type)
{
    ;
}

//-------------------------------------------------------------------------------------------------
// RESET
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
/// Reset whole chip
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_ResetChip(void)
{
    HAL_SYS_ResetChip();
}

//-------------------------------------------------------------------------------------------------
/// Reset CPU only
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_ResetCPU(void)
{
    ;
}

//-------------------------------------------------------------------------------------------------
// TEE INFO
//-------------------------------------------------------------------------------------------------
MS_U32 MDrv_SYS_SetTEEInfo(MS_U32 osType)
{
    if (osType == SYS_TEEINFO_OSTYPE_OPTEE) {
        strncpy(tee_mode, "optee", sizeof(tee_mode)-1);
        tee_mode[sizeof(tee_mode)-1] = '\n';
    } else if (osType == SYS_TEEINFO_OSTYPE_NUTTX) {
        strncpy(tee_mode, "nuttx", sizeof(tee_mode)-1);
        tee_mode[sizeof(tee_mode)-1] = '\n';
    } else {
        strncpy(tee_mode, "notee", sizeof(tee_mode)-1);
        tee_mode[sizeof(tee_mode)-1] = '\n';
    }

    return 0;
}

MS_U32 MDrv_SYS_GetTEEInfo(SYS_TEEINFO* TeeInfo)
{
    if (!strcmp(tee_mode,"optee")) {
#ifdef NCI_ENABLE
        //Forece Utopia working in REE mode
        TeeInfo->OsType = SYS_TEEINFO_OSTYPE_NOTEE;
#else
        TeeInfo->OsType = SYS_TEEINFO_OSTYPE_OPTEE;
#endif
    } else if (!strcmp(tee_mode,"nuttx")) {
        TeeInfo->OsType = SYS_TEEINFO_OSTYPE_NUTTX;
    }else{
        TeeInfo->OsType = SYS_TEEINFO_OSTYPE_NOTEE;
    }

    return 0;
}

//-------------------------------------------------------------------------------------------------
// TEEC
//-------------------------------------------------------------------------------------------------
/*
 * This function initializes a new TEE Context, connecting this Client
 * application to the TEE identified by the name name.
 *
 * name == NULL will give the default TEE.
 */
MS_U32 MDrv_SYS_TEEC_InitializeContext(const char *name, TEEC_Context *context)
{
#if defined(MSOS_TYPE_LINUX) || defined(MSOS_TYPE_LINUX_KERNEL)
   return _MDrv_SYS_TEEC_InitializeContext(name, context);
#else
   return TEEC_ERROR_NOT_IMPLEMENTED;
#endif
}

/*
 * This function opens a new Session between the Client application and the
 * specified TEE application.
 *
 * Only connection_method == TEEC_LOGIN_PUBLIC is supported connection_data and
 * operation shall be set to NULL.
 */
MS_U32 MDrv_SYS_TEEC_Open(TEEC_Context *context, TEEC_Session *session, const TEEC_UUID *destination, MS_U32 connection_method, const void *connection_data, TEEC_Operation *operation, MS_U32 *error_origin)
{
#if defined(MSOS_TYPE_LINUX) || defined(MSOS_TYPE_LINUX_KERNEL)
   return _MDrv_SYS_TEEC_Open(context, session, destination, connection_method, connection_data, operation, error_origin);
#else
   return TEEC_ERROR_NOT_IMPLEMENTED;
#endif
}

/*
 * This function destroys an initialized TEE Context, closing the connection
 * between the Client and the TEE.
 * The function implementation MUST do nothing if context is NULL
 */
void MDrv_SYS_TEEC_FinalizeContext(TEEC_Context *context)
{
#if defined(MSOS_TYPE_LINUX) || defined(MSOS_TYPE_LINUX_KERNEL)
   _MDrv_SYS_TEEC_FinalizeContext(context);
#endif
}

/*
 * This function closes a session which has been opened with a TEE
 * application.
 */
void MDrv_SYS_TEEC_Close(TEEC_Session *session)
{
#if defined(MSOS_TYPE_LINUX) || defined(MSOS_TYPE_LINUX_KERNEL)
   _MDrv_SYS_TEEC_Close(session);
#endif
}

/*
 * Invokes a TEE command (secure service, sub-PA or whatever).
 */
MS_U32 MDrv_SYS_TEEC_InvokeCmd(TEEC_Session *session, MS_U32 cmd_id, TEEC_Operation *operation, MS_U32 *error_origin)
{
#if defined(MSOS_TYPE_LINUX) || defined(MSOS_TYPE_LINUX_KERNEL)
   return _MDrv_SYS_TEEC_InvokeCmd(session, cmd_id, operation, error_origin);
#else
   return TEEC_ERROR_NOT_IMPLEMENTED;
#endif
}

#ifdef MSOS_TYPE_LINUX_KERNEL //Kernel Space
EXPORT_SYMBOL(MDrv_SYS_TEEC_InvokeCmd);
EXPORT_SYMBOL(MDrv_SYS_TEEC_Close);
EXPORT_SYMBOL(MDrv_SYS_TEEC_InitializeContext);
EXPORT_SYMBOL(MDrv_SYS_TEEC_Open);
EXPORT_SYMBOL(MDrv_SYS_TEEC_FinalizeContext);
#endif
//-------------------------------------------------------------------------------------------------
// PAD MUTEX TABLE
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// WDT
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
/// Enable/Disable watchdog timer
/// @param  bEnable                 \b IN:  Enable/disable
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_WDTEnable(MS_BOOL bEnable)
{
    if (bEnable)
    {
#if 0 //TODO: Utopia & HAL
        WDT_REG(REG_WDT_DISABLEWORD_L) = 0;
        WDT_REG(REG_WDT_DISABLEWORD_H) = 0;
#endif
        _sysStatus.WDT_Active = TRUE;
    }
    else
    {
#if 0 //TODO: Utopia & HAL
        WDT_REG(REG_WDT_DISABLEWORD_L) = 0xaa55;
        WDT_REG(REG_WDT_DISABLEWORD_H) = 0x55aa;
#endif
        _sysStatus.WDT_Active = FALSE;
    }
}

//-------------------------------------------------------------------------------------------------
/// Clear watchdog timer
/// @return None
/// @note The monitor task should call this API regularly
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_WDTClear(void)
{
#if 0 //TODO: Utopia & HAL
    WDT_REG(REG_WDT_CLEAR_STATUS) = 0x0001;
#endif
}

//-------------------------------------------------------------------------------------------------
/// Query whether Watchdog timer was triggered last time and clear the last status
/// @return @ref MS_BOOL
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_SYS_WDTLastStatus(void)
{
    MS_BOOL bRet = FALSE;

#if 0 //TODO: Utopia & HAL
    if (WDT_REG(REG_WDT_CLEAR_STATUS) & 0x0004)
    {
        bRet = TRUE;
    }

    WDT_REG(REG_WDT_CLEAR_STATUS) = 0x0002;
#endif

    return bRet;
}

//-------------------------------------------------------------------------------------------------
/// Set tigger time of watchdog timer
/// @param  u32Ms                   \b IN:  msec (350~357564 ms at 12 MHz; 146~148958 ms at 28.8 MHz)
/// @return None
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_WDTSetTime(MS_U32 u32Ms)
{
#if 0 //TODO: Utopia & HAL
    WDT_REG(REG_WDT_SETTIME) = (u32Ms * (XTAL_CLOCK_FREQ / 1000)) >> 22; //1 ~ 0x3FF
#endif
}

//-------------------------------------------------------------------------------------------------
// TS CLOCK
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// DOLBY
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
/// Quert Dolby Hash Info.
/// @param index \b IN: E_SYS_DOLBY_VERSION: Dolby version, E_SYS_DOLBY_CONTROL_BIT: Dolby control bit, E_SYS_DOLBY_REBERSE_BIT: Dolby reverse bit.
/// @return The info about Dolby hash info accroding to the input param index.
//------------------------------------------------------------------------------------------------
MS_U32 MDrv_SYS_QueryDolbyHashInfo(E_SYS_DOLBY_HASH_INFO index)
{
    return HAL_SYS_QueryDolbyHashInfo(index);
}

//-------------------------------------------------------------------------------------------------
/// @brief \b Function  \b Name: MDrv_SYS_GetDolbyKeyCustomer
/// @brief \b Function  \b Description: get the Key Customer
/// @param eLevel       \b IN/OUT: The point of key customer
/// @return             \b void
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_GetDolbyKeyCustomer(MS_U8 * u8pkey)
{
    MS_U8 u8index;
    static volatile MS_U8 u8gDolbyKeyCustomer[24] =
                            {'H','K','C','S',
                              0x44,0x10,0x91,0x68,0xff,0x0e,0x4c,0x28,0x77,0x0e,0xf5,0x57,0xe0,0x20,0x02,0xdf
                            ,'H','K','C','E'};

    for(u8index =4; u8index <(16+4); u8index ++)
    {
        u8pkey[u8index-4] = u8gDolbyKeyCustomer [u8index];
    }
}

//-------------------------------------------------------------------------------------------------
// BRICK TERMINATOR
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_SYS_ReadBrickTerminatorStatus(void)
{
    return HAL_SYS_ReadBrickTerminatorStatus();
}

void MDrv_SYS_WriteBrickTerminatorStatus(MS_U16 u16Status)
{
    HAL_SYS_WriteBrickTerminatorStatus(u16Status);
}

//-------------------------------------------------------------------------------------------------
// EFUSE
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_SYS_GetEfuseDid(MS_U16 *u16efuse_did)
{
    if(NULL == u16efuse_did)
    {
        ULOGE(TAG_SYS, "%s NULL pointer is invalid!\n",__FUNCTION__);
        return FALSE;
    }

    HAL_SYS_GetEfuseDid(u16efuse_did);

    return TRUE;
}

MS_BOOL MDrv_SYS_ReadEfuseHDCPKey(MS_U16 u16ReadAddr, MS_U32 *u32HDCPKey)
{
    return HAL_SYS_ReadEfuseHDCPKey(u16ReadAddr,u32HDCPKey);
}

MS_BOOL MDrv_SYS_ReadHDCPKey(MS_U8 *u8HDCPKey)
{
    MS_U16  efuse_addr  = 0;
    MS_U32  efuse_data  = 0;
    MS_U16  rdata_lo    = 0;
    MS_U16  rdata_hi    = 0;
    while(efuse_addr < HDCP_KEY_SIZE)
    {
        if (!HAL_SYS_ReadEfuseHDCPKey((efuse_addr/4), &efuse_data))
        {
            return FALSE;
        }
        rdata_lo = (MS_U16) (efuse_data & 0xffff);
        rdata_hi = (MS_U16) ((efuse_data >> 16) & 0xffff);

        if (efuse_addr < HDCP_KEY_SIZE)
            u8HDCPKey[efuse_addr++] = (rdata_lo >> 8) & 0xff;
        if (efuse_addr < HDCP_KEY_SIZE)
            u8HDCPKey[efuse_addr++] = rdata_lo & 0xff;
        if (efuse_addr < HDCP_KEY_SIZE)
            u8HDCPKey[efuse_addr++] = (rdata_hi >> 8) & 0xff;
        if (efuse_addr < HDCP_KEY_SIZE)
            u8HDCPKey[efuse_addr++] = rdata_hi & 0xff;
    }
    return TRUE;
}

MS_BOOL MDrv_SYS_Query(E_SYS_QUERY id)
{
    MS_BOOL bRet;
#if ENABLE_VIRTUAL_EFUSE
//Init Virtual
    if(_bEfuseInVirtualInit == FALSE)
    {
        if (HAL_SYS_Query(E_SYS_QUERY_SECURED_IC_SUPPORTED) == FALSE)
        {
            _bEfuseInVirtualMode = HAL_SYS_VirtualInit();
        }
        else
        {
            _bEfuseInVirtualMode = FALSE;
        }
        _bEfuseInVirtualInit = TRUE;
    }

    bRet = (id == E_SYS_QUERY_SECURED_IC_SUPPORTED || _bEfuseInVirtualMode == FALSE) ? HAL_SYS_Query(id) : HAL_SYS_VirtualQuery(id);
#else
    bRet = HAL_SYS_Query(id);
#endif

    return bRet;
}

//-------------------------------------------------------------------------------------------------
// OTHER
//-------------------------------------------------------------------------------------------------
#ifdef _SYS_DAC_GAIN_CTRL_
//-------------------------------------------------------------------------------------------------
/// Set DAC 1/2 enhance gain.
/// @param u8DAC_ID \b IN: 1: DAC1, 2: DAC2, Other: Invalid DAC ID.
/// @param bEnable \b IN: enable/disable enhance gain.
/// @return void
//-------------------------------------------------------------------------------------------------
void MDrv_SYS_EnhanceDACGain(MS_U8 u8DAC_ID, MS_BOOL bEnable)
{
    HAL_SYS_EnhanceDACGain(u8DAC_ID, bEnable);
}
#endif

MS_U32 MDrv_SYS_SetPowerState(EN_POWER_MODE u16PowerState)
{
   static EN_POWER_MODE _prev_u16PowerState = E_POWER_MECHANICAL;
   MS_U16 u32Return = UTOPIA_STATUS_FAIL;

   if (u16PowerState == E_POWER_SUSPEND)
   {
      _prev_u16PowerState = u16PowerState;
      u32Return = UTOPIA_STATUS_SUCCESS; //SUSPEND_OK
   }
   else if (u16PowerState == E_POWER_RESUME)
   {
      if (_prev_u16PowerState == E_POWER_SUSPEND)
      {
            MDrv_SYS_Init();

         _prev_u16PowerState = u16PowerState;
         u32Return = UTOPIA_STATUS_SUCCESS; //RESUME_OK
      }
      else
      {
         ULOGE(TAG_SYS, "[%s,%5d]It is not suspended yet. We shouldn't resume\n",__FUNCTION__,__LINE__);
         u32Return = UTOPIA_STATUS_FAIL; //SUSPEND_FAILED
      }
   }
   else
   {
      ULOGE(TAG_SYS, "[%s,%5d]Do Nothing: %d\n",__FUNCTION__,__LINE__,u16PowerState);
      u32Return = UTOPIA_STATUS_FAIL;
   }

   return u32Return;
}

//-------------------------------------------------------------------------------------------------
/// Get System Hardware List
//-------------------------------------------------------------------------------------------------
MS_U32 MDrv_SYS_GetIpList(E_SYS_IP_TYPE TYPE)
{
   MS_U32 ret = 0;
   switch(TYPE)
   {
      case E_SYS_IP_ENCODER:
         if(CHIP_IP_MFE)
            ret |= SYS_ENCODER_MFE;
         if(CHIP_IP_VE)
            ret |= SYS_ENCODER_VE;
         return ret;

      case E_SYS_IP_FRONTEND:
         if(CHIP_IP_AVD)
            ret |= SYS_FRONTEND_AVD;
         if(CHIP_IP_DEMOD)
            ret |= SYS_FRONTEND_DEMOD;
         if(CHIP_IP_VBI)
            ret |= SYS_FRONTEND_VBI;
         if(CHIP_IP_VIF)
            ret |= SYS_FRONTEND_VIF;
         return ret;

      case E_SYS_IP_DEMUX:
         if(CHIP_IP_DMX)
            ret |= SYS_DEMUX_DMX;
         return ret;

      case E_SYS_IP_SYSTEM:
         if(CHIP_IP_CEC)
            ret |= SYS_SYSTEM_CEC;
         if(CHIP_IP_MBX)
            ret |= SYS_SYSTEM_MBX;
         if(CHIP_IP_SWI2C)
            ret |= SYS_SYSTEM_SWI2C;
         if(CHIP_IP_BDMA)
            ret |= SYS_SYSTEM_BDMA;
         if(CHIP_IP_CPU)
            ret |= SYS_SYSTEM_CPU;
         if(CHIP_IP_GPIO)
            ret |= SYS_SYSTEM_GPIO;
         if(CHIP_IP_HWI2C)
            ret |= SYS_SYSTEM_HWI2C;
         if(CHIP_IP_IR)
            ret |= SYS_SYSTEM_IR;
         if(CHIP_IP_MIU)
            ret |= SYS_SYSTEM_MIU;
         if(CHIP_IP_MPIF)
            ret |= SYS_SYSTEM_MPIF;
         if(CHIP_IP_MSPI)
            ret |= SYS_SYSTEM_MSPI;
         if(CHIP_IP_PM)
            ret |= SYS_SYSTEM_PM;
         if(CHIP_IP_PWM)
            ret |= SYS_SYSTEM_PWM;
         if(CHIP_IP_PWS)
            ret |= SYS_SYSTEM_PWS;
         if(CHIP_IP_RTC)
            ret |= SYS_SYSTEM_RTC;
         if(CHIP_IP_SAR)
            ret |= SYS_SYSTEM_SAR;
         if(CHIP_IP_URDMA)
            ret |= SYS_SYSTEM_URDMA;
         if(CHIP_IP_WDT)
            ret |= SYS_SYSTEM_WDT;
         return ret;

      case E_SYS_IP_CA:
         if(CHIP_IP_AESDMA)
            ret |= SYS_CA_AESDMA;
         if(CHIP_IP_CA)
            ret |= SYS_CA_CA;
         if(CHIP_IP_PCMCIA)
            ret |= SYS_CA_PCMCIA;
         if(CHIP_IP_SC)
            ret |= SYS_CA_SC;
         return ret;

      case E_SYS_IP_CODEC:
         if(CHIP_IP_GPD)
            ret |= SYS_CODEC_GPD;
         if(CHIP_IP_JPEG)
            ret |= SYS_CODEC_JPEG;
         if(CHIP_IP_VDEC)
            ret |= SYS_CODEC_VDEC;
         return ret;

      case E_SYS_IP_DISPLAY:
         if(CHIP_IP_ACE)
            ret |= SYS_DISPLAY_ACE;
         if(CHIP_IP_DAC)
            ret |= SYS_DISPLAY_DAC;
         if(CHIP_IP_DDC2BI)
            ret |= SYS_DISPLAY_DDC2BI;
         if(CHIP_IP_DIP)
            ret |= SYS_DISPLAY_DIP;
         if(CHIP_IP_DLC)
            ret |= SYS_DISPLAY_DLC;
         if(CHIP_IP_GOP)
            ret |= SYS_DISPLAY_GOP;
         if(CHIP_IP_HDMITX)
            ret |= SYS_DISPLAY_HDMITX;
         if(CHIP_IP_MHL)
            ret |= SYS_DISPLAY_MHL;
         if(CHIP_IP_MVOP)
            ret |= SYS_DISPLAY_MVOP;
         if(CHIP_IP_PNL)
            ret |= SYS_DISPLAY_PNL;
         if(CHIP_IP_XC)
            ret |= SYS_DISPLAY_XC;
         return ret;

      case E_SYS_IP_GFX:
         if(CHIP_IP_GFX)
            ret |= SYS_GFX_GFX;
         return ret;

      case E_SYS_IP_AUDIO:
         if(CHIP_IP_AUDIO)
            ret |= SYS_AUDIO_AUDIO;
         return ret;

      case E_SYS_IP_STORAGE:
         if(CHIP_IP_SERFLASH)
            ret |= SYS_STORAGE_SERFLASH;
         return ret;

      default:
         SYS_DBG_INFO("Not Support IP type %d\n",TYPE);
         break;
   }

   return ret;
}

//FOR INTERNAL USE ONLY
MS_U32 MDrv_SYS_RegisterIoProc(E_SYS_IO_PROC dwIoType, void *pdwIoFunction )
{
    MS_U32  dwReturnStatus = TRUE;

    switch(dwIoType)
    {
        case E_SYS_IO_SWI2C_READ:
        {
            //Register the Function Pointer of SWI2C Read Procedure
            g_SysIoProc.SysSwI2CReadBytes = (MS_BOOL (*)(MS_U16, MS_U8, MS_U8 *, MS_U16, MS_U8 *))(pdwIoFunction);
        }
        break;

        case E_SYS_IO_SWI2C_WRITE:
        {
            //Register the Function Pointer of SWI2C Write Procedure
            g_SysIoProc.SysSwI2CWriteBytes = (MS_BOOL (*)(MS_U16, MS_U8, MS_U8 *, MS_U16, MS_U8 *))(pdwIoFunction);
        }
        break;

        default:
        {
            SYS_DBG_INFO("Unknown IO Type\n");
            dwReturnStatus = FALSE;
        }
        break;
    }

    return dwReturnStatus;
}

char* MDrv_SYS_GetUtopiaReleaseLabel(void)
{
    return UTOPIA_RELEASE_LABEL;
}

#if defined (MSOS_TYPE_LINUX)
static void _MDrv_SYS_ParserCmdLine(char *name)
{
    char *value = strchr(name, '=');

    if (value == 0) return;
    *value++ = 0;
    if (*name == 0) return;

    if (!strcmp(name,"utopia_mode")) {
            strncpy(utopia_mode, value, sizeof(utopia_mode)-1);
            utopia_mode[sizeof(utopia_mode)-1] = '\n';
        } else if (!strcmp(name,"tee_mode")) {
            strncpy(tee_mode, value, sizeof(tee_mode)-1);
            tee_mode[sizeof(utopia_mode)-1] = '\n';
        }
}
#endif

void MDrv_SYS_ReadKernelCmdLine(void)
{
#if defined(MSOS_TYPE_LINUX)
    char cmdline[1024];
    char *ptr;
    int fd,n;
    fd = open("/proc/cmdline", O_RDONLY);
    if(fd<0)
    {
          SYS_DBG_ERR("Open failed! Try to open /proc/system_cmdline \n");
            fd = open("/proc/system_cmdline",O_RDONLY);
      if(fd<0)
      {
         SYS_DBG_ERR("Open /proc/system_cmdline failed!\n");
                     cmdline[0] = 0;
         goto fail_open;
      }
     }
     n = read(fd, cmdline, 1023);
     if (n < 0) n = 0;

     /* get rid of trailing newline, it happens */
     if (n > 0 && cmdline[n-1] == '\n') n--;

     cmdline[n] = 0;
     close(fd);

fail_open :
     ptr = cmdline;
     while (ptr && *ptr) {
         char *x = strchr(ptr, ' ');
         if (x != 0) *x = 0;
         _MDrv_SYS_ParserCmdLine(ptr);
         ptr = x ? x + 1 : x;
     }
#elif defined(MSOS_TYPE_LINUX_KERNEL)
    extern int TEEINFO_TYPTE;
    switch(TEEINFO_TYPTE)
    {
        case SYS_TEEINFO_OSTYPE_OPTEE:
        case SYS_TEEINFO_OSTYPE_SECARM:
            strncpy(tee_mode, "optee", sizeof(tee_mode)-1);
            tee_mode[sizeof(tee_mode)-1] = '\n';
            break;
        case SYS_TEEINFO_OSTYPE_NUTTX:
            strncpy(tee_mode, "nuttx", sizeof(tee_mode)-1);
            tee_mode[sizeof(tee_mode)-1] = '\n';
            break;
        default:
            strncpy(tee_mode, "notee", sizeof(tee_mode)-1);
            tee_mode[sizeof(tee_mode)-1] = '\n';
    }
#else
    SYS_DBG_INFO("Not Support\n");
#endif
}

E_SYS_ReturnValue  _MDrv_SYS_XcInputMapping (MS_U32 u32SourceType, E_SYS_SOURCE_TYPE *retSrcType)
{
    //
    // Input Source Mapping,
    //
    switch (u32SourceType)
    {
    case E_SYS_INPUT_SOURCE_DTV:
    case E_SYS_INPUT_SOURCE_DTV2:
    case E_SYS_INPUT_SOURCE_DTV3:
        *retSrcType = E_SYS_SOURCE_DTV;
        break;

    case E_SYS_INPUT_SOURCE_HDMI:
    case E_SYS_INPUT_SOURCE_HDMI2:
    case E_SYS_INPUT_SOURCE_HDMI3:
    case E_SYS_INPUT_SOURCE_HDMI4:
        if(Input_Type == E_SYS_INPUT_TYPE_HDMI)
        {
            *retSrcType = E_SYS_SOURCE_HDMI;
        }
        else if(Input_Type == E_SYS_INPUT_TYPE_DOLBY_HDMI)
        {
            *retSrcType = E_SYS_SOURCE_DOLBY_HDMI;
        }
        break;

    case E_SYS_INPUT_SOURCE_STORAGE:
    case E_SYS_INPUT_SOURCE_STORAGE2:
        *retSrcType = E_SYS_SOURCE_MM;
        break;

    case E_SYS_INPUT_SOURCE_GOOGLE_CAST:
        *retSrcType = E_SYS_SOURCE_GOOGLE_CAST;
        break;

    case E_SYS_INPUT_SOURCE_VGA:
    case E_SYS_INPUT_SOURCE_VGA2:
    case E_SYS_INPUT_SOURCE_VGA3:
    case E_SYS_INPUT_SOURCE_ATV:
    case E_SYS_INPUT_SOURCE_CVBS:
    case E_SYS_INPUT_SOURCE_CVBS2:
    case E_SYS_INPUT_SOURCE_CVBS3:
    case E_SYS_INPUT_SOURCE_CVBS4:
    case E_SYS_INPUT_SOURCE_CVBS5:
    case E_SYS_INPUT_SOURCE_CVBS6:
    case E_SYS_INPUT_SOURCE_CVBS7:
    case E_SYS_INPUT_SOURCE_CVBS8:
    case E_SYS_INPUT_SOURCE_SVIDEO:
    case E_SYS_INPUT_SOURCE_SVIDEO2:
    case E_SYS_INPUT_SOURCE_SVIDEO3:
    case E_SYS_INPUT_SOURCE_SVIDEO4:
    case E_SYS_INPUT_SOURCE_YPBPR:
    case E_SYS_INPUT_SOURCE_YPBPR2:
    case E_SYS_INPUT_SOURCE_YPBPR3:
    case E_SYS_INPUT_SOURCE_SCART:
    case E_SYS_INPUT_SOURCE_SCART2:
    case E_SYS_INPUT_SOURCE_DVI:
    case E_SYS_INPUT_SOURCE_DVI2:
    case E_SYS_INPUT_SOURCE_DVI3:
    case E_SYS_INPUT_SOURCE_DVI4:
        *retSrcType = E_SYS_SOURCE_MAX;
      break;

    default:
        SYS_DBG_ERR ("Not support! Source type: %d\n", u32SourceType);
        return E_SYS_ERROR_PARAMETER;
    }
    return E_SYS_OK;
}

E_SYS_ReturnValue  _MDrv_SYS_XcTimingMapping (MS_U32 u32Width, MS_U32 u32Height, MS_U32 u32FrameRate, E_SYS_INPUT_TIMING *retTimingType)
{
#define TIMING_TOLERANCE 20
#define TIMING_HD_W      1280
#define TIMING_HD_H      720
#define TIMING_FHD_W     1920
#define TIMING_FHD_H     1080
#define TIMING_4K_W      3840
#define TIMING_4K_H      2160

    MS_U32 _u32Width = 0x0, _u32Height = 0x0;

    if ( u32Width <= (TIMING_FHD_W+TIMING_TOLERANCE) &&
         u32Width >= (TIMING_FHD_W-TIMING_TOLERANCE))
    {
        _u32Width = TIMING_FHD_W;
    }
    else if ( u32Width >= (TIMING_4K_W-TIMING_TOLERANCE))
    {
        //To set the width to 4K_W if the width is large than TIMING_4K_W
        _u32Width = TIMING_4K_W;
    }
    else if ( u32Width < (TIMING_FHD_W-TIMING_TOLERANCE))
    {
        //To set the width to HD_W if the width is smaller than TIMING_FHD_W
        _u32Width = TIMING_HD_W;
    }

    if ( u32Height <= (TIMING_FHD_H+TIMING_TOLERANCE) &&
         u32Height >= (TIMING_FHD_H-TIMING_TOLERANCE))
    {
        _u32Height = TIMING_FHD_H;
    }
    else if ( u32Height >= (TIMING_4K_H-TIMING_TOLERANCE))
    {
        //To set the height to 4K_W if the height is large than TIMING_4K_H
        _u32Height = TIMING_4K_H;
    }
    else if ( u32Height < (TIMING_FHD_H-TIMING_TOLERANCE))
    {
        //To set the width to HD_H if the width is smaller than TIMING_FHD_H
        _u32Height = TIMING_HD_H;
    }

    if (_u32Width == TIMING_FHD_W && _u32Height == TIMING_FHD_H)
    {
        if (u32FrameRate <=30)
        {
            *retTimingType = E_SYS_INPUT_FHD24_25_30;
        }
        else if (u32FrameRate <=48 && u32FrameRate > 30)
        {
            *retTimingType = E_SYS_INPUT_FHD48;
        }
        else if (u32FrameRate <=60 && u32FrameRate > 48)
        {
            *retTimingType = E_SYS_INPUT_FHD50_60;
        }
        else
        {
            SYS_DBG_ERR ("FHD with unknown frame rate: %d\n", u32FrameRate);
            *retTimingType = E_SYS_INPUT_FHD_UNKNOWN;
        }
    }
    else if (_u32Width == TIMING_4K_W && _u32Height == TIMING_4K_H)
    {
        if (u32FrameRate <=30)
        {
            *retTimingType = E_SYS_INPUT_4K2K24_25_30;
        }
        else if (u32FrameRate <=48 && u32FrameRate > 30)
        {
            *retTimingType = E_SYS_INPUT_4K2K48;
        }
        else if (u32FrameRate <=60 && u32FrameRate > 48)
        {
            *retTimingType = E_SYS_INPUT_4K2K50_60;
        }
        else if (u32FrameRate >=100 && u32FrameRate <= 120)
        {
            *retTimingType = E_SYS_INPUT_4K2K120;
        }
        else
        {
            SYS_DBG_ERR ("4k2k with unknown frame rate: %d\n", u32FrameRate);
            *retTimingType = E_SYS_INPUT_4K2K_UNKNOWN;
        }
    }
    else if (_u32Width == TIMING_HD_W && _u32Height == TIMING_HD_H)
    {
        *retTimingType = E_SYS_INPUT_FHD24_25_30;
    }
    else
    {
        SYS_DBG_ERR ("Not Mapped timing! W: %d, H: %d, F: %d\n", u32Width, u32Height, u32FrameRate);
        return E_SYS_NOT_SUPPORT;
    }

    return E_SYS_OK;
}

E_SYS_ReturnValue  _MDrv_SYS_InputTimingMapping (MS_U32 u32Width, MS_U32 u32Height, MS_U32 u32FrameRate, E_SYS_INPUT_TIMING *retTimingType)
{
#define TIMING_FHD_W     1920
#define TIMING_FHD_H     1080

    if (u32Width > TIMING_FHD_W && u32Height > TIMING_FHD_H)
    {
        if (u32FrameRate <=30)
        {
            *retTimingType = E_SYS_INPUT_4K2K24_25_30;
        }
        else if (u32FrameRate <=48 && u32FrameRate > 30)
        {
            *retTimingType = E_SYS_INPUT_4K2K48;
        }
        else if (u32FrameRate <=60 && u32FrameRate > 48)
        {
            *retTimingType = E_SYS_INPUT_4K2K50_60;
        }
        else
        {
            SYS_DBG_ERR ("4k2k with unknown frame rate: %d\n", u32FrameRate);
            *retTimingType = E_SYS_INPUT_4K2K_UNKNOWN;
        }
    }
    else
    {
        if (u32FrameRate <=30)
        {
            *retTimingType = E_SYS_INPUT_FHD24_25_30;
        }
        else if (u32FrameRate <=48 && u32FrameRate > 30)
        {
            *retTimingType = E_SYS_INPUT_FHD48;
        }
        else if (u32FrameRate <=60 && u32FrameRate > 48)
        {
            *retTimingType = E_SYS_INPUT_FHD50_60;
        }
        else
        {
            SYS_DBG_ERR ("FHD with unknown frame rate: %d\n", u32FrameRate);
            *retTimingType = E_SYS_INPUT_FHD_UNKNOWN;
        }
    }

    return E_SYS_OK;
}
E_SYS_ReturnValue  MDrv_SYS_GetMemcConfg(MS_U32 u32SourceType, SYS_XC_TIMING_t *stXCTiming, MS_BOOL *retEnMemc)
{
    E_SYS_SOURCE_TYPE eSource = E_SYS_SOURCE_DTV;
    E_SYS_INPUT_TIMING eTiming = E_SYS_INPUT_MAX;

    if (stXCTiming == NULL || retEnMemc == NULL)
    {
        SYS_DBG_ERR ("stXCTiming == NULL || retEnMemc == NULL\n");
        return E_SYS_ERROR_PARAMETER;
    }

    if (stXCTiming->u32TimingVersion <= MDRV_SYS_XC_TIMING_VERSION &&
        stXCTiming->u32StructLength <= sizeof (SYS_XC_TIMING_t))
    {

        SYS_DBG_INFO ("Source: %d, u32Width: %d, u32Height: %d, u16Framerate: %d\n",
                      u32SourceType, stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate);

        //
        // Input Source Type Mapping
        //
        if ( _MDrv_SYS_XcInputMapping (u32SourceType, &eSource) != E_SYS_OK)
        {
            SYS_DBG_ERR ("Unmapped source type: %d\n", u32SourceType);
            return E_SYS_ERROR_PARAMETER;
        }

        //
        // Input Timing Mapping
        //
        if ( _MDrv_SYS_XcTimingMapping (stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate, &eTiming) != E_SYS_OK)
        {
            //
            //Error Handling,
            //Unable to determeternate the timing,
            //Config is up to frame rate only.
            //
            SYS_DBG_ERR ("Not support! Timing, W:%d, H:%d, F:%d\n", stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate);
            if (stXCTiming->u16Framerate > 30 )
            {
                eTiming = E_SYS_INPUT_ALWAYS_OFF;
            }
            else
            {
                eTiming = E_SYS_INPUT_ALWAYS_ON;
            }
        }
    } //End of Version check,  #define MDRV_SYS_XC_TIMING_VERSION 0x00000001UL
    else
    {
        SYS_DBG_ERR ("Version Error! Please update drvSYS\n");
        return E_SYS_FAIL;
    }

    return HAL_SYS_GetMemcConfg (eSource, eTiming, retEnMemc);
}

E_SYS_ReturnValue  _MDrv_SYS_GetMemcLevel (MS_U32 u32SourceType, SYS_XC_TIMING_t *stXCTiming, E_SYS_MEMC_LEVEL *retSt)
{
    E_SYS_SOURCE_TYPE eSource = E_SYS_SOURCE_DTV;
    E_SYS_INPUT_TIMING eTiming = E_SYS_INPUT_MAX;

    if (stXCTiming == NULL)
    {
        SYS_DBG_ERR ("stXCTiming == NULL\n");
        return E_SYS_ERROR_PARAMETER;
    }

    if (stXCTiming->u32TimingVersion <= MDRV_SYS_XC_TIMING_VERSION &&
        stXCTiming->u32StructLength <= sizeof (SYS_XC_TIMING_t))
    {

        SYS_DBG_INFO ("Source: %d, u32Width: %d, u32Height: %d, u16Framerate: %d\n",
                      u32SourceType, stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate);

        //
        // Input Source Type Mapping
        //
        if ( _MDrv_SYS_XcInputMapping (u32SourceType, &eSource) != E_SYS_OK)
        {
            SYS_DBG_ERR ("Unmapped source type: %d\n", u32SourceType);
            return E_SYS_ERROR_PARAMETER;
        }

        //
        // Input Timing Mapping
        //
        if ( _MDrv_SYS_InputTimingMapping (stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate, &eTiming) != E_SYS_OK)
        {
            //
            //Error Handling,
            //Unable to determeternate the timing,
            //Config is up to frame rate only.
            //
            SYS_DBG_ERR ("Not support! Timing, W:%d, H:%d, F:%d\n", stXCTiming->u32Width, stXCTiming->u32Height, stXCTiming->u16Framerate);
            if (stXCTiming->u16Framerate > 30 )
            {
                eTiming = E_SYS_INPUT_ALWAYS_OFF;
            }
            else
            {
                eTiming = E_SYS_INPUT_ALWAYS_ON;
            }
        }
    } //End of Version check,  #define MDRV_SYS_XC_TIMING_VERSION 0x00000001UL
    else
    {
        SYS_DBG_ERR ("Version Error! Please update drvSYS\n");
        return E_SYS_NOT_SUPPORT;
    }

    return HAL_SYS_GetMemcLevel (eSource, eTiming, (MS_U32 *)retSt);
}

E_SYS_ReturnValue  _MDrv_SYS_GetXcByPartConfg(MS_U32 u32SourceType, SYS_XC_BYPART_TIMING_t *stInfo, MS_BOOL *retEn)
{
    E_SYS_SOURCE_TYPE eSource = E_SYS_SOURCE_DTV;
    E_SYS_INPUT_TIMING eInputTiming = E_SYS_INPUT_MAX;
    E_SYS_INPUT_TIMING eOutputTiming = E_SYS_INPUT_MAX;

    if (stInfo == NULL || retEn == NULL)
    {
        SYS_DBG_ERR ("stInfo == NULL || retEnMemc == NULL\n");
        return E_SYS_ERROR_PARAMETER;
    }

    if (stInfo->u32TimingVersion <= MDRV_SYS_XC_BYPART_TIMING_VERSION &&
        stInfo->u32StructLength <= sizeof (SYS_XC_BYPART_TIMING_t))
    {
        SYS_DBG_INFO ("Source: %d, Input(W: %d, H: %d, F: %d), Output(W: %d, H: %d, F: %d)\n",
                      u32SourceType, stInfo->stInput.u32Width, stInfo->stInput.u32Height, stInfo->stInput.u16Framerate,
                      stInfo->stOutput.u32Width, stInfo->stOutput.u32Height, stInfo->stOutput.u16Framerate);

        //
        // Input Source Type Mapping
        //
        if ( _MDrv_SYS_XcInputMapping (u32SourceType, &eSource) != E_SYS_OK)
        {
            SYS_DBG_ERR ("Unmapped source type: %d\n", u32SourceType);
            return E_SYS_ERROR_PARAMETER;
        }

        //
        // Input Timing Mapping
        //
        _MDrv_SYS_XcTimingMapping (stInfo->stInput.u32Width, stInfo->stInput.u32Height, stInfo->stInput.u16Framerate, &eInputTiming);
        _MDrv_SYS_XcTimingMapping (stInfo->stOutput.u32Width, stInfo->stOutput.u32Height, stInfo->stOutput.u16Framerate, &eOutputTiming);

    }
    return HAL_SYS_GetXcByPartConfg (eSource, eInputTiming, eOutputTiming, retEn);
}

E_SYS_ReturnValue  MDrv_SYS_GetQoSConfig(E_SYS_QOS_QUERY_TYPE eQueryType, MS_U32 u32SourceType, void *stInfo, MS_BOOL *retEn)
{
    E_SYS_ReturnValue eRet;

    if (stInfo == NULL || retEn == NULL)
    {
        SYS_DBG_ERR ("Error Parameters\n");
        return E_SYS_ERROR_PARAMETER;
    }

    switch (eQueryType)
    {
    case E_SYS_QOS_QUERY_XC_BYPART_EN:
        eRet = _MDrv_SYS_GetXcByPartConfg (u32SourceType, (SYS_XC_BYPART_TIMING_t *)stInfo, retEn);
        break;

    case E_SYS_QOS_QUERY_MEMC_EN:
        eRet = MDrv_SYS_GetMemcConfg (u32SourceType, (SYS_XC_TIMING_t *)stInfo, retEn);
        break;

    default:
        SYS_DBG_ERR ("Unknown Query Type: %d\n", eQueryType);
        eRet = E_SYS_ERROR_PARAMETER;
        break;
    };

    return eRet;
}

E_SYS_ReturnValue  MDrv_SYS_GetQoSStatus(E_SYS_QOS_QUERY_TYPE eQueryType, MS_U32 u32SourceType, void *stInfo, MS_U32 *retSt)
{
    E_SYS_ReturnValue eRet = E_SYS_NOT_SUPPORT;
#ifdef UFO_SYS_QOS_STATUS_SUPPORT
    if (stInfo == NULL || retSt == NULL)
    {
        SYS_DBG_ERR ("Error Parameters\n");
        return E_SYS_ERROR_PARAMETER;
    }

    switch (eQueryType)
    {
    case E_SYS_QOS_QUERY_MEMC_EN:
        eRet = _MDrv_SYS_GetMemcLevel (u32SourceType, (SYS_XC_TIMING_t *)stInfo, (E_SYS_MEMC_LEVEL *)retSt);
        break;

    default:
        SYS_DBG_ERR ("Unknown Query Type: %d\n", eQueryType);
        eRet = E_SYS_ERROR_PARAMETER;
        break;
    };
#endif
    return eRet;
}

void MDrv_SYS_SetInputType(E_SYS_INPUT_TYPE TYPE)
{
    if(TYPE < E_SYS_INPUT_TYPE_MAX)
        Input_Type = TYPE;
}

/////////////////////////////////////////////////////////////////
//////// UTOPIA2 CODE ///////////////////////////////////////////
/////////////////////////////////////////////////////////////////
enum
{
    SYS_POOL_ID_SYS0=0
} eSYS_PoolID;

#ifdef CONFIG_UTOPIA_PROC_DBG_SUPPORT
RIUINFO_CMD stRiuInfoCmd[RIUINFO_END_OF_CMD] = {
    {RIUINFO_DUMP, "Dump"},
    {RIUINFO_BYTE_OFFSET_R, "ByteR"},
    {RIUINFO_WORD_OFFSET_R, "WordR"},
    {RIUINFO_BYTE_OFFSET_W, "ByteW"},
    {RIUINFO_BYTE_OFFSET_WBIT, "ByteW1"},
    {RIUINFO_WORD_OFFSET_W, "WordW"},
    {RIUINFO_WORD_OFFSET_WBIT, "WordW1"},
    {RIUINFO_HELP, "Help"},
    {RIUINFO_BYTE_OFFSET_POLL, "BytePoll"},
    {RIUINFO_WORD_OFFSET_POLL, "WordPoll"},
};

static MS_U32 RIUMdbHelp(MS_U64* u64ReqHdl)
{
    MdbPrint(u64ReqHdl,"---------MStar RIU Info Help---------\n");
    MdbPrint(u64ReqHdl,"Parameters: CMD,BANK,OFFSET,VALUE/INTERVAL,MASK/BIT/TIMES\n\n");
    MdbPrint(u64ReqHdl,"1. Dump: dump bank, (Dump,BANK)\n ex: echo Dump,0x101e > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"2. ByteR: read (8 bit addr), (ByteR,BANK,OFFSET)\n ex: echo ByteR,0x101e,0xa0 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"3. WordR: read (16 bit addr), (WordR,BANK,OFFSET)\n ex: echo WordR,0x101e,0x50 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"4. ByteW: write (8 bit addr), (ByteW,BANK,OFFSET,VALUE,MASK)\n ex: echo ByteW,0x101e,0xa1,0x80,0x80 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"5. ByteW1: bit write (8 bit addr), (ByteW1,BANK,OFFSET,VALUE,BIT)\n ex: echo ByteW1,0x101e,0xa1,1,7 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"6. WordW: write (16 bit addr), (WordW,BANK,OFFSET,VALUE,MASK)\n ex: echo WordW,0x101e,0x50,0x8000,0x8000 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"7. WordW1: bit write (16 bit addr), (WordW1,BANK,OFFSET,VALUE,BIT)\n ex: echo WordW1,0x101e,0x50,1,15 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"8. BytePoll: poll (8 bit addr), (BytePoll,BANK,OFFSET,INTERVAL(ms),TIMES)\n ex: echo BytePoll,0x101e,0xa1,100,5 > riuinfo\n\n");
    MdbPrint(u64ReqHdl,"9. WordPoll: poll (16 bit addr), (WordPoll,BANK,OFFSET,INTERVAL(ms),TIMES)\n ex: echo WordPoll,0x101e,0x50,100,5 > riuinfo\n\n");
    return 0;
}

static MS_U32 RIUMdbIoctl(MS_U32 cmd, const void* const pArgs)
{
    MDBCMD_CMDLINE_PARAMETER *paraCmdLine;
    MDBCMD_GETINFO_PARAMETER *paraGetInfo;
    MS_VIRT virtBank = 0x0;
    MS_PHY u32BankSize;
    MS_U8 i, j, u8Offset = 0x0;
    MS_U16 u16TmpReg, u16TmpRegs[8], u16Bank = 0x101E, u16Value = 0x0, u16MaskOrBit = 0x0;
    MS_U32 u32Base;
    MS_BOOL bFlagPoll = FALSE;
    E_RIUINFO_CMD u32Cmd = RIUINFO_DUMP;
    char *pch, tok[]=", ", *psep;
    switch(cmd)
    {
        case MDBCMD_CMDLINE:
            paraCmdLine = (MDBCMD_CMDLINE_PARAMETER *)pArgs;
            psep = paraCmdLine->pcCmdLine;
            for(i=0;(pch = strsep(&psep, tok))!=NULL;i++)
            {
                switch(i)
                {
                    case RIUINFO_PARA_CMD:
                        for(j=0;j<RIUINFO_END_OF_CMD;j++)
                        {
                            if(0 == strcasecmp(pch,stRiuInfoCmd[j].sRiuInfoCmdList))
                            {
                                u32Cmd = stRiuInfoCmd[j].eCmd;
                                break;
                            }
                        }
                        if(RIUINFO_HELP == u32Cmd || RIUINFO_END_OF_CMD == j)
                        {
                            RIUMdbHelp(paraCmdLine->u64ReqHdl);
                            goto CMD_DONE;
                        }
                        else if(RIUINFO_BYTE_OFFSET_POLL == u32Cmd || RIUINFO_WORD_OFFSET_POLL == u32Cmd)
                        {
                            bFlagPoll = TRUE;
                            u32Cmd = (RIUINFO_BYTE_OFFSET_POLL == u32Cmd)?RIUINFO_BYTE_OFFSET_R:RIUINFO_WORD_OFFSET_R;
                        }
                        break;
                    case RIUINFO_PARA_BANK:
                        u16Bank = mst_atoi(pch);
                        switch((u16Bank&0xF000)>>12)
                        {
                            case 0x0:
                                u32Base = MS_MODULE_PM;
                                break;
                            case 0x1:
                                u32Base = MS_MODULE_CHIPTOP;
                                break;
                            case 0x3:
                                u32Base = MS_MODULE_EXTRIU;
                                break;
                            default:
                                u32Base = MS_MODULE_CHIPTOP;
                                break;
                        }
                        if(FALSE == MDrv_MMIO_GetBASE(&virtBank, &u32BankSize, u32Base))
                        {
                            MdbPrint(paraCmdLine->u64ReqHdl,"unknown bank\n");
                            goto CMD_DONE;
                        }
                        if(RIUINFO_DUMP == u32Cmd)
                        {
                            goto CMD_PARSING_OK;
                        }
                        break;
                    case RIUINFO_PARA_OFFSET:
                        u8Offset = mst_atoi(pch);
                        if((RIUINFO_BYTE_OFFSET_R == u32Cmd || RIUINFO_WORD_OFFSET_R == u32Cmd) && (FALSE == bFlagPoll))
                        {
                            goto CMD_PARSING_OK;
                        }
                        break;
                    case RIUINFO_PARA_VALUE:
                        u16Value = mst_atoi(pch);
                        break;
                    case RIUINFO_PARA_MASK:
                        u16MaskOrBit = mst_atoi(pch);
                        break;
                    default:
                        break;
                }
            }
CMD_PARSING_OK:
            switch(u32Cmd)
            {
                case RIUINFO_DUMP:
                    MdbPrint(paraCmdLine->u64ReqHdl,"\nBank: 0x%x\n\nBit8/16\n\n", u16Bank);
                    for(i=0;i<128;i+=8)
                    {
                        u16TmpRegs[0] = READ_WORD(RIU16BIT(virtBank,u16Bank,i));
                        u16TmpRegs[1] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+1)));
                        u16TmpRegs[2] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+2)));
                        u16TmpRegs[3] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+3)));
                        u16TmpRegs[4] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+4)));
                        u16TmpRegs[5] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+5)));
                        u16TmpRegs[6] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+6)));
                        u16TmpRegs[7] = READ_WORD(RIU16BIT(virtBank,u16Bank,(i+7)));
                        MdbPrint(paraCmdLine->u64ReqHdl," h%x%x/%x%x %x%x%x%x %x%x%x%x %x%x%x%x %x%x%x%x %x%x%x%x %x%x%x%x %x%x%x%x %x%x%x%x\n",    \
                                                (MS_U8)(i*2&0xf0)>>4, (MS_U8)(i*2&0xf), (MS_U8)(i&0xf0)>>4, (MS_U8)(i&0xf),                         \
                                                (u16TmpRegs[0]&0xf000)>>12, (u16TmpRegs[0]&0xf00)>>8, (u16TmpRegs[0]&0xf0)>>4, (u16TmpRegs[0]&0xf), \
                                                (u16TmpRegs[1]&0xf000)>>12, (u16TmpRegs[1]&0xf00)>>8, (u16TmpRegs[1]&0xf0)>>4, (u16TmpRegs[1]&0xf), \
                                                (u16TmpRegs[2]&0xf000)>>12, (u16TmpRegs[2]&0xf00)>>8, (u16TmpRegs[2]&0xf0)>>4, (u16TmpRegs[2]&0xf), \
                                                (u16TmpRegs[3]&0xf000)>>12, (u16TmpRegs[3]&0xf00)>>8, (u16TmpRegs[3]&0xf0)>>4, (u16TmpRegs[3]&0xf), \
                                                (u16TmpRegs[4]&0xf000)>>12, (u16TmpRegs[4]&0xf00)>>8, (u16TmpRegs[4]&0xf0)>>4, (u16TmpRegs[4]&0xf), \
                                                (u16TmpRegs[5]&0xf000)>>12, (u16TmpRegs[5]&0xf00)>>8, (u16TmpRegs[5]&0xf0)>>4, (u16TmpRegs[5]&0xf), \
                                                (u16TmpRegs[6]&0xf000)>>12, (u16TmpRegs[6]&0xf00)>>8, (u16TmpRegs[6]&0xf0)>>4, (u16TmpRegs[6]&0xf), \
                                                (u16TmpRegs[7]&0xf000)>>12, (u16TmpRegs[7]&0xf00)>>8, (u16TmpRegs[7]&0xf0)>>4, (u16TmpRegs[7]&0xf));
                    }
                    MdbPrint(paraCmdLine->u64ReqHdl,"\n\n");
                    break;
                case RIUINFO_BYTE_OFFSET_R: // 8 Bit addr, read, poll
                    i=0;
                    do {
                        u16TmpReg = READ_WORD(RIU8BIT(virtBank,u16Bank,u8Offset));
                        MdbPrint(paraCmdLine->u64ReqHdl,"\nReg 0x%x%x (8BitOffset) = 0x%x%x\n\n", u16Bank, u8Offset, (u8Offset&BIT(0))?((u16TmpReg&0xf000)>>12):((u16TmpReg&0xf0)>>4), (u8Offset&BIT(0))?((u16TmpReg&0xf00)>>8):(u16TmpReg&0xf));
                        if(FALSE == bFlagPoll)
                        {
                            break;
                        }
                        MsOS_DelayTask((MS_U32)u16Value);
                        i++;
                    } while(i<u16MaskOrBit);
                    break;
                case RIUINFO_WORD_OFFSET_R: // 16 Bit addr, read, poll
                    OUT_OF_RANGE(u8Offset,RIU16BIT_BOUND);
                    i=0;
                    do {
                        u16TmpReg = READ_WORD(RIU16BIT(virtBank,u16Bank,u8Offset));
                        MdbPrint(paraCmdLine->u64ReqHdl,"\nReg 0x%x%x (16BitOffset) = 0x%x%x%x%x\n\n", u16Bank, u8Offset, (u16TmpReg&0xf000)>>12, (u16TmpReg&0xf00)>>8, (u16TmpReg&0xf0)>>4, (u16TmpReg&0xf));
                        if(FALSE == bFlagPoll)
                        {
                            break;
                        }
                        MsOS_DelayTask((MS_U32)u16Value);
                        i++;
                    } while(i<u16MaskOrBit);
                    break;
                case RIUINFO_BYTE_OFFSET_W: // 8 Bit addr, write
                    if(u8Offset&BIT(0))
                    {
                        u16Value = u16Value << 8;
                        u16MaskOrBit = u16MaskOrBit << 8;
                    }
                    u16TmpReg = READ_WORD(RIU8BIT(virtBank,u16Bank,u8Offset));
                    u16TmpReg = (u16TmpReg&(~u16MaskOrBit))|(u16Value&u16MaskOrBit);
                    WRITE_WORD(RIU8BIT(virtBank,u16Bank,u8Offset), u16TmpReg);
                    break;
                case RIUINFO_BYTE_OFFSET_WBIT: // 8 Bit addr, bit write
                    OUT_OF_RANGE(u16MaskOrBit,8);
                    if(u8Offset&BIT(0))
                    {
                        u16MaskOrBit += 8;
                    }
                    u16TmpReg = READ_WORD(RIU8BIT(virtBank,u16Bank,u8Offset));
                    u16TmpReg = (u16Value>0)?((u16TmpReg&(~(BIT(u16MaskOrBit))))|((BIT(u16MaskOrBit)))):((u16TmpReg&(~(BIT(u16MaskOrBit))))&(~(BIT(u16MaskOrBit))));
                    WRITE_WORD(RIU8BIT(virtBank,u16Bank,u8Offset), u16TmpReg);
                    break;
                case RIUINFO_WORD_OFFSET_W: // 16 Bit addr, write
                    OUT_OF_RANGE(u8Offset,RIU16BIT_BOUND);
                    u16TmpReg = READ_WORD(RIU16BIT(virtBank,u16Bank,u8Offset));
                    u16TmpReg = (u16TmpReg&(~u16MaskOrBit))|(u16Value&u16MaskOrBit);
                    WRITE_WORD(RIU16BIT(virtBank,u16Bank,u8Offset), u16TmpReg);
                    break;
                case RIUINFO_WORD_OFFSET_WBIT: // 16 Bit addr, bit write
                    OUT_OF_RANGE(u8Offset,RIU16BIT_BOUND);
                    OUT_OF_RANGE(u16MaskOrBit,16);
                    u16TmpReg = READ_WORD(RIU16BIT(virtBank,u16Bank,u8Offset));
                    u16TmpReg = (u16Value>0)?((u16TmpReg&(~(BIT(u16MaskOrBit))))|((BIT(u16MaskOrBit)))):((u16TmpReg&(~(BIT(u16MaskOrBit))))&(~(BIT(u16MaskOrBit))));
                    WRITE_WORD(RIU16BIT(virtBank,u16Bank,u8Offset), u16TmpReg);
                    break;
                default:
                    break;
            }
CMD_DONE:
            paraCmdLine->result = MDBRESULT_SUCCESS_FIN;
            break;
        case MDBCMD_GETINFO:
            paraGetInfo = (MDBCMD_GETINFO_PARAMETER *)pArgs;
            RIUMdbHelp(paraGetInfo->u64ReqHdl);
            paraGetInfo->result = MDBRESULT_SUCCESS_FIN;
            break;
        default:
            paraGetInfo = (MDBCMD_GETINFO_PARAMETER *)pArgs;
            MdbPrint(paraGetInfo->u64ReqHdl,"unknown cmd\n");
            break;
    }
    return 0;
}

#endif

//--------------------------------------------------------------------------------------------------
// Utopia2.0 will call this function to register module
//--------------------------------------------------------------------------------------------------
#ifdef MSOS_TYPE_LINUX_KERNEL
extern MS_U32 QOSMdbIoctl(MS_U32 cmd, const void* const pArgs);
#endif
void SYSRegisterToUtopia(FUtopiaOpen ModuleType)
{
    // 1. create a module(module_name, SHM_size), and register to utopia2.0
    void* pUtopiaModule = NULL;
    UtopiaModuleCreate(MODULE_SYS, 0, &pUtopiaModule);
    UtopiaModuleRegister(pUtopiaModule);

    // register func for module, after register here, then ap call UtopiaOpen/UtopiaIoctl/UtopiaClose can call to these registered standard func
    UtopiaModuleSetupFunctionPtr(pUtopiaModule, (FUtopiaOpen)SYSOpen, (FUtopiaClose)SYSClose, (FUtopiaIOctl)SYSIoctl);
#ifdef CONFIG_UTOPIA_PROC_DBG_SUPPORT
    UtopiaModuleRegisterMdbNode("riuinfo", (FUtopiaMdbIoctl)RIUMdbIoctl);
#ifdef MSOS_TYPE_LINUX_KERNEL
    UtopiaModuleRegisterMdbNode("qos", (FUtopiaMdbIoctl)QOSMdbIoctl);
#endif
#endif
    // 2. Resource register
    void* psResource = NULL;

    // start func to add resources of a certain Resource_Pool
    UtopiaModuleAddResourceStart(pUtopiaModule, SYS_POOL_ID_SYS0);

    // create a resouce and regiter it to a certain Resource_Pool, resource can alloc private for internal use
    UtopiaResourceCreate("sys0", sizeof(SYS_RESOURCE_PRIVATE), &psResource);
#ifdef CONFIG_MSTAR_DVFS_ENABLE
    ((SYS_RESOURCE_PRIVATE*) psResource)->bDvfsInitFlag = 0;
#endif
    UtopiaResourceRegister(pUtopiaModule, psResource, SYS_POOL_ID_SYS0);

    //UtopiaResourceCreate("sys1", sizeof(SYS_RESOURCE_PRIVATE), &psResource);
    //UtopiaResourceRegister(pUtopiaModule, psResource, SYS_POOL_ID_SYS0);

    // end func to add resources of a certain Resource_Pool(this will set the ResourceSemaphore of this ResourcePool)
    UtopiaModuleAddResourceEnd(pUtopiaModule, SYS_POOL_ID_SYS0);
}

//--------------------------------------------------------------------------------------------------
// Utopia2.0 will call this function to get a instance to use SYS
// @ \b in: 32ModuleVersion => this is for checking if API version is the same
//--------------------------------------------------------------------------------------------------
MS_U32 SYSOpen(void** ppInstance, void* pAttribute)
{
    // for multi-process safe, check if already other Instance exist
    // 1. use moduleID to get module, then try to get resource
    void *pModule = NULL;
    void *pResource = NULL;
    UtopiaModuleGetPtr(MODULE_SYS, &pModule);

    if(UtopiaResourceObtain(pModule, SYS_POOL_ID_SYS0, &pResource) != 0)
    {
        ULOGE(TAG_SYS, "UtopiaResourceObtainToInstant fail\n");
        return UTOPIA_STATUS_FAIL;
    }

    // if get a resource, check the module private SHM(default to be 0, only create at the first time) to decide whether this process can open instance
    void *pSYSResPri = NULL;
    UtopiaResourceGetPrivate(pResource, &pSYSResPri);

    SYS_DBG_INFO("\033[35mFunction = %s, Line = %d, current resource pri_shm content is: %u\033[m\n", __PRETTY_FUNCTION__, __LINE__, *(unsigned int *)pSYSResPri);

    //if(*(MS_U32 *)pSYSResPri == 0)
    {
#ifdef CONFIG_MSTAR_DVFS_ENABLE
        if(((SYS_RESOURCE_PRIVATE*) pSYSResPri)->bDvfsInitFlag == 0)
        {
            ((SYS_RESOURCE_PRIVATE*) pSYSResPri)->bDvfsInitFlag = 0;
            g_bDvfsInitFlag = 0;
        }
        else
        {
            g_bDvfsInitFlag = 1;
        }
#endif
    }

    UtopiaResourceRelease(pResource);

    SYS_DBG_INFO("\033[35mFunction = %s, Line = %d, [SYS INFO] OPEN INSTANCE...\033[m\n", __PRETTY_FUNCTION__, __LINE__);
    SYS_INSTANT_PRIVATE *pSYSPri = NULL;

    // instance is allocated here, also can allocate private for internal use, ex, SYS_INSTANT_PRIVATE, ppInstance will point to a pointer of the created UTOPIA_INSTANCE
    UtopiaInstanceCreate(sizeof(SYS_INSTANT_PRIVATE), ppInstance);
    // set the pSYSPri point to the private of UTOPIA_INSTANCE
    UtopiaInstanceGetPrivate(*ppInstance, (void**)&pSYSPri);

    // setup func in private and assign the calling func in func ptr in instance private
    pSYSPri->fpSYS_GetChipRev                       = (IOCTL_SYS_GETCHIPREV)MDrv_SYS_GetChipRev;
    pSYSPri->fpSYS_GetChipID                        = (IOCTL_SYS_GETCHIPID)MDrv_SYS_GetChipID;
    pSYSPri->fpSYS_GetInfo                          = (IOCTL_SYS_GETINFO)MDrv_SYS_GetInfo;
    pSYSPri->fpSYS_GetLibVer                        = (IOCTL_SYS_GETLIBVER)MDrv_SYS_GetLibVer;
    pSYSPri->fpSYS_GetStatus                        = (IOCTL_SYS_GETSTATUS)MDrv_SYS_GetStatus;
    pSYSPri->fpSYS_GetSoftwareVersion               = (IOCTL_SYS_GETSOFTWAREVERSION)MDrv_SYS_GetSoftwareVersion;
    pSYSPri->fpSYS_DisableDebugPort                 = (IOCTL_SYS_DISABLEDEBUGPORT)MDrv_SYS_DisableDebugPort;
    pSYSPri->fpSYS_EnableDebugPort                  = (IOCTL_SYS_ENABLEDEBUGPORT)MDrv_SYS_EnableDebugPort;
    pSYSPri->fpSYS_SetDbgLevel                      = (IOCTL_SYS_SETDBGLEVEL)MDrv_SYS_SetDbgLevel;
    pSYSPri->fpSYS_WDTEnable                        = (IOCTL_SYS_WDTENABLE)MDrv_SYS_WDTEnable;
    pSYSPri->fpSYS_WDTClear                         = (IOCTL_SYS_WDTCLEAR)MDrv_SYS_WDTClear;
    pSYSPri->fpSYS_WDTLastStatus                    = (IOCTL_SYS_WDTLASTSTATUS)MDrv_SYS_WDTLastStatus;
    pSYSPri->fpSYS_WDTSetTime                       = (IOCTL_SYS_WDTSETTIME)MDrv_SYS_WDTSetTime;
    pSYSPri->fpSYS_ResetChip                        = (IOCTL_SYS_RESETCHIP)MDrv_SYS_ResetChip;
    pSYSPri->fpSYS_ResetCPU                         = (IOCTL_SYS_RESETCPU)MDrv_SYS_ResetCPU;
    pSYSPri->fpSYS_SetAGCPadMux                     = (IOCTL_SYS_SETAGCPADMUX)MDrv_SYS_SetAGCPadMux;
    pSYSPri->fpSYS_SetPCMCardDetectMode             = (IOCTL_SYS_SETPCMCARDDETECTMODE)MDrv_SYS_SetPCMCardDetectMode;
    pSYSPri->fpSYS_SetPadMux                        = (IOCTL_SYS_SETPADMUX)MDrv_SYS_SetPadMux;
    pSYSPri->fpSYS_SetTSOutClockPhase               = (IOCTL_SYS_SETTSOUTCLOCKPHASE)MDrv_SYS_SetTSOutClockPhase;
    pSYSPri->fpSYS_Query                            = (IOCTL_SYS_QUERY)MDrv_SYS_Query;
#ifdef _SYS_DAC_GAIN_CTRL
    pSYSPri->fpSYS_EnhanceDACGain                   = (IOCTL_SYS_ENHANCEDACGAIN)MDrv_SYS_EnhanceDACGain;
#endif
    pSYSPri->fpSYS_Init                             = (IOCTL_SYS_INIT)MDrv_SYS_Init;
    pSYSPri->fpSYS_GlobalInit                       = (IOCTL_SYS_GLOBALINIT)MDrv_SYS_GlobalInit;
#if defined (MSOS_TYPE_LINUX) || defined (MSOS_TYPE_LINUX_KERNEL)
    pSYSPri->fpSYS_TeecInitContext                  = (IOCTL_SYS_TEEC_INIT_CONTEXT)MDrv_SYS_TEEC_InitializeContext;
    pSYSPri->fpSYS_TeecOpen                         = (IOCTL_SYS_TEEC_OPEN)MDrv_SYS_TEEC_Open;
    pSYSPri->fpSYS_TeecClose                        = (IOCTL_SYS_TEEC_CLOSE)MDrv_SYS_TEEC_Close;
    pSYSPri->fpSYS_TeecInvokeCmd                    = (IOCTL_SYS_TEEC_INVOKE_CMD)MDrv_SYS_TEEC_InvokeCmd;
    pSYSPri->fpSYS_TeecFinalizeContext              = (IOCTL_SYS_TEEC_FINAL_CONTEXT)MDrv_SYS_TEEC_FinalizeContext;
#endif

    return UTOPIA_STATUS_SUCCESS;
}

MS_U32 SYSIoctl(void* pInstance, MS_U32 u32Cmd, void* pArgs)
{
    void* pModule = NULL;
    UtopiaInstanceGetModule(pInstance, &pModule);

    SYS_INSTANT_PRIVATE *psSYSInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psSYSInstPri);

    PSYS_GETSTATUS_PARAM pGetStatusParam = NULL;
    PSYS_GETSOFTWAREVERSION_PARAM pGetSoftwareVersionParam = NULL;
    PSYS_SETDBGLEVEL_PARAM pSetDbgLevelParam = NULL;
    PSYS_WDTENABLE_PARAM pWDTEnableParam = NULL;
    PSYS_WDTSETTIME_PARAM pWDTSetTimeParam = NULL;
    PSYS_SETAGCPADMUX_PARAM pSetAGCPadMuxParam = NULL;
    PSYS_SETPCMCARDDETECT_PARAM pSetPCMCardDetectModeParam = NULL;
    PSYS_SETPADMUX_PARAM pSetPadMuxParam = NULL;
    PSYS_SETTSOUTCLOCKPHASE_PARAM pSetTSOutClockPhaseParam = NULL;
    PSYS_QUERY_PARAM pQueryParam = NULL;
#ifdef _SYS_DAC_GAIN_CTRL
    PSYS_ENHANCEDACGAIN_PARAM pEnhanceDACGainParam = NULL;
#endif
#if defined (MSOS_TYPE_LINUX) || defined (MSOS_TYPE_LINUX_KERNEL)
    PSYS_SYS_TEECOPEN_PARAM pTeecOpenParam = NULL;
    PSYS_SYS_TEECCLOSE_PARAM pTeecCloseParam = NULL;
    PSYS_SYS_TEECINVOKECMD_PARAM pTeecInvokeCmdParam = NULL;
    PSYS_SYS_TEECINITCONTEXT_PARAM pTeecInitContextParam = NULL;
#endif

    MS_U32 u32Ret;

    switch(u32Cmd)
    {
        case MDrv_CMD_SYS_GetChipRev:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetChipRev\n");
            *((MS_U8 *)(pArgs)) = psSYSInstPri->fpSYS_GetChipRev();
            return TRUE;
        case MDrv_CMD_SYS_GetChipID:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetChipID\n");
            *((MS_U16 *)(pArgs)) = psSYSInstPri->fpSYS_GetChipID();
            return TRUE;
        case MDrv_CMD_SYS_GetInfo:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetInfo\n");
            memcpy(pArgs, psSYSInstPri->fpSYS_GetInfo(), sizeof(const SYS_Info));
            return TRUE;
        case MDrv_CMD_SYS_GetLibVer:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetLibVer\n");
            memcpy(pArgs, psSYSInstPri->fpSYS_GetLibVer(), sizeof(const MSIF_Version));
            return TRUE;
        case MDrv_CMD_SYS_GetStatus:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetStatus\n");
            pGetStatusParam = (PSYS_GETSTATUS_PARAM)pArgs;
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_GetStatus(pGetStatusParam->pStatus);
            return u32Ret;
        case MDrv_CMD_SYS_GetSoftwareVersion:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GetSoftwareVersion\n");
            pGetSoftwareVersionParam = (PSYS_GETSOFTWAREVERSION_PARAM)pArgs;
            psSYSInstPri->fpSYS_GetSoftwareVersion(pGetSoftwareVersionParam->pSoftwareVersionInfo);
            return TRUE;
        case MDrv_CMD_SYS_DisableDebugPort:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_DisableDebugPort\n");
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_DisableDebugPort();
            return u32Ret;
        case MDrv_CMD_SYS_EnableDebugPort:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_EnableDebugPort\n");
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_EnableDebugPort();
            return u32Ret;
        case MDrv_CMD_SYS_SetDbgLevel:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_SetDbgLevel\n");
            pSetDbgLevelParam = (PSYS_SETDBGLEVEL_PARAM)pArgs;
            psSYSInstPri->fpSYS_SetDbgLevel(pSetDbgLevelParam->eLevel);
            return TRUE;
        case MDrv_CMD_SYS_WDTEnable:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_WDTEnable\n");
            pWDTEnableParam = (PSYS_WDTENABLE_PARAM)pArgs;
            psSYSInstPri->fpSYS_WDTEnable(pWDTEnableParam->bEnable);
            return TRUE;
        case MDrv_CMD_SYS_WDTClear:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_WDTClear\n");
            psSYSInstPri->fpSYS_WDTClear();
            return TRUE;
        case MDrv_CMD_SYS_WDTLastStatus:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_WDTLastStatus\n");
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_WDTLastStatus();
            return u32Ret;
        case MDrv_CMD_SYS_WDTSetTime:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_WDTSetTime\n");
            pWDTSetTimeParam = (PSYS_WDTSETTIME_PARAM)pArgs;
            psSYSInstPri->fpSYS_WDTSetTime(pWDTSetTimeParam->u32Ms);
            return TRUE;
        case MDrv_CMD_SYS_ResetChip:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_ResetChip\n");
            psSYSInstPri->fpSYS_ResetChip();
            return TRUE;
        case MDrv_CMD_SYS_ResetCPU:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_ResetCPU\n");
            psSYSInstPri->fpSYS_ResetCPU();
            return TRUE;
        case MDrv_CMD_SYS_SetAGCPadMux:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_SetAGCPadMux\n");
            pSetAGCPadMuxParam = (PSYS_SETAGCPADMUX_PARAM)pArgs;
            psSYSInstPri->fpSYS_SetAGCPadMux(pSetAGCPadMuxParam->eAgcPadMux);
            return TRUE;
        case MDrv_CMD_SYS_SetPCMCardDetectMode:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_SetPCMCardDetectMode\n");
            pSetPCMCardDetectModeParam = (PSYS_SETPCMCARDDETECT_PARAM)pArgs;
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_SetPCMCardDetectMode(pSetPCMCardDetectModeParam->ePCMCDMode);
            return u32Ret;
        case MDrv_CMD_SYS_SetPadMux:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_SetPadMux\n");
            pSetPadMuxParam = (PSYS_SETPADMUX_PARAM)pArgs;
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_SetPadMux(pSetPadMuxParam->ePadMuxType, pSetPadMuxParam->ePadSel);
            return u32Ret;
        case MDrv_CMD_SYS_SetTSOutClockPhase:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_SetTSOutClockPhase\n");
            pSetTSOutClockPhaseParam = (PSYS_SETTSOUTCLOCKPHASE_PARAM)pArgs;
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_SetTSOutClockPhase(pSetTSOutClockPhaseParam->u16Val);
            return u32Ret;
        case MDrv_CMD_SYS_Query:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_Query\n");
            pQueryParam = (PSYS_QUERY_PARAM)pArgs;
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_Query(pQueryParam->id);
            return u32Ret;
#ifdef _SYS_DAC_GAIN_CTRL
        case MDrv_CMD_SYS_EnhanceDACGain:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_EnhanceDACGain\n");
            pEnhanceDACGainParam = (PSYS_ENHANCEDACGAIN_PARAM)pArgs;
            psSYSInstPri->fpSYS_EnhanceDACGain(pEnhanceDACGainParam->u8DAC_ID, pEnhanceDACGainParam->bEnable);
            return TRUE;
#endif
        case MDrv_CMD_SYS_Init:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_Init\n");
            u32Ret = (MS_U32)psSYSInstPri->fpSYS_Init();
            return u32Ret;
        case MDrv_CMD_SYS_GlobalInit:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_GlobalInit\n");
            psSYSInstPri->fpSYS_GlobalInit();
            return TRUE;
#if defined (MSOS_TYPE_LINUX) || defined (MSOS_TYPE_LINUX_KERNEL)
        case MDrv_CMD_SYS_TeecInitContext:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_TeecInitContext\n");
            pTeecInitContextParam = (PSYS_SYS_TEECINITCONTEXT_PARAM)pArgs;
            u32Ret = psSYSInstPri->fpSYS_TeecInitContext(pTeecInitContextParam->name, pTeecInitContextParam->context);
            return u32Ret;
        case MDrv_CMD_SYS_TeecOpen:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_TeecOpen\n");
            pTeecOpenParam = (PSYS_SYS_TEECOPEN_PARAM)pArgs;
            u32Ret = psSYSInstPri->fpSYS_TeecOpen(pTeecOpenParam->context, pTeecOpenParam->session, pTeecOpenParam->destination, pTeecOpenParam->connection_method, pTeecOpenParam->connection_data, pTeecOpenParam->operation, pTeecOpenParam->error_origin);
            return u32Ret;
        case MDrv_CMD_SYS_TeecClose:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_TeecClose\n");
            pTeecCloseParam = (PSYS_SYS_TEECCLOSE_PARAM)pArgs;
            psSYSInstPri->fpSYS_TeecClose(pTeecCloseParam->session);
            return TRUE;
        case MDrv_CMD_SYS_TeecInvokeCmd:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_TeecInvokeCmd\n");
            pTeecInvokeCmdParam = (PSYS_SYS_TEECINVOKECMD_PARAM)pArgs;
            u32Ret = psSYSInstPri->fpSYS_TeecInvokeCmd(pTeecInvokeCmdParam->session, pTeecInvokeCmdParam->cmd_id, pTeecInvokeCmdParam->operation, pTeecInvokeCmdParam->error_origin);
            return u32Ret;
        case MDrv_CMD_SYS_TeecFinalContext:
            ULOGD(TAG_SYS, "SYSIoctl - MDrv_CMD_SYS_TeecFinalContext\n");
            pTeecInitContextParam = (PSYS_SYS_TEECINITCONTEXT_PARAM)pArgs;
            psSYSInstPri->fpSYS_TeecFinalizeContext(pTeecInitContextParam->context);
            return TRUE;
#endif
        default:
            break;
    };

   return 0; // FIXME: error code
}

MS_U32 SYSClose(void* pInstance)
{
    UtopiaInstanceDelete(pInstance);

    // Restore resource pri_shm content
    // 1. use moduleID to get module, then try to get resource
    void *pModule = NULL;
    void *pResource = NULL;
    UtopiaModuleGetPtr(MODULE_SYS, &pModule);

    if(UtopiaResourceObtain(pModule, SYS_POOL_ID_SYS0, &pResource) != 0)
    {
        ULOGE(TAG_SYS, "UtopiaResourceObtainToInstant fail\n");
        return FALSE;
    }

    // if get a resource, check the module private SHM(default to be 0, only create at the first time) to decide whether this process can open instance
    void *pSYSResPri = NULL;
    UtopiaResourceGetPrivate(pResource, &pSYSResPri);

    if(*(MS_U32 *)pSYSResPri == 0)
    {
        ULOGE(TAG_SYS, "\033[35mFunction = %s, Line = %d, [SYSClose] Strange resource pri_shm content!!\033[m\n", __PRETTY_FUNCTION__, __LINE__);
        UtopiaResourceRelease(pResource);
        return FALSE;
    }
    else
    {
        ULOGD(TAG_SYS, "\033[35mFunction = %s, Line = %d, [Multi-process Safe] Release an Instance!!\033[m\n", __PRETTY_FUNCTION__, __LINE__);
        *(MS_U32 *)pSYSResPri = 0;
        UtopiaResourceRelease(pResource);
        return TRUE;
    }
}
