//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// @file   mdrv_xc_st.h
// @brief  XC KMD Driver Interface
// @author MStar Semiconductor Inc.
//////////////////////////////////////////////////////////////////////////////////////////////////


#ifndef _MDRV_XC_ST_H
#define _MDRV_XC_ST_H

//=============================================================================
// Includs
//=============================================================================

//=============================================================================
// Type and Structure Declaration
//=============================================================================
#define PQ_GAMMA_LUT_ENTRY (256)
#define PQ_GAMMA_CHANNEL  (3)
#define LEN_HSY_CONFIG    (2)
#define ST_KDRV_CFD_HSY_VERSION (1)
#define ST_KDRV_CFD_HSY_TABLE_VERSION (1)
#define ST_KDRV_CFD_HSY_TABLE_ENTRY_VERSION (1)

#define KDRV_XC_HDMI_HDR_METADATA_VERSION 0
#define KDRV_XC_PANEL_FORMAT_VERSION 0
/// CFD OSD structure version
/// Version 0: Init structure
/// Version 1: Add backlight and hue/contrast/saturation valid flag
/// Version 2: Add color range and main/sub window.
/// Version 3: Add ultra black / white
/// Version 4: Add skip picture setting (hue/ contrast/ saturation)
/// Version 5: Add color correction matrix
/// Version 6: Add RGB Matrix
/// Version 7: Add view Mode (switch view mode for dolby)
#define KDRV_XC_OSD_CONTROL_VERSION  (9)
#define KDRV_XC_MAIN_CONTROL_VERSION 0
#define KDRV_XC_HDMI_INFOFRAME_PARSER_VERSION 0
#define KDRV_XC_HDMI_EDID_PARSER_VERSION 0
#define KDRV_XC_MM_PARSER_VERSION 0
/// CFD HDMI structure version
/// Version 0: Init structure
/// Version 1: Add Additional Colorimetry for AVI info version 4
#define KDRV_XC_CFD_HDMI_VERSION 1
/// CFD Panel structure version
/// Version 0: Init structure
/// Version 1: Add Linear RGB and customer color primaries setting
/// Version 2: Add PWM port & dolby global dimming on/off & dolby global delay frame count.
#define KDRV_XC_CFD_PANEL_VERSION 2
#define KDRV_XC_CFD_HDR_VERSION 0
#define KDRV_XC_CFD_EDID_VERSION 0
#define KDRV_XC_CFD_FIRE_VERSION 0
// Version 1 : Initial structure
// Version 2 : add HDMI port info
// Version 3: Add HVSP Status
// Version 4: Add HVSP Sub1 mode info
#define KDRV_XC_CFD_XC_STATUS_VERSION (4)
#define ST_KDRV_XC_CFD_SET_OUTPUT_COLORIMETRY_VERSION (1)
#define ST_KDRV_XC_CFD_SET_OUTPUT_HDR_TYPE_VERSION (1)
///CFD ip histogram
/// Version 0: Init structure
#define KDRV_XC_CFD_TMO_CUS_MAPPING_VERSION (1)
#define ST_KDRV_XC_PQ_GAMMA_IN_VERSION (3)
#define ST_KDRV_XC_PQ_RGB_PATTERN_IN_VERSION (1)
#define ST_KDRV_XC_PQ_BLUE_STRETCH_IN_VERSION (2)
#define ST_KDRV_XC_CFD_NR_VERSION (1)
/// AUTODOWNLOAD_FORMAT_INFO version of current XC lib
/// version 2 : add bDataParsed
/// version 3 : add enPQGammaFormat
/// version 4 : add Section index
#define ST_KDRV_XC_AUTODOWNLOAD_FORMAT_INFO_VERSION          (4)
#define ST_KDRV_XC_AUTODOWNLOAD_DATA_INFO_VERSION          (1)
/// set manual luma curve version of current XC lib
/// version 2 : add u16ManualBlendFactor
#define ST_KDRV_XC_PQ_MANUAL_LUMA_CURVE_VERSION (2)
#define ST_KDRV_XC_PQ_LUMA_INFO_VERSION (1)
#define ST_KDRV_XC_PQ_STRETCH_SETTINGS_VERSION (1)
#define ST_KDRV_XC_PQ_CHROMA_INFO_VERSION (1)
#define HISTOGRAM_SATOFHUE_WIN_NUM (6)
#define ST_KDRV_XC_PQ_NONLINEARSCALING_API_PARAM_IN_VERSION (1)
#define KDRV_XC_CFD_XC_GET_SHARPNESS_VERSION (1)
#define ST_KDRV_XC_PQ_SEND_CMD_VERSION (1)
#define ST_KDRV_XC_PQ_SEND_CMD_BIN_VERSION (1)
//version 1 : initial
//version 2 : add hdmi port info
#define KDRV_XC_DOVI_SOURCE_SET_SINK_EDID_VERSION (2)

#define KDRV_XC_EDID_EXTENDED_TAG_CODE_DATA_BLOCK_MAX_SIZE (32)
#define ST_KDRV_XC_AUTODOWNLOAD_TABLE_VERSION (1)
#define ST_KDRV_XC_ML_CMD_TABLE_VERSION (1)
#define ST_KDRV_XC_ML_CMDS_TABLE_VERSION (1)

#if defined (CONFIG_MSTAR_XC_HDR_1_4_0) || defined (CONFIG_MSTAR_XC_HDR_1_4_2)
#define IS_DOLBY_VERSION_140_OR_142 TRUE
#else
#define IS_DOLBY_VERSION_140_OR_142 FALSE
#endif

#define HSY2_DRAMTOSRAM_BASE (16) //128 bit

#define PQ_SPF_XRULE_OP_START_INDEX (1)
#define PQ_SPF_XRULE_UCM_START_INDEX (2)

// Kernel XC Event Structure
// Version 1 : Initial structure
#define ST_KDRV_XC_EVENT_VERSION (1)

/// Version 0: Init structure
/// Version 1: Add content is full range and ultra black & white active
/// Version 2: Add color type of the source
/// Version 3: Add HDR metadata of the source(ott or hdmi)
/// Version 4: Add HDR Mdebug Information
/// Version 5: Add TMO Color Type
/// Version 6: Add HDR Seamless Information
/// Version 7: Add BOX Output Type
/// Version 8: Add XC HW Point Color info
/// Version 9: Add HDMI VSIF Type
typedef enum
{
    E_KDRV_XC_CFD_STATUS_VERSION_INIT,
    E_KDRV_XC_CFD_STATUS_VERSION_FULL_RANGE,
    E_KDRV_XC_CFD_STATUS_VERSION_COLOR_TYPE,
    E_KDRV_XC_CFD_STATUS_VERSION_HDR_METADATA,
    E_KDRV_XC_CFD_STATUS_VERSION_HDR_MDEBUG,
    E_KDRV_XC_CFD_STATUS_VERSION_TMO_COLOR_TYPE,
    E_KDRV_XC_CFD_STATUS_VERSION_HDR_SEAMLESS,
    E_KDRV_XC_CFD_STATUS_VERSION_BOX_OUTPUT_TYPE,
    E_KDRV_XC_CFD_STATUS_VERSION_POINT_COLOR_INFO,
    E_KDRV_XC_CFD_STATUS_VERSION_HDMI_VSIF_TYPE,
    E_KDRV_XC_CFD_STATUS_VERSION_MAX,
} EN_KDRV_XC_CFD_STATUS_VERSION;

#define ST_KDRV_XC_CFD_STATUS_VERSION (E_KDRV_XC_CFD_STATUS_VERSION_HDMI_VSIF_TYPE)
/// Version 1: Init structure
#define ST_KDRV_XC_HDR_CFD_POINT_INFOR_VERSION (1)

// DLC Version
// Version 1: Initial Structure
#define ST_KDRV_DLC_GET_HIST32INFO_VERSION  (1)
#define ST_KDRV_DLC_GET_HISTMINMAX_VERSION  (1)
#define ST_KDRV_DLC_GET_LUMICOUNT_VERSION   (1)
#define ST_KDRV_DLC_GET_LUMACURVE16_VERSION (1)

#define DLC_HISTOGRAMINFO (32)
#define DLC_LUMACURVE     (16)

//UCD related
#define UCD_CURVE_NUM    (162)
#define UCD_INDEX_NUM    (4096)
#define UCD_VERSION      (0)

//UCD mode
#define UCD_MODE_DEFAULT (0)
#define UCD_MODE_LG      (1)

#define ST_KDRV_XC_CFD_XRULE_VERSION (1)
#define ST_KDRV_XC_CFD_DISP_PATH_VERSION (1)
#define ST_KDRV_DS_SET_DSFORCEINDEX_EXT_INFO_VERSION (1)

typedef struct
{
    MS_BOOL bCurveChange;  // Curve Change
    MS_U8  u8LumaCurve[16];  // Default luma curve from Supernova DLC.ini
    MS_U8  u8LumaCurve2_a[16];  // Default luma curve 2a  from Supernova DLC.ini
    MS_U8  u8LumaCurve2_b[16];  // Default luma curve 2b  from Supernova DLC.ini
    MS_U8  u8Result; //Out
} ST_KDRV_XC_DLC_INFO;

typedef struct
{
    MS_U64 u64VRAddress;  // Default VR Address from Supernova MMAP
    MS_U32 u32VRSize;  // Default VR Size from Supernova MMAP
    MS_U8  u8Result; //Out
} ST_KDRV_XC_DLC_SET_VR_ADDRESS;

typedef struct
{
    MS_BOOL bBLEPointChange;  // BLE Point Change
    MS_U16  u16BLESlopPoint[6];  // Default BLE Slop Point from Supernova DLC.ini
    MS_U8  u8Result; //Out
} ST_KDRV_XC_DLC_BLE_INFO;

typedef struct
{
    MS_BOOL bInitInfoChange;  // Init Info Change

    // Default luma curve
    MS_U8 au8LumaCurve[16];
    MS_U8 au8LumaCurve2_a[16];
    MS_U8 au8LumaCurve2_b[16];
    MS_U8 au8DlcHistogramLimitCurve[17];

    MS_U8 u8DlcPureImageMode; // Compare difference of max and min bright
    MS_U8 u8DlcLevelLimit; // n = 0 ~ 4 => Limit n levels => ex. n=2, limit 2 level 0xF7, 0xE7
    MS_U8 u8DlcAvgDelta; // n = 0 ~ 50, default value: 12
    MS_U8 u8DlcAvgDeltaStill; // n = 0 ~ 15 => 0: disable still curve, 1 ~ 15 => 0.1 ~ 1.5 enable still curve
    MS_U8 u8DlcFastAlphaBlending; // min 17 ~ max 32
    MS_U8 u8DlcYAvgThresholdL; // default value: 0
    MS_U8 u8DlcYAvgThresholdH; // default value: 128

    MS_U8 u8DlcBLEPoint; // n = 24 ~ 64, default value: 48
    MS_U8 u8DlcWLEPoint; // n = 24 ~ 64, default value: 48
    MS_U8 bEnableBLE : 1; // 1: enable; 0: disable
    MS_U8 bEnableWLE : 1; // 1: enable; 0: disable
    MS_U8 u8DlcYAvgThresholdM;
    MS_U8 u8DlcCurveMode;
    MS_U8 u8DlcCurveModeMixAlpha;
    MS_U8 u8DlcAlgorithmMode;
    MS_U8 u8DlcSepPointH;
    MS_U8 u8DlcSepPointL;
    MS_U16 u16DlcBleStartPointTH;
    MS_U16 u16DlcBleEndPointTH;
    MS_U8 u8DlcCurveDiff_L_TH;
    MS_U8 u8DlcCurveDiff_H_TH;
    MS_U16 u16DlcBLESlopPoint_1;
    MS_U16 u16DlcBLESlopPoint_2;
    MS_U16 u16DlcBLESlopPoint_3;
    MS_U16 u16DlcBLESlopPoint_4;
    MS_U16 u16DlcBLESlopPoint_5;
    MS_U16 u16DlcDark_BLE_Slop_Min;
    MS_U8 u8DlcCurveDiffCoringTH;
    MS_U8 u8DlcAlphaBlendingMin;
    MS_U8 u8DlcAlphaBlendingMax;
    MS_U8 u8DlcFlicker_alpha;
    MS_U8 u8DlcYAVG_L_TH;
    MS_U8 u8DlcYAVG_H_TH;
    MS_U8 u8DlcDiffBase_L;
    MS_U8 u8DlcDiffBase_M;
    MS_U8 u8DlcDiffBase_H;
    // NEW DLC
    MS_U8 u8LMaxThreshold;
    MS_U8 u8LMinThreshold;
    MS_U8 u8LMaxCorrection;
    MS_U8 u8LMinCorrection;
    MS_U8 u8RMaxThreshold;
    MS_U8 u8RMinThreshold;
    MS_U8 u8RMaxCorrection;
    MS_U8 u8RMinCorrection;
    MS_U8 u8AllowLoseContrast;
} ST_KDRV_XC_DLC_INIT_INFO;

typedef struct
{
    MS_U32 u32OutputNits[512];
    MS_U16 u16InputPoint[512];
    MS_U16 u16ControlPoints;
    MS_S16 s16LastLess1IndexOutputnits;
    MS_S16 s16LastIndexM10000Outputs;
    MS_S16 s16LastIndexM100Outputs;
}ST_KDRV_XC_DLC_TMO_1DLUT;


// HDR new tone mapping parameters.
typedef struct
{
    // TMO
    MS_U16 u16SrcMinRatio;          //default 10
    MS_U16 u16SrcMedRatio;          //default 512
    MS_U16 u16SrcMaxRatio;          //default 990

    MS_U16 u16TgtMin;               //default 500
    MS_U16 u16TgtMed;
    MS_U16 u16TgtMax;               //default 300

    MS_U16 u16FrontSlopeMin;        //default 256
    MS_U16 u16FrontSlopeMax;        //default 512
    MS_U16 u16BackSlopeMin;         //default 128
    MS_U16 u16BackSlopeMax;         //default 256

    MS_U16 u16SceneChangeThrd;      //default 1024
    MS_U16 u16SceneChangeRatioMax;  //default 1024

    MS_U8 u8IIRRatio;            //default 31
    MS_U8 u8TMOTargetMode;      // default 0. 0 : keeps the value in initial function  1 : from output source
    MS_U16 u16SDRPanelGain;        //

    MS_U16 u16Smin;
    MS_U16 u16Smed;
    MS_U16 u16Smax;
    MS_U16 u16Tmin;
    MS_U16 u16Tmed;
    MS_U16 u16Tmax;

    MS_BOOL bRefMode;

    MS_U8 u8TMOUseIniControls;
    ST_KDRV_XC_DLC_TMO_1DLUT stTMO1DLUT;

} ST_KDRV_XC_TMO_INFO;
#define XC_HSE_INFO_VERSION (2)
typedef struct
{
    // HSE structure version
    MS_U32 u32Version;
    // HSE buffer adddress
    MS_U32 u32PhyAddress;
    // HSE bufffer size
    MS_U32 u32Length;
    // HSE buffer MIU select
    MS_U8  u8MiuSel;
    // HSE init done
    MS_BOOL bInitDone;
    // HSE buffer adddress
    MS_U64 u64PhyAddress;
} ST_KDRV_XC_HSE_INFO;

typedef enum
{
    E_KDRV_HDMI_EMP_DOLBY_HDR,
    E_KDRV_HDMI_EMP_DSC,
    E_KDRV_HDMI_EMP_DYNAMIC_HDR,
    E_KDRV_HDMI_EMP_VRR,
    E_KDRV_HDMI_EMP_QFT,
    E_KDRV_HDMI_EMP_QMS,

    //MAX
    E_KDRV_HDMI_EMP_MAX
} EN_KDRV_HDMI_EMP_TYPE;

typedef struct
{
    MS_U8 enHDMIPort;
    EN_KDRV_HDMI_EMP_TYPE enEmpType;
    MS_U8 u8CurrentPacketIndex;
    MS_U8 *pu8TotalPacketNumber;
    MS_U8 *pu8EmPacket;
} ST_KDRV_XC_EMP_INFO;

typedef enum
{
    E_BRIGHTNESS_RED = 0,
    E_BRIGHTNESS_GREEN,
    E_BRIGHTNESS_BLUE,
    E_BRIGHTNESS_MAX,
}EN_BRIGHTNESS_CHANNEL_TYPE;

typedef enum
{
    E_KDRV_XC_HDR_OUTPUT_MODE_AUTO  = 0x0,
    E_KDRV_XC_HDR_OUTPUT_MODE_ON    = 0x1,
    E_KDRV_XC_HDR_OUTPUT_MODE_OFF   = 0x2,
    E_KDRV_XC_HDR_OUTPUT_MODE_MAX,
} EN_KDRV_XC_HDR_OUTPUT_MODE;

typedef enum
{
    EN_KDRV_XC_HDR_DMA_BYPASS,
    EN_KDRV_XC_HDR_DMA_ENABLE,
    EN_KDRV_XC_HDR_DMA_MAX,
} EN_KDRV_XC_HDR_DMA_PATH;

//IO Ctrl struct defines:
typedef enum
{
    E_KDRV_XC_HDR_NONE,
    E_KDRV_XC_HDR_DOLBY,
    E_KDRV_XC_HDR_OPEN,
    E_KDRV_XC_HDR_TECHNICOLOR,
    E_KDRV_XC_HDR_DOLBY_LL,
    E_KDRV_XC_HDR_HLG,
    E_KDRV_XC_HDR_MAX,
} EN_KDRV_XC_HDR_TYPE;

typedef enum
{
    E_KDRV_XC_INPUT_SOURCE_NONE,
    E_KDRV_XC_INPUT_SOURCE_HDMI,
    E_KDRV_XC_INPUT_SOURCE_OTT,
    E_KDRV_XC_INPUT_SOURCE_DTV,
    E_KDRV_XC_INPUT_SOURCE_MAX,
} EN_KDRV_XC_INPUT_SOURCE_TYPE;

typedef enum
{
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_OP2GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_FRCOP2GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_XVYCC,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_ODTABLE1,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_ODTABLE2,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_ODTABLE3,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_DEMURA,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_OP2LUT,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_T3D_0,        /// t3d of sc0
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_T3D_1,        /// t3d of sc1
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_FRCSPTPOPM,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_FOOPM,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HVSP,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HVSP_DIP,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_SCRAMBLE,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_THDR,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_ADL2RIU,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HSY,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_PQ_GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_PANEL_GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_DLC_256,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_ICC_IHC,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_3DLUT,         ///Dolby, TMO
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_SC2VIP,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_RGBWGEN2,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_OD,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_PCID,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_UCD_STAT_TOP,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_GUCD_B2P,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_GUCD_18_LUT,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_GUCD_256_LUT,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_DC0_FILM_GRAIN,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HFR_HDR,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HFR_3DLUT,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HFR_ADL2RIU,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HFR_VIP,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_HSY2,
    E_KDRV_XC_AUTODOWNLOAD_CLIENT_MAX,
} EN_KDRV_XC_AUTODOWNLOAD_CLIENT;

typedef enum
{
    E_KDRV_XC_AUTOUPLOAD_CLIENT_YHSL,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_YLITE,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_ZNRME,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_ZNRMI,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_HFR_YHSL,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_HFR_YLITE,
    E_KDRV_XC_AUTOUPLOAD_CLIENT_MAX,
}EN_KDRV_XC_AUTOUPLOAD_CLIENT;

typedef enum
{
    E_KDRV_XC_AUTODOWNLOAD_TRIGGER_MODE,
    E_KDRV_XC_AUTODOWNLOAD_ENABLE_MODE,
} EN_KDRV_XC_AUTODOWNLOAD_MODE;

typedef enum
{
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_TMO,
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_3DLUT,
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_DEGAMMA,
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OOTF,            // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_RGB3DLUT,
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0105,           // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0202_TMLUTI,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0202_TMLUTS,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0202_SMLUTI,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0202_SMLUTS,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_B0103,           // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OSD_B0103,           // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OSD_B0202_TMLUTI,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OSD_B0202_TMLUTS,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OSD_B0202_SMLUTI,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_OSD_B0202_SMLUTS,    // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_HDR
    E_KDRV_XC_AUTODOWNLOAD_HDR_SUB_TOSD_DEGAMMA,        // for E_KDRV_XC_AUTODOWNLOAD_CLIENT_OSD_THDR
    E_KDRV_XC_AUTODOWNLOAD_ICC_IHC_SUB_ICC,
    E_KDRV_XC_AUTODOWNLOAD_ICC_IHC_SUB_IHC,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_TMO,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_3DLUT,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_DEGAMMA,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_OOTF,
    E_KDRV_XC_AUTODOWNLOAD_HFR_HDR_SUB_RGB3DLUT,

    E_KDRV_XC_AUTODOWNLOAD_SUB_CLIENT_MAX
} EN_KDRV_XC_AUTODOWNLOAD_SUB_CLIENT;

typedef enum
{
    E_KDRV_XC_AUTODOWNLOAD_XVYCC_SUB_GAMMA,
    E_KDRV_XC_AUTODOWNLOAD_XVYCC_SUB_DEGAMMA
} EN_KDRV_XC_AUTODOWNLOAD_XVYCC_SUB_CLIENT;

typedef enum
{
    /// each data is 12bit, the table size is 384*8bit
    E_KDRV_XC_ADL_PQ_GAMMA_12BIT_UNFORMATTED = 0,
    /// each data is 12bit, the table size is 256*16bit
    E_KDRV_XC_ADL_PQ_GAMMA_12BIT_FORMATTED,
    /// each data is 14bit, the table size is 256*16bit
    E_KDRV_XC_ADL_PQ_GAMMA_14BIT_FORMATTED,
    // Max enum value
    E_KDRV_XC_ADL_PQ_GAMMA_FORMAT_MAX,
} EN_KDRV_XC_ADL_PQ_GAMMA_FORMAT;

typedef enum
{
    E_KDRV_XC_HDR_COLOR_NONE,
    E_KDRV_XC_HDR_COLOR_YUV420,
    E_KDRV_XC_HDR_COLOR_YUV422,
    E_KDRV_XC_HDR_COLOR_YUV444,
    E_KDRV_XC_HDR_COLOR_RGB,
    E_KDRV_XC_HDR_COLOR_MAX,
} EN_KDRV_XC_HDR_COLOR_FORMAT;

typedef enum
{
    E_KDRV_MLOAD_UNSUPPORTED = 0,
    E_KDRV_MLOAD_DISABLED    = 1,
    E_KDRV_MLOAD_ENABLED     = 2,
} EN_KDRV_MLOAD_TYPE;

typedef enum
{
    E_CLIENT_MAIN_XC = 0,
    E_CLIENT_MAIN_HDR = 1,
    E_CLIENT_SUB_XC = 2,
    E_CLIENT_SUB_HDR = 3,
    E_CLIENT_MAIN_DEFLICKER = 4,
    /// For MultiWindow Usage
    E_CLIENT_MULTIWIN_SETTING = 5,
    E_CLIENT_MULTIWIN_SETTING_NEXTFRAME = 6,
    E_CLIENT_MULTIWIN_STR = 7,
    E_CLIENT_MULTIWIN_LDR = 8,
    E_CLIENT_MAIN_DOLBY = 9, // for hdmi dolby
    E_CLIENT_MAIN_DLC = 10,  // Add for GCE

    E_CLIENT_2D_MENULOAD_00 = 0x10,//the first num:triger point, the sencend num:client.
    E_CLIENT_2D_MENULOAD_01,
    E_CLIENT_2D_MENULOAD_02 ,
    E_CLIENT_2D_MENULOAD_03,
    E_CLIENT_2D_MENULOAD_04,
    E_CLIENT_2D_MENULOAD_05,
    E_CLIENT_2D_MENULOAD_10,
    E_CLIENT_2D_MENULOAD_11,
    E_CLIENT_2D_MENULOAD_12,
    E_CLIENT_2D_MENULOAD_13,
    E_CLIENT_2D_MENULOAD_14,
    E_CLIENT_2D_MENULOAD_15,
    E_CLIENT_2D_MENULOAD_20,
    E_CLIENT_2D_MENULOAD_21,
    E_CLIENT_2D_MENULOAD_22,
    E_CLIENT_2D_MENULOAD_23,
    E_CLIENT_2D_MENULOAD_24,
    E_CLIENT_2D_MENULOAD_25,
    E_CLIENT_2D_MENULOAD_30,
    E_CLIENT_2D_MENULOAD_31,
    E_CLIENT_2D_MENULOAD_32,
    E_CLIENT_2D_MENULOAD_33,
    E_CLIENT_2D_MENULOAD_34,
    E_CLIENT_2D_MENULOAD_35,
    E_CLIENT_2D_MENULOAD_40,
    E_CLIENT_2D_MENULOAD_41,
    E_CLIENT_2D_MENULOAD_42,
    E_CLIENT_2D_MENULOAD_43,
    E_CLIENT_2D_MENULOAD_44,
    E_CLIENT_2D_MENULOAD_45,

    E_CLIENT_MAX,
} EN_MLOAD_CLIENT_TYPE;

typedef enum
{
    E_KDRV_MAIN_WINDOW = 0,
    E_KDRV_SUB_WINDOW,

    E_KDRV_MULTI_WINDOW0,          /// multiWindow0
    E_KDRV_MULTI_WINDOW1,          /// multiWindow1
    E_KDRV_MULTI_WINDOW2,          /// multiWindow2
    E_KDRV_MULTI_WINDOW3,          /// multiWindow3
    E_KDRV_MULTI_WINDOW4,          /// multiWindow4
    E_KDRV_MULTI_WINDOW5,          /// multiWindow5
    E_KDRV_MULTI_WINDOW6,          /// multiWindow6
    E_KDRV_MULTI_WINDOW7,          /// multiWindow7

    E_KDRV_MULTI_WINDOW8,          /// multiWindow8
    E_KDRV_MULTI_WINDOW9,          /// multiWindow9
    E_KDRV_MULTI_WINDOW10,         /// multiWindow10
    E_KDRV_MULTI_WINDOW11,         /// multiWindow11
    E_KDRV_MULTI_WINDOW12,         /// multiWindow12
    E_KDRV_MULTI_WINDOW13,         /// multiWindow13
    E_KDRV_MULTI_WINDOW14,         /// multiWindow14
    E_KDRV_MULTI_WINDOW15,         /// multiWindow15
    E_KDRV_MAX_WINDOW,
}EN_KDRV_WINDOW;

typedef enum
{
    E_KDRV_XC = 0,
    E_KDRV_HDR,
}EN_KDRV_DS_CLIENT_TYPE;

typedef enum
{
    E_KDRV_XC_SC0 = 0,
    E_KDRV_XC_SC1,
    E_KDRV_XC_MAX,
}EN_KDRV_SC_DEVICE;

typedef enum //DS HW
{
    E_KDRV_DS_HW0 = 0,
    E_KDRV_DS_HW1,
    E_KDRV_DS_NUM,
} EN_KDRV_DS_DEVICE;

typedef enum
{
    E_K_APIXC_RET_FAIL = 0,                   ///<Fail
    E_K_APIXC_RET_OK,                         ///<OK
    E_K_APIXC_RET_FAIL_FUN_NOT_SUPPORT,       ///<Fail
    E_K_APIXC_RET_FAIL_ENUM_NOT_SUPPORT,      ///<Fail
    E_K_APIXC_RET_FAIL_INVALID_PARAMETER,     ///<Fail
} E_K_APIXC_ReturnValue;

typedef enum
{
    E_K_XC_DS_INDEX_MVOP,
    E_K_XC_DS_INDEX_SCALER,
    E_K_XC_DS_INDEX_MAX,
}E_K_XC_DS_INDEX_SOURCE;

typedef struct
{
    MS_U16 u16CMDCNT_IPM;
    MS_U16 u16CMDCNT_IPS;
    MS_U16 u16CMDCNT_OPM;
    MS_U16 u16CMDCNT_OPS;
    MS_U16 u16CMDCNT_FRCIP;
    MS_U16 u16CMDCNT_FRCOP;
    MS_U16 u16CMDCNT_UCMR;
} K_XC_DS_CMDCNT;

typedef enum
{
    E_MLOAD_TRIGGER_BY_OP_SYNC=0,
    E_MLOAD_TRIGGER_BY_IP_MAIN_SYNC=1,
    E_MLOAD_TRIGGER_BY_IP_SUB_SYNC=2,
    E_MLOAD_TRIGGER_BY_SW=3,
    E_MLOAD_TRIGGER_BY_OP2_SYNC=4,
    E_MLOAD_TRIGGER_BY_ISR_OP_SYNC=5,
    E_MLOAD_TRIGGER_MAX,
} EN_MLOAD_TRIG_SYNC;

typedef enum
{
    E_MLOAD_2D_TRIGER0 = 0,
    E_MLOAD_2D_TRIGER1 = 1,
    E_MLOAD_2D_TRIGER2 = 2,
    E_MLOAD_2D_TRIGER3 = 3,
    E_MLOAD_2D_TRIGER4 = 4,
    E_MLOAD_2D_TRIGERMAX,
}EN_MLOAD_TRIGER;

typedef enum
{
    E_KDRV_XC_DLC_SOURCE_IP     = 0,
    E_KDRV_XC_DLC_SOURCE_VDEC   = 1,
    E_KDRV_XC_DLC_SOURCE_VIP    = 2,
    E_KDRV_XC_DLC_SOURCE_MAX,
} EN_KDRV_XC_DLC_SOURCE;

typedef struct __attribute__((packed))
{
    MS_U64  u64DS_Info_BaseAddr;         ///< The memory base address to update IP/OP dynamic scaling registers. Absolute without any alignment.
    MS_U8  u8MIU_Select;                ///< 0: MIU0, 1: MIU1, etc.
    MS_U8  u8DS_Index_Depth;            ///< The index depth (how many entries to fire per MVOP Vsync falling) of DS
    MS_BOOL bOP_DS_On;                  ///< OP dynamic scaling on/off
    MS_BOOL bIPS_DS_On;                 ///< IPS dynamic scaling on/off
    MS_BOOL bIPM_DS_On;                 ///< IPM dynamic scaling on/off
    MS_U32  u32DSBufferSize;            ///< Ds buffer size
    EN_KDRV_DS_DEVICE enDSDeviceID;        ///<DSDeviceID
} K_XC_DynamicScaling_Info;

typedef enum
{
    K_DS_IP,
    K_DS_OP,
    K_DS_FRC_IP,
    K_DS_FRC_OP,
    K_DS_UCMR,
    K_DS_IPOP_MAX,
} k_ds_reg_ip_op_sel;

typedef enum
{
    K_DS_XC,
    K_DS_MVOP,
    K_DS_GOP,
} k_ds_reg_source_sel;

//*****************DS NEW*****************//
typedef enum
{
    E_DS_CLIENT_XC,
    E_DS_CLIENT_HDR,
    E_DS_CLIENT_PQ,
    E_DS_CLIENT_DSWINDOW,
    E_DS_CLIENT_MAX,
} E_DS_CLIENT;

typedef struct
{
    E_DS_CLIENT client;
    MS_U32  max_num;
} K_DS_register_info;

typedef struct __attribute__((packed))
{
    //MVOP
    MS_U32 u32MFCodecInfo;

    //XC
    MS_BOOL bUpdate_DS_CMD;
    MS_BOOL bEnableDNR;
    MS_U32  u32DSBufferSize;
    MS_BOOL bEnable_ForceP;
    MS_BOOL bDynamicScalingEnable;

    // DS HW limitation, for sub DS can only get DS index from mvop0
    // TRUE: force MVOP1 to SC0, MVOP0 to SC1
    // FALSE: MVOP0 to SC0, MVOP1 to SC1
    MS_BOOL bSwitchMVOPToSCPath;
    MS_BOOL bFBL;
    MS_BOOL bR_FBL;

    //DSDeviceID
    EN_KDRV_DS_DEVICE enDSDeviceID;
} K_XC_SETDS_Info;

typedef struct __attribute__((packed))
{
    MS_BOOL bDynamicScalingEnable;
    //DSDeviceID
    EN_KDRV_DS_DEVICE enDSDeviceID;
    MS_BOOL bUpdate_DS_CMD;
} K_XC_GETDS_Info;
//****************************************//
typedef enum
{
    // New
    /// Initialize
    E_KDRV_XC_CFD_CTRL_SET_INIT = 0,
    /// VGA
    E_KDRV_XC_CFD_CTRL_SET_VGA = 1,
    /// TV (ATV)
    E_KDRV_XC_CFD_CTRL_SET_TV = 2,
    /// CVBS (AV)
    E_KDRV_XC_CFD_CTRL_SET_CVBS = 3,
    /// S-Video
    E_KDRV_XC_CFD_CTRL_SET_SVIDEO = 4,
    /// YPbPr
    E_KDRV_XC_CFD_CTRL_SET_YPBPR = 5,
    /// Scart
    E_KDRV_XC_CFD_CTRL_SET_SCART = 6,
    /// HDMI
    E_KDRV_XC_CFD_CTRL_SET_HDMI = 7,
    /// DTV
    E_KDRV_XC_CFD_CTRL_SET_DTV = 8,
    /// DVI
    E_KDRV_XC_CFD_CTRL_SET_DVI = 9,
    /// MM
    E_KDRV_XC_CFD_CTRL_SET_MM = 10,
    /// Panel
    E_KDRV_XC_CFD_CTRL_SET_PANEL = 11,
    /// HDR
    E_KDRV_XC_CFD_CTRL_SET_HDR = 12,
    /// EDID
    E_KDRV_XC_CFD_CTRL_SET_EDID = 13,
    /// OSD
    E_KDRV_XC_CFD_CTRL_SET_OSD = 14,
    /// Fire
    E_KDRV_XC_CFD_CTRL_SET_FIRE = 15,
    /// DLC
    E_KDRV_XC_CFD_CTRL_SET_DLC = 16,
    /// Linear RGB
    E_KDRV_XC_CFD_CTRL_SET_LINEAR_RGB = 17,
    /// Get CFD Output to HDMI Status and dicison.
    E_KDRV_XC_CFD_CTRL_GET_HDMI_STATUS = 18,
    /// Set force HDR Onoff or auto select setting
    E_KDRV_XC_CFD_CTRL_SET_HDR_ONOFF_SETTING = 19,
    /// Status
    E_KDRV_XC_CFD_CTRL_GET_STATUS = 20,
    /// TMO
    E_KDRV_XC_CFD_CTRL_SET_TMO = 21,
    /// Update Status
    E_KDRV_XC_CFD_CTRL_UPDATE_STATUS = 22,
    /// Get Picture Attributes
    E_KDRV_XC_CFD_CTRL_PICTURE_ATTRIBUTE = 23,
    /// Update Info into CFD
    E_KDRV_XC_CFD_CTRL_UPDATE_INFO = 24,
    /// Set HDMI Seamless
    E_KDRV_XC_CFD_CTRL_SET_HDMI_SEAMLESS = 26,
    /// Get Status of Paring HDMI Packet
    E_KDRV_XC_CFD_CTRL_GET_PARSING_HDMI_PACKET = 27,
    /// Set QMAP Data
    E_KDRV_XC_CFD_CTRL_SET_QMAPDATA = 28,
    /// Get Histogram
    E_KDRV_XC_CFD_CTRL_GET_IP_HISTOGRAM = 29,
    ///3DLUT
    E_KDRV_XC_CFD_CTRL_3DLUT = 30,
    ///EOTF
    E_KDRV_XC_CFD_CTRL_EOTF = 31,
    ///OETF
    E_KDRV_XC_CFD_CTRL_OETF = 32,
    ///HLG GAIN
    E_KDRV_XC_CFD_CTRL_HLGGAIN = 33,
    ///GAMMA enable/disable
    E_KDRV_XC_CFD_CTRL_GAMMA_ENABLE = 34,
    ///Gamut Pre Control
    E_KDRV_XC_CFD_CTRL_GAMUT_PREVIOUS = 35,
    ///Gamut Post Control
    E_KDRV_XC_CFD_CTRL_GAMUT_POST = 36,
    ///SDR Gamma Control
    E_KDRV_XC_CFD_CTRL_SDR_GAMMA = 37,
    ///SDR Degamma Control
    E_KDRV_XC_CFD_CTRL_SDR_DEGAMMA = 38,
    ///Set Yc offset and gain in
    E_KDRV_XC_CFD_CTRL_YCOFFSETGAININ = 39,
    ///Set Yc offset and gain out
    E_KDRV_XC_CFD_CTRL_YCOFFSETGAINOUT = 40,
    ///Set Yc offset and gain in Sub
    E_KDRV_XC_CFD_CTRL_YCOFFSETGAININSUB = 41,
    ///SDR 3DLUT Control
    E_KDRV_XC_CFD_CTRL_SDR_3DLUT = 42,
    /// CFD_CTRL_PICTURE_MODE
    E_KDRV_XC_CFD_CTRL_PICTURE_MODE = 43,
    /// Set Customer IP parameter
    E_KDRV_XC_CFD_CTRL_UPDATE_CUSTOMER_SETTING = 44,
    /// Get CFD version
    E_KDRV_XC_CFD_CTRL_GET_CFD_VERSION = 45,
    /// Update XC status into CFD, refer to ST_KDRV_XC_STATUS_INFO
    E_KDRV_XC_CFD_CTRL_UPDATE_XC_STATUS = 46,
    /// CFD_CTRL_TMO_CUS_MAPPING
    E_KDRV_XC_CFD_CTRL_TMO_CUS_MAPPING = 47,
    //NR SETTINGS
    E_KDRV_XC_CFD_CTRL_SET_NR_SETTINGS = 48,
    /// Set Metadata Path
    E_KDRV_XC_CFD_SET_METADATA_PATH = 49,
    /// Set Metadata to CFD
    E_KDRV_XC_CFD_SET_METADATA = 50,
    //set manual luma curve
    E_KDRV_XC_CFD_CTRL_SET_MANUAL_LUMA_CURVE = 51,
    //Get Luma Info
    E_KDRV_XC_CFD_CTRL_GET_LUMA_INFO = 52,
    //Set Stretch Settings
    E_KDRV_XC_CFD_CTRL_SET_STRETCH_SETTINGS = 53,
    //Get Chroma Info
    E_KDRV_XC_CFD_CTRL_GET_CHROMA_INFO = 54,
    //Set hash Info
    E_KDRV_XC_CFD_CTRL_SET_HASH_INFO = 55,
    //Send PQ cmd
    E_KDRV_XC_CFD_CTRL_SEND_PQ_CMD = 56,
    //Send PQ cmd bin
    E_KDRV_XC_CFD_CTRL_SEND_PQ_CMD_BIN = 57,
    //Get Sharpness
    E_KDRV_XC_CFD_CTRL_GET_SHARPNESS = 58,
    /// Set HSY Table to CFD
    E_KDRV_XC_CFD_CTRL_SET_HSY_TABLE = 59,
    /// HSY Main Rule
    E_KDRV_XC_CFD_CTRL_SET_HSY = 60,
    //Set UCD Info
    E_KDRV_XC_CFD_CTRL_UCD_INFO = 61,
    //Set SPF_XRULE
    E_KDRV_XC_CFD_CTRL_SPF_XRULE = 62,
    // Get PQ cmd Value
    E_KDRV_XC_CFD_CTRL_READ_PQ_CMD_VALUE = 63,
    // Set Mod Settings
    E_KDRV_XC_CFD_CTRL_SET_MODULE_SETTINGS = 64,
    // Get Mod Status
    E_KDRV_XC_CFD_CTRL_GET_MODULE_STATUS = 65,
    //Set display path
    E_KDRV_XC_CFD_CTRL_SET_DISP_PATH = 66,
    //Set Scaler Path Control Info
    E_KDRV_XC_CFD_CTRL_MUX_INFO = 67,
    //Set HDR attr
    E_KDRV_XC_CFD_CTRL_SET_HDR_ATTR = 68,
    // Set Dimming Panel Type
    E_KDRV_XC_CFD_CTRL_SET_DIMMING_PANEL_TYPE = 69,

    /// CFD CTRL_TYPE_EX start
    /// (Set this base is 0x40000000 for int type)
    E_KDRV_XC_CFD_CTRL_TYPE_EX_BASE = 0x40000000,
    /// OSD_STATUS
    E_KDRV_XC_CFD_CTRL_GET_OSD_PROCESS_CONFIGS = 0x40000001,
    /// HDR Output Seamless
    E_KDRV_XC_CFD_CTRL_SET_OUTPUT_SEAMLESS = 0x40000002,
    /// HDR Output Colorimetry
    E_KDRV_XC_CFD_CTRL_SET_OUTPUT_COLORIMETRY = 0x40000003,
    /// HDR Output HDR Type
    E_KDRV_XC_CFD_CTRL_SET_OUTPUT_HDR_TYPE = 0x40000004,
    /// Enable XC Kernel Log
    E_KDRV_XC_CFD_CTRL_SET_DEBUG_LEVEL = 0x40000005,

    /// For Dolby Vision Source Device API
    /// (Set this base is 0x41000000 for DOVI SOURCE)
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_TYPE_BASE = 0x41000000,
    /// Dolby Vision source Initialize
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_INIT = 0x41000001,
    /// Dolby Vision source get HDR associated Information
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_GET_HDR_INFORMATION = 0x41000002,
    /// Dolby Vision source set output format
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_SET_OUTPUT_FORMAT = 0x41000003,
    /// Dolby Vision source set control InfoFrame
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_SET_INFOFRAME_CONTROL = 0x41000004,
    /// Dolby Vision source set Sink EDID Data
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_SET_SINK_EDID = 0x41000005,
    /// Dolby Vision source reset HDR settings
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_RESET = 0x41000006,
    /// Dolby Vision source get Auxiliary Video Information Infoframe Data
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_GET_AVI_INFOFRAME_DATA = 0x41000007,
    /// Dolby Vision source set graphic parameters to dovi driver
    E_KDRV_XC_CFD_CTRL_DOVI_SOURCE_SET_GRAPHIC_PARAM = 0x41000008,

    /// Max
    E_KDRV_XC_CFD_CTRL_MAX,
} EN_KDRV_XC_CFD_CTRL_TYPE;

typedef enum
{
    E_KDRV_XC_HDR_METADATA_AUTO = 0,     ///Set Metadata to CFD by vdec, default path
    E_KDRV_XC_HDR_METADATA_STATIC = 1,   /// Set Metadata to CFD by uper layer, such as AN P
    E_KDRV_XC_HDR_METADATA_MAX,
} EN_KDRV_XC_HDR_METADATA_PATH;

typedef enum
{
    E_KDRV_XC_DIMMING_PANEL_TYPE_GLOBAL_DIMMING = 0,     ///Set Dimming panel to Global Dimming panel
    E_KDRV_XC_DIMMING_PANEL_TYPE_LOCAL_DIMMING = 1,   ///Set Dimming panel to Local Dimming panel 60hz
    E_KDRV_XC_DIMMING_PANEL_TYPE_LOCAL_DIMMING_WITH_URSA = 2, ///Set Dimming panel to Local Dimming panel 120hz
    E_KDRV_XC_DIMMING_PANEL_TYPE_MAX,
} EN_KDRV_XC_DIMMING_PANEL_TYPE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Reg Bank (IN)
    MS_U32 u32Bank;
    /// Reg Address (IN)
    MS_U32 u32Addr;
    /// Reg Data (IN)
    MS_U16 u16Data;
    ///Reg Mask (IN)
    MS_U16 u16Mask;
} ST_KDRV_XC_REG;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Control MultiWindow Window Number
    EN_KDRV_WINDOW enWindow;
    /// Control MultiWindow Node
    MS_U16 u16NodeIndex;
} ST_KDRV_XC_MLOAD_CONTROLINFO;

typedef enum
{
    /// Fire MultiWIndow, Get Fire Control REG
    E_KDRV_XC_MULTIWIN_FIRE,
    /// Get Wait Done REG for MultiWindow
    E_KDRV_XC_MULTIWIN_WAITDONE,
}EN_KDRV_XC_MULTIWIN_FIRETYPE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Control MultiWindow Window Number
    EN_KDRV_WINDOW enWindow;
    /// Control MultiWindow Node
    MS_U16 u16NodeIndex;
    /// Control MultiWindow Fire Type
    EN_KDRV_XC_MULTIWIN_FIRETYPE enFiretype;

    //Menuload Client Node Start Hbye Address (OUT)
    ST_KDRV_XC_REG stHbyteAddr;
    //Menuload Client Node Start Lbyte Address (OUT)
    ST_KDRV_XC_REG stLbyteAddr;
    //Menuload Command Depth (OUT)
    ST_KDRV_XC_REG stDepth;
    //Menuload Enable(OUT)
    ST_KDRV_XC_REG stEnable;
    //Menuload Triger(OUT)
    ST_KDRV_XC_REG stTriger;
    //Menuload Finish Polling Reg(OUT)
    ST_KDRV_XC_REG stWaitDone;
} ST_KDRV_XC_MLOAD_MULTIWIN_FIREINFO;

typedef struct __attribute__((packed))
{
    MS_U8* pu8Data;                     /// pointer to data
#if !defined (__aarch64__)
    void    *pDummy;
#endif
    MS_U32 u32Size;                     /// data size
} ST_KDRV_XC_OPEN_METADATA_INFO;

typedef struct __attribute__((packed))
{
    MS_U8* pu8Data;                     /// pointer to data
#if !defined (__aarch64__)
    void    *pDummy;
#endif
    MS_U32 u32Size;                     /// data size
} ST_KDRV_XC_3DLUT_INFO;

/// CFD TMO_CUS_MAPPING, ReGen the Source && Target curve for Mapping Video and Graphic.
typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Enable/Disable Mapping
    MS_BOOL bEnable;
} ST_KDRV_XC_CFD_TMO_CUS_MAPPING;

typedef struct
{
    MS_U8 u8Version;
    MS_U32 u32Size;
    MS_U32 *pu32RGBHistogram;
#if !defined (__aarch64__)
    void *pDummy;
#endif
}ST_KDRV_XC_RGBHistogram;

typedef struct
{
    MS_U64    phyBaseAddr;                      /// baseaddr
    MS_U32    u32Size;
    MS_U32    u32MiuNo;
} ST_KDRV_XC_SHARE_MEMORY_INFO;

typedef struct
{
    MS_U64    phyBaseAddr;                      /// baseaddr
    MS_U32    u32Size;
    MS_U32    u32MiuNo;
    MS_BOOL   bIsStartEmpDma;                   /// Start EMP DMA
} ST_KDRV_HDMI_SHARE_MEMORY_INFO;

typedef struct
{
    MS_U16 u16X;           ///<start x of the window
    MS_U16 u16Y;           ///<start y of the window
    MS_U16 u16Width;       ///<width of the window
    MS_U16 u16Height;      ///<height of the window
} ST_KDRV_XC_WINDOW_INFO;

typedef struct
{
    EN_KDRV_XC_AUTODOWNLOAD_CLIENT enClient; /// current client
    MS_U64 phyBaseAddr;                      /// baseaddr
    MS_U32 u32Size;                          /// size
    MS_U32 u32MiuNo;                         /// miu no
    MS_BOOL bEnable;                         /// enable/disable the client
    EN_KDRV_XC_AUTODOWNLOAD_MODE enMode;     /// work mode
} ST_KDRV_XC_AUTODOWNLOAD_CONFIG_INFO;

typedef struct
{
    MS_U32 u32FormatInfo_Version;               ///<Version of current structure
    MS_U16 u16FormatInfo_Length;                ///<Length of this structure

    EN_KDRV_XC_AUTODOWNLOAD_SUB_CLIENT enSubClient; /// sub client
    MS_BOOL bEnableRange;                           /// whether enable range, 0: write data to (0-511), 1: write data to (u32StartAddr-u16EndAddr)
    MS_U16  u16StartAddr;                            /// start addr, range: 0-511, startaddr <= endaddr
    MS_U16  u16EndAddr;                              /// end addr: range: 0-511, startaddr <= endaddr
    MS_BOOL bDataParsed;                      ///Is data dram format
    EN_KDRV_XC_ADL_PQ_GAMMA_FORMAT enPQGammaFormat;   //PQ Gamma Data Format
    MS_BOOL bIsSecIdxValid;                       /// section index is valid?
    MS_U8 u8AutoDownloadSecIdx;                 /// section index
} ST_KDRV_XC_AUTODOWNLOAD_FORMAT_INFO;

typedef struct
{
    EN_KDRV_XC_AUTODOWNLOAD_XVYCC_SUB_CLIENT enSubClient; /// sub client
} ST_KDRV_XC_AUTODOWNLOAD_XVYCC_FORMAT_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32DataInfo_Version;         ///<Version of current structure.
    MS_U16 u16DataInfo_Length;          ///<Length of this structure

    EN_KDRV_XC_AUTODOWNLOAD_CLIENT enClient;  /// current client
    MS_U8* pu8Data;                           /// pointer to data
#if !defined (__aarch64__)
    void    *pDummy;
#endif
    MS_U32 u32Size;                         /// data size
    void* pParam;                           /// pointer to ST_KDRV_XC_AUTODOWNLOAD_FORMAT_INFO
#if !defined (__aarch64__)
    void    *pDummy2;
#endif
} ST_KDRV_XC_AUTODOWNLOAD_DATA_INFO;

typedef struct
{
    EN_KDRV_XC_AUTODOWNLOAD_CLIENT enClient;  /// current client
    MS_BOOL bSupported;                       /// whether it is supported
} ST_KDRV_XC_AUTODOWNLOAD_CLIENT_SUPPORTED_CAPS;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_U64 PhyAddr;
    MS_U32 u32BufByteLen;
} ST_KDRV_MLOAD_INIT;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_BOOL bEnable;
} ST_KDRV_MLOAD_ENABLE;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    EN_KDRV_MLOAD_TYPE eReturn;
} ST_KDRV_MLOAD_GETSTATUS;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_BOOL bImmeidate;
    MS_BOOL bReturn;
} ST_KDRV_MLOAD_FIRE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    EN_MLOAD_CLIENT_TYPE enClientType;
    EN_MLOAD_TRIG_SYNC enTriggerSync;
    ST_KDRV_XC_MLOAD_CONTROLINFO *pstInfo;
#if !defined (__aarch64__)
    MS_U32    u32AlignDummy;
#endif
    MS_BOOL bReturn;
} ST_KDRV_XC_MLOAD_FIRE_EXT;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    EN_MLOAD_CLIENT_TYPE enClientType;
    ST_KDRV_XC_MLOAD_MULTIWIN_FIREINFO *pstInfo;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    MS_BOOL bReturn;
} ST_KDRV_XC_MLOAD_MULTIWINFIRE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    EN_MLOAD_CLIENT_TYPE enClientType;
    EN_KDRV_WINDOW enWindow;
    MS_BOOL bReturn;
} ST_KDRV_XC_MLOAD_MULTIWINRESET;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_U32 u32Addr;
    MS_U16 u16Data;
    MS_U16 u16Mask;
    MS_BOOL bReturn;
} ST_KDRV_MLOAD_WRT_CMD;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    EN_MLOAD_CLIENT_TYPE enClientType;
    EN_MLOAD_TRIG_SYNC enTriggerSync;
    ST_KDRV_XC_REG stKReg;
    ST_KDRV_XC_MLOAD_CONTROLINFO *pstInfo;
#if !defined (__aarch64__)
    MS_U32    u32AlignDummy;
#endif
    MS_BOOL bReturn;
} ST_KDRV_XC_MLOAD_WRT_CMD_EXT;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    ST_KDRV_XC_REG *stReg;
#if !defined (__aarch64__)
    MS_U32    u32AlignDummy;
#endif
    EN_KDRV_WINDOW enWindow;
    MS_BOOL bReturn;
} ST_KDRV_XC_MLOAD_READ_CMD;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_U32 u32Bank;
    MS_U32 u32Addr;
    MS_U16 u16Data;
    MS_U16 u16Mask;
    MS_BOOL bReturn;
} ST_KDRV_MLOAD_WRT_CMD_NONXC;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_U16 u16train;
    MS_U16 u16disp;
    MS_BOOL bReturn;
} ST_KDRV_MLOAD_SET_IP_TRIG_P;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    MS_U16 *pu16Train;
    MS_U16 *pu16Disp;
    MS_BOOL bReturn;
} ST_KDRV_MLOAD_GET_IP_TRIG_P;

typedef struct
{
    EN_MLOAD_CLIENT_TYPE _client_type;
    EN_MLOAD_TRIG_SYNC eTriggerSync;
} ST_KDRV_MLOAD_SET_TRIG_SYNC;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW  eWindow;
    E_K_APIXC_ReturnValue eReturn;
} ST_KDRV_DS_GET_DSForceIndexSupported;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    E_K_XC_DS_INDEX_SOURCE eDSIdxSrc;
    EN_KDRV_WINDOW  eWindow;
    E_K_APIXC_ReturnValue eReturn;
} ST_KDRV_DS_SET_DSIndexSourceSelect;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    MS_BOOL bEnable;
    MS_U8 u8Index;
    EN_KDRV_WINDOW  eWindow;
    E_K_APIXC_ReturnValue eReturn;
} ST_KDRV_DS_SET_DSForceIndex;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;                              ///Version of current structure. it should be ST_KDRV_DS_SET_DSForceIndex_EXT_INFO_VERSION as input
    MS_U32 u32Length;                               ///Length of current structure. it should be sizeof(ST_KDRV_DS_SET_DSForceIndexExtInfo)
    MS_BOOL bEnable;
    k_ds_reg_ip_op_sel enSelIPOrOP;
    MS_U8 u8Index;
    EN_KDRV_DS_DEVICE enDSDevice;
} ST_KDRV_DS_SET_DSForceIndexExtInfo;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    ST_KDRV_DS_SET_DSForceIndexExtInfo *pstDSForceIndexExtInfo;
    E_K_APIXC_ReturnValue enReturn;
} ST_KDRV_DS_SET_DSForceIndex_EXT;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    K_XC_DynamicScaling_Info *pstDSInfo;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    MS_U32 u32DSInfoLen;
    EN_KDRV_WINDOW  eWindow;
    MS_BOOL bReturn;
} ST_KDRV_DS_SetDynamicScaling;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    MS_BOOL bEnable;
} ST_KDRV_DS_SET_DynamicScalingFlag;

typedef struct
{
    EN_KDRV_SC_DEVICE enDeviceID;
    MS_BOOL bEnable;
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_SET_DynamicScalingFlag_EXT;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    MS_BOOL bReturn;
} ST_KDRV_DS_GetDynamicScalingStatus;

typedef struct
{
    EN_KDRV_SC_DEVICE enDeviceID;
    MS_BOOL bReturn;
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_GetDynamicScalingStatus_EXT;


typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    MS_BOOL bEnable;
    MS_BOOL bReturn;
} ST_KDRV_DS_EnableIPMTuneAfterDS;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
    E_DS_CLIENT client;
    MS_U32 u32CmdRegAddr;
    MS_U16 u16CmdRegValue;
    k_ds_reg_ip_op_sel IPOP_Sel;
    k_ds_reg_source_sel Source_Select;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_DS_WriteSWDSCommand;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    E_DS_CLIENT enClient;
    MS_U32 u32CmdRegAddr;
    MS_U16 u16CmdRegValue;
    k_ds_reg_ip_op_sel enSelIPOrOP;
    k_ds_reg_source_sel enSelSourceReg;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_WriteSWDSCommand_EXT;


typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
    E_DS_CLIENT client;
    MS_U32 u32Bank;
    MS_U32 u32Addr;
    MS_U16 u16Data;
    MS_U16 u16Mask;
    k_ds_reg_ip_op_sel IPOP_Sel;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_DS_WriteSWDSCommandNonXC;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    E_DS_CLIENT enClient;
    MS_U32 u32Bank;
    MS_U32 u32Addr;
    MS_U16 u16Data;
    MS_U16 u16Mask;
    k_ds_reg_ip_op_sel enSelIPOrOP;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_WriteSWDSCommandNonXC_EXT;


typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
    E_DS_CLIENT client;
    MS_U32 u32CmdRegAddr;
    MS_U16 u16CmdRegValue;
    MS_U16 u16Mask;
    k_ds_reg_ip_op_sel IPOP_Sel;
    k_ds_reg_source_sel Source_Select;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_DS_WriteSWDSCommand_Mask;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    E_DS_CLIENT enClient;
    MS_U32 u32CmdRegAddr;
    MS_U16 u16CmdRegValue;
    MS_U16 u16Mask;
    k_ds_reg_ip_op_sel enSelIPOrOP;
    k_ds_reg_source_sel enSelSourceReg;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_WriteSWDSCommand_Mask_EXT;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
    E_DS_CLIENT client;
    k_ds_reg_ip_op_sel IPOP_Sel;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_DS_Add_NullCommand;

typedef struct __attribute__((packed))
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    E_DS_CLIENT enClient;
    k_ds_reg_ip_op_sel enSelIPOrOP;
    K_XC_DS_CMDCNT *pstXC_DS_CmdCnt;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_Add_NullCommand_EXT;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    E_DS_CLIENT client;
    MS_U32 max_num;
    MS_BOOL bReturn;
} ST_KDRV_DS_SET_CLIENT;

typedef struct
{
    EN_KDRV_SC_DEVICE enDeviceID;
    E_DS_CLIENT enClient;
    MS_U32 u32MaxNum;
    MS_BOOL bReturn;
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_SET_CLIENT_EXT;

typedef struct
{
    MS_U8 u8SupportIndexNum;
} ST_KDRV_DS_GET_SUPPORT_INDEX_NUM;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
    MS_U8 u8DSIndex;
} ST_KDRV_DS_GET_INDEX_INFO;

typedef struct
{
    EN_KDRV_SC_DEVICE u32DeviceID;
    EN_KDRV_WINDOW eWindow;
} ST_KDRV_DS_FIRE_INDEX_INFO;

typedef struct
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_FIRE_INDEX_INFO_EXT;

/// Define Window position and size attribute
/// Note!!!! For CFD plus, DO not use this struct, Plz use Vdec Sharememory Struct
typedef struct
{
    MS_U16 u16x;           ///<start x of the window
    MS_U16 u16y;           ///<start y of the window
    MS_U16 u16width;       ///<width of the window
    MS_U16 u16height;      ///<height of the window
} ST_KDRV_WINDOW_TYPE;

typedef struct
{
    MS_U8   u8CurrentIndex;
    MS_U64  phyRegAddr;      // HDR register
    MS_U32  u32RegSize;
    MS_U64  phyLutAddr;      // look-up table
    MS_U32  u32LutSize;
    MS_BOOL bDMEnable;
    MS_BOOL bCompEnable;
    ST_KDRV_WINDOW_TYPE stCapWin;//real source resolution
    MS_BOOL bIsNeedSetWindow; //Is need setwindow or not
    MS_BOOL bIsAISRValid;
} ST_KDRV_XC_DS_HDRInfo;

typedef struct __attribute__((packed))
{
    EN_KDRV_WINDOW eWindow;
    EN_KDRV_SC_DEVICE u32DeviceID;
    K_XC_SETDS_Info *stDSInfo;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_SET_XC_DS_Info;

typedef struct __attribute__((packed))
{
    EN_KDRV_WINDOW eWindow;
    EN_KDRV_SC_DEVICE u32DeviceID;
    K_XC_GETDS_Info *stGetDSInfo;
#if !defined (__aarch64__)
    MS_U32    pDummy;
#endif
} ST_KDRV_GET_XC_DS_Info;

typedef struct  __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U16 au16Histogram[DLC_HISTOGRAMINFO];
} ST_KDRV_DLC_GET_HIST32INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U8  u8HistogramMin;
    MS_U8  u8HistogramMax;
} ST_KDRV_DLC_GET_HISTMINMAX;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U16 u16LumiTotalCount;
    MS_U16 u16LumiAverageTemp;
} ST_KDRV_DLC_GET_LUMICOUNT;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U8  au8Curve16[DLC_LUMACURVE];
} ST_KDRV_DLC_GET_LUMACURVE16;

typedef struct  __attribute__((packed))
{
    EN_KDRV_XC_CFD_CTRL_TYPE enCtrlType;
    void *pParam;
#if !defined (__aarch64__)
    void *pDummy;
#endif
    MS_U32 u32ParamLen; //param length
    MS_U16 u16ErrCode;
} ST_KDRV_XC_CFD_CONTROL_INFO;

typedef struct
{
    MS_U32 u32Bank;
    MS_U32 u32RegNum;
    MS_U32 u32Offset;
} ST_KDRV_RIU_ENTRY;

typedef struct  __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    MS_U32  pu32Label[LEN_HSY_CONFIG];
    MS_U32  pu32Mask[LEN_HSY_CONFIG];
    //RIU
    ST_KDRV_RIU_ENTRY  *pstRIU;
    MS_U16 u16RIURegGroupNum;
    //DS
    MS_U16  u16RegNum;
    MS_U32  u32DsScriptOffset;
    //ADL
    MS_U16  u16ClientIdentifier;
    MS_U32  u32Size;
    MS_U32  u32AdlOffset;
} ST_KDRV_XC_PQ_HSY_TABLE_ENTRY;

typedef struct __attribute__((packed))
{
    // Structure version
    MS_U32 u32Version;
    // Structure length
    MS_U16 u16Length;
    // Main Rule
    ST_KDRV_XC_PQ_HSY_TABLE_ENTRY *pstTableEntry;
#if !defined (__aarch64__)
    // Dummy parameter
    void *pDummy0;
#endif
    // GRule
    ST_KDRV_XC_PQ_HSY_TABLE_ENTRY **pstGruleTableEntry;
#if !defined (__aarch64__)
    // Dummy parameter
    void *pDummy1;
#endif
    MS_U8 *pu8Config;
#if !defined (__aarch64__)
    // Dummy parameter
    void *pDummy2;
#endif

    MS_U32 u32ConfigSize;
    MS_U16 u16EntryNum;
    MS_U32 u32GRuleIPNum;
    MS_U16 *pu16EntryNumGRule;
#if !defined (__aarch64__)
    // Dummy parameter
    void *pDummy3;
#endif
} ST_KDRV_XC_PQ_HSY_TABLE;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    /// HSY Type
    MS_U32 *pu32PQSrcTypeHSY;
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
} ST_KDRV_XC_CFD_HSY;

/// CFD update status
typedef enum
{
    E_KDRV_XC_CFD_STATUS_SYNC_GEN_BLACK_SCREEN_DISABLE,
    E_KDRV_XC_CFD_STATUS_SYNC_GEN_BLACK_SCREEN_ENABLE,
    E_KDRV_XC_CFD_STATUS_SYNC_IP2VE_SETTING_FINISHED,
    E_KDRV_XC_CFD_STATUS_SYNC_PRE_GEN_BLACK_SCREEN_DISABLE, // pre notify kernel when unmute
    E_KDRV_XC_CFD_STATUS_SYNC_PRE_GEN_BLACK_SCREEN_ENABLE,  // pre notify kernel when mute
    ///MAX
    E_KDRV_XC_CFD_STATUS_MAX
} EN_KDRV_XC_CFD_UPDATE_STATUS;

//Reference in E_CFD_CFIO  color_format_input.h
typedef enum
{
    //start of RGB group
    E_KDRV_XC_CFD_CFIO_RGB_NOTSPECIFIED      = 0x0, //means RGB, but no specific colorspace
    E_KDRV_XC_CFD_CFIO_RGB_BT601_625         = 0x1,
    E_KDRV_XC_CFD_CFIO_RGB_BT601_525         = 0x2,
    E_KDRV_XC_CFD_CFIO_RGB_BT709             = 0x3,
    E_KDRV_XC_CFD_CFIO_RGB_BT2020            = 0x4,
    E_KDRV_XC_CFD_CFIO_SRGB                  = 0x5,
    E_KDRV_XC_CFD_CFIO_ADOBE_RGB             = 0x6,
    E_KDRV_XC_CFD_CFIO_RGB_DCIP3_D65         = 0x7,
    E_KDRV_XC_CFD_CFIO_RGB_DCIP3_THEATER     = 0x8,
    E_KDRV_XC_CFD_CFIO_RGB_END               = 0x9,

    //start of YUV group
    E_KDRV_XC_CFD_CFIO_YUV_NOTSPECIFIED      = 0x80, //means YUV, but no specific colorspace
    E_KDRV_XC_CFD_CFIO_YUV_BT601_625         = 0x81,
    E_KDRV_XC_CFD_CFIO_YUV_BT601_525         = 0x82,
    E_KDRV_XC_CFD_CFIO_YUV_BT709             = 0x83,
    E_KDRV_XC_CFD_CFIO_YUV_BT2020_NCL        = 0x84, //132
    E_KDRV_XC_CFD_CFIO_YUV_BT2020_CL         = 0x85,
    E_KDRV_XC_CFD_CFIO_XVYCC_601             = 0x86,
    E_KDRV_XC_CFD_CFIO_XVYCC_709             = 0x87,
    E_KDRV_XC_CFD_CFIO_SYCC601               = 0x88,
    E_KDRV_XC_CFD_CFIO_ADOBE_YCC601          = 0x89,
    E_KDRV_XC_CFD_CFIO_YUV_END               = 0x8A,

    //start of RGB/YUV group
    E_KDRV_XC_CFD_CFIO_DOLBY_HDR_TEMP        = 0x8B,
    E_KDRV_XC_CFD_CFIO_RESERVED_START
} EN_KDRV_XC_CFD_CFIO;

typedef struct
{
    EN_KDRV_XC_CFD_UPDATE_STATUS enCFD_status;
} ST_KDRV_XC_CFD_UPDATE_STATUS;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    /// data size
    MS_U32 u32GammutSize;
    /// pointer to gamma B
    MS_U16* pu16SDRGammut;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy;
#endif
}ST_KDRV_XC_CFD_SDRGAMMUT;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_S16 s16YOffset;
    MS_U16 u16YGain;
    MS_S16 s16COffset;
    MS_U16 u16CGain;
}ST_KDRV_XC_CFD_YC_OffsetGain;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_S16 s16YOffsetIn;
    MS_U16 u16YGainIn;
    MS_S16 s16COffsetIn;
    MS_U16 u16CGainIn;
    MS_S16 s16YOffsetOut;
    MS_U16 u16YGainOut;
    MS_S16 s16COffsetOut;
    MS_U16 u16CGainOut;
}ST_KDRV_XC_CFD_XC_YC_OffsetGainBoth;

typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Gamut mmaping 3*3 data
    MS_S32 s32GamutLutData[9];
}ST_KDRV_XC_CFD_GAMUTLUT_PREVIOUS;

typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Gamma enable/disable
    MS_BOOL bSDRGammaEnable;
    /// Degamma enable/disable
    MS_BOOL bSDRDegammaEnable;
    /// SDR 3D LUT enable
    MS_BOOL bSDR3DLUTEnable;
    /// Gamut mmaping 3*3 data
    MS_S32 s32GamutLutData[9];
}ST_KDRV_XC_CFD_GAMUTLUT_POST;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Window (main or sub window)
    MS_U8 u8Win;

    /// pointer to gamma R
    MS_U8* pu8GammaR;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy1;
#endif
    /// data size
    MS_U32 u32GammaRSize;

    /// pointer to gamma G
    MS_U8* pu8GammaG;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy2;
#endif
    /// data size
    MS_U32 u32GammaGSize;

    /// pointer to gamma B
    MS_U8* pu8GammaB;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy3;
#endif
    /// data size
    MS_U32 u32GammaBSize;
}ST_KDRV_XC_CFD_SDRGAMMA;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    /// Window (main or sub window)
    MS_U8 u8Win;

    /// pointer to gamma R
    MS_U8* pu8DegammaR;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy1;
#endif
    /// data size
    MS_U32 u32DegammaRSize;

    /// pointer to gamma G
    MS_U8* pu8DegammaG;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy2;
#endif
    /// data size
    MS_U32 u32DegammaGSize;

    /// pointer to gamma B
    MS_U8* pu8DegammaB;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy3;
#endif
    /// data size
    MS_U32 u32DegammaBSize;
} ST_KDRV_XC_CFD_SDRDEGAMMA;

//SWDR information
typedef struct __attribute__((packed))
{
    MS_BOOL bDRE_En;                            //1: enable; 0: disable
    MS_BOOL bDRE_SWDR_En;                       //1: enable; 0: disable
    MS_U16  *pu16Data;
#if !defined (__aarch64__)
    void *pDummy;
#endif
    MS_U16  u16NumofData;
    MS_U16  u16DataIndex;
    EN_KDRV_WINDOW eWindow;
    MS_U8   u8DRE_DR_Set_BasicStrength;         //default = 4, max = 32
    MS_U8   u8DRE_SWDR_Set_BasicStrength;       //default = 4, max = 32
    MS_U16  u16DRE_SWDR_Set_StrengthDk;         //dentity = 0x100; max = 0x200
    MS_U16  u16DRE_SWDR_Set_StrengthBr;         //dentity = 0x100; max = 0x200
    MS_U8   u8DRE_SWDR_Set_SceneProtectLevel;   //default = 0x08, max = 0x20
    MS_U8   u8DRE_Set_TemporalFilterLevel;      //default = 0x06, max = 0x20
    MS_U8   u8DRE_Set_ColorCorrectLevel;        //default = 0x10, max = 0x20
}ST_KDRV_XC_SWDR_INFO;

//cfd ctrl: set init
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Input source
    MS_U8 u8InputSource;
} ST_KDRV_XC_CFD_INIT;

//cfd ctrl: set vga/tv/cvbs/svideo/ypbpr/scart/mm/dvi
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;

    /// Color format, reference EN_KDRV_XC_CFD_COLOR_FORMAT.
    MS_U8 u8ColorFormat;
    /// Color data format, reference EN_KDRV_XC_CFD_COLOR_DATA_FORMAT.
    MS_U8 u8ColorDataFormat;
    /// Full range
    MS_BOOL bIsFullRange;
    /// Color primaries
    MS_U8 u8ColorPrimaries;
    /// Transfer characteristics
    MS_U8 u8TransferCharacteristics;
    /// Matrix coefficients
    MS_U8 u8MatrixCoefficients;
} ST_KDRV_XC_CFD_ANALOG;

//hdr10 plus vsif for ST_KDRV_XC_CFD_HDMI, refer to  STU_CFD_HDR10PLUS_VSIF in mdrv_alg_interface.h
typedef struct
{
    MS_U16 u16Version;      /// Version.
    MS_U16 u16Size;         /// Structure size.
    MS_BOOL bValid;         /// if vsif info frame is valid
    MS_U8 u8VSIFTypeCode;
    MS_U8 u8VSIFVersion;
    MS_U8 u8Length;
    MS_U32 u32IEEECode;
    MS_U8 u8ApplicationVersion;
    MS_U8 u8TargetSystemDisplayMaxLuminance;
    MS_U8 u8AverageMaxRGB;
    MS_U8 au8DistributionValues[9];
    MS_U16 u16KneePointX;
    MS_U16 u16KneePointY;
    MS_U8 u8NumBezierCurveAnchors;
    MS_U8 au8BezierCurveAnchors[9];
    MS_U8 u8GraphicsOverlayFlag;
    MS_U8 u8NoDelayFlag;
} ST_KDRV_HDMI_VSIF_PACKET_HDR10_PLUS;

//cfd ctrl: set hdmi and get hdmi status
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;

    /// Full range
    MS_BOOL bIsFullRange;

    /// AVI infoframe
    /// Pixel Format
    MS_U8 u8PixelFormat;
    /// Color imetry
    MS_U8 u8Colorimetry;
    /// Extended Color imetry
    MS_U8 u8ExtendedColorimetry;
    /// Rgb Quantization Range
    MS_U8 u8RgbQuantizationRange;
    /// Ycc Quantization Range
    MS_U8 u8YccQuantizationRange;

    /// HDR infoframe
    /// HDR infoframe valid
    MS_BOOL bHDRInfoFrameValid;
    /// EOTF (/// 0:SDR gamma, 1:HDR gamma, 2:SMPTE ST2084, 3:Future EOTF, 4-7:Reserved)
    MS_U8 u8EOTF;
    /// Static metadata ID (0: Static Metadata Type 1, 1-7:Reserved for future use)
    MS_U8 u8SMDID;
    /// Display primaries x
    MS_U16 u16Display_Primaries_x[3];
    /// Display primaries y
    MS_U16 u16Display_Primaries_y[3];
    /// White point x
    MS_U16 u16White_point_x;
    /// White point y
    MS_U16 u16White_point_y;
    /// Panel max luminance
    MS_U16 u16MasterPanelMaxLuminance;
    /// Panel min luminance
    MS_U16 u16MasterPanelMinLuminance;
    /// Max content light level
    MS_U16 u16MaxContentLightLevel;
    /// Max frame average light level
    MS_U16 u16MaxFrameAvgLightLevel;
    /// CFD status
    MS_U8 u8CFDProcessStatus;
    //AVI info for version 4
    MS_U8 u8AdditionalColorimetry;
} ST_KDRV_XC_CFD_HDMI;

//cfd ctrl: set panel
typedef struct
{
    /// Structure version, refer to CFD_PANEL_VERSION
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Color format, reference EN_KDRV_XC_CFD_COLOR_FORMAT.
    MS_U8 u8ColorFormat;
    /// Color data format, reference EN_KDRV_XC_CFD_COLOR_DATA_FORMAT.
    MS_U8 u8ColorDataFormat;
    /// Full range
    MS_BOOL bIsFullRange;

    /// Display primaries x, data *0.00002 0xC350 = 1
    MS_U16 u16Display_Primaries_x[3];
    /// Display primaries y, data *0.00002 0xC350 = 1
    MS_U16 u16Display_Primaries_y[3];
    /// White point x, data *0.00002 0xC350 = 1
    MS_U16 u16White_point_x;
    /// White point y, data *0.00002 0xC350 = 1
    MS_U16 u16White_point_y;
    /// Max luminance, data * 1 nits
    MS_U16 u16MaxLuminance;
    /// Med luminance, data * 1 nits
    MS_U16 u16MedLuminance;
    /// Min luminance, data * 0.0001 nits
    MS_U16 u16MinLuminance;
    /// Linear RGB
    MS_BOOL bLinearRgb;
    /// Customer color primaries
    MS_BOOL bCustomerColorPrimaries;
    /// Source white x
    MS_U16 u16SourceWx;
    /// Source white y
    MS_U16 u16SourceWy;
    /// PWM port
    MS_U8 u8PWMPort;
    /// dolby global dimming on/off
    MS_BOOL bGlobalDimming;
    /// OTT dolby global delay frame count
    MS_S8 s8DelayFrame;
    /// HDMI dolby global delay frame count
    MS_S8 s8HDMIDelayFrame;
} ST_KDRV_XC_CFD_PANEL;

//cfd ctrl: set hdr
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// HDR type, reference EN_KDRV_XC_CFD_HDR_TYPE
    MS_U8 u8HdrType;
    ///  HDR10/HLG TMO level. 0: low, 1: middle, 2: high
    MS_U8 u8TmoLevel;
    /// reference mode parameter invalid
    MS_BOOL bRefModeValid;
    /// reference mode enable
    MS_BOOL bRefMode;
} ST_KDRV_XC_CFD_HDR;

//hdr type for ST_KDRV_XC_CFD_HDR
typedef enum
{
    /// None HDR
    E_KDRV_XC_CFD_HDR_TYPE_NONE = 0x0000,
    /// Dolby HDR
    E_KDRV_XC_CFD_HDR_TYPE_DOLBY = 0x0001,
    /// Open HDR (HDR10)
    E_KDRV_XC_CFD_HDR_TYPE_OPEN = 0x0002,
    /// TCH
    E_KDRV_XC_CFD_HDR_TYPE_TCH = 0x0004,
    /// HLG
    E_KDRV_XC_CFD_HDR_TYPE_HLG = 0x0008,
    /// HDR10 plus
    E_KDRV_XC_CFD_HDR_TYPE_HDR10_PLUS = 0x0010,
    /// Open HDR in DV (Internal use)
    E_KDRV_XC_CFD_HDR_TYPE_OPEN_IN_DOLBY = 0x0100,
    /// HDMI Low Latency in DV (Internal use)
    E_KDRV_XC_CFD_HDR_TYPE_LOWLATENCY_HDMI_IN_DOLBY = 0x0101,
    /// HDMI Low Latency RGB in DV (Internal use)
    E_KDRV_XC_CFD_HDR_TYPE_LOWLATENCY_RGB_HDMI_IN_DOLBY = 0x0110,
    /// Max
    E_KDRV_XC_CFD_HDR_TYPE_MAX = 0xffff
} EN_KDRV_XC_CFD_HDR_TYPE;

typedef enum
{
    E_KDRV_XC_CFD_OUTPUT_TR_BT1886 = 0,
    E_KDRV_XC_CFD_OUTPUT_TR_PQ = 1,
    E_KDRV_XC_CFD_OUTPUT_TR_LINEAR = 2,

    E_KDRV_XC_CFD_OUTPUT_TR_MAX,
}EN_KDRV_XC_CFD_OUTPUT_TR;

//hdr Curve type
typedef enum
{
    /// Refer to metadata HDR type
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_AUTO = 0,
    /// Force using SDR curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_SDR = 1,
        /// Force using SDR curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_DOLBY = 2, // not support forced hdr curve changing
    /// Force using HDR10 curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_HDR10 = 3,
        /// Force using SDR curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_TCH = 4, // not support forced hdr curve changing
    /// Force using HLG curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_HLG = 5,
    /// Force using HLG curve
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_HDR10_PLUS = 6, // not support forced hdr curve changing
    /// Max
    E_KDRV_XC_CFD_HDR_CURVE_TYPE_TYPE_MAX,
} EN_KDRV_XC_CFD_HDR_CURVE_TYPE;

//cfd ctrl: set hdr
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Apply settings to all HDR types, ignore enHdrType
    MS_BOOL bApplyToAllHdr;
    /// Apply settings to which HDR type, reference EN_KDRV_XC_CFD_HDR_TYPE
    EN_KDRV_XC_CFD_HDR_TYPE enHdrType;

    /// support this hdr valid
    MS_BOOL bEnableHdrValid;
    /// support this hdr
    MS_BOOL bEnableHdr;
    ///output TR parameter invalid
    MS_BOOL bOutputTR;
    ///output TR parameter
    EN_KDRV_XC_CFD_OUTPUT_TR enOutputTR;
    ///output max Lumimance
    MS_BOOL bOutputMaxLuminance;
    ///output TR parameter
    MS_U16 u16OutputMaxLuminance;

    /// HDR curve UI setting valid
    MS_BOOL bHdrCurveTypeValid;
    /// HDR curve UI setting, force HDR curve type
    EN_KDRV_XC_CFD_HDR_CURVE_TYPE enHdrCurveType;
}ST_KDRV_XC_CFD_HDR_ATTR;

//cfd ctrl: set edid
typedef struct
{
    MS_U32 u32Version;   ///<Version of current structure. Please always set to "CFD_HDMI_EDID_ST_VERSION" as input
    MS_U16 u16Length;    ///<Length of this structure, u16Length=sizeof(STU_CFDAPI_HDMI_EDID_PARSER)

    MS_U8 u8HDMISinkHDRDataBlockValid;
    //assign by E_CFD_VALIDORNOT
    //0 :Not valid
    //1 :valid

    MS_U8 u8HDMISinkEOTF;
    //byte 3 in HDR static Metadata Data block

    MS_U8 u8HDMISinkSM;
    //byte 4 in HDR static Metadata Data block

    MS_U8 u8HDMISinkDesiredContentMaxLuminance;           //need a LUT to transfer
    MS_U8 u8HDMISinkDesiredContentMaxFrameAvgLuminance; //need a LUT to transfer
    MS_U8 u8HDMISinkDesiredContentMinLuminance;           //need a LUT to transfer
    //byte 5 ~ 7 in HDR static Metadata Data block

    MS_U8 u8HDMISinkHDRDataBlockLength;
    //byte 1[4:0] in HDR static Metadata Data block

    //order R->G->B
    //MS_U16 u16display_primaries_x[3];                       //data *1/1024 0x03FF = 0.999
    //MS_U16 u16display_primaries_y[3];                       //data *1/1024 0x03FF = 0.999
    //MS_U16 u16white_point_x;                                //data *1/1024 0x03FF = 0.999
    //MS_U16 u16white_point_y;                                //data *1/1024 0x03FF = 0.999
    /// Display primaries x, data *0.00002 0xC350 = 1
    MS_U16 u16Display_Primaries_x[3];
    /// Display primaries y, data *0.00002 0xC350 = 1
    MS_U16 u16Display_Primaries_y[3];
    /// White point x, data *0.00002 0xC350 = 1
    MS_U16 u16White_point_x;
    /// White point y, data *0.00002 0xC350 = 1
    MS_U16 u16White_point_y;
    //address 0x19h to 22h in base EDID

    MS_U8 u8HDMISinkEDIDBaseBlockVersion;                //for debug
    //address 0x12h in EDID base block

    MS_U8 u8HDMISinkEDIDBaseBlockReversion;              //for debug
    //address 0x13h in EDID base block

    MS_U8 u8HDMISinkEDIDCEABlockReversion;               //for debug
    //address 0x01h in EDID CEA block

    //table 59 Video Capability Data Block (VCDB)
    //0:VCDB is not avaliable
    //1:VCDB is avaliable
    MS_U8 u8HDMISinkVCDBValid;

    MS_U8 u8HDMISinkSupportYUVFormat;
    //bit 0:Support_YUV444
    //bit 1:Support_YUV422
    //bit 2:Support_YUV420

    //QY in Byte#3 in table 59 Video Capability Data Block (VCDB)
    //bit 3:RGB_quantization_range

    //QS in Byte#3 in table 59 Video Capability Data Block (VCDB)
    //bit 4:Y_quantization_range 0:no data(due to CE or IT video) ; 1:selectable


    MS_U8 u8HDMISinkExtendedColorspace;
    //byte 3 of Colorimetry Data Block
    //bit 0:xvYCC601
    //bit 1:xvYCC709
    //bit 2:sYCC601
    //bit 3:Adobeycc601
    //bit 4:Adobergb
    //bit 5:BT2020 cl
    //bit 6:BT2020 ncl
    //bit 7:BT2020 RGB

    MS_U8 u8HDMISinkEDIDValid;
} ST_KDRV_XC_CFD_EDID;

//cfd ctrl: set osd
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Hue
    MS_U16 u16Hue;
    /// Saturation
    MS_U16 u16Saturation;
    /// Contrast
    MS_U16 u16Contrast;

    //0:off
    //1:on
    //default on , not in the document
    MS_U8  u8OSDUIEn;

    //Mode 0: update matrix by OSD and color format driver
    //Mode 1: only update matrix by OSD controls
    //for mode1 : the configures of matrix keep the same as the values by calling CFD last time
    MS_U8  u8OSDUIMode;

    ///
    MS_BOOL bHueValid;
    MS_BOOL bSaturationValid;
    MS_BOOL bContrastValid;

    MS_BOOL bBacklightValid;
    MS_U32 u32MinBacklight;
    MS_U32 u32MaxBacklight;
    MS_U32 u32Backlight;

    MS_U8 u8Win;
    MS_BOOL bColorRangeValid;
    MS_U8 u8ColorRange; // 0: Auto, 1: Full Range (0~255), 2: Limit Range (16~235)

    MS_BOOL bUltraBlackLevelValid;
    MS_U8 u8UltraBlackLevel;
    MS_BOOL bUltraWhiteLevelValid;
    MS_U8 u8UltraWhiteLevel;

    MS_BOOL bSkipPictureSettingValid;
    MS_BOOL bSkipPictureSetting;

    MS_BOOL bColorCorrectionValid;
    MS_S16 s16ColorCorrectionMatrix[32];

    MS_BOOL bYVUtoRGBMatrixValid;
    MS_S16 s16YVUtoRGBMatrix[32];

    MS_U16 u16R;
    MS_U16 u16G;
    MS_U16 u16B;

    MS_BOOL bRValid;
    MS_BOOL bGValid;
    MS_BOOL bBValid;

    //0:auto depends on STB rule
    //1:always do HDR2SDR for HDR input
    //2:always not do HDR2SDR for HDR input
    MS_U8  u8HDR_UI_H2SMode;

    MS_BOOL bViewModeValid;
    MS_U8 u8ViewMode;

    MS_BOOL bSharpnessValid;
    MS_U8 u8Sharpness;

    MS_BOOL bBrightnessValid;
    MS_U16 au16Brightness[E_BRIGHTNESS_MAX];

    //Force to fire OSD setting, use case example: set contrast for white balance test pattern while no signal
    MS_BOOL bForceFireOSD;
} ST_KDRV_XC_CFD_OSD;

typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window
    MS_U8 u8Win;
    /// Sharpness value
    MS_U8 *pu8Sharpness;
#if !defined (__aarch64__)
    void *pDummy0;    /// Dummy parameter
#endif
} ST_KDRV_XC_CFD_SHARPNESS;

typedef struct
{
    MS_U8 u8Vid;
    MS_U8 u8DeflickerEn;
    MS_U8 u8SnrMotionEn;
    MS_U8 u8DefaultSnrGain;
    MS_U8 u8DeflickerStr;
    MS_U8 u8HiCplxStr;
}ST_KDRV_PQ_DEFLICKER;

//cfd ctrl: set_fire
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Input source, reference PQ_INPUT_SOURCE_TYPE in apxXC.h
    MS_U8 u8InputSource;
    /// Update type, reference EN_KDRV_XC_CFD_UPDATE_TYPE
    MS_U8 u8UpdateType;
    /// RGB bypass
    MS_BOOL bIsRgbBypass;
    /// HD mode
    MS_BOOL bIsHdMode;
} ST_KDRV_XC_CFD_FIRE;

//cfd ctrl: set dlc
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// customer dlc curve
    MS_BOOL bUseCustomerDlcCurve;
} ST_KDRV_XC_CFD_DLC;

//cfd ctrl: set linear rgb
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// Linear RGB enable
    MS_BOOL bEnable;
} ST_KDRV_XC_CFD_LINEAR_RGB;

//mm hdr memory formate for ST_KDRV_XC_CFD_STATUS
typedef struct
{
    MS_U8 u8CurrentIndex;
    MS_U8 u8InputFormat;
    MS_U8 u8InputDataFormat;
    MS_BOOL u8Video_Full_Range_Flag;

    MS_BOOL bVUIValid;
    MS_U8 u8Colour_primaries;
    MS_U8 u8Transfer_Characteristics;
    MS_U8 u8Matrix_Coeffs;

    MS_BOOL bSEIValid;
    MS_U16 u16Display_Primaries_x[3];
    MS_U16 u16Display_Primaries_y[3];
    MS_U16 u16White_point_x;
    MS_U16 u16White_point_y;
    MS_U32 u32Master_Panel_Max_Luminance;
    MS_U32 u32Master_Panel_Min_Luminance;

    MS_BOOL bContentLightLevelEnabled;
    MS_U16  u16maxContentLightLevel;
    MS_U16  u16maxPicAverageLightLevel;
} ST_KDRV_XC_HDR_CFD_MEMORY_FORMAT;

typedef enum
{
    /// IP1 input, equal to XC input
    E_KDRV_XC_CFD_POINT_IP1_INPUT = 1,
    /// VOP output
    E_KDRV_XC_CFD_POINT_VOP_OUTPUT = 2,
    E_KDRV_XC_CFD_POINT_MAX,
} EN_KDRV_XC_CFD_POINT;

//Color info get from CFD
typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// format information, define in HDMI spec
    MS_U8 u8Format;
    /// data format, such as YUV420, YUV422, YUV444, RGB
    MS_U8 u8DataFormat;
    /// color range, true: full range, false: limit range
    MS_U8 u8IsFullRange;
    /// useful in MM source, format can get from ColorPriamrie, TransferCharacterstic and MatrixCoefficient when transfer MM to HDMI in Box
    MS_U8 u8ColorPriamries;
    /// useful in MM source
    MS_U8 u8TransferCharacterstics;
    /// useful in MM source
    MS_U8 u8MatrixCoefficients;
} ST_KDRV_XC_HDR_CFD_POINT_INFO;

typedef enum
{
    /// SDR
    E_KDRV_XC_HDMI_VSIF_TYPE_NONE,
    /// DolbyStanard HDR in H14bVSIF
    E_KDRV_XC_HDMI_VSIF_TYPE_DOLBY_STANDARD_H14BVSIF,
    /// DolbyStanard HDR in DolbyVSIF
    E_KDRV_XC_HDMI_VSIF_TYPE_DOLBY_STANDARD_DOLBYVSIF,
    /// DolbyLowlentency HDR
    E_KDRV_XC_HDMI_VSIF_TYPE_DOLBY_LOWLATENCY,
    /// HDR10 plus
    E_KDRV_XC_HDMI_VSIF_TYPE_HDR10_PLUS,

    /// Max
    E_KDRV_XC_HDMI_VSIF_TYPE_MAX
} EN_KDRV_XC_HDMI_VSIF_TYPE;

//cfd ctrl: get status
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// HDR type (0: SDR, 1: Dolby HDR, 2: Open HDR)
    MS_U8 u8VideoHdrType;
    /// HDR running
    MS_BOOL bIsHdrRunning;
    /// Is full range
    MS_BOOL bIsFullRange;
    /// Ultra black & white active
    MS_BOOL bUltraBlackWhiteActive;
    /// Color Type
    MS_U8 u8ColorType;
    /// TMO Color Type
    MS_U8 u8TMOColorType;

    /// open HDR metadata
    union
    {
        ST_KDRV_XC_CFD_HDMI stHdmiInfoCFD;
        ST_KDRV_XC_HDR_CFD_MEMORY_FORMAT  stHDRMemFormatCFD;
    } HDRMetadata;
    //CRC Status
    MS_BOOL bCRCStatus;
    //set_HDRDS_info average time
    MS_U16 u16SetDSAverageTime;
    //set_HDRDS_info peak time
    MS_U16 u16SetDSPeakTime;
    //MM HDR vdec version number
    MS_U8 u8VdecVersion;
    //set_HDRDS_info peak frame
    MS_U64 u64SetDSPeakFrame;
    //HDMI packet change || MM Version change
    MS_BOOL bHDRDataChanged;
    //Seamless CFD Done
    MS_BOOL bSeamlessCFDDone;
    // GOP after done
    MS_BOOL bGOPDone;
    // Reset Freeze
    MS_BOOL bResetSeamlessFlag;
    // HDR Mode status
    EN_KDRV_XC_HDR_OUTPUT_MODE eHDROutputMode;
    /// point infor from CFD
    ST_KDRV_XC_HDR_CFD_POINT_INFO *pstPointInfo;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy;
#endif
    /// date size, equal to sizeof(ST_KDRV_XC_HDR_CFD_POINT_INFO) * point number
    MS_U16 u16DataSize;
    /// HDMI VSIF type
    EN_KDRV_XC_HDMI_VSIF_TYPE enHdmiVsifType;
} ST_KDRV_XC_CFD_STATUS;

//tmo levle for ST_KDRV_XC_CFD_TMO
typedef struct
{
    MS_U32 u16HdrType;
    MS_U16 u16Level;

    MS_U16 u16ControlSize;
    MS_U8 *pu8data;
#if !defined (__aarch64__)
    /// Dummy parameter
    void *pDummy;
#endif
} ST_KDRV_XC_CFD_TMO_LEVEL;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;

    /// Enable user's colorimetry (User's or CFD control)
    MS_BOOL bEnableUserColorimetry;
    EN_KDRV_XC_CFD_CFIO enOutputColorimetry;
} ST_KDRV_XC_CFD_SET_OUTPUT_COLORIMETRY;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;

    /// Enable user's HDR type (User's or CFD control)
    MS_BOOL bEnableUserHDRType;
    EN_KDRV_XC_HDR_TYPE enOutputHDRType;
} ST_KDRV_XC_CFD_SET_OUTPUT_HDR_TYPE;

typedef enum
{
    E_KDRV_XC_CFD_DISP_PATH_NONE,
    E_KDRV_XC_CFD_DISP_PATH_XC0_MAIN,
    E_KDRV_XC_CFD_DISP_PATH_XC0_SUB,
    E_KDRV_XC_CFD_DISP_PATH_XC1_MAIN,
    E_KDRV_XC_CFD_DISP_PATH_XC1_SUB,
    E_KDRV_XC_CFD_DISP_PATH_HFR_MAIN,
    E_KDRV_XC_CFD_DISP_PATH_HFR_SUB,

    E_KDRV_XC_CFD_DISP_PATH_MAX
} EN_KDRV_XC_CFD_DISP_PATH;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              ///<Version of current structure.
    MS_U16 u16Length;               ///<Length of current structure.
    MS_U8 u8Win;
    EN_KDRV_XC_CFD_DISP_PATH enDispPath;
} ST_KDRV_XC_CFD_DISP_PATH;

typedef struct
{
    MS_U8  u8ActionMode;
    MS_U32 u32NumofReg;
    MS_U16 u16ClientID;
} ST_KDRV_PQ_SAVE_BIN_INFO;

typedef enum
{
    /// Customer
    E_KDRV_XC_PQ_BIN_PATH_CUSTOMER,
    /// Default
    E_KDRV_XC_PQ_BIN_PATH_DEFAULT,
    /// INI
    E_KDRV_XC_PQ_BIN_PATH_INI,
    /// Bandwidth
    E_KDRV_XC_PQ_BIN_PATH_BANDWIDTH,
    /// The max support number of paths
    E_KDRV_XC_PQ_BIN_PATH_MAX
}EN_KDRV_XC_PQ_BIN_PATH;

typedef struct __attribute__((packed))
{
    EN_KDRV_XC_PQ_BIN_PATH enXCPqBinPath;
    MS_U8 u8size;
    char* b_PQBinFilePath;
#if !defined (__aarch64__)
    void *pDummy0;    /// Dummy parameter
#endif
} ST_KDRV_XC_SET_PQBINPATH;
typedef enum
{
    E_KDRV_XC_SWDR_HIST_TYPE_PMode                 = 0x0,
    E_KDRV_XC_SWDR_HIST_TYPE_IFrameMode            = 0x1,
    E_KDRV_XC_SWDR_HIST_TYPE_I1stFieldMode         = 0x2,
    E_KDRV_XC_SWDR_HIST_TYPE_I2ndFieldMode         = 0x3,
    E_KDRV_XC_SWDR_HIST_TYPE_NULL                  = 0xFF,
} EN_KDRV_XC_SWDR_HIST_TYPE;

///CFD pq gamma channel
typedef enum
{
    E_KDRV_XC_CFD_PQ_GAMMA_CHANNEL_R,
    E_KDRV_XC_CFD_PQ_GAMMA_CHANNEL_G,
    E_KDRV_XC_CFD_PQ_GAMMA_CHANNEL_B,
    E_KDRV_XC_CFD_PQ_GAMMA_CHANNEL_MAX
}EN_KDRV_XC_CFD_PQ_GAMMA_CHANNEL_TYPE;

typedef enum
{
    E_KDRV_XC_IP_RANGE_COVERT,             //Pre SDR IP
    E_KDRV_XC_IP_CSC,
    E_KDRV_XC_IP_HISTOGRAM_LOCATION,       //HDR IP
    E_KDRV_XC_IP_PRE_RANGE_COVERT,
    E_KDRV_XC_IP_PRE_CSC,
    E_KDRV_XC_IP_EOTF,
    E_KDRV_XC_IP_PRE_GAMUT,
    E_KDRV_XC_IP_OOTF,
    E_KDRV_XC_IP_RGB3DLUT,                 //value of 3D lut
    E_KDRV_XC_IP_OETF,
    E_KDRV_XC_IP_POST_CSC,
    E_KDRV_XC_IP_TMO,
    E_KDRV_XC_IP_POST_RANGE_COVERT,
    E_KDRV_XC_IP_POST_CSC_SDR,                 //post HDR IP
    E_KDRV_XC_IP_POST_GAMUT,
    E_KDRV_XC_IP_POST_DEGAMMA,
    E_KDRV_XC_IP_POST_RGB3D,
    E_KDRV_XC_IP_POST_GAMMA,
    E_KDRV_XC_IP_PRE_RANGE_COVERT_SDR,
    E_KDRV_XC_IP_POST_RANGE_COVERT_SDR,
    E_KDRV_XC_IP_DLC_RANGE,
    E_KDRV_XC_IP_VIP_OUT_RANGE,
    E_KDRV_XC_IP_RGB3DLUT_INTERVAL,          //interval of 3D lut, only for LG at present
    E_KDRV_XC_IP_MAX,
}EN_KDRV_XC_CFD_CUSTOMER_IP;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U8  u8Win;
    EN_KDRV_XC_CFD_CUSTOMER_IP enCustomerIp;
}ST_KDRV_CUSTOMER_IP;

//Ver 0: init Ver
//Ver 1 : add u16IpHistAulData_size && u16IpYLiteHistAulData_size
#define SWDR_KDRV_INPUT_STRUCTURE_VERSION   (1)

typedef struct __attribute__((packed))
{
    MS_U32 u32Version; //<Version of current structure. Please always set to "SWDR_KDRV_INPUT_STRUCTURE_VERSION" as input
    MS_U32 u32Length; //<Length of this structure, u16Length=sizeof(XC_SWDR_Input)
    MS_U32 *pHist_Curr; // ( Top + Bottom ) 24bits * 32bins * 2
#if !defined (__aarch64__)
    void *pDummy0;    /// Dummy parameter
#endif
    MS_U32 *pHist_Prev; // ( Top + Bottom ) 24bits * 32bins * 2
#if !defined (__aarch64__)
    void *pDummy1;    /// Dummy parameter
#endif
    MS_U16 u16Hist_size; // 32bins * 2
    MS_U16 *pOOTFLut; // 1.15bits * 512bins
#if !defined (__aarch64__)
    void *pDummy2;    /// Dummy parameter
#endif
    MS_U16 u16OOTF_size; // 512 bins
    EN_KDRV_XC_SWDR_HIST_TYPE  eHist_type; // _DRV_HIST_TYPE;
    MS_U8  u8Src_type; // _E_CFD_CFIO_TR
    MS_U8  u8Out_type; // _E_CFD_CFIO_TR
    EN_KDRV_WINDOW  eWindow;
    MS_U8 *pu8IpHistAulData;
#if !defined (__aarch64__)
    void *pDummy3;    /// Dummy parameter
#endif
    MS_U16 u16IpHistAulData_size;
    MS_U8 *pu8IPYLiteHistAulData;
#if !defined (__aarch64__)
    void *pDummy4;    /// Dummy parameter
#endif
    MS_U16 u16IpYLiteHistAulData_size;
}ST_KDRV_XC_SWDR_INPUT;

typedef enum
{
    E_KDRV_VIDEO_CODEC_UNKNOWN = -1,     /// Video codec type is unknown.
    E_KDRV_VIDEO_CODEC_MPEG4,            /// Video codec type is MPEG 4.
    E_KDRV_VIDEO_CODEC_MJPEG,            /// Video codec type is motion JPG.
    E_KDRV_VIDEO_CODEC_H264,             /// Video codec type is H264.
    E_KDRV_VIDEO_CODEC_RM,               /// Video codec type is RealVideo.
    E_KDRV_VIDEO_CODEC_TS,               /// Video codec type is TS File.
    E_KDRV_VIDEO_CODEC_MPEG,             /// Video codec type is MPEG 1/2.
    E_KDRV_VIDEO_CODEC_VC1,              /// Video codec type is VC1.
    E_KDRV_VIDEO_CODEC_AVS,              /// Video codec type is Audio Video Standard.
    E_KDRV_VIDEO_CODEC_FLV,              /// Video codec type is FLV.
    E_KDRV_VIDEO_CODEC_MVC,              /// Video codec type is MVC.
    E_KDRV_VIDEO_CODEC_VP6,              /// Video codec type is VP6.
    E_KDRV_VIDEO_CODEC_VP8,              /// Video codec type is VP8.
    E_KDRV_VIDEO_CODEC_HEVC,             /// Video codec type is HEVC.
    E_KDRV_VIDEO_CODEC_VP9,              /// Video codec type is VP9.
} EN_KDRV_XC_VIDEO_CODEC;

typedef enum
{
    E_KDRV_FRAME_TYPE_I,
    E_KDRV_FRAME_TYPE_P,
    E_KDRV_FRAME_TYPE_B,
    E_KDRV_FRAME_TYPE_OTHER,
    E_KDRV_FRAME_TYPE_MAX,
} EN_KDRV_XC_FRAMETYPE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version; // <Version of current structure. Please always set to "FRAME_INFO_STRUCTURE_VERSION" as input
    MS_U32 u32Length;  // <Length of this structure, u16Length=sizeof(ST_KDRV_XC_VDEC_INFO)
    EN_KDRV_WINDOW  eWindow;
    EN_KDRV_XC_SWDR_HIST_TYPE enScanType; // scan type (P/I frame/I field/NULL)
    MS_U32 u32QPMin; // Min of quantization parameter
    MS_U32 u32QPAvg; // Avg of quantization parameter
    MS_U32 u32QPMax; // Max of quantization parameter
    MS_U8 u8BotFlag; // Bottom field = 1, top field = 0
    EN_KDRV_XC_VIDEO_CODEC enCodecType; // Codec type (H264/H265/MPEG....)
    EN_KDRV_XC_FRAMETYPE enFrameType; // Frame type (I/P/B mode)
    EN_KDRV_XC_HDR_COLOR_FORMAT enStreamColorFormat; //YUV420/YUV422/YUV444/RGB
    MS_S32 s32CPLX; // vdec CPLX info
    MS_U32 u32VDECInVfreq;
}ST_KDRV_XC_FRAME_INFO;

//cfd ctrl: picture attr
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Picture attribute control type, reference EN_XC_CFD_PICTURE_ATTR_CTRL_TYPE
    MS_U8 u8ControlType;
    /// Picture attribute value
    MS_U16 u16AttrValue;
} ST_KDRV_XC_CFD_PICTURE_ATTRIBUTE;

//cfd ctrl: hdmi par
typedef struct
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    /// enable to parse HDMI packet
    MS_BOOL bHDMIPacketParsing;
} ST_KDRV_XC_CFD_HDMI_PARSE;

/// Connect tool fucntion type
typedef enum
{
    E_KDRV_XC_TOOL_FILE_TO_SYSTEM,
    E_KDRV_XC_TOOL_WRITE_REGISTER,
    E_KDRV_XC_TOOL_READ_REGISTER,
    E_KDRV_XC_TOOL_WRITE_VR,
    E_KDRV_XC_TOOL_READ_VR,
    E_KDRV_XC_TOOL_WRITE_ADL,
    E_KDRV_XC_TOOL_READ_ADL,
    E_KDRV_XC_TOOL_MAX
} EN_KDRV_XC_TOOL_FUNCTION_TYPE;

typedef enum
{
    E_KDRV_XC_SAVE_MAIN_BIN,
    E_KDRV_XC_SAVE_HSY_BIN,
    E_KDRV_XC_SAVE_MAX
} EN_KDRV_XC_SAVE_BIN_TYPE;

// Qmap type for ST_KDRV_XC_PQ_QMAPDATA
/// E_KDRV_XC_CFD_TMO_XXX: tmo qmap
/// E_KDRV_XC_CFD_COLOR_XXX: small qmap
/// E_KDRV_XC_CFD_QMAP_XXX: normal qmap
typedef enum
{
    /// TMO Qmap
    E_KDRV_XC_CFD_TMO_QMAP = 0,
    /// HDR Grule
    E_KDRV_XC_CFD_HDR_GRULE = 1,
    /// small PQ HDR Grule
    E_KDRV_XC_CFD_COLOR_HDR_GRULE = 2,
    /// TMO DLC Grule
    E_KDRV_XC_CFD_TMO_DLC_GRULE = 3,
    /// SPF Xrule
    E_KDRV_XC_CFD_SPF_XRULE = 4,
    ///TMO Grule
    E_KDRV_XC_CFD_TMO_TMO_GRULE = 5,
    /// Max
    E_KDRV_XC_CFD_QMAP_MAX
}EN_KDRV_XC_CFD_QMAP_TYPE;

typedef enum
{
    E_KDRV_XC_CFD_QMAPDATA_INFO_GROUP_NUM_ALIENMENT = 0,
    E_KDRV_XC_CFD_QMAPDATA_INFO_REG_NUM_ALIENMENT,
    E_KDRV_XC_CFD_QMAPDATA_INFO_TABLE_TYPE_ALIENMENT,

    E_KDRV_XC_CFD_QMAPDATA_INFO_MAX,
}EN_KDRV_XC_CFD_QMAPDATA_INFOMATION_MAP;

typedef enum
{
    E_KDRV_XC_CFD_QMAPDATA_TABTYPE_INVALID = 0,
    E_KDRV_XC_CFD_QMAPDATA_TABTYPE_SCALER,
    E_KDRV_XC_CFD_QMAPDATA_TABTYPE_TWOBYTE_BANK,

    E_KDRV_XC_CFD_QMAPDATA_TABTYPE_MAX,
}EN_KDRV_XC_CFD_QMAPDATA_TABLE_TYPE;


//Histogram type for ST_KDRV_XC_CFD_IP_HISTOGRAM
typedef enum
{
    E_KDRV_XC_CFD_HISTOGRAM_HUE_IP,
    E_KDRV_XC_CFD_HISTOGRAM_SATURATION_IP,
    E_KDRV_XC_CFD_HISTOGRAM_LUMA_IP,
    E_KDRV_XC_CFD_HISTOGRAM_RGBMAX_IP,
    E_KDRV_XC_CFD_HISTOGRAM_MAX
} EN_KDRV_XC_CFD_HISTOGRAM_TYPE;

//cfd ctrl: get ip histogram
typedef struct  __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U16 u16Length;
    MS_U8  u8Win;
    EN_KDRV_XC_CFD_HISTOGRAM_TYPE enHistogramType;
    MS_U32 u32HistogramBin;                       // histogram bin of hw provide
    MS_U32 *pu32HistogramValue;                   // histogram value, array size = u32HistogramBin
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
} ST_KDRV_XC_CFD_IP_HISTOGRAM;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    MS_U32 u32Qmapid;
    MS_U32 u32Quality_InputType_Num;
    MS_U32 u32Quality_IP_Num;
    MS_U32 u32Table_Data_Size;
    MS_U32 u32Table_IP_Num;
    MS_U16 *pu16Quality_Map_Array;
#if !defined (__aarch64__)
    /// Dummy parameter
    MS_U32 *pu32Alignment;
#endif
    MS_U8 *pu8Table_Data_Array;
#if !defined (__aarch64__)
    /// Dummy parameter
    MS_U32 *pu32Alignment1;
#endif
    MS_U16 *pu16Table_Map_Array;
#if !defined (__aarch64__)
    /// Dummy parameter
    MS_U32 *pu32Alignment2;
#endif
    /// Qmap type
    EN_KDRV_XC_CFD_QMAP_TYPE enQmapType;
} ST_KDRV_XC_PQ_QMAPDATA;

typedef enum
{
    E_KDRV_XC_HSY_HUE_SETTING,
    E_KDRV_XC_HSY_SAT_SETTING,
    E_KDRV_XC_HSY_LUMA_SETTING,
    E_KDRV_XC_HSY_NUMS,
}EN_KDRV_XC_HSY_FUNC;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;               // Struct version
    MS_U16 u16Length;                // Sturct length

    EN_KDRV_XC_HSY_FUNC  enXCHSYFunc; // HSY setting
    MS_S32 *ps32Data;                //degrees or gain value
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
    MS_U8 u8DataLength;             //The data length of Setting
    MS_U8 u8Win;                        // 0:main window 1:sub window
}ST_KDRV_XC_HSY_SETTING_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;               // Struct version
    MS_U16 u16Length;                // Sturct length

    EN_KDRV_XC_HSY_FUNC  enXCHSYFunc; // HSY setting
    MS_S32 *ps32MinValue;
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
    MS_S32 *ps32MaxValue;
#if !defined (__aarch64__)
    void    *pDummy1;
#endif
    MS_U8 *pu8NumofRegion;            // The number of hue region
#if !defined (__aarch64__)
    void    *pDummy2;
#endif
    MS_U8 u8Win;                        // 0:main window 1:sub window
}ST_KDRV_XC_HSY_ADJ_RANGE_INFO;

typedef struct
{
    MS_U32 u32Version;               // Struct version
    MS_U16 u16Length;                // Sturct length

    MS_U8 u8Win;
    MS_U8 u8Bff;                     //0: top first  1: bottom first
    MS_S32 s32Width;
    MS_S32 s32Height;
    MS_BOOL bFieldOrderChange;               //the result of field order detection
}ST_KDRV_XC_FOD_INFO;

/// DMA info
typedef struct
{
    MS_U8 u8FrameNumber;
    MS_U8 u8MiuSel;
    MS_U8 u8IPMCurFrameNum;
    MS_U32 u32Addr;
    MS_U8 u8BytePerWord;
} ST_KDRV_XC_DMA_INFO;

typedef enum
{
    E_KDRV_XC_MEMORY_TYPE_AUTOUPLOAD,
    E_KDRV_XC_MEMORY_TYPE_CMDQ,
    E_KDRV_XC_MEMORY_TYPE_XC_PQ,
    E_KDRV_XC_MEMORY_TYPE_XC_CPU_DRAM,
    E_KDRV_XC_MEMORY_TYPE_DYNAMICSCALING,
    E_KDRV_XC_MEMORY_TYPE_MAX,
} EN_KDRV_XC_MEMORY_TYPE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;               // Struct version
    MS_U32 u32Length;                // Struct size
    EN_KDRV_XC_MEMORY_TYPE enType;   // memory type
    MS_U64 u64BaseAddr;              // Memory address
    MS_U32 u32Size;                  // size
} ST_KDRV_XC_MEMORY_INFO;

typedef enum
{
    E_KDRV_XC_HDMI_PORT_TX = 0,
    E_KDRV_XC_HDMI_PORT_RX_0,
    E_KDRV_XC_HDMI_PORT_RX_1,
    E_KDRV_XC_HDMI_PORT_RX_2,
    E_KDRV_XC_HDMI_PORT_RX_3,
} EN_KDRV_XC_HDMI_PORT;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;
    /// Vendor-Specific Video Data Block
    MS_U8 u8VSVDB[KDRV_XC_EDID_EXTENDED_TAG_CODE_DATA_BLOCK_MAX_SIZE];
    /// HDR Static Metadata Data Block
    MS_U8 u8HDRSMDB[KDRV_XC_EDID_EXTENDED_TAG_CODE_DATA_BLOCK_MAX_SIZE];
    // HDMI port
    EN_KDRV_XC_HDMI_PORT enHDMIPort;
} ST_KDRV_XC_DOVI_SOURCE_SET_SINK_EDID;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U16 u16X;
    MS_U16 u16Y;
    MS_U16 u16Width;
    MS_U16 u16Height;
} ST_KDRV_XC_WIN_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;             // Struct version
    MS_U32 u32Length;              // Struct size
    MS_BOOL bEnable;               // Set HVSP Sub1 mode by RIU or not
    MS_BOOL bIsHScalingUp;         // H HVSP Sub1 mode on/off
    MS_BOOL bIsVScalingUp;         // V HVSP Sub1 mode on/off
    MS_U8 u8DelayFrame;            // Frame delay for HVSP Sub1 mode trigger
} ST_KDRV_XC_HVSP_SUB1_MODE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;
    MS_U8 u8Win;
    MS_U8 u8InputSource;  //input source

    ST_KDRV_XC_WIN_INFO stCapWin;        //cap window
    ST_KDRV_XC_WIN_INFO stCropWin;       //crop window
    ST_KDRV_XC_WIN_INFO stDispWin;       //display window
    ST_KDRV_XC_WIN_INFO stScaledCropWin; //Scaled crop window

    ST_KDRV_XC_WIN_INFO stFSCCapWin;     //cap window of FSC
    ST_KDRV_XC_WIN_INFO stFSCCropWin;    //crop window of FSC
    ST_KDRV_XC_WIN_INFO stFSCScaledCropWin;  //scaled crop window of FSC

    MS_BOOL bInterlace;    //Interlace or Progressive
    MS_U16 u16InputVFreq;  //IP Vsync, x10
    MS_U16 u16OP1VFreq;    //OP1 Vsync, x10
    MS_U16 u16OutputVFreq; //output Vsync, x10

    MS_U16 u16SizeAfterPreScalingH;    //width of after prescaling
    MS_U16 u16SizeAfterPreScalingV;    //height of after prescaling
    MS_U16 u16FSCSizeAfterPreScalingH; //width of after FSC prescaling
    MS_U16 u16FSCSizeAfterPreScalingV; //height of after FSC prescaling

    EN_KDRV_XC_HDMI_PORT enHDMIPort;
    MS_U32  u32H_PostScalingRatio;
    MS_U32  u32V_PostScalingRatio;
    MS_BOOL bPreHScalingDown;
    MS_BOOL bPreVScalingDown;
    ST_KDRV_XC_HVSP_SUB1_MODE stHVSPSub1Mode;
    MS_U8 u8WRBankMappingNum;
    MS_BOOL bScmiZeroFrameMode;
} ST_KDRV_XC_STATUS_INFO;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U8   u8Val;
} ST_KDRV_XC_VR_READBYTE;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U16  u16Val;
} ST_KDRV_XC_VR_R2BYTE;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U16  u16Mask;
    MS_U16  u16Val;
} ST_KDRV_XC_VR_R2BYTEMASK;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U8   u8Val;
} ST_KDRV_XC_VR_WRITEBYTE;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U16  u16Val;
} ST_KDRV_XC_VR_W2BYTE;

typedef struct __attribute__((packed))
{
    MS_U32  u32Reg;
    MS_U16  u16Mask;
    MS_U16  u16Val;
} ST_KDRV_XC_VR_W2BYTEMASK;

typedef enum
{
    E_KDRV_XC_PQ_VIDEO_CTRL_MODE_MAIN,
    E_KDRV_XC_PQ_VIDEO_CTRL_MODE_SUB,
    E_KDRV_XC_PQ_VIDEO_CTRL_MODE_MAX,
} EN_KDRV_XC_PQ_VIDEO_HW_CTRL_SEL;

typedef struct __attribute__((packed))
{
    MS_U16 u16Gain;
    MS_U16 u16Offset;
} ST_KDRV_XC_PQ_GAIN_OFFSET;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;

    MS_BOOL bInit;

    EN_KDRV_XC_PQ_VIDEO_HW_CTRL_SEL enMainSubSel;

    ST_KDRV_XC_PQ_GAIN_OFFSET astGainOff1[PQ_GAMMA_CHANNEL];
    ST_KDRV_XC_PQ_GAIN_OFFSET astGainOff2[PQ_GAMMA_CHANNEL];

    MS_BOOL bOne_Drv3; // One drives 3 channels, R=G, B=G
    MS_BOOL bEnable;
    MS_U32 au32_u20p12MaxBase[PQ_GAMMA_CHANNEL];
    MS_U32 au32_u20p12Lut[PQ_GAMMA_LUT_ENTRY*PQ_GAMMA_CHANNEL];
} ST_KDRV_XC_PQ_GAMMA_IN;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;

    MS_BOOL bEnable;
    MS_U16 u16_u0p16R;
    MS_U16 u16_u0p16G;
    MS_U16 u16_u0p16B;
} ST_KDRV_XC_PQ_RGB_PATTERN_IN;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;
    MS_U32 u32Length;

    MS_U8 u8Strength;
    MS_U8 u8SatLevel;
    MS_BOOL bEn;
} ST_KDRV_XC_PQ_BLUE_STRETCH_IN;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;

    MS_U8 u8Ctrl;
    MS_U8 u8Ctrl2;
    MS_BOOL bControlFlag;
    MS_U8 u8NRStrength;
    MS_U8 au8DNRLut[8];
} ST_KDRV_XC_CFD_NR;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Dimming panel type
    EN_KDRV_XC_DIMMING_PANEL_TYPE enDimmingPanelType;
} ST_KDRV_XC_CFD_DIMMING_PANEL_TYPE;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;
    /// Content Light level valid info
    MS_BOOL bContentLightLevelValid;
    /// Max Content Light Level in source, enhet is nit
    MS_U16  u16MaxContentLightLevel;
    /// Max average Picture Content Light Level in source, enhet is nit
    MS_U16  u16MaxPicAverageLightLevel;
}ST_KDRV_XC_HDR_CLL_INFO;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;
    /// SEI info valid info
    MS_BOOL bSEIValid;
    /// SEI Info, DisplayPrimariesX in R/G/B, real range is 0-1, multiply 50000
    MS_U16 u16DisplayPrimariesX[3];
    /// SEI Info, DisplayPrimariesY in R/G/B, real range is 0-1, multiply 50000
    MS_U16 u16DisplayPrimariesY[3];
    /// SEI Info, White point X, real range is 0-1, multiply 50000
    MS_U16 u16WhitePointX;
    /// SEI Info, White point Y, real range is 0-1, multiply 50000
    MS_U16 u16WhitePointY;
    /// SEI Info, MaxPanelLuminance, real range is 0-1, multiply 1000
    MS_U32 u32MaxPanelLuminance;
    /// SEI Info, MinPanelLuminance, real range is 0-1, multiply 1000
    MS_U32 u32MinPanelLuminance;
} ST_KDRV_XC_HDR_SEI_INFO;


typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U32 u32Length;
    /// Window (main or sub window)
    MS_U8 u8Win;
    ///HDR SEI info
    ST_KDRV_XC_HDR_SEI_INFO stHdrSEIInfo;
    ///HDR CLL Info
    ST_KDRV_XC_HDR_CLL_INFO stHdrCLLInfo;
}ST_KDRV_XC_HDR_METADATA;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;          //<Version of current structure.
    MS_U32 u32Length;           //<Length of this structure, u16Length=sizeof(ST_KDRV_XC_GCE_IN)

    MS_BOOL bEnable;
    MS_U16 u16PairNum;          //For Manual Mode: 18to256 curve
    MS_U32 *pu32IndexLut;       //For Manual Mode: 18to256 curve
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
    MS_U32 *pu32OutputLut;      //For Manual Mode: 18to256 curve
#if !defined (__aarch64__)
    void    *pDummy1;
#endif
    MS_U16 u16ManualBlendFactor; // For manual curve blends auto generated curve
} ST_KDRV_XC_PQ_MANUAL_LUMA_CURVE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;          //<Version of current structure.
    MS_U32 u32Length;           //<Length of this structure, u16Length=sizeof(ST_KDRV_XC_PQ_LUMA_INFO)

    MS_U8 u8Win;
    MS_U8 u8IpMaximumPixel;
    MS_U8 u8IpMinimumPixel;
    MS_S32 s32Yavg;
} ST_KDRV_XC_PQ_LUMA_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;          //<Version of current structure.
    MS_U32 u32Length;           //<Length of this structure, u16Length=sizeof(ST_KDRV_XC_PQ_STRETCH_SETTINGS)

    MS_U8 u8Win;
    MS_U8 u8_u0p8BlackStretchRatio;
    MS_U8 u8_u0p8WhiteStretchRatio;
    MS_U16 u16_u4p12BlackStretchGain;
    MS_U16 u16_u4p12WhiteStretchGain;
} ST_KDRV_XC_PQ_STRETCH_SETTINGS;

typedef enum
{
    E_KDRV_XC_CHROMA_INFO_CONTROL_SET_WINDOW_INFO = 0,
    E_KDRV_XC_CHROMA_INFO_CONTROL_GET_HIST,
    E_KDRV_XC_CHROMA_INFO_CONTROL_NUM,
} EN_KDRV_XC_CHROMA_INFO_CONTROL;

#define HISTOGRAM_SATOFHUE_WIN_NUM (6)
typedef struct __attribute__((packed))
{
    MS_U32 u32Version;          //<Version of current structure.
    MS_U32 u32Length;           //<Length of this structure, u16Length=sizeof(ST_KDRV_XC_PQ_CHROMA_INFO)

    MS_U8 u8Win;
    EN_KDRV_XC_CHROMA_INFO_CONTROL enChormaInfoCtl;
    MS_U8 u8SatbyHue_HueStart_Window0;
    MS_U8 u8SatbyHue_HueRange_Window0;
    MS_U8 u8SatbyHue_HueStart_Window1;
    MS_U8 u8SatbyHue_HueRange_Window1;
    MS_U8 u8SatbyHue_HueStart_Window2;
    MS_U8 u8SatbyHue_HueRange_Window2;
    MS_U8 u8SatbyHue_HueStart_Window3;
    MS_U8 u8SatbyHue_HueRange_Window3;
    MS_U8 u8SatbyHue_HueStart_Window4;
    MS_U8 u8SatbyHue_HueRange_Window4;
    MS_U8 u8SatbyHue_HueStart_Window5;
    MS_U8 u8SatbyHue_HueRange_Window5;
    MS_BOOL bHistValid;
    MS_U32 *apu32SatOfHueHist[HISTOGRAM_SATOFHUE_WIN_NUM];
    MS_U8  au8SatOfHueHistSize[HISTOGRAM_SATOFHUE_WIN_NUM];
} ST_KDRV_XC_PQ_CHROMA_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;          //<Version of current structure.
    MS_U32 u32Length;           //<Length of this structure, u32Length=sizeof(ST_KDRV_XC_PQ_NONLINEARSCALING_API_PARAM_IN)

    MS_U8  u8Win;//o:main 1:sub
    MS_U8  u8HNLEn; //by user
    MS_U16 u16SourceWidth;
    MS_U16 u16TargetWidth;     //target width of image for scaler , by user
    MS_U32 u32IniFactor;
    MS_U8  u8HShiftMode;
    MS_U32 u32CurrentRatioH;
    MS_U32 u32TargetRatioH;       //by user
    MS_U32 u32TargetRatioHStart;  //by user
    MS_U16 u16Width0;             //by user
    MS_U16 u16Width1;             //by user
    MS_U8 u8ParamFromVREn; //1: from VR  0: from settings
} ST_KDRV_XC_PQ_NONLINEARSCALING_API_PARAM_IN;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              //<Version of current structure.
    MS_U16 u16Length;               //<Length of this structure,

    MS_U32 u32CmdID;               //
    MS_U16 u16Data;                  //
} ST_KDRV_XC_PQ_SEND_CMD;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              //<Version of current structure.
    MS_U16 u16Length;               //<Length of this structure,

    MS_U32 u32CmdLen;           //
    MS_U32 *pu32Cmds;           //
#if !defined (__aarch64__)
    void    *pDummy0;
#endif
} ST_KDRV_XC_PQ_SEND_CMD_BIN;

//Scaler Path Control Info Start

// version 1 : Init Struct
#define CFD_MUX_INFO_VERSION (1)

typedef enum
{
    E_KDRV_XC_CFD_MUX_CREATE,
    E_KDRV_XC_CFD_MUX_DELETE,
    E_KDRV_XC_CFD_MUX_MAX,
} EN_KDRV_XC_CFD_MUX_ACTION;

typedef struct __attribute__((packed))
{
    // Structure version
    MS_U32 u32Version;
    // Structure length
    MS_U16 u16Length;
    // Action
    EN_KDRV_XC_CFD_MUX_ACTION enAction;
    // Window (main or sub window)
    MS_U8 u8Win;
    // Input source, reference to E_CFD_INPUT_SOURCE in mdrv_video_info_if.h
    MS_U8 u8InputSource;
} ST_KDRV_XC_MUX_INFO;

//Scaler Path Control Info END

//UCD related
typedef enum
{
    /// Init the UCD
    E_KDRV_XC_CFD_UCD_INIT,
    /// Set the UCD Setting
    E_KDRV_XC_CFD_UCD_SET,
    /// Get the UCD Setting
    E_KDRV_XC_CFD_UCD_GET,
    /// Set AI PQ UCD base addr
    E_KDRV_XC_CFD_UCD_SETAIPQADDRESS,
    /// Max
    E_KDRV_XC_CFD_UCD_MAX,
} EN_KDRV_XC_CFD_UCD_ACTION;

typedef struct __attribute__((packed))
{
  MS_U64 u64PhyBaseAddr;           /// baseaddr
  MS_U32 u32Size;                  /// size
  MS_U32 u32MiuNo;                 /// miu no
} ST_KDRV_XC_MEM_CLIENT;

typedef struct __attribute__((packed))
{
  ST_KDRV_XC_MEM_CLIENT stGUCDMemInfo;
  MS_U8 u8Mode;                 //Mode0:Default Mode1:LG
  MS_U8 u8width;                // LG mode Need
  MS_U8 u8height;               // LG mode Need
  ST_KDRV_XC_MEM_CLIENT stAIPQMemInfo;
} ST_KDRV_XC_UCD_INIT;

typedef struct __attribute__((packed))
{
    MS_BOOL benable;

    //only LG mode need
    MS_U32 u32CurveSize; //size of pu16_curve,from LG
    MS_U32 u32IndexSize; //size of pu16_index,from LG
    MS_U16 *pu16_curve; // from LG
#if !defined (__aarch64__)
    void    *pDummy1;
#endif
    MS_U16 *pu16_index; // from LG
#if !defined (__aarch64__)
    void    *pDummy2;
#endif
 } ST_KDRV_XC_UCD_SET;

typedef struct __attribute__((packed))
{
    //only LG mode need
    MS_U8 u8CurveNum; // to LG
    MS_U8 u8KnotNum;  // to LG
    MS_U32 u32AvgSize; // to LG
    MS_U32 u32MaxSize; // to LG
    MS_U32 u32MinSize; // to LG
    MS_U8 *pu8avg;  // to LG
#if !defined (__aarch64__)
    void    *pDummy1;
#endif
    MS_U8 *pu8max;  // to LG
#if !defined (__aarch64__)
    void    *pDummy2;
#endif
    MS_U8 *pu8min;  // to LG
#if !defined (__aarch64__)
    void    *pDummy3;
#endif
} ST_KDRV_XC_UCD_GET;


typedef struct __attribute__((packed))
{
    /// Structure version(XC_UCD_VERSION)
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;

    /// Window (main or sub window)
    MS_U8 u8Win;
    /// UCD action
    EN_KDRV_XC_CFD_UCD_ACTION enUCDAction;
    /// UCD Init Struct
    ST_KDRV_XC_UCD_INIT stUCDInit;
    /// UCD Set Struct
    ST_KDRV_XC_UCD_SET stUCDSet;
    /// UCD Get Struct
    ST_KDRV_XC_UCD_GET stUCDGet;

} ST_KDRV_XC_CFD_UCD;

// Version 1 : Initial structure
#define ST_KDRV_XC_EVENT_VERSION (1)

/// Define Poll Evnet MISC
/// please enum use BIT0 = 1, BIT1 = 2, BIT3 = 4
typedef enum
{
    E_KDRV_XC_EVENT_MISC_A_NULL = 0,
    E_KDRV_XC_EVENT_MISC_A_DOLBY_LOGO = 0x1,
    E_KDRV_XC_EVENT_MISC_A_HDR_SEAMLESS_FREEZE = 0x2,
    E_KDRV_XC_EVENT_MISC_A_HDR_SEAMLESS_UNFREEZE = 0x4,
    E_KDRV_XC_EVENT_MISC_A_RELOAD_SCALER_SETTING = 0x8,
    E_KDRV_XC_EVENT_MISC_A_SOURCE_CHANGE = 0x10,
    E_KDRV_XC_EVENT_MISC_A_HDR_TYPE_CHANGE = 0x20,
} EN_KDRV_XC_EVENT_MISC_A;

typedef enum
{
    E_KDRV_XC_EVENT_MISC_B_NULL = 0,
} EN_KDRV_XC_EVENT_MISC_B;

typedef enum
{
    E_KDRV_XC_EVENT_MISC_C_NULL = 0,
} EN_KDRV_XC_EVENT_MISC_C;

typedef enum
{
    E_KDRV_XC_EVENT_MISC_D_NULL = 0,
} EN_KDRV_XC_EVENT_MISC_D;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;                   ///<Version of current structure.
    MS_U32 u32Length;                   ///<Length of current structure.
    EN_KDRV_XC_EVENT_MISC_A enEventMISC_A;
    EN_KDRV_XC_EVENT_MISC_B enEventMISC_B;
    EN_KDRV_XC_EVENT_MISC_C enEventMISC_C;
    EN_KDRV_XC_EVENT_MISC_D enEventMISC_D;
} ST_KDRV_XC_EVENT_MISC;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              ///<Version of current structure.
    MS_U32 u32Length;               ///<Length of current structure.
    MS_U32 u32SupportHdrType;       ///refer to EN_KDRV_XC_CFD_HDR_TYPE.
} ST_KDRV_XC_HASH_INFO;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              ///<Version of current structure.
    MS_U32 u32Length;               ///<Length of current structure.
    ST_KDRV_XC_AUTODOWNLOAD_DATA_INFO* pstAutodownloadDataInfo;
    MS_U32 u32UsedSize;                         ///Used data size
} ST_KDRV_XC_AUTODOWNLOAD_TABLE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              ///<Version of current structure.
    MS_U32 u32Length;               ///<Length of current structure.
    MS_U32 u32MaxSize;
    MS_U32 u32UsedSize;
    MS_U32 *pu32Address;
#if !defined (__aarch64__)
    void    *pDummy;
#endif
    MS_U16 *pu16Value;
#if !defined (__aarch64__)
    void    *pDummy1;
#endif
    MS_U16 *pu16Mask;
#if !defined (__aarch64__)
    void    *pDummy2;
#endif
    EN_MLOAD_CLIENT_TYPE enClient;
} ST_KDRV_XC_ML_CMDS_TABLE;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;              ///<Version of current structure.
    MS_U32 u32Length;               ///<Length of current structure.
    MS_U32 u32Address;
    MS_U16 u16Value;
    MS_U16 u16Mask;
    EN_MLOAD_CLIENT_TYPE enClient;
} ST_KDRV_XC_ML_CMD_TABLE;

typedef enum
{
    E_KDRV_XC_DPU_CLIENT_INTERNAL,
    E_KDRV_XC_DPU_CLIENT_EXTERNAL,

    E_KDRV_XC_DPU_CLIENT_MAX
}EN_KDRV_XC_DPU_CLIENT;

typedef enum
{
    E_KDRV_XC_DPU_MOD_SET_NONE = 0,
    E_KDRV_XC_DPU_MOD_SET_BYPASS,
    E_KDRV_XC_DPU_MOD_SET_RESUME,
    E_KDRV_XC_DPU_MOD_SET_TRIGGER_ALL,

    E_KDRV_XC_DPU_MOD_SET_MAX
}  EN_KDRV_XC_DPU_MOD_SETTING;

typedef enum
{
    E_KDRV_XC_DPU_MOD_STATUS_NONE = 0,
    E_KDRV_XC_DPU_MOD_STATUS_BYPASS,
    E_KDRV_XC_DPU_MOD_STATUS_AVAIL,

    E_KDRV_XC_DPU_MOD_STATUS_MAX
}  EN_KDRV_XC_DPU_MOD_STATUS;

typedef enum
{
    E_KDRV_DPU_MOD_HDR_IDX = 0,
    E_KDRV_DPU_MOD_TNR_IDX,
    E_KDRV_DPU_MOD_DI_FILM_IDX,
    E_KDRV_DPU_MOD_MPEGNR_IDX,
    E_KDRV_DPU_MOD_VIDEO_ENHANCE_IDX,
    E_KDRV_DPU_MOD_RESOLUTION_ENHANCE_IDX,
    E_KDRV_DPU_MOD_CONSTRAST_ENHANCE_IDX,
    E_KDRV_DPU_MOD_COLOR_ENHANCE_IDX,
    E_KDRV_DPU_MOD_GAMUT_MAPPING_IDX,
    E_KDRV_DPU_MOD_PQ_GAMMA_IDX,
    E_KDRV_DPU_MOD_LOCALDIMING_IDX,
    E_KDRV_DPU_MOD_PANEL_IDX,
    E_KDRV_DPU_MOD_IDX_EXTERNAL_MAX,
    E_KDRV_DPU_MOD_ABF_IDX,
    E_KDRV_DPU_MOD_IDX_MAX
}  EN_KDRV_DPU_MOD_IDX;


typedef enum
{
    E_KDRV_XC_SPF_RESOLUTION_4K = 0,
    E_KDRV_XC_SPF_RESOLUTION_BELLOW4K,
    E_KDRV_XC_SPF_RESOLUTION_MAX,
}  EN_KDRV_XC_SPF_RESOLUTION;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;                              ///Version of current structure.
    MS_U32 u32Length;                               ///Length of current structure.
    MS_U8 u8Win;
    EN_KDRV_XC_DPU_CLIENT enClient;                 ///DPU moduels distinguish between internel(kernel xc or utopia xc) and external(utopia DPU api) call
    EN_KDRV_XC_DPU_MOD_SETTING enDpuModSetting;     ///DPU modules setting refer to EN_KDRV_XC_DPU_MOD_SETTING.
    MS_U64 u64DpuMod;                               ///DPU modules.
} ST_KDRV_XC_DPU_SETTING;

typedef struct __attribute__((packed))
{
    MS_U32 u32Version;                              ///Version of current structure.
    MS_U32 u32Length;                               ///Length of current structure.
    MS_U8 u8Win;
    EN_KDRV_XC_DPU_MOD_STATUS enDpuModStatus;       ///DPU modules setting refer to EN_KDRV_XC_DPU_MOD_STATUS.
    MS_U64 u64DpuMod;                               ///DPU modules.
} ST_KDRV_XC_DPU_STATUS;

typedef struct __attribute__((packed))
{
    /// Structure version
    MS_U32 u32Version;
    /// Structure length
    MS_U16 u16Length;
    ///for Error handler
    MS_U16 u16XruleTypeNum;
    ///is support?
    MS_BOOL bIsSupportSPF;
    ///SPF Xrule Resolution
    EN_KDRV_XC_SPF_RESOLUTION enXCSPFResolution;
} ST_KDRV_XC_CFD_SPF_XRULE;

//HDMI2.1 VRR
typedef enum
{
    E_VRR_FSM_CTRL_ST_NVRR = 0,
    E_VRR_FSM_CTRL_ST_VRR,
    E_VRR_FSM_CTRL_ST_CVRR,
    E_VRR_FSM_CTRL_ST_NC2V_P1,
    E_VRR_FSM_CTRL_ST_NC2V_P2,
    E_VRR_FSM_CTRL_ST_NV2C_P1,
    E_VRR_FSM_CTRL_ST_NV2C_P2,
    E_VRR_FSM_CTRL_ST_NV2C_P3,
    E_VRR_FSM_CTRL_ST_CV2N_P1,
    E_VRR_FSM_CTRL_ST_CV2N_P2,
} EN_VRR_FSM_CTRL_STATE;

typedef enum
{
    E_VRR_IRQ_FIN_ST_INIT = 0x00,
    E_VRR_IRQ_FIN_ST_NVRR_NC2VP1 = 0x01,
    E_VRR_IRQ_FIN_ST_NC2VP1_NC2VP2 = 0x02,
    E_VRR_IRQ_FIN_ST_NC2VP2_VRR = 0x04,
    E_VRR_IRQ_FIN_ST_VRR_NV2CP1 = 0x08,
    E_VRR_IRQ_FIN_ST_NV2CP1_NV2CP2 = 0x10,
    E_VRR_IRQ_FIN_ST_NV2CP2_NV2CP3 = 0x20,
    E_VRR_IRQ_FIN_ST_NV2CP3_CVRR = 0x40,
    E_VRR_IRQ_FIN_ST_CVRR_CV2NP1 = 0x80,
    E_VRR_IRQ_FIN_ST_CV2NP1_CV2NP2 = 0x100,
    E_VRR_IRQ_FIN_ST_CV2NP2_NVRR = 0x200,
    E_VRR_IRQ_FIN_ST_NVRR_NV2CP2 = 0x400,
    E_VRR_IRQ_FIN_ST_VRR_CV2NP1 = 0x800,
    E_VRR_IRQ_FIN_ST_CVRR_NC2VP1 = 0x1000,
    E_VRR_IRQ_FIN_ST_NV2CP1_NC2VP1 = 0x2000,
} EN_VRR_IRQ_FINITE_STATE;

typedef struct
{
    MS_BOOL bVRR_en;
    MS_BOOL bM_const;
    EN_VRR_FSM_CTRL_STATE eVrrFsmCtrlSt;
    EN_VRR_IRQ_FINITE_STATE eVrrIrqFinSt;
} ST_KDRV_XC_VRR_INFO;

typedef enum
{
    E_KDRV_XC_TEST_NONE = 0,
    E_KDRV_XC_TEST_PASS,
    E_KDRV_XC_TEST_FAIL,
    E_KDRV_XC_TEST_NOT_SUPPORTED,
    E_KDRV_XC_TEST_MAX
} EN_KDRV_XC_TEST_STATUS;

typedef enum
{
    E_KDRV_AUTO_UPLOAD_PATTERN_32_GRAY = 0,
    E_KDRV_AUTO_UPLOAD_PATTERN_BLACK,
    E_KDRV_AUTO_UPLOAD_PATTERN_RED,
    E_KDRV_AUTO_UPLOAD_PATTERN_SAT_240,
    E_KDRV_AUTO_UPLOAD_PATTERN_WHITE,
    E_KDRV_AUTO_UPLOAD_PATTERN_PICTURE,
    E_KDRV_AUTO_UPLOAD_PATTERN_MAX
} EN_KDRV_AUTO_UPLOAD_PATTERN;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;

    void (*fpSetHSEMemory)(MS_BOOL bIsIommuEnabled);
} ST_KDRV_XC_HSE_CALLBACK;

typedef enum
{
   E_KDRV_XC_EVENT_NONE,
   E_KDRV_XC_EVENT_CUS_PQ_DONE,                       ///Cus pq load done Event. pParam: pointer to ST_KDRV_XC_EVENT_CUS_PQ_DONE
   E_KDRV_XC_EVENT_MAX,
} EN_KDRV_XC_EVENT;

typedef struct
{
    MS_U32 u32Version;                                ///Version of current structure. it should be ST_KDRV_XC_EVENT_CUS_PQ_DONE_VERSION as input
    MS_U16 u16Length;                                 ///Length of current structure. it should be sizeof(ST_KDRV_XC_EVENT_CUS_PQ_DONE)
    MS_BOOL bCusPQDone;                               ///Cus pq load done status
} ST_KDRV_XC_EVENT_CUS_PQ_DONE;

typedef struct
{
    MS_U32 u32Version;
    MS_U32 u32Length;

    EN_KDRV_XC_EVENT enXCEvent;
    void *pParam;                                     ///pParam of current structure. it should be EN_KDRV_XC_EVENT internal enum corresponding structure as input
} ST_KDRV_XC_EVENT;

typedef struct
{
    EN_KDRV_SC_DEVICE enDeviceID;
    EN_KDRV_WINDOW enWindow;
    k_ds_reg_ip_op_sel enIpOpSel;
    MS_BOOL bReturn;
    EN_KDRV_DS_DEVICE enDSDeviceID;
} ST_KDRV_DS_GET_CMD_COUNT_EXT;

typedef struct __attribute__((packed))
{
    MS_BOOL bEnable;
    MS_BOOL bReturn;
} ST_KDRV_XC_ENABLE_PICTUREMODE_WRITING;
#endif //_MDRV_XC_ST_H
