//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
///
/// @file   drvMMIO.h
/// @brief  MMIO SYS Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __DRV_MMIO_H__
#define __DRV_MMIO_H__


#ifdef __cplusplus
extern "C"
{
#endif

#include "MsTypes.h"

//-------------------------------------------------------------------------------------------------
//  Driver Capability
//-------------------------------------------------------------------------------------------------
#define MSIF_MMIO_LIB_CODE              {'M','M','I','O'}    //Lib code
#define MSIF_MMIO_LIBVER                {'0','1'}            //LIB version
#define MSIF_MMIO_BUILDNUM              {'0','2'}            //Build Number
#define MSIF_MMIO_CHANGELIST            {'0','0','4','0','9','9','0','8'} //P4 ChangeList Number

#define MMIO_DRV_VERSION                /* Character String for DRV/API version             */  \
    MSIF_TAG,                           /* 'MSIF'                                           */  \
    MSIF_CLASS,                         /* '00'                                             */  \
    MSIF_CUS,                           /* 0x0000                                           */  \
    MSIF_MOD,                           /* 0x0000                                           */  \
    MSIF_CHIP,                                                                                  \
    MSIF_CPU,                                                                                   \
    MSIF_MMIO_LIB_CODE,                  /* IP__                                             */  \
    MSIF_MMIO_LIBVER,                    /* 0.0 ~ Z.Z                                        */  \
    MSIF_MMIO_BUILDNUM,                  /* 00 ~ 99                                          */  \
    MSIF_MMIO_CHANGELIST,                /* CL#                                              */  \
    MSIF_OS

//-------------------------------------------------------------------------------------------------
//  Macro and Define
//-------------------------------------------------------------------------------------------------
// HW register space
#define DRV_MMIO_PM_BANK            0x0001
#define DRV_MMIO_NONPM_BANK         0x0002
#define DRV_MMIO_FLASH_BANK0        0x0003
#define DRV_MMIO_FRC_BANK           0x0004
#define DRV_MMIO_EXTRIU_BANK        0x0005
#define DRV_MMIO_OTP_BANK           0x0010
#define DRV_MMIO_SPRAM_BANK         0x0011
#define DRV_MMIO_OTP_BANK2          0x0012
#define DRV_MMIO_XC_EXT_BANK        0x0013
#define DRV_MMIO_VDEC_BANK	    0x0014


// Specific HW IP base
#define DRV_MMIO_SC_BANK            0x0100
#define DRV_MMIO_TSP_BANK           0x0101
#define DRV_MMIO_NDS_BANK           0x0102
#define DRV_MMIO_NSK_BANK           0x0103
#define DRV_MMIO_CAM_BANK           0x0104
#define DRV_MMIO_NGA_BANK           0x0105
#define DRV_MMIO_SC1_BANK           0x0106


//-------------------------------------------------------------------------------------------------
//  Type and Structure
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
//  Function and Variable
//-------------------------------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////////////////////////
/// @file   drvMMIO.h
/// @brief  MMIO
/// @author MStar Semiconductor Inc.
/// @version 1.0
/////////////////////////////////////////////////////////////////////////////////////////////////////

/*! \defgroup SYS SYS modules
 *
 * \defgroup MMIO MMIO interface (drvMMIO.h)
 *  \ingroup SYS

	\brief

	MMIO maps the RIU to userspace so that drivers are capable to access it directly.

	<b> Diagram </b>
	\image html mmio_1.png

	<b> Code Flow </b>
	-# Init(setup mapping)
	-# Each driver get its own base
	\image html mmio_2.png

 * \defgroup MMIO_BASIC basic mmio interface
 *  \ingroup MMIO
*/

//-------------------------------------------------------------------------------------------------
/// Init MMIO driver
/// @ingroup MMIO_BASIC
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_MMIO_Init(void);
//-------------------------------------------------------------------------------------------------
/// Get the virtual address of iomap for register
/// @ingroup MMIO_BASIC
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_MMIO_GetBASE(MS_VIRT *ptrBaseaddr, MS_PHY *u32Basesize, MS_U32 u32Module);
//-------------------------------------------------------------------------------------------------
/// DeInit MMIO driver
/// @ingroup MMIO_BASIC
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_MMIO_Close(void);


#ifdef __cplusplus
}
#endif


#endif // __DRV_MMIO_H__
