/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <debug_impl.h>
#include <standby_impl.h>
#include <system_impl.h>
#include <mtk_standby.h>
#include <standby.h>
#include <upgrade_utility.h>
#if defined(CONFIG_MTK_PM)
#include <mtk-pm.h>
#endif
#include <mmc.h>
#define ENV_OAD_UPGRADE_POWER_ON "upgrade_power_mode"

int mtk_main_chip_power_standby(void)
{
    int ret = 0;

    UBOOT_TRACE("start\n");
    ret = emmc_set_pon_off_short();
    if (ret != 0)
    {
        UBOOT_ERROR("power off short failure\n");
    }
#if defined(CONFIG_MTK_PM)
    pm_set_boot_reason(PM_BR_SECONDARY);
#endif
    run_command("poweroff",0);

    UBOOT_TRACE("end\n");
    return 0;
}

int mtk_standby_mode_framework(void)
{
    int ret = 0;
    UBOOT_TRACE("start\n");
    char* upgrade_power_on_flag = NULL;
    char *usb_upgrade_complete_flag = NULL;
#if defined(CONFIG_MTK_PM)
    int bootreason = pm_get_boot_reason();
    if (bootreason == PM_BR_SECONDARY || bootreason == PM_BR_DC || bootreason == PM_BR_MAX_CNT ||
        bootreason == PM_BR_REBOOT || bootreason == PM_BR_REBOOT_SHELL || bootreason == PM_BR_REBOOT_FORCE)
    {
        UBOOT_DEBUG("Normal boot reason 0x%x \n", bootreason);
        return 0;
    }
#endif
    // after usb upgrade complete control
    usb_upgrade_complete_flag = env_get(ENV_UPGRADE_COMPLETE);
    if (usb_upgrade_complete_flag != NULL && strncmp(usb_upgrade_complete_flag, "1", 1) == 0)
    {
        env_set(ENV_UPGRADE_COMPLETE, NULL); //clean power on flag
        UBOOT_DEBUG("Normal boot reason: USB Upgrade Power on \n");
        env_save();
        return 0;
    }
    // after (Oad / update logic) upgrade power on control
    upgrade_power_on_flag=strdup(env_get(ENV_OAD_UPGRADE_POWER_ON));
    if(upgrade_power_on_flag!=NULL && strncmp(upgrade_power_on_flag, "power_on", strlen("power_on")) == 0)
    {
        env_set(ENV_OAD_UPGRADE_POWER_ON,NULL);//clean power on flag
        UBOOT_DEBUG("Normal boot reason: OAD Upgrade Power on \n");
        env_save();
        free(upgrade_power_on_flag);
        return 0;
    }
    /* Write remote boot status */
    PMU_REG_ACCESS(PMU_DUMMY_ADDR,PMU_DUMMY_OFFSET_BOOT_STATUS) = REMOTE_STATE_SECOND_STANDBY;
    UBOOT_DEBUG("Enter standby [%lx:%lx]=%lx\n",(unsigned long)PMU_DUMMY_ADDR, (unsigned long)PMU_DUMMY_OFFSET_BOOT_STATUS, (unsigned long)PMU_REG_ACCESS(PMU_DUMMY_ADDR,PMU_DUMMY_OFFSET_BOOT_STATUS));

    ret = do_standby_mode_power_control();
    if(ret != 0)
        UBOOT_ERROR("power control framework execuate failure\n");
    if(upgrade_power_on_flag)
        free(upgrade_power_on_flag);
    UBOOT_TRACE("end\n");
    return 0;
}
