/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */
#include <common.h>
#include <utility.h>
#include <iniparser.h>
#include <iniutility.h>
#include <debug_impl.h>
#include <standby_mode.h>
#include <mtk-pm.h>
#ifdef CONFIG_DATA_SEPARATION
#include <mtk_dataindex.h>
#endif

static bool already_init = false;
static struct standby_qhb_info standby_info;

int standby_init(void)
{
    int ret=0;
#ifdef CONFIG_DATA_SEPARATION
    char filepath[FILE_PATH_SIZE],part[PART_NAME_SIZE];
    const char *relpath;

    memset(part, 0, sizeof(part));
    memset(filepath, 0, sizeof(filepath));
    UBOOT_DEBUG("standby_init with data separation : STANDBY_SECTION[%s]STANDBY_KEY[%s]\n",STANDBY_SECTION,STANDBY_KEY);
    if(dataindex_get_key(filepath, FILE_PATH_SIZE, STANDBY_SECTION, STANDBY_KEY, NULL) == 0)
    {
        if(dataindex_resolve_path(part, PART_NAME_SIZE, &relpath, filepath) == 0)
        {
            ret = load_standby_qhb_info(part,relpath,&standby_info);
            if(ret < 0)
            {
                UBOOT_ERROR("Read ini information failure with data separation.[%s][%s]\n",part,relpath);
                return -1;
            }
        }
        else
        {
            UBOOT_ERROR("resolve path fail: %s \n",filepath);
            return -1;
        }
    }
    else
    {
        UBOOT_ERROR("cannot get %s:%s from dataindex file\n",STANDBY_SECTION,STANDBY_KEY);
        return -1;
    }
#else
    ret = load_standby_qhb_info(STANDBY_PARTITION,STANDBY_FILE_PATH,&standby_info);
    if (ret < 0)
    {
        UBOOT_ERROR("Read ini information failure with default setting.[%s][%s]\n",STANDBY_PARTITION,STANDBY_FILE_PATH);
        return -1;
    }
#endif

    return 0;
}

int standby_mode_get_mode(void)
{
    if (!already_init) {
        standby_init();
        already_init = true;
    }
    UBOOT_DEBUG("qhb_mode: %d\n",standby_info.qhb_mode);

    return standby_info.qhb_mode;
}

int standby_mode_is_enter_standby(void)
{
    if (!already_init) {
        standby_init();
        already_init = true;
    }
    UBOOT_DEBUG("enter_standby: %d\n",standby_info.enter_standby);

    return standby_info.enter_standby;
}

int standby_mode_get_second_standby_mode(void)
{
    if (!already_init) {
        standby_init();
        already_init = true;
    }
    UBOOT_DEBUG("second_standby_mode: %d\n",standby_info.second_standby_mode);

    return standby_info.second_standby_mode;
}

/*
* standby_mode_enter_standby: Check standby mode status API
* 0: Normal booting mode
* 1: second standby mode
*/
int standby_mode_enter_standby(void)
{
    int bootreason, standby_mode_pm_status;
    if (!already_init)
    {
        if (standby_init() != 0)
        {
            printf("Get standby flag failure, Normal booting mode!\n");
            return 0;
        }
        already_init = true;
    }
    UBOOT_DEBUG("second_standby_mode: %d enter_standby: %d\n", standby_info.second_standby_mode, standby_info.enter_standby);

    bootreason = pm_get_boot_reason();
    UBOOT_DEBUG("boot reason 0x%x\n", bootreason);
    if (bootreason == PM_BR_SECONDARY || bootreason == PM_BR_DC || bootreason == PM_BR_MAX_CNT ||
        bootreason == PM_BR_REBOOT || bootreason == PM_BR_REBOOT_SHELL || bootreason == PM_BR_REBOOT_FORCE)
    {
        standby_mode_pm_status = 0;
    }
    else
    {
        standby_mode_pm_status = 1;
    }

    if ((standby_info.second_standby_mode == STANDBY_MOMORY_MODE && standby_info.enter_standby == 1 && standby_mode_pm_status == 1)
       || (standby_info.second_standby_mode == STANDBY_SECOND_MODE && standby_mode_pm_status == 1))
    {
        printf("Enter second standby mode!\n");
        return 1;
    }

    printf("Normal booting mode!\n");
    return 0;
}
