/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <command.h>
#include <debug_impl.h>
#include <system_impl.h>

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
EN_DEBUG_LEVEL dbgLevel=DEFAULT_DEBUG_LEVEL;
EN_DEBUG_MODULE dbgModule=DEFAULT_DEBUG_MODULE;

int _init_debug_level(void)
{
    char *pEnv=NULL;

    pEnv=env_get(ENV_DEBUG_LEVLE);
    if(pEnv==NULL)
    {
        dbgLevel=DEFAULT_DEBUG_LEVEL;
        return 0;
    }

    if(strcmp(pEnv,STR_ERROR)==0)
    {
        dbgLevel=EN_DEBUG_LEVEL_ERROR;
        return 0;
    }
    else if(strcmp(pEnv,STR_INFO)==0)
    {
        dbgLevel=(EN_DEBUG_LEVEL_INFO+EN_DEBUG_LEVEL_ERROR);
        return 0;
    }
    else if(strcmp(pEnv,STR_TRACE)==0)
    {
        dbgLevel=(EN_DEBUG_LEVEL_INFO+EN_DEBUG_LEVEL_ERROR+EN_DEBUG_LEVEL_TRACE);
        return 0;
    }
    else if(strcmp(pEnv,STR_DEBUG)==0)
    {
        dbgLevel=(EN_DEBUG_LEVEL_INFO+EN_DEBUG_LEVEL_ERROR+EN_DEBUG_LEVEL_TRACE+EN_DEBUG_LEVEL_DEBUG+EN_DEBUG_LEVEL_BOOTTIME);
        return 0;
    }
    else if(strcmp(pEnv,STR_BOOTTIME)==0)
    {
        dbgLevel=EN_DEBUG_LEVEL_BOOTTIME+DEFAULT_DEBUG_LEVEL;
        return 0;
    }
    else if(strcmp(pEnv,STR_UDTBDEBUG)==0)
    {
        dbgLevel=(EN_DEBUG_LEVEL_ERROR+EN_DEBUG_LEVEL_UDTBDEBUG);
        return 0;
    }
    else if(strcmp(pEnv,STR_KDTBDEBUG)==0)
    {
        dbgLevel=(EN_DEBUG_LEVEL_ERROR+EN_DEBUG_LEVEL_KDTBDEBUG);
        return 0;
    }
    else
    {
        dbgLevel=DEFAULT_DEBUG_LEVEL;
    }

    return 0;
}

void debug_dtb(unsigned long address, char *option)
{
    char command[64];
    char *level;
    int ret;
    int snprintf_len = 0;

    level = env_get("dbgLevel");
    if(level != NULL && address != 0)
    {
        if(strncmp(level, option, strlen(option)) == 0)
        {
            printf("%s address:0x%lx\n", level, address);
            memset(command, 0, sizeof(command));
            snprintf_len = snprintf(command, sizeof(command)-1, "fdt addr 0x%lx", address);
            if(snprintf_len < 0 || (unsigned)snprintf_len >= (sizeof(command)-1))
            {
                UBOOT_ERROR("The array size is too small(%lu), snprintf fail 'fdt addr 0x%lx'.\n", sizeof(command)-1, address);
                return;
            }
            
            ret = run_command(command, 0);
            if(ret != 0)
                printf("Set dtb address failure for debug dtb content\n");

            memset(command, 0, sizeof(command));
            strncpy(command, "fdt print /", sizeof(command)-1);
            ret = run_command(command, 0);
            if(ret != 0)
                printf("Print dtb content data failure\n");
        }
    }
}

void _dump(void *addr, unsigned int size)
{
    char buffer[CMD_BUF]="\0";
    snprintf(buffer,CMD_BUF,"md.b %p %x",addr,size);
    run_command(buffer,0);
}

