/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#ifndef _INIUTILITY_H
#define _INIUTILITY_H
#define INI_INFO_SIZE   64
#define UPGRADE_FILE_NAME_SIZE   64
#define ETH_STRING_SIZE 18
#define WDT_STRING_SIZE 32
#define CONSOLE_INFO_SIZE  5
#define PWM_DUTY_INVALID 0xFFFFFFFF

struct bootlogo_info {
    char type[INI_INFO_SIZE];
    char path[INI_INFO_SIZE];
    int width;
    int height;
    int color_format;
};

struct network_info {
    char ethaddr[ETH_STRING_SIZE];
};

struct voc_env_info {
    int enable_mic;
    int enable_wakeup;
    int enable_seamless;
    int enable_smartspeaker;
};

struct standby_qhb_info {
    int qhb_mode;
    int enter_standby;
    int second_standby_mode;//0(Direct mode),1(Memory mode),2(Secondary mode)
};

struct wdt_info {
    char status[WDT_STRING_SIZE];
};

struct panel_ssc_info {
	int ssc_overwrite_enable;
	int ssc_enable;
	int ssc_modulation;
	int ssc_deviation;
};

struct panel_mirror_info {
    int valid;
    int mirror_mode;
};
struct upgrade_info {
    char upgrade_mode[INI_INFO_SIZE];
    char upgrade_filename[UPGRADE_FILE_NAME_SIZE];
    char upgrade_partition[INI_INFO_SIZE];
    char upgrade_storage_info[INI_INFO_SIZE];
    char upgrade_power_mode[INI_INFO_SIZE];
    char upgrade_complete[INI_INFO_SIZE];
    char auto_usb_upgrade[INI_INFO_SIZE];
};

struct bootmusic_info {
    bool enable;
    int volume;
    char filename[INI_INFO_SIZE];
    char part[INI_INFO_SIZE];
};

struct console_info {
    char console_hdmi[CONSOLE_INFO_SIZE];
    char console_onoff[CONSOLE_INFO_SIZE];
};

struct panel_dlg_info {
    int panel_dlg;
};

struct panel_pwm_duty_info {
    int pwm_duty;
    int dual_pwm_duty;
};

int get_bootlogo_info(const char *partition, const char *file, struct bootlogo_info *bootlogo, const char *avb_state);
int get_network_info(const char *partition, const char *file, struct network_info *network);
int get_voc_info(const char *partition, const char *file, struct voc_env_info *voc_env);
int load_standby_qhb_info(const char *partition, const char *file, struct standby_qhb_info *standby_info);
int store_standby_qhb_info(const char *partition, const char *file, struct standby_qhb_info *standby_info);
int get_wdt_info(const char *partition, const char *file, struct wdt_info *wdtinfo);
int get_panel_mirror_info(const char *partition, const char *file, struct panel_mirror_info *info);
int get_upgrade_info(const char *partition, const char *file, struct upgrade_info *upgradeinfo);
int get_bootmusic_info(const char *partition, const char *file, struct bootmusic_info *bootmusic);
int set_upgrade_complete(const char *partition, const char *file, struct upgrade_info *upgradeinfo, int complete_flag, unsigned char **rawdata_out, unsigned int *size_out);
int get_panel_ssc_info(const char *partition, const char *file, struct panel_ssc_info *ssc_info);
int get_console_info(const char *partition, const char *file, struct console_info *consoleinfo);
int get_panel_dlg_info(const char *partition, const char *file, struct panel_dlg_info *info);
int get_panel_pwm_duty_info(const char *partition, const char *file, struct panel_pwm_duty_info *info);

#endif // #ifndef _INIUTILITY_H