/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#ifndef _MTK_DTBO_H_
#define _MTK_DTBO_H_
#include <linux/list.h>

typedef struct{
	unsigned long dtb_addr;
	unsigned long dtbo_img_addr;
	bool is_uboot_dtb;
}DTB_INFO_T;

/**
* This interface should be used for customer to get u-boot flag.
*/
bool is_uboot_dtb(void);

void get_uboot_dtb_addr(unsigned long *dtb_addr);

/**
 * step1: u-boot should call set_dtbo_image_address() to set dtbo_img_addr
 */
void set_dtbo_image_address(unsigned long dtbo_img_addr);

/**
 * step2: u-boot should call set_dtb_image_address(dtb_addr, is_uboot_dtb)
 *   to set dtb_addr to this module with is_uboot_dtb
 *   set_dtb_image_address(gd->fdt_blob, true) for overlay u-boot dtb with dtbos
 *   set_dtb_image_address(knl_fdt_blob, false) for overlay kernel dtb with dtbos
 */
void set_dtb_image_address(unsigned long dtb_addr, bool is_uboot_dtb);

/**
 * step3: u-boot should call init_dtbo_selection().
 */
void init_dtbo_selection(void);

/**
 * This interface should be used for customer to implement its device
 * auto-detection and apply dt overlay
 */
void uboot_dtbo_selection(void);

/**
 * This interface should be used for customer to load dtbo from Android dtbo
 * format image according to dtbo_id. After calling, u-boot dtbo is overlay
 * to u-boot dtb. Besides, the selected dtbos are stored into this module so
 * that it's no need to select dtbo again when Linux kernel dtb perform
 * overlaying dtbos
 *
 * dtbo_id: dtbo_id in Android dtbo format image according to Android spec
 * return: zero for success
 *         negative value to indicate error
 */
int dtb_overlay(int dtbo_id);

/**
* This interface is used for getting kernel dtb addr.
*/
unsigned long get_dtb_image_address(void);

#endif