/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#ifndef _INIT_SECURE_BOOT_H_
#define _INIT_SECURE_BOOT_H_
typedef struct{ //ree_arg_data
    unsigned int reeloader_version;
    unsigned int uboot_version;
    unsigned int rollback_enabled;
    unsigned int teefail_flag;
}ST_REE_ARG_DATA; 

typedef enum
{
    KEY_TYPE_CKB_A_RI_KEY = 0,
    KEY_TYPE_CKB_D_RI_KEY,
    KEY_TYPE_CKB_A_UPG_KEY ,
    KEY_TYPE_CKB_D_UPG_KEY ,
    KEY_TYPE_CKB_A_CE_KEY ,
    KEY_TYPE_CKB_D_CE_KEY ,
    KEY_TYPE_NULL = 0xFFFF
}EN_SECURE_KEY_TYPE;

#define MAX_KEY_SIZE 0x6


typedef struct {
    unsigned int size;
    unsigned int offset;
    unsigned int keytype;
    unsigned int reserved;
}st_key_info;

typedef struct
{
    unsigned int len;
    unsigned char* data;
}st_secure_key;

//-------------------------------------------------------------------------------------------------
/// check tee arg data is tee fail
/// @return  int                              \b OUT: 1,  sucessfully , 0 ,   unknow , -1,  fail
//-------------------------------------------------------------------------------------------------
int secure_is_tee_fail(void);
/******************************************************************************
 *
 * secure_get_key:     get secure key.
 * EN_SECURE_KEY_TYPE: secure key type,  ex: KEY_TYPE_CKB_A_RI_KEY,KEY_TYPE_CKB_D_RI_KEY...
 * st_secure_key:      key,
 *
 * return :            1 = Success , -1 = Failure
 *
*****************************************************************************/
int secure_get_key(EN_SECURE_KEY_TYPE secureKeyType,st_secure_key* secure_key);

/******************************************************************************
 *
 * secure_clean_key:     clean secure key.
 *
 * return :            1 = Success , -1 = Failure
 *
*****************************************************************************/
int secure_clean_key(void);
#endif

