/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <debug_impl.h>
#include "u-boot/drv_aes.h"
#include "hal_aes.h"

void array_reverse(unsigned char* ary, unsigned int len)
{
    int i, j;
    char tmp;
    for (i=0, j=len-1; i<j; ++i, --j)
    {
        tmp = ary[i];
        ary[i] = ary[j];
        ary[j] = tmp;
    }
}

int HW_AESDMA_Encrypt_Ext(unsigned long long inaddr, unsigned long long outaddr, unsigned int len, unsigned int engine, E_AESDMA_KEYTYPE keytype, unsigned char *key, unsigned char *iv)
{
    unsigned int timeout_count = 0;

    HAL_AESDMA_Reset();
    HAL_AESDMA_ResetSource();

    HAL_AESDMA_SetEngine(engine);
    HAL_AESDMA_SetDecrypt(0);
    HAL_AESDMA_SetKeyIndex(keytype, (unsigned int*)key);
    HAL_AESDMA_SetIV((unsigned int*) iv);
    HAL_AESDMA_SetFileInAddr(inaddr);
    HAL_AESDMA_SetFileOutAddr(outaddr);
    HAL_AESDMA_SetFileSize(len);
    HAL_AESDMA_FileOutEnable(1);

    flush_dcache_all();
    HAL_AESDMA_Start(1);

    while (!HAL_AESDMA_IsFinished())
    {
        udelay(AES_DELAY_10);
        timeout_count++;
        if(timeout_count > AES_TIMEOUT_VALUE)
        {
            UBOOT_ERROR("%s: timeout!\n", __func__);
            HAL_AESDMA_Reset();
            return -1;
        }
    }

    HAL_AESDMA_Reset();
    return 0;
}

int HW_AESDMA_Dncrypt_Ext(unsigned long long inaddr, unsigned long long outaddr, unsigned int len, unsigned int engine, E_AESDMA_KEYTYPE keytype, unsigned char *key, unsigned char *iv)
{
    unsigned int timeout_count = 0;

    HAL_AESDMA_Reset();
    HAL_AESDMA_ResetSource();

    HAL_AESDMA_SetEngine(engine);
    HAL_AESDMA_SetDecrypt(1);//Encrypt:0, Decrypt:1
    HAL_AESDMA_SetKeyIndex(keytype, (unsigned int*)key);
    HAL_AESDMA_SetIV((unsigned int*) iv);
    HAL_AESDMA_SetFileInAddr(inaddr);
    HAL_AESDMA_SetFileOutAddr(outaddr);
    HAL_AESDMA_SetFileSize(len);
    HAL_AESDMA_FileOutEnable(1);

    flush_dcache_all();
    HAL_AESDMA_Start(1);

    while (!HAL_AESDMA_IsFinished())
    {
        udelay(AES_DELAY_10);
        timeout_count++;
        if(timeout_count > AES_TIMEOUT_VALUE)
        {
            UBOOT_ERROR("%s: timeout!\n", __func__);
            HAL_AESDMA_Reset();
            return -1;
        }
    }

    HAL_AESDMA_Reset();
    return 0;
}
