/* Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * Copyright  (C) [2020]  MediaTek Inc. All rights reserved.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 */

#include <common.h>
#include <system_impl.h>
#include "hal_sha.h"
#include "u-boot/drv_sha.h"

int _HW_SHA_Calculate(Drv_SHAMode eMode, unsigned long long PAInBuf, unsigned int Size, unsigned long long PAOutBuf)
{
    unsigned int timeout_count = 0;
    unsigned long long VAOutBuf = 0;

    // Clear all SHA setting(clear the last outputs)
    HAL_SHA_Reset();
    // Error handler for 16-byte alignment limitation
    if(PAInBuf & WORD_MASK)
    {
        printf("Input Address should be 16-byte alignment\n");
        return -1;
    }
    if(HAL_SHA_Calculate(eMode, PAInBuf, Size, PAOutBuf) != 0)
        return -1;

    udelay(SHA_DELAY_40);
    flush_dcache_all();
    udelay(SHA_DELAY_200);

    HAL_SHA_Start(1);
    while ((HAL_SHA_Get_Status() & SHARNG_CTRL_SHA_READY) == 0)
    {
        udelay(SHA_DELAY_10);
        timeout_count++;
#if (SHA_WAIT == 1)
        //500ms timeout
        if(timeout_count > SHA_TIMEOUT_VALUE)
        {
            HAL_SHA_Reset();
            return -1;
        }
#endif
    }

    VAOutBuf = phyical_addr_to_virtual_addr(PAOutBuf);
    if(VAOutBuf == INVALID_ADDR)
    {
        return -1;
    }

    HAL_SHA_Out(eMode, VAOutBuf);
    while ((HAL_SHA_Get_Status() & SHARNG_CTRL_SHA_BUSY) != 0);

    HAL_SHA_Start(0);

    return 0;
}

int HW_SHA_Calculate(Drv_SHAMode eMode, unsigned long long PAInBuf, unsigned int Size, unsigned long long PAOutBuf)
{
    return _HW_SHA_Calculate(eMode, PAInBuf, Size, PAOutBuf);
}

