//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
#include "eMMC.h"

void fde_aes_sw_reset(void)
{
	REG_FCIE_SETBIT(FDE_AES_SW_RESET, BIT0);
	REG_FCIE_CLRBIT(FDE_AES_SW_RESET, BIT0);
}

void fde_aes_set_key(U8 *pu8_key, U32 u32_key_size)
{
	int i;
	for (i = 0; i < 8; i++)
		REG_FCIE_W(GET_REG_ADDR(FDE_AES_KEY, i), (pu8_key[i * 2 + 1] << 8) | pu8_key[i * 2]);

	REG_FCIE_CLRBIT(FDE_AES_CONTROL0, BIT_KEY_128BIT_SIZE);	//key size is 128 bit
	if (u32_key_size >= 24) {
		for (i = 8; i < 12; i++)
			REG_FCIE_W(GET_REG_ADDR(FDE_AES_KEY, i), (pu8_key[i * 2 + 1] << 8) | pu8_key[i * 2]);
		REG_FCIE_SETBIT(FDE_AES_CONTROL0, BIT_KEY_192BIT_SIZE); //key size is 192 bit
	}
	if (u32_key_size >= 32) {
		for (i = 12; i < 16; i++)
			REG_FCIE_W(GET_REG_ADDR(FDE_AES_KEY, i), (pu8_key[i * 2 + 1] << 8) | pu8_key[i * 2]);
		REG_FCIE_CLRBIT(FDE_AES_CONTROL0, BIT_MASK_KEY_SIZE);
		REG_FCIE_SETBIT(FDE_AES_CONTROL0, BIT_KEY_256BIT_SIZE); //key size is 256 bit
	}
}

void fde_set_set_iv(U8 *pu8_iv)
{
	int i;
	for (i = 0; i < 8; i++)
		REG_FCIE_W(GET_REG_ADDR(FDE_AES_IV, i), (pu8_iv[i * 2 + 1] << 8) | pu8_iv[i * 2]);
}

void fde_aes_set_ctr(U8 *pu8_ctr)
{
	int i;
	for (i = 0; i < 8; i++)
		REG_FCIE_W(GET_REG_ADDR(FDE_AES_CTR, i), (pu8_ctr[i * 2 + 1] << 8) | pu8_ctr[i * 2]);
}

void fde_aes_set_tcalc_key(U8 *pu8_tkey)
{
	int i;
	for (i = 0; i < 16; i++)
		REG_FCIE_W(GET_REG_ADDR(FDE_AES_XEX_HW_T_CALC_KEY, i), (pu8_tkey[i * 2 + 1] << 8) | pu8_tkey[i * 2]);
}

int fde_aes_set_mode(U8 u8_mode)
{
	if (u8_mode != ECB_MODE && u8_mode != CBC_MODE &&
	    u8_mode != CTR_MODE &&
	    u8_mode != OFB_MODE &&
	    u8_mode != CFB_MODE &&
	    u8_mode != XTS_MODE) {
		return -1;
	}
	REG_FCIE_CLRBIT(FDE_AES_CONTROL0, BIT_MASK_AES_MODE);
	REG_FCIE_SETBIT(FDE_AES_CONTROL0, u8_mode << BIT_AES_MODE_SHIFT);
	return 0;
}

int fde_aes_set_codec_dir(U8 u8_dir)
{
	if (u8_dir == ENABLE_ENC)
		REG_FCIE_CLRBIT(FDE_AES_CONTROL0, BIT_ENC_OR_DEC);
	else if (u8_dir == ENABLE_DEC)
		REG_FCIE_SETBIT(FDE_AES_CONTROL0, BIT_ENC_OR_DEC);
	else
		return -1;

	return 0;
}

void fde_aes_set_xts_lba(U32 u32_lba)
{
	U8 u8_ctr[16];
	U16 u16_mode;
	int i;
	REG_FCIE_R(FDE_AES_CONTROL0, u16_mode);
	u16_mode &= BIT_MASK_AES_MODE;
	u16_mode >>= BIT_AES_MODE_SHIFT;
	if (u16_mode != XTS_MODE)
		return ;
	//emmc_debug(0, 1, "LBA %Xh\n", u32_lba);
	for (i = 0; i < 16; i++) {
		if (i < sizeof(U32)) {
			u8_ctr[i] = u32_lba & 0xFF;
			u32_lba >>= 8;
		} else {
			u8_ctr[i] = 0;
		}
	}
	fde_aes_set_ctr(u8_ctr);
}

void fde_aes_set_xex_hw_t_calc_kick(int val)
{
	if (val == 1) {
		while ((REG_FCIE(FDE_AES_BUSY) & BIT_AES_BUSY))
			;

		REG_FCIE_SETBIT(FDE_AES_XEX_HW_T_CALC_KICK, BIT_AES_XEX_HW_T_CALC_KICK);
	} else if (val == 0)
		REG_FCIE_CLRBIT(FDE_AES_XEX_HW_T_CALC_KICK, BIT_AES_XEX_HW_T_CALC_KICK);
}

char mode_string[9][20] = {
	"ECB Mode",
	"CBC_Mode",
	"CTR_Mode",
	"",
	"",
	"OFB_Mode",
	"CFB_Mode",
	"",
	"XTS_Mode"
};

int fde_aes_init(void)//should be configured in TEE world
{
	U8 u8_mode;
	U8 u8_key[32], u8_iv[16], u8_ctr[16], u8_t_key[32];
	int i;
//	u8_mode = ECB_MODE;
	u8_mode = XTS_MODE;
//	u8_mode = CBC_MODE;
//	u8_mode = CTR_MODE;
//	u8_mode = OFB_MODE;
//	u8_mode = CFB_MODE;


	emmc_debug(0, 1, "Test %s\n", mode_string[u8_mode]);

	memset(u8_iv, 0, sizeof(u8_iv));
	memset(u8_ctr, 0, sizeof(u8_ctr));
	memset(u8_t_key, 0, sizeof(u8_t_key));
	for (i = 0 ; i < 32; i++) {
		u8_key[i] = i;
		if (u8_mode == XTS_MODE)
			u8_t_key[i] = 31 - i;//((0x3f - i*2 + 12) * 3) & 0xFF;

		#if 0
		if (u8_mode == CBC_MODE || u8_mode == OFB_MODE || u8_mode == CFB_MODE)
			if (i < 16)
				u8_IV[i] = 15 - i;//((0x3f - i * 2 + 12) * 3) & 0xFF;

		if (u8_mode == CTR_MODE)
			if (i < 16)
				u8_ctr[i] = 15 - i;//((0x3f - i * 2 + 12) * 3) & 0xFF;
		#endif
	}
//	if (u8_mode != XTS_MODE)
//		REG_FCIE_SETBIT(FDE_AES_FUN, BIT2);
	fde_aes_set_key(u8_key, 16);
	fde_set_set_iv(u8_iv);
	fde_aes_set_ctr(u8_ctr);
	fde_aes_set_tcalc_key(u8_t_key);
	fde_aes_set_mode(u8_mode);
	REG_FCIE_W(FDE_AES_DATA_UNIT0, 0x200);
	REG_FCIE_W(FDE_AES_DATA_UNIT1, 0x0);
	return 0;
}

void fde_aes_open(void)
{
	REG_FCIE_SETBIT(FDE_AES_FUN, BIT_AES_FUNC_EN);
}

void fde_aes_close(void)
{
	REG_FCIE_CLRBIT(FDE_AES_FUN, BIT_AES_FUNC_EN);
}

void FDE_AES_CQE_Open(void)
{
	REG_FCIE_SETBIT(FDE_AES_FUN, BIT_AES_AUTO_CONFIG_CMDQ);
}

void FDE_AES_CQE_Close(void)
{
	REG_FCIE_CLRBIT(FDE_AES_FUN, BIT_AES_AUTO_CONFIG_CMDQ);
}

int fde_aes_switch_state(void)
{
	return ((REG_FCIE(FDE_AES_FUN) & BIT_AES_SWITCH_ON)) ? 1 : 0;
}

void fde_aes_func_open(U8 u8_dir, U32 lba)
{
	if ((REG_FCIE(FDE_AES_FUN) & BIT_AES_SWITCH_ON)) {
		fde_aes_sw_reset();
		fde_aes_set_xts_lba(lba);
		fde_aes_set_codec_dir(u8_dir);
		fde_aes_set_xex_hw_t_calc_kick(1);
		fde_aes_open();
    }
}

void fde_aes_func_close(void)
{
	if ((REG_FCIE(FDE_AES_FUN) & (BIT_AES_SWITCH_ON | BIT_AES_FUNC_EN))
		== (BIT_AES_SWITCH_ON|BIT_AES_FUNC_EN)) {
		fde_aes_set_xex_hw_t_calc_kick(0);
		fde_aes_close();
	}
}

