//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// (!¡±MStar Confidential Information!¡L) by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

#include "MsCommon.h"
#ifndef MSOS_TYPE_LINUX_KERNEL
#include <string.h>
#endif
#include "MsTypes.h"
#include "halGOP.h"
#include "regGOP.h"
#include "halCHIP.h"
#include "drvSYS.h"

//------------------------------------------------------------------------------
//  Driver Compiler Options
//------------------------------------------------------------------------------
#define HAL_GOP_DEBUGINFO(x)   //x

//------------------------------------------------------------------------------
//  Local Defines
//------------------------------------------------------------------------------
#define RIU     ((unsigned short volatile *) pGOPHalLocal->va_mmio_base)
#define GOP_WRITE2BYTE(addr, val)    { RIU[addr] = val; }
#define GOP_READ2BYTE(addr)            RIU[addr]
#define GOP_DST_MASK                   0xFUL

// Debug Log
#include "ULog.h"
MS_U32 u32GOPDbgLevel_hal = 0;

#ifdef CONFIG_GOP_DEBUG_LEVEL
// Debug Logs, level form low(INFO) to high(FATAL, always show)
// Function information, ex function entry
#define GOP_H_INFO(x, args...) if(u32GOPDbgLevel_hal >= E_GOP_Debug_Level_HIGH) {ULOGI("GOP HAL", x, ##args);}
// Warning, illegal paramter but can be self fixed in functions
#define GOP_H_WARN(x, args...) if(u32GOPDbgLevel_hal >= E_GOP_Debug_Level_HIGH) {ULOGW("GOP HAL", x, ##args);}
//  Need debug, illegal paramter.
#define GOP_H_DBUG(x, args...) if(u32GOPDbgLevel_hal >= E_GOP_Debug_Level_MED) {ULOGD("GOP HAL", x, ##args);}
// Error, function will be terminated but system not crash
#define GOP_H_ERR(x, args...) {ULOGE("GOP HAL", x, ##args);}
// Critical, system crash. (ex. assert)
#define GOP_H_FATAL(x, args...) {ULOGF("GOP HAL", x, ##args);}
#else
#define GOP_H_INFO(x, args...)
// Warning, illegal paramter but can be self fixed in functions
#define GOP_H_WARN(x, args...)
//  Need debug, illegal paramter.
#define GOP_H_DBUG(x, args...)
// Error, function will be terminated but system not crash
#define GOP_H_ERR(x, args...)
// Critical, system crash. (ex. assert)
#define GOP_H_FATAL(x, args...)
#endif

//------------------------------------------------------------------------------
//  Local Var
//------------------------------------------------------------------------------
MS_BOOL bIsMuxVaildToGopDst[MAX_GOP_MUX][MAX_DRV_GOP_DST_SUPPORT] =
{
    /*IP0,      IP0_SUB,  MIXER2VE, OP0,         VOP,   IP1,       IP1_SUB, MIXER2OP*/
    {TRUE,    FALSE, FALSE,    TRUE,    TRUE, FALSE, FALSE,  FALSE},         /*All Gop Dst case is vaild or FALSE for mux 0 */
    {TRUE,    FALSE, FALSE,    FALSE,   TRUE, FALSE, FALSE,  FALSE},        /*All Gop Dst case is vaild or FALSE for mux 1 */
    {TRUE,    FALSE, FALSE,    TRUE,    TRUE, FALSE, FALSE,  FALSE},         /*All Gop Dst case is vaild or FALSE for mux 0 */
    {TRUE,    FALSE, FALSE,    TRUE,    TRUE, FALSE, FALSE,  FALSE},         /*All Gop Dst case is vaild or FALSE for mux 0 */
};
#ifdef GOP_CMDQ_ENABLE
extern MS_U16 u16MIUSelect[MAX_GOP_SUPPORT];
extern MS_U8 bMIUSelect[MAX_GOP_SUPPORT];
#endif
static MS_BOOL _bHalCSCSelect[MAX_GOP_SUPPORT]={FALSE};
static ST_GOP_CSC_TABLE _stHalCSCTbl[MAX_GOP_SUPPORT];

MS_U16 gu16GopR2yBt601[GOP_CSC_ELEMENT_NUM] =
{
    0xE010, 0x01C0, 0x1E88,
    0x1FB7, 0x0106, 0x0203,
    0x0064, 0x1F69, 0x1ED7,0x01C0
};
MS_U16 gu16GopY2rBt601[GOP_CSC_ELEMENT_NUM] =
{
    0x001D, 0x0667, 0x04AC,
    0x0000, 0x1CBD, 0x04AC,
    0x1E6E, 0x0000, 0x04AC,0x0818
};
MS_U16 gu16GopY2rCSC[GOP_CSC_ELEMENT_NUM] =
{
    0x001D, 0x0667, 0x04AC,
    0x0000, 0x1CBD, 0x04AC,
    0x1E6E, 0x0000, 0x04AC,0x0818
};
MS_U16 gu16GopR2rCSC[GOP_CSC_ELEMENT_NUM] =
{
    0x0030, 0x0400, 0x0000,
    0x0000, 0x0000, 0x0400,
    0x0000, 0x0000, 0x0000,0x0400
};

GOP_CHIP_PROPERTY g_GopChipPro =
{
    .bSetHSyncInverse =         FALSE,
    .bGop1GPalette =            FALSE,
    .bSetHPipeOfst =            FALSE,
    .bNeedCheckMVOP =           FALSE,
    .bNeedSetMUX1ToIP0 =        FALSE,
    .bNeedSetMUX3ToIP0 =        FALSE,
    .bNewMux   =                TRUE,
    .bNewPalette   =            TRUE,
    .bNewBwReg =                TRUE,
    .bGop2VStretch =            TRUE,
    .bIgnoreIPHPD  =            TRUE,  //Uranus4 has handshack with XC, should not set HPD
    .bIgnoreVEHPD  =            FALSE,  //Uranus4 to VE through Mixer, do not need adjust HPD
    .bhastilemode  =            TRUE,
    .bInitNotEnableGOPToSC =    FALSE,  //For Uranus4 mux init setting, enable GOP to SC in GOP init would cause problem
    .bAutoAdjustMirrorHSize =   TRUE,   //whether hw will auto adjust start addr when H mirror is enable
    .bGOPVscalePipeDelay =     {FALSE, FALSE, TRUE, FALSE, FALSE}, //setting GOP with/without Vscale
    .bGOPWithVscale =           {TRUE, TRUE, FALSE, FALSE, FALSE},
    .bGOPWithScaleDown      =   {FALSE, FALSE, FALSE, FALSE, FALSE},

    .bDWINSupport   =           FALSE,
    .DwinVer =                  0x1,
    .bTstPatternAlpha =         TRUE,
    .bXCDirrectBankSupport =    TRUE,   /*XC Dirrect Bank R/W*/
    .bFRCSupport =              FALSE,  /*OC path*/
    .bGOPMixerToVE=             FALSE,  /*Mixer to VE path*/
    .bBnkForceWrite =           TRUE,   /*Direct Bank Force Write*/
    .bPixelModeSupport =        TRUE,   /*Pixel Mode Support*/
    .bScalingDownSupport=       TRUE,
    .b2Pto1PSupport=            FALSE,
    .bTLBSupport=               {TRUE, FALSE, TRUE, FALSE, FALSE, FALSE},
    .GOP_TestPattern_Vaild=     E_GOP0,
    .bInternalMIUSelect=        {TRUE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE}, //{E_GOP0, E_GOP1, E_GOP2, E_GOP3, E_GOP4, E_GOP_Dwin, E_GOP_MIXER, E_GOP5}

#ifdef ENABLE_GOP_T3DPATCH
    .GOP_PD =                   GOP_PD_T3D,
#else
    .GOP_PD =                   GOP_PD_NORMAL,
#endif
    .GOP_IP_PD =                0x00,
    .GOP_MVOP_PD =              0x69,
    .GOP_VE_PD =                0x89,
    .GOP_MIXER_PD =             0x0,
    .GOP_NonVS_PD_Offset =      0x5, //GOP without Vsacle might need add offset on pipedelay
    .GOP_VE_V_Offset =          0x0,

    .GOP_MUX_Delta  =           0x1,
    .GOP_Mux_Offset =           {0x0, 0x3, 0x6, 0x9, 0xC},
    .GOP_MapLayer2Mux =         {E_GOP_MUX0, E_GOP_MUX1, E_GOP_MUX2, E_GOP_MUX3, E_GOP_MUX4},
    .GOP_Mux_FRC_offset=        0x13,

    .WordUnit =                 GOP_WordUnit,
    .TotalGwinNum =             GOP_TotalGwinNum,
    .Default_ConsAlpha_bits =   DRV_VALID_8BITS,
    .enGOP3DType =              E_DRV_3D_DUP_FULL,
    .bH4tap256Phase = TRUE,
    .bSupportMixerSDR2HDR =     FALSE,
    .bSupporBrightness3Chan =   TRUE,
    .bSupportContrast =         FALSE,
    .bSupportV4tap =           {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE},
    .bSupportV_BiLinear =      {TRUE, TRUE, TRUE, FALSE, FALSE, FALSE},
    .bSupportVOPPathSel =       TRUE,
    .u16BefRGB3DLupPDOffset =   0xA,
    .u16BefPQGammaPDOffset =    0x7,
    .u16AftPQGammaPDOffset =    0x3,
    .enBrightnessInfo      =    E_DRV_BRI_2SCOMPLEMENT,
    .bSupportCSCTuning =        TRUE,
    .bSupportXC2DMenuLoad =     TRUE,
    .bSupportH6Tap_8Phase =     {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE},
    .bSupportHDuplicate =       {TRUE, TRUE, TRUE, FALSE, FALSE, FALSE},
    .bSupportH4Tap_256Phase =   {TRUE, TRUE, TRUE, FALSE, FALSE, FALSE},
    .bAutoDetectVersion_1_0  =   {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE},
    .bUse3x3MartixCSC =    TRUE,
    .bAutoDetectVersion_2_0 = {TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE},
    .bSupportV2tap_16Phase = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE},
    .bOPMuxDoubleBuffer = TRUE,
    .bSupportCSCDoubleBuffer = TRUE,
};


//------------------------------------------------------------------------------
//  Global Functions
//------------------------------------------------------------------------------
void HAL_GOP_GetGOPEnum(GOP_CTX_HAL_LOCAL *pGOPHalLocal, GOP_TYPE_DEF* GOP_TYPE)
{
    GOP_TYPE->GOP0 = E_GOP0;
    GOP_TYPE->GOP1 = E_GOP1;
    GOP_TYPE->GOP2 = E_GOP2;
    GOP_TYPE->GOP3 = E_GOP3;
    GOP_TYPE->GOP4 = E_GOP4;
    GOP_TYPE->GOP5 = E_GOP5;
    GOP_TYPE->DWIN = E_GOP_Dwin;
    GOP_TYPE->MIXER = E_GOP_MIXER;
}

GOP_Result HAL_GOP_SetWinFmt(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 regForm, MS_U8 u8GOPNum, MS_U8 u8GwinNum, MS_U16 colortype)
{
    MS_U32 u32BankOffSet = 0;

    _GetBnkOfstByGop(u8GOPNum, &u32BankOffSet);

    if (((regForm & E_GOP_REG_FORM_MASK) == E_GOP_REG_FORM_T21G) || ((regForm & E_GOP_REG_FORM_MASK) == E_GOP_REG_FORM_T81G))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_GWIN0_CTRL(Gop23_GwinCtl_Ofet), colortype, 0x00f0);
    }
    else
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_GWIN0_CTRL(u8GwinNum), colortype, 0x00f0);
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_Set_PINPON(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOPNum, MS_BOOL bEn, E_DRV_GOP_PINPON_MODE pinpon_mode)
{
    MS_U32 u32BankOffSet =0;
    MS_U32 u32BitMask,Regval;

    _GetBnkOfstByGop(u8GOPNum, &u32BankOffSet);

    switch(pinpon_mode)
    {
        default:
        case E_DRV_GOP_PINPON_G3D:
            Regval = bEn << 5;
            u32BitMask = GOP_BIT5;
            break;
        case E_DRV_GOP_PINPON_DWIN:
            Regval = bEn << 6;
            u32BitMask = GOP_BIT6;
            break;
        case E_DRV_GOP_PINPON_DIP:
            Regval = bEn << 7;
            u32BitMask = GOP_BIT7;
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MULTI_ALPHA, GOP_BIT9,(GOP_BIT8|GOP_BIT9));
            break;
        case E_DRV_GOP_PINPON_DWIN0:
            Regval = bEn << 7;
            u32BitMask = GOP_BIT7;
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MULTI_ALPHA, 0x0,(GOP_BIT8|GOP_BIT9));
            break;
        case E_DRV_GOP_PINPON_DWIN1:
            Regval = bEn << 7;
            u32BitMask = GOP_BIT7;
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MULTI_ALPHA, GOP_BIT8,(GOP_BIT8|GOP_BIT9));
            break;
            break;
    }

    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CTRL1, Regval, u32BitMask);
    return GOP_SUCCESS;
}


MS_BOOL _GetBnkOfstByGop(MS_U8 gop, MS_U32 *pBnkOfst)
{
    if(gop >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gop);
        return FALSE;
    }

    if (gop==E_GOP0)
        *pBnkOfst = GOP_4G_OFST<<16;
    else if (gop==E_GOP1)
        *pBnkOfst = GOP_2G_OFST<<16;
    else if (gop==E_GOP2)
        *pBnkOfst = GOP_1G_OFST<<16;

    return TRUE;
}

GOP_Result HAL_GOP_SetGOPACKMask(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U16 u16GopMask)
{
    MS_U16 u16Mask0 = 0;
    MS_U16 u16Mask1 = 0;
    if (u16GopMask&GOP_BIT0)
    {
        u16Mask0 |= GOP_BIT12;
    }
    if (u16GopMask&GOP_BIT1)
    {
        u16Mask0 |= GOP_BIT13;
    }
    if (u16GopMask&GOP_BIT2)
    {
        u16Mask0 |= GOP_BIT14;
    }
    if (u16GopMask&GOP_BIT3)
    {
        u16Mask0 |= GOP_BIT15;
    }
    if (u16GopMask&GOP_BIT4)
    {
        u16Mask1 |= GOP_BIT4;
    }
    if (u16Mask0 != 0)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_BAK_SEL, 0xFFFF , u16Mask0);
    }
    if (u16Mask1 != 0)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_BAK_SEL_EX, 0xFFFF , u16Mask1);
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPACK(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 gop)
{
    if(gop >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gop);
        return GOP_FAIL;
    }

    switch(gop)
    {
        case E_GOP0:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_BAK_SEL, GOP_BIT12 , GOP_BIT12);
            break;
        case E_GOP1:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_BAK_SEL, GOP_BIT13 , GOP_BIT13);
            break;
        case E_GOP2:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_BAK_SEL, GOP_BIT14 , GOP_BIT14);
            break;
        default:
            break;
    }
    return GOP_SUCCESS;
}

MS_U16 HAL_GOP_GetGOPACK(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 gop)
{
    MS_U16 u16GopAck = 0,reg_val =0;

    if(gop >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gop);
        return FALSE;
    }

    switch(gop)
    {
        case E_GOP0:
            HAL_GOP_Read16Reg(pGOPHalLocal,GOP_BAK_SEL,&reg_val);
            if(reg_val&GOP_BIT12)
                u16GopAck = FALSE;
            else
                u16GopAck = TRUE;
            break;
        case E_GOP1:
            HAL_GOP_Read16Reg(pGOPHalLocal,GOP_BAK_SEL,&reg_val);
            if(reg_val&GOP_BIT13)
                u16GopAck = FALSE;
            else
                u16GopAck = TRUE;
            break;
        case E_GOP2:
            HAL_GOP_Read16Reg(pGOPHalLocal,GOP_BAK_SEL,&reg_val);
            if(reg_val&GOP_BIT14)
                u16GopAck = FALSE;
            else
                u16GopAck = TRUE;
            break;
        default:
            break;
    }
    return u16GopAck;
}

GOP_Result HAL_GOP_EnableTwoLineBufferMode(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
    MS_U32 u32BankOffSet;

    if (u8GOP < 2) /* only GOP0/GOP1 can support two linebuufer mode */
    {
        _GetBnkOfstByGop(u8GOP, &u32BankOffSet);
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MULTI_ALPHA, bEnable?GOP_BIT12:0, GOP_BIT12);
    }

    return GOP_SUCCESS;
}

void HAL_GOP_Init(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOPNum)
{
	MS_U8 u8Idx = 0;
    MS_U32 u32bankoff = 0;

    if(u8GOPNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,u8GOPNum);
        return;
    }

    _GetBnkOfstByGop(u8GOPNum, &u32bankoff);
	HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_CTRL0, 0x1, GOP_BIT0);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_CTRL0, GOP_BIT14,(GOP_BIT0|GOP_BIT1|GOP_BIT2|GOP_BIT9|GOP_BIT10|GOP_BIT14));
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_CTRL1, 0x4100, GOP_REG_HW_MASK);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_HSTRCH, 0x1000, GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_VSTRCH, 0x1000, GOP_REG_WORD_MASK);
#if (SCALING_MULITPLIER == RATIO_20BITS)
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_H_V_STRCH_LSB, 0x0, GOP_REG_WORD_MASK);
#endif

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_RATE, 0x0018, GOP_REG_WORD_MASK);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_BW, GOP_FIFO_BURST_ALL, GOP_FIFO_BURST_MASK );
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_BW, GOP_FIFO_THRESHOLD, GOP_REG_LW_MASK );

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_NEW_BW, GOP_BIT15, GOP_BIT15);

    if(g_GopChipPro.bGOPAutoClkGating)
        HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_OLDADDR, (GOP_BIT1 | GOP_BIT0), (GOP_BIT1 | GOP_BIT0) );
    else
        HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_OLDADDR, 0, (GOP_BIT1 | GOP_BIT0) );

    /*set clamping value between 2 pixels, it can avoid some cases garbage happen.*/
    HAL_GOP_Write16Reg(pGOPHalLocal, (u32bankoff+0x20000)+GOP_4G_STRCH_HSZ, 0, GOP_BIT0);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_BW, GOP_DMA_ACCESS_HIGH_PRIORITY, GOP_DMA_ACCESS_PRIORITY_MASK);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_NEW_BW, GOP_BIT14, GOP_BIT14);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_NEW_BW, GOP_BIT12, GOP_BIT13|GOP_BIT12);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_HW_USAGE, 0, GOP_BIT0);

    if((u8GOPNum == E_GOP0)||(u8GOPNum == E_GOP1))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_MULTI_ALPHA, GOP_BIT12, GOP_BIT12);

        HAL_GOP_Write16Reg(pGOPHalLocal, CKG_GOPG0_SCALING, GOP_BIT2|GOP_BIT3 , GOP_REG_WORD_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, CKG_GOPG0_MG, 0, GOP_BIT2);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPSC_SRAM_CTRL, GOP_BIT8|GOP_BIT10|GOP_BIT12, (GOP_BIT8|GOP_BIT9)|(GOP_BIT10|GOP_BIT11)|(GOP_BIT12|GOP_BIT13));
    }

    /* enable GOP clock */
    HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SRAMCLK, 0, CKG_SRAM1_DISABLE_CLK|CKG_SRAM0_DISABLE_CLK);  /* GOP palette SRAM0&1 */
    HAL_GOP_Write16Reg(pGOPHalLocal, GOP_LB_SRAMCLK, 0, BMASK(7:0));                                /* GOP Line buffer sram clock */

    switch(u8GOPNum)
    {
        case E_GOP0:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, 0, CKG_GOPG0_DISABLE_CLK_MASK);  /* GOP 0 */
            break;
        case E_GOP1:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, 0, CKG_GOPG1_DISABLE_CLK_MASK);  /* GOP 1 */
            break;
        case E_GOP2:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, 0, CKG_GOPG2_DISABLE_CLK_MASK);  /* GOP 2 */
            break;
    }


    // set VIP/VOP timing select, always select VOP should be OK.
    HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_MULTI_ALPHA, GOP_BIT10, GOP_BIT10);

    //adjust XC IP timing for GOP dst IP
    HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_IP2GOP_SRCSEL, XC_IP_HVSYNC_DE_GAP, BMASK(4:0));

    if(g_GopChipPro.bOPMuxDoubleBuffer == TRUE)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_OP_MUX_DBF, GOP_BIT15 , GOP_BIT15);
    }

	if (u8GOPNum == 0) {
		for (u8Idx=0; u8Idx<MAX_GOP0_GWIN; u8Idx++) {
			HAL_GOP_SetDram_Addr(pGOPHalLocal, E_GOP0, u8Idx, 0, E_DRV_GOP_DRAMVstr);
			HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_DRAM_HSTR(u8Idx),0, GOP_REG_WORD_MASK);
			HAL_GOP_SetDram_Addr(pGOPHalLocal, E_GOP0, u8Idx, 0, E_DRV_GOP_ScrollHVStopAddr);
		}
	}

	if (g_GopChipPro.bSetHSyncInverse) {
		HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_CTRL0, GOP_BIT2, GOP_BIT2);
	}

	if(g_GopChipPro.bPixelModeSupport) {
		HAL_GOP_Write16Reg(pGOPHalLocal, u32bankoff+GOP_4G_BANK_FWR, GOP_BIT7, GOP_BIT7);
	}
}

void HAL_GOP_Chip_Proprity_Init(GOP_CTX_HAL_LOCAL *pGOPHalLocal)
{
    pGOPHalLocal->pGopChipPro = &g_GopChipPro;
}

MS_U8 HAL_GOP_GetMaxGwinNumByGOP(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GopNum)
{
    switch(u8GopNum)
    {
        case E_GOP0:
        return (MS_U8)MAX_GOP0_GWIN;
            break;
        case E_GOP1:
        return (MS_U8)MAX_GOP1_GWIN;
            break;
        case E_GOP2:
        return (MS_U8)MAX_GOP2_GWIN;
            break;
        default:
        MS_ASSERT(0);
        return 0xFF;
            break;
    }
}

MS_U8 HAL_GOP_SelGwinIdByGOP(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8Gop, MS_U8 u8Idx)
{
    MS_U8 u8GWinId = 0;

    //Adjust GWIN ID by different Chip
    if(u8Gop >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,u8Gop);
        MS_ASSERT(0);
        return 0xFF;
    }

    switch(u8Gop)
    {
        case E_GOP0:
            u8GWinId = GOP0_GwinIdBase + u8Idx;
            break;
        case E_GOP1:
            u8GWinId = GOP1_GwinIdBase + u8Idx;
            break;
        case E_GOP2:
            u8GWinId = GOP2_GwinIdBase + u8Idx;
            break;
        default:
            break;
    }
    return u8GWinId;

}

GOP_Result HAL_GOP_GOPSel(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOPNum)
{
    if(u8GOPNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,u8GOPNum);
        MS_ASSERT(0);
        return GOP_FAIL;
    }

    switch(u8GOPNum)
    {
    case E_GOP0: // GOP4G
            pGOPHalLocal->bank_offset = GOP_4G_OFST<<16;
            return GOP_SUCCESS;
    case E_GOP1: // GOP2G
            pGOPHalLocal->bank_offset = GOP_2G_OFST<<16;
            return GOP_SUCCESS;
    case E_GOP2: // GOP1G
            pGOPHalLocal->bank_offset = GOP_1G_OFST<<16;
            return GOP_SUCCESS;
    default:
        MS_ASSERT(0);
        return GOP_FAIL;
    }
}

void HAL_GOP_BANK_SEL(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8bank)
{
    MS_U16 u16Bank;
    u16Bank = GOP_READ2BYTE(GOP_BAK_SEL);
    u16Bank &= ~BMASK(3:0);
    u16Bank |= (u8bank&BMASK(3:0));
    GOP_WRITE2BYTE(GOP_BAK_SEL, u16Bank);
    u16Bank = GOP_READ2BYTE(GOP_BAK_SEL_EX);
    u16Bank &= ~GOP_BIT4;
    u16Bank |= ((u8bank&GOP_BIT4)<<GOP_BIT4);
    GOP_WRITE2BYTE(GOP_BAK_SEL_EX, u16Bank);
}

MS_U8 HAL_GOP_Get_BANK(GOP_CTX_HAL_LOCAL *pGOPHalLocal)
{
    MS_U16 u16GetBank;
    u16GetBank = GOP_READ2BYTE(GOP_BAK_SEL)&0xF;
    return u16GetBank;
}

void HAL_GOP_Read16Reg(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32Addr, MS_U16* pu16Ret)
{
    MS_U16 u16XcSubbank=0, u16BankAddr=0, u16BankTemp=0;
    MS_U32 u32Bank = 0;
    MS_U32 u32DirectAddr = 0;
    MS_U16 u16FssSource = 0;
    MS_U16 u16FssBankOffset = 0;

    HAL_GOP_DEBUGINFO(printf("HAL_GOP_Read16Reg[%x]\n", u32Addr));

    //* Gop driver should access another HW IP register
    //* ex: SC's IP and OP setting, GE's det frame buffer setting, ChipTop GOP clk setting
    switch (u32Addr & 0xFF00)
    {
        case GOP_REG_BASE:
        {
            u32Bank = (u32Addr & 0xFF0000) >> 8;

            if(u32Bank==0xE00)//GOP4:  0x121B00
            {
                u32Bank=GOP_REG_GOP4_BK_OFFSET;
            }
            else if(u32Bank==0xF00)//GWIN4: 0x121E00
            {
                u32Bank=GOP_REG_GOP4_GW_OFFSET;
            }
            else if(u32Bank==0x1000)//GOP4_ST
            {
                u32Bank=GOP_REG_GOP4_ST_OFFSET;
            }

            u32DirectAddr = GOP_REG_DIRECT_BASE + u32Bank + (u32Addr & 0xFF);  //Direct_Base + bank + addr_offset
            *pu16Ret = GOP_READ2BYTE((u32DirectAddr&0xFFFFF));
            break;
        }
        case SC1_REG_BASE:
        {
            if(g_GopChipPro.bXCDirrectBankSupport)
            {
                u16XcSubbank =  (u32Addr & 0xFF0000)>>8;
                if((GOP_READ2BYTE(SC_REG_IPMUX_73_L) & BIT(0)) && ((u16XcSubbank>>8) == SC_IP_SUBBANK))  // check sc fast switch enable & SubBank = 0x1
                {
                    u16FssSource = (GOP_READ2BYTE(SC_REG_IPMUX_71_L) & BMASK(3:0));
                    u16FssBankOffset = (((FASTSOURCESWITCH_BANK_OFFSET + u16FssSource)<<8) & BMASK(15:8));
                    u32Addr = SC1_DIRREG_BASE+ u16FssBankOffset + (u32Addr & 0xFF);
                }
                else
                {
                    u32Addr = SC1_DIRREG_BASE + u16XcSubbank + (u32Addr & 0xFF);
                }
                *pu16Ret = GOP_READ2BYTE((u32Addr&0xFFFFF));
            }
            else
            {
                u16XcSubbank =  (u32Addr & 0xFF0000)>>16;
                u16BankAddr = GOP_SC_BANKSEL+0;
                u32Addr = SC1_REG_BASE + (u32Addr & 0xFF);

                u16BankTemp = GOP_READ2BYTE(u16BankAddr&0xFFFF);
                if((GOP_READ2BYTE(SC_REG_IPMUX_73_L) & BIT(0)) && (u16XcSubbank == SC_IP_SUBBANK))
                {
                    u16FssSource = (GOP_READ2BYTE(SC_REG_IPMUX_71_L) & BMASK(3:0));
                    u16FssBankOffset = (((FASTSOURCESWITCH_BANK_OFFSET + u16FssSource)) & BMASK(7:0));
                    GOP_WRITE2BYTE(u16BankAddr&0xFFFF, u16FssBankOffset);
                }
                else
                {
                    GOP_WRITE2BYTE(u16BankAddr&0xFFFF, u16XcSubbank);
                }
                *pu16Ret = GOP_READ2BYTE((u32Addr&0xFFFF));
                GOP_WRITE2BYTE(u16BankAddr&0xFFFF, u16BankTemp);
            }
            break;
        }
        case GE_REG_BASE:
        case CKG_REG_BASE:
        case MIU0_REG_BASE:
        case MIU_REG_BASE:
        case MVOP_REG_BASE:
        {
            *pu16Ret = GOP_READ2BYTE((u32Addr&0xFFFF));
            break;
        }
#ifdef GOP_MIU_GROUP2
        case (MIU2_REG_BASE & 0xFF00):
        {
            u32DirectAddr = MIU2_REG_BASE + (u32Addr & 0xFF);  //Direct_Base + addr_offset
            *pu16Ret = GOP_READ2BYTE((u32DirectAddr&0xFFFFF));

            break;
        }
#endif
        default:
        {
            //Gop lib current do not support this HW ip base
            MS_ASSERT(0);
            *pu16Ret =0;
            break;
        }
    }
}

void HAL_GOP_Write16Reg(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32Addr, MS_U16 u16Val, MS_U16 u16Mask)
{
    MS_U16 u16tmp=0;
    MS_U16 u16XcSubbank=0,u16BankAddr=0,u16Ret=0;
    MS_U32 u32Bank = 0;
    MS_U32 u32DirectAddr = 0;
    MS_U16 u16FssSource = 0;
    MS_U16 u16FssBankOffset = 0;

    HAL_GOP_DEBUGINFO(printf("HAL_GOP_Write16Reg[%04x] = %04x\n", u32Addr, u16Val));

    if(u16Mask!=0xffff)
    {
        HAL_GOP_Read16Reg(pGOPHalLocal, u32Addr, &u16tmp);
        u16tmp &= ~u16Mask;
        u16Val &=  u16Mask;
        u16Val |=  u16tmp;
    }

    //* Gop driver should access another HW IP register
    //* ex: SC's IP and OP setting, GE's det frame buffer setting, ChipTop GOP clk setting
    switch (u32Addr & 0xFF00)
    {
        case GOP_REG_BASE:
        {
            u32Bank = (u32Addr & 0xFF0000) >> 8;

            if(u32Bank==0xE00)//GOP4:  0x121B00
            {
                u32Bank=GOP_REG_GOP4_BK_OFFSET;
            }
            else if(u32Bank==0xF00)//GWIN4: 0x121E00
            {
                u32Bank=GOP_REG_GOP4_GW_OFFSET;
            }
            else if(u32Bank==0x1000) //GOP4_ST
            {
                u32Bank=GOP_REG_GOP4_ST_OFFSET;
            }

            u32DirectAddr = GOP_REG_DIRECT_BASE + u32Bank + (u32Addr & 0xFF);
            GOP_WRITE2BYTE((u32DirectAddr&0xFFFFF), u16Val);
            break;
        }
        case SC1_REG_BASE:
            if(g_GopChipPro.bXCDirrectBankSupport)
            {   /*Derrick Bank*/
                u16XcSubbank =  (u32Addr & 0xFF0000)>>8 ;
                if((GOP_READ2BYTE(SC_REG_IPMUX_73_L) & BIT(0)) && ((u16XcSubbank>>8) == SC_IP_SUBBANK))  // check sc fast switch enable & SubBank = 0x1
                {
                    u16FssSource = (GOP_READ2BYTE(SC_REG_IPMUX_71_L) & BMASK(3:0));
                    u16FssBankOffset = (((FASTSOURCESWITCH_BANK_OFFSET + u16FssSource)<<8) & BMASK(15:8));
                    u32DirectAddr = SC1_DIRREG_BASE+ u16FssBankOffset + (u32Addr & 0xFF);
                }
                else
                {
                    u32DirectAddr = SC1_DIRREG_BASE + u16XcSubbank+ (u32Addr & 0xFF);
                }
                GOP_WRITE2BYTE((u32DirectAddr&0xFFFFF), u16Val);
            }
            else
            {   /*Sub Bank*/
                u16XcSubbank =  (u32Addr & 0xFF0000)>>16 ;
                u16BankAddr = GOP_SC_BANKSEL+0;
                u32Addr = SC1_REG_BASE + (u32Addr & 0xFF);

                u16Ret = GOP_READ2BYTE(u16BankAddr&0xFFFF);
                if((GOP_READ2BYTE(SC_REG_IPMUX_73_L) & BIT(0)) && (u16XcSubbank == SC_IP_SUBBANK))
                {
                    u16FssSource = (GOP_READ2BYTE(SC_REG_IPMUX_71_L) & BMASK(3:0));
                    u16FssBankOffset = (((FASTSOURCESWITCH_BANK_OFFSET + u16FssSource)) & BMASK(7:0));
                    GOP_WRITE2BYTE((u16BankAddr&0xFFFF), u16FssBankOffset);
                }
                else
                {
                    GOP_WRITE2BYTE((u16BankAddr&0xFFFF), u16XcSubbank);
                }
                GOP_WRITE2BYTE((u32Addr&0xFFFF), u16Val);
                GOP_WRITE2BYTE((u16BankAddr&0xFFFF), u16Ret);
            }
            break;
        case GE_REG_BASE:
        case CKG_REG_BASE:
        case MIU0_REG_BASE:
        case MIU_REG_BASE:
        {
            GOP_WRITE2BYTE((u32Addr&0xFFFF), u16Val);
            break;
        }
#ifdef GOP_MIU_GROUP2
        case (MIU2_REG_BASE & 0xFF00):
        {
            u32DirectAddr = MIU2_REG_BASE + (u32Addr & 0xFF);  //Direct_Base + addr_offset
            GOP_WRITE2BYTE((u32DirectAddr&0xFFFFF), u16Val);
            break;
        }
#endif
        default:
        {
            //Gop lib current do not support this HW ip base
            MS_ASSERT(0);
            break;
        }

    }
}


void HAL_GOP_Write32Reg(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32addr, MS_U32 u32val)
{
    MS_U32 bank;
    MS_U32 direct_addr;

    HAL_GOP_DEBUGINFO(printf("HAL_GOP_Write32Reg[%bx] = %lx\n", u32addr, u32val));

    //* Gop driver should access another HW IP register
    //* ex: SC's IP and OP setting, GE's det frame buffer setting, ChipTop GOP clk setting
    switch (u32addr & 0xFF00)
    {
        case GOP_REG_BASE:
        {
            bank = (u32addr & 0xFF0000) >> 8;
            if(bank==0xE00)//GOP4:  0x121B00
            {
                    bank=GOP_REG_GOP4_BK_OFFSET;
            }
            else if(bank==0xF00)//GWIN4: 0x121E00
            {
                    bank=GOP_REG_GOP4_GW_OFFSET;
            }
            else if(bank==0x1000) //GOP4_ST
            {
                    bank=GOP_REG_GOP4_ST_OFFSET;
            }

            direct_addr = GOP_REG_DIRECT_BASE + bank + (u32addr & 0xFF);
            GOP_WRITE2BYTE((direct_addr&0xFFFFF), (u32val&0xFFFF));
            GOP_WRITE2BYTE((direct_addr&0xFFFFF)+2, (u32val&0xFFFF0000)>>16);
            break;
        }

        case GE_REG_BASE:
        case SC1_REG_BASE:
        case CKG_REG_BASE:
        {
            GOP_WRITE2BYTE((u32addr&0xFFFF), (u32val&0xFFFF));
            GOP_WRITE2BYTE((u32addr&0xFFFF)+2, (u32val&0xFFFF0000)>>16);
            break;
        }

        default:
        {
            //Gop lib current do not support this HW ip base
            MS_ASSERT(0);
            break;
        }

    }
}

MS_U16 HAL_GOP_GetBPP(GOP_CTX_HAL_LOCAL *pGOPHalLocal, DRV_GOPColorType fbFmt)
{
    MS_U16 bpp=0;

    switch ( fbFmt )
    {
    case E_DRV_GOP_COLOR_RGB555_BLINK :
    case E_DRV_GOP_COLOR_RGB565 :
    case E_DRV_GOP_COLOR_ARGB1555:
    case E_DRV_GOP_COLOR_RGBA5551:
    case E_DRV_GOP_COLOR_ARGB4444 :
    case E_DRV_GOP_COLOR_RGBA4444 :
    case E_DRV_GOP_COLOR_RGB555YUV422:
    case E_DRV_GOP_COLOR_YUV422:
    case E_DRV_GOP_COLOR_2266:
        bpp = 16;
        break;
    case E_DRV_GOP_COLOR_ARGB8888 :
    case E_DRV_GOP_COLOR_ABGR8888 :
        bpp = 32;
        break;

    case E_DRV_GOP_COLOR_I8 :
        bpp = 8;
        break;

    default :
        //print err
        MS_ASSERT(0);
        bpp = 0xFFFF;
        break;
    }
    return bpp;

}

void HAL_GOP_GWIN_SetBlending(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U8 u8win, MS_BOOL bEnable, MS_U8 u8coef)
{
    MS_U16 regval;
    /*For compatibility
        Old chip only have alpha coeffient 6 bits. Charka2 define UI alpha value base on it.*/
    MS_U8 u8coef_cpt = u8coef;

    //if UI alpha value based on alpha coeffient 6 bits, and our chip is 8bit, please do it.
    if( pGOPHalLocal->User_ConsAlpha_bits != g_GopChipPro.Default_ConsAlpha_bits)
    {
        switch(u8coef)
        {
            case 0x0 :
                u8coef_cpt = u8coef<<2;
                break;
            case 0x3f :
                u8coef_cpt = ((u8coef<<2)|0x3);
                break;
            default:
                u8coef_cpt = ((u8coef<<2)|0x1);
                break;
        }
    }

	/*alpha coeffient 6/8bit chip has GOP0,GOP1*/
    if (u8win<(MAX_GOP0_GWIN+MAX_GOP1_GWIN))
    {
        regval = (MS_U16)(bEnable?(1<<14):0)|(MS_U16)((u8coef&0x3F)<<8);
        HAL_GOP_Write16Reg(pGOPHalLocal, (u8win < MAX_GOP0_GWIN)? GOP_4G_GWIN0_CTRL(u8win):GOP_2G_GWIN_CTRL(u8win-MAX_GOP0_GWIN), regval, 0x4000);
        HAL_GOP_Write16Reg(pGOPHalLocal, (u8win < MAX_GOP0_GWIN)? GOP_4G_GWIN_ALPHA01(u8win):GOP_2G_GWIN_ALPHA01(u8win-MAX_GOP0_GWIN), u8coef_cpt, 0xFF);
    }
    /*Only alpha coeffient 8bit chip has GOP2,GOP3...*/
    else
    {
        if (u8win==(MAX_GOP0_GWIN+MAX_GOP1_GWIN))
        {
            regval = (MS_U16)(bEnable?(1<<14):0);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1G_GWIN0_CTRL, regval, 0x4000);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1G_GWIN_ALPHA01, u8coef, 0xFF);
        }
        else if (u8win==(MAX_GOP0_GWIN+MAX_GOP1_GWIN+MAX_GOP2_GWIN))
        {
            regval = (MS_U16)(bEnable?(1<<14):0);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GX_GWIN0_CTRL, regval, 0x4000);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GX_GWIN_ALPHA01, u8coef, 0xFF);
        }
        else
        {
            regval = (MS_U16)(bEnable?(1<<14):0);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GS0_GWIN0_CTRL, regval, 0x4000);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GS0_GWIN_ALPHA01, u8coef, 0xFF);
        }
    }
}

void HAL_GOP_SetIOMapBase(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_VIRT addr)
{
    pGOPHalLocal->va_mmio_base = GOP_MMIO_NONPM_BASE;
}
void HAL_GOP_SetIOFRCMapBase(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_VIRT addr)
{
}
void HAL_GOP_SetIOPMMapBase(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_VIRT addr)
{
}


GOP_Result HAL_GOP_GWIN_SetDstPlane(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 GopNum, DRV_GOPDstType eDstType,MS_BOOL bOnlyCheck)
{
    /* GOP dst type:
        1: IP (Main)
        0: IP (Sub)
        2: OP
    */
    MS_U16 u16RegVal;
    MS_U32 u32BankOffSet;
    MS_BOOL bEnable=FALSE;
    MS_U16 u16HSyncMask=GOP_BIT14;

    if( GopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,GopNum);
        return GOP_INVALID_PARAMETERS;
    }

    switch (eDstType)
    {
        case E_DRV_GOP_DST_IP0:
            u16RegVal = 0x0;
            bEnable = FALSE;
            u16HSyncMask=0;
            break;

        case E_DRV_GOP_DST_IP0_SUB:
            u16RegVal = 0x1;
            bEnable = TRUE;
            u16HSyncMask=0;
            break;

        case E_DRV_GOP_DST_OP0:
            u16RegVal = 0x2;
            bEnable = FALSE;
            u16HSyncMask=GOP_BIT14;
            break;

        case E_DRV_GOP_DST_VOP:
            u16RegVal = 0x3;
            bEnable = FALSE;
            u16HSyncMask=GOP_BIT14;
            break;

        case E_DRV_GOP_DST_VOP_SUB:
            u16RegVal = 0x4;
            bEnable = FALSE;
            u16HSyncMask=GOP_BIT14;
            break;
        case E_DRV_GOP_DST_FRC:
            u16RegVal = 0x6;
            bEnable = TRUE;
            u16HSyncMask=GOP_BIT14;
            break;

        default:
            return GOP_FUN_NOT_SUPPORTED;
    }

    if(bOnlyCheck == FALSE)
    {
        _GetBnkOfstByGop(GopNum, &u32BankOffSet);
        HAL_GOP_SetGOP2Pto1P(pGOPHalLocal, GopNum, bEnable);
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CTRL1, u16RegVal, BMASK(GOP_BIT3:0));

        //for Monaco/Muji/Monet, IP need set Vsync inverse in IP
        if (eDstType == E_DRV_GOP_DST_IP0)
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CTRL0, (MS_U16)GOP_BIT1, GOP_BIT1);
        else
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CTRL0, (MS_U16)~GOP_BIT1, GOP_BIT1);

        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CTRL0, u16HSyncMask, GOP_BIT14);               // Set mask Hsync when VFDE is low
    }

    return GOP_SUCCESS;
}

void HAL_GOP_GWIN_GetMUX(GOP_CTX_HAL_LOCAL*pGOPHalLocal, MS_U8* u8GOPNum, Gop_MuxSel eGopMux)
{
    MS_U16 u16GopMux=0;
    switch(eGopMux)
    {
        case E_GOP_MUX0:
        case E_GOP_MUX1:
        case E_GOP_MUX2:
        case E_GOP_MUX3:
        case E_GOP_MUX4:
        HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &u16GopMux);
            *u8GOPNum = (u16GopMux >> (eGopMux*GOP_MUX_SHIFT))& GOP_REGMUX_MASK;
            break;
        case E_GOP_IP0_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_IPVOP, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_IP_MAIN_MUX_MASK)>>GOP_IP_MAIN_MUX_SHIFT;
            break;
        case E_GOP_IP0_SUB_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_IPVOP, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_IP_SUB_MUX_MASK)>>GOP_IP_SUB_MUX_SHIFT;
            break;
        case E_GOP_VOP0_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_IPVOP, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_IP_VOP0_MUX_MASK)>>GOP_IP_VOP0_MUX_SHIFT;
            break;
        case E_GOP_VOP0_SUB_MUX:
        HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_IPVOP, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_IP_VOP1_MUX_MASK)>>GOP_IP_VOP1_MUX_SHIFT;
            break;
        case E_GOP_Mix_MUX0:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_Mix_MUX0_MASK)>>GOP_Mix_MUX0_SHIFT;
            break;
        case E_GOP_Mix_MUX1:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_Mix_MUX1_MASK)>>GOP_Mix_MUX1_SHIFT;
            break;
        case E_GOP_VE0_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_VE0_MUX_MASK)>>GOP_VE0_MUX_SHIFT;
            break;
        case E_GOP_VE1_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_VE1_MUX_MASK)>>GOP_VE1_MUX_SHIFT;
            break;
        case E_GOP_DIP_MUX:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_4K2K, &u16GopMux);
            *u8GOPNum = (u16GopMux&GOP_DIP_MUX_MASK)>>GOP_DIP_MUX_SHIFT;
            break;
        case E_GOP_FRC_MUX0:
        case E_GOP_FRC_MUX1:
        case E_GOP_FRC_MUX2:
        case E_GOP_FRC_MUX3:
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_4K2K, &u16GopMux);
            *u8GOPNum = (u16GopMux >> ((eGopMux%E_GOP_FRC_MUX0)*GOP_FRC_MUX_SHIFT))& GOP_FRC_REGMUX_MASK;
            break;
        case E_GOP_BYPASS_MUX0:
            break;
        default:
            GOP_H_ERR("[%s]ERROR, not support the mux[%d]\n",__FUNCTION__,eGopMux);
            break;
        }
}

void HAL_GOP_GWIN_SetMUX(GOP_CTX_HAL_LOCAL*pGOPHalLocal, MS_U8 u8GOPNum, Gop_MuxSel eGopMux)
{
    MS_U16 u16Ret[4]={0};

    switch(eGopMux)
    {
        case E_GOP_MUX0:
        case E_GOP_MUX1:
        case E_GOP_MUX2:
        case E_GOP_MUX3:
        case E_GOP_MUX4:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX, u8GOPNum <<(GOP_MUX_SHIFT*eGopMux), GOP_REGMUX_MASK<<(GOP_MUX_SHIFT*eGopMux));
          break;
        case E_GOP_IP0_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_IPVOP, u8GOPNum << GOP_IP_MAIN_MUX_SHIFT, GOP_IP_MAIN_MUX_MASK);
            break;
        case E_GOP_IP1_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_IPVOP, u8GOPNum << GOP_IP_MAIN_MUX_SHIFT, GOP_IP_MAIN_MUX_MASK);
            break;
        case E_GOP_VOP0_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_IPVOP, u8GOPNum << GOP_IP_VOP0_MUX_SHIFT, GOP_IP_VOP0_MUX_MASK);
            break;
        case E_GOP_VOP1_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_IPVOP, u8GOPNum << GOP_IP_VOP1_MUX_SHIFT, GOP_IP_VOP1_MUX_MASK);
            break;
        case E_GOP_Mix_MUX0:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, u8GOPNum << GOP_Mix_MUX0_SHIFT, GOP_Mix_MUX0_MASK);
            break;
        case E_GOP_Mix_MUX1:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, u8GOPNum << GOP_Mix_MUX1_SHIFT, GOP_Mix_MUX1_MASK);
            break;
        case E_GOP_VE0_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, u8GOPNum << GOP_VE0_MUX_SHIFT, GOP_VE0_MUX_MASK);
            break;
        case E_GOP_VE1_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX4_MIX_VE, u8GOPNum << GOP_VE1_MUX_SHIFT, GOP_VE1_MUX_MASK);
            break;
        case E_GOP_DIP_MUX:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum << GOP_DIP_MUX_SHIFT, GOP_DIP_MUX_MASK);
            break;
        case E_GOP_FRC_MUX0:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)));
            break;
        case E_GOP_FRC_MUX1:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)));
            break;
        case E_GOP_FRC_MUX2:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)));
            break;
        case E_GOP_FRC_MUX3:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(eGopMux%E_GOP_FRC_MUX0)));
            break;
        case E_GOP_BYPASS_MUX0:
                    //Check MUX is using? Checking from priority high to low
                    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER2_L_EN, &u16Ret[0]);
                    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER2_R_EN, &u16Ret[1]);
                    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER1_L_EN, &u16Ret[2]);
                    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER1_R_EN, &u16Ret[3]);

                    //Checking if un-use
                    if((u16Ret[0] & GOP_BIT0) ==0)
                    {
                        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX2%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX2%E_GOP_FRC_MUX0)));
                    }
                    else if((u16Ret[1] & GOP_BIT0) ==0)
                    {
                        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX3%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX3%E_GOP_FRC_MUX0)));
                    }
                    else if((u16Ret[2] & GOP_BIT0) ==0)
                    {
                        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX0%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX0%E_GOP_FRC_MUX0)));
                    }
                    else if((u16Ret[3] & GOP_BIT0) ==0)
                    {
                        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, u8GOPNum <<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX1%E_GOP_FRC_MUX0)) , GOP_REGMUX_MASK<<(GOP_FRC_MUX_SHIFT*(E_GOP_FRC_MUX1%E_GOP_FRC_MUX0)));
                    }
                    else
                    {
                        GOP_H_ERR("[%s][%d]FRC mux is already full.\n",__FUNCTION__,__LINE__);
                    }
                    break;
             break;
        default:
            GOP_H_ERR("[%s]ERROR mux setting\n",__FUNCTION__);
            break;
    }
}

GOP_Result HAL_GOP_SetGOPEnable2SC(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL bEnable)
{
    /* GOP OP Path enable to SC Setting
        A5: GOP OP Path blending with SC sequence
        mux1-->mux0-->mux2-->mux3
    */
    MS_U16 muxValue=0, regval=0;

    DRV_GOPDstType pGopDst = E_DRV_GOP_DST_INVALID;
    MS_U16 u164K2KMuxValue=0;

    if(gopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gopNum);
        return GOP_FAIL;
    }

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &muxValue);
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPEN, &regval);

    if (gopNum== (muxValue & GOP_MUX0_MASK))    //enable mux0 to SC
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, bEnable?(regval |GOP_BIT15):(regval & ~GOP_BIT15), GOP_BIT15);
    }

    if (gopNum== ((muxValue & GOP_MUX1_MASK)>>(GOP_MUX_SHIFT*1))) //enable mux1
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, bEnable?(regval | GOP_BIT14):(regval & ~GOP_BIT14), GOP_BIT14);
    }

    if (gopNum== ((muxValue & GOP_MUX2_MASK)>>(GOP_MUX_SHIFT*2))) //enable mux2
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, bEnable?(regval |GOP_BIT13):(regval & ~GOP_BIT13), GOP_BIT13);
    }

    if (gopNum== ((muxValue & GOP_MUX3_MASK)>>(GOP_MUX_SHIFT*3))) //enable mux3
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, bEnable?(regval |GOP_BIT12):(regval & ~GOP_BIT12), GOP_BIT12);
    }

    if (gopNum== ((muxValue & GOP_MUX4_MASK)>> (GOP_MUX_SHIFT*4))) //enable mux4
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, bEnable?(regval |GOP_BIT11):(regval & ~GOP_BIT11), GOP_BIT11);
    }

    /*For FRC mux switch*/
    HAL_GOP_GetGOPDst(pGOPHalLocal, gopNum, &pGopDst);
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_4K2K, &u164K2KMuxValue);
    if (gopNum== (u164K2KMuxValue & GOP_MUX0_MASK))
    {
        /* Switch FRC blending */
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER1_L_EN, (pGopDst==E_DRV_GOP_DST_BYPASS)?GOP_BIT0: ~GOP_BIT0, GOP_BIT0);
    }

    if (gopNum== ((u164K2KMuxValue & GOP_MUX1_MASK)>>(GOP_MUX_SHIFT*1)))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER1_R_EN, (pGopDst==E_DRV_GOP_DST_BYPASS)?GOP_BIT0: ~GOP_BIT0, GOP_BIT0);
    }

    if (gopNum== ((u164K2KMuxValue & GOP_MUX2_MASK)>>(GOP_MUX_SHIFT*2)))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER2_L_EN, (pGopDst==E_DRV_GOP_DST_BYPASS)?GOP_BIT0: ~GOP_BIT0, GOP_BIT0);
    }

    if (gopNum== ((u164K2KMuxValue & GOP_MUX3_MASK)>>(GOP_MUX_SHIFT*3)))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_FRC_LAYER2_R_EN, (pGopDst==E_DRV_GOP_DST_BYPASS)?GOP_BIT0: ~GOP_BIT0, GOP_BIT0);
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOP2Pto1P(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL bEnable)
{
    MS_U32 u32pBankOffSet=0;
    MS_U16 u16Regval;

    if(g_GopChipPro.b2Pto1PSupport)
    {
    _GetBnkOfstByGop(gopNum, &u32pBankOffSet);
        HAL_GOP_Read16Reg(pGOPHalLocal, u32pBankOffSet + GOP_4G_SRAM_BORROW, &u16Regval);
        HAL_GOP_Write16Reg(pGOPHalLocal, u32pBankOffSet + GOP_4G_SRAM_BORROW, bEnable?(u16Regval |0x0800):(u16Regval & ~0x0800), 0x0800);
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPEnable2Mode1(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL bEnable)
{
    /* GOP OP Path enable to SC Setting
        A5: GOP OP Path blending with SC sequence
        mux1-->mux0-->mux2-->mux3
    */
    MS_U16 muxValue=0, regval=0;

    if(gopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gopNum );
        return GOP_FAIL;
    }

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &muxValue);
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, &regval);
    if (gopNum== (muxValue & GOP_MUX0_MASK))    //enable mux0 to SC
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, bEnable?(regval |0x10):(regval & ~0x10), 0x30);
    }
    else if (gopNum== ((muxValue & GOP_MUX1_MASK)>>(GOP_MUX_SHIFT*1))) //enable mux1
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, bEnable?(regval |0x40):(regval & ~0x40), 0xC0);
    }
    else if (gopNum== ((muxValue & GOP_MUX2_MASK)>>(GOP_MUX_SHIFT*2))) //enable mux2
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, bEnable?(regval |0x100):(regval & ~0x100), 0x300);
    }
    else if (gopNum== ((muxValue & GOP_MUX3_MASK)>>(GOP_MUX_SHIFT*3))) //enable mux3
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, bEnable?(regval |0x1000):(regval & ~0x1000), 0x3000);
    }
    else if (gopNum== ((muxValue & GOP_MUX4_MASK)>> (GOP_MUX_SHIFT*4))) //enable mux4
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, bEnable?(regval |0x4000):(regval & ~0x4000), 0xc000);
    }
    else
    {
        return GOP_FAIL;
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetGOPAlphaMode1(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL *pbEnable)
{
    MS_U16 muxValue=0, regval=0;

    if(gopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gopNum );
        return GOP_FAIL;
    }

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &muxValue);
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, &regval);
    if (gopNum== (muxValue & GOP_MUX0_MASK))    //enable mux0 to SC
    {
        *pbEnable = (regval & GOP_BIT4) == GOP_BIT4;
    }
    else if (gopNum== ((muxValue & GOP_MUX1_MASK)>>(GOP_MUX_SHIFT*1))) //enable mux1
    {
        *pbEnable = (regval & GOP_BIT6) == GOP_BIT6;
    }
    else if (gopNum== ((muxValue & GOP_MUX2_MASK)>>(GOP_MUX_SHIFT*2))) //enable mux2
    {
        *pbEnable = (regval & GOP_BIT8) == GOP_BIT8;
    }
    else if (gopNum== ((muxValue & GOP_MUX3_MASK)>>(GOP_MUX_SHIFT*3))) //enable mux3
    {
        *pbEnable = (regval & GOP_BIT12) == GOP_BIT12;
    }
    else if (gopNum== ((muxValue & GOP_MUX4_MASK)>> (GOP_MUX_SHIFT*4))) //enable mux4
    {
        *pbEnable = (regval & GOP_BIT14) == GOP_BIT14;
    }
    else
    {
        return GOP_FAIL;
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPHighPri(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum)
{
    /*T8
    OP path: support 4 mux (mux0, mux1, mux 2, mux3) to blend with SC simultaneously
    IP  path: support mux0 and mux1 to IPMain/IPSub. Only one mux of mux0 and mux1 can be blended to IPMain/IPSub
    */
    MS_U16 Mux3Gop, muxValue=0, i;
    MS_U16 MuxShift;

    MuxShift = GOP_MUX_SHIFT * E_GOP_MUX3;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &muxValue);
    for (i=0; i<MAX_GOP_MUX;i++) //T8 4 mux
    {
        if (gopNum== ((muxValue&(GOP_REGMUX_MASK<<(i*GOP_MUX_SHIFT)))>>(i*GOP_MUX_SHIFT)))
        {
            Mux3Gop = (muxValue&GOP_MUX3_MASK)>> MuxShift; //save mux2 gop

            muxValue &= ~GOP_MUX3_MASK; //clear mux2 setting
            muxValue &= ~(GOP_REGMUX_MASK<<(i*GOP_MUX_SHIFT));  //clear current mux setting
            muxValue |= ((gopNum<< MuxShift)|(Mux3Gop<<(i*GOP_MUX_SHIFT)));
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX, muxValue, GOP_REG_WORD_MASK);
            break;

        }

    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPClk(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, DRV_GOPDstType eDstType)
{
    /*T8 GOP dst type:
    0: IP (Main)
    1: IP (Sub)
    2: OP
    3: MVOP
    */

    /* Monaco clkgen setting */
    /******************
    0: odclk
    1: idclk_f2  (ipm)
    2: idclk_f1 (ips)
    3: ocmixer
    4: ve
    5 : mvop main
    6 : mvop sub
    ******************/
    if(gopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,gopNum);
        MS_ASSERT(0);
        return GOP_INVALID_PARAMETERS;
    }

    switch(eDstType)
    {
        case E_DRV_GOP_DST_IP0:
            if (gopNum==E_GOP1)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_IDCLK2, CKG_GOPG1_MASK);
            }
            else if (gopNum ==E_GOP2)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPG2_IDCLK2, CKG_GOPG2_MASK);
            }
            else if (gopNum==E_GOP0)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_IDCLK2, CKG_GOPG0_MASK);
            }
            else
            {
                MS_ASSERT(0);
                return GOP_INVALID_PARAMETERS;
            }
            break;

        case E_DRV_GOP_DST_IP0_SUB:
            if (gopNum==E_GOP1)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_IDCLK1, CKG_GOPG1_MASK);
            }
            else if (gopNum==E_GOP2)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPG2_IDCLK1, CKG_GOPG2_MASK);
            }
            else if (gopNum==E_GOP0)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_IDCLK1, CKG_GOPG0_MASK);
            }
            else
            {
                MS_ASSERT(0);
                return GOP_INVALID_PARAMETERS;
            }
            break;

        case E_DRV_GOP_DST_OP0:
            if (gopNum==E_GOP1)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_ODCLK, CKG_GOPG1_MASK);
            }
            else if (gopNum==E_GOP2)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPG2_ODCLK, CKG_GOPG2_MASK);
            }
            else if (gopNum==E_GOP0)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_ODCLK, CKG_GOPG0_MASK);
            }
            else
            {
                MS_ASSERT(0);
                return GOP_INVALID_PARAMETERS;
            }
            break;

        case E_DRV_GOP_DST_VOP:
            if (gopNum==E_GOP1)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_IDCLK2, CKG_GOPG1_MASK);
            }
            else if (gopNum ==E_GOP2)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG2_IDCLK2, CKG_GOPG2_MASK);
            }
            else if (gopNum==E_GOP0)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_IDCLK2, CKG_GOPG0_MASK);
            }
            else
            {
                MS_ASSERT(0);
                return GOP_INVALID_PARAMETERS;
            }
            break;
        case E_DRV_GOP_DST_FRC:
            if (gopNum==E_GOP0)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_OCC_FRCCLK, CKG_GOPG0_MASK);
            }
            else if (gopNum==E_GOP1)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_OCC_FRCCLK, CKG_GOPG1_MASK);
            }
            else if (gopNum==E_GOP2)
            {
                HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPG2_OCC_FRCCLK, CKG_GOPG2_MASK);
            }
            else
            {
                 MS_ASSERT(0);
                 return GOP_INVALID_PARAMETERS;
            }
            break;

        default:
            MS_ASSERT(0);
            return GOP_ENUM_NOT_SUPPORTED;
    }
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOPCLK, &pGOPHalLocal->u16Clk0Setting); //Backup current GOPG clock settings
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SRAMCLK,&pGOPHalLocal->u16Clk2Setting); //Backup current SRAM clock settings

    return GOP_SUCCESS;
}


GOP_Result HAL_GOP_SetClkForCapture(GOP_CTX_HAL_LOCAL *pGOPHalLocal, DRV_GOP_DWIN_SRC_SEL enSrcSel)
{
    HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, 0, GOP_BIT8);
    if (enSrcSel==GOP_DRV_DWIN_SRC_OP)
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPD_CLK_ODCLK, CKG_GOPD_MASK);
    else if (enSrcSel==GOP_DRV_DWIN_SRC_MVOP)
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPD_CLK_DC0CLK, CKG_GOPD_MASK);
    else if (enSrcSel==GOP_DRV_DWIN_SRC_IP)
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPD_CLK_IDCLK2, CKG_GOPD_MASK);
    else
        return GOP_INVALID_PARAMETERS;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOP2CLK, &pGOPHalLocal->u16Clk1Setting); //Backup current GOPD clock settings
    return GOP_SUCCESS;
}
GOP_Result HAL_GOP_SetClock(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_BOOL bEnable)
{
    if (bEnable)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, pGOPHalLocal->u16Clk0Setting, GOP_REG_WORD_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, pGOPHalLocal->u16Clk1Setting, GOP_REG_WORD_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SRAMCLK, pGOPHalLocal->u16Clk2Setting, GOP_REG_WORD_MASK);
    }
    else
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG0_DISABLE_CLK, CKG_GOPG0_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, CKG_GOPG1_DISABLE_CLK, CKG_GOPG1_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPG2_DISABLE_CLK, CKG_GOPG2_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, CKG_GOPD_DISABLE_CLK, CKG_GOPD_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SRAMCLK, CKG_SRAM0_DISABLE_CLK, CKG_SRAM0_MASK);
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SRAMCLK, CKG_SRAM1_DISABLE_CLK, CKG_SRAM1_MASK);
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_MIXER_SetGOPEnable2Mixer(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL bEnable)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_MIXER_SetMux(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_U8 muxNum, MS_BOOL bEnable)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_MIXER_EnableOldBlendMode(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEn)
{
    return GOP_FUN_NOT_SUPPORTED;
}

void  HAL_GOP_Init_Context(GOP_CTX_HAL_LOCAL *pGOPHalLocal, GOP_CTX_HAL_SHARED *pHALShared, MS_BOOL bNeedInitShared)
{
    MS_U32 u32GopIdx;

    memset(pGOPHalLocal, 0, sizeof(*pGOPHalLocal));
    pGOPHalLocal->pHALShared = pHALShared;

    for(u32GopIdx=0; u32GopIdx<MAX_GOP_SUPPORT; u32GopIdx++)
    {
        pGOPHalLocal->drvGFlipGOPDst[u32GopIdx] = E_DRV_GOP_DST_OP0;
    }
    pGOPHalLocal->pGopChipPro = &g_GopChipPro;
    pGOPHalLocal->pbIsMuxVaildToGopDst = (MS_BOOL *)bIsMuxVaildToGopDst;
}
void  HAL_GOP_Restore_Ctx(GOP_CTX_HAL_LOCAL *pGOPHalLocal)
{
}

GOP_Result HAL_ConvertAPIAddr(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gwinid, MS_PHY* u64Adr)
{
    MS_U8 u8Miu;
    MS_U16 u16RegVal_L=0;
#ifdef GOP_MIU_GROUP2
    MS_U16 u16RegVal_H=0;
#endif
    MS_PHY phyTempAddr = 0;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_MIU_SEL, &u16RegVal_L);
#ifdef GOP_MIU_GROUP2
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MIU_GROUP2, &u16RegVal_H);
#endif
    if (gwinid<MAX_GOP0_GWIN)   //gop0
    {
        u16RegVal_L &= BIT(GOP_MIU_CLIENT_GOP0);
#ifdef GOP_MIU_GROUP2
        u16RegVal_H &= BIT(GOP_MIU_CLIENT_GOP0);
        u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP0) | (u16RegVal_H>>GOP_MIU_CLIENT_GOP0)<<1;
#else
        u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP0);
#endif
    }
    else if (gwinid>=MAX_GOP0_GWIN && gwinid<MAX_GOP0_GWIN+MAX_GOP1_GWIN)   //gop1
    {
        u16RegVal_L &= BIT(GOP_MIU_CLIENT_GOP1);
#ifdef GOP_MIU_GROUP2
        u16RegVal_H &= BIT(GOP_MIU_CLIENT_GOP1);
        u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP1) | (u16RegVal_H>>GOP_MIU_CLIENT_GOP1)<<1;
#else
        u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP1);
#endif
    }
    else if (gwinid==GOP2_GwinIdBase)  //gop2
    {
         u16RegVal_L &= BIT(GOP_MIU_CLIENT_GOP2);
#ifdef GOP_MIU_GROUP2
         u16RegVal_H &= BIT(GOP_MIU_CLIENT_GOP2);
         u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP2) | (u16RegVal_H>>GOP_MIU_CLIENT_GOP2)<<1;
#else
         u8Miu = (u16RegVal_L>>GOP_MIU_CLIENT_GOP2);
#endif
    }
    else
    {
        return GOP_FAIL;
    }

    if (u8Miu <= 2)
    {
        _miu_offset_to_phy(u8Miu, *u64Adr, phyTempAddr);
        *u64Adr = phyTempAddr;
    }
    else
    {
        GOP_H_ERR("[%s] ERROR GOP miu client\n",__FUNCTION__);
        return GOP_FAIL;
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetGOPDst(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8gopNum, DRV_GOPDstType *pGopDst)
{
    MS_U16 u16Regval;
    MS_U32 u32pBankOffSet=0;
    GOP_Result ret;

    if(u8gopNum >= MAX_GOP_SUPPORT)
    {
        GOP_H_ERR("[%s][%d] Out of GOP support!!! GOP=%d\n",__FUNCTION__,__LINE__ ,u8gopNum);
        *pGopDst = E_DRV_GOP_DST_INVALID;
        return GOP_FAIL;
    }

    _GetBnkOfstByGop(u8gopNum, &u32pBankOffSet);
    HAL_GOP_Read16Reg(pGOPHalLocal, u32pBankOffSet + GOP_4G_CTRL1, &u16Regval);

    switch (u16Regval&GOP_DST_MASK)
    {
        case 0:
            *pGopDst = E_DRV_GOP_DST_IP0;
            ret = GOP_SUCCESS;
            break;
        case 1:
            *pGopDst = E_DRV_GOP_DST_IP0_SUB;
            ret = GOP_SUCCESS;
            break;
        case 2:
            *pGopDst = E_DRV_GOP_DST_OP0;
            ret = GOP_SUCCESS;
            break;
        case 6:
            *pGopDst = E_DRV_GOP_DST_FRC;
            ret = GOP_SUCCESS;
            break;
        default:
            *pGopDst = E_DRV_GOP_DST_INVALID;
            ret = GOP_FAIL;
            break;
    }

    return ret;

}

void HAL_GOP_SetIPSel2SC(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_IPSEL_GOP ipSelGop)
{
    MS_U16 muxValue=0;
    MS_U16 u16RegVal= 0, u16RegMsk = 0;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX_IPVOP, &muxValue);

    switch(ipSelGop)
    {
    case MS_DRV_IP0_SEL_GOP0:
        u16RegVal = GOP_BIT7 | GOP_BIT5 ;
        if(E_GOP0 == (muxValue &GOP_MUX0_MASK))
            u16RegVal |= GOP_BIT12;//mux0
        else
            u16RegVal |= GOP_BIT13;//mux1
        u16RegMsk = 0xFFE7;//Skip bit3/4 for GOP_SUB_IP control
        break;

    case MS_DRV_IP0_SEL_GOP1:
        u16RegVal = GOP_BIT7 | GOP_BIT5 ;
        if(E_GOP1 == (muxValue &GOP_MUX0_MASK))
            u16RegVal |= GOP_BIT12;//mux0
        else
            u16RegVal |= GOP_BIT13;//mux1
        u16RegMsk = 0xFFE7;//Skip bit3/4 for GOP_SUB_IP control
        break;
    case MS_DRV_IP0_SEL_GOP2:
        u16RegVal = GOP_BIT7 | GOP_BIT5 ;
        if(E_GOP2 == (muxValue &GOP_MUX0_MASK))
            u16RegVal |= GOP_BIT12;//mux0
        else
            u16RegVal |= GOP_BIT13;//mux1
        u16RegMsk = 0xFFE7;//Skip bit3/4 for GOP_SUB_IP control
        break;

    case MS_DRV_NIP_SEL_GOP0:
        if(E_GOP0 == (muxValue &GOP_MUX0_MASK))
        {
                u16RegVal = ~GOP_BIT12;//mux0
                u16RegMsk = GOP_BIT12;
        }
            else
        {
                u16RegVal = ~GOP_BIT13;//mux1
                u16RegMsk = GOP_BIT13;
        }
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, GOP_BIT11, GOP_BIT11);
        break;

    case MS_DRV_NIP_SEL_GOP1:
        if(E_GOP1 == (muxValue &GOP_MUX0_MASK))
        {
                u16RegVal = ~GOP_BIT12;//mux0
                u16RegMsk = GOP_BIT12;
        }
        else
        {
                u16RegVal = ~GOP_BIT13;//mux1
                u16RegMsk = GOP_BIT13;
        }
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, GOP_BIT11, GOP_BIT11);

        break;
    case MS_DRV_NIP_SEL_GOP2:
        if(E_GOP2 == (muxValue &GOP_MUX0_MASK))
        {
                u16RegVal = ~GOP_BIT12;//mux0
                u16RegMsk = GOP_BIT12;
        }
            else
        {
                u16RegVal = ~GOP_BIT13;//mux1
                u16RegMsk = GOP_BIT13;
        }
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, GOP_BIT11, GOP_BIT11);

        break;
    case MS_DRV_MVOP_SEL:
        u16RegVal = GOP_BIT7 | GOP_BIT5 ;
        u16RegVal |= GOP_BIT12; //mux0
        u16RegMsk = 0xFFE7;//Skip bit3/4 for GOP_SUB_IP control
        break;
    default:
        GOP_H_ERR("[%s] ERROR invalid source select\n",__FUNCTION__);
        break;
    }
    if(0 != u16RegMsk)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, u16RegVal, u16RegMsk);
    }
}

GOP_Result HAL_GOP_DWIN_SetSourceSel(GOP_CTX_HAL_LOCAL *pGOPHalLocal, DRV_GOP_DWIN_SRC_SEL enSrcSel)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_DWIN_EnableR2YCSC(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_BOOL bEnable)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_MIXER_SetOutputTiming(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32mode, GOP_DRV_MixerTiming *pTM)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_MIXER_EnableVfilter(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_BOOL bEn)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_SetUVSwap(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOPNum,MS_BOOL bEn)
{
    MS_U32 u32BankOffSet =0;

    _GetBnkOfstByGop(u8GOPNum, &u32BankOffSet);

    if (bEn)
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_YUV_SWAP, GOP_BIT14, GOP_BIT14);
    else
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_YUV_SWAP, ~GOP_BIT14, GOP_BIT14);

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetYCSwap(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOPNum,MS_BOOL bEn)
{
    MS_U32 u32BankOffSet =0;

    _GetBnkOfstByGop(u8GOPNum, &u32BankOffSet);

    if (bEn)
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_YUV_SWAP, GOP_BIT15, GOP_BIT15);
    else
        HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_YUV_SWAP, 0x0, GOP_BIT15);

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_GetNewAlphaMode(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8win, MS_BOOL* pEnable)
{
    MS_U16 u16Val =0x0;
    if (u8win < GOP1_GwinIdBase)
    {
        HAL_GOP_Read16Reg(pGOPHalLocal, GOP_4G_GWIN_ALPHA01(u8win), &u16Val);
        *pEnable = (MS_BOOL)(u16Val>>15);
    }
    else if (u8win < GOP2_GwinIdBase)
    {
        HAL_GOP_Read16Reg(pGOPHalLocal, GOP_2G_GWIN_ALPHA01(u8win - MAX_GOP0_GWIN), &u16Val);
        *pEnable = (MS_BOOL)(u16Val>>15);
    }
    else if (u8win < GOP3_GwinIdBase)
    {
        HAL_GOP_Read16Reg(pGOPHalLocal, GOP_1G_GWIN_ALPHA01, &u16Val);
        *pEnable = (MS_BOOL)(u16Val>>15);
    }
    else
    {
        GOP_H_ERR("%s Not support this GWIN num%d!!!\n",__FUNCTION__, u8win);
        return GOP_INVALID_PARAMETERS;
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_SetNewAlphaMode(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8win, MS_BOOL bEnable)
{
    MS_U16 u16Val = bEnable << 15;
    if (u8win < GOP1_GwinIdBase)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_GWIN_ALPHA01(u8win), u16Val, GOP_BIT15);
    }
    else if (u8win < GOP2_GwinIdBase)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_2G_GWIN_ALPHA01(u8win - MAX_GOP0_GWIN), u16Val, GOP_BIT15);
    }
    else if (u8win < GOP3_GwinIdBase)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1G_GWIN_ALPHA01, u16Val, GOP_BIT15);
    }
    else
    {
        GOP_H_ERR("%s Not support this GWIN num%d!!!\n",__FUNCTION__, u8win);
        return GOP_INVALID_PARAMETERS;
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPToVE(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 gopNum, MS_BOOL bEn )
{
    return GOP_FUN_NOT_SUPPORTED;
}

E_GOP_VIDEOTIMING_MIRRORTYPE HAL_GOP_GetVideoTimingMirrorType(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_BOOL bHorizontal , EN_DRV_GOP_XC_MIRRORMODE enMirrorMode)
{
    E_GOP_VIDEOTIMING_MIRRORTYPE enMirrorType = E_GOP_VIDEOTIMING_MIRROR_BYSCALER;
    MS_U16 u16MVOPMirrorCfg = 0;
    //MS_U16 u16ScalerMirrorCfg = 0;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MVOP_MIRRORCFG, &u16MVOPMirrorCfg);
    //HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_MIRRORCFG, &u16ScalerMirrorCfg);

    if((enMirrorMode == E_DRV_GOP_XC_MIRROR_H_ONLY) || (enMirrorMode == E_DRV_GOP_XC_MIRROR_V_ONLY) || (enMirrorMode == E_DRV_GOP_XC_MIRROR_HV))
    {
        enMirrorType = E_GOP_VIDEOTIMING_MIRROR_BYSCALER;
        return enMirrorType;
    }
    else if((u16MVOPMirrorCfg & GOP_BIT0) || u16MVOPMirrorCfg & GOP_BIT1)
    {
        enMirrorType = E_GOP_VIDEOTIMING_MIRROR_BYMVOP;
        return enMirrorType;
    }
    else
    {
        return E_GOP_VIDEOTIMING_MIRROR_MAX;
    }
}

GOP_Result HAL_GOP_OC_SetOCEn(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_BOOL bOCEn)
{
    MS_U8  i=0, eGopMux=0, FRCMuxOffset=0;
    MS_U16 u16Val=0;
    MS_U16 u16Mux[MAX_GOP_MUX];

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MUX, &u16Val);
    u16Mux[0] =( u16Val & GOP_MUX0_MASK);
    u16Mux[1] =( u16Val & GOP_MUX1_MASK)    >> (GOP_MUX_SHIFT*1);
    u16Mux[2] =( u16Val & GOP_MUX2_MASK)    >> (GOP_MUX_SHIFT*2);
    u16Mux[3] =( u16Val & GOP_MUX3_MASK)    >> (GOP_MUX_SHIFT*3);
    u16Mux[4] =( u16Val & GOP_MUX4_MASK)    >> (GOP_MUX_SHIFT*4);

    switch(u8GOP)
    {
        case E_GOP0:
        case E_GOP1:
        case E_GOP2:
                for(i=0; i<MAX_GOP_MUX; i++)
                {
                    if(u8GOP == u16Mux[i])
                    {
                        eGopMux = i;
						if(eGopMux == 4)
						{
							HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_4K2K, bOCEn<<15, GOP_BIT15);
						}
						else
						{
							FRCMuxOffset = 12 + eGopMux;
							HAL_GOP_Write16Reg(pGOPHalLocal, GOP_MUX_IPVOP, bOCEn<<FRCMuxOffset, 1<<FRCMuxOffset);
						}
                    }
                }
                break;
        default:
                return GOP_FAIL;
                break;
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_OC_SetOCInfo(GOP_CTX_HAL_LOCAL *pGOPHalLocal, DRV_GOP_OC_INFO* pOCinfo)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_OC_Get_MIU_Sel(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 *MIUId)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_DWIN_SetRingBuffer(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32RingSize,MS_U32 u32BufSize)
{
	return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_AdjustField(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 GopNum, DRV_GOPDstType eDstType)
{
    return GOP_FUN_NOT_SUPPORTED;
}

/********************************************************************************/
///GOP Scaling down (internal)
/********************************************************************************/

MS_BOOL HAL_GOP_EnableScalingDownSram(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 u8GOP,MS_BOOL bEn)
{
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_DeleteWinHVSize(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 u8GOP, MS_U16 u16HSize, MS_U16 u16VSize)
{
    MS_U32 u32BankOffSet=0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_BANK_HVAILDSIZE, u16HSize, GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_BANK_VVAILDSIZE, u16VSize, GOP_REG_WORD_MASK);
    return GOP_SUCCESS;
}

GOP_Result  HAL_GOP_DumpGOPReg(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32GopIdx, MS_U16 u16BankIdx, MS_U16 u16Addr, MS_U16* u16Val)
{
    MS_U32 u32BankOffSet=0;
    _GetBnkOfstByGop(u32GopIdx, &u32BankOffSet);

    if (u32GopIdx < MAX_GOP_SUPPORT)
    {
        HAL_GOP_Read16Reg(pGOPHalLocal, (u32BankOffSet+ (u16BankIdx<<16) + u16Addr +GOP_4G_CTRL0), u16Val);
    }
    else
    {
        GOP_H_ERR("[%s][%d] Data is zero!!!\n",__FUNCTION__,__LINE__);
        *u16Val = 0;
    }
    return GOP_SUCCESS;
}

GOP_Result  HAL_GOP_RestoreGOPReg(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32GopIdx, MS_U16 u16BankIdx, MS_U16 u16Addr, MS_U16 u16Val)
{
    MS_U32 u32BankOffSet=0;
    _GetBnkOfstByGop(u32GopIdx, &u32BankOffSet);

    if (u32GopIdx < MAX_GOP_SUPPORT)
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, (u32BankOffSet+ (u16BankIdx<<16) + u16Addr +GOP_4G_CTRL0), u16Val, GOP_REG_WORD_MASK);
    }
    else
    {
        GOP_H_ERR("[%s][%d] Data is zero!!!\n",__FUNCTION__,__LINE__);
    }
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_PowerState(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U32 u32PowerState, GFLIP_REGS_SAVE_AREA* pGOP_STRPrivate)
{
    switch(u32PowerState)
    {
        case E_POWER_SUSPEND:
        {
            //CLK
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOPCLK, &(pGOP_STRPrivate->CKG_GopReg[0]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOP2CLK, &(pGOP_STRPrivate->CKG_GopReg[1]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOP3CLK, &(pGOP_STRPrivate->CKG_GopReg[2]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_GOP4CLK, &(pGOP_STRPrivate->CKG_GopReg[3]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SRAMCLK, &(pGOP_STRPrivate->CKG_GopReg[4]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_LB_SRAMCLK, &(pGOP_STRPrivate->CKG_GopReg[5]));

            //SRAM
            HAL_GOP_Read16Reg(pGOPHalLocal, CKG_GOPG0_SCALING, &(pGOP_STRPrivate->GS_GopReg[0]));
            HAL_GOP_Read16Reg(pGOPHalLocal, CKG_GOPG0_MG, &(pGOP_STRPrivate->GS_GopReg[1]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPSC_SRAM_CTRL, &(pGOP_STRPrivate->GS_GopReg[2]));

            //XC
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_BANKSEL, &(pGOP_STRPrivate->XC_GopReg[0]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, &(pGOP_STRPrivate->XC_GopReg[1]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPEN, &(pGOP_STRPrivate->XC_GopReg[2]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_IP_SYNC, &(pGOP_STRPrivate->XC_GopReg[3]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_IP2GOP_SRCSEL, &(pGOP_STRPrivate->XC_GopReg[4]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_OSD_CHECK_ALPHA, &(pGOP_STRPrivate->XC_GopReg[5]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_VOPNBL, &(pGOP_STRPrivate->XC_GopReg[6]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, &(pGOP_STRPrivate->XC_GopReg[7]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_MIRRORCFG, &(pGOP_STRPrivate->XC_GopReg[8]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_OCMIXER, &(pGOP_STRPrivate->XC_GopReg[9]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_OCMISC, &(pGOP_STRPrivate->XC_GopReg[10]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_OCALPHA, &(pGOP_STRPrivate->XC_GopReg[11]));
            HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_GOPSC_SRAM_CTRL, &(pGOP_STRPrivate->XC_GopReg[12]));
        }
            break;
        case E_POWER_RESUME:
        {
            //CLK
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOPCLK, pGOP_STRPrivate->CKG_GopReg[0], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP2CLK, pGOP_STRPrivate->CKG_GopReg[1], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP3CLK, pGOP_STRPrivate->CKG_GopReg[2], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_GOP4CLK, pGOP_STRPrivate->CKG_GopReg[3], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SRAMCLK, pGOP_STRPrivate->CKG_GopReg[4], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_LB_SRAMCLK, pGOP_STRPrivate->CKG_GopReg[5], GOP_REG_WORD_MASK);

            //SRAM
            HAL_GOP_Write16Reg(pGOPHalLocal, CKG_GOPG0_SCALING, pGOP_STRPrivate->GS_GopReg[0], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, CKG_GOPG0_MG, pGOP_STRPrivate->GS_GopReg[1], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPSC_SRAM_CTRL, pGOP_STRPrivate->GS_GopReg[2], GOP_REG_WORD_MASK);

            //XC
            //HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_BANKSEL, pGOP_STRPrivate->XC_GopReg[0], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_CHANNELSYNC, pGOP_STRPrivate->XC_GopReg[1], GOP_BIT11);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPEN, pGOP_STRPrivate->XC_GopReg[2], GOP_REG_WORD_MASK);
            //HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_IP_SYNC, pGOP_STRPrivate->XC_GopReg[3], GOP_REG_WORD_MASK);
            //HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_IP2GOP_SRCSEL, pGOP_STRPrivate->XC_GopReg[4], GOP_BIT15);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_OSD_CHECK_ALPHA, pGOP_STRPrivate->XC_GopReg[5], GOP_BIT6);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_VOPNBL, pGOP_STRPrivate->XC_GopReg[6], GOP_BIT5);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, pGOP_STRPrivate->XC_GopReg[7], GOP_REG_WORD_MASK);
            //HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_MIRRORCFG, pGOP_STRPrivate->XC_GopReg[8], GOP_REG_WORD_MASK);
            //HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_OCMIXER, pGOP_STRPrivate->XC_GopReg[9], GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_OCMISC, pGOP_STRPrivate->XC_GopReg[10], GOP_BIT2);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_OCALPHA, pGOP_STRPrivate->XC_GopReg[11], GOP_BIT2);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPSC_SRAM_CTRL, pGOP_STRPrivate->XC_GopReg[12], GOP_REG_WORD_MASK);
        }
            break;
        default:
            break;
    }
    return GOP_SUCCESS;
}
GOP_Result HAL_GOP_GWIN_SetGPUTileMode(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 gwinid, EN_DRV_GOP_GPU_TILE_MODE tile_mode)
{
    MS_U16 regval=0;

    switch(tile_mode)
    {
        case E_DRV_GOP_GPU_TILE_LINEAR:
            regval=0;
            break;
        case E_DRV_GOP_GPU_TILE_DIP8X:
            regval=1<<12;
            break;
        case E_DRV_GOP_GPU_TILE_DIP16X:
            regval=2<<12;
            break;
        case E_DRV_GOP_GPU_TILE_GPU:
            regval=3<<12;
            break;
        default:
            return GOP_FAIL;
            break;
    }
    regval |= GPU_TILE_FORMAT_ARGB8888<<4;

    if (gwinid<(MAX_GOP0_GWIN+MAX_GOP1_GWIN))
    {
        HAL_GOP_Write16Reg(pGOPHalLocal, (gwinid < MAX_GOP0_GWIN)? GOP_4G_GWIN0_CTRL(gwinid):GOP_2G_GWIN_CTRL(gwinid-MAX_GOP0_GWIN), regval, 0x31f0);
    }
    else
    {
        if (gwinid==(MAX_GOP0_GWIN+MAX_GOP1_GWIN))
        {
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1G_GWIN0_CTRL, regval, 0x31f0);
        }
        else if (gwinid==(MAX_GOP0_GWIN+MAX_GOP1_GWIN+MAX_GOP2_GWIN))
        {
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GX_GWIN0_CTRL, regval, 0x31f0);
        }
        else
        {
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_1GS0_GWIN0_CTRL, regval, 0x31f0);
        }
    }

    return GOP_SUCCESS;

}

GOP_Result HAL_GOP_Set_GWIN_INTERNAL_MIU(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP,MS_U8 miusel)
{
    MS_U32 u32BankOffSet=0xFFFF;
    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MIU_SEL, miusel, GOP_BIT0|GOP_BIT1 );//GWIN MIU Select
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MIU_SEL, miusel<<2, GOP_BIT2|GOP_BIT3 );//GWIN_3D MIU Select
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_MIU_SEL, miusel<<4, GOP_BIT4|GOP_BIT5 );//GWIN Palette MIU Select

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetIPInterlace(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_BOOL *bInterlace)
{
    MS_U16 reg_val = 0;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_IP_MAIN_INTERLACE, &reg_val);
    if(reg_val & BIT(11))
        *bInterlace = TRUE;
    else
        *bInterlace = FALSE;
    return GOP_SUCCESS;
}

GOP_Result Hal_SetCropWindow(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 u8GOP,EN_GOP_CROP_CTL crop_mode)
{
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGOPMixerValid(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8Gop, MS_BOOL bEnable)
{
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetDram_Addr(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U8 u8win, MS_PHY PhyAddr, E_DRV_GOP_AddrType eAddrType)
{
    MS_U16 u16tmp_REG_GOP_BASE_MSB = 0;
    MS_U32 u32BankOffSet = 0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    switch(eAddrType)
    {
        case E_DRV_GOP_RBLKAddr:
            HAL_GOP_Write32Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_L(u8win), PhyAddr);
            u16tmp_REG_GOP_BASE_MSB = ((PhyAddr&0x300000000) >> 32);
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_EX(u8win), (u16tmp_REG_GOP_BASE_MSB << 14), GOP_BIT15 | GOP_BIT14);
            break;
        case E_DRV_GOP_DRAMVstr:
            HAL_GOP_Write32Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_VSTR_L(u8win), PhyAddr);
            u16tmp_REG_GOP_BASE_MSB = ((PhyAddr&0x300000000) >> 32);
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_VSTR_EX(u8win), (u16tmp_REG_GOP_BASE_MSB << 14), GOP_BIT15 | GOP_BIT14);
            break;
        case E_DRV_GOP_RBLKSize:
            HAL_GOP_Write32Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_SIZE_L(u8win), PhyAddr);
            u16tmp_REG_GOP_BASE_MSB = ((PhyAddr&0x300000000) >> 32);
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_SIZE_EX(u8win), u16tmp_REG_GOP_BASE_MSB, GOP_BIT1 | GOP_BIT0);
            break;
        case E_DRV_GOP_ScrollHVStopAddr:
            HAL_GOP_Write32Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_HVSTOP_L(u8win), PhyAddr);
            u16tmp_REG_GOP_BASE_MSB = ((PhyAddr&0x300000000) >> 32);
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_HVSTOP_EX(u8win), (u16tmp_REG_GOP_BASE_MSB << 14), GOP_BIT15 | GOP_BIT14);
            break;
        case E_DRV_GOP_RBLK3DAddr:
            HAL_GOP_Write32Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_3DOSD_SUB_RBLK_L(u8win), PhyAddr);
            u16tmp_REG_GOP_BASE_MSB = ((PhyAddr&0x300000000) >> 32);
            HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_3DOSD_SUB_RBLK_EX(u8win), (u16tmp_REG_GOP_BASE_MSB << 14), GOP_BIT15 | GOP_BIT14);
            break;
        default:
            GOP_H_ERR("[%s][%d]Addr Type Error!!!!\n",__FUNCTION__,__LINE__);
            break;
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetDram_Addr(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U8 u8win, MS_PHY* pphyAddr, E_DRV_GOP_AddrType eAddrType)
{
    MS_U16 u16RegVal = 0;
    MS_U32 u32BankOffSet = 0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    switch(eAddrType)
    {
        case E_DRV_GOP_RBLKAddr:
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_L(u8win), &u16RegVal);
            *pphyAddr = (MS_U64)u16RegVal ;
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_H(u8win), &u16RegVal);
            *pphyAddr |= ((MS_U64)u16RegVal << 16);
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_EX(u8win), &u16RegVal);
            u16RegVal= ((u16RegVal >> 14) & 0x3);
            *pphyAddr |= ((MS_U64)u16RegVal << 32);
            break;
        case E_DRV_GOP_DRAMVstr:
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_VSTR_L(u8win), &u16RegVal);
            *pphyAddr = (MS_U64)u16RegVal ;
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_VSTR_H(u8win), &u16RegVal);
            *pphyAddr |= ((MS_U64)u16RegVal << 16);
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_VSTR_EX(u8win), &u16RegVal);
            u16RegVal= ((u16RegVal >> 14) & 0x3);
            *pphyAddr |= ((MS_U64)u16RegVal << 32);
            break;
        case E_DRV_GOP_RBLKSize:
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_SIZE_L(u8win), &u16RegVal);
            *pphyAddr = (MS_U64)u16RegVal ;
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_SIZE_H(u8win), &u16RegVal);
            *pphyAddr |= ((MS_U64)u16RegVal << 16);
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_DRAM_RBLK_SIZE_EX(u8win), &u16RegVal);
            u16RegVal= (u16RegVal & 0x3);
            *pphyAddr |= ((MS_U64)u16RegVal << 32);
            break;
        case E_DRV_GOP_ScrollHVStopAddr:
            break;
        case E_DRV_GOP_RBLK3DAddr:
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_3DOSD_SUB_RBLK_L(u8win), &u16RegVal);
            *pphyAddr = (MS_U64)u16RegVal ;
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_3DOSD_SUB_RBLK_H(u8win), &u16RegVal);
            *pphyAddr |= ((MS_U64)u16RegVal << 16);
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_3DOSD_SUB_RBLK_EX(u8win), &u16RegVal);
            u16RegVal= ((u16RegVal >> 14) & 0x3);
            *pphyAddr |= ((MS_U64)u16RegVal << 32);
            break;
        default:
            GOP_H_ERR("[%s][%d]Addr Type Error!!!!\n",__FUNCTION__,__LINE__);
            break;
    }

    return GOP_SUCCESS;
}

#ifdef GOP_CMDQ_ENABLE
GOP_Result HAL_GOP_CMDQ_WriteCommand(GOP_CTX_HAL_LOCAL *pGOPHalLocal,CAF_Struct *cmdq_struct,MS_U32 *number,MS_U32 u32addr, MS_U16 u16val, MS_U16 mask)
{
    MS_U16 u16xcSubbank=0;
    MS_U32 bank;
    MS_U32 direct_addr;

    switch (u32addr & 0xFF00)
    {
        case GOP_REG_BASE:
        {
            bank = (u32addr & 0xFF0000) >> 8;

            if(bank==0xE00)//GOP4:  0x121B00
            {
                    bank=GOP_REG_GOP4_BK_OFFSET;
            }
            else if(bank==0xF00)//GWIN4: 0x121E00
            {
                    bank=GOP_REG_GOP4_GW_OFFSET;
            }
            else if(bank==0x1000) //GOP4_ST
            {
                    bank=GOP_REG_GOP4_ST_OFFSET;
            }

            direct_addr = GOP_REG_DIRECT_BASE + bank + (u32addr & 0xFF);

            cmdq_struct[(*number)].destionation_address = (direct_addr&0xFFFFFF);
            cmdq_struct[(*number)].destionation_value = u16val;
            cmdq_struct[(*number)].mask = ((~mask)&0xFFFF);
            cmdq_struct[(*number)].operation = 0x57;
            (*number)++;
            break;
        }
        case SC1_REG_BASE:
            u16xcSubbank =  (u32addr & 0xFF0000)>>8 ;
            direct_addr = SC1_DIRREG_BASE + u16xcSubbank+ (u32addr & 0xFF);

            cmdq_struct[(*number)].destionation_address = (direct_addr&0xFFFFFF);
            cmdq_struct[(*number)].destionation_value = u16val;
            cmdq_struct[(*number)].mask = ((~mask)&0xFFFF);
            cmdq_struct[(*number)].operation = 0x57;
            (*number)++;
            break;
        case GE_REG_BASE:
        case CKG_REG_BASE:
        case MIU_REG_BASE:
        {
            cmdq_struct[(*number)].destionation_address = (u32addr&0xFFFFF)+0x100000;
            cmdq_struct[(*number)].destionation_value = u16val;
            cmdq_struct[(*number)].mask = ((~mask)&0xFFFF);
            cmdq_struct[(*number)].operation = 0x57;
            (*number)++;
            break;
        }
#ifdef GOP_MIU_GROUP2
        case (MIU2_REG_BASE & 0xFF00):
        {
            direct_addr = MIU2_REG_BASE + (u32addr & 0xFF);  //Direct_Base + addr_offset

            cmdq_struct[(*number)].destionation_address = (direct_addr&0xFFFFFF);
            cmdq_struct[(*number)].destionation_value = u16val;
            cmdq_struct[(*number)].mask = ((~mask)&0xFFFF);
            cmdq_struct[(*number)].operation = 0x57;
            (*number)++;
            break;
        }
#endif
        default:
        {
            //Gop lib current do not support this HW ip base
            MS_ASSERT(0);
            break;
        }

    }
    return GOP_SUCCESS;
}
GOP_Result HAL_GOP_CMDQ_BegineDraw(GOP_CTX_HAL_LOCAL *pGOPHalLocal,CAF_Struct *target,MS_U32 *number, MS_U32 u32GopIdx)
{
    MS_U32 u32BankOffSet=0xFFFF;
    MS_U16 u16RegVal1 = 0;
    MS_BOOL bCheckValidGop = FALSE;
    MS_U8 u8CheckTimeCnt = 0;
    MS_U8 u8CurrentCmdGop = 0;

    while (!bCheckValidGop && (u8CheckTimeCnt < (GOPG3_GOP_CMDQ_INT_3 - GOPG0_GOP_CMDQ_INT_0 + 2)))
    {
        if (u8CurrentCmdGop > (GOPG3_GOP_CMDQ_INT_3 - GOPG0_GOP_CMDQ_INT_0))
        {
            u8CurrentCmdGop = 0;
        }
        _GetBnkOfstByGop(u8CurrentCmdGop, &u32BankOffSet);
        HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, &u16RegVal1);
        if ((u16RegVal1 & GOP_BIT0) != 0) // gop not init, cmdq won't work
        {
            bCheckValidGop = FALSE;
        }
        else
        {
            bCheckValidGop = TRUE;
            break;
        }

        // if current gop not init, use next gop instead, check order 0->2->1->3
        switch (u8CurrentCmdGop)
        {
            case 0:
                u8CurrentCmdGop = 2;
                break;
            case 1:
                u8CurrentCmdGop = 0;
                break;
            case 2:
                u8CurrentCmdGop = 3;
                break;
            case 3:
                u8CurrentCmdGop = 1;
                break;
            default:
                u8CurrentCmdGop = 0;
                break;
        }
        u8CheckTimeCnt++;
    }
    if (!bCheckValidGop)
    {
        GOP_H_ERR("[%s] Error message no avalible gop can support current cmdq!!\n",__FUNCTION__);
    }
    _GetBnkOfstByGop(u8CurrentCmdGop, &u32BankOffSet);
    HAL_GOP_Read16Reg(pGOPHalLocal,u32BankOffSet+GOP_4G_MULTI_ALPHA,&u16RegVal1);
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,target,number,u32BankOffSet+GOP_4G_MULTI_ALPHA,(u16RegVal1|GOP_BIT4),0xFFFF);//reset mask
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,target,number,u32BankOffSet+GOP_4G_MULTI_ALPHA,(u16RegVal1&(~GOP_BIT4)) ,0xFFFF);    //reset nable detect
    MDrv_CMDQ_Gen_WaitTrigger_Bus_Command(&(target[(*number)]),GOPG0_GOP_CMDQ_INT_0 + u8CurrentCmdGop,FALSE);
    (*number)++;
    return GOP_SUCCESS;
}
GOP_Result HAL_GOP_CMDQ_EndDraw(GOP_CTX_HAL_LOCAL *pGOPHalLocal,CAF_Struct *target,MS_U32 *number, MS_U32 u32GopIdx)
{
    CH_Struct ch_fire;
    MS_U32 Receive_Return_Value = 0,u32BankOffSet = 0,timer1 = 0x0,timer2 = 0x0;
    MS_U16 u16ret = 0,u16ret1 = 0;
    int i = 0;


    _GetBnkOfstByGop(0, &u32BankOffSet);
    HAL_GOP_Read16Reg(pGOPHalLocal,u32BankOffSet+GOP_4G_BG_CLR(1),&u16ret);
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,target,number,u32BankOffSet+GOP_4G_BG_CLR(1),u16ret+1,0xFFFF);//current GOP force write dis
#if 0  //for Debug
    for(i=0;i<(*number);i++)
    {
        printf("\33[0;36m [%d]op = %d, addr = %lx,value = %x,mask = %x\33[m \n",i,target[i].operation,target[i].destionation_address,target[i].destionation_value,target[i].mask);
    }
#endif
    ch_fire.Command_Number = *number;
    ch_fire.Pointer_To_CAFArray = target;
    Receive_Return_Value = MDrv_CMDQ_Receive(&ch_fire);
    if(Receive_Return_Value == DRVCMDQ_CMDQ_FULL)
    {
        Receive_Return_Value = 0;
        MDrv_CMDQ_Printf_Crash_Command();
    }
    HAL_GOP_Read16Reg(pGOPHalLocal,u32BankOffSet+GOP_4G_BG_CLR(1),&u16ret1);
    timer1 = MsOS_GetSystemTime();
    timer2 = MsOS_GetSystemTime();
    while( u16ret1 != (u16ret+1) && ((timer2 - timer1)<100))
    {
        HAL_GOP_Read16Reg(pGOPHalLocal,u32BankOffSet+GOP_4G_BG_CLR(1),&u16ret1);
        MsOS_DelayTask(1);
        timer2 = MsOS_GetSystemTime();
    }
    if(u16ret1 != (u16ret+1))
        GOP_H_ERR("\33[0;36m   %s:%d  timeout = %ld org = %d target = %d\33[m \n",__FUNCTION__,__LINE__,(timer2 - timer1),u16ret1,(u16ret+1));
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_CMDQ_SetGOPACK(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 gop)
{
    CAF_Struct fire_struct[24];
    MS_U32 u32BankOffSet=0xFFFF;
    MS_U32 number = 0;
    MS_U16 u16RegVal1 = 0,u16RegVal2 = 0,u16RegVal3 = 0,u16MiuClient = 0;
    _GetBnkOfstByGop(gop, &u32BankOffSet);

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_MIU_SEL, &u16RegVal1);
#ifdef GOP_MIU_GROUP2
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MIU_GROUP2, &u16RegVal3);
#endif
    HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_BANK_FWR,&u16RegVal2);
    HAL_GOP_CMDQ_BegineDraw(pGOPHalLocal,fire_struct,&number,(MS_U32)0);

    switch(gop)
    {
        case E_GOP0:
        {
            u16MiuClient = GOP_MIU_CLIENT_GOP0;
            break;
        }
        case E_GOP1:
        {
            u16MiuClient = GOP_MIU_CLIENT_GOP1;
            break;
        }
        case E_GOP2:
        {
            u16MiuClient = GOP_MIU_CLIENT_GOP2;
            break;
        }
        default:
        {
            MS_ASSERT(0);
            break;
        }
    }
    if(bMIUSelect[gop] == TRUE)
    {
        if(u16MIUSelect[gop] == 0)
        {
            u16RegVal1 &= ~(1<<u16MiuClient);
            u16RegVal3 &= ~(1<<u16MiuClient);
        }
        else if(u16MIUSelect[gop] == 1)
        {
            u16RegVal1 |= (1<<u16MiuClient);
            u16RegVal3 &= ~(1<<u16MiuClient);
        }
        else if(u16MIUSelect[gop] == 2)
        {
            u16RegVal1 &= ~(1<<u16MiuClient);
            u16RegVal3 |= (1<<u16MiuClient);
        }
        else if(u16MIUSelect[gop] == 3)
        {
            u16RegVal1 |= (1<<u16MiuClient);
            u16RegVal3 |= (1<<u16MiuClient);
        }
        bMIUSelect[gop] = FALSE;
    }
    if(_bHalCSCSelect[gop] == TRUE)
    {
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_CSC_CTRL, _stHalCSCTbl[gop].u16CscControl, 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB11, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][0], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB12, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][1], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB13, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][2], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB21, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][0], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB22, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][1], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB23, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][2], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB31, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][0], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB32, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][1], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB33, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][2], 0xffff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_SLOW, _stHalCSCTbl[gop].u16BrightnessOffsetR, 0x7ff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_BRI, _stHalCSCTbl[gop].u16BrightnessOffsetG, 0x7ff);
        HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_CON, _stHalCSCTbl[gop].u16BrightnessOffsetB, 0x7ff);
        _bHalCSCSelect[gop] = FALSE;
    }
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,u32BankOffSet+GOP_4G_BANK_FWR,(u16RegVal2|(GOP_BIT0)) ,0xFFFF);//current GOP force write en
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,u32BankOffSet+GOP_4G_BANK_FWR,(u16RegVal2&(~GOP_BIT0)) ,0xFFFF);//current GOP force write en
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,GOP_SC_MIU_SEL,u16RegVal1,0xFFFF);
#ifdef GOP_MIU_GROUP2
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,GOP_MIU_GROUP2,u16RegVal3,0xFFFF);
#endif
    HAL_GOP_CMDQ_EndDraw(pGOPHalLocal,fire_struct,&number,(MS_U32)0);
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_CMDQ_SetGOPACKMask(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U16 u16GopMask)
{
    CAF_Struct fire_struct[24];
    MS_U32 u32BankOffSet;
    MS_U32 number = 0;
    MS_U8 gop;
    MS_U16 u16RegVal1=0,u16RegVal2=0,u16RegVal3 = 0,u16MiuClient = 0;

    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_SC_MIU_SEL, &u16RegVal1);
#ifdef GOP_MIU_GROUP2
    HAL_GOP_Read16Reg(pGOPHalLocal, GOP_MIU_GROUP2, &u16RegVal3);
#endif
    HAL_GOP_CMDQ_BegineDraw(pGOPHalLocal,fire_struct,&number,(MS_U32)0);

    for(gop = 0; gop<MAX_GOP_SUPPORT; gop++)
    {
        switch(gop)
        {
            case E_GOP0:
            {
                u16MiuClient = GOP_MIU_CLIENT_GOP0;
                break;
            }
            case E_GOP1:
            {
                u16MiuClient = GOP_MIU_CLIENT_GOP1;
                break;
            }
            case E_GOP2:
            {
                u16MiuClient = GOP_MIU_CLIENT_GOP2;
                break;
            }
            default:
            {
                continue;
            }
        }
        if( ( u16GopMask & (1<<gop) ) )
        {
            _GetBnkOfstByGop(gop, &u32BankOffSet);
            if(bMIUSelect[gop] == TRUE)
            {
                if(u16MIUSelect[gop] == 0)
                {
                    u16RegVal1 &= ~(1<<u16MiuClient);
                    u16RegVal3 &= ~(1<<u16MiuClient);
                }
                else if(u16MIUSelect[gop] == 1)
                {
                    u16RegVal1 |= (1<<u16MiuClient);
                    u16RegVal3 &= ~(1<<u16MiuClient);
                }
                else if(u16MIUSelect[gop] == 2)
                {
                    u16RegVal1 &= ~(1<<u16MiuClient);
                    u16RegVal3 |= (1<<u16MiuClient);
                }
                else if(u16MIUSelect[gop] == 3)
                {
                    u16RegVal1 |= (1<<u16MiuClient);
                    u16RegVal3 |= (1<<u16MiuClient);
                }
                bMIUSelect[gop] = FALSE;
            }
            if(_bHalCSCSelect[gop] == TRUE)
            {
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_CSC_CTRL, _stHalCSCTbl[gop].u16CscControl, 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB11, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][0], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB12, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][1], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB13, _stHalCSCTbl[gop].stCSCMatrix.Matrix[0][2], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB21, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][0], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB22, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][1], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB23, _stHalCSCTbl[gop].stCSCMatrix.Matrix[1][2], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB31, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][0], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB32, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][1], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_R2Y_SRGB33, _stHalCSCTbl[gop].stCSCMatrix.Matrix[2][2], 0xffff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_SLOW, _stHalCSCTbl[gop].u16BrightnessOffsetR, 0x7ff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_BRI, _stHalCSCTbl[gop].u16BrightnessOffsetG, 0x7ff);
                HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number, u32BankOffSet+GOP_4G_CON, _stHalCSCTbl[gop].u16BrightnessOffsetB, 0x7ff);
                _bHalCSCSelect[gop] = FALSE;
            }
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_BANK_FWR,&u16RegVal2);
            HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,u32BankOffSet+GOP_4G_BANK_FWR,(u16RegVal2|(GOP_BIT0)) ,0xFFFF);//current GOP force write en
            HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,u32BankOffSet+GOP_4G_BANK_FWR,(u16RegVal2&(~GOP_BIT0)) ,0xFFFF);//current GOP force write en
        }
    }
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,GOP_SC_MIU_SEL,u16RegVal1,0xFFFF);
#ifdef GOP_MIU_GROUP2
    HAL_GOP_CMDQ_WriteCommand(pGOPHalLocal,fire_struct,&number,GOP_MIU_GROUP2,u16RegVal3,0xFFFF);
#endif
    HAL_GOP_CMDQ_EndDraw(pGOPHalLocal,fire_struct,&number,(MS_U32)0);
    return GOP_SUCCESS;
}
#endif
GOP_Result MHal_GOP_SetCSCInfo(MS_U32 u32GOP,ST_GOP_CSC_TABLE *pstCSCTbl,MS_BOOL bFlag)
{
    _bHalCSCSelect[u32GOP] = bFlag;
    memcpy(&_stHalCSCTbl[u32GOP],pstCSCTbl,sizeof(ST_GOP_CSC_TABLE));
    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_IsHDREnabled(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_BOOL *pbHDREnable)
{
    *pbHDREnable= FALSE;
    return GOP_FUN_NOT_SUPPORTED;
}

GOP_Result HAL_GOP_SetGopGwinHVPixel(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U8 u8win, MS_U16 hstart, MS_U16 hend, MS_U16 vstart, MS_U16 vend)
{
    GOP_Result ret = GOP_SUCCESS;

    switch(u8GOP)
    {
        case E_GOP0:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_HSTR(u8win), hstart, GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_HEND(u8win), hend, GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_VSTR(u8win), vstart, GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_4G_VEND(u8win), vend, GOP_REG_WORD_MASK);
            break;
        case E_GOP1:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_2G_HSTR(u8win), hstart, GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_2G_HEND(u8win), hend, GOP_REG_WORD_MASK);
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_2G_VSTR(u8win), vstart, GOP_REG_WORD_MASK);    // 1 pixel
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_2G_VEND(u8win), vend, GOP_REG_WORD_MASK);    // 1 pixel
            break;
        case E_GOP2:
            HAL_GOP_Write16Reg(pGOPHalLocal, (u8win==GOP2_Gwin0Id)?GOP_1G_HSTR:GOP_1GX_HSTR, hstart, GOP_REG_WORD_MASK);    // word pixels
            HAL_GOP_Write16Reg(pGOPHalLocal, (u8win==GOP2_Gwin0Id)?GOP_1G_HEND:GOP_1GX_HEND, hend, GOP_REG_WORD_MASK);    // word pixels
            HAL_GOP_Write16Reg(pGOPHalLocal, (u8win==GOP2_Gwin0Id)?GOP_1G_VSTR:GOP_1GX_VSTR, vstart, GOP_REG_WORD_MASK);    // 1 pixel
            HAL_GOP_Write16Reg(pGOPHalLocal, (u8win==GOP2_Gwin0Id)?GOP_1G_VEND:GOP_1GX_VEND, vend, GOP_REG_WORD_MASK);    // 1 pixel
            break;
        default:
            GOP_H_ERR("invalid Gwin number:%d\n",u8win);
            break;
    }

    return ret;
}

GOP_Result HAL_GOP_SetDbgLevel(EN_GOP_DEBUG_LEVEL level)
{
    u32GOPDbgLevel_hal= level;
	return GOP_SUCCESS;
}
GOP_Result MHal_GOP_VOP_Path_Sel(GOP_CTX_HAL_LOCAL *pGOPHalLocal,EN_DRV_GOP_VOP_PATH enVOPPath)
{
    switch(enVOPPath)
    {
        case E_DRV_GOP_VOPPATH_DEF:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, 0x0, (GOP_BIT0 | GOP_BIT1));
            break;
        case E_DRV_GOP_VOPPATH_BEF_RGB3DLOOKUP:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, GOP_BIT0, (GOP_BIT0 | GOP_BIT1));
            break;
        case E_DRV_GOP_VOPPATH_BEF_PQGAMMA:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, GOP_BIT1, (GOP_BIT0 | GOP_BIT1));
            break;
        case E_DRV_GOP_VOPPATH_AFT_PQGAMMA:
            HAL_GOP_Write16Reg(pGOPHalLocal, GOP_SC_GOPENMODE1, (GOP_BIT0 | GOP_BIT1), (GOP_BIT0 | GOP_BIT1));
            break;
        default:
            break;
    }
    return GOP_SUCCESS;
}
GOP_Result MHal_GOP_SetCSCCtrl(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U32 u32GOP,ST_GOP_CSC_TABLE *pstCSCTbl)
{
    if(pGOPHalLocal == NULL || pstCSCTbl == NULL)
    {
        return GOP_FAIL;
    }
    MS_U32 u32BankOffSet=0xFFFF;
    _GetBnkOfstByGop(u32GOP, &u32BankOffSet);

    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CSC_CTRL, pstCSCTbl->u16CscControl, 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB11, pstCSCTbl->stCSCMatrix.Matrix[0][0], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB12, pstCSCTbl->stCSCMatrix.Matrix[0][1], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB13, pstCSCTbl->stCSCMatrix.Matrix[0][2], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB21, pstCSCTbl->stCSCMatrix.Matrix[1][0], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB22, pstCSCTbl->stCSCMatrix.Matrix[1][1], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB23, pstCSCTbl->stCSCMatrix.Matrix[1][2], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB31, pstCSCTbl->stCSCMatrix.Matrix[2][0], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB32, pstCSCTbl->stCSCMatrix.Matrix[2][1], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB33, pstCSCTbl->stCSCMatrix.Matrix[2][2], 0xffff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_SLOW, pstCSCTbl->u16BrightnessOffsetR, 0x7ff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_BRI, pstCSCTbl->u16BrightnessOffsetG, 0x7ff);
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_CON, pstCSCTbl->u16BrightnessOffsetB, 0x7ff);
    return GOP_SUCCESS;
}

static GOP_Result _MHal_GOP_MenuLoad_WriteCommand(GOP_CTX_HAL_LOCAL *pGOPHalLocal,ST_MENULOAD_REGISTER *pstReg,MS_U32 *pu32Cnt,MS_U32 u32addr, MS_U16 u16Val, MS_U16 u16Mask)
{
    MS_U32 u32Bank = 0;
    MS_U32 u32DirectAddr = 0;

    switch (u32addr & 0xFF00)
    {
        case GOP_REG_BASE:
        {
            u32Bank = (u32addr & 0xFF0000) >> 8;

            u32DirectAddr = GOP_REG_DIRECT_BASE + u32Bank + (u32addr & 0xFF);

            pstReg[(*pu32Cnt)].u32Bank = ((u32DirectAddr&0xFFFF00)>>8);
            pstReg[(*pu32Cnt)].u32Addr = (u32DirectAddr&0xFF);
            pstReg[(*pu32Cnt)].u16Data = u16Val;
            pstReg[(*pu32Cnt)].u16Mask = u16Mask;
            (*pu32Cnt)++;
            break;
        }
        default:
        {
            //Gop lib current do not support this HW ip base
            MS_ASSERT(0);
            break;
        }

    }
    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_SetGOPMenuLoad(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U8 u8GOP,ST_MENULOAD_REGISTER *pstReg,MS_U32 *pu16MLCnt)
{
    MS_U32 u32BankOffSet = 0;
    MS_U16 u16Tmp = 0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    if(_bHalCSCSelect[u8GOP] == TRUE)
    {
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_CSC_CTRL, _stHalCSCTbl[u8GOP].u16CscControl, 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB11, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[0][0], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB12, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[0][1], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB13, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[0][2], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB21, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[1][0], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB22, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[1][1], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB23, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[1][2], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB31, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[2][0], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB32, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[2][1], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB33, _stHalCSCTbl[u8GOP].stCSCMatrix.Matrix[2][2], 0xffff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_SLOW, _stHalCSCTbl[u8GOP].u16BrightnessOffsetR, 0x7ff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_BRI, _stHalCSCTbl[u8GOP].u16BrightnessOffsetG, 0x7ff);
        _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_CON, _stHalCSCTbl[u8GOP].u16BrightnessOffsetB, 0x7ff);
        _bHalCSCSelect[u8GOP] = FALSE;
    }
    HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_BANK_FWR,&u16Tmp);
    _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt,u32BankOffSet+GOP_4G_BANK_FWR,(u16Tmp|(GOP_BIT0)) ,0xFFFF);//current GOP force write en
    _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt,u32BankOffSet+GOP_4G_BANK_FWR,(u16Tmp&(~GOP_BIT0)) ,0xFFFF);//current GOP force write en

    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_SetGOPMenuLoadMask(GOP_CTX_HAL_LOCAL *pGOPHalLocal,MS_U16 u16GopMask,ST_MENULOAD_REGISTER *pstReg,MS_U32 *pu16MLCnt)
{
    MS_U8 u8Gop = 0;
    MS_U32 u32BankOffSet = 0;
    MS_U16 u16Tmp = 0;
    for(u8Gop = 0; u8Gop <MAX_GOP_SUPPORT; u8Gop ++)
    {
        if( ( u16GopMask & (1<<u8Gop) ) )
        {
            _GetBnkOfstByGop(u8Gop, &u32BankOffSet);
            if(_bHalCSCSelect[u8Gop] == TRUE)
            {
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_CSC_CTRL, _stHalCSCTbl[u8Gop].u16CscControl, 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB11, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[0][0], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB12, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[0][1], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB13, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[0][2], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB21, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[1][0], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB22, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[1][1], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB23, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[1][2], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB31, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[2][0], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB32, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[2][1], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_R2Y_SRGB33, _stHalCSCTbl[u8Gop].stCSCMatrix.Matrix[2][2], 0xffff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_SLOW, _stHalCSCTbl[u8Gop].u16BrightnessOffsetR, 0x7ff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_BRI, _stHalCSCTbl[u8Gop].u16BrightnessOffsetG, 0x7ff);
                _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt, u32BankOffSet+GOP_4G_CON, _stHalCSCTbl[u8Gop].u16BrightnessOffsetB, 0x7ff);
                _bHalCSCSelect[u8Gop] = FALSE;
            }
            HAL_GOP_Read16Reg(pGOPHalLocal, u32BankOffSet+GOP_4G_BANK_FWR,&u16Tmp);
            _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt,u32BankOffSet+GOP_4G_BANK_FWR,(u16Tmp|(GOP_BIT0)) ,0xFFFF);//current GOP force write en
            _MHal_GOP_MenuLoad_WriteCommand(pGOPHalLocal,pstReg,pu16MLCnt,u32BankOffSet+GOP_4G_BANK_FWR,(u16Tmp&(~GOP_BIT0)) ,0xFFFF);//current GOP force write en
        }
    }
    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_AutoDetectBuf(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, ST_GOP_AUTO_DETECT_BUF_INFO* pstAutoDectInfo)
{
    MS_BOOL bAlphaInv = FALSE;
    MS_U8 u8AlphaTh = 0;
    MS_U16 u16RegVal = 0;
    MS_U32 u32BankOffSet = 0;
    if(pstGOPHalLocal == NULL || pstAutoDectInfo == NULL)
    {
        printf("[%s][%d] pointer is NULL \n",__FUNCTION__,__LINE__);
        return GOP_FAIL;
    }
    if(_GetBnkOfstByGop(pstAutoDectInfo->u8GOPNum, &u32BankOffSet) != TRUE)
    {
        return GOP_FAIL;
    }

    if(pstAutoDectInfo->bEnable)
    {
         HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, GOP_AUTO_DETECT_BUF_EN, GOP_AUTO_DETECT_BUF_EN);
         HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_INT, GOP_INT_V0_MSK, GOP_INT_V0_MSK);
    }
    else
    {
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, 0x0, GOP_AUTO_DETECT_BUF_EN);
    }

    HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_CTRL0, &u16RegVal);
    bAlphaInv = (MS_BOOL) ((u16RegVal & GOP_ALPHA_INV_MSK) >> GOP_ALPHA_INV_SHIFT);
    if(bAlphaInv == TRUE)
    {
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, pstAutoDectInfo->u8AlphaTh, GOP_TRANS_FRAME_ALPHA_TH_MSK);
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, (pstAutoDectInfo->bLargeThanTH)? GOP_AUTO_TRANS_FRAME_DIRECTION : 0x0 , GOP_AUTO_TRANS_FRAME_DIRECTION);
    }
    else
    {
        u8AlphaTh = (255 -(pstAutoDectInfo->u8AlphaTh));
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, u8AlphaTh, GOP_TRANS_FRAME_ALPHA_TH_MSK);
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, GOP_AUTO_TRANS_FRAME_DIRECTION, GOP_AUTO_TRANS_FRAME_DIRECTION);
    }

    HAL_GOP_SetGOPACKMask(pstGOPHalLocal,1<<((pstAutoDectInfo->u8GOPNum)));

    return GOP_SUCCESS;
}

MS_BOOL MHal_GOP_GetAutoDetectEn(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOPNum)
{
    MS_U32 u32BankOffSet = 0;
    MS_U16 u16RegVal = 0;

    if(_GetBnkOfstByGop(u8GOPNum, &u32BankOffSet) != TRUE)
    {
        return FALSE;
    }

    HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, &u16RegVal);
    if(u16RegVal & GOP_AUTO_DETECT_BUF_EN)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

MS_BOOL MHal_GOP_IsTransparentBuff(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOPNum)
{
    MS_U32 u32BankOffSet = 0;
    MS_U16 u16RegVal = 0;

    if(_GetBnkOfstByGop(u8GOPNum, &u32BankOffSet) != TRUE)
    {
        return FALSE;
    }

    HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, &u16RegVal);
    if(u16RegVal & GOP_FRAME_VISIBLE)
    {
        return FALSE;
    }
    else
    {
        return TRUE;
    }
}

GOP_Result _MHal_GOP_SetCscMatrix(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP,MS_U16 *pu16CscMatrix)
{
    MS_U32 u32BankOffSet = 0;

    if(_GetBnkOfstByGop(u8GOP, &u32BankOffSet) != TRUE)
    {
        return GOP_FAIL;
    }

    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_CSC_CTRL, pu16CscMatrix[0], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB11, pu16CscMatrix[1], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB12, pu16CscMatrix[2], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB13, pu16CscMatrix[3], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB21, pu16CscMatrix[4], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB22, pu16CscMatrix[5], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB23, pu16CscMatrix[6], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB31, pu16CscMatrix[7], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB32, pu16CscMatrix[8], GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_R2Y_SRGB33, pu16CscMatrix[9], GOP_REG_WORD_MASK);

    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_GWIN_SetColorMatrix(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, GOP_OupputColor enGopOutColor, DRV_GOPColorType enColorType, MS_BOOL bCscEnable)
{
    MS_U32 u32BankOffSet = 0;

    if(_GetBnkOfstByGop(u8GOP, &u32BankOffSet) != TRUE)
    {
        return GOP_FAIL;
    }

    switch(enColorType)
    {
        case E_DRV_GOP_COLOR_YUV422:
        case E_DRV_GOP_COLOR_AYUV8888:
            if(enGopOutColor == DRV_GOPOUT_RGB)
            {
                if(bCscEnable == FALSE)
                {
                    _MHal_GOP_SetCscMatrix(pstGOPHalLocal, u8GOP,gu16GopY2rBt601);
                }
                else
                {

                    _MHal_GOP_SetCscMatrix(pstGOPHalLocal, u8GOP,gu16GopY2rCSC);
                }
            }
            else
            {
                if(bCscEnable == FALSE)
                {
                    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_CSC_CTRL, 0x0, GOP_BIT4);
                }
            }
            break;
        case E_DRV_GOP_COLOR_RGB555_BLINK:
        case E_DRV_GOP_COLOR_RGB565:
        case E_DRV_GOP_COLOR_ARGB4444:
        case E_DRV_GOP_COLOR_2266:
        case E_DRV_GOP_COLOR_I8:
        case E_DRV_GOP_COLOR_ARGB8888:
        case E_DRV_GOP_COLOR_ARGB1555:
        case E_DRV_GOP_COLOR_ABGR8888:
        case E_DRV_GOP_COLOR_RGB555YUV422:
        case E_DRV_GOP_COLOR_RGBA5551:
        case E_DRV_GOP_COLOR_RGBA4444:
        case E_DRV_GOP_COLOR_RGBA8888:
        case E_DRV_GOP_COLOR_BGR565:
        case E_DRV_GOP_COLOR_ABGR4444:
        case E_DRV_GOP_COLOR_ABGR1555:
        case E_DRV_GOP_COLOR_BGRA5551:
        case E_DRV_GOP_COLOR_BGRA4444:
        case E_DRV_GOP_COLOR_BGRA8888:
        default:
            if(enGopOutColor == DRV_GOPOUT_YUV)
            {
                if(bCscEnable == FALSE)
                {
                    _MHal_GOP_SetCscMatrix(pstGOPHalLocal, u8GOP,gu16GopR2yBt601);
                }
            }
            else
            {
                if(bCscEnable == FALSE)
                {
                    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_CSC_CTRL, 0x0, GOP_BIT4);
                }
                else
                {
                    _MHal_GOP_SetCscMatrix(pstGOPHalLocal, u8GOP,gu16GopR2rCSC);
                }
            }
            break;
    }

    return GOP_SUCCESS;
}

GOP_Result MHAL_GOP_Load_HStretch_New4TapMode_Table(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U16* pu16GopHStretchTable)
{
    MS_U8 u8CoefIdx = 0;
    MS_U32 u32BankOffSet = 0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    while(u8CoefIdx < GOP_HSTRETCH_4TAP_TABLE_NUM)
    {
        switch(u8CoefIdx)
        {
            case GOP_HSTRETCH_4TAP_COEF0_00:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_H4TAP_C, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_H4TAP_C, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_HSTRETCH_4TAP_COEF0_25:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_H4TAP_C, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_H4TAP_C, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_HSTRETCH_4TAP_COEF3_25:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_H4TAP_C, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_H4TAP_C, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_HSTRETCH_4TAP_COEF0_50:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_H4TAP_C, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_H4TAP_ALPHA, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_H4TAP_C, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_H4TAP_Y, pu16GopHStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
        }
        u8CoefIdx = u8CoefIdx + 2;
    }
    HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_CUBIC_6TAP, 0x0, GOP_BIT15);

    return GOP_SUCCESS;
}

GOP_Result MHAL_GOP_Load_VStretch_V4TapMode_Table(GOP_CTX_HAL_LOCAL *pGOPHalLocal, MS_U8 u8GOP, MS_U16* pu16GopVStretchTable)
{
    MS_U8 u8CoefIdx = 0;
    MS_U32 u32BankOffSet = 0;

    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

    while(u8CoefIdx < GOP_VSTRETCH_4TAP_TABLE_NUM)
    {
        switch(u8CoefIdx)
        {
            case GOP_VSTRETCH_4TAP_COEF0_00:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_V4TAP_C, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_00_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_V4TAP_C, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_00_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_VSTRETCH_4TAP_COEF0_25:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_V4TAP_C, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_25_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_V4TAP_C, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_25_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_VSTRETCH_4TAP_COEF3_25:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_V4TAP_C, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF5_25_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_V4TAP_C, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF3_25_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
            case GOP_VSTRETCH_4TAP_COEF0_50:
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_V4TAP_C, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF2_50_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_V4TAP_ALPHA, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_V4TAP_C, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                HAL_GOP_Write16Reg(pGOPHalLocal, u32BankOffSet + REG_GOP_COEF0_50_V4TAP_Y, pu16GopVStretchTable[u8CoefIdx+1], GOP_REG_WORD_MASK);
                break;
        }
        u8CoefIdx = u8CoefIdx + 2;
    }
    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_AutoDetectBuf_V2(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, ST_GOP_AUTO_DETECT_BUF_INFO* pstAutoDectInfo)
{
    MS_BOOL bAlphaInv = FALSE;
    MS_U8 u8AlphaTh = 0;
    MS_U16 u16RegVal = 0;
    MS_U32 u32BankOffSet = 0;
    if(pstGOPHalLocal == NULL || pstAutoDectInfo == NULL)
    {
        printf("[%s][%d] pointer is NULL \n",__FUNCTION__,__LINE__);
        return GOP_FAIL;
    }
    if(_GetBnkOfstByGop(pstAutoDectInfo->u8GOPNum, &u32BankOffSet) != TRUE)
    {
        return GOP_FAIL;
    }

    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, (pstAutoDectInfo->bEnable)?GOP_AUTO_DETECT_BUF_EN:0x0, GOP_AUTO_DETECT_BUF_EN);

    HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_CTRL0, &u16RegVal);
    bAlphaInv = (MS_BOOL) ((u16RegVal & GOP_ALPHA_INV_MSK) >> GOP_ALPHA_INV_SHIFT);
    if(bAlphaInv == TRUE)
    {
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, pstAutoDectInfo->u8AlphaTh, GOP_TRANS_FRAME_ALPHA_TH_MSK);
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, (pstAutoDectInfo->bLargeThanTH)? GOP_AUTO_TRANS_FRAME_DIRECTION : 0x0 , GOP_AUTO_TRANS_FRAME_DIRECTION);
    }
    else
    {
        u8AlphaTh = (255 -(pstAutoDectInfo->u8AlphaTh));
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, u8AlphaTh, GOP_TRANS_FRAME_ALPHA_TH_MSK);
        HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, GOP_AUTO_TRANS_FRAME_DIRECTION, GOP_AUTO_TRANS_FRAME_DIRECTION);
    }

    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet+GOP_4G_AUTO_DETECT_BUF, GOP_AUTO_DETECT_BUF_TRIG, GOP_AUTO_DETECT_BUF_TRIG);

    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_GetOsdNonTransCnt(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U32* pu32Count)
{
    MS_U16 u16RegValMsb = 0, u16RegValLsb = 0 ,u16RegVal = 0;

    HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_BAK_SEL, &u16RegVal);
    if(u16RegVal & (GOP4G_VAL_WR_ACK|GOP2G_VAL_WR_ACK|GOPD_VAL_WR_ACK))
    {
        MsOS_DelayTask(16); //Index count will update at the next Vsync after GOP setting take effect
    }
    MsOS_DelayTask(16); //Index count will update at the next Vsync after GOP setting take effect

    HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_SC_OSD_NONTRANS_COUNT_MSB, &u16RegValMsb);
    HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_SC_OSD_NONTRANS_COUNT_LSB, &u16RegValLsb);
    u16RegValMsb &= BMASK(11:0);
    u16RegValLsb &= BMASK(11:0);

    *pu32Count = (((MS_U32)u16RegValMsb)<<12)|((MS_U32)u16RegValLsb);

    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_Trigger_MUX(GOP_CTX_HAL_LOCAL *pstGOPHalLocal)
{
    HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL_EX, GOP_OPMUX_TRIGGER, GOP_OPMUX_TRIGGER);
    return GOP_SUCCESS;
}

GOP_Result MHal_GOP_FillCscMartrix(GOP_CTX_HAL_LOCAL *pstGOPHalLocal,MS_U16 *pu16CSCValue, EN_GOP_CFD_MC_FORMAT enInFmt, EN_GOP_CFD_MC_FORMAT enOutFmt)
{
    if(enInFmt == E_GOP_CFD_MC_FORMAT_YUV422 && enOutFmt == E_GOP_CFD_MC_FORMAT_RGB)
    {
        memcpy(gu16GopY2rCSC,pu16CSCValue,sizeof(MS_U16)*GOP_CSC_ELEMENT_NUM);
    }
    else if (enInFmt == E_GOP_CFD_MC_FORMAT_RGB && enOutFmt == E_GOP_CFD_MC_FORMAT_RGB)
    {
        memcpy(gu16GopR2rCSC,pu16CSCValue,sizeof(MS_U16)*GOP_CSC_ELEMENT_NUM);
    }

    return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_SetHMirror(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, bEnable?GOP_BIT12:0, GOP_BIT12);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetVMirror(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32Bankoffset = 0;

	_GetBnkOfstByGop(u8GOP, &u32Bankoffset);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_CTRL0, bEnable?GOP_BIT13:0, GOP_BIT13);

	return GOP_SUCCESS;
}

MS_BOOL HAL_GOP_GetVMirror(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP)
{
	MS_U32 u32Bankoffset = 0;
	MS_U16 u16RegVal = 0;
	MS_BOOL bEnable;

	_GetBnkOfstByGop(u8GOP, &u32Bankoffset);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_CTRL0, &u16RegVal);
	bEnable = (u16RegVal & GOP_BIT13) >> 13;

	return bEnable;
}

MS_BOOL HAL_GOP_GetHMirror(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP)
{
	MS_U32 u32Bankoffset = 0;
	MS_U16 u16RegVal = 0;
	MS_BOOL bEnable;

	_GetBnkOfstByGop(u8GOP, &u32Bankoffset);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_CTRL0, &u16RegVal);
	bEnable = (u16RegVal & GOP_BIT12) >> 12;

	return bEnable;
}

GOP_Result HAL_GOP_SetGWIN_Enable(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8win, MS_BOOL bEnable)
{
	MS_U16 u16RegVal = 0;
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	if(bEnable == TRUE) {
		HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_MULTI_ALPHA, 0, GOP_BIT11);
		HAL_GOP_Read16Reg(pstGOPHalLocal, REG_SC_IP_FILED, &u16RegVal);
		if(u16RegVal & SC_IP_FILED_INV)
			HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, 0, 0x10);
		else
			HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, 1<<4, 0x10);
	}

	switch(u8GOP) {
	case E_GOP0:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_4G_GWIN0_CTRL(u8win), bEnable?1:0, 0x0001);
	break;
	case E_GOP1:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_2G_GWIN0_CTRL(u8win), bEnable?1:0, 0x0001);
	break;
	case E_GOP2:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_1G_GWIN0_CTRL, bEnable?1:0, 0x0001);
	break;
	default:
	break;
	}

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetWinFmt(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8Win, MS_U16 *pu16Colortype)
{
	switch(u8GOP) {
	case E_GOP0:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_GWIN0_CTRL(u8Win), pu16Colortype);
	break;
	case E_GOP1:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_GWIN_CTRL(u8Win), pu16Colortype);
	break;
	case E_GOP2:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_GWIN0_CTRL, pu16Colortype);
	break;
	default:
	break;
	}

	*pu16Colortype = ((*pu16Colortype >> GOP_REG_COLORTYPE_SHIFT) & GOP_REG_COLORTYPE_MASK);
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetPitch(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8Win, MS_U16 u16FbWidth, MS_U16 u16FbHeight)
{
	switch(u8GOP) {
	case E_GOP0:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_4G_DRAM_RBLK_HSIZE(u8Win), u16FbWidth, GOP_REG_WORD_MASK);
		break;
	case E_GOP1:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_2G_DRAM_RBLK_HSIZE(u8Win), u16FbWidth, GOP_REG_WORD_MASK);
	break;
	case E_GOP2:
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_1G_DRAM_RBLK_HSIZE, u16FbWidth, GOP_REG_WORD_MASK);
	break;
	default:
	break;
	}

	return GOP_SUCCESS;
}

MS_U16 HAL_GOP_GetPitch(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8Win)
{
	MS_U16 u16RegVal = 0;

	switch(u8GOP) {
	case E_GOP0:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_DRAM_RBLK_HSIZE(u8Win), &u16RegVal);
		break;
	case E_GOP1:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_DRAM_RBLK_HSIZE(u8Win), &u16RegVal);
	break;
	case E_GOP2:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_DRAM_RBLK_HSIZE, &u16RegVal);
	break;
	default:
	break;
	}

	return u16RegVal;
}

GOP_Result HAL_GOP_GetGopGwinHVPixel(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8Win, MS_U16 *u16Hstart, MS_U16 *u16Hend, MS_U16 *u16Vstart, MS_U16 *u16Vend)
{
	switch(u8GOP) {
	case E_GOP0:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_HSTR(u8Win), u16Hstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_HEND(u8Win), u16Hend);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_VSTR(u8Win), u16Vstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_VEND(u8Win), u16Vend);
		break;
	case E_GOP1:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_HSTR(u8Win), u16Hstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_HEND(u8Win), u16Hend);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_VSTR(u8Win), u16Vstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_VEND(u8Win), u16Vend);
		break;
	case E_GOP2:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_HSTR, u16Hstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_HEND, u16Hend);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_VSTR, u16Vstart);
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_VEND, u16Vend);
		break;
	default:
		break;
	}

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_SetProgressive(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, bEnable?0x8:0x0, 0x8);

	return GOP_SUCCESS;
}

MS_BOOL HAL_GOP_GWIN_GetProgressive(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP)
{
	MS_U16 u16Regval = 0;
	MS_U32 u32BankOffSet = 0;
	
    _GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, &u16Regval);

	if ((u16Regval & 8) == 0)
		return FALSE;
	else
		return TRUE;
}

GOP_Result HAL_GOP_SetGopStretchWin(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16x, MS_U16 u16y, MS_U16 u16width, MS_U16 u16height)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_HSTR, u16x, GOP_REG_WORD_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_HSZ, (u16width/GOP_STRETCH_WIDTH_UNIT), GOP_REG_WORD_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_VSTR, u16y, GOP_REG_WORD_MASK);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_VSZ, u16height, GOP_REG_WORD_MASK);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetGopStretchWin(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 *u16x, MS_U16 *u16y, MS_U16 *u16width, MS_U16 *u16height)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_HSTR, u16x);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_VSTR, u16y);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_HSZ, u16width);
	*u16width = *u16width * GOP_STRETCH_WIDTH_UNIT;
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_STRCH_VSZ, u16height);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_SetAlphaInverse(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	if (bEnable)
		HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, GOP_BIT15, GOP_BIT15);
    else
		HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, ~GOP_BIT15, GOP_BIT15);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetGopScaleRatio(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U32 *u32Hratio, MS_U32 *u32Vratio)
{
	MS_U16 u16RegVal = 0;
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HSTRCH, &u16RegVal);
    *u32Hratio = (u16RegVal << 8);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_H_V_STRCH_LSB, &u16RegVal);
    *u32Hratio |= (u16RegVal & 0xFF);

	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_VSTRCH, &u16RegVal);
    *u32Vratio = (u16RegVal << 8);
    HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_H_V_STRCH_LSB, &u16RegVal);
    *u32Vratio |= ((u16RegVal & 0xFF00) >> 8);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGopVaildH(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16ValidH)
{
	MS_U32 u32Bankoffset = 0;

    _GetBnkOfstByGop(u8GOP, &u32Bankoffset);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_SVM_VSTR, u16ValidH, GOP_REG_WORD_MASK);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetGopVaildV(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16ValidV)
{
	MS_U32 u32Bankoffset = 0;

    _GetBnkOfstByGop(u8GOP, &u32Bankoffset);
    HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_SVM_VEND, u16ValidV, GOP_REG_WORD_MASK);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GWIN_SetTransClr(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, GOP_TransClrFmt fmt, MS_BOOL bEnable)
{
	MS_U16 u16Regval = 0;
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, &u16Regval);
	if (bEnable) {
		if(fmt == DRV_GOPTRANSCLR_FMT3)
			u16Regval |= 0x020;
		else
			u16Regval |= 0x0800;
	}
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CTRL0, u16Regval, 0x820);

	return GOP_SUCCESS;
}

EN_DRV_GOP_VOP_PATH Hal_GOP_GetVOPPathSel(GOP_CTX_HAL_LOCAL *pstGOPHalLocal)
{
	MS_U16 u16RegVal = 0;
	EN_DRV_GOP_VOP_PATH enVOPSel;

	HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_SC_GOPENMODE1, &u16RegVal);
	u16RegVal = (u16RegVal & (GOP_BIT0 | GOP_BIT1));

	switch (u16RegVal) {
	case E_DRV_GOP_VOPPATH_BEF_RGB3DLOOKUP:
		enVOPSel = E_DRV_GOP_VOPPATH_BEF_RGB3DLOOKUP;
	break;
	case E_DRV_GOP_VOPPATH_BEF_PQGAMMA:
		enVOPSel = E_DRV_GOP_VOPPATH_BEF_PQGAMMA;
	break;
	case E_DRV_GOP_VOPPATH_AFT_PQGAMMA:
		enVOPSel = E_DRV_GOP_VOPPATH_AFT_PQGAMMA;
	break;
	default:
		enVOPSel = E_DRV_GOP_VOPPATH_DEF;
	break;
	}

	return enVOPSel;
}

GOP_Result HAL_GOP_SetPipe(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16HSPipe)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HS_PIPE, u16HSPipe, 0x1FFF);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetVOPNBL(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_BOOL bEnable)
{
	if (bEnable == TRUE) {
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_SC_VOPNBL, 0x20, 0x20);
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_SC_OSD_CHECK_ALPHA, 0x0, 0x40);
	} else {
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_SC_VOPNBL, 0x0, 0x20);
		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_SC_OSD_CHECK_ALPHA, 0x40, 0x40);
	}

	return GOP_SUCCESS;
}

MS_BOOL HAL_GOP_GetGWINEnable(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U8 u8Win)
{
	MS_U16 regval = 0;
	MS_BOOL bEnable = FALSE, bAutoDetectEn = FALSE, bIsTransBuf = FALSE;

	switch (u8GOP) {
	case E_GOP0:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_4G_GWIN0_CTRL(u8Win), &regval);
	break;
	case E_GOP1:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_2G_GWIN0_CTRL(u8Win), &regval);
	break;
	case E_GOP2:
		HAL_GOP_Read16Reg(pstGOPHalLocal, GOP_1G_GWIN0_CTRL, &regval);
	break;
	default:
	break;
	}

	if (regval & 0x0001)
		bEnable = TRUE;
	else
		bEnable = FALSE;

	if(bEnable == TRUE) {
		if((g_GopChipPro.bAutoDetectVersion_1_0[u8GOP] == TRUE) || (g_GopChipPro.bAutoDetectVersion_2_0[u8GOP] == TRUE)) {
            bAutoDetectEn = MHal_GOP_GetAutoDetectEn(pstGOPHalLocal, u8GOP);
            bIsTransBuf = MHal_GOP_IsTransparentBuff(pstGOPHalLocal, u8GOP);

            bEnable = (bAutoDetectEn==TRUE && bIsTransBuf == TRUE)?FALSE:TRUE;
        }
    }
	return bEnable;
}

GOP_Result HAL_GOP_GWIN_TriggerRegWriteIn(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable, E_DRV_GOPTriggerMode eGOPTriggerMode)
{
	MS_U16 u16Mask = 0;
	MS_U32 u32BankOffSet = 0;

	if (eGOPTriggerMode == E_DRV_GOP_FORCEWRITE) {
		if(bEnable == TRUE)
			HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, GOP_BIT9, GOP_BIT9);
		else
			HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, 0, GOP_BIT9);
	} else if (eGOPTriggerMode == E_DRV_GOP_BNKFORCEWRITE) {
		_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
		if(bEnable == TRUE)
			HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_BANK_FWR, GOP_BIT0, GOP_BIT0);
		else
			HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_BANK_FWR, 0, GOP_BIT0);
	} else if (eGOPTriggerMode == E_DRV_GOP_WITHVSYNC) {
		switch(u8GOP) {
		case 0:
			HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, GOP_BIT12, GOP_BIT12);
		break;
		case 1:
			HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, GOP_BIT13, GOP_BIT13);
		break;
		case 2:
			HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, GOP_BIT14, GOP_BIT14);
		break;
		default:
		break;
		}
	} else if (eGOPTriggerMode == E_DRV_GOP_WITHSYNC_MULTIGOPUPDATED) {
		if (u8GOP & GOP_BIT0)
			u16Mask |= GOP_BIT12;

		if (u8GOP & GOP_BIT1)
			u16Mask |= GOP_BIT13;

		if (u8GOP & GOP_BIT2)
			u16Mask |= GOP_BIT14;

		HAL_GOP_Write16Reg(pstGOPHalLocal, GOP_BAK_SEL, 0xFFFF, u16Mask);
	}

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetFirstHs(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_RATE, bEnable << 7, BIT(7));

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetHSCALE(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16HInitPhase, MS_U64 u64Hratio)
{
	MS_U32 u32Bankoffset = 0;

	_GetBnkOfstByGop(u8GOP, &u32Bankoffset);

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_H_V_STRCH_LSB, (u64Hratio & 0xFF), GOP_REG_LW_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_HSTRCH, ((u64Hratio >> 8) & 0xFFFF), GOP_REG_WORD_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_HSTRCH_INI, u16HInitPhase, GOP_REG_WORD_MASK);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetVSCALE(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16VInitPhase, MS_U64 u64Vratio)
{
	MS_U32 u32Bankoffset = 0;

	_GetBnkOfstByGop(u8GOP, &u32Bankoffset);

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_H_V_STRCH_LSB, (u64Vratio & 0xFF), GOP_REG_HW_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_VSTRCH, ((u64Vratio >> 8) & 0xFFFF), GOP_REG_WORD_MASK);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32Bankoffset + GOP_4G_VSTRCH_INI, u16VInitPhase, GOP_REG_WORD_MASK);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetStep1HVSPHscale(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable, MS_U32 u32Hratio)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetStep1HVSPVscale(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable, MS_U32 u32Vratio)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetStep2HVSPHscale(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_BOOL bEnable, MS_U32 u32Hratio)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetStep2HVSPVscale(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_BOOL bEnable, MS_U32 u32Vratio)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetMixer2OutSize(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U16 u16Width, MS_U16 u16Height)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetMixer4OutSize(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U16 u16Width, MS_U16 u16Height)
{
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetHTotal(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16Width)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_RDMA_HT, (u16Width+3)/2, 0x07ff);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetInterrupt(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_INT, (bEnable?0:GOP_BIT0) , GOP_BIT0);

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_SetBrightness(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 u16BriVal, MS_BOOL bMSB)
{
	MS_U32 u32BankOffSet = 0;
    MS_U16 u16RegVal = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	u16RegVal = (bMSB<<10) | (u16BriVal<<2);
	u16RegVal += BRIGHTNESS_OFFSET;
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_SLOW, u16RegVal, BMASK(10:0) );
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_BRI, u16RegVal, BMASK(10:0) );
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_CON, u16RegVal, BMASK(10:0) );

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_GetBrightness(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_U16 *u16BriVal, MS_BOOL *bMSB)
{
	MS_U32 u32BankOffSet = 0;
	MS_U16 u16RegVal = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	HAL_GOP_Read16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_BRI, &u16RegVal);
	u16RegVal -= BRIGHTNESS_OFFSET;
	*bMSB = (MS_BOOL)((u16RegVal&0x400)>>10);
	*u16BriVal = (u16RegVal & 0x3FF)>>2;

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_Set_HStretchMode(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, DRV_GOPStrchHMode HStrchMode)
{
	MS_U16 u16Val = 0;
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);
	
	switch (HStrchMode) {
	case E_DRV_GOP_HSTRCH_6TAPE:
		u16Val = 0;
	break;
	case E_DRV_GOP_HSTRCH_DUPLICATE:
		u16Val = 1;
	break;
	case E_DRV_GOP_HSTRCH_4TAPE:
	case E_DRV_GOP_HSTRCH_NEW4TAP_45:
	case E_DRV_GOP_HSTRCH_NEW4TAP_55:
	case E_DRV_GOP_HSTRCH_NEW4TAP_65:
	case E_DRV_GOP_HSTRCH_NEW4TAP_75:
	case E_DRV_GOP_HSTRCH_NEW4TAP_85:
	case E_DRV_GOP_HSTRCH_NEW4TAP_95:
	case E_DRV_GOP_HSTRCH_NEW4TAP_105:
	case E_DRV_GOP_HSTRCH_NEW4TAP_100:
		u16Val = 3;
	break;
	default:
		u16Val = 0;
	break;
	}
	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HVSTRCHMD, u16Val, GOP_BIT0|GOP_BIT1);
	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_Set_VStretchMode(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, DRV_GOPStrchVMode VStrchMode)
{
	MS_U16 u16Val = 0;
	MS_U32 u32BankOffSet = 0;

	_GetBnkOfstByGop(u8GOP, &u32BankOffSet);

	switch (VStrchMode) {
	case E_DRV_GOP_VSTRCH_LINEAR:
		u16Val = 0;
	break;
	case E_DRV_GOP_VSTRCH_DUPLICATE:
		u16Val = 1;
	break;
	case E_DRV_GOP_VSTRCH_NEAREST:
		u16Val = 2;
	break;
	case E_DRV_GOP_VSTRCH_LINEAR_GAIN2:
	case E_DRV_GOP_VSTRCH_4TAP:
	case E_DRV_GOP_HSTRCH_V4TAP_100:
		u16Val = 3;
	break;
	default:
		u16Val = 0;
	break;
	}

	HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HVSTRCHMD, u16Val<<2, GOP_BIT2|GOP_BIT3);

	if (VStrchMode == E_DRV_GOP_VSTRCH_4TAP) {
		if (pstGOPHalLocal->pGopChipPro->bSupportV4tap[u8GOP] == TRUE)
			HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HVSTRCHMD, GOP_BIT6, GOP_BIT6);
		else
			GOP_H_WARN("\nGOP_%d Not Support 4-Tap V stretch mode!'\n",u8GOP);
	} else {
		HAL_GOP_Write16Reg(pstGOPHalLocal, u32BankOffSet + GOP_4G_HVSTRCHMD, 0, GOP_BIT6);
	}

	return GOP_SUCCESS;
}

GOP_Result HAL_GOP_AFBC_Core_Enable(GOP_CTX_HAL_LOCAL *pstGOPHalLocal, MS_U8 u8GOP, MS_BOOL bEnable)
{
	return GOP_SUCCESS;
}