
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2008-2009 MStar Semiconductor, Inc.
// All rights reserved.
//
// Unless otherwise stipulated in writing, any and all information contained
// herein regardless in any format shall remain the sole proprietary of
// MStar Semiconductor Inc. and be kept in strict confidence
// ("MStar Confidential Information") by the recipient.
// Any unauthorized act including without limitation unauthorized disclosure,
// copying, use, reproduction, sale, distribution, modification, disassembling,
// reverse engineering and compiling of the contents of MStar Confidential
// Information is unlawful and strictly prohibited. MStar hereby reserves the
// rights to any and all damages, losses, costs and expenses resulting therefrom.
//
////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
/// file    Mdrv_hdmi.c
/// @brief  Driver Interface
/// @author MStar Semiconductor Inc.
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef MSOS_TYPE_NUTTX
#define  MDRV_HDMI_C
//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------
// Common Definition
#include "MsCommon.h"
#include "MsVersion.h"
#ifdef MSOS_TYPE_LINUX_KERNEL
#include <linux/string.h>
#include <linux/notifier.h>
#include <linux/reboot.h>
#else
#include <string.h>
#endif
#include "MsIRQ.h"
#include "MsOS.h"
#include "mhal_xc_chip_config.h"
#include "utopia.h"
#include "utopia_dapi.h"

// Internal Definition
#include "drvXC_IOPort.h"
#include "apiXC.h"
#include "drvXC_HDMI_Internal.h"
#include "mhal_hdmi.h"
#include "drv_sc_display.h"
#include "drv_sc_isr.h"
#include "apiXC_PCMonitor.h"
#include "apiXC_ModeParse.h"
#include "apiXC_Adc.h"
#include "apiXC_Auto.h"
#include "drvXC_ADC_Internal.h"
#include "mhal_adc.h"
#include "xc_hwreg_utility2.h"
#include "drv_sc_mux.h"
#include "mvideo_context.h"
#include "drv_sc_ip.h"
#if (LD_ENABLE==1)
#include "mdrv_ld.h"
#include "mdrv_ldalgo.h"
#endif
#include "mdrv_sc_3d.h"
#include "drv_sc_menuload.h"
#include "mhal_sc.h"
#if FRC_INSIDE
#include "mdrv_frc.h"
#include "mhal_frc.h"
#endif
#include "XC_private.h"
#include "apiXC_v2.h"
#include "drvXC_HDMI_if_v2.h"
#include "apiHDMIRx_private.h"

#include "mdrv_xc_st.h"
#include "mdrv_xc_io.h"
#include "drvPM.h"

#if (defined MSOS_TYPE_LINUX) || (defined ANDROID)
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h> // O_RDWR
#include <poll.h>
#endif

#ifdef  MSOS_TYPE_LINUX
#include <assert.h>
#else
#define assert(b)   \
    do {    \
        if (!(b)){  \
            printf("assert at [%s][%d] \n", __FUNCTION__, __LINE__);   \
            *((int*)0) = 0; \
        }   \
    } while(0)
#endif

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------
#define msg_hdmi(x) //x
#define DRV_HDMI_HDR(x) //x
//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#define HDMI_HDCP_WAIT_ENENT_TIMEOUT    100U

#if(defined(CONFIG_MLOG))
#include "ULog.h"

#define MDRV_HDMIRX_MSG_INFO(format, args...)       ULOGI("HDMIRX", format, ##args)
#define MDRV_HDMIRX_MSG_WARNING(format, args...)    ULOGW("HDMIRX", format, ##args)
#define MDRV_HDMIRX_MSG_DEBUG(format, args...)      ULOGD("HDMIRX", format, ##args)
#define MDRV_HDMIRX_MSG_ERROR(format, args...)      ULOGE("HDMIRX", format, ##args)
#define MDRV_HDMIRX_MSG_FATAL(format, args...)      ULOGF("HDMIRX", format, ##args)

#else
#define MDRV_HDMIRX_MSG_INFO(format, args...)       printf(format, ##args)
#define MDRV_HDMIRX_MSG_WARNING(format, args...)    printf(format, ##args)
#define MDRV_HDMIRX_MSG_DEBUG(format, args...)      printf(format, ##args)
#define MDRV_HDMIRX_MSG_ERROR(format, args...)      printf(format, ##args)
#define MDRV_HDMIRX_MSG_FATAL(format, args...)      printf(format, ##args)

#endif

#ifdef MSOS_TYPE_LINUX_KERNEL
#define atoi(str) simple_strtoul(((str != NULL) ? str : ""), NULL, 0)
#endif

//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------
MS_BOOL bInitEnable = FALSE;
MS_U8 ucHDMIInfoSourceSelect = HDMI_INFO_SOURCE0;

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
HDCP22_Recv_CBF fpHdcp22RecMsgCBFunc = NULL; //wilson for u2k //type, portIdx, msg, msg length, context
#ifdef MSOS_TYPE_LINUX
static MS_S32   _s32FdScaler = -1;
#endif

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
/// Debug information
static MSIF_Version _drv_hdmi_version = {.DDI = { HDMI_DRV_VERSION },};
void* pulHDMIRxInst;
void* pulHDMIEARCInst;

MS_BOOL bHDCPInterruptProcFlag = FALSE;
MS_BOOL bHDCPInterruptCreateFlag = FALSE;
MS_S32 slHDCPInterruptTaskID = -1;
static MS_S32 slHDCPMutexID = -1 ;
MS_U8 ucHDCPInterruptStack[HDMI_HDCP_INTERRUPT_STACK_SIZE];

//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------
static void MDrv_HDMI_ReadEDID(E_XC_DDCRAM_PROG_TYPE eDDCRamType, MS_U8 *u8EDID, MS_U16 u8EDIDSize);
static void MDrv_HDMI_WriteEDID(E_XC_DDCRAM_PROG_TYPE eDDCRamType, MS_U8 *u8EDID, MS_U16 u16EDIDSize);

extern MS_U32 MsOS_GetSystemTime(void);
#ifdef MSOS_TYPE_LINUX_KERNEL
extern int tee_get_supplicant_count(void);
#endif

#ifdef MSOS_TYPE_LINUX_KERNEL
extern MS_BOOL KDrv_XC_GetEmPacket(MS_U8 enHDMIPort, EN_KDRV_HDMI_EMP_TYPE enEmpType, MS_U8 u8CurrentPacketIndex, MS_U8 *pu8TotalPacketNumber, MS_U8 *pu8EmPacket);
extern ssize_t MDrv_PM_Read_Key(unsigned short u16Key, const char *buf);
#endif

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_CheckPacketReceiveProc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_CheckPacketReceiveProc(HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate)
{
    MS_U8 u8HDMIInfoSource = 0;
    MS_U32 u32PacketStatus = 0;

    for(u8HDMIInfoSource = HDMI_INFO_SOURCE0; u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX; u8HDMIInfoSource++)
    {
        if(pHDMIRxResourcePrivate->ucReceiveIntervalCount[u8HDMIInfoSource] < pHDMIRxResourcePrivate->u8PacketReceiveCount[u8HDMIInfoSource])
        {
            pHDMIRxResourcePrivate->ucReceiveIntervalCount[u8HDMIInfoSource]++;
        }
        else
        {
            u32PacketStatus = Hal_HDMI_packet_info(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);

            pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] = u32PacketStatus;

            if(GET_HDMI_SYSTEM_FLAG(pHDMIRxResourcePrivate->u32PrePacketStatus[u8HDMIInfoSource], HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG))
            {
                pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] |= HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG;
            }

            if(GET_HDMI_SYSTEM_FLAG(u32PacketStatus, HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG))
            {
                SET_HDMI_SYSTEM_FLAG(pHDMIRxResourcePrivate->u32PrePacketStatus[u8HDMIInfoSource], HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG);
            }
            else
            {
                CLR_HDMI_SYSTEM_FLAG(pHDMIRxResourcePrivate->u32PrePacketStatus[u8HDMIInfoSource], HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG);
            }

            pHDMIRxResourcePrivate->u8PacketReceiveCount[u8HDMIInfoSource] = Hal_HDMI_GetPacketReceiveCount(u8HDMIInfoSource, pHDMIRxResourcePrivate->u8PacketReceiveCount[u8HDMIInfoSource], pHDMIRxResourcePrivate->stHDMIPollingInfo);

            pHDMIRxResourcePrivate->ucReceiveIntervalCount[u8HDMIInfoSource] = 0;
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_CheckHDCP14RiProc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_CheckHDCP14RiProc(HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate)
{
    E_MUX_INPUTPORT enPortIndex = INPUT_PORT_DVI0;

    for(enPortIndex = INPUT_PORT_DVI0; enPortIndex <= INPUT_PORT_DVI_END; enPortIndex++)
    {
        if(pHDMIRxResourcePrivate->usHDCP14RiCount[HDMI_GET_PORT_SELECT(enPortIndex)] < HDMI_HDCP14_RI_COUNT)
        {
            pHDMIRxResourcePrivate->usHDCP14RiCount[HDMI_GET_PORT_SELECT(enPortIndex)]++;
        }
        else
        {
            if(Hal_HDCP_getstatus(enPortIndex) & BIT(13)) // [13] Ri
            {
                pHDMIRxResourcePrivate->bHDCP14RiFlag[HDMI_GET_PORT_SELECT(enPortIndex)] = TRUE;
                Hal_HDCP_ClearStatus(enPortIndex, BIT(5));
            }
            else
            {
                pHDMIRxResourcePrivate->bHDCP14RiFlag[HDMI_GET_PORT_SELECT(enPortIndex)] = FALSE;
            }

            pHDMIRxResourcePrivate->usHDCP14RiCount[HDMI_GET_PORT_SELECT(enPortIndex)] = 0;
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_PollingTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_HDMI_PollingTask(void)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bHDMITaskProcFlag = TRUE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaModuleGetPtr(MODULE_HDMIRX, &pModule);

    while(bHDMITaskProcFlag)
    {
        if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
        }
        else
        {
            if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
            }
            else
            {
                Hal_HDMI_StablePolling(pHDMIRxResourcePrivate->ucMHLSupportPath, pHDMIRxResourcePrivate->ulPacketStatus, pHDMIRxResourcePrivate->stHDMIPollingInfo, pHDMIRxResourcePrivate->fpHdcp14_AccessWriteProtectedReg);

                _mdrv_HDMI_CheckPacketReceiveProc(pHDMIRxResourcePrivate);
                _mdrv_HDMI_CheckHDCP14RiProc(pHDMIRxResourcePrivate);

                bHDMITaskProcFlag = pHDMIRxResourcePrivate->bHDMITaskProcFlag;
            }
        }

        UtopiaResourceRelease(pHDMIRxResource);

        MsOS_DelayTask(HDMI_POLLING_INTERVAL);
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_CreateTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL _mdrv_HDMI_CreateTask(HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate)
{
    MS_BOOL bCreateSuccess = TRUE;

    if(pHDMIRxResourcePrivate->slHDMIPollingTaskID < 0)
    {
        pHDMIRxResourcePrivate->slHDMIPollingTaskID = MsOS_CreateTask((TaskEntry) _mdrv_HDMI_PollingTask,
                                     (MS_VIRT) NULL,
                                     E_TASK_PRI_MEDIUM,
                                     TRUE,
                                     (void *)pHDMIRxResourcePrivate->ucHDMIPollingStack,
                                     HDMI_POLLING_STACK_SIZE,
                                     "HDMI Rx task");

        pHDMIRxResourcePrivate->bHDMITaskProcFlag = TRUE;
    }

    if(pHDMIRxResourcePrivate->slHDMIPollingTaskID < 0)
    {
        MDRV_HDMIRX_MSG_ERROR("MsOS_CreateTask failed!!\n");

        if(MsOS_DeleteEventGroup(pHDMIRxResourcePrivate->slHDMIPollingTaskID))
        {
            //for coverity Unchecked return value
        }

        bCreateSuccess = FALSE;
    }

    return bCreateSuccess;
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_EARC_PollingTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_EARC_PollingTask(void)
{
    void* pEARCModule = NULL;
    void* pHDMIEARCResource = NULL;
    MS_BOOL bEARCaskProcFlag = TRUE;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaModuleGetPtr(MODULE_HDMIEARC, &pEARCModule);

    while(bEARCaskProcFlag)
    {
        if(UtopiaResourceObtain(pEARCModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
        }
        else
        {
            if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
            }
            else
            {
                Hal_EARC_TX_PollingHandler(pHDMIEARCTxResourcePrivate);

                bEARCaskProcFlag = pHDMIEARCTxResourcePrivate->bEARCTaskProcFlag;
            }
        }

        UtopiaResourceRelease(pHDMIEARCResource);

        MsOS_DelayTask(HDMI_POLLING_INTERVAL);
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_EARC_CreateTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL _mdrv_EARC_CreateTask(HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate)
{
    MS_BOOL bCreateSuccess = TRUE;

    if(pHDMIEARCTxResourcePrivate->slEARCPollingTaskID < 0)
    {
        pHDMIEARCTxResourcePrivate->slEARCPollingTaskID = MsOS_CreateTask((TaskEntry) _mdrv_EARC_PollingTask,
                                     (MS_VIRT) NULL,
                                     E_TASK_PRI_MEDIUM,
                                     TRUE,
                                     (void *)pHDMIEARCTxResourcePrivate->ucEARCPollingStack,
                                     HDMI_POLLING_STACK_SIZE,
                                     "HDMI EARC task");

        pHDMIEARCTxResourcePrivate->bEARCTaskProcFlag = TRUE;
    }

    if(pHDMIEARCTxResourcePrivate->slEARCPollingTaskID < 0)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d]MsOS_CreateTask failed!!\n",__FUNCTION__,__LINE__);

        if(MsOS_DeleteEventGroup(pHDMIEARCTxResourcePrivate->slEARCPollingTaskID))
        {
            //for coverity Unchecked return value
        }

        bCreateSuccess = FALSE;
    }

    return bCreateSuccess;
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_CheckHDCP14Bksv()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL _mdrv_HDMI_CheckHDCP14Bksv(MS_U8 *ucBksvTable)
{
    MS_BOOL bHDCP14KeyVaild = FALSE;
    MS_U8 uctemp = 0;
    MS_U8 ucOneCount = 0;
    MS_U8 ucBksvValue = 0;

    if(ucBksvTable != NULL)
    {
        for(uctemp = 0; uctemp < HDMI_HDCP_BKSV_LENGTH; uctemp++)
        {
            ucBksvValue = ucBksvTable[uctemp];

            while(ucBksvValue > 0)
            {
                if(ucBksvValue & BIT(0))
                {
                    ucOneCount++;
                }

                ucBksvValue = ucBksvValue >> 1;
            }
        }

        if(ucOneCount == 20)
        {
            bHDCP14KeyVaild = TRUE;
        }
    }

    return bHDCP14KeyVaild;
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_ParseHDRInfoFrame()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_ParseHDRInfoFrame(void *pPacket, sHDR_METADATA *pHdrMetadata, MS_BOOL bByteFormat)
{
    if(pHdrMetadata->u16Version == HDMI_HDR_METADATA_VERSION)
    {
        // HDR metadata structure version is the same.
        if(bByteFormat)
        {
            MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

            // 0: HB1
            // 1: HB2
            // 2: PB0
            // 3: PB1
            // 4: PB2
            // 5: PB3
            // 6: PB4
            pHdrMetadata->u8Version = pucInfoFrame[0];
            pHdrMetadata->u8Length = pucInfoFrame[1];
            pHdrMetadata->u8EOTF = pucInfoFrame[3] &BMASK(2:0);
            pHdrMetadata->u8Static_Metadata_ID = pucInfoFrame[4] &BMASK(2:0);

            if(pHdrMetadata->u8Static_Metadata_ID == HDMI_HDR_STATIC_METADATA_TYPE_0)
            {
                pHdrMetadata->u16Display_Primaries_X[0] = (pucInfoFrame[6] << 8) |pucInfoFrame[5];
                pHdrMetadata->u16Display_Primaries_Y[0] = (pucInfoFrame[8] << 8) |pucInfoFrame[7];
                pHdrMetadata->u16Display_Primaries_X[1] = (pucInfoFrame[10] << 8) |pucInfoFrame[9];
                pHdrMetadata->u16Display_Primaries_Y[1] = (pucInfoFrame[12] << 8) |pucInfoFrame[11];
                pHdrMetadata->u16Display_Primaries_X[2] = (pucInfoFrame[14] << 8) |pucInfoFrame[13];
                pHdrMetadata->u16Display_Primaries_Y[2] = (pucInfoFrame[16] << 8) |pucInfoFrame[15];
                pHdrMetadata->u16White_Point_X = (pucInfoFrame[18] << 8) |pucInfoFrame[17];
                pHdrMetadata->u16White_Point_Y = (pucInfoFrame[20] << 8) |pucInfoFrame[19];
                pHdrMetadata->u16Max_Display_Mastering_Luminance = (pucInfoFrame[22] << 8) |pucInfoFrame[21];
                pHdrMetadata->u16Min_Display_Mastering_Luminance = (pucInfoFrame[24] << 8) |pucInfoFrame[23];
                pHdrMetadata->u16Maximum_Content_Light_Level = (pucInfoFrame[26] << 8) |pucInfoFrame[25];
                pHdrMetadata->u16Maximum_Frame_Average_Light_Level = (pucInfoFrame[28] << 8) |pucInfoFrame[27];
            }
        }
        else
        {
            MS_U16 *pInfoFrame = (MS_U16*)pPacket;

            pHdrMetadata->u8EOTF = ((pInfoFrame[0]>>8)&0x07); // 8~15bits, 0:SDR gamma,  1:HDR gamma,  2:SMPTE ST2084,  3:Future EOTF,  4-7:Reserved
            pHdrMetadata->u8Static_Metadata_ID = (pInfoFrame[1]&0x0007);    // 0:Static Metadata Type 1,  1-7:Reserved for future use
            pHdrMetadata->u16Display_Primaries_X[0] = (pInfoFrame[2]<<8) + (pInfoFrame[1]>>8);

            pHdrMetadata->u16Display_Primaries_Y[0] = (pInfoFrame[3]<<8) + (pInfoFrame[2]>>8);
            pHdrMetadata->u16Display_Primaries_X[1] = (pInfoFrame[4]<<8) + (pInfoFrame[3]>>8);
            pHdrMetadata->u16Display_Primaries_Y[1] = (pInfoFrame[5]<<8) + (pInfoFrame[4]>>8);
            pHdrMetadata->u16Display_Primaries_X[2] = (pInfoFrame[6]<<8) + (pInfoFrame[5]>>8);
            pHdrMetadata->u16Display_Primaries_Y[2] = (pInfoFrame[7]<<8) + (pInfoFrame[6]>>8);
            pHdrMetadata->u16White_Point_X = (pInfoFrame[8]<<8) + (pInfoFrame[7]>>8);
            pHdrMetadata->u16White_Point_Y = (pInfoFrame[9]<<8) + (pInfoFrame[8]>>8);
            pHdrMetadata->u16Max_Display_Mastering_Luminance = (pInfoFrame[10]<<8) + (pInfoFrame[9]>>8);
            pHdrMetadata->u16Min_Display_Mastering_Luminance = (pInfoFrame[11]<<8) + (pInfoFrame[10]>>8);
            pHdrMetadata->u16Maximum_Content_Light_Level = (pInfoFrame[12]<<8) + (pInfoFrame[11]>>8);
            pHdrMetadata->u16Maximum_Frame_Average_Light_Level = (pInfoFrame[13]<<8) + (pInfoFrame[12]>>8);
        }
    }
    else
    {
        // Error handle.
        if (pHdrMetadata->u16Size <= HDMI_HDR_METADATA_VERSION_1_SIZE)
        {
            if(bByteFormat)
            {
                MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

                // 0: HB1
                // 1: HB2
                // 2: PB0
                // 3: PB1
                // 4: PB2
                // 5: PB3
                // 6: PB4
                pHdrMetadata->u8EOTF = pucInfoFrame[3] &BMASK(2:0);
                pHdrMetadata->u8Static_Metadata_ID = pucInfoFrame[4] &BMASK(2:0);

                if(pHdrMetadata->u8Static_Metadata_ID == HDMI_HDR_STATIC_METADATA_TYPE_0)
                {
                    pHdrMetadata->u16Display_Primaries_X[0] = (pucInfoFrame[6] << 8) |pucInfoFrame[5];
                    pHdrMetadata->u16Display_Primaries_Y[0] = (pucInfoFrame[8] << 8) |pucInfoFrame[7];
                    pHdrMetadata->u16Display_Primaries_X[1] = (pucInfoFrame[10] << 8) |pucInfoFrame[9];
                    pHdrMetadata->u16Display_Primaries_Y[1] = (pucInfoFrame[12] << 8) |pucInfoFrame[11];
                    pHdrMetadata->u16Display_Primaries_X[2] = (pucInfoFrame[14] << 8) |pucInfoFrame[13];
                    pHdrMetadata->u16Display_Primaries_Y[2] = (pucInfoFrame[16] << 8) |pucInfoFrame[15];
                    pHdrMetadata->u16White_Point_X = (pucInfoFrame[18] << 8) |pucInfoFrame[17];
                    pHdrMetadata->u16White_Point_Y = (pucInfoFrame[20] << 8) |pucInfoFrame[19];
                    pHdrMetadata->u16Max_Display_Mastering_Luminance = (pucInfoFrame[22] << 8) |pucInfoFrame[21];
                    pHdrMetadata->u16Min_Display_Mastering_Luminance = (pucInfoFrame[24] << 8) |pucInfoFrame[23];
                    pHdrMetadata->u16Maximum_Content_Light_Level = (pucInfoFrame[26] << 8) |pucInfoFrame[25];
                    pHdrMetadata->u16Maximum_Frame_Average_Light_Level = (pucInfoFrame[28] << 8) |pucInfoFrame[27];
                }
            }
            else
            {
                MS_U16 *pInfoFrame = (MS_U16*)pPacket;

                pHdrMetadata->u8EOTF = ((pInfoFrame[0]>>8)&0x07); // 8~15bits, 0:SDR gamma,  1:HDR gamma,  2:SMPTE ST2084,  3:Future EOTF,  4-7:Reserved
                pHdrMetadata->u8Static_Metadata_ID = (pInfoFrame[1]&0x0007);    // 0:Static Metadata Type 1,  1-7:Reserved for future use
                pHdrMetadata->u16Display_Primaries_X[0] = (pInfoFrame[2]<<8) + (pInfoFrame[1]>>8);

                pHdrMetadata->u16Display_Primaries_Y[0] = (pInfoFrame[3]<<8) + (pInfoFrame[2]>>8);
                pHdrMetadata->u16Display_Primaries_X[1] = (pInfoFrame[4]<<8) + (pInfoFrame[3]>>8);
                pHdrMetadata->u16Display_Primaries_Y[1] = (pInfoFrame[5]<<8) + (pInfoFrame[4]>>8);
                pHdrMetadata->u16Display_Primaries_X[2] = (pInfoFrame[6]<<8) + (pInfoFrame[5]>>8);
                pHdrMetadata->u16Display_Primaries_Y[2] = (pInfoFrame[7]<<8) + (pInfoFrame[6]>>8);
                pHdrMetadata->u16White_Point_X = (pInfoFrame[8]<<8) + (pInfoFrame[7]>>8);
                pHdrMetadata->u16White_Point_Y = (pInfoFrame[9]<<8) + (pInfoFrame[8]>>8);
                pHdrMetadata->u16Max_Display_Mastering_Luminance = (pInfoFrame[10]<<8) + (pInfoFrame[9]>>8);
                pHdrMetadata->u16Min_Display_Mastering_Luminance = (pInfoFrame[11]<<8) + (pInfoFrame[10]>>8);
                pHdrMetadata->u16Maximum_Content_Light_Level = (pInfoFrame[12]<<8) + (pInfoFrame[11]>>8);
                pHdrMetadata->u16Maximum_Frame_Average_Light_Level = (pInfoFrame[13]<<8) + (pInfoFrame[12]>>8);
            }
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_ParseAVIInfoFrame()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_ParseAVIInfoFrame(void *pPacket, stAVI_PARSING_INFO *pstAVIParsingInfo)
{
    MS_BOOL bExtendColormetryFlag = FALSE;

    pstAVIParsingInfo->enColorForamt = MS_HDMI_COLOR_RGB;
    pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_DEFAULT;
    pstAVIParsingInfo->enColormetry = MS_HDMI_COLORIMETRY_NO_DATA;
    pstAVIParsingInfo->enPixelRepetition = MS_HDMI_PIXEL_REPETITION_1X;
    pstAVIParsingInfo->enAspectRatio = HDMI_Pic_AR_NODATA;
    pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_SAME;

    if(pstAVIParsingInfo->u16StructVersion == HDMI_AVI_PARSING_INFO_VERSION)
    {
        MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

        // 0: PB1
        // 1: PB2
        // 2: PB3
        // 3: PB4
        // 4: PB5
        // 5: PB6
        // 12: PB13
        // 13: PB14
        // 14: AVI Version
        pstAVIParsingInfo->u8Version = pucInfoFrame[13];
        pstAVIParsingInfo->u8Length = HDMI_AVI_PACKET_LENGTH; // No length in register
        pstAVIParsingInfo->u8S10Value = pucInfoFrame[0] &BMASK(1:0);
        pstAVIParsingInfo->u8B10Value = (pucInfoFrame[0] &BMASK(3:2)) >> 2;
        pstAVIParsingInfo->u8A0Value = (pucInfoFrame[0] &BIT(4)) >> 4;
        pstAVIParsingInfo->u8Y210Value = (pucInfoFrame[0] &BMASK(7:5)) >> 5;
        pstAVIParsingInfo->u8R3210Value = pucInfoFrame[1] &BMASK(3:0);
        pstAVIParsingInfo->u8M10Value = (pucInfoFrame[1] &BMASK(5:4)) >> 4;
        pstAVIParsingInfo->u8C10Value = (pucInfoFrame[1] &BMASK(7:6)) >> 6;
        pstAVIParsingInfo->u8SC10Value = pucInfoFrame[2] &BMASK(1:0);
        pstAVIParsingInfo->u8Q10Value = (pucInfoFrame[2] &BMASK(3:2)) >> 2;
        pstAVIParsingInfo->u8EC210Value = (pucInfoFrame[2] &BMASK(6:4)) >> 4;
        pstAVIParsingInfo->u8ITCValue = (pucInfoFrame[2] &BIT(7)) >> 7;
        pstAVIParsingInfo->u8VICValue = pucInfoFrame[3] &BMASK(6:0);
        pstAVIParsingInfo->u8PR3210Value = pucInfoFrame[4] &BMASK(3:0);
        pstAVIParsingInfo->u8CN10Value = (pucInfoFrame[4] &BMASK(5:4)) >> 4;
        pstAVIParsingInfo->u8YQ10Value = (pucInfoFrame[4] &BMASK(7:6)) >> 6;
        pstAVIParsingInfo->u8ACE3210Value = (pucInfoFrame[13] &BMASK(7:4)) >> 4;
    }
    else
    {
        // Error handle.
        if(pstAVIParsingInfo->u16StructSize <= HDMI_AVI_PARSING_INFO_VERSION_1_SIZE)
        {
            MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

            // 0: PB1
            // 1: PB2
            // 2: PB3
            // 3: PB4
            // 4: PB5
            // 5: PB6
            // 12: PB13
            // 13: HB1
            pstAVIParsingInfo->u8Version = pucInfoFrame[13];
            pstAVIParsingInfo->u8Length = HDMI_AVI_PACKET_LENGTH; // No length in register
            pstAVIParsingInfo->u8S10Value = pucInfoFrame[0] &BMASK(1:0);
            pstAVIParsingInfo->u8B10Value = (pucInfoFrame[0] &BMASK(3:2)) >> 2;
            pstAVIParsingInfo->u8A0Value = (pucInfoFrame[0] &BIT(4)) >> 4;
            pstAVIParsingInfo->u8Y210Value = (pucInfoFrame[0] &BMASK(7:5)) >> 5;
            pstAVIParsingInfo->u8R3210Value = pucInfoFrame[1] &BMASK(3:0);
            pstAVIParsingInfo->u8M10Value = (pucInfoFrame[1] &BMASK(5:4)) >> 4;
            pstAVIParsingInfo->u8C10Value = (pucInfoFrame[1] &BMASK(7:6)) >> 6;
            pstAVIParsingInfo->u8SC10Value = pucInfoFrame[2] &BMASK(1:0);
            pstAVIParsingInfo->u8Q10Value = (pucInfoFrame[2] &BMASK(3:2)) >> 2;
            pstAVIParsingInfo->u8EC210Value = (pucInfoFrame[2] &BMASK(6:4)) >> 4;
            pstAVIParsingInfo->u8ITCValue = (pucInfoFrame[2] &BIT(7)) >> 7;
            pstAVIParsingInfo->u8VICValue = pucInfoFrame[3] &BMASK(6:0);
            pstAVIParsingInfo->u8PR3210Value = pucInfoFrame[4] &BMASK(3:0);
            pstAVIParsingInfo->u8CN10Value = (pucInfoFrame[4] &BMASK(5:4)) >> 4;
            pstAVIParsingInfo->u8YQ10Value = (pucInfoFrame[4] &BMASK(7:6)) >> 6;
        }
    }

    switch(pstAVIParsingInfo->u8Y210Value)
    {
        case 0x0:
            pstAVIParsingInfo->enColorForamt = MS_HDMI_COLOR_RGB;
            break;

        case 0x2:
            pstAVIParsingInfo->enColorForamt = MS_HDMI_COLOR_YUV_444;
            break;

        case 0x1:
            pstAVIParsingInfo->enColorForamt = MS_HDMI_COLOR_YUV_422;
            break;

        case 0x3:
            pstAVIParsingInfo->enColorForamt = MS_HDMI_COLOR_YUV_420;
            break;

        default:
            break;
    };

    if(pstAVIParsingInfo->enColorForamt == MS_HDMI_COLOR_RGB)  // RGB
    {
        switch(pstAVIParsingInfo->u8Q10Value)
        {
            case 0x0:
                //CEA spec VIC=1 640*480, is the only timeing of table 1,
                //the default color space shall be RGB using Full Range
                if(pstAVIParsingInfo->u8VICValue == 0x01)
                {
                    pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_FULL;
                }
                else if(pstAVIParsingInfo->u8VICValue == 0x00)
                {
                    pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_FULL;
                }
                else
                {
                    pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                }
                break;

            case 0x1:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                break;

            case 0x2:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_FULL;
                break;

            case 0x3:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                break;

            default:
                break;
        };
    }
    else // YUV
    {
        switch(pstAVIParsingInfo->u8YQ10Value)
        {
            case 0x0:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                break;

            case 0x1:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_FULL;
                break;

            case 0x2:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                break;

            case 0x3:
                pstAVIParsingInfo->enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                break;

            default:
                break;
        };
    }

    switch(pstAVIParsingInfo->u8C10Value)
    {
        case 0:
            if(pstAVIParsingInfo->enColorForamt != MS_HDMI_COLOR_RGB)
            {
                pstAVIParsingInfo->enColormetry = MS_HDMI_COLORIMETRY_NO_DATA;
            }
            break;

        case 0x1:
            if(pstAVIParsingInfo->enColorForamt != MS_HDMI_COLOR_RGB)
            {
                pstAVIParsingInfo->enColormetry = MS_HDMI_COLORIMETRY_SMPTE_170M;
            }
            break;

        case 0x2:
            if(pstAVIParsingInfo->enColorForamt != MS_HDMI_COLOR_RGB)
            {
                pstAVIParsingInfo->enColormetry = MS_HDMI_COLORIMETRY_ITU_R_BT_709;
            }
            break;

        case 0x3:
            bExtendColormetryFlag = TRUE;
            break;

        default:
            pstAVIParsingInfo->enColormetry = MS_HDMI_COLORIMETRY_UNKNOWN;
            break;
    };

    if(bExtendColormetryFlag)
    {
        switch(pstAVIParsingInfo->u8EC210Value)
        {
            case 0:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_XVYCC601;
                break;

            case 0x1:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_XVYCC709;
                break;

            case 0x2:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_SYCC601;
                break;

            case 0x3:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_ADOBEYCC601;
                break;

            case 0x4:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_ADOBERGB;
                break;

            case 0x5:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_BT2020YcCbcCrc;
                break;

            case 0x6:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_BT2020RGBYCbCr;
                break;

            case 0x7:
                if(pstAVIParsingInfo->u8ACE3210Value == 0x00)
                {
                    pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_D65;
                }
                else if(pstAVIParsingInfo->u8ACE3210Value == 0x01)
                {
                    pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_THEATER;
                }
                break;

            default:
                pstAVIParsingInfo->enColormetry = MS_HDMI_EXT_COLOR_UNKNOWN;
                break;
        };
    }

    pstAVIParsingInfo->enPixelRepetition = (MS_HDMI_PIXEL_REPETITION)pstAVIParsingInfo->u8PR3210Value;

    switch(pstAVIParsingInfo->u8M10Value)
    {
        case 0x0:
            pstAVIParsingInfo->enAspectRatio = HDMI_Pic_AR_NODATA;
            break;

        case 0x1:
            pstAVIParsingInfo->enAspectRatio = HDMI_Pic_AR_4x3;
            break;

        case 0x2:
            pstAVIParsingInfo->enAspectRatio = HDMI_Pic_AR_16x9;
            break;

        case 0x3:
        default:
            pstAVIParsingInfo->enAspectRatio = HDMI_Pic_AR_RSV;
            break;
    };

    switch(pstAVIParsingInfo->u8R3210Value)
    {
        case 0x2:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_16x9_Top;
            break;

        case 0x3:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_14x9_Top;
            break;

        case 0x4:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_GT_16x9;
            break;

        case 0x8:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_SAME;
            break;

        case 0x9:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_4x3_C;
            break;

        case 0xA:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_16x9_C;
            break;

        case 0xB:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_14x9_C;
            break;

        case 0xD:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_4x3_with_14x9_C;
            break;

        case 0xE:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_16x9_with_14x9_C;
            break;

        case 0xF:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_16x9_with_4x3_C;
            break;

        default:
            pstAVIParsingInfo->enActiveFormatAspectRatio = HDMI_AF_AR_SAME;
            break;
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_ParseVSInfoFrame()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_ParseVSInfoFrame(void *pPacket, stVS_PARSING_INFO *pstVSParsingInfo)
{
    if(pstVSParsingInfo->u16StructVersion == HDMI_VS_PARSING_INFO_VERSION)
    {
        MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

        // 0: HB0
        // 1: HB1
        // 2: HB2
        // 3: NULL
        // 4: PB0
        // 5: PB1
        // 6: PB2
        // 7: PB3
        // 8: PB4
        // 9: PB5
        // 10: PB6
        pstVSParsingInfo->u8Version = pucInfoFrame[1];
        pstVSParsingInfo->u8Length = pucInfoFrame[2] &BMASK(4:0);

        if((pucInfoFrame[7] == 0x00) && (pucInfoFrame[6] == 0x0C) && (pucInfoFrame[5] == 0x03))
        {
            pstVSParsingInfo->enHDMIVideoFormat = (E_HDMI_ADDITIONAL_VIDEO_FORMAT)(pucInfoFrame[8] &BMASK(7:5)) >> 5;

            if(pstVSParsingInfo->enHDMIVideoFormat == E_HDMI_4Kx2K_FORMAT)
            {
                pstVSParsingInfo->u8HDMIVIC = pucInfoFrame[9];
            }
            else if(pstVSParsingInfo->enHDMIVideoFormat == E_HDMI_3D_FORMAT)
            {
                pstVSParsingInfo->u83DStructure = (pucInfoFrame[9] &BMASK(7:4)) >> 4;
                pstVSParsingInfo->u83DExtData = (pucInfoFrame[10] &BMASK(7:4)) >> 4;
            }
        }
        else if((pucInfoFrame[7] == 0xC4) && (pucInfoFrame[6] == 0x5D) && (pucInfoFrame[5] == 0xD8))
        {
            if(pucInfoFrame[9] &BIT(1))
            {
                pstVSParsingInfo->u8ALLMMode = 1;
            }
        }
    }
    else
    {
        // Error handle.
        if(pstVSParsingInfo->u16StructSize <= HDMI_VS_PARSING_INFO_VERSION_1_SIZE)
        {
            MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

            // 0: HB0
            // 1: HB1
            // 2: HB2
            // 3: NULL
            // 4: PB0
            // 5: PB1
            // 6: PB2
            // 7: PB3
            // 8: PB4
            // 9: PB5
            // 10: PB6
            pstVSParsingInfo->u8Version = pucInfoFrame[1];
            pstVSParsingInfo->u8Length = pucInfoFrame[2] &BMASK(4:0);

            if((pucInfoFrame[7] == 0x00) && (pucInfoFrame[6] == 0x0C) && (pucInfoFrame[5] == 0x03))
            {
                pstVSParsingInfo->enHDMIVideoFormat = (E_HDMI_ADDITIONAL_VIDEO_FORMAT)(pucInfoFrame[8] &BMASK(7:5)) >> 5;

                if(pstVSParsingInfo->enHDMIVideoFormat == E_HDMI_4Kx2K_FORMAT)
                {
                    pstVSParsingInfo->u8HDMIVIC = pucInfoFrame[9];
                }
                else if(pstVSParsingInfo->enHDMIVideoFormat == E_HDMI_3D_FORMAT)
                {
                    pstVSParsingInfo->u83DStructure = (pucInfoFrame[9] &BMASK(7:4)) >> 4;
                    pstVSParsingInfo->u83DExtData = (pucInfoFrame[10] &BMASK(7:4)) >> 4;
                }
            }
        }
    }

    switch(pstVSParsingInfo->u8HDMIVIC)
    {
        case 0x1:
            pstVSParsingInfo->en4Kx2KVICCode = E_VIC_4Kx2K_30Hz;
            break;

        case 0x2:
            pstVSParsingInfo->en4Kx2KVICCode = E_VIC_4Kx2K_25Hz;
            break;

        case 0x3:
            pstVSParsingInfo->en4Kx2KVICCode = E_VIC_4Kx2K_24Hz;
            break;

        case 0x4:
            pstVSParsingInfo->en4Kx2KVICCode = E_VIC_4Kx2K_24Hz_SMPTE;
            break;

        default:
            pstVSParsingInfo->en4Kx2KVICCode = E_VIC_RESERVED;
            break;
    };

    switch(pstVSParsingInfo->u83DStructure)
    {
        case 0x0:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_FRAME_PACKING;
            break;

        case 0x1:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_FIELD_ALTERNATIVE;
            break;

        case 0x2:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_LINE_ALTERNATIVE;
            break;

        case 0x3:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_SIDE_BY_SIDE_FULL;
            break;

        case 0x4:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_L_DEPTH;
            break;

        case 0x5:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_L_DEPTH_GRAPHICS_GRAPHICS_DEPTH;
            break;

        case 0x6:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_TOP_BOTTOM;
            break;

        case 0x8:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_SIDE_BY_SIDE_HALF;
            break;

        default:
            pstVSParsingInfo->en3DStructure = E_XC_3D_INPUT_MODE_NONE;
            break;
    };

    switch(pstVSParsingInfo->u83DExtData)
    {
        case 0x0:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_HOR_SUB_SAMPL_0;
            break;

        case 0x1:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_HOR_SUB_SAMPL_1;
            break;

        case 0x2:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_HOR_SUB_SAMPL_2;
            break;

        case 0x3:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_HOR_SUB_SAMPL_3;
            break;

        case 0x4:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_QUINCUNX_MATRIX_0;
            break;

        case 0x5:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_QUINCUNX_MATRIX_1;
            break;

        case 0x6:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_QUINCUNX_MATRIX_2;
            break;

        case 0x7:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_QUINCUNX_MATRIX_3;
            break;

        default:
            pstVSParsingInfo->en3DExtendedData = E_3D_EXT_DATA_MODE_MAX;
            break;
    };
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_ParseGCInfoFrame()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void _mdrv_HDMI_ParseGCInfoFrame(void *pPacket, stGC_PARSING_INFO *pstGCParsingInfo)
{
    if(pstGCParsingInfo->u16StructVersion == HDMI_GC_PARSING_INFO_VERSION)
    {
        MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

        pstGCParsingInfo->u8ControlAVMute = pucInfoFrame[0] &BIT(0);
        pstGCParsingInfo->u8DefaultPhase = (pucInfoFrame[0] &BIT(1)) >> 1;
        pstGCParsingInfo->u8LastPPValue = (pucInfoFrame[0] &BMASK(4:2)) >> 2;
        pstGCParsingInfo->u8PreLastPPValue = (pucInfoFrame[0] &BMASK(7:5)) >> 5;
        pstGCParsingInfo->u8CDValue = pucInfoFrame[1] &BMASK(3:0);
        pstGCParsingInfo->u8PPValue = (pucInfoFrame[1] &BMASK(7:4)) >> 4;
    }
    else
    {
        // Error handle.
        if(pstGCParsingInfo->u16StructSize <= HDMI_GC_PARSING_INFO_VERSION_1_SIZE)
        {
            MS_U8 *pucInfoFrame = (MS_U8*)pPacket;

            pstGCParsingInfo->u8ControlAVMute = pucInfoFrame[0] &BIT(0);
            pstGCParsingInfo->u8DefaultPhase = (pucInfoFrame[0] &BIT(1)) >> 1;
            pstGCParsingInfo->u8LastPPValue = (pucInfoFrame[0] &BMASK(4:2)) >> 2;
            pstGCParsingInfo->u8PreLastPPValue = (pucInfoFrame[0] &BMASK(7:5)) >> 5;
            pstGCParsingInfo->u8CDValue = pucInfoFrame[1] &BMASK(3:0);
            pstGCParsingInfo->u8PPValue = (pucInfoFrame[1] &BMASK(7:4)) >> 4;
        }
    }

    switch(pstGCParsingInfo->u8CDValue)
    {
        case HDMI_GC_INFO_CD_NOT_INDICATED:
        case HDMI_GC_INFO_CD_24BITS:
            pstGCParsingInfo->enColorDepth = MS_HDMI_COLOR_DEPTH_8_BIT;
            break;

        case HDMI_GC_INFO_CD_30BITS:
            pstGCParsingInfo->enColorDepth = MS_HDMI_COLOR_DEPTH_10_BIT;
            break;

        case HDMI_GC_INFO_CD_36BITS:
            pstGCParsingInfo->enColorDepth = MS_HDMI_COLOR_DEPTH_12_BIT;
            break;

        case HDMI_GC_INFO_CD_48BITS:
            pstGCParsingInfo->enColorDepth = MS_HDMI_COLOR_DEPTH_16_BIT;
            break;

        default:
            pstGCParsingInfo->enColorDepth = MS_HDMI_COLOR_DEPTH_UNKNOWN;
            break;
    };
}

#ifdef MSOS_TYPE_LINUX_KERNEL
//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_ReLoadRxKeyThread
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_HDMI_ReLoadRxKeyThread(HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate)
{
    #define DER_TEE_SUPPLICANT_TIMEOUT_CNT 10000
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    MS_U32 u32InTime = MsOS_GetSystemTime();
    while((tee_get_supplicant_count()<1) && ((MsOS_GetSystemTime() - u32InTime )<DER_TEE_SUPPLICANT_TIMEOUT_CNT));  // wait for TEE supplicant
    // STR resume flow
    if(pHDMIRxResourcePrivate->fpHdcp14RxResumeCBFunc != NULL)
    {
        pHDMIRxResourcePrivate->fpHdcp14RxResumeCBFunc(1, NULL); //Rx:1, Tx:0
    }
    if(pHDMIRxResourcePrivate->fpHdcp22RxResumeCBFunc != NULL)
    {
        pHDMIRxResourcePrivate->fpHdcp22RxResumeCBFunc(1, NULL); //Rx:1, Tx:0
    }

    // Delay 100ms
    if(pHDMIRxResourcePrivate->u8HotplugDelay != 0)
    {
        MsOS_DelayTask((pHDMIRxResourcePrivate->u8HotplugDelay)*100);
    }
    else
    {
        MsOS_DelayTask(100);
    }
    // Set all HPD
    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
    {
        if(pHDMIRxResourcePrivate->bHDCP22EnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
        {
            Hal_HDCP22_PortInit(HDMI_GET_PORT_SELECT(enInputPortType));
            Hal_HDCP_WriteDoneInterruptEnable(enInputPortType, TRUE);

            pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = TRUE;
        }
        if(pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc != NULL)
        {
            Hal_HDCP_ReadR0InterruptEnable(enInputPortType, TRUE);
        }

        Hal_DVI_ClkPullLow(FALSE, enInputPortType);
        Hal_HDMI_DataRtermControl(enInputPortType, TRUE);
        Hal_HDMI_pullhpd(TRUE, enInputPortType, pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)], pHDMIRxResourcePrivate->ucMHLSupportPath);

        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bNoInputFlag = TRUE;
        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bPowerOnLane = FALSE;
        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag = FALSE;
        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bYUV420Flag = FALSE;
        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource = Hal_HDMI_GetSourceSelect(enInputPortType);
    }
    //MsOS_DeleteTask(pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID);
    pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID = -1;

}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDMI_CreateReLoadKeyTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL _mdrv_HDMI_CreateReLoadKeyTask(HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate)
{
    MS_BOOL bCreateSuccess = TRUE;

    pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID = MsOS_CreateTask((TaskEntry) _mdrv_HDMI_ReLoadRxKeyThread,
                                     (MS_VIRT) pHDMIRxResourcePrivate,
                                     E_TASK_PRI_MEDIUM,
                                     TRUE,
                                     NULL,
                                     0,
                                     "HDMI_HDCP_RxReLoaded_Key Task");

    if(pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID < 0)
    {
        MDRV_HDMIRX_MSG_ERROR("MsOS_CreateTask RxLoadkey failed!!\n");

        if(MsOS_DeleteEventGroup(pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID))
        {
            //for coverity Unchecked return value
        }

        bCreateSuccess = FALSE;
    }

    return bCreateSuccess;
}

//**************************************************************************
//  [Function Name]:
//                   _mdrv_HDMI_RebootHandler()
//  [Description]
//                  HDMI reboot handler
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static int _mdrv_HDMI_RebootHandler(struct notifier_block* nb, unsigned long event, void *unused)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    PM_WakeCfg stData;

    switch(event)
    {
        case SYS_POWER_OFF:
            UtopiaModuleGetPtr(MODULE_HDMIRX, &pModule);

            if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
            }
            else
            {
                if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
                {
                    MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
                }
                else
                {
                    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
                    {
                        Hal_HDMI_ForcePowerDownPort(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)], TRUE);
                    }

                    MDrv_PM_Read_Key(PM_KEY_DEFAULT, (const char *)&stData);
                    if (stData.bPmWakeEnableDVI0)
                    {
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI0);
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI1);
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI3);
                    }
                    if (stData.bPmWakeEnableDVI2)
                    {
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI2);
                    }

                    MDRV_HDMIRX_MSG_INFO("** HDMI config reboot handler\r\n");
                }
            }

            UtopiaResourceRelease(pHDMIRxResource);
            break;

        default:
            break;
    };

    return NOTIFY_OK;
}

static struct notifier_block _mdrv_HDMI_reboot_notifier = {
    .notifier_call = _mdrv_HDMI_RebootHandler,
};

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDCP_InterruptTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_HDCP_InterruptTask(void)
{
    MS_BOOL bHDCPTaskProcFlag = TRUE;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    stHDCP22_HANDLER XCArgs = {0};
    stHDCP22_WAIT_EVENT stHDMIRxArgs = {0};

    XCArgs.bHdcp22RecvMsgFlag = FALSE;
    XCArgs.bIRQModeFlag = TRUE;
    XCArgs.ucHdcp22PortType = E_HDCP22_IF_HDMI;
    XCArgs.ucHdcp22MsgLen = 0;
    stHDMIRxArgs.ucHDCPWriteDoneIndex = 0;

    if(slHDCPMutexID != -1)
    {
        MsOS_ObtainMutex(slHDCPMutexID, MSOS_WAIT_FOREVER);
    }

    while(bHDCPTaskProcFlag)
    {
        //MDRV_HDMIRX_MSG_INFO("** TMDS enter wait event kernel space !!!!!!\r\n");

        if(pulHDMIRxInst == NULL)
        {
            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
        }
        else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_WAIT_EVENT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }

        //MDRV_HDMIRX_MSG_INFO("** TMDS wait event happen user space !!!!!!\r\n");

        for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
        {
            if(stHDMIRxArgs.ucHDCPWriteDoneIndex &BIT(HDMI_GET_PORT_SELECT(enInputPortType)))
            {
                XCArgs.ucPortIdx = HDMI_GET_PORT_SELECT(enInputPortType);

                if(pulHDMIRxInst == NULL)
                {
                    MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
                }
                else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_HANDLER, (void*)&XCArgs) != 0)
                {
                    MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                }

                if(XCArgs.bHdcp22RecvMsgFlag)
                {
                    if (fpHdcp22RecMsgCBFunc != NULL)
                    {
                        fpHdcp22RecMsgCBFunc(XCArgs.ucHdcp22PortType, HDMI_GET_PORT_SELECT(enInputPortType), XCArgs.pucHdcp22MsgData, XCArgs.ucHdcp22MsgLen, NULL);
                    }
                }
            }
        }

        bHDCPTaskProcFlag = bHDCPInterruptProcFlag;
    }

    if(bHDCPInterruptCreateFlag)
    {
        slHDCPInterruptTaskID = -1;
        bHDCPInterruptCreateFlag = FALSE;
    }

    if(slHDCPMutexID != -1)
    {
        MsOS_ReleaseMutex(slHDCPMutexID);
    }
}

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDCP_CreateInterruptTask
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL _mdrv_HDCP_CreateInterruptTask(void)
{
    MS_BOOL bCreateSuccess = TRUE;

    if(slHDCPInterruptTaskID < 0)
    {
        slHDCPInterruptTaskID = MsOS_CreateTask((TaskEntry) _mdrv_HDCP_InterruptTask,
                                     (MS_VIRT) NULL,
                                     E_TASK_PRI_MEDIUM,
                                     TRUE,
                                     (void *)ucHDCPInterruptStack,
                                     HDMI_HDCP_INTERRUPT_STACK_SIZE,
                                     "HDCP int task");

        bHDCPInterruptProcFlag = TRUE;
    }

    if(slHDCPInterruptTaskID < 0)
    {
        MDRV_HDMIRX_MSG_ERROR("MsOS_CreateTask failed!!\n");

        if(MsOS_DeleteEventGroup(slHDCPInterruptTaskID))
        {
            //for coverity Unchecked return value
        }

        bCreateSuccess = FALSE;
    }

    return bCreateSuccess;
}

#endif

//**************************************************************************
//  [Function Name]:
//                  _mdrv_HDCP_IsrHandler()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_HDCP_IsrHandler(InterruptNum eIntNum)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetResourceFlag = FALSE;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaModuleGetPtr(MODULE_HDMIRX, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        bGetResourceFlag = TRUE;
    }

    if(eIntNum > 0)
    {
        // Do nothing
    }

    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
    {
        if(Hal_HDCP22_PollingWriteDone(HDMI_GET_PORT_SELECT(enInputPortType)) && bGetResourceFlag)
        {
            pHDMIRxResourcePrivate->ucHDCPWriteDoneIndex |= BIT(HDMI_GET_PORT_SELECT(enInputPortType));

            if(GET_HDMI_FLAG(pHDMIRxResourcePrivate->ucHDCPReadDoneIndex, BIT(HDMI_GET_PORT_SELECT(enInputPortType))))
            {
                CLR_HDMI_FLAG(pHDMIRxResourcePrivate->ucHDCPReadDoneIndex, BIT(HDMI_GET_PORT_SELECT(enInputPortType)));
            }
        }

        if(Hal_HDCP22_PollingReadDone(HDMI_GET_PORT_SELECT(enInputPortType)) && bGetResourceFlag)
        {
            if(pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[HDMI_GET_PORT_SELECT(enInputPortType)])
            {
                Hal_HDCP22_SetReadyBit(HDMI_GET_PORT_SELECT(enInputPortType), FALSE); //clear ready bit after source read re

                pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
            }

            SET_HDMI_FLAG(pHDMIRxResourcePrivate->ucHDCPReadDoneIndex, BIT(HDMI_GET_PORT_SELECT(enInputPortType)));
        }

        if(Hal_HDCP14_PollingR0ReadDone(HDMI_GET_PORT_SELECT(enInputPortType)) && bGetResourceFlag)
        {

            if(pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc != NULL)
            {
                pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc(1, HDMI_GET_PORT_SELECT(enInputPortType), NULL);
            }
        }
    }

    if(bGetResourceFlag)
    {
        if(pHDMIRxResourcePrivate->ucHDCPWriteDoneIndex > 0)
        {
            MsOS_SetEvent(pHDMIRxResourcePrivate->slHDMIHDCPEventID, HDMI_HDCP_EVENT_WRITE_DONE);
        }
    }

    MsOS_EnableInterrupt(E_INT_IRQ_DVI_HDMI_HDCP);

    UtopiaResourceRelease(pHDMIRxResource);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDCP22_FillCipherKey(MS_U8 ucPortIdx, MS_U8* pucRiv, MS_U8* pucSessionKey)
{
    stHDCP22_FILL_CIPHER_KEY XCArgs;

    XCArgs.ucPortIdx = ucPortIdx;
    XCArgs.pucRiv = pucRiv;
    XCArgs.pucSessionKey = pucSessionKey;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_FILL_CIPHER_KEY, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

void MDrv_HDCP22_InitCBFunc(HDCP22_Recv_CBF pCBFunc, void* pContext)
{
    if(pCBFunc != NULL)
    {
        fpHdcp22RecMsgCBFunc = pCBFunc;
    }
}

void MDrv_HDCP22_PortInit(MS_U8 ucPortIdx)
{
    stHDCP22_PORT_INIT XCArgs = {0};

    XCArgs.ucPortIdx = ucPortIdx;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_PORT_INIT, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

MS_BOOL MDrv_HDCP22_PollingReadDone(MS_U8 ucPortIdx)
{
    stHDCP22_POLLING_READ_DONE XCArgs = {0};

    XCArgs.ucPortIdx = ucPortIdx;
    XCArgs.bReturnValue = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_POLLING_READ_DONE, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return XCArgs.bReturnValue;
}

void MDrv_HDCP22_EnableCipher(MS_U8 ucPortType, MS_U8 ucPortIdx, MS_BOOL bIsEnable)
{
    stHDCP22_ENABLE_CIPHER XCArgs = {0};

    XCArgs.ucPortType = ucPortType;
    XCArgs.ucPortIdx = ucPortIdx;
    XCArgs.bIsEnable = bIsEnable;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_ENABLE_CIPHER, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

void MDrv_HDCP22_SendMsg(MS_U8 ucPortType, MS_U8 ucPortIdx, MS_U8* pucData, MS_U32 dwDataLen, void* pDummy)
{
    stHDCP22_SEND_MESSAGE XCArgs;

    XCArgs.ucPortType = ucPortType;
    XCArgs.ucPortIdx = ucPortIdx;
    XCArgs.pucData = pucData;
    XCArgs.dwDataLen = dwDataLen;
    XCArgs.pDummy = pDummy;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_SEND_MESSAGE, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

void MDrv_HDCP22_Handler(MS_U8 ucPortIdx)
{
    stHDCP22_HANDLER XCArgs = {0};

    XCArgs.bHdcp22RecvMsgFlag = FALSE;
    XCArgs.bIRQModeFlag = FALSE;
    XCArgs.ucPortIdx = ucPortIdx;
    XCArgs.ucHdcp22PortType = E_HDCP22_IF_HDMI;
    XCArgs.ucHdcp22MsgLen = 0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_HANDLER, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    if(XCArgs.bHdcp22RecvMsgFlag)
    {
        if (fpHdcp22RecMsgCBFunc != NULL)
        {
            fpHdcp22RecMsgCBFunc(XCArgs.ucHdcp22PortType, ucPortIdx, XCArgs.pucHdcp22MsgData, XCArgs.ucHdcp22MsgLen, NULL);
        }
    }
}

#endif

//**************************************************************************
//  [Function Name]:
//                  _mdrv_SCDC_IsrHandler()
//  [Description]
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
static void _mdrv_SCDC_IsrHandler(InterruptNum eIntNum)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetResourceFlag = FALSE;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaModuleGetPtr(MODULE_HDMIRX, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        bGetResourceFlag = TRUE;
    }

    if(eIntNum > 0)
    {
        // Do nothing
    }

    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
    {
        if(bGetResourceFlag)
        {
            Hal_HDMI_SCDCIsr(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)]);
        }
    }

    MsOS_EnableInterrupt(E_INT_IRQ_SCDC_PM_INT);

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
/// report which patch function hdmi needs
/// @return @ref MS_U16
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_HDMI_Func_Caps_U2(void* pInstance)
{
    return Hal_HDMI_Func_Caps();
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U16 MDrv_HDMI_Func_Caps(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return 0;
    }

    stHDMI_FUNC_CAPS XCArgs;
    XCArgs.u16ReturnValue = 0;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_FUNC_CAPS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return 0;
    }
    else
    {
        return XCArgs.u16ReturnValue;
    }
}

//-------------------------------------------------------------------------------------------------
/// Check the HDCP is exist or not
/// @return @ref MS_BOOL
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_HDCP_isexist(void)
{
    MS_HDCP_STATUS_INFO_t* enStatus_info;
    enStatus_info = MDrv_HDCP_GetStatus();

    if(enStatus_info->b_St_Reserve_1 && enStatus_info->b_St_ENC_En)
    {
        MDRV_HDMIRX_MSG_INFO("HDCP is exit");
        return TRUE;
    }
    else
    {
        MDRV_HDMIRX_MSG_INFO("No HDCP");
        return FALSE;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// DDC clock on/off control
/// @param  bEnable      \b IN: ON/OFF control
//-------------------------------------------------------------------------------------------------
void MDrv_HDCP_Enable_U2(void* pInstance, MS_BOOL bEnable)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        Hal_HDCP_ddc_en(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[MAIN_WINDOW].enInputSourceType), bEnable);
    }
    else if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
    {
        Hal_HDCP_ddc_en(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[SUB_WINDOW].enInputSourceType), bEnable);
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#else
    Hal_HDCP_ddc_en(bEnable);
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDCP_Enable(MS_BOOL bEnable)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDCP_ENABLE XCArgs;
    XCArgs.bEnable = bEnable;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDCP_IF_CMD_ENABLE, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// DDC clock on/off control
/// @param  enInputPortType  \b IN: Input source selection
/// @param  bEnable          \b IN: ON/OFF control
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_HDMI_SetHdcpEnable_U2(void* pInstance, E_MUX_INPUTPORT enInputPortType,MS_BOOL bEnable)
{
    /*Only DVI port can enable/disable hdcp key*/
    if(enInputPortType < INPUT_PORT_DVI0 ||  enInputPortType > INPUT_PORT_DVI3)
    {
        return FALSE;
    }

#if SUPPORT_HDMI_RX_NEW_FEATURE
    Hal_HDCP_ddc_en(enInputPortType, bEnable);
#else
    enInputPortType = enInputPortType;
    Hal_HDCP_ddc_en(bEnable);
#endif
    return TRUE;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_SetHdcpEnable(E_MUX_INPUTPORT enInputPortType,MS_BOOL bEnable)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_SET_HDCPENABLE XCArgs;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bEnable = bEnable;
    XCArgs.bReturnValue = FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SET_HDCP_ENABLE, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bReturnValue;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Check the ending of the Vsync
/// @param  bStable      \b IN: Stable or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDCP_Vsync_end_en_U2(void* pInstance, MS_BOOL bStable)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(bStable)
    {
        if(!pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_count)
        {
            pXCResourcePrivate->stdrv_HDMI.u32VsyncTimer = MsOS_GetSystemTime();
            pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_count = TRUE;
        }
        if ((pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_done) && ((MsOS_GetSystemTime() - pXCResourcePrivate->stdrv_HDMI.u32VsyncTimer) > 4500) )
        {
            Hal_HDCP_Vsync_end_en(TRUE);
            pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_done = FALSE;
        }
    }
    else
    {
        if (!pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_done)
        {
            Hal_HDCP_Vsync_end_en(FALSE);
            pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_done = TRUE;
            pXCResourcePrivate->stdrv_HDMI._bHDCP_Vsync_count = FALSE;
        }
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDCP_Vsync_end_en(MS_BOOL bStable)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDCP_VSYNC_END_EN XCArgs;
    XCArgs.bStable = bStable;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDCP_IF_CMD_VSYNC_END_EN, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Exit the HDMI and turn off the related setting of HDMI
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_Exit_U2( void* pInstance )
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        Hal_HDMI_exit(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[MAIN_WINDOW].enInputSourceType));
    }
    else if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
    {
        Hal_HDMI_exit(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[SUB_WINDOW].enInputSourceType));
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#else
    Hal_HDMI_exit();
#endif
#if 0
    if(g_bIMMESWITCH_DVI_POWERSAVING)
        memset(u8PSDVIStableCount, 0, 4*sizeof(MS_U8));
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Exit( void )
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_EXIT, NULL) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Init the setting for HDMI
//-------------------------------------------------------------------------------------------------
stHDMI_INITIAL_TABLE MDrv_HDMI_init_internal(void *pInstance)
{
    stHDMI_INITIAL_TABLE stInitialTable = {0};
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    pXCResourcePrivate->stdrv_HDMI._info.u8SupportPorts = 0;
    //pXCResourcePrivate->stdrv_HDMI._status.bIsInitialized = TRUE;
    pXCResourcePrivate->stdrv_HDMI._status.bIsRunning     = TRUE;

    bInitEnable = TRUE;

    if (SUPPORT_IMMESWITCH)
    {
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH = (pXCResourcePrivate->stdrvXC_MVideo_Context.g_XC_Init_Misc.u32MISC_A & E_XC_INIT_MISC_A_IMMESWITCH);
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIMMESWITCH_DVI_POWERSAVING = (pXCResourcePrivate->stdrvXC_MVideo_Context.g_XC_Init_Misc.u32MISC_A & E_XC_INIT_MISC_A_IMMESWITCH_DVI_POWERSAVING) >> 1;
    }
    else
    {
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH = FALSE;
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIMMESWITCH_DVI_POWERSAVING = FALSE;
    }
    #ifdef K3_U2
    if (SUPPORT_DVI_AUTO_EQ)
    {
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bDVI_AUTO_EQ = (pXCResourcePrivate->stdrvXC_MVideo_Context.g_XC_Init_Misc.u32MISC_A & E_XC_INIT_MISC_A_DVI_AUTO_EQ) >> 2;
    }
    else
    {
        pXCResourcePrivate->stdrvXC_MVideo_Context.g_bDVI_AUTO_EQ = FALSE;
    }
    #else
    pXCResourcePrivate->stdrvXC_MVideo_Context.g_bDVI_AUTO_EQ = SUPPORT_DVI_AUTO_EQ;
    #endif

    MDRV_HDMIRX_MSG_INFO("[%s][%d]pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH=%x\n", __FUNCTION__, __LINE__,
            pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH);
    MDRV_HDMIRX_MSG_INFO("[%s][%d]pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIMMESWITCH_DVI_POWERSAVING=%x\n", __FUNCTION__, __LINE__,
            pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIMMESWITCH_DVI_POWERSAVING);

    if(!pXCResourcePrivate->stdrv_HDMI._status.bIsInitialized)
    {
        Hal_HDMI_init(pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH, pXCResourcePrivate->stdrvXC_MVideo_Context._u8MhlSupportInfo);

        pXCResourcePrivate->stdrv_HDMI._status.bIsInitialized = TRUE;
    }

    stInitialTable.bImmeswitchSupport = pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH;
    stInitialTable.ucMHLSupportPath = pXCResourcePrivate->stdrvXC_MVideo_Context._u8MhlSupportInfo;

    return stInitialTable;
}

stHDMI_INITIAL_TABLE MDrv_HDMI_init_U2( void* pInstance )
{
    stHDMI_INITIAL_TABLE stInitialTable = {0};
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    stInitialTable = MDrv_HDMI_init_internal(pInstance);

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    return stInitialTable;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_init( void )
{
    MS_BOOL bInitialFlag = FALSE;
    stHDMI_INITIAL_SETTING XCArgs;

    XCArgs.bCreateHDCPTaskFlag = FALSE;
    XCArgs.stInitialTable.bImmeswitchSupport = FALSE;
    XCArgs.stInitialTable.ucMHLSupportPath = 0;

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_INIT, (void*)&XCArgs) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("UtopiaOpen pulHDMIRxInst failed\n");

            bInitialFlag = TRUE;
        }
    }

    if(!bInitialFlag)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_INITIAL, (void*)&XCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    if(XCArgs.bCreateHDCPTaskFlag)
    {
        slHDCPMutexID = MsOS_CreateMutex(E_MSOS_FIFO, "HDCP_Mutex", MSOS_PROCESS_SHARED);

        if(!bHDCPInterruptCreateFlag)
        {
            _mdrv_HDCP_CreateInterruptTask();

            bHDCPInterruptCreateFlag = TRUE;
        }
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Set the equalizer volumn for HDMI
/// @param  enEq      \b IN: ON/OFF the equlizer
/// @param  u8EQValue      \b IN: The volumn of equalizer
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_Set_EQ_U2(void* pInstance, MS_HDMI_EQ enEq, MS_U8 u8EQValue)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    INPUT_SOURCE_TYPE_t src;
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    for(src = INPUT_SOURCE_HDMI; src < INPUT_SOURCE_HDMI_MAX; src++)
    {
        Hal_HDMI_Set_EQ(MDrv_XC_Mux_GetHDMIPort(pInstance, src), enEq, u8EQValue);
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#else
    Hal_HDMI_Set_EQ(enEq, u8EQValue);
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Set_EQ(MS_HDMI_EQ enEq, MS_U8 u8EQValue)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_SET_EQ XCArgs;
    XCArgs.enEq = enEq;
    XCArgs.u8EQValue = u8EQValue;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SET_EQ, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Set the equalizer volumn for HDMI to single port
/// @param  enEq      \b IN: ON/OFF the equlizer
/// @param  u8EQValue      \b IN: The volumn of equalizer
/// @param  enInputPortType      \b IN: The appointed port
/// @return @ref MS_BOOL
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_HDMI_Set_EQ_To_Port_U2(void* pInstance, MS_HDMI_EQ enEq, MS_U8 u8EQValue, E_MUX_INPUTPORT enInputPortType)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    Hal_HDMI_Set_EQ(enInputPortType, enEq, u8EQValue);
    return TRUE;
#else
    MDRV_HDMIRX_MSG_INFO("[Warning]Unsupport set EQ to single port!");
    return FALSE;
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_Set_EQ_To_Port(MS_HDMI_EQ enEq, MS_U8 u8EQValue, E_MUX_INPUTPORT enInputPortType)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_SET_EQ_TO_PORT XCArgs;
    XCArgs.enEq = enEq;
    XCArgs.u8EQValue = u8EQValue;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bReturnValue = FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SET_EQ_TO_PORT, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bReturnValue;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Enable the Audio mute event for Vivaldi
/// @param  u16MuteEvent      \b IN: The mute event
/// @param  u16MuteMask       \b IN: The mute mask
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_Audio_MUTE_Enable_U2(void* pInstance, MS_U16 u16MuteEvent,MS_U16 u16MuteMask)
{
    Hal_HDMI_Audio_MUTE_Enable(u16MuteEvent, u16MuteMask);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Audio_MUTE_Enable(MS_U16 u16MuteEvent,MS_U16 u16MuteMask)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_AUDIO_MUTE_ENABLE XCArgs;
    XCArgs.u16MuteEvent = u16MuteEvent;
    XCArgs.u16MuteMask = u16MuteMask;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_AUDIO_MUTE_ENABLE, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Clear the Audio mute status.
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_Audio_Status_Clear_U2(void* pInstance)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        MS_U8 ucAudioMux = Hal_HDMI_Get_Audio_Mux();
        Hal_HDMI_Audio_Status_Clear(&pHDMIRxResourcePrivate->stHDMIPollingInfo[ucAudioMux]);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Audio_Status_Clear(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_AUDIO_STATUS_CLEAR, NULL) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Control the Hot Plug Detection
/// in HIGH Level, the voltage level is 2.4 ~ 5.3V
/// in LOW Level,  the voltage level is 0 ~ 0.4V
/// Enable hotplug GPIO[0] output and set output value
/// @param  bHighLow                \b IN: High/Low control
/// @param  enInputPortType      \b IN: Input source selection
/// @param  bInverse                 \b IN: Inverse or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_pullhpd_internal(void *pInstance, MS_BOOL bHighLow, E_MUX_INPUTPORT enInputPortType, MS_BOOL bInverse)
{
    MDRV_HDMIRX_MSG_INFO("src hpd = %x \n",enInputPortType);
    // Fix me. Do not use HDMI_PORT_ALL and HDMI_PORT_A.
    // use E_MUX_INPUTPORT.
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    pXCResourcePrivate->stdrvXC_MVideo_Context.g_bHDMIInverseBackUp[enInputPortType - INPUT_PORT_DVI0] = bInverse;

    Hal_HDMI_pullhpd(bHighLow, enInputPortType, bInverse, pXCResourcePrivate->stdrvXC_MVideo_Context._u8MhlSupportInfo);
}

void MDrv_HDMI_pullhpd_U2( void* pInstance, MS_BOOL bHighLow, E_MUX_INPUTPORT enInputPortType, MS_BOOL bInverse)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;

    //MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Enter  enInputPortType= %d, bHighLow=%x, tick=%d \n",__FUNCTION__,__LINE__, enInputPortType, bHighLow , MsOS_GetSystemTime());

    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    MDrv_HDMI_pullhpd_internal(pInstance, bHighLow, enInputPortType, bInverse);

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    //MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Exit  enInputPortType= %d, bHighLow=%x, tick=%d \n",__FUNCTION__,__LINE__, enInputPortType, bHighLow , MsOS_GetSystemTime());
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_pullhpd( MS_BOOL bHighLow, E_MUX_INPUTPORT enInputPortType, MS_BOOL bInverse)
{
    stHDMI_PULLHPD XCArgs = {0};
    stHDMI_SET_HPD_INVERT_FLAG stHDMIRxArgs = {0};

    stHDMIRxArgs.enInputPortType = enInputPortType;
    stHDMIRxArgs.bInverse = bInverse;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_HPD_INVERT_FLAG, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    XCArgs.bHighLow = bHighLow;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bInverse = stHDMIRxArgs.bInverse;

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_PULLHPD, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U16 MDrv_HDMI_GC_Info(HDMI_GControl_INFO_t gcontrol)
{
    stHDMI_GC_INFO stHDMIRxArgs = {gcontrol, 0, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_GC_PACKET_INFO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u16ReturnValue;
}

#endif

//-------------------------------------------------------------------------------------------------
/// HDMI PLL control setting 1
/// @param  pllctrl                \b IN: PLL control
/// @param  bread                \b IN: Read/Write (1/0)
/// @param  u16value           \b IN: Write value
/// @return @ref MS_U16
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_HDMI_pll_ctrl1(HDMI_PLL_CTRL_t pllctrl, MS_BOOL bread, MS_U16 u16value)
{
    return (Hal_HDMI_pll_ctrl1(pllctrl, bread, u16value));
}
//-------------------------------------------------------------------------------------------------
/// HDMI PLL control setting 2
/// @param  pllctrl                \b IN: PLL control
/// @param  bread                \b IN: Read/Write (1/0)
/// @param  u16value           \b IN: Write value
/// @return @ref MS_U16
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_HDMI_pll_ctrl2(HDMI_PLL_CTRL2_t pllctrl, MS_BOOL bread, MS_U16 u16value)
{
    return (Hal_HDMI_pll_ctrl2(pllctrl, bread, u16value));
}

////////////////////////

/******************************************************************************/
/// for DVI
/******************************************************************************/
//-------------------------------------------------------------------------------------------------
/// Check the clock is less 70MHz or not with DVI source
/// @param  enInputPortType                \b IN: Input source
/// @return @ref MS_BOOL
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_HDMI_SetForClockLessThan70Mhz_U2( void* pInstance, E_MUX_INPUTPORT enInputPortType )
{
    switch(enInputPortType)
    {
        default:
        case INPUT_PORT_DVI0:
        case INPUT_PORT_DVI1:
        case INPUT_PORT_DVI3:
            return Hal_DVI_clk_lessthan70mhz_usingportc(FALSE);
        case INPUT_PORT_DVI2:
            return Hal_DVI_clk_lessthan70mhz_usingportc(TRUE);
    }
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_SetForClockLessThan70Mhz( E_MUX_INPUTPORT enInputPortType )
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_SETFORCLOCKLESSTHAN70MHZ XCArgs;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bReturnValue = FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SETFORCLOCKLESSTHAN70MHZ, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bReturnValue;
    }
}

#endif

void MDrv_DVI_SoftwareReset_U2( void* pInstance, MS_U16 u16Reset )
{
    Hal_DVI_sw_reset(u16Reset);

}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_DVI_SoftwareReset( MS_U16 u16Reset )
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stDVI_SOFTWARERESET XCArgs;
    XCArgs.u16Reset = u16Reset;

    if(UtopiaIoctl(pu32XCInst, E_XC_DVI_IF_CMD_SOFTWARERESET, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Detect the the clock is lose or not from ADC DTOP
/// @param  enInputPortType                \b IN: Input source
/// @return @ref MS_BOOL
//-------------------------------------------------------------------------------------------------
MS_BOOL MDrv_HDMI_dvi_clock_detect(E_MUX_INPUTPORT enInputPortType)
{
    MS_BOOL bIsLoss;

#if SUPPORT_HDMI_RX_NEW_FEATURE
    bIsLoss = Hal_DVI_clklose_det(enInputPortType);
#else
    if (enInputPortType == INPUT_PORT_DVI2)
    {
        bIsLoss = Hal_DVI_clklose_det(TRUE);
    }
    else
    {
        bIsLoss = Hal_DVI_clklose_det(FALSE);
    }
#endif

    return bIsLoss;
}

//-------------------------------------------------------------------------------------------------
/// Reset the DVI setting
//-------------------------------------------------------------------------------------------------
void mdrv_dvi_reset_U2(void* pInstance)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    INPUT_SOURCE_TYPE_t enInputSourceType; // For coverity unused value
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        enInputSourceType = gSrcInfo[MAIN_WINDOW].enInputSourceType;
    }
    else if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
    {
        enInputSourceType = gSrcInfo[SUB_WINDOW].enInputSourceType;
    }
    else
    {
        _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
        return;
    }
    if(Hal_DVI_irq_info(MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType), IRQ_DVI_CK_CHG) && (!pXCResourcePrivate->stdrv_HDMI._ResetDone))
    {
        MS_U16 u8_IRQ_STATUS;

        u8_IRQ_STATUS = Hal_DVI_irq_info(MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType), IRQ_DVI_CK_CHG);
        Hal_DVI_irq_clear(MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType), u8_IRQ_STATUS);
        Hal_DVI_irq_clear(MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType), (MS_U16)(~IRQ_ALL_BIT));

        if (u8_IRQ_STATUS == IRQ_DVI_CK_CHG)
        {
            // not do DVI software reset at here; Avoid the snow image

            pXCResourcePrivate->stdrv_HDMI._ResetDone = TRUE;
        }
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#else
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(Hal_DVI_irq_info(IRQ_DVI_CK_CHG) && (!pXCResourcePrivate->stdrv_HDMI._ResetDone))
    {
        MS_U16 u8_IRQ_STATUS;

        u8_IRQ_STATUS = Hal_DVI_irq_info(IRQ_DVI_CK_CHG);
        Hal_DVI_irq_clear(u8_IRQ_STATUS);
        Hal_DVI_irq_clear(~IRQ_ALL_BIT);

        if (u8_IRQ_STATUS == IRQ_DVI_CK_CHG)
        {
            // not do DVI software reset at here; Avoid the snow image

            pXCResourcePrivate->stdrv_HDMI._ResetDone = TRUE;
        }
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void mdrv_dvi_reset(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    if(UtopiaIoctl(pu32XCInst, E_XC_DVI_IF_CMD_RESET, NULL) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

#ifdef DVI_NEW_Patch
// 20081226 - DVI+HDCP snow noise patch - start ...
//-------------------------------------------------------------------------------------------------
/// avoid the snow image while AC on/off and source change for DVI 70Mhz issue, setting 1
/// @param  enInputPortType                \b IN: Input source
//-------------------------------------------------------------------------------------------------
void mdrv_dvi_clock_70mhz_swpatch1(void *pInstance, INPUT_SOURCE_TYPE_t enInputSourceType)
{
    MS_BOOL bClockLess70MHz;
    E_MUX_INPUTPORT enMuxPort =MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType);

//    if(MDrv_ReadByte( H( REG_HDMI_INT_STATUS) ) & BIT5)
    if(R2BYTEMSK(REG_HDMI_61_L, BIT13))
    {
        #if ( WATCH_DOG == ENABLE )
            MDrv_Sys_ClearWatchDog();
        #endif

        switch(enMuxPort)
        {
        default:
        case INPUT_PORT_DVI0:
        case INPUT_PORT_DVI1:
        case INPUT_PORT_DVI3:
//            if(!(MDrv_ReadByte(H_BK_ADC_DTOP(0x57)) & BIT3)) // DVI clock is stable
            if(!Hal_DVI_clk_stable_usingportc(NO)) // DVI clock is stable
            {
                //MDRV_HDMIRX_MSG_INFO("\r\nAaron - HDCP status = %bx\r\n", MDrv_ReadByte(L_BK_ADC_DTOP(0x79)));
//                if(!(R2BYTEMSK(0x2C33, BIT5))) // Video clock is unstable
                if(!(R2BYTEMSK(0x2C32, BIT13))) // Video clock is unstable
                {
                    g_DVI70MhzUnstableCounter++;
                    if(g_DVI70MhzUnstableCounter >= 2)
                    {
                        g_DVI70MhzUnstableCounter = 0;
                        //bClockLess70MHz = (MDrv_ReadByte(H_BK_ADC_DTOP(0x57)) & BIT2) > 0 ? 1 : 0;
                        bClockLess70MHz = Hal_DVI_clk_lessthan70mhz_usingportc(NO); //Not Port C
                        //MDRV_HDMIRX_MSG_INFO("\r\nAaron - CLK_CNT = %bx, AF= %bx\r\n", MDrv_ReadByte(L_BK_ADC_DTOP(0x57)), MDrv_ReadByte(H_BK_ADC_DTOP(0x57)));
                        //MDRV_HDMIRX_MSG_INFO("\r\nAaron - VCO= %bx\r\n", MDrv_ReadByte(L_BK_ADC_ATOP(0x68)));

                        if (g_bDVILess70MHz != bClockLess70MHz)
                        {
                            //MDRV_HDMIRX_MSG_INFO("\r\nAaron - SWP1  CL70_old = %d, CL70_new = %d\r\n", g_bDVILess70MHz, bClockLess70MHz);
                            g_bDVILess70MHz = bClockLess70MHz;
                            MDrv_HDMI_dvi_adjust(bClockLess70MHz);

                        }
                    }
                }
                g_DVI70MhzPollingCounter = 20; // 20 msec
            }
            break;

        case INPUT_PORT_DVI2:

//            if(!(MDrv_ReadByte(H_BK_ADC_DTOP(0x58)) & BIT3)) // DVI clock is stable
            if(!Hal_DVI_clk_stable_usingportc(YES)) // DVI clock is stable
            {
//                if(!(R2BYTEMSK(0x2C33, BIT5))) // Video clock is unstable
                if(!(R2BYTEMSK(0x2C32, BIT13))) // Video clock is unstable
                {
                    g_DVI70MhzUnstableCounter++;
                    if(g_DVI70MhzUnstableCounter >= 2)
                    {
                        g_DVI70MhzUnstableCounter = 0;

                        //bClockLess70MHz = (MDrv_ReadByte(H_BK_ADC_DTOP(0x58)) & BIT2) > 0 ? 1 : 0;
                        bClockLess70MHz = Hal_DVI_clk_lessthan70mhz_usingportc(YES); //Is Port C
                        //MDRV_HDMIRX_MSG_INFO("\r\nAaron - CLK_CNT = %bx, AF= %bx\r\n", MDrv_ReadByte(L_BK_ADC_DTOP(0x58)), MDrv_ReadByte(H_BK_ADC_DTOP(0x58)));
                        //MDRV_HDMIRX_MSG_INFO("\r\nAaron - VCO= %bx\r\n", MDrv_ReadByte(L_BK_ADC_ATOP(0x68)));

                        if (g_bDVILess70MHz != bClockLess70MHz)
                        {
                            //MDRV_HDMIRX_MSG_INFO("\r\nAaron - CL70_old = %d, CL70_new = %d\r\n", g_bDVILess70MHz, bClockLess70MHz);
                            g_bDVILess70MHz = bClockLess70MHz;
                            MDrv_HDMI_dvi_adjust(bClockLess70MHz);

                        }
                    }
                }
                g_DVI70MhzPollingCounter = 20; // 20 msec
            }
            break;
        }

        // clear interrupt status
        W2BYTEMSK(REG_HDMI_63_L, BIT13, BIT13);  //REG_HDMI_INT_CLEAR
        W2BYTEMSK(REG_HDMI_63_L,     0, BIT13);  //REG_HDMI_INT_CLEAR
        //MDRV_HDMIRX_MSG_INFO("\r\nAaron- DVICLOCK_PATH = %d\r\n", DVICLOCK_PATH);
    }
}

//-------------------------------------------------------------------------------------------------
/// avoid the snow image while AC on/off and source change for DVI 70Mhz issue, setting 2
/// @param  enInputPortType                \b IN: Input source
//-------------------------------------------------------------------------------------------------
void mdrv_dvi_clock_70mhz_swpatch2(void *pInstance, INPUT_SOURCE_TYPE_t enInputSourceType) //NO_USE
{
    MS_BOOL bClockLess70MHz;
    static MS_U16 g_DVITMDSClock = 0;
    MS_U16 u16DVITMDSClock, u16DVIClockDiff;
    E_MUX_INPUTPORT enMuxPort =MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType);

//    if(R2BYTEMSK(0x2C33, BIT5)) // Video clock is stable
    if(R2BYTEMSK(0x2C32, BIT13)) // Video clock is stable
    {
        g_DVI70MhzUnstableCounter = 0;
        g_DVI70MhzPollingCounter = 0;

        #if ( WATCH_DOG == ENABLE )
            //msAPI_Timer_ResetWDT();
            MDrv_Sys_ClearWatchDog();
        #endif

        switch(enMuxPort)
        {
        default:
        case INPUT_PORT_DVI0:
        case INPUT_PORT_DVI1:
        case INPUT_PORT_DVI3:
            if(Hal_DVI_clk_stable_usingportc(NO)) // input clock is not really stable
            {
                //MDRV_HDMIRX_MSG_INFO("\r\nAaron - False alarm, SWP2 TMDS Clock= %d\r\n", MDrv_Read2Byte(__REG_HDMI(0x11)) & 0x1FFF);
                return;
            }
            break;

        case INPUT_PORT_DVI2:
            if(Hal_DVI_clk_stable_usingportc(YES)) // input clock is not really stable
            {
                //MDRV_HDMIRX_MSG_INFO("\r\nAaron - False alarm, SWP2 TMDS Clock= %d\r\n", MDrv_Read2Byte(__REG_HDMI(0x11)) & 0x1FFF);
                return;
            }
            break;
        }

//        u16DVITMDSClock = MDrv_Read2Byte(__REG_HDMI(0x11)) & 0x1FFF;
        u16DVITMDSClock = R2BYTE(REG_HDMI_11_L) & 0x1FFF;
        u16DVIClockDiff = (g_DVITMDSClock >= u16DVITMDSClock) ? (g_DVITMDSClock - u16DVITMDSClock) : (u16DVITMDSClock - g_DVITMDSClock);
        if(u16DVIClockDiff > 10) // tolerance = 1MHz
        {
            //MDRV_HDMIRX_MSG_INFO("\r\nAaron - SWP2 Clock_Old = %d <-> Clock_New = %d \r\n", g_DVITMDSClock, u16DVITMDSClock);
            g_DVITMDSClock = u16DVITMDSClock;
            g_DVI70MhzPollingCounter = 20; // 20 msec
            return;
        }
//        bClockLess70MHz = (MDrv_ReadByte(H(__REG_HDMI(0x11))) & 0x60) > 0 ? 0 : 1;
        bClockLess70MHz = (R2BYTEMSK(REG_HDMI_11_L, HBMASK) & 0x60) > 0 ? 0 : 1;
     //MDRV_HDMIRX_MSG_INFO("\r\nAaron - HDCP status = %bx, INT timer= %d\r\n", MDrv_ReadByte(L_BK_ADC_DTOP(0x79)), MDrv_ReadByte(H(__REG_HDMI(0x09))));
     //MDRV_HDMIRX_MSG_INFO("\r\nAaron - SWP2 CL70_old = %d, CL70_new = %d\r\n", g_bDVILess70MHz, bClockLess70MHz);
     //MDRV_HDMIRX_MSG_INFO("\r\nAaron - SWP2 TMDS Clock = %d\r\n", MDrv_Read2Byte(__REG_HDMI(0x11)) & 0x1FFF );
        if (g_bDVILess70MHz != bClockLess70MHz)
        {
            g_bDVILess70MHz = bClockLess70MHz;
            MDrv_HDMI_dvi_adjust(bClockLess70MHz);
        }
    }
}
// 20081226 - DVI+HDCP snow noise patch - end ...
#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U8 MDrv_HDMI_err_status_update(MS_U8 u8value, MS_BOOL bread)
{
    stHDMI_ERR_STATUS_UPDATE stHDMIRxArgs = {0};

    stHDMIRxArgs.u8value = u8value;
    stHDMIRxArgs.bread = bread;
    stHDMIRxArgs.u8ReturnValue = 0;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bNewPathFlag = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_ERROR_STATUS_UPDATE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#endif

MS_U16 MDrv_HDMI_GetTMDSFreq_U2(void* pInstance)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    return Hal_HDMI_GetTMDSFreq();
#else
    return 0;
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U16 MDrv_HDMI_GetTMDSFreq(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return 0;
    }

    stHDMI_GET_TMDS_FREQ XCArgs;
    XCArgs.u8ReturnValue = 0;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GET_TMDS_FREQ, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return 0;
    }
    else
    {
        return XCArgs.u8ReturnValue;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// HDMI packet reset
/// @param  breset                \b IN: Enabl reset or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_pkt_reset_internal(void *pInstance, HDMI_REST_t breset )
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        Hal_HDMI_pkt_reset(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[MAIN_WINDOW].enInputSourceType), HDMI_INFO_SOURCE0, breset, NULL);
    }
    else if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
    {
        Hal_HDMI_pkt_reset(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[SUB_WINDOW].enInputSourceType), HDMI_INFO_SOURCE0, breset, NULL);
    }
#else
    Hal_HDMI_pkt_reset(HDMI_INFO_SOURCE0, breset);
#endif

#if SUPPORT_HDMI_RX_NEW_FEATURE
    //pkt reset cannot clear status in DVI mode  in non-immeswitch case, RD suggest to use DVI sw reset
    if(!pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH) // Only apply in non-imme switch case
    {
        if(breset & REST_HDMI_STATUS )
        {
            Hal_DVI_Software_Reset(INPUT_PORT_DVI0, BIT(6));
        }
    }
#else
    // Do nothing
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_pkt_reset( HDMI_REST_t breset )
{
    stHDMI_PKT_RESET stHDMIRxArgs = {breset, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_PACKET_RESET, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Enable Hot plug Detection out enable
/// @param  benable                \b IN: Enabl output or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_hpd_en(MS_BOOL benable)
{
    Hal_HDMI_hpd_en(benable);
}

void MDrv_HDCP_initproductionkey_internal(void *pInstance, MS_U8 * pu8HdcpKeyData )
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    memcpy(pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8HdcpKeyDataBackUp, pu8HdcpKeyData, HDCP_KEY_LENGTH * sizeof(MS_U8));// HDCP key length is 289 byte

    if(bInitEnable == FALSE)
    {
        MDRV_HDMIRX_MSG_INFO("[%s] Please follow the flow: MDrv_HDMI_init() -> MDrv_HDCP_initproductionkey()\n", __FUNCTION__);
    }
    else
    {
        if(!pXCResourcePrivate->stdrv_HDMI.bHDCP14RxREEFlag)
        {
            pXCResourcePrivate->stdrv_HDMI.bHDCP14RxREEFlag = TRUE;
        }

        Hal_HDCP_initproductionkey(pu8HdcpKeyData);
    }
}

//-------------------------------------------------------------------------------------------------
/// Init the HDCP production key
/// @param  pu8HdcpKeyData                \b IN: HDCP key
//-------------------------------------------------------------------------------------------------
void MDrv_HDCP_initproductionkey_U2( void* pInstance, MS_U8 * pu8HdcpKeyData )
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    MDrv_HDCP_initproductionkey_internal(pInstance, pu8HdcpKeyData);

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDCP_initproductionkey( MS_U8 * pu8HdcpKeyData )
{
    stHDCP_INITPRODUCTIONKEY XCArgs;
    XCArgs.pu8HdcpKeyData = pu8HdcpKeyData;

    //if(pu32XCInst == NULL)
    //{
        //MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    //}
    //else if(UtopiaIoctl(pu32XCInst, E_XC_HDCP_IF_CMD_INITPRODUCTIONKEY, (void*)&XCArgs) != 0)
    //{
        //MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    //}

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CHECK_HDCP14_KEY, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Clear the HDCP Flag
//-------------------------------------------------------------------------------------------------
void MDrv_HDCP_ClearStatus_U2( void* pInstance )
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        Hal_HDCP_clearflag(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[MAIN_WINDOW].enInputSourceType));
    }
    else if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
    {
        Hal_HDCP_clearflag(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[SUB_WINDOW].enInputSourceType));
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
#else
    Hal_HDCP_clearflag();
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDCP_ClearStatus( void )
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    if(UtopiaIoctl(pu32XCInst, E_XC_HDCP_IF_CMD_CLEARSTATUS, NULL) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Enable audio output or not
/// @param  bEnable                \b IN: Enable or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_audio_output_U2( void* pInstance, MS_BOOL bEnable )
{
    Hal_HDMI_audio_output(bEnable);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_audio_output( MS_BOOL bEnable )
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_AUDIO_OUTPUT XCArgs;
    XCArgs.bEnable = bEnable;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_AUDIO_OUTPUT, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U8 MDrv_HDMI_audio_cp_hdr_info(void)
{
    stHDMI_AUDIO_CP_HDR_INFO stHDMIRxArgs = {0};

    stHDMIRxArgs.u8ReturnValue = 0;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bNewPathFlag = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_AUDIO_CONTENT_PROTECT_INFO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#endif

//-------------------------------------------------------------------------------------------------
/// Get the AVI infoframe information
/// @param  u8byte                \b IN: byte info selection
/// @return @ref MS_U8
//-------------------------------------------------------------------------------------------------
MS_U8 MDrv_HDMI_avi_infoframe_info(MS_U8 u8byte)
{
    stHDMI_AVI_INFOFRAME_INFO stHDMIRxArgs = {u8byte, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMI_AVI_INFOFRAME_INFO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

MS_U8 MDrv_HDMI_AVIInfoframePort(E_MUX_INPUTPORT enInputPortType, MS_U8 u8byte)
{
    stHDMI_AVIINFOFRAMEPORT stHDMIRxArgs = {enInputPortType, u8byte, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMI_AVIINFOFRAMEPORT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Get_MID_info(MS_U8 * pu8ManufactorData)
{
    stHDMI_GET_MID_INFO stHDMIRxArgs;

    stHDMIRxArgs.pu8ManufactorData = pu8ManufactorData;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_MID_INFO_VALUE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_get_packet_value(MS_HDMI_PACKET_STATE_t u8state, MS_U8 u8byte_idx, MS_U8 *pValue)
{
    stHDMI_GET_PACKET_VALUE stHDMIRxArgs;

    stHDMIRxArgs.u8state = u8state;
    stHDMIRxArgs.u8byte_idx = u8byte_idx;
    stHDMIRxArgs.pValue = pValue;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bReturnValue = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PACKET_VALUE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U8 MDrv_HDMI_audio_channel_status(MS_U8 u8byte)
{
    stHDMI_AUDIO_CHANNEL_STATUS stHDMIRxArgs = {0};

    stHDMIRxArgs.u8byte = u8byte;
    stHDMIRxArgs.u8ReturnValue = 0;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_AUDIO_CHANNEL_STATUS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#endif

//-------------------------------------------------------------------------------------------------
/// Get the HDCP status
/// @return @ref MS_HDCP_STATUS_INFO_t
//-------------------------------------------------------------------------------------------------
MS_U16 _MDrv_HDCP_GetStatus(void* pInstance)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
#if SUPPORT_HDMI_RX_NEW_FEATURE
    MS_U16 u16Status = 0;
    if(IsSrcTypeHDMI(gSrcInfo[MAIN_WINDOW].enInputSourceType))
    {
        u16Status = Hal_HDCP_getstatus(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[MAIN_WINDOW].enInputSourceType));
    }
    else
    {
#if PIP_PATCH_USING_SC1_MAIN_AS_SC0_SUB
        XC_RESOURCE_PRIVATE* pXCResourcePrivate_1 = NULL;
        UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(1)],(void**)(&pXCResourcePrivate_1));
        if(IsSrcTypeHDMI(gSrcInfo_1[MAIN_WINDOW].enInputSourceType))
        {
            u16Status = Hal_HDCP_getstatus(MDrv_XC_Mux_GetHDMIPort(g_pDevice1Instance, gSrcInfo_1[MAIN_WINDOW].enInputSourceType));
        }
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_HDMI_Mode = u16Status & BIT(0);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_Reserve_1 = u16Status & BIT(1);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_HDCP_Ver = u16Status & BIT(2);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_AVMute = u16Status & BIT(3);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_ENC_En = u16Status & BIT(4);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_AKsv_Rcv = u16Status & BIT(5);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_ENC_Dis = u16Status & BIT(6);
        pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_Reserve_7 = u16Status & BIT(7);
        UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
#else
        if(IsSrcTypeHDMI(gSrcInfo[SUB_WINDOW].enInputSourceType))
        {
            u16Status = Hal_HDCP_getstatus(MDrv_XC_Mux_GetHDMIPort(pInstance, gSrcInfo[SUB_WINDOW].enInputSourceType));
        }
#endif
    }
#else
    MS_U16 u16Status = Hal_HDCP_getstatus();
#endif

#if defined(STELLAR) && defined(PIP_PATCH_USING_SC1_MAIN_AS_SC0_SUB) //for L project live zoom update hdmi info to main2
    XC_RESOURCE_PRIVATE* pXCResourcePrivate_1 = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(1)],(void**)(&pXCResourcePrivate_1));
    if(IsSrcTypeHDMI(gSrcInfo_1[MAIN_WINDOW].enInputSourceType))
    {
        u16Status = Hal_HDCP_getstatus(MDrv_XC_Mux_GetHDMIPort(g_pDevice1Instance, gSrcInfo_1[MAIN_WINDOW].enInputSourceType));
    }
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_HDMI_Mode = u16Status & BIT(0);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_Reserve_1 = u16Status & BIT(1);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_HDCP_Ver = u16Status & BIT(2);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_AVMute = u16Status & BIT(3);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_ENC_En = u16Status & BIT(4);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_AKsv_Rcv = u16Status & BIT(5);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_ENC_Dis = u16Status & BIT(6);
    pXCResourcePrivate_1->stdrv_HDMI.enStatus_info.b_St_Reserve_7 = u16Status & BIT(7);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
#endif


    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_HDMI_Mode = u16Status & BIT(0);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_Reserve_1 = u16Status & BIT(1);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_HDCP_Ver = u16Status & BIT(2);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_AVMute = u16Status & BIT(3);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_ENC_En = u16Status & BIT(4);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_AKsv_Rcv = u16Status & BIT(5);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_ENC_Dis = u16Status & BIT(6);
    pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_Reserve_7 = u16Status & BIT(7);

    return u16Status;
}

MS_U16 MDrv_HDCP_GetStatus_U2(void* pInstance)
{
    MS_U16 usHDCPStatus = 0;
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    usHDCPStatus = _MDrv_HDCP_GetStatus(pInstance);
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    return usHDCPStatus;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDCP_STATUS_INFO_t* MDrv_HDCP_GetStatus(void )
{
    stHDCP_GETSTATUS XCArgs = {0};
    static MS_HDCP_STATUS_INFO_t stHDCPStatus = {0};

    XCArgs.usHDCPStatus = 0;

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_HDCP_IF_CMD_GETSTATUS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }
    else
    {
        stHDCPStatus.b_St_HDMI_Mode = ((XCArgs.usHDCPStatus &BIT(0))? TRUE: FALSE);
        stHDCPStatus.b_St_Reserve_1 = ((XCArgs.usHDCPStatus &BIT(1))? TRUE: FALSE);
        stHDCPStatus.b_St_HDCP_Ver = ((XCArgs.usHDCPStatus &BIT(2))? TRUE: FALSE);
        stHDCPStatus.b_St_AVMute = ((XCArgs.usHDCPStatus &BIT(3))? TRUE: FALSE);
        stHDCPStatus.b_St_ENC_En = ((XCArgs.usHDCPStatus &BIT(4))? TRUE: FALSE);
        stHDCPStatus.b_St_AKsv_Rcv = ((XCArgs.usHDCPStatus &BIT(5))? TRUE: FALSE);
        stHDCPStatus.b_St_ENC_Dis = ((XCArgs.usHDCPStatus &BIT(6))? TRUE: FALSE);
        stHDCPStatus.b_St_Reserve_7 = ((XCArgs.usHDCPStatus &BIT(7))? TRUE: FALSE);
    }

    return &stHDCPStatus;
}

#endif

void MDrv_DVI_HF_adjust_internal(void)
{
    Hal_DVI_HF_adjust();
}

void MDrv_DVI_HF_adjust_U2( void* pInstance )
{
    MDrv_DVI_HF_adjust_internal();
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_DVI_HF_adjust(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_DVI_HF_ADJUST, NULL) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Returne Phase status
/// @param  bClockLessThan70MHz                \b IN: u8Channel
//-------------------------------------------------------------------------------------------------
MS_U16 MDrv_DVI_ChannelPhaseStatus_U2(void* pInstance, MS_DVI_CHANNEL_TYPE u8Channel)
{
    return Hal_DVI_ChannelPhaseStatus(u8Channel);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U16 MDrv_DVI_ChannelPhaseStatus( MS_DVI_CHANNEL_TYPE u8Channel)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return 0;
    }

    stDVI_CHANNELPHASESTATUS XCArgs;
    XCArgs.u8Channel = u8Channel;
    XCArgs.u16ReturnValue = 0;

    if(UtopiaIoctl(pu32XCInst, E_XC_DVI_IF_CMD_CHANNELPHASESTATUS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return 0;
    }
    else
    {
        return XCArgs.u16ReturnValue;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// Check DVI accumulator and reset status.
/// @param
/// Due to DVI accumulator may be overflow when previous source is MHL,
/// this function will monitor MHL port's accumulator, and reset when it hits boundary.
//-------------------------------------------------------------------------------------------------
void MDrv_DVI_Accumulator_Monitor(void)
{
    Hal_DVI_Accumulator_Monitor();
}

//-------------------------------------------------------------------------------------------------
/// Adjust the ADC for DVI by the condition is less than 70Mhz or not
/// @param  bClockLessThan70MHz                \b IN: Is Less 70Mhz or not
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_dvi_adjust_U2( void* pInstance, MS_BOOL bClockLessThan70MHz)
{
    Hal_DVI_adc_adjust(bClockLessThan70MHz);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_dvi_adjust( MS_BOOL bClockLessThan70MHz)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_DVI_ADJUST XCArgs;
    XCArgs.bClockLessThan70MHz = bClockLessThan70MHz;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_DVI_ADJUST, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//-------------------------------------------------------------------------------------------------
/// audio content payload info
/// @param  u8byte                \b IN: Byte number
/// @return @ref MS_U8
//-------------------------------------------------------------------------------------------------
MS_U8 MDrv_HDMI_audio_cp_data_info(MS_U8 u8byte)
{
    //return Hal_HDMI_audio_payload_info(HDMI_INFO_SOURCE0, u8byte);

    stHDMI_AUDIO_CP_DATA_INFO stHDMIRxArgs = {INPUT_PORT_DVI0, u8byte, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMI_AUDIO_CP_DATA_INFO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

//-------------------------------------------------------------------------------------------------
/// Get the HDMI packet
/// @param  bEnable                \b IN: Enable receive or not
/// @return @ref MS_HDMI_PACKET_VALUE_t
//-------------------------------------------------------------------------------------------------
void MDrv_HDMI_Packet_Received_U2(void* pInstance, MS_U8 ucHDMIInfoSource, MS_BOOL bEnable, MS_U32 ulPacketStatus)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);

    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));

    if(bEnable)
    {
        pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] = ulPacketStatus;
    }

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDMI_PACKET_VALUE_t MDrv_HDMI_Packet_Received(MS_BOOL bEnable)
{
    stHDMI_PACKET_RECEIVED XCArgs = {0};
    MS_HDMI_PACKET_VALUE_t stPacketValue = {0};

    XCArgs.bEnable = bEnable;
    XCArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    XCArgs.ulPacketStatus = 0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PACKET_RECEIVE, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            stPacketValue.PKT_MPEG_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_MPEG_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_AUI_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_AUDIO_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_SPD_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_SPD_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_AVI_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_GC_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_GCP_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ASAMPLE_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ACR_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_ACR_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_VS_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_NULL_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_NULL_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ISRC2_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_ISRC2_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ISRC1_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ACP_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_ACP_PACKET_RECEIVE_FLAG;
            stPacketValue.PKT_ONEBIT_AUD_VALUE = (MS_U16)XCArgs.ulPacketStatus &HDMI_STATUS_DSD_PACKET_RECEIVE_FLAG;
        }
    }

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_PACKET_RECEIVED, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }

    return stPacketValue;
}

#endif

MS_HDMI_COLOR_FORMAT MDrv_HDMI_Get_ColorFormat(void)
{
    stHDMI_GET_COLORFORMAT stHDMIRxArgs = {MS_HDMI_COLOR_UNKNOWN, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_COLOR_FORMAT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
EN_HDMI_COLOR_RANGE MDrv_HDMI_Get_ColorRange(void)
{
    stHDMI_GET_COLORRANGE stHDMIRxArgs = {E_HDMI_COLOR_RANGE_RESERVED, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_COLOR_RANGE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDMI_AR_TYPE MDrv_HDMI_Get_AspectRatio(MS_BOOL bEn_PAR, MS_BOOL bEn_AFAR)
{
    stHDMI_GET_ASPECTRATIO stHDMIRxArgs = {0};

    stHDMIRxArgs.bEn_PAR = bEn_PAR;
    stHDMIRxArgs.bEn_AFAR = bEn_AFAR;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.eReturnValue = HDMI_AF_AR_Reserve_0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_ASPECT_RATIO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDMI_AR_TYPE MDrv_HDMI_Get_ActiveFormat_AspectRatio(MS_BOOL bEn_AFAR)
{
    stHDMI_GET_ACTIVEFORMAT_ASPECTRATIO stHDMIRxArgs = {0};

    stHDMIRxArgs.bEn_AFAR = bEn_AFAR;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.eReturnValue = HDMI_AF_AR_Reserve_0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_ACTIVE_FORMAT_ASPECT_RATIO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDMI_CONTENT_TYPE MDrv_HDMI_Get_Content_Type(void)
{
    stHDMI_GET_CONTENT_TYPE stHDMIRxArgs = {MS_HDMI_CONTENT_NoData, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_CONTENT_TYPE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.enReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U8 MDrv_HDMI_Get_Pixel_Repetition(void)
{
    stHDMI_GET_PIXEL_REPETITION stHDMIRxArgs = {0};

    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.u8ReturnValue = 0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PIXEL_REPETITION, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
EN_AVI_INFOFRAME_VERSION MDrv_HDMI_Get_AVIInfoFrameVer(void)
{
    stHDMI_GET_AVIINFOFRAMEVER stHDMIRxArgs = {E_AVI_INFOFRAME_VERSION_NON, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_AVI_INFOFRAME_VERSION, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_Get_AVIInfoActiveInfoPresent(void)
{
    stHDMI_GET_AVIIINFO_ACTIVEINFOPRESENT stHDMIRxArgs = {0};

    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bReturnValue = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_ACTIVE_INFORMATION_PRESENT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bReturnValue;
}

#endif

//-------------------------------------------------------------------------------------------------
/// Pull DVI Clock to low
/// @param  bPullLow                          \b IN: Set DVI clock to low
/// @param  enInputPortType              \b IN: Input source
//-------------------------------------------------------------------------------------------------
void MDrv_DVI_ClkPullLow_U2(void* pInstance, MS_BOOL bPullLow, E_MUX_INPUTPORT enInputPortType)
{
    Hal_DVI_ClkPullLow(bPullLow, enInputPortType);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_DVI_ClkPullLow(MS_BOOL bPullLow, E_MUX_INPUTPORT enInputPortType)
{
    stDVI_CLKPULLLOW XCArgs = {bPullLow, enInputPortType};
    stHDMI_DATA_RTERM_CONTROL stHDMIRxArgs = {enInputPortType, TRUE, TRUE};

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_DVI_IF_CMD_CLKPULLLOW, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }
    else if(!bPullLow)
    {
        if(pulHDMIRxInst == NULL)
        {
            if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
            }
        }

        if(pulHDMIRxInst != NULL)
        {
            if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_DATA_RTERM_CONTROL, (void*)&stHDMIRxArgs) != 0)
            {
                MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
            }
        }
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_DVI_SwitchSrc(E_MUX_INPUTPORT enInputPortType)
{
    stDVI_SWITCHSRC stHDMIRxArgs = {enInputPortType, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_DVI_MUX_SWITCH, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

void MDrv_DVI_IMMESWITCH_PS_SW_Path(void *pInstance)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(pXCResourcePrivate->stdrvXC_MVideo_Context.g_bIsIMMESWITCH) // Only apply in imme switch case
    {
        Hal_DVI_IMMESWITCH_PS_SW_Path();
    }
}

/// lib version
HDMI_Ret_Value MDrv_HDMI_GetLibVer_U2(void* pInstance, const MSIF_Version **ppVersion)
{
    if (!ppVersion)
    {
        return E_HDMI_RET_FAIL;
    }

    *ppVersion = &_drv_hdmi_version;
    return E_HDMI_RET_OK;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
HDMI_Ret_Value MDrv_HDMI_GetLibVer(const MSIF_Version **ppVersion)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return E_HDMI_RET_FAIL;
    }

    stHDMI_GETLIBVER XCArgs;
    XCArgs.ppVersion = ppVersion;
    XCArgs.eReturnValue = E_HDMI_RET_FAIL;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GETLIBVER, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return E_HDMI_RET_FAIL;
    }
    else
    {
        return XCArgs.eReturnValue;
    }
}

#endif

// lib information
HDMI_Ret_Value  MDrv_HDMI_GetInfo_U2(void* pInstance, MS_HDMI_INFO *pInfo)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));

    memcpy((void*)pInfo, (void*)&(pXCResourcePrivate->stdrv_HDMI._info), sizeof(MS_HDMI_INFO));

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    return E_HDMI_RET_OK;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
HDMI_Ret_Value  MDrv_HDMI_GetInfo(MS_HDMI_INFO *pInfo)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return E_HDMI_RET_FAIL;
    }

    stHDMI_GETINFO XCArgs;
    XCArgs.pInfo = pInfo;
    XCArgs.eReturnValue = E_HDMI_RET_FAIL;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GETINFO, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return E_HDMI_RET_FAIL;
    }
    else
    {
        return XCArgs.eReturnValue;
    }
}

#endif

// curretn status
HDMI_Ret_Value  MDrv_HDMI_GetStatus_U2(void* pInstance, MS_HDMI_Status *pStatus)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    memcpy((void*)pStatus, (void*)&(pXCResourcePrivate->stdrv_HDMI._status), sizeof(MS_HDMI_Status));
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
    return E_HDMI_RET_OK;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
HDMI_Ret_Value  MDrv_HDMI_GetStatus(MS_HDMI_Status *pStatus)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return E_HDMI_RET_FAIL;
    }

    stHDMI_GETSTATUS XCArgs;
    XCArgs.pStatus = pStatus;
    XCArgs.eReturnValue = E_HDMI_RET_FAIL;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GETSTATUS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return E_HDMI_RET_FAIL;
    }
    else
    {
        return XCArgs.eReturnValue;
    }
}

#endif

// DDC
static void MDrv_HDMI_ReadEDID(E_XC_DDCRAM_PROG_TYPE eDDCRamType, MS_U8 *u8EDID, MS_U16 u8EDIDSize)
{
    MS_U16 u16temp = 0;

    MS_U16 u16EDIDSupportSize = Hal_HDMI_GetDataInfo(E_HDMI_GET_SUPPORT_EDID_SIZE, HDMI_INFO_SOURCE0, NULL);

    if(u8EDIDSize <= u16EDIDSupportSize)
    {
        HAL_HDMI_DDCRam_SelectPort(eDDCRamType);

        if(u16EDIDSupportSize > HDMI_EDID_SUPPORT_256)
        {
            HAL_HDMI_ReadEDIDNewMode(eDDCRamType, u8EDIDSize, u8EDID);
        }
        else
        {
            for(u16temp = 0; u16temp < u8EDIDSize; u16temp++)
            {
                if(eDDCRamType == E_XC_PROG_VGA_EDID)
                {
                    DDC_WRITE_MASK(REG_DDC_A_ACCESS,REG_DDC_A_ACCESS_BITS(1),REG_DDC_A_ACCESS_MASK);
                    DDC_WRITE_MASK(REG_DDC_A_WRITEADDR,REG_DDC_A_WRITEADDR_BITS(u16temp),REG_DDC_A_WRITEADDR_MASK);
                    DDC_WRITE_MASK(REG_DDC_A_READPULSE,REG_DDC_A_READPULSE_BITS(1),REG_DDC_A_READPULSE_MASK);
                    MsOS_DelayTaskUs(2);
                    *(u8EDID +u16temp) = (MS_U8)(DDC_READ_MASK(REG_DDC_A_READDATA, REG_DDC_A_READDATA_MASK)>>8);
                }
                else
                {
                    DDC_WRITE_MASK(REG_DDC_D_ACCESS,REG_DDC_D_ACCESS_BITS(1),REG_DDC_D_ACCESS_MASK);
                    DDC_WRITE_MASK(REG_DDC_D_WRITEADDR,REG_DDC_D_WRITEADDR_BITS(u16temp),REG_DDC_D_WRITEADDR_MASK);
                    DDC_WRITE_MASK(REG_DDC_D_READPULSE,REG_DDC_D_READPULSE_BITS(1),REG_DDC_D_READPULSE_MASK);
                    MsOS_DelayTaskUs(2);
                    *(u8EDID +u16temp) = (MS_U8)(DDC_READ_MASK(REG_DDC_D_READDATA, REG_DDC_D_READDATA_MASK)>>8);
                }
            }
        }
    }
}

// DDC
static void MDrv_HDMI_WriteEDID(E_XC_DDCRAM_PROG_TYPE eDDCRamType, MS_U8 *u8EDID, MS_U16 u16EDIDSize)
{
    MS_U16 u16temp = 0;

    MS_U16 u16EDIDSupportSize = Hal_HDMI_GetDataInfo(E_HDMI_GET_SUPPORT_EDID_SIZE, HDMI_INFO_SOURCE0, NULL);

    if(u16EDIDSize <= u16EDIDSupportSize)
    {
        HAL_HDMI_DDCRam_SelectPort(eDDCRamType);

        if(u16EDIDSupportSize > HDMI_EDID_SUPPORT_256)
        {
            HAL_HDMI_LoadEDIDNewMode(eDDCRamType, u16EDIDSize, u8EDID);
        }
        else
        {
            for(u16temp = 0; u16temp < u16EDIDSize; u16temp++)
            {
                if(eDDCRamType == E_XC_PROG_VGA_EDID)
                {
                    DDC_WRITE_MASK(REG_DDC_A_ACCESS,REG_DDC_A_ACCESS_BITS(0),REG_DDC_A_ACCESS_MASK);
                    DDC_WRITE_MASK(REG_DDC_A_WRITEADDR,REG_DDC_A_WRITEADDR_BITS(u16temp),REG_DDC_A_WRITEADDR_MASK);
                    DDC_WRITE_MASK(REG_DDC_A_WRITEDATA,REG_DDC_A_WRITEDATA_BITS(*(u8EDID +u16temp)),REG_DDC_A_WRITEDATA_MASK);
                    DDC_WRITE_MASK(REG_DDC_A_WRITEPULSE,REG_DDC_A_WRITEPULSE_BITS(1),REG_DDC_A_WRITEPULSE_MASK);
                }
                else if(eDDCRamType <= HDMI_GET_PORT_SELECT(INPUT_PORT_DVI_END))
                {
                    DDC_WRITE_MASK(REG_DDC_D_ACCESS,REG_DDC_D_ACCESS_BITS(0),REG_DDC_D_ACCESS_MASK);
                    DDC_WRITE_MASK(REG_DDC_D_WRITEADDR,REG_DDC_D_WRITEADDR_BITS(u16temp),REG_DDC_D_WRITEADDR_MASK);
                    DDC_WRITE_MASK(REG_DDC_D_WRITEDATA,REG_DDC_D_WRITEDATA_BITS(*(u8EDID +u16temp)),REG_DDC_D_WRITEDATA_MASK);
                    DDC_WRITE_MASK(REG_DDC_D_WRITEPULSE,REG_DDC_D_WRITEPULSE_BITS(1),REG_DDC_D_WRITEPULSE_MASK);
                }
            }
        }

        switch(eDDCRamType)
        {
            case E_XC_PROG_DVI0_EDID:
                DDC_WRITE_MASK(REG_DDC_DVI0_WP,BITS(13:13,1),BMASK(13:13)); // Write Protect 0 ddc
                break;
            case E_XC_PROG_DVI1_EDID:
                DDC_WRITE_MASK(REG_DDC_DVI1_WP,BITS(13:13,1),BMASK(13:13)); // Write Protect 1 ddc
                break;
            case E_XC_PROG_DVI2_EDID:
                DDC_WRITE_MASK(REG_DDC_DVI2_WP,BITS(5:5,1),BMASK(5:5));     // Write Protect 2 ddc
                break;
            case E_XC_PROG_DVI3_EDID:
                DDC_WRITE_MASK(REG_DDC_DVI3_WP,BITS(5:5,1),BMASK(5:5));     //Write Protect  3 ddc
                break;
            case E_XC_PROG_VGA_EDID:
                DDC_WRITE_MASK(REG_DDC_ADC0_WP,BITS(13:13,1),BMASK(13:13)); // Write Protect analog ddc
                break;
            default:
                MS_ASSERT(0);
        }
    }
}

static void MDrv_HDMI_DDCRAM_Enable(E_XC_DDCRAM_PROG_TYPE eDDCRamType)
{
    switch(eDDCRamType)
    {
        case E_XC_PROG_DVI0_EDID:
            DDC_WRITE_MASK(REG_DDC_DVI0_EN,BITS(15:15,1),BMASK(15:15)); // enable digital 0 ddc
            break;
        case E_XC_PROG_DVI1_EDID:
            DDC_WRITE_MASK(REG_DDC_DVI1_EN,BITS(15:15,1),BMASK(15:15)); // enable digital 1 ddc
            break;
        case E_XC_PROG_DVI2_EDID:
            DDC_WRITE_MASK(REG_DDC_DVI2_EN,BITS(7:7,1),BMASK(7:7));     // enable digital 2 ddc
            break;
        case E_XC_PROG_DVI3_EDID:
            DDC_WRITE_MASK(REG_DDC_DVI3_EN,BITS(7:7,1),BMASK(7:7));     // enable digital 3 ddc
            break;
        case E_XC_PROG_VGA_EDID:
            DDC_WRITE_MASK(REG_DDC_ADC0_EN,BITS(15:15,1),BMASK(15:15)); // enable analog ddc
            break;
        default:
            MS_ASSERT(0);
    }
}

void MDrv_HDMI_READ_DDCRAM_U2(void* pInstance, XC_DDCRAM_PROG_INFO *pstDDCRam_Info, MS_U32 u32SizeOfInfo)
{
    if(u32SizeOfInfo > 0)
    {
        // Do nothing
    }

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    MDrv_HDMI_DDCRAM_Enable(pstDDCRam_Info->eDDCProgType);

    MDrv_HDMI_ReadEDID(pstDDCRam_Info->eDDCProgType, pstDDCRam_Info->EDID, pstDDCRam_Info->u16EDIDSize);

    HAL_HDMI_DDCRAM_SetPhyAddr(pstDDCRam_Info);

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_READ_DDCRAM(XC_DDCRAM_PROG_INFO *pstDDCRam_Info, MS_U32 u32SizeOfInfo)
{
    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(pstDDCRam_Info != NULL)
    {
        MS_U8 u8EDIDTable[HDMI_EDID_SUPPORT_SIZE] = {0};
        stHDMI_READ_DDCRAM XCArgs;
        XC_DDCRAM_PROG_INFO stDDCRamInfo;

        stDDCRamInfo.EDID = u8EDIDTable;
        stDDCRamInfo.u16EDIDSize = pstDDCRam_Info->u16EDIDSize;
        stDDCRamInfo.eDDCProgType = pstDDCRam_Info->eDDCProgType;
        XCArgs.pstDDCRam_Info = &stDDCRamInfo;
        XCArgs.u32SizeOfInfo = u32SizeOfInfo;

        if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_READ_DDCRAM, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        }
        else
        {
            if(stDDCRamInfo.u16EDIDSize > pstDDCRam_Info->u16EDIDSize)
            {
                stDDCRamInfo.u16EDIDSize = pstDDCRam_Info->u16EDIDSize;
            }

            memcpy(pstDDCRam_Info->EDID, &u8EDIDTable, stDDCRamInfo.u16EDIDSize);
        }
    }
}

#endif

void MDrv_HDMI_PROG_DDCRAM_internal(void *pInstance, XC_DDCRAM_PROG_INFO *pstDDCRam_Info, MS_U32 u32SizeOfInfo)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if(pstDDCRam_Info->eDDCProgType != E_XC_PROG_VGA_EDID)
    {
        memcpy(pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8EDID_DVIBackUp[pstDDCRam_Info->eDDCProgType - E_XC_PROG_DVI0_EDID], pstDDCRam_Info->EDID, EDID_KEY_LENGTH*sizeof(MS_U8));
    }

    if(u32SizeOfInfo > 0)
    {
        // Do nothing
    }

    MDrv_HDMI_DDCRAM_Enable(pstDDCRam_Info->eDDCProgType);

    MDrv_HDMI_WriteEDID(pstDDCRam_Info->eDDCProgType, pstDDCRam_Info->EDID, pstDDCRam_Info->u16EDIDSize);

    HAL_HDMI_DDCRAM_SetPhyAddr(pstDDCRam_Info);
}

void MDrv_HDMI_PROG_DDCRAM_U2(void* pInstance, XC_DDCRAM_PROG_INFO *pstDDCRam_Info, MS_U32 u32SizeOfInfo)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    MDrv_HDMI_PROG_DDCRAM_internal(pInstance, pstDDCRam_Info, u32SizeOfInfo);
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_PROG_DDCRAM(XC_DDCRAM_PROG_INFO *pstDDCRam_Info, MS_U32 u32SizeOfInfo)
{
    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(pstDDCRam_Info != NULL)
    {
        MS_U8 u8EDIDTable[HDMI_EDID_SUPPORT_SIZE] = {0};
        stHDMI_PROG_DDCRAM XCArgs;
        XC_DDCRAM_PROG_INFO stDDCRamInfo;

        if(pstDDCRam_Info->u16EDIDSize <= HDMI_EDID_SUPPORT_SIZE)
        {
            memcpy(&u8EDIDTable, pstDDCRam_Info->EDID, pstDDCRam_Info->u16EDIDSize);

            stDDCRamInfo.EDID = u8EDIDTable;
            stDDCRamInfo.u16EDIDSize = pstDDCRam_Info->u16EDIDSize;
            stDDCRamInfo.eDDCProgType = pstDDCRam_Info->eDDCProgType;
            XCArgs.pstDDCRam_Info = &stDDCRamInfo;
            XCArgs.u32SizeOfInfo = u32SizeOfInfo;

            if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_PROG_DDCRAM, (void*)&XCArgs) != 0)
            {
                MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
            }
        }
        else
        {
            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]Input EDID size large than support size\n",__FUNCTION__,__LINE__);
        }
    }
}

#endif

/** According to the previous experience, some specific source devices would
send strange DVI / HDMI signal in slow frequency(like 480p) let us in trouble.
So it's better to increase DVI updated phase count to 100 lines in low
resolution to overcome that situation, originally DVI updated phase count is 1.
So the patch would check input DVI frequency first, if small than 50MHz, DVI
updated phase count would increase to 100 lines. Otherwise, DVI updated phase
count would be recovered to 1 line. */
void MDrv_HDMI_SetUpdatePhaseLineCount_U2(void* pInstance, INPUT_SOURCE_TYPE_t enInputSourceType)
{
    E_MUX_INPUTPORT enPort = INPUT_PORT_NONE_PORT;
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    enPort = MDrv_XC_Mux_GetHDMIPort(pInstance, enInputSourceType);
    if( enPort == INPUT_PORT_DVI0 || enPort == INPUT_PORT_DVI1 ||
        enPort == INPUT_PORT_DVI3 )
    {
        if( R2BYTEMSK(REG_DVI_ATOP_61_L, BMASK(9:8)) == 2<<8 ||
            R2BYTEMSK(REG_DVI_ATOP_61_L, BMASK(9:8)) == 3<<8 ) //<50Mhz
        {
            W2BYTEMSK(REG_DVI_DTOP_29_L, 0x0000, 0xFF00);
            W2BYTE(REG_DVI_DTOP_2A_L, 0x0000);
//            MDRV_HDMIRX_MSG_INFO("Low resolution\n");

        }
        else
        {
            W2BYTEMSK(REG_DVI_DTOP_29_L, 0, 0xFF00);
            W2BYTE(REG_DVI_DTOP_2A_L, 0);
//            MDRV_HDMIRX_MSG_INFO("High resolution\n");
        }

    }
    else if(enPort == INPUT_PORT_DVI2)
    {
        if( R2BYTEMSK(REG_DVI_ATOP_6A_L, BMASK(9:8)) == 2<<8 ||
            R2BYTEMSK(REG_DVI_ATOP_6A_L, BMASK(9:8)) == 3<<8 ) //<50Mhz
        {
            W2BYTEMSK(REG_DVI_DTOP_29_L, 0x0000, 0xFF00);
            W2BYTE(REG_DVI_DTOP_2A_L, 0x0000);
//            MDRV_HDMIRX_MSG_INFO("Low resolution 2\n");
        }
        else
        {
            W2BYTEMSK(REG_DVI_DTOP_29_L, 0, 0xFF00);
            W2BYTE(REG_DVI_DTOP_2A_L, 0);
//            MDRV_HDMIRX_MSG_INFO("High resolution 2\n");
        }

    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_SetUpdatePhaseLineCount(INPUT_SOURCE_TYPE_t enInputSourceType)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_SETUPDATEPHASELINECOUNT XCArgs;
    XCArgs.enInputSourceType = enInputSourceType;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SETUPDATEPHASELINECOUNT, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

void MDrv_HDMI_Get_PollingStatus_U2(void* pInstance, HDMI_POLLING_STATUS_t **pStatus)
{
    *pStatus = &g_HdmiPollingStatus;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Get_PollingStatus(HDMI_POLLING_STATUS_t **pStatus)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_GET_POLLINGSTATUS XCArgs;
    XCArgs.pStatus = NULL;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GET_POLLINGSTATUS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

MS_BOOL MDrv_HDMI_IsHDMI_Mode_U2(void* pInstance)
{
    MS_BOOL tmp;
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    tmp= pXCResourcePrivate->stdrv_HDMI.enStatus_info.b_St_HDMI_Mode;
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
    return tmp;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_IsHDMI_Mode(void)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_ISHDMI_MODE XCArgs;
    XCArgs.bReturnValue = FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_ISHDMI_MODE, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bReturnValue;
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_HDMI_EXT_COLORIMETRY_FORMAT MDrv_HDMI_Get_ExtColorimetry(void)
{
    stHDMI_GET_EXT_COLORIMETRY stHDMIRxArgs = {MS_HDMI_EXT_COLOR_UNKNOWN, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_COLORMETRY, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.enReturnValue;
}

#endif

MS_U16 MDrv_HDMI_GetHDE(void)
{
    return Hal_HDMI_GetHDEInfo(HDMI_INFO_SOURCE0);
}
#if 0
void MDrv_HDMI_ENABLE_IMMESWITCH( MS_BOOL bEnImmeswitch,MS_BOOL bEnPowSaving )
{
    if(SUPPORT_IMMESWITCH)
    {
        g_bIsIMMESWITCH = bEnImmeswitch;
        g_bIMMESWITCH_DVI_POWERSAVING = bEnPowSaving;
    }
    else
    {
        g_bIsIMMESWITCH = FALSE;
        g_bIMMESWITCH_DVI_POWERSAVING = FALSE;
    }
}
#endif

#if 0
void MDrv_HDMI_ImmeswitchPS_Handler(void)
{
    E_MUX_INPUTPORT port;
    if(g_bIMMESWITCH_DVI_POWERSAVING)
    {
        for(port = INPUT_PORT_DVI0; port <=INPUT_PORT_DVI3; port++)
        {
            //if(!Hal_DVI_GetPowerSavingStatus(port)) // not need to check if power saving mode is not enable
            //continue;

            if(Hal_DVI_GetDEStableStatus(port)) // stable
            {
                if(u8PSDVIStableCount[port-INPUT_PORT_DVI0] < 255)
                    u8PSDVIStableCount[port-INPUT_PORT_DVI0] += 1;
            }
            else // unstable
            {
                if(u8PSDVIStableCount[port-INPUT_PORT_DVI0] > 0)
                {
                    Hal_DVI_EnhanceImmeswitch(port, FALSE);
                    MDRV_HDMIRX_MSG_INFO("$$$ MDrv_HDMI_ImmeswitchPS_Handler:  port %d is unstable\n", port-INPUT_PORT_DVI0);
                }
                u8PSDVIStableCount[port-INPUT_PORT_DVI0] = 0;
            }

            if(u8PSDVIStableCount[port-INPUT_PORT_DVI0] >= 3) // enable Immeswithc enhancement only when continous DE stable
            {
                if(u8PSDVIStableCount[port-INPUT_PORT_DVI0] == 3)
                {
                    Hal_DVI_EnhanceImmeswitch(port, TRUE);
                    MDRV_HDMIRX_MSG_INFO("$$$ MDrv_HDMI_ImmeswitchPS_Handler:  port %d is stable\n", port-INPUT_PORT_DVI0);
                }
            }
        }
    }
}

void MDrv_DVI_ForceAllPortsEnterPS(void)
{
    if(g_bIMMESWITCH_DVI_POWERSAVING)
    {
        MDRV_HDMIRX_MSG_INFO("$$$ MDrv_DVI_ForceAllPortsEnterPS \n");
        Hal_DVI_ForceAllPortsEnterPS();
    }
}
#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_Check4K2K(void)
{
    stHDMI_CHECK_4K2K stHDMIRxArgs = {0};

    stHDMIRxArgs.bReturnValue = FALSE;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CHECK_VS_PACKET_4K2K, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_3D_4Kx2K_Process(void)
{
    stHDMI_3D_4K2K_PROCESS stHDMIRxArgs = {0};

    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_3D_4K2K_PROCESS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_AVG_ScaleringDown(MS_BOOL bTrue)
{
    stHDMI_AVG_SCALERINGDOWN stHDMIRxArgs = {0};

    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bTrue = bTrue;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_AVERAGE_SCALING_DOWN, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
E_HDMI_ADDITIONAL_VIDEO_FORMAT MDrv_HDMI_Check_Additional_Format(void)
{
    stHDMI_CHECK_ADDITIONAL_FORMAT stHDMIRxArgs = {E_HDMI_NA, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_ADDITIONAL_FORMAT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
E_XC_3D_INPUT_MODE MDrv_HDMI_Get_3D_Structure(void)
{
    stHDMI_GET_3D_STRUCTURE stHDMIRxArgs = {E_XC_3D_INPUT_MODE_NONE, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_3D_STRUCTURE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
E_HDMI_3D_EXT_DATA_T MDrv_HDMI_Get_3D_Ext_Data(void)
{
    stHDMI_GET_3D_EXT_DATA stHDMIRxArgs = {E_3D_EXT_DATA_MODE_MAX, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_3D_EXTEND_DATA, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_Get_3D_Meta_Field(sHDMI_3D_META_FIELD *pdata)
{
    stHDMI_GET_3D_META_FIELD stHDMIRxArgs = {0};

    stHDMIRxArgs.pdata = pdata;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_3D_META_FIELD, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U8 MDrv_HDMI_Get_VIC_Code(void)
{
    stHDMI_GET_VIC_CODE stHDMIRxArgs = {0};

    stHDMIRxArgs.u8ReturnValue = 0;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_VS_VIC_CODE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u8ReturnValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
E_HDMI_VIC_4Kx2K_CODE MDrv_HDMI_Get_4Kx2K_VIC_Code(void)
{
    stHDMI_GET_4KX2K_VIC_CODE stHDMIRxArgs = {E_VIC_NA, ucHDMIInfoSourceSelect};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_VS_4K2K_VIC_CODE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.eReturnValue;
}

#endif

// HDMI ARC pin control
//     - enInputPortType: INPUT_PORT_DVI0 / INPUT_PORT_DVI1 / INPUT_PORT_DVI2 / INPUT_PORT_DVI3
//     - bEnable: ARC enable or disable
//     - bDrivingHigh: ARC driving current high or low, suggest driving current should be set to high when ARC is enable
void MDrv_HDMI_ARC_PINControl_U2(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bEnable, MS_BOOL bDrivingHigh)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    Hal_HDMI_ARC_PINControl(enInputPortType, bEnable, bDrivingHigh);
#else
    // Do nothing
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_HDMI_ARC_PINControl(E_MUX_INPUTPORT enInputPortType, MS_BOOL bEnable, MS_BOOL bDrivingHigh)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_ARC_PINCONTROL XCArgs;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bEnable = bEnable;
    XCArgs.bDrivingHigh = bDrivingHigh;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_ARC_PINCONTROL, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

//[6]: soft-reset hdmi
//[5]: soft-reset hdcp
//[4]: soft-reset dvi
void MDrv_DVI_Software_Reset_U2(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U16 u16Reset)
{
#if SUPPORT_HDMI_RX_NEW_FEATURE
    Hal_DVI_Software_Reset(enInputPortType, u16Reset);
#else
    // Do nothing
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void MDrv_DVI_Software_Reset(E_MUX_INPUTPORT enInputPortType, MS_U16 u16Reset)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stDVI_SOFTWARE_RESET XCArgs;
    XCArgs.enInputPortType= enInputPortType;
    XCArgs.u16Reset = u16Reset;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_DVI_SOFTWARE_RESET, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

MS_U32 MDrv_HDMI_SetPowerState_U2(void* pInstance, EN_POWER_MODE enPowerState)
{
    MS_U16 u32Return = UTOPIA_STATUS_FAIL;
    //MS_CEC_INIT_INFO stCECInfo;
    XC_DDCRAM_PROG_INFO stDDCRamInfo;
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));
    if (enPowerState == E_POWER_SUSPEND)
    {
        pXCResourcePrivate->stdrv_HDMI._prev_enPowerState = enPowerState;
        u32Return = UTOPIA_STATUS_SUCCESS;//SUSPEND_OK;
    }
    else if (enPowerState == E_POWER_RESUME)
    {
        if (pXCResourcePrivate->stdrv_HDMI._prev_enPowerState == E_POWER_SUSPEND)
        {
            // EDID and DDC Ram
            {
            if(INPUT_PORT_DVI_END >= INPUT_PORT_DVI0)
            {
                // init Digital EDID 0
                stDDCRamInfo.EDID = pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8EDID_DVIBackUp[E_XC_PROG_DVI0_EDID];
                stDDCRamInfo.u16EDIDSize = EDID_KEY_LENGTH;
                stDDCRamInfo.eDDCProgType = E_XC_PROG_DVI0_EDID;
                stDDCRamInfo.u16CECPhyAddr = 0x10; //0x5555;
                stDDCRamInfo.u8CECPhyAddrIdxL = 0xA1;    // the CEC physical address start from 0xA1 byte in EDID
                stDDCRamInfo.u8CECPhyAddrIdxH = 0xA0;

                MDrv_HDMI_PROG_DDCRAM_internal(pInstance, &stDDCRamInfo, sizeof(stDDCRamInfo));
            }

            if(INPUT_PORT_DVI_END >= INPUT_PORT_DVI1)
            {
                // init Digital EDID 1
                stDDCRamInfo.EDID = pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8EDID_DVIBackUp[E_XC_PROG_DVI1_EDID];
                stDDCRamInfo.u16EDIDSize = EDID_KEY_LENGTH;
                stDDCRamInfo.eDDCProgType = E_XC_PROG_DVI1_EDID;
                stDDCRamInfo.u16CECPhyAddr = 0x20; //0x3131;
                stDDCRamInfo.u8CECPhyAddrIdxL = 0xA1;    // the CEC physical address start from 0xA1 byte in EDID
                stDDCRamInfo.u8CECPhyAddrIdxH = 0xA0;

                MDrv_HDMI_PROG_DDCRAM_internal(pInstance, &stDDCRamInfo, sizeof(stDDCRamInfo));
            }

            if(INPUT_PORT_DVI_END >= INPUT_PORT_DVI2)
            {
                // init Digital EDID 2
                stDDCRamInfo.EDID = pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8EDID_DVIBackUp[E_XC_PROG_DVI2_EDID];
                stDDCRamInfo.u16EDIDSize = EDID_KEY_LENGTH;
                stDDCRamInfo.eDDCProgType = E_XC_PROG_DVI2_EDID;
                stDDCRamInfo.u16CECPhyAddr = 0x30; //9292;
                stDDCRamInfo.u8CECPhyAddrIdxL = 0xA1;    // the CEC physical address start from 0xA1 byte in EDID
                stDDCRamInfo.u8CECPhyAddrIdxH = 0xA0;

                MDrv_HDMI_PROG_DDCRAM_internal(pInstance, &stDDCRamInfo, sizeof(stDDCRamInfo));
            }

            if(INPUT_PORT_DVI_END >= INPUT_PORT_DVI3)
            {
                // init Digital EDID 3
                stDDCRamInfo.EDID = pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8EDID_DVIBackUp[E_XC_PROG_DVI3_EDID];
                stDDCRamInfo.u16EDIDSize = EDID_KEY_LENGTH;
                stDDCRamInfo.eDDCProgType = E_XC_PROG_DVI3_EDID;
                stDDCRamInfo.u16CECPhyAddr = 0x40; //0x9292;
                stDDCRamInfo.u8CECPhyAddrIdxL = 0x9D;    // the CEC physical address start from 0xA1 byte in EDID
                stDDCRamInfo.u8CECPhyAddrIdxH = 0x9C;

                MDrv_HDMI_PROG_DDCRAM_internal(pInstance, &stDDCRamInfo, sizeof(stDDCRamInfo));
            }
            }

            E_MUX_INPUTPORT eHDMIPort;
            //clear all HPD
            for (eHDMIPort = INPUT_PORT_DVI0;eHDMIPort <= INPUT_PORT_DVI_END;eHDMIPort++)
            {
                MDrv_HDMI_pullhpd_internal(pInstance, DISABLE, eHDMIPort, pXCResourcePrivate->stdrvXC_MVideo_Context.g_bHDMIInverseBackUp[eHDMIPort - INPUT_PORT_DVI0]);
                Hal_DVI_ClkPullLow(TRUE, eHDMIPort);
                Hal_HDMI_DataRtermControl(eHDMIPort, FALSE);
            }

            // HDMI initial
            MDrv_HDMI_init_internal(pInstance);

            // init HDCP key again.
            if(pXCResourcePrivate->stdrv_HDMI.bHDCP14RxREEFlag)
            {
                MDrv_HDCP_initproductionkey_internal(pInstance, pXCResourcePrivate->stdrvXC_MVideo_Context.g_u8HdcpKeyDataBackUp);
            }

            // Delay 100ms
            MsOS_DelayTask(100);

            //set all HPD
            for (eHDMIPort = INPUT_PORT_DVI0;eHDMIPort <= INPUT_PORT_DVI_END;eHDMIPort++)
            {
                Hal_DVI_ClkPullLow(FALSE, eHDMIPort);
                Hal_HDMI_DataRtermControl(eHDMIPort, TRUE);
                MDrv_HDMI_pullhpd_internal(pInstance, ENABLE, eHDMIPort, pXCResourcePrivate->stdrvXC_MVideo_Context.g_bHDMIInverseBackUp[eHDMIPort - INPUT_PORT_DVI0]);
            }

#if 0
            // CEC
            stCECInfo.u32XTAL_CLK_Hz = MST_XTAL_CLOCK_HZ;
            MApi_CEC_Init(&stCECInfo);
#endif
            pXCResourcePrivate->stdrv_HDMI._prev_enPowerState = enPowerState;
            u32Return = UTOPIA_STATUS_SUCCESS;//RESUME_OK;
        }
        else
        {
            MDRV_HDMIRX_MSG_INFO("[%s,%5d]It is not suspended yet. We shouldn't resume\n",__func__,__LINE__);
            u32Return = UTOPIA_STATUS_FAIL;//SUSPEND_FAILED;
        }
    }
    else
    {
        MDRV_HDMIRX_MSG_INFO("[%s,%5d]Do Nothing: %d\n",__func__,__LINE__,enPowerState);
        u32Return = UTOPIA_STATUS_NOT_SUPPORTED;
    }
    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);
    return u32Return;// for success
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_U32 MDrv_HDMI_SetPowerState(EN_POWER_MODE enPowerState)
{
    stHDMI_SET_POWERSTATE XCArgs;
    stHDCP22_SET_EVENT stHDMIRxHDCPArgs;
    stHDMI_INITIAL_SETTING stHDMIRxArgs;

    XCArgs.enPowerState = enPowerState;
    XCArgs.u32ReturnValue = UTOPIA_STATUS_FAIL;
    stHDMIRxArgs.stInitialTable.bImmeswitchSupport = FALSE;
    stHDMIRxArgs.stInitialTable.ucMHLSupportPath = 0;
    stHDMIRxHDCPArgs.usHDCPEventIndex = HDMI_HDCP_EVENT_STR_SUSPEND;

    if(enPowerState == E_POWER_SUSPEND)
    {
        if(bHDCPInterruptProcFlag)
        {
            bHDCPInterruptProcFlag = FALSE;
        }

        if(pulHDMIRxInst == NULL)
        {
            if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
            }
        }

        if(pulHDMIRxInst != NULL)
        {
            if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_HDCP22_SET_EVENT, (void*)&stHDMIRxHDCPArgs) != 0)
            {
                MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
            }
        }
    }

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SET_POWERSTATE, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
    }
    else if(enPowerState == E_POWER_RESUME)
    {
        if(pulHDMIRxInst == NULL)
        {
            if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
            }
        }

        if(pulHDMIRxInst != NULL)
        {
            if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_INITIAL, (void*)&stHDMIRxArgs) != 0)
            {
                MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
            }
            else
            {
                if(!bHDCPInterruptCreateFlag)
                {
                    _mdrv_HDCP_CreateInterruptTask();

                    bHDCPInterruptCreateFlag = TRUE;
                }
            }
        }
    }

    return XCArgs.u32ReturnValue;
}

#endif

MS_BOOL MDrv_HDMI_CheckHDMI20_Setting_U2(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
#if SUPPORT_HDMI20
    return Hal_HDMI_CheckHDMI20_Setting(enInputPortType);
#else
    return FALSE;
#endif
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_CheckHDMI20_Setting(E_MUX_INPUTPORT enInputPortType)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_CHECK_HDMI20_SETTING XCArgs;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bReturnValue = FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_CHECK_HDMI20_SETTING, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bReturnValue;
    }
}

void MDrv_HDMI_StablePolling(void)
{
    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_STABLE_POLLING, NULL) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

MS_U8 MDrv_HDMI_GetSourceVersion(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_GET_SOURCE_VERSION stHDMIRxArgs = {enInputPortType, HDMI_SOURCE_VERSION_NOT_SURE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SOURCE_VERSION, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.ucSourceVersion;
}

#endif

void MDrv_HDMI_Set5VDetectGPIOSelect_U2(void* pInstance, MS_U32 ul5VDetectGPIOIndex)
{
    Hal_HDMI_Set5VDetectGPIOSelect(ul5VDetectGPIOIndex);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
void SYMBOL_WEAK MDrv_HDMI_Set5VDetectGPIOSelect(MS_U32 ul5VDetectGPIOIndex)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return;
    }

    stHDMI_SET_5V_DETECT_GPIO_SELECT XCArgs;
    XCArgs.ul5VDetectGPIOSelect = ul5VDetectGPIOIndex;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_SET_5V_DETECT_GPIO_SELECT, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return;
    }
    else
    {
        return;
    }
}

#endif

MS_BOOL MDrv_HDMI_GetDEStableStatus_U2(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    return Hal_DVI_GetDEStableStatus(enInputPortType);
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL SYMBOL_WEAK MDrv_HDMI_GetDEStableStatus(E_MUX_INPUTPORT enInputPortType)
{
    if (pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        return FALSE;
    }

    stHDMI_GET_DE_STABLE_STATUS XCArgs;
    XCArgs.enInputPortType = enInputPortType;
    XCArgs.bStatus= FALSE;

    if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_GET_DE_STABLE_STATUS, (void*)&XCArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        return FALSE;
    }
    else
    {
        return XCArgs.bStatus;
    }
}

MS_BOOL SYMBOL_WEAK MDrv_HDMI_CheckHDCP14(void)
{
    stHDMI_GET_HDCP14_KEY_VAILD XCArgs = {0};

    XCArgs.bHDCP14KeyVaildFlag = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HDCP14_KEY_VAILD, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return XCArgs.bHDCP14KeyVaildFlag;
}

E_HDMI_HDCP_STATE MDrv_HDMI_CheckHDCPState(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_CHECK_HDCP_STATE stHDMIRxArgs = {enInputPortType, E_HDCP_NO_ENCRYPTION};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CHECK_HDCP_STATE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return (E_HDMI_HDCP_STATE)stHDMIRxArgs.ucHDCPState;
}

E_HDMI_HDCP_ENCRYPTION_STATE MDrv_HDMI_CheckHDCPENCState(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_CHECK_HDCP_ENC_STATE stHDMIRxArgs = {enInputPortType, E_HDCP_NOT_ENCRYPTION};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CHECK_HDCP_ENC_STATE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return (E_HDMI_HDCP_ENCRYPTION_STATE)stHDMIRxArgs.ucHDCPENCState;
}

#endif

MS_BOOL MDrv_HDMI_SwitchVSDBtoHDRPacket(MS_BOOL bEnableHDR)
{
    Hal_HDMI_SwitchVSDBtoHDRPacket(bEnableHDR);
    return TRUE;
}

MS_BOOL MDrv_HDMI_ParseHDRInfoFrame(void* pInstance, MS_U8 ucHDMIInfoSource, void *pPacket, sHDR_METADATA *pHdrMetadata)
{
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;
    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));

    MS_BOOL bRet = FALSE;

    if(pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)
    {
        _mdrv_HDMI_ParseHDRInfoFrame(pPacket, pHdrMetadata, FALSE);

        bRet = TRUE;
    }

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    return bRet;
}

MS_BOOL MDrv_HDMI_Extend_Packet_Received(void* pInstance, MS_U8 ucHDMIInfoSource, MS_BOOL bEnable, MS_HDMI_EXTEND_PACKET_RECEIVE_t *pExtendPacketReceive)
{
    MS_BOOL bReturn = FALSE;
    MS_U16 u16CopiedLength = sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t);
    MS_HDMI_EXTEND_PACKET_RECEIVE_t stExtendPacketReceive = {0};
    XC_INSTANCE_PRIVATE *psXCInstPri = NULL;
    XC_RESOURCE_PRIVATE* pXCResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&psXCInstPri);
    _XC_SEMAPHORE_ENTRY(pInstance,E_XC_ID_VAR);

    UtopiaResourceGetPrivate(g_pXCResource[_XC_SELECT_INTERNAL_VARIABLE(psXCInstPri->u32DeviceID)],(void**)(&pXCResourcePrivate));

    if((NULL == pExtendPacketReceive) || (pExtendPacketReceive->u16Size == 0))
    {
        printf("MDrv_HDMI_Extend_Packet_Received: Null parameter or Wrong u16Size\n");
        return bReturn;
    }

    if(pExtendPacketReceive->u16Version < 1)
    {
        printf("MDrv_HDMI_Extend_Packet_Received: please check your u16Version, it should not set to 0\n");
        return bReturn;
    }

    if (pExtendPacketReceive->u16Version > HDMI_EXTEND_PACKET_RECEIVE_VERSION)
    {
#if defined (__aarch64__)
        printf("MDrv_HDMI_Extend_Packet_Received: this old version xc lib has only length:%lu driver status!!\n",sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t));
#else
        printf("MDrv_HDMI_Extend_Packet_Received: this old version xc lib has only length:%u driver status!!\n",sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t));
#endif
        u16CopiedLength = sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t);
    }

    if ((pExtendPacketReceive->u16Version < HDMI_EXTEND_PACKET_RECEIVE_VERSION) || (pExtendPacketReceive->u16Size < sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t)))
    {
        printf("MDrv_HDMI_Extend_Packet_Received: new version xc lib shouldn't access to the space which doesn't exist in old structure!!\n");
        u16CopiedLength = pExtendPacketReceive->u16Size;
    }

    stExtendPacketReceive.bPKT_MPEG_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_MPEG_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUI_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_SPD_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_SPD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AVI_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_GC_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_GCP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ASAMPLE_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ACR_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_ACR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_VS_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_NULL_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_NULL_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ISRC2_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_ISRC2_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ISRC1_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ACP_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_ACP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_ONEBIT_AUD_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_DSD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_GM_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_GM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_HBR_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_HBR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_VBI_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_VBI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_HDR_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_RSV_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_RESERVED_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_EDR_VALID = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_EDR_VALID_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_DST_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_DST_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_3D_ASP_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_3D_ASP_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_3D_DSD_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_3D_DSD_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_METADATA_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_METADATA_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_MULTI_ASP_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_MULTI_ASP_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_AUDIO_MULTI_DSD_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AUDIO_MULTI_DSD_RECEIVE_FLAG)? TRUE: FALSE;
    stExtendPacketReceive.bPKT_EM_RECEIVE = (pXCResourcePrivate->stdrv_HDMI.ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_EM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;

    memcpy(pExtendPacketReceive, &stExtendPacketReceive, u16CopiedLength);

    _XC_SEMAPHORE_RETURN(pInstance,E_XC_ID_VAR);

    return TRUE;
}

MS_BOOL MDrv_HDMI_Ctrl_U2(void* pInstance, MS_U8 ucHDMIInfoSource, MS_U32 u32Cmd, void *pBuf, MS_U32 u32BufSize)
{
    MS_BOOL bRet = FALSE;

    switch(u32Cmd)
    {
        case E_HDMI_SWITCH_VSDB_TO_HDR_PACKET:
            if (u32BufSize == sizeof(MS_HDMI_Switch_VSDB_to_HDR_Parameters))
            {
                MS_HDMI_Switch_VSDB_to_HDR_Parameters *pParameters = (MS_HDMI_Switch_VSDB_to_HDR_Parameters*)pBuf;
                bRet = MDrv_HDMI_SwitchVSDBtoHDRPacket(pParameters->bEnableHDR);
            }
            break;

        case E_HDMI_HDR_INFOFRAME_GET:
            if (u32BufSize == sizeof(MS_HDMI_HDR_InfoFrame_Get_Parameters))
            {
                MS_HDMI_HDR_InfoFrame_Get_Parameters *pParameters = (MS_HDMI_HDR_InfoFrame_Get_Parameters*)pBuf;
                MS_U16 HdrPacket[HDMI_HDR_PACKET_SIZE];
                Hal_HDMI_Get_InfoFrame(ucHDMIInfoSource, PKT_HDR, HdrPacket, NULL);
                bRet = MDrv_HDMI_ParseHDRInfoFrame(pInstance, ucHDMIInfoSource, HdrPacket, pParameters->pstHDRMetadata);
            }
            break;

        case E_HDMI_EXTEND_PACKET_RECEIVE_GET:
            if (u32BufSize == sizeof(MS_HDMI_Extend_Packet_Receive_Get_Parameters))
            {
                MS_HDMI_Extend_Packet_Receive_Get_Parameters *pParameters = (MS_HDMI_Extend_Packet_Receive_Get_Parameters*)pBuf;
                bRet = MDrv_HDMI_Extend_Packet_Received(pInstance, ucHDMIInfoSource, pParameters->bEnable, pParameters->pstPacketReceive);
            }
            break;

        default:
            break;
    }

    return bRet;
}

#ifndef UTOPIAXP_REMOVE_WRAPPER
MS_BOOL MDrv_HDMI_Ctrl(MS_U32 u32Cmd, void *pBuf, MS_U32 u32BufSize)
{
    MS_BOOL bRetValue = FALSE;

    if(u32Cmd == E_HDMI_HDR_INFOFRAME_GET)
    {
        u32Cmd = E_HDMI_INTERFACE_CMD_HDR_INFOFRAME_GET;
    }
    else if(u32Cmd == E_HDMI_EXTEND_PACKET_RECEIVE_GET)
    {
        u32Cmd = E_HDMI_INTERFACE_CMD_EXTEND_PACKET_RECEIVE_GET;
    }

    if(u32Cmd < E_HDMI_EXTEND_XC_RESOURCE_CMD_END) // XC resource
    {
        stHDMI_CONTROL XCArgs;

        XCArgs.u32Cmd = u32Cmd;
        XCArgs.pBuf = pBuf;
        XCArgs.u32BufSize = u32BufSize;
        XCArgs.bRet = FALSE;
        XCArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;

        if(pu32XCInst == NULL)
        {
            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
        }
        else if(UtopiaIoctl(pu32XCInst, E_XC_HDMI_IF_CMD_CTRL, (void*)&XCArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain XC engine fail\n");
        }

        bRetValue = XCArgs.bRet;
    }
    else // HDMIRX resource
    {
        bRetValue = MDrv_HDMI_IOCTL(u32Cmd, pBuf, u32BufSize);

        if(!bRetValue)
        {
            bRetValue = MDrv_EARC_IOCTL(u32Cmd, pBuf, u32BufSize);
        }
    }

    return bRetValue;
}

#endif

#ifndef UTOPIAXP_REMOVE_WRAPPER
//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetDataInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 MDrv_HDMI_GetDataInfo(E_HDMI_GET_DATA_INFO enInfo)
{
    stHDMI_GET_DATA_INFO stHDMIRxArgs = {0};

    stHDMIRxArgs.usReturnValue = 0;
    stHDMIRxArgs.enInfo = enInfo;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.bNewPathFlag = FALSE;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TIMING_INFO, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.usReturnValue;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetHDRInfoFrame
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_GetHDRInfoFrame(MS_U16 *pusHDRInfoFrame)
{
    stHDMI_GET_HDR_INFOFRAME stHDMIRxArgs = {0};

    stHDMIRxArgs.bHDRReceiveFlag = FALSE;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.pusHDRInfoFrame = pusHDRInfoFrame;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HDR_INFO_FRAME, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bHDRReceiveFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_ExtendPacketReceived
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_ExtendPacketReceived(MS_HDMI_EXTEND_PACKET_RECEIVE_t *pExtendPacketReceive)
{
    MS_BOOL bReturn = FALSE;
    MS_BOOL bInvalidFlag = FALSE;
    MS_U16 usCopiedLength = sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t);
    stHDMI_PACKET_RECEIVED stHDMIRxArgs = {0};
    MS_HDMI_EXTEND_PACKET_RECEIVE_t stExtendPacketReceive = {0};

    stHDMIRxArgs.bEnable = TRUE;
    stHDMIRxArgs.ucHDMIInfoSource = ucHDMIInfoSourceSelect;
    stHDMIRxArgs.ulPacketStatus = 0;

    if((NULL == pExtendPacketReceive) || (pExtendPacketReceive->u16Size == 0))
    {
        bInvalidFlag = TRUE;
        MDRV_HDMIRX_MSG_ERROR("MDrv_HDMI_Extend_Packet_Received: Null parameter or Wrong u16Size\n");
    }
    else if(pExtendPacketReceive->u16Version < 1)
    {
        bInvalidFlag = TRUE;
        MDRV_HDMIRX_MSG_ERROR("MDrv_HDMI_Extend_Packet_Received: please check your u16Version, it should not set to 0\n");
    }
    else if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            bInvalidFlag = TRUE;
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(!bInvalidFlag)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PACKET_RECEIVE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            if((pExtendPacketReceive->u16Version < HDMI_EXTEND_PACKET_RECEIVE_VERSION) || (pExtendPacketReceive->u16Size < sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t)))
            {
                MDRV_HDMIRX_MSG_ERROR("MDrv_HDMI_Extend_Packet_Received: new version xc lib shouldn't access to the space which doesn't exist in old structure!!\n");
                usCopiedLength = pExtendPacketReceive->u16Size;
            }

            stExtendPacketReceive.bPKT_MPEG_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_MPEG_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUI_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_SPD_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_SPD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AVI_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_GC_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_GCP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ASAMPLE_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ACR_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_ACR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_VS_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_NULL_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_NULL_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ISRC2_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_ISRC2_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ISRC1_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ACP_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_ACP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ONEBIT_AUD_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_DSD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_GM_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_GM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_HBR_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_HBR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_VBI_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_VBI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_HDR_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_RSV_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_RESERVED_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_EDR_VALID = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_EDR_VALID_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_DST_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_DST_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_3D_ASP_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_3D_ASP_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_3D_DSD_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_3D_DSD_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_METADATA_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_METADATA_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_MULTI_ASP_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_MULTI_ASP_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_MULTI_DSD_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_AUDIO_MULTI_DSD_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_EM_RECEIVE = (stHDMIRxArgs.ulPacketStatus &HDMI_STATUS_EM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;

            memcpy(pExtendPacketReceive, &stExtendPacketReceive, usCopiedLength);

            bReturn = TRUE;
        }
    }

    return bReturn;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_DataRtermControl
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDMI_DataRtermControl(E_MUX_INPUTPORT enInputPortType, MS_BOOL bDataRtermEnable)
{
    stHDMI_DATA_RTERM_CONTROL stHDMIRxArgs = {enInputPortType, bDataRtermEnable, FALSE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_DATA_RTERM_CONTROL, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetHDCPStatusPort
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDCP_STATUS_INFO_t MDrv_HDMI_GetHDCPStatusPort(E_MUX_INPUTPORT enInputPortType)
{
    MS_HDCP_STATUS_INFO_t stHDCPStatusInfo = {0};
    stHDMI_GET_HDCP_STATUS stHDMIRxArgs = {enInputPortType, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HDCP_STATUS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            stHDCPStatusInfo.b_St_HDMI_Mode = ((stHDMIRxArgs.usHDCPStatus &BIT(0))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_Reserve_1 = ((stHDMIRxArgs.usHDCPStatus &BIT(1))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_HDCP_Ver = ((stHDMIRxArgs.usHDCPStatus &BIT(2))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_AVMute = ((stHDMIRxArgs.usHDCPStatus &BIT(3))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_ENC_En = ((stHDMIRxArgs.usHDCPStatus &BIT(4))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_AKsv_Rcv = ((stHDMIRxArgs.usHDCPStatus &BIT(5))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_ENC_Dis = ((stHDMIRxArgs.usHDCPStatus &BIT(6))? TRUE: FALSE);
            stHDCPStatusInfo.b_St_Reserve_7 = ((stHDMIRxArgs.usHDCPStatus &BIT(7))? TRUE: FALSE);
        }
    }

    return stHDCPStatusInfo;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetSCDCValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 MDrv_HDMI_GetSCDCValue(E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset)
{
    stHDMI_GET_SCDC_VALUE stHDMIRxArgs = {enInputPortType, ucOffset, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SCDC_VALUE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.ucRetData;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetTMDSRatesKHz
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 MDrv_HDMI_GetTMDSRatesKHz(E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_TMDS_RATES enType)
{
    stHDMI_GET_TMDS_RATES_KHZ stHDMIRxArgs = {enInputPortType, enType, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TMDS_RATES_KHZ, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.ulRetRates;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetSCDCCableDetectFlag
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_GetSCDCCableDetectFlag(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_GET_SCDC_CABLE_DETECT stHDMIRxArgs = {enInputPortType, FALSE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SCDC_CABLE_DETECT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bCableDetectFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_AssignSourceSelect
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDMI_AssignSourceSelect(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_GET_SOURCE_SELECT stHDMIRxArgs = {enInputPortType, HDMI_INFO_SOURCE_NONE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SOURCE_SELECT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_ForcePowerDownPort
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_ForcePowerDownPort(E_MUX_INPUTPORT enInputPortType, MS_BOOL bPowerDown)
{
    MS_BOOL bDoneFlag = FALSE;
    stHDMI_FORCE_POWER_DOWN_PORT stHDMIRxArgs = {enInputPortType, bPowerDown};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_FORCE_POWER_DOWN_PORT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetPacketStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_GetPacketStatus(E_MUX_INPUTPORT enInputPortType, MS_HDMI_EXTEND_PACKET_RECEIVE_t *pExtendPacketReceive)
{
    MS_BOOL bGetStatusFlag = FALSE;
    MS_BOOL bInvalidFalg = FALSE;
    MS_U16 usCopiedLength = sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t);
    stHDMI_GET_PACKET_STATUS stHDMIRxArgs = {enInputPortType, 0};
    MS_HDMI_EXTEND_PACKET_RECEIVE_t stExtendPacketReceive = {0};

    if((NULL == pExtendPacketReceive) || (pExtendPacketReceive->u16Size == 0))
    {
        bInvalidFalg = TRUE;
        MDRV_HDMIRX_MSG_ERROR("MDrv_HDMI_Extend_Packet_Received: Null parameter or Wrong u16Size\n");
    }
    else if(pExtendPacketReceive->u16Version < 1)
    {
        bInvalidFalg = TRUE;
        MDRV_HDMIRX_MSG_ERROR("MDrv_HDMI_Extend_Packet_Received: please check your u16Version, it should not set to 0\n");
    }
    else if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            bInvalidFalg = TRUE;
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(!bInvalidFalg)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PACKET_STATUS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            if((pExtendPacketReceive->u16Version < HDMI_EXTEND_PACKET_RECEIVE_VERSION) || (pExtendPacketReceive->u16Size < sizeof(MS_HDMI_EXTEND_PACKET_RECEIVE_t)))
            {
                printf("MDrv_HDMI_Extend_Packet_Received: new version xc lib shouldn't access to the space which doesn't exist in old structure!!\n");
                usCopiedLength = pExtendPacketReceive->u16Size;
            }

            stExtendPacketReceive.bPKT_MPEG_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_MPEG_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUI_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_SPD_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_SPD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AVI_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_GC_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_GCP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ASAMPLE_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ACR_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_ACR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_VS_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_NULL_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_NULL_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ISRC2_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_ISRC2_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ISRC1_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ACP_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_ACP_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_ONEBIT_AUD_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_DSD_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_GM_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_GM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_HBR_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_HBR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_VBI_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_VBI_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_HDR_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_RSV_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_RESERVED_PACKET_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_EDR_VALID = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_EDR_VALID_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_DST_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_DST_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_3D_ASP_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_3D_ASP_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_3D_DSD_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_3D_DSD_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_METADATA_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_METADATA_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_MULTI_ASP_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_MULTI_ASP_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_AUDIO_MULTI_DSD_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_AUDIO_MULTI_DSD_RECEIVE_FLAG)? TRUE: FALSE;
            stExtendPacketReceive.bPKT_EM_RECEIVE = (stHDMIRxArgs.u32PacketStatus &HDMI_STATUS_EM_PACKET_RECEIVE_FLAG)? TRUE: FALSE;

            memcpy(pExtendPacketReceive, &stExtendPacketReceive, usCopiedLength);

            bGetStatusFlag = TRUE;
        }
    }

    return bGetStatusFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetPacketContent
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_GetPacketContent(E_MUX_INPUTPORT enInputPortType, MS_HDMI_PACKET_STATE_t enPacketType, MS_U8 u8PacketLength, MS_U8 *pu8PacketContent)
{
    MS_BOOL bGetContentFlag = FALSE;

    if(pu8PacketContent != NULL)
    {
        if(enPacketType == PKT_MULTI_VS)
        {
            stHDMI_GET_MULTIVS_PACKET_CONTENT stHDMIRxArgs;

            if(u8PacketLength > HDMI_GET_MULTIVS_PACKET_LENGTH)
            {
                u8PacketLength = HDMI_GET_MULTIVS_PACKET_LENGTH;
            }

            stHDMIRxArgs.bGetContentFlag = FALSE;
            stHDMIRxArgs.enInputPortType = (E_MUX_INPUTPORT)ucHDMIInfoSourceSelect;
            stHDMIRxArgs.u8PacketLength = u8PacketLength;
            stHDMIRxArgs.enPacketType = enPacketType;

            if(pulHDMIRxInst == NULL)
            {
                if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
                {
                    MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
                }
            }

            if(pulHDMIRxInst != NULL)
            {
                if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_MULTIVS_PACKET_CONTENT, (void*)&stHDMIRxArgs) != 0)
                {
                    MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                }
                else if(stHDMIRxArgs.bGetContentFlag)
                {
                    if(stHDMIRxArgs.u8PacketLength > u8PacketLength)
                    {
                        stHDMIRxArgs.u8PacketLength = u8PacketLength;
                    }

                    memcpy(pu8PacketContent, &stHDMIRxArgs.u8PacketContent, stHDMIRxArgs.u8PacketLength);

                    bGetContentFlag = TRUE;
                }
            }
        }
        else
        {
            stHDMI_GET_PACKET_CONTENT stHDMIRxArgs;

            if(u8PacketLength > HDMI_GET_PACKET_LENGTH)
            {
                u8PacketLength = HDMI_GET_PACKET_LENGTH;
            }

            stHDMIRxArgs.bGetContentFlag = FALSE;
            stHDMIRxArgs.enInputPortType = enInputPortType;
            stHDMIRxArgs.u8PacketLength = u8PacketLength;
            stHDMIRxArgs.enPacketType = enPacketType;

            if(pulHDMIRxInst == NULL)
            {
                if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
                {
                    MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
                }
            }

            if(pulHDMIRxInst != NULL)
            {
                if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PACKET_CONTENT, (void*)&stHDMIRxArgs) != 0)
                {
                    MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                }
                else if(stHDMIRxArgs.bGetContentFlag)
                {
                    if(stHDMIRxArgs.u8PacketLength > u8PacketLength)
                    {
                        stHDMIRxArgs.u8PacketLength = u8PacketLength;
                    }

                    memcpy(pu8PacketContent, &stHDMIRxArgs.u8PacketContent, stHDMIRxArgs.u8PacketLength);

                    bGetContentFlag = TRUE;
                }
            }
        }
    }

    return bGetContentFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_CreatePathInternal
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDMI_CreatePathInternal(INPUT_SOURCE_TYPE_t enInputPortType)
{
    stXC_GET_MUX_HDMIPORT XCArgs = {enInputPortType, INPUT_PORT_DVI0};
    stHDMI_CREATE_PATH stHDMIRxArgs = {INPUT_PORT_DVI0, HDMI_INFO_SOURCE0};

    if(pu32XCInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MApi_XC_Init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pu32XCInst, E_XC_CMD_GET_MUX_HDMIPORT, (void*)&XCArgs) != UTOPIA_STATUS_SUCCESS)
    {
        printf("Obtain XC engine fail\n");
    }
    else
    {
        stHDMIRxArgs.enInputPortType = XCArgs.eReturnValue;

        if(pulHDMIRxInst == NULL)
        {
            if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
            {
                MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
            }
        }

        if(pulHDMIRxInst != NULL)
        {
            if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CREATE_PATH, (void*)&stHDMIRxArgs) != 0)
            {
                MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
            }
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetTimingInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 MDrv_HDMI_GetTimingInfo(E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_DATA_INFO enInfoType)
{
    MS_U16 usInfoData = 0;
    stHDMI_GET_SOURCE_SELECT stHDMIRxArgs = {enInputPortType, HDMI_INFO_SOURCE_NONE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SOURCE_SELECT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else if(stHDMIRxArgs.u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            switch(enInfoType)
            {
                case E_HDMI_GET_HDE:
                case E_HDMI_GET_VDE:
                case E_HDMI_GET_HTT:
                case E_HDMI_GET_VTT:
                case E_HDMI_GET_CHIP_HDCP_CAPABILITY:
                case E_HDMI_GET_EMP_ENABLE_STATUS:
                    {
                        stHDMI_GET_DATA_INFO stHDMIRxArgs1 = {0};

                        stHDMIRxArgs1.usReturnValue = 0;
                        stHDMIRxArgs1.enInfo = enInfoType;
                        stHDMIRxArgs1.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
                        stHDMIRxArgs1.bNewPathFlag = TRUE;

                        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TIMING_INFO, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                        }
                        else
                        {
                            usInfoData = stHDMIRxArgs1.usReturnValue;
                        }
                    }
                    break;

                case E_HDMI_GET_AUDIO_PROTECT_INFO:
                    {
                        stHDMI_AUDIO_CP_HDR_INFO stHDMIRxArgs1 = {0};

                        stHDMIRxArgs1.u8ReturnValue = 0;
                        stHDMIRxArgs1.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
                        stHDMIRxArgs1.bNewPathFlag = TRUE;

                        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_AUDIO_CONTENT_PROTECT_INFO, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                        }
                        else
                        {
                            usInfoData = (MS_U16)stHDMIRxArgs1.u8ReturnValue;
                        }
                    }
                    break;

                case E_HDMI_GET_ERROR_STATUS:
                    {
                        stHDMI_ERR_STATUS_UPDATE stHDMIRxArgs1 = {0};

                        stHDMIRxArgs1.bread = TRUE;
                        stHDMIRxArgs1.u8ReturnValue = 0;
                        stHDMIRxArgs1.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
                        stHDMIRxArgs1.bNewPathFlag = TRUE;

                        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_ERROR_STATUS_UPDATE, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                        }
                        else
                        {
                            usInfoData = (MS_U16)stHDMIRxArgs1.u8ReturnValue;
                        }
                    }
                    break;

                case E_HDMI_GET_FRAMERATE:
                    {
                        usInfoData = (MS_U16)mdrv_HDMI_GetFrameRate(enInputPortType, enInfoType);
                    }
                    break;

                case E_HDMI_GET_CLOCKSTABLE_STATUS:
                    {
                        stHDMI_GET_CLOCKSTABLE_STATUS stHDMIRxArgs1 = {0};
                        stHDMIRxArgs1.enInputPortType = enInputPortType;
                        stHDMIRxArgs1.bClockStableStatus = FALSE;
                        if(pulHDMIRxInst == NULL)
                        {
                            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
                        }
                        else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_CLOCKSTABLE_STATUS, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail tucker MAPI_CMD_HDMIRX_CLOCKSTABLE_STATUS \n");
                        }
                        else
                        {
                            usInfoData = (MS_U16)stHDMIRxArgs1.bClockStableStatus;
                        }
                    }
                    break;

                case E_HDMI_GET_PIPELINE_MODE:
                    {
                        stHDMI_PIPELINE_MODE stHDMIRxArgs1 = {0};
                        if(pulHDMIRxInst == NULL)
                        {
                            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
                        }
                        else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_PIPELINE_MODE, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail MAPI_CMD_HDMIRX_GET_PIPELINE_MODE \n");
                        }
                        else
                        {
                            usInfoData = (MS_U16)stHDMIRxArgs1.enPipeLineMode;
                        }
                    }
                    break;

                case E_HDMI_GET_ISRC1_HEADER_INFO:
                    {
                        stHDMI_AUDIO_CP_HDR_INFO stHDMIRxArgs1 = {0};

                        stHDMIRxArgs1.u8ReturnValue = 0;
                        stHDMIRxArgs1.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
                        stHDMIRxArgs1.bNewPathFlag = TRUE;

                        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_ISRC1_HEADER_INFO, (void*)&stHDMIRxArgs1) != 0)
                        {
                            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
                        }
                        else
                        {
                            usInfoData = (MS_U16)stHDMIRxArgs1.u8ReturnValue;
                        }
                    }
                    break;

                default:
                    break;
            };
        }
    }

    return usInfoData;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_Get_EMP
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_Get_EMP(E_MUX_INPUTPORT enInputPortType, E_HDMI_EMPACKET_TYPE enEmpType, MS_U8 u8CurrentPacketIndex, MS_U8 *pu8TotalPacketNumber, MS_U8 *pu8EmPacket)
{
    ST_KDRV_XC_EMP_INFO stEmpInfo;
    EN_KDRV_HDMI_EMP_TYPE enXCEmpType;

    if(pu8TotalPacketNumber == NULL || pu8EmPacket == NULL)
    {
        MDRV_HDMIRX_MSG_ERROR("HDMI EMP pointer of total packet number or packet content should not be NULL\r\n");
        return FALSE;
    }

    switch(enEmpType)
    {
        case E_HDMI_EMP_DOLBY_HDR:
            enXCEmpType = E_KDRV_HDMI_EMP_DOLBY_HDR;
            break;

        case E_HDMI_EMP_DSC:
            enXCEmpType = E_KDRV_HDMI_EMP_DSC;
            break;

        case E_HDMI_EMP_DYNAMIC_HDR:
            enXCEmpType = E_KDRV_HDMI_EMP_DYNAMIC_HDR;
            break;

        case E_HDMI_EMP_VRR:
            enXCEmpType = E_KDRV_HDMI_EMP_VRR;
            break;

        case E_HDMI_EMP_QFT:
            enXCEmpType = E_KDRV_HDMI_EMP_QFT;
            break;

        case E_HDMI_EMP_QMS:
            enXCEmpType = E_KDRV_HDMI_EMP_QMS;
            break;

        case E_HDMI_EMP_MAX:
            enXCEmpType = E_KDRV_HDMI_EMP_MAX;
            break;

        default:
            enXCEmpType = E_KDRV_HDMI_EMP_MAX;
            break;
    };

    stEmpInfo.enHDMIPort = HDMI_GET_PORT_SELECT(enInputPortType);
    stEmpInfo.enEmpType = enXCEmpType;
    stEmpInfo.u8CurrentPacketIndex = u8CurrentPacketIndex;
    stEmpInfo.pu8TotalPacketNumber = pu8TotalPacketNumber;
    stEmpInfo.pu8EmPacket = pu8EmPacket;

    //user mode path
    #if (defined MSOS_TYPE_LINUX) || (defined ANDROID)
        MDRV_HDMIRX_MSG_ERROR(" Get EMP User Path!!!!\n");
        if(0 > _s32FdScaler)
        {
            _s32FdScaler = open("/dev/scaler", O_RDWR);

            if(0 > _s32FdScaler)
            {
                MDRV_HDMIRX_MSG_ERROR("Scaler device not opened!!!!\n");
                return E_APIXC_RET_FAIL;
            }
        }

        if (ioctl(_s32FdScaler, MDRV_XC_IOC_GET_EMPACKET, &stEmpInfo))
        {
            MDRV_HDMIRX_MSG_ERROR("MDRV_XC_IOC_HSE_CONTROL fail!!!!\n");
            return E_APIXC_RET_FAIL;
        }

    #endif

    //kernel mode path
    #ifdef MSOS_TYPE_LINUX_KERNEL
        MDRV_HDMIRX_MSG_ERROR(" Get EMP Kernel Path!!!!\n");
        if(KDrv_XC_GetEmPacket(stEmpInfo.enHDMIPort,
                               stEmpInfo.enEmpType,
                               stEmpInfo.u8CurrentPacketIndex,
                               stEmpInfo.pu8TotalPacketNumber,
                               stEmpInfo.pu8EmPacket) != TRUE)
        {
            return FALSE;
        }
    #endif

    return TRUE;
}


//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_GetAuthVersion
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_HDCP_STATE MDrv_HDCP_GetAuthVersion(E_MUX_INPUTPORT enInputPortType)
{
    stHDMI_HDCP_AUTHVERSION stHDMIRxArgs;

    stHDMIRxArgs.enInputPortType = enInputPortType;
    stHDMIRxArgs.enHDCPAuthVersion= E_HDCP_NO_ENCRYPTION;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HDCP_AUTHVERSION, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.enHDCPAuthVersion;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_WriteX74
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_WriteX74(E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset, MS_U8 ucData)
{
    stHDCP_WRITE_X74 stHDMIRxArgs = {enInputPortType, ucOffset, ucData};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_WRITE_X74, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_ReadX74
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 MDrv_HDCP_ReadX74(E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset)
{
    stHDCP_READ_X74 stHDMIRxArgs = {enInputPortType, ucOffset, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_READ_X74, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.ucRetData;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_SetRepeater
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_SetRepeater(E_MUX_INPUTPORT enInputPortType, MS_BOOL bIsRepeater)
{
    stHDCP_SET_REPEATER stHDMIRxArgs = {enInputPortType, bIsRepeater};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_REPEATER, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_SetBstatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_SetBstatus(E_MUX_INPUTPORT enInputPortType, MS_U16 usBstatus)
{
    stHDCP_SET_BSTATUS stHDMIRxArgs = {enInputPortType, usBstatus};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_BSTATUS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_SetHDMIMode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_SetHDMIMode(E_MUX_INPUTPORT enInputPortType, MS_BOOL bHDMIMode)
{
    stHDCP_SET_HDMI_MODE stHDMIRxArgs = {enInputPortType, bHDMIMode};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_HDMI_MODE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_GetInterruptStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 MDrv_HDCP_GetInterruptStatus(E_MUX_INPUTPORT enInputPortType)
{
    stHDCP_GET_INTERRUPT_STATUS stHDMIRxArgs = {enInputPortType, 0};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_INTERRUPT_STATUS, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.ucRetIntStatus;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_WriteKSVList
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_WriteKSVList(E_MUX_INPUTPORT enInputPortType, MS_U8* pucKSV, MS_U32 ulDataLen)
{
    stHDCP_WRITE_KSV_LIST stHDMIRxArgs = {enInputPortType, pucKSV, ulDataLen};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_WRITE_KSV_LIST, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP_SetVPrime
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP_SetVPrime(E_MUX_INPUTPORT enInputPortType, MS_U8* pucVPrime)
{
    stHDCP_SET_VPRIME stHDMIRxArgs = {enInputPortType, pucVPrime};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_VPRIME, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP14_InitResumeCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP14_InitResumeCBFunc(HDCP14_Resume_CBF pCBFunc, void* pContext)
{
    stHDCP14RXRESUME stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP14_INITRESUMECBFUNC, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}
//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP22_InitResumeCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP22_InitResumeCBFunc(HDCP22_Resume_CBF pCBFunc, void* pContext)
{
    stHDCP22RXRESUME stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP22_INITRESUMECBFUNC, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}
//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP14_InitSuspendCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP14_InitSuspendCBFunc(HDCP14_Suspend_CBF pCBFunc, void* pContext)
{
    stHDCP14RXSUSPEND stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP14_INITSUSPENDCBFUNC, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}


//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP22_InitSuspendCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP22_InitSuspendCBFunc(HDCP22_Suspend_CBF pCBFunc, void* pContext)
{
    stHDCP22RXSUSPEND stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP22_INITSUSPENDCBFUNC, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP14_InitSVPLockCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP14_InitSVPLockCBFunc(HDCP14_SVPLock_CBF pCBFunc, void* pContext)
{
    stHDCP14SVPLOCK stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP14_INITSVPLOCKCBFUNC, (void*)&stHDMIRxArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDCP14_InitAccessWriteProtectedRegCBFunc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void MDrv_HDCP14_InitAccessWriteProtectedRegCBFunc(HDCP14_AccessWriteProtectedReg_CBF pCBFunc, void* pContext)
{
    stHDCP14RX_ACCESS_WRITE_PROTECTED_REG stHDMIRxArgs = {pCBFunc, pContext};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDCP14_INITACCESSWRITEPROTECTEDREGCBFUNC, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Init
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Init(void)
{
    MS_BOOL bInitialFlag = FALSE;
    stEARC_INITIAL_SETTING stHDMIEARCArgs = {0};

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);

            bInitialFlag = TRUE;
        }
    }

    if(!bInitialFlag)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_INITIAL_SETTING, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
    }

    return stHDMIEARCArgs.bReDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Get_Connect_Status
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
EN_EARC_GET_CONNECT_STATUS MDrv_EARC_Get_Connect_Status(void)
{
    stEARC_GET_CONNECT_STATUS stHDMIEARCArgs = {0};

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_GET_CONNECT_STATUS, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
    }

    return (EN_EARC_GET_CONNECT_STATUS)stHDMIEARCArgs.u8EARCConnectStatus;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Get_Capability_Info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Get_Capability_Info(stEARC_CAPABILITY_INFO *pstEARCCapabilityInfo)
{
    MS_BOOL bDoneFlag = FALSE;

    stEARC_GET_CAPABILITY_INFO stHDMIEARCArgs = {{0}};


    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_GET_CAPABILITY_INFO, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else if(pstEARCCapabilityInfo != NULL)
        {
            //stHDMIEARCArgs.stCapabilityInfo.u16StructVersion = pstEARCCapabilityInfo->u16StructVersion;
            //stHDMIEARCArgs.stCapabilityInfo.u16StructSize = pstEARCCapabilityInfo->u16StructSize;
            memcpy(pstEARCCapabilityInfo, &stHDMIEARCArgs.stCapabilityInfo, pstEARCCapabilityInfo->u16StructSize);
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Get_Latency_Info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Get_Latency_Info(stEARC_LATENCY_INFO *pstEARCLatencyInfo)
{
    MS_BOOL bDoneFlag = FALSE;
    stEARC_GET_LATENCY_INFO stHDMIEARCArgs = {{0}};

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_GET_LATENCY_INFO, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else if(pstEARCLatencyInfo != NULL)
        {
            //stHDMIEARCArgs.stEARCLatencyInfo.u16StructVersion = pstEARCLatencyInfo->u16StructVersion;
            //stHDMIEARCArgs.stEARCLatencyInfo.u16StructSize = pstEARCLatencyInfo->u16StructSize;
            memcpy(pstEARCLatencyInfo, &stHDMIEARCArgs.stEARCLatencyInfo, pstEARCLatencyInfo->u16StructSize);

            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Set_ATOPSetting_AUPLL
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Set_ATOPSetting_AUPLL(stEARC_AUDIO_INFO *pstEARCAudioInfo)
{
    MS_BOOL bDoneFlag = FALSE;
    stEARC_SET_ATOPSETTING_AUPLL stHDMIEARCArgs;

    stHDMIEARCArgs.stAudioInfo.enAudioFrequency = pstEARCAudioInfo->enAudioFrequency;
    stHDMIEARCArgs.stAudioInfo.enAudioChannelLayout = pstEARCAudioInfo->enAudioChannelLayout;

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_SET_ATOPSETTING_AUPLL, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Set_AudioOutputStream
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Set_AudioOutputStream(MS_BOOL bEnableFlag)
{
    MS_BOOL bDoneFlag = FALSE;
    stEARC_SET_AUDIO_OUTPUT_STREAM stHDMIEARCArgs;

    stHDMIEARCArgs.bEnableFlag = bEnableFlag;

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_SET_AUDIO_OUTPUT_STREAM, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Set_SupportMode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_Set_SupportMode(EN_EARC_SUPPORT_MODE enSupportMode)
{
    MS_BOOL bDoneFlag = FALSE;
    stEARC_SET_SUPPORT_MODE stHDMIEARCArgs;

    stHDMIEARCArgs.enSupportMode = enSupportMode;

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_SET_SUPPORT_MODE, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Get_ChipCapability
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
EN_EARC_SUPPORT_MODE MDrv_EARC_Get_ChipCapability(void)
{
    stEARC_GET_CHIP_CAPABILITY stHDMIEARCArgs = {0};

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_GET_CHIPCAPABILITY, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
    }

    return (EN_EARC_SUPPORT_MODE)stHDMIEARCArgs.u8ChipCapability;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_Set_OutputPortSelect
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_OutputPortSelect(E_MUX_INPUTPORT enInputPortType, MS_BOOL bEnable5VDetectInvert, MS_BOOL bEnableHPDInvert)
{
    MS_BOOL bDoneFlag = FALSE;
    stEARC_OUTPUT_PORT_SELECT stHDMIEARCArgs;

    stHDMIEARCArgs.enInputPortType = enInputPortType;
    stHDMIEARCArgs.bEnable5VDetectInvert = bEnable5VDetectInvert;
    stHDMIEARCArgs.bEnableHPDInvert = bEnableHPDInvert;

    if(pulHDMIEARCInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIEARC, &pulHDMIEARCInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIEARCInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIEARCInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIEARCInst, MAPI_CMD_HDMIEARC_OUTPUT_PORT_SELECT, (void*)&stHDMIEARCArgs) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]Obtain HDMI EARC engine fail\n",__FUNCTION__,__LINE__);
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetCRCValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_GetCRCValue(E_MUX_INPUTPORT enInputPortType, MS_DVI_CHANNEL_TYPE u8Channel, MS_U16* pu16CRCVal)
{
    MS_BOOL bVaildFlag = FALSE;
    stCMD_HDMI_CMD_GET_CRC_VALUE stHDMIRxArgs;

    stHDMIRxArgs.enInputPortType = enInputPortType;
    stHDMIRxArgs.u8Channel =u8Channel;
    stHDMIRxArgs.bIsValid = FALSE;
    stHDMIRxArgs.u16CRCVal = pu16CRCVal;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_CRC_VALUE, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.bIsValid;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_SetHotplugDelay
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_SetHotplugDelay(MS_U8 u8HotplugDelay)
{
    MS_BOOL bDoneFlag = FALSE;
    stHDMI_SET_HPD_DELAY stHDMIRxArgs;
    stHDMIRxArgs.u8HotplugDelay = u8HotplugDelay;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_SET_HPD_DELAY, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else
        {
            bDoneFlag = TRUE;
        }
    }

    return bDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_Get_HW_Ability
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 MDrv_HDMI_Get_HW_Ability(E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_WHICH_HW_ABILITY enWhichAbility)
{
    MS_BOOL bVaildFlag = FALSE;
    stHDMI_HW_ABILITY stHDMIRxArgs;

    stHDMIRxArgs.enInputPortType = enInputPortType;
    stHDMIRxArgs.enWhichAbility = enWhichAbility;
    stHDMIRxArgs.u32Result = 0;

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HW_ABILITY, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
    }

    return stHDMIRxArgs.u32Result;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_GetHDMI_Status
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 MDrv_HDMI_GetHDMI_Status(E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_HDMI_STATUS_IDX enInfoType, MS_U8 u8Param)
{
    MS_U16 usInfoData = 0;
    stHDMI_GET_SOURCE_SELECT stHDMIRxArgs = {enInputPortType, HDMI_INFO_SOURCE_NONE};

    if(pulHDMIRxInst == NULL)
    {
        if(UtopiaOpen(MODULE_HDMIRX, &pulHDMIRxInst, 0, NULL) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d]No instance existed, UtopiaOpen pulHDMIRxInst failed\n",__FUNCTION__,__LINE__);
        }
    }

    if(pulHDMIRxInst != NULL)
    {
        if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SOURCE_SELECT, (void*)&stHDMIRxArgs) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        else if(stHDMIRxArgs.u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
             stHDMI_GET_HDMI_STATUS stHDMIRxArgs1 = {0};
             stHDMIRxArgs1.enInputPortType = enInputPortType;
             stHDMIRxArgs1.enInfoType = enInfoType;
             stHDMIRxArgs1.u8Param = u8Param;
             stHDMIRxArgs1.u16Ret = 0;

             if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_HDMI_STATUS, (void*)&stHDMIRxArgs1) != 0)
             {
                 MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
             }
             else
             {
                 usInfoData = (MS_U16)stHDMIRxArgs1.u16Ret;
             }
        }
    }

    return usInfoData;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_HDMI_IOCTL
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_HDMI_IOCTL(MS_U32 ulCommand, void *pBuffer, MS_U32 ulBufferSize)
{
    MS_BOOL bret = FALSE;

    switch(ulCommand)
    {
        case E_HDMI_INTERFACE_CMD_GET_HDCP_STATE:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_CHECK_HDCP_STATE))
                {
                    ((stCMD_HDMI_CHECK_HDCP_STATE *)pBuffer)->ucHDCPState = (MS_U8)MDrv_HDMI_CheckHDCPState(((stCMD_HDMI_CHECK_HDCP_STATE *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
                else
                {
                    ((stCMD_HDMI_CHECK_HDCP_STATE *)pBuffer)->ucHDCPState  = HDMI_HDCP_NO_ENCRYPTION;
                }
            }
            break;

        /*************************** HDCP Repeater ***************************/
        case E_HDMI_INTERFACE_CMD_WRITE_X74:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_WRITE_X74))
                {
                    MDrv_HDCP_WriteX74(((stCMD_HDCP_WRITE_X74 *)pBuffer)->enInputPortType, ((stCMD_HDCP_WRITE_X74 *)pBuffer)->ucOffset, ((stCMD_HDCP_WRITE_X74 *)pBuffer)->ucData);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_READ_X74:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_READ_X74))
                {
                    ((stCMD_HDCP_READ_X74 *)pBuffer)->ucRetData = MDrv_HDCP_ReadX74(((stCMD_HDCP_READ_X74 *)pBuffer)->enInputPortType, ((stCMD_HDCP_READ_X74 *)pBuffer)->ucOffset);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_SET_REPEATER:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_SET_REPEATER))
                {
                    MDrv_HDCP_SetRepeater(((stCMD_HDCP_SET_REPEATER *)pBuffer)->enInputPortType, ((stCMD_HDCP_SET_REPEATER *)pBuffer)->bIsRepeater);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_SET_BSTATUS:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_SET_BSTATUS))
                {
                    MDrv_HDCP_SetBstatus(((stCMD_HDCP_SET_BSTATUS *)pBuffer)->enInputPortType, ((stCMD_HDCP_SET_BSTATUS *)pBuffer)->usBstatus);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_SET_HDMI_MODE:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_SET_HDMI_MODE))
                {
                    MDrv_HDCP_SetHDMIMode(((stCMD_HDCP_SET_HDMI_MODE *)pBuffer)->enInputPortType, ((stCMD_HDCP_SET_HDMI_MODE *)pBuffer)->bHDMIMode);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_INTERRUPT_STATUS:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_GET_INTERRUPT_STATUS))
                {
                    ((stCMD_HDCP_GET_INTERRUPT_STATUS *)pBuffer)->ucRetIntStatus = MDrv_HDCP_GetInterruptStatus(((stCMD_HDCP_GET_INTERRUPT_STATUS *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_WRITE_KSV_LIST:
            {
                MS_U8 ucKSVBuffer[HDMI_HDCP_KSV_LIST_LENGTH] = {0};
                MS_U8 *pucKSVBuffer = NULL;
                MS_U32 ultemp = 0;
                MS_U32 ulLength = 0;

                if(ulBufferSize == sizeof(stCMD_HDCP_WRITE_KSV_LIST))
                {
                    ulLength = ((stCMD_HDCP_WRITE_KSV_LIST *)pBuffer)->ulLen;
                    pucKSVBuffer = ((stCMD_HDCP_WRITE_KSV_LIST *)pBuffer)->pucKSV;

                    if(ulLength > HDMI_HDCP_KSV_LIST_LENGTH)
                    {
                        ulLength = HDMI_HDCP_KSV_LIST_LENGTH;
                    }

                    for(ultemp = 0; ultemp < ulLength; ultemp++)
                    {
                        ucKSVBuffer[ultemp] = pucKSVBuffer[ultemp];
                    }

                    MDrv_HDCP_WriteKSVList(((stCMD_HDCP_WRITE_KSV_LIST *)pBuffer)->enInputPortType, ucKSVBuffer, ulLength);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_SET_VPRIME:
            {
                if(ulBufferSize == sizeof(stCMD_HDCP_SET_VPRIME))
                {
                    MDrv_HDCP_SetVPrime(((stCMD_HDCP_SET_VPRIME *)pBuffer)->enInputPortType, ((stCMD_HDCP_SET_VPRIME *)pBuffer)->pucVPrime);

                    bret = TRUE;
                }
            }
            break;
        /*************************** HDCP Repeater ***************************/
        case E_HDMI_INTERFACE_CMD_GET_HDCP_ENC_STATE:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_CHECK_HDCP_ENC_STATE))
                {
                    ((stCMD_HDMI_CHECK_HDCP_ENC_STATE *)pBuffer)->ucHDCPENCState = (MS_U8)MDrv_HDMI_CheckHDCPENCState(((stCMD_HDMI_CHECK_HDCP_ENC_STATE *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
                else
                {
                    ((stCMD_HDMI_CHECK_HDCP_ENC_STATE *)pBuffer)->ucHDCPENCState  = HDMI_HDCP_NOT_ENCRYPTION;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDR_INFOFRAME_GET:
            {
                MS_U16 ucHDRInfoFrame[HDMI_HDR_PACKET_SIZE] = {0};

                if(ulBufferSize == sizeof(MS_HDMI_HDR_InfoFrame_Get_Parameters))
                {
                    MS_HDMI_HDR_InfoFrame_Get_Parameters *pParameters = (MS_HDMI_HDR_InfoFrame_Get_Parameters*)pBuffer;

                    if(MDrv_HDMI_GetHDRInfoFrame(ucHDRInfoFrame))
                    {
                        _mdrv_HDMI_ParseHDRInfoFrame(ucHDRInfoFrame, pParameters->pstHDRMetadata, FALSE);

                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_EXTEND_PACKET_RECEIVE_GET:
            {
                if(ulBufferSize == sizeof(MS_HDMI_Extend_Packet_Receive_Get_Parameters))
                {
                    MS_HDMI_Extend_Packet_Receive_Get_Parameters *pParameters = (MS_HDMI_Extend_Packet_Receive_Get_Parameters*)pBuffer;

                    bret = MDrv_HDMI_ExtendPacketReceived(pParameters->pstPacketReceive);
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_DATA_RTERM_CONTROL:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_DATA_RTERM_CONTROL))
                {
                    MDrv_HDMI_DataRtermControl(((stCMD_HDMI_DATA_RTERM_CONTROL *)pBuffer)->enInputPortType, ((stCMD_HDMI_DATA_RTERM_CONTROL *)pBuffer)->bDataRtermEnable);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_HDCP_STATUS:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_HDCP_STATUS))
                {
                    ((stCMD_HDMI_GET_HDCP_STATUS *)pBuffer)->stHDCPStatusInfo = MDrv_HDMI_GetHDCPStatusPort(((stCMD_HDMI_GET_HDCP_STATUS *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_SCDC_VALUE:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_SCDC_VALUE))
                {
                    ((stCMD_HDMI_GET_SCDC_VALUE *)pBuffer)->ucRetData = MDrv_HDMI_GetSCDCValue(((stCMD_HDMI_GET_SCDC_VALUE *)pBuffer)->enInputPortType, ((stCMD_HDMI_GET_SCDC_VALUE *)pBuffer)->ucOffset);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_TMDS_RATES_KHZ:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_TMDS_RATES_KHZ))
                {
                    ((stCMD_HDMI_GET_TMDS_RATES_KHZ *)pBuffer)->ulRetRates = MDrv_HDMI_GetTMDSRatesKHz(((stCMD_HDMI_GET_TMDS_RATES_KHZ *)pBuffer)->enInputPortType, ((stCMD_HDMI_GET_TMDS_RATES_KHZ *)pBuffer)->enType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_SCDC_CABLE_DETECT:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_SCDC_CABLE_DETECT))
                {
                    ((stCMD_HDMI_GET_SCDC_CABLE_DETECT *)pBuffer)->bCableDetectFlag = MDrv_HDMI_GetSCDCCableDetectFlag(((stCMD_HDMI_GET_SCDC_CABLE_DETECT *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_ASSIGN_SOURCE_SELECT:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_ASSIGN_SOURCE_SELECT))
                {
                    MDrv_HDMI_AssignSourceSelect(((stCMD_HDMI_ASSIGN_SOURCE_SELECT *)pBuffer)->enInputPortType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_PACKET_STATUS:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_PACKET_STATUS))
                {
                    stCMD_HDMI_GET_PACKET_STATUS *pParameters = (stCMD_HDMI_GET_PACKET_STATUS*)pBuffer;

                    if(MDrv_HDMI_GetPacketStatus(pParameters->enInputPortType, pParameters->pExtendPacketReceive))
                    {
                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_PACKET_CONTENT:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_PACKET_CONTENT))
                {
                    stCMD_HDMI_GET_PACKET_CONTENT *pParameters = (stCMD_HDMI_GET_PACKET_CONTENT*)pBuffer;

                    if(MDrv_HDMI_GetPacketContent(pParameters->enInputPortType, pParameters->enPacketType, pParameters->u8PacketLength, pParameters->pu8PacketContent))
                    {
                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_HDR_METADATA:
            {
                MS_U8 u8PacketContent[HDMI_GET_PACKET_LENGTH] = {0};

                if(ulBufferSize == sizeof(stCMD_HDMI_GET_HDR_METADATA))
                {
                    stCMD_HDMI_GET_HDR_METADATA *pParameters = (stCMD_HDMI_GET_HDR_METADATA*)pBuffer;

                    if(MDrv_HDMI_GetPacketContent(pParameters->enInputPortType, PKT_HDR, HDMI_GET_PACKET_LENGTH, u8PacketContent))
                    {
                        _mdrv_HDMI_ParseHDRInfoFrame(u8PacketContent, pParameters->pstHDRMetadata, TRUE);

                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_AVI_PARSING_INFO:
            {
                MS_U8 u8PacketContent[HDMI_GET_PACKET_LENGTH] = {0};

                if(ulBufferSize == sizeof(stCMD_HDMI_GET_AVI_PARSING_INFO))
                {
                    stCMD_HDMI_GET_AVI_PARSING_INFO *pParameters = (stCMD_HDMI_GET_AVI_PARSING_INFO*)pBuffer;

                    if(MDrv_HDMI_GetPacketContent(pParameters->enInputPortType, PKT_AVI, HDMI_GET_PACKET_LENGTH, u8PacketContent))
                    {
                        _mdrv_HDMI_ParseAVIInfoFrame(u8PacketContent, pParameters->pstAVIParsingInfo);

                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_VS_PARSING_INFO:
            {
                MS_U8 u8PacketContent[HDMI_GET_PACKET_LENGTH] = {0};

                if(ulBufferSize == sizeof(stCMD_HDMI_GET_VS_PARSING_INFO))
                {
                    stCMD_HDMI_GET_VS_PARSING_INFO *pParameters = (stCMD_HDMI_GET_VS_PARSING_INFO*)pBuffer;

                    if(MDrv_HDMI_GetPacketContent(pParameters->enInputPortType, PKT_VS, HDMI_GET_PACKET_LENGTH, u8PacketContent))
                    {
                        _mdrv_HDMI_ParseVSInfoFrame(u8PacketContent, pParameters->pstVSParsingInfo);

                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_GC_PARSING_INFO:
            {
                MS_U8 ucPacketContent[HDMI_GET_PACKET_LENGTH] = {0};

                if(ulBufferSize == sizeof(stCMD_HDMI_GET_GC_PARSING_INFO))
                {
                    stCMD_HDMI_GET_GC_PARSING_INFO *pParameters = (stCMD_HDMI_GET_GC_PARSING_INFO*)pBuffer;

                    if(MDrv_HDMI_GetPacketContent(pParameters->enInputPortType, PKT_GC, HDMI_GET_PACKET_LENGTH, ucPacketContent))
                    {
                        _mdrv_HDMI_ParseGCInfoFrame(ucPacketContent, pParameters->pstGCParsingInfo);

                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_TIMING_INFO:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_TIMING_INFO))
                {
                    stCMD_HDMI_GET_TIMING_INFO *pParameters = (stCMD_HDMI_GET_TIMING_INFO*)pBuffer;

                    pParameters->u16TimingInfo = MDrv_HDMI_GetTimingInfo(pParameters->enInputPortType, pParameters->enInfoType);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_HDCP_AUTHVERSION:
        {
            if(ulBufferSize == sizeof(stCMD_HDMI_GET_HDCP_AUTHVERSION))
            {
                ((stCMD_HDMI_GET_HDCP_AUTHVERSION *)pBuffer)->enHDCPAuthVersion = MDrv_HDCP_GetAuthVersion(((stCMD_HDMI_GET_HDCP_AUTHVERSION *)pBuffer)->enInputPortType);

                bret = TRUE;
            }
        }
        break;

        case E_HDMI_INTERFACE_CMD_HDCP14_INITRESUMECBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP14RXRESUME))
                {
                    MDrv_HDCP14_InitResumeCBFunc(((stHDCP14RXRESUME *)pBuffer)->fpHdcp14RxResumeCBFunc, ((stHDCP14RXRESUME *)pBuffer)->pContext);
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDCP22_INITRESUMECBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP22RXRESUME))
                {
                    if(((stHDCP22RXRESUME *)pBuffer)->fpHdcp22RxResumeCBFunc != NULL)
                    {
                        MDrv_HDCP22_InitResumeCBFunc(((stHDCP22RXRESUME *)pBuffer)->fpHdcp22RxResumeCBFunc, ((stHDCP22RXRESUME *)pBuffer)->pContext);
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDCP14_INITSUSPENDCBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP14RXSUSPEND))
                {
                    if(((stHDCP14RXSUSPEND *)pBuffer)->fpHdcp14RxSuspendCBFunc != NULL)
                    {
                        MDrv_HDCP14_InitSuspendCBFunc(((stHDCP14RXSUSPEND *)pBuffer)->fpHdcp14RxSuspendCBFunc, ((stHDCP14RXSUSPEND *)pBuffer)->pContext);
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDCP22_INITSUSPENDCBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP22RXSUSPEND))
                {
                    if(((stHDCP22RXSUSPEND *)pBuffer)->fpHdcp22RxSuspendCBFunc != NULL)
                    {
                        MDrv_HDCP22_InitSuspendCBFunc(((stHDCP22RXSUSPEND *)pBuffer)->fpHdcp22RxSuspendCBFunc, ((stHDCP22RXSUSPEND *)pBuffer)->pContext);
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDCP14_INITSVPLOCKCBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP14SVPLOCK))
                {
                    if(((stHDCP14SVPLOCK *)pBuffer)->fpHdcp14RxSVPLockCBFunc != NULL)
                    {
                        MDrv_HDCP14_InitSVPLockCBFunc(((stHDCP14SVPLOCK *)pBuffer)->fpHdcp14RxSVPLockCBFunc, ((stHDCP14SVPLOCK *)pBuffer)->pContext);
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_HDCP14_INITACCESSWRITEPROTECTEDREGCBFUNC:
            {
                if(ulBufferSize == sizeof(stHDCP14RX_ACCESS_WRITE_PROTECTED_REG))
                {
                    if(((stHDCP14RX_ACCESS_WRITE_PROTECTED_REG *)pBuffer)->fpHdcp14RxAccessWriteProtectedRegCBFunc != NULL)
                    {
                        MDrv_HDCP14_InitAccessWriteProtectedRegCBFunc(((stHDCP14RX_ACCESS_WRITE_PROTECTED_REG *)pBuffer)->fpHdcp14RxAccessWriteProtectedRegCBFunc, ((stHDCP14RX_ACCESS_WRITE_PROTECTED_REG *)pBuffer)->pContext);
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_FORCE_POWER_DOWN_PORT:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_FORCE_POWER_DOWN_PORT))
                {
                    stCMD_HDMI_FORCE_POWER_DOWN_PORT *pParameters = (stCMD_HDMI_FORCE_POWER_DOWN_PORT*)pBuffer;
                    if(MDrv_HDMI_ForcePowerDownPort(pParameters->enInputPortType, pParameters->bPowerDown))
                    {
                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_CRC_VALUE:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_CMD_GET_CRC_VALUE))
                {
                    stCMD_HDMI_CMD_GET_CRC_VALUE *pParameters = (stCMD_HDMI_CMD_GET_CRC_VALUE*)pBuffer;

                    pParameters->bIsValid = MDrv_HDMI_GetCRCValue(pParameters->enInputPortType, pParameters->u8Channel, pParameters->u16CRCVal);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_EMP:
            if(ulBufferSize == sizeof(ST_KDRV_XC_EMP_INFO))
            {
                bret = MDrv_HDMI_Get_EMP(((ST_KDRV_XC_EMP_INFO *)pBuffer)->enHDMIPort,
                                     ((ST_KDRV_XC_EMP_INFO *)pBuffer)->enEmpType,
                                     ((ST_KDRV_XC_EMP_INFO *)pBuffer)->u8CurrentPacketIndex,
                                     ((ST_KDRV_XC_EMP_INFO *)pBuffer)->pu8TotalPacketNumber,
                                     ((ST_KDRV_XC_EMP_INFO *)pBuffer)->pu8EmPacket);
            }
            break;

        case E_HDMI_INTERFACE_CMD_SET_HPD_DELAY:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_CMD_SET_HPD_DELAY))
                {
                    stCMD_HDMI_CMD_SET_HPD_DELAY *pParameters = (stCMD_HDMI_CMD_SET_HPD_DELAY*)pBuffer;
                    if(MDrv_HDMI_SetHotplugDelay(pParameters->u8HotplugDelay))
                    {
                        bret = TRUE;
                    }
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_HW_ABILITY:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_CMD_GET_HW_ABILITY))
                {
                    stCMD_HDMI_CMD_GET_HW_ABILITY *pParameters = (stCMD_HDMI_CMD_GET_HW_ABILITY*)pBuffer;

                    pParameters->u32Result = MDrv_HDMI_Get_HW_Ability(pParameters->enInputPortType, pParameters->enWhichAbility);

                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_INTERFACE_CMD_GET_HDMI_STATUS:
            {
                if(ulBufferSize == sizeof(stCMD_HDMI_GET_HDMI_STATUS))
                {
                    stCMD_HDMI_GET_HDMI_STATUS *pParameters = (stCMD_HDMI_GET_HDMI_STATUS*)pBuffer;

                    pParameters->u16Ret = MDrv_HDMI_GetHDMI_Status(pParameters->enInputPortType, pParameters->enInfoType, pParameters->u8Param);

                    if(pParameters->u16Ret)
                    {
                        bret = TRUE;
                    }
                }
            }
            break;

        default:
            //pBuffer = NULL;
            //ulBufferSize = 0;
            bret = FALSE;
            break;
    };

    return bret;
}

//**************************************************************************
//  [Function Name]:
//                  MDrv_EARC_IOCTL
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL MDrv_EARC_IOCTL(MS_U32 ulCommand, void *pBuffer, MS_U32 ulBufferSize)
{
    MS_BOOL bret = FALSE;

    switch(ulCommand)
    {
        case E_HDMI_EARC_CMD_EARC_INIT:
            if(ulBufferSize == sizeof(stCMD_EARC_INITIAL_SETTING))
            {
                stCMD_EARC_INITIAL_SETTING *pParameters = (stCMD_EARC_INITIAL_SETTING*)pBuffer;

                pParameters->bDoneFlag = MDrv_EARC_Init();

                bret = TRUE;

            }
            break;

        case E_HDMI_EARC_CMD_GET_CONNECT_STATUS:
            if(ulBufferSize == sizeof(stCMD_EARC_GET_CONNECT_STATUS))
            {
                stCMD_EARC_GET_CONNECT_STATUS *pParameters = (stCMD_EARC_GET_CONNECT_STATUS*)pBuffer;

                pParameters->u8ConnectStatus = MDrv_EARC_Get_Connect_Status();

                bret = TRUE;
            }
            break;

        case E_HDMI_EARC_CMD_GET_CAPABILITY_INFO:
            if(ulBufferSize == sizeof(stCMD_EARC_GET_CAPABILITY_INFO))
            {
                stCMD_EARC_GET_CAPABILITY_INFO *pParameters = (stCMD_EARC_GET_CAPABILITY_INFO*)pBuffer;

                if(MDrv_EARC_Get_Capability_Info(pParameters->pstEARCCapabilityInfo))
                {
                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_EARC_CMD_GET_LATENCY_INFO:
            if(ulBufferSize == sizeof(stCMD_EARC_GET_LATENCY_INFO))
            {
                stCMD_EARC_GET_LATENCY_INFO *pParameters = (stCMD_EARC_GET_LATENCY_INFO*)pBuffer;

                if(MDrv_EARC_Get_Latency_Info(pParameters->pstEARCLatencyInfo))
                {
                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_EARC_CMD_SET_ATOPSETTING_AUPLL:
            if(ulBufferSize == sizeof(stCMD_EARC_SET_ATOPSETTING_AUPLL))
            {
                stCMD_EARC_SET_ATOPSETTING_AUPLL *pParameters = (stCMD_EARC_SET_ATOPSETTING_AUPLL*)pBuffer;

                if(MDrv_EARC_Set_ATOPSetting_AUPLL(pParameters->pstEARCAudioInfo))
                {
                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_EARC_CMD_SET_AUDIO_OUTPUT_STREAM:
            if(ulBufferSize == sizeof(stCMD_EARC_SET_AUDIO_OUTPUT_STREAM))
            {
                stCMD_EARC_SET_AUDIO_OUTPUT_STREAM *pParameters = (stCMD_EARC_SET_AUDIO_OUTPUT_STREAM*)pBuffer;

                if(MDrv_EARC_Set_AudioOutputStream(pParameters->bEnableFlag))
                {
                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_EARC_CMD_SET_SUPPORT_MODE:
            if(ulBufferSize == sizeof(stCMD_EARC_SET_SUPPORT_MODE))
            {
                stCMD_EARC_SET_SUPPORT_MODE *pParameters = (stCMD_EARC_SET_SUPPORT_MODE*)pBuffer;

                if(MDrv_EARC_Set_SupportMode(pParameters->enSupportMode))
                {
                    bret = TRUE;
                }
            }
            break;

        case E_HDMI_EARC_CMD_GET_CHIP_CAPABILITY:
            if(ulBufferSize == sizeof(stCMD_EARC_GET_CHIP_CAPABILITY))
            {
                stCMD_EARC_GET_CHIP_CAPABILITY *pParameters = (stCMD_EARC_GET_CHIP_CAPABILITY*)pBuffer;

                pParameters->enChipCapability = MDrv_EARC_Get_ChipCapability();

                bret = TRUE;
            }
            break;

        case E_HDMI_EARC_CMD_OUTPUT_PORT_SELECT:
            if(ulBufferSize == sizeof(stCMD_EARC_OUTPUT_PORT_SELECT))
            {
                stCMD_EARC_OUTPUT_PORT_SELECT *pParameters = (stCMD_EARC_OUTPUT_PORT_SELECT*)pBuffer;

                if(MDrv_EARC_OutputPortSelect(pParameters->enInputPortType, pParameters->bEnable5VDetectInvert, pParameters->bEnableHPDInvert))
                {
                    bret = TRUE;
                }
            }
            break;

        default:
            bret = FALSE;
            break;
    }

    return bret;
}
#endif

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_STREventProc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_STREventProc(void* pModule, EN_POWER_MODE usPowerState)
{
    void* pHDMIRxResource = NULL;
    MS_U32 ulReturnValue = UTOPIA_STATUS_FAIL;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
#ifdef MSOS_TYPE_LINUX_KERNEL
    PM_WakeCfg stData;
#endif

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIRxResourcePrivate->usPrePowerState != usPowerState)
        {
            if(usPowerState == E_POWER_SUSPEND)
            {
                //disable hdcp22 irq84 and clear status when suspend
                for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
                {
                    if(pHDMIRxResourcePrivate->bHDCP22EnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
                    {
                        //disable hdcp22 irq84
                        Hal_HDCP22_PortInit(HDMI_GET_PORT_SELECT(enInputPortType));
                        Hal_HDCP_WriteDoneInterruptEnable(enInputPortType, FALSE);

                        //clear status
                        if(Hal_HDCP22_PollingWriteDone(HDMI_GET_PORT_SELECT(enInputPortType)))
                        {
                            //for coverity Unchecked return value
                        }

                        if(Hal_HDCP22_PollingReadDone(HDMI_GET_PORT_SELECT(enInputPortType)))
                        {
                            //for coverity Unchecked return value
                        }

                        pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
                    }
                    if(pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc != NULL)
                    {
                        Hal_HDCP_ReadR0InterruptEnable(enInputPortType, FALSE);
                    }
                    Hal_HDMI_ForcePowerDownPort(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)], TRUE);
#ifdef MSOS_TYPE_LINUX_KERNEL
                    MDrv_PM_Read_Key(PM_KEY_DEFAULT, (const char *)&stData);
                    if (stData.bPmWakeEnableDVI0)
                    {
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI0);
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI1);
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI3);
                    }
                    if (stData.bPmWakeEnableDVI2)
                    {
                        Hal_DVI_ClkPullLow(FALSE, INPUT_PORT_DVI2);
                    }
#endif
                }

                ulReturnValue = UTOPIA_STATUS_SUCCESS;

                //STR suspend flow
                if(pHDMIRxResourcePrivate->fpHdcp14RxSuspendCBFunc != NULL)
                {
                    pHDMIRxResourcePrivate->fpHdcp14RxSuspendCBFunc(1, NULL); //Rx:1, Tx:0
                }
                if(pHDMIRxResourcePrivate->fpHdcp22RxSuspendCBFunc != NULL)
                {
                    pHDMIRxResourcePrivate->fpHdcp22RxSuspendCBFunc(1, NULL); //Rx:1, Tx:0
                }

                MsOS_SetEvent(pHDMIRxResourcePrivate->slHDMIHDCPEventID, HDMI_HDCP_EVENT_STR_SUSPEND);
            }
            else if(usPowerState == E_POWER_RESUME)
            {
                if(pHDMIRxResourcePrivate->ucInitialIndex == HDMI_INITIAL_DONE_INDEX)
                {
                    // Clear all HPD
                    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
                    {
                        Hal_HDMI_ForcePowerDownPort(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)], FALSE);
                        Hal_HDMI_pullhpd(FALSE, enInputPortType, pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)], pHDMIRxResourcePrivate->ucMHLSupportPath);
                        Hal_DVI_ClkPullLow(TRUE, enInputPortType);
                        Hal_HDMI_DataRtermControl(enInputPortType, FALSE);
                    }

                    // HDMI initial
                    Hal_HDMI_init(pHDMIRxResourcePrivate->bImmeswitchSupport, pHDMIRxResourcePrivate->ucMHLSupportPath);

                    // Load HDCP key
                    if(pHDMIRxResourcePrivate->bHDCP14RxREEFlag)
                    {
                        Hal_HDCP_initproductionkey(pHDMIRxResourcePrivate->ucHDCPKeyTable);
                    }


                    #ifdef MSOS_TYPE_LINUX_KERNEL
                    if(_mdrv_HDMI_CreateReLoadKeyTask(pHDMIRxResourcePrivate))
                    {
                        printf("[HMDI] ReLoad Key thread create OK!\r\n");
                    }
                    else
                    {
                        printf("[HDMI] ReLoad Key thread create Failed\r\n");
                    }
                    #else
                    // STR resume flow
                    if(pHDMIRxResourcePrivate->fpHdcp14RxResumeCBFunc != NULL)
                    {
                        pHDMIRxResourcePrivate->fpHdcp14RxResumeCBFunc(1, NULL); //Rx:1, Tx:0
                    }
                    if(pHDMIRxResourcePrivate->fpHdcp22RxResumeCBFunc != NULL)
                    {
                        pHDMIRxResourcePrivate->fpHdcp22RxResumeCBFunc(1, NULL); //Rx:1, Tx:0
                    }

                    // Delay 100ms
                    if(pHDMIRxResourcePrivate->u8HotplugDelay != 0)
                    {
                        MsOS_DelayTask((pHDMIRxResourcePrivate->u8HotplugDelay)*100);
                    }
                    else
                    {
                        MsOS_DelayTask(100);
                    }
                    // Set all HPD
                    for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
                    {
                        if(pHDMIRxResourcePrivate->bHDCP22EnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
                        {
                            //before open mask, clear status
                            if(Hal_HDCP22_PollingWriteDone(HDMI_GET_PORT_SELECT(enInputPortType)))
                            {
                                 //for coverity Unchecked return value
                            }
                            if(Hal_HDCP22_PollingReadDone(HDMI_GET_PORT_SELECT(enInputPortType)))
                            {
                                //for coverity Unchecked return value
                            }

                            //enable hdcp22 irq84
                            Hal_HDCP22_PortInit(HDMI_GET_PORT_SELECT(enInputPortType));
                            Hal_HDCP_WriteDoneInterruptEnable(enInputPortType, TRUE);

                            pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = TRUE;
                        }
                        if(pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc != NULL)
                        {
                            Hal_HDCP_ReadR0InterruptEnable(enInputPortType, TRUE);
                        }

                        Hal_DVI_ClkPullLow(FALSE, enInputPortType);
                        Hal_HDMI_DataRtermControl(enInputPortType, TRUE);
                        Hal_HDMI_pullhpd(TRUE, enInputPortType, pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)], pHDMIRxResourcePrivate->ucMHLSupportPath);

                        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bNoInputFlag = TRUE;
                        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bPowerOnLane = FALSE;
                        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag = FALSE;
                        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bYUV420Flag = FALSE;
                        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource = Hal_HDMI_GetSourceSelect(enInputPortType);
                    }
                    #endif  // end MSOS_TYPE_LINUX_KERNEL
                }

                ulReturnValue = UTOPIA_STATUS_SUCCESS;
#if defined(MSOS_TYPE_LINUX_KERNEL)
                UtopiaStrSendCondition("hdmi_rx", usPowerState, 0);
#endif
            }

            pHDMIRxResourcePrivate->usPrePowerState = usPowerState;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ulReturnValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_STREventProc
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_EARC_STREventProc(void* pModule, EN_POWER_MODE usPowerState)
{
    void* pHDMIEARCResource = NULL;
    MS_U32 ulReturnValue = UTOPIA_STATUS_FAIL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIEARCTxResourcePrivate->usPrePowerState != usPowerState)
        {
            if(usPowerState == E_POWER_RESUME)
            {
                // eARC initial
                Hal_EARC_TX_InitialSetting();
                Hal_EARC_TX_OutputPortSelect(pHDMIEARCTxResourcePrivate->u8EARCOutputPort, pHDMIEARCTxResourcePrivate->bEnable5VDetectInvert, pHDMIEARCTxResourcePrivate->bEnableHPDInvert);

                pHDMIEARCTxResourcePrivate->u8EARCTXConnectState = EARC_CONNECTION_LOST_5V;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeTmpFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeTmpFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bSetLatencyFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeUpdateDoneFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeUpdateDoneFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeCapBusyFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeStatBusyFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeSetLatencyBusyFlag = FALSE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactioinRetryCnt = 0;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactionResult = HDMI_EARCTX_FREE;
                pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactionState = HDMI_EARCTX_TRANSATION_NONE;

                ulReturnValue = UTOPIA_STATUS_SUCCESS;

                MDRV_HDMIRX_MSG_INFO("** EARC E_POWER_RESUME init done\r\n");
            }

            pHDMIEARCTxResourcePrivate->usPrePowerState = usPowerState;
        }
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return ulReturnValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_InitialSetting
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_InitialSetting(void* pInstance, stHDMI_INITIAL_TABLE stInitialTable)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bCreateHDCPTaskFlag = FALSE;
    MS_U8 ucHDMIInfoSource = 0;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIRxResourcePrivate->ucInitialIndex != HDMI_INITIAL_DONE_INDEX)
        {
            for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
            {
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMIModeFlag = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bTimingStableFlag = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bAutoEQRetrigger = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucSourceVersion = HDMI_SOURCE_VERSION_NOT_SURE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPState = HDMI_HDCP_NO_ENCRYPTION;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bIsRepeater = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPInt = 0;
                pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
                pHDMIRxResourcePrivate->bHDCP22EnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
                pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
                pHDMIRxResourcePrivate->bHPDEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
                pHDMIRxResourcePrivate->bDataEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = FALSE;
            }

            for(ucHDMIInfoSource = HDMI_INFO_SOURCE0; ucHDMIInfoSource < HDMI_INFO_SOURCE_MAX; ucHDMIInfoSource++)
            {
                pHDMIRxResourcePrivate->enMuxInputPort[ucHDMIInfoSource] = INPUT_PORT_DVI0;
                pHDMIRxResourcePrivate->ucReceiveIntervalCount[ucHDMIInfoSource] = 0;
                pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource] = 0;
                pHDMIRxResourcePrivate->u32PrePacketStatus[ucHDMIInfoSource] = 0;
                pHDMIRxResourcePrivate->u8PacketReceiveCount[ucHDMIInfoSource] = HDMI_PACKET_RECEIVE_COUNT;
            }

            pHDMIRxResourcePrivate->bSelfCreateTaskFlag = FALSE;
            pHDMIRxResourcePrivate->bHDMITaskProcFlag = FALSE;
            pHDMIRxResourcePrivate->bHDCPIRQAttachFlag = FALSE;
            pHDMIRxResourcePrivate->slHDMIPollingTaskID = -1;
            pHDMIRxResourcePrivate->slHDMIRxLoadKeyTaskID = -1;
            pHDMIRxResourcePrivate->slHDMIHDCPEventID = MsOS_CreateEventGroup("HDMI_HDCP_Event");
            pHDMIRxResourcePrivate->bImmeswitchSupport = stInitialTable.bImmeswitchSupport;
            pHDMIRxResourcePrivate->ucMHLSupportPath = stInitialTable.ucMHLSupportPath;
            pHDMIRxResourcePrivate->ucHDCPReadDoneIndex = 0;
            pHDMIRxResourcePrivate->ucHDCPWriteDoneIndex = 0;
            pHDMIRxResourcePrivate->bHDCP14RxREEFlag = FALSE;
            pHDMIRxResourcePrivate->bSTRSuspendFlag = TRUE;
            pHDMIRxResourcePrivate->bSCDCIRQAttachFalg = FALSE;
            pHDMIRxResourcePrivate->u8HotplugDelay = 0;

            bCreateHDCPTaskFlag = TRUE;

#ifdef MSOS_TYPE_LINUX_KERNEL
            register_reboot_notifier(&_mdrv_HDMI_reboot_notifier);
#endif

            pHDMIRxResourcePrivate->ucInitialIndex = HDMI_INITIAL_DONE_INDEX;
        }
        else if(pHDMIRxResourcePrivate->bSTRSuspendFlag)
        {
            // Load HDCP key
            if(pHDMIRxResourcePrivate->bHDCP14RxREEFlag)
            {
                Hal_HDCP_initproductionkey(pHDMIRxResourcePrivate->ucHDCPKeyTable);
            }
        }

        if(pHDMIRxResourcePrivate->bSTRSuspendFlag)
        {
            for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
            {
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bNoInputFlag = TRUE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bPowerOnLane = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bYUV420Flag = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bPowerSavingFlag = FALSE;
                pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource = Hal_HDMI_GetSourceSelect(enInputPortType);

                if(pHDMIRxResourcePrivate->bHDCP22EnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
                {
                    Hal_HDCP22_PortInit(HDMI_GET_PORT_SELECT(enInputPortType));
                    Hal_HDCP_WriteDoneInterruptEnable(enInputPortType, TRUE);

                    pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = TRUE;
                }
                if(pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc != NULL)
                {
                    Hal_HDCP_ReadR0InterruptEnable(enInputPortType, TRUE);
                }
            }

            pHDMIRxResourcePrivate->bSTRSuspendFlag = FALSE;
        }

        if(!pHDMIRxResourcePrivate->bSelfCreateTaskFlag)
        {
            if(_mdrv_HDMI_CreateTask(pHDMIRxResourcePrivate))
            {
                pHDMIRxResourcePrivate->bSelfCreateTaskFlag = TRUE;
            }
        }

        if(!pHDMIRxResourcePrivate->bHDCPIRQAttachFlag)
        {
            MsOS_AttachInterrupt(E_INT_IRQ_DVI_HDMI_HDCP, _mdrv_HDCP_IsrHandler);
            MsOS_EnableInterrupt(E_INT_IRQ_DVI_HDMI_HDCP);

            pHDMIRxResourcePrivate->bHDCPIRQAttachFlag = TRUE;
        }

        if(!pHDMIRxResourcePrivate->bSCDCIRQAttachFalg)
        {
            MsOS_AttachInterrupt(E_INT_IRQ_SCDC_PM_INT, _mdrv_SCDC_IsrHandler);
            MsOS_EnableInterrupt(E_INT_IRQ_SCDC_PM_INT);

            pHDMIRxResourcePrivate->bSCDCIRQAttachFalg = TRUE;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bCreateHDCPTaskFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_StablePolling
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_StablePolling(void* pInstance)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_HDMI_StablePolling(pHDMIRxResourcePrivate->ucMHLSupportPath, pHDMIRxResourcePrivate->ulPacketStatus, pHDMIRxResourcePrivate->stHDMIPollingInfo, pHDMIRxResourcePrivate->fpHdcp14_AccessWriteProtectedReg);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetSourceVersion
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetSourceVersion(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 ucSourceVersion = HDMI_SOURCE_VERSION_NOT_SURE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucSourceVersion != HDMI_SOURCE_VERSION_NOT_SURE)
        {
            if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucSourceVersion == HDMI_SOURCE_VERSION_HDMI21)
            {
                ucSourceVersion = HDMI_SOURCE_VERSION_HDMI21;
            }
            else
            {
                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bYUV420Flag)
                {
                    ucSourceVersion = HDMI_SOURCE_VERSION_HDMI20;
                }
                else if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag)
                {
                    ucSourceVersion = HDMI_SOURCE_VERSION_HDMI20;
                }
                else if(!pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMIModeFlag)
                {
                    ucSourceVersion = HDMI_SOURCE_VERSION_HDMI14;
                }
                else
                {
                    ucSourceVersion = pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucSourceVersion;
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucSourceVersion;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_CheckHDCPState
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_CheckHDCPState(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 ucHDCPState = HDMI_HDCP_NO_ENCRYPTION;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bIsRepeater) // not Repeater mode
        {
            if(Hal_HDCP_GetEncryptionFlag(enInputPortType) && (!Hal_DVI_clklose_det(enInputPortType))) // HDCP RX detection ENC_EN status
            {
                if(pHDMIRxResourcePrivate->bHDCP14RiFlag[HDMI_GET_PORT_SELECT(enInputPortType)]) // [13] Ri: HDCP1.4
                {
                    ucHDCPState = HDMI_HDCP_1_4;

                    //MDRV_HDMIRX_MSG_INFO("** ENC_EN, HDMI HDCP state 1.4, port %d\r\n", (enInputPortType -INPUT_PORT_DVI0));
                }
                else if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPState == HDMI_HDCP_2_2) // AKE_Init interrupt status: HDCP2.2
                {
                    ucHDCPState = HDMI_HDCP_2_2;

                    //MDRV_HDMIRX_MSG_INFO("** ENC_EN, HDMI HDCP state 2.2, port %d\r\n", (enInputPortType -INPUT_PORT_DVI0));
                }
            }
        }
        else // is Repeater mode
        {
            ucHDCPState = pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPState;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucHDCPState;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_CheckHDCPENCState
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_CheckHDCPENCState(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 ucHDCPENCState = HDMI_HDCP_NOT_ENCRYPTION;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(Hal_HDCP_GetEncryptionFlag(enInputPortType) && (!Hal_DVI_clklose_det(enInputPortType))) // HDCP RX detection ENC_EN status
        {
            if(pHDMIRxResourcePrivate->bHDCP14RiFlag[HDMI_GET_PORT_SELECT(enInputPortType)]) // [13] Ri: HDCP1.4
            {
                ucHDCPENCState = HDMI_HDCP_1_4_ENCRYPTION;

                //MDRV_HDMIRX_MSG_INFO("** ENC_EN, HDMI HDCP ENC state 1.4, port %d\r\n", (enInputPortType -INPUT_PORT_DVI0));
            }
            else if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPState == HDMI_HDCP_2_2) // AKE_Init interrupt status: HDCP2.2
            {
                ucHDCPENCState = HDMI_HDCP_2_2_ENCRYPTION;

                //MDRV_HDMIRX_MSG_INFO("** ENC_EN, HDMI HDCP ENC state 2.2, port %d\r\n", (enInputPortType -INPUT_PORT_DVI0));
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucHDCPENCState;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_CheckHDCP14KeyVaild
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_CheckHDCP14KeyVaild(void* pInstance, MS_U8 *ucHDCPKeyData)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U16 ustemp = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        for(ustemp = 0; ustemp < HDMI_HDCP_KEY_LENGTH; ustemp++)
        {
            pHDMIRxResourcePrivate->ucHDCPKeyTable[ustemp] = ucHDCPKeyData[ustemp];
        }

        if(_mdrv_HDMI_CheckHDCP14Bksv(pHDMIRxResourcePrivate->ucHDCPKeyTable))
        {
            pHDMIRxResourcePrivate->bHDCP14KeyVaildFlag = TRUE;

            Hal_HDCP_initproductionkey(pHDMIRxResourcePrivate->ucHDCPKeyTable);
        }
        else
        {
            pHDMIRxResourcePrivate->bHDCP14KeyVaildFlag = FALSE;

            MDRV_HDMIRX_MSG_INFO("** TMDS HDCP 1.4 key Bksv data fail~~\r\n");
        }

        if(!pHDMIRxResourcePrivate->bHDCP14RxREEFlag)
        {
            pHDMIRxResourcePrivate->bHDCP14RxREEFlag = TRUE;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetHDCP14KeyVaildFlag
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetHDCP14KeyVaildFlag(void* pInstance)
{
    MS_BOOL bHDCP14KeyVaild = FALSE;
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        bHDCP14KeyVaild = pHDMIRxResourcePrivate->bHDCP14KeyVaildFlag;
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bHDCP14KeyVaild;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_SetHPDInvertFlag
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_SetHPDInvertFlag(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bHPDInvertFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetHPDInvertFlag = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
        }
        else
        {
            if(!pHDMIRxResourcePrivate->bHPDEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
            {
                pHDMIRxResourcePrivate->bHPDEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = TRUE;
                pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = bHPDInvertFlag;

                MDRV_HDMIRX_MSG_INFO("** TMDS set HPD invert flag = %d port %d\r\n", bHPDInvertFlag, HDMI_GET_PORT_SELECT(enInputPortType));
            }

            bGetHPDInvertFlag = pHDMIRxResourcePrivate->bHPDInvertFlag[HDMI_GET_PORT_SELECT(enInputPortType)];
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bGetHPDInvertFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPacketReceiveStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_GetPacketReceiveStatus(void* pInstance, MS_U8 ucHDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U32 ulPacketStatus = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucHDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[ucHDMIInfoSource]);

        if(ucHDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            ulPacketStatus = pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource];
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ulPacketStatus;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetHDRInfoFrame
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetHDRInfoFrame(void* pInstance, MS_U8 ucHDMIInfoSource, MS_U16 *pusHDRInfoFrame)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bHDRReceiveFlag = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucHDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[ucHDMIInfoSource]);

        if(ucHDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)
            {
                bHDRReceiveFlag = TRUE;

                Hal_HDMI_Get_InfoFrame(ucHDMIInfoSource, PKT_HDR, pusHDRInfoFrame, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bHDRReceiveFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_DataRtermControl
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_DataRtermControl(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bDataRtermEnable, MS_BOOL bCheckInitFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(bCheckInitFlag)
        {
            if(!pHDMIRxResourcePrivate->bDataEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)])
            {
                Hal_HDMI_DataRtermControl(enInputPortType, bDataRtermEnable);
            }
        }
        else
        {
            Hal_HDMI_DataRtermControl(enInputPortType, bDataRtermEnable);
        }

        pHDMIRxResourcePrivate->bDataEnableFlag[HDMI_GET_PORT_SELECT(enInputPortType)] = TRUE;
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetHDCPStatusPort
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 mdrv_HDMI_GetHDCPStatusPort(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    return Hal_HDCP_getstatus(enInputPortType);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetSCDCValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetSCDCValue(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucRetData = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucRetData = Hal_HDMI_GetSCDCValue(enInputPortType, ucOffset);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucRetData;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetTMDSRatesKHz
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_GetTMDSRatesKHz(void* pInstance, E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_TMDS_RATES enType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U32 ulClockRates = 0, ulRetRates = 0;
    MS_U8 u8ColorDepth = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bNoInputFlag)
        {
            ulClockRates = 1000 * pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].usClockCount;
        }

        switch(enType)
        {
            case E_HDMI_GET_TMDS_BIT_RATES:
                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag)
                {
                    ulRetRates = ulClockRates * 40;
                }
                else
                {
                    ulRetRates = ulClockRates * 10;
                }

                break;

            case E_HDMI_GET_TMDS_CHARACTER_RATES:
                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag)
                {
                    ulRetRates = ulClockRates * 4;
                }
                else
                {
                    ulRetRates = ulClockRates;
                }

                break;

            case E_HDMI_GET_TMDS_CLOCK_RATES:
                ulRetRates = ulClockRates;

                break;

            case E_HDMI_GET_PIXEL_CLOCK_RATES:
                u8ColorDepth = Hal_HDMI_GetGCPColorDepth(enInputPortType, pHDMIRxResourcePrivate->ulPacketStatus[pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource]);

                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucSourceVersion == HDMI_SOURCE_VERSION_HDMI20)
                {
                    ulClockRates = ulClockRates * 4;
                }

                switch (u8ColorDepth)
                {
                    case HDMI_COLOR_DEPTH_8BIT:
                        ulRetRates = ulClockRates;
                        break;

                    case HDMI_COLOR_DEPTH_10BIT:
                        ulRetRates = ulClockRates *8/10;
                        break;

                    case HDMI_COLOR_DEPTH_12BIT:
                        ulRetRates = ulClockRates *8/12;
                        break;

                    case HDMI_COLOR_DEPTH_16BIT:
                        ulRetRates = ulClockRates *8/16;
                        break;

                    default:

                        break;
                };

                break;

            default:

                break;
        };

    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ulRetRates;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetSCDCCableDetect
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetSCDCCableDetect(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bReValue = FALSE;

    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        bReValue = Hal_HDMI_GetSCDCCableDetectFlag(enInputPortType);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetSourceSelect
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetSourceSelect(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8HDMIInfoSource = HDMI_INFO_SOURCE_NONE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetSourceSelect(enInputPortType);

        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource = u8HDMIInfoSource;
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8HDMIInfoSource;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPacketStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_GetPacketStatus(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8HDMIInfoSource = HDMI_INFO_SOURCE_NONE;
    MS_U32 u32PacketStatus = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource;

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u32PacketStatus = pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource];
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u32PacketStatus;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_ForcePowerDownPort
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_ForcePowerDownPort(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bPowerDown)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_HDMI_ForcePowerDownPort(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)], bPowerDown);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPacketContent
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetPacketContent(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_HDMI_PACKET_STATE_t enPacketType, MS_U8 u8PacketLength, MS_U8 *pu8PacketContent)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetContentFlag = FALSE;
    MS_BOOL bSwitchContentFlag = FALSE;
    MS_U8 u8temp = 0;
    MS_U8 u8PacketData = 0;
    MS_U8 u8HDMIInfoSource = HDMI_INFO_SOURCE_NONE;
    MS_U32 ulPacketStatus = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDMIInfoSource;

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            ulPacketStatus = pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource];

            switch(enPacketType)
            {
                case PKT_MPEG:
                    if(ulPacketStatus &HDMI_STATUS_MPEG_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_MPEG_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_MPEG_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_AUI:
                    if(ulPacketStatus &HDMI_STATUS_AUDIO_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_AUDIO_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_AUDIO_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_SPD:
                    if(ulPacketStatus &HDMI_STATUS_SPD_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_SPD_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_SPD_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_AVI:
                    if(ulPacketStatus &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_AVI_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_AVI_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_GC:
                    if(ulPacketStatus &HDMI_STATUS_GCP_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_GCP_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_GCP_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_ASAMPLE:
                    break;

                case PKT_ACR:
                    break;

                case PKT_VS:
                    if(ulPacketStatus &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_VS_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_VS_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_NULL:
                    break;

                case PKT_ISRC2:
                    if(ulPacketStatus &HDMI_STATUS_ISRC2_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_ISRC2_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_ISRC2_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_ISRC1:
                    if(ulPacketStatus &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_ISRC1_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_ISRC1_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_ACP:
                    if(ulPacketStatus &HDMI_STATUS_ACP_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_ACP_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_ACP_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_ONEBIT_AUD:
                    break;

                case PKT_GM:
                    break;

                case PKT_HBR:
                    break;

                case PKT_VBI:
                    break;

                case PKT_HDR:
                    if(ulPacketStatus &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;
                        bSwitchContentFlag = TRUE;

                        if(u8PacketLength > HDMI_HDR_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_HDR_PACKET_LENGTH;
                        }
                    }
                    break;

                case PKT_RSV:
                    break;

                case PKT_EDR:
                    break;

                default:
                    break;
            };

            if(bGetContentFlag)
            {
                for(u8temp = 0; u8temp < u8PacketLength; u8temp++)
                {
                    if(Hal_HDMI_get_packet_value(u8HDMIInfoSource, enPacketType, u8temp, &u8PacketData, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]))
                    {
                        if(bSwitchContentFlag)
                        {
                            if(u8temp %2)
                            {
                                pu8PacketContent[u8temp -1] = u8PacketData;
                            }
                            else
                            {
                                if(u8temp == (u8PacketLength-1))
                                {
                                    pu8PacketContent[u8temp] = u8PacketData;
                                }
                                else
                                {
                                    pu8PacketContent[u8temp +1] = u8PacketData;
                                }
                            }
                        }
                        else
                        {
                            pu8PacketContent[u8temp] = u8PacketData;
                        }
                    }
                }
            }

            switch(enPacketType)
            {
                case PKT_CHANNEL_STATUS:
                    if(ulPacketStatus &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        if(u8PacketLength > HDMI_AUDIO_CHANNEL_STATUS_LENGTH)
                        {
                            u8PacketLength = HDMI_AUDIO_CHANNEL_STATUS_LENGTH;
                        }

                        for(u8temp = 0; u8temp < u8PacketLength; u8temp++)
                        {
                            pu8PacketContent[u8temp] = Hal_HDMI_audio_channel_status(u8HDMIInfoSource, u8temp, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                        }
                    }
                    break;

                case PKT_MULTI_VS:
                    if(ulPacketStatus &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
                    {
                        bGetContentFlag = TRUE;

                        pu8PacketContent[HDMI_MULTI_VS_PACKET_LENGTH] = Hal_HDMI_GetDataInfo(E_HDMI_GET_MULTIVS_COUNT, u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);

                        if(pu8PacketContent[HDMI_MULTI_VS_PACKET_LENGTH] > 4)
                        {
                            pu8PacketContent[HDMI_MULTI_VS_PACKET_LENGTH] = 4;
                        }

                        u8PacketLength = pu8PacketContent[HDMI_MULTI_VS_PACKET_LENGTH] *(HDMI_VS_PACKET_LENGTH -1);

                        if(u8PacketLength > HDMI_MULTI_VS_PACKET_LENGTH)
                        {
                            u8PacketLength = HDMI_MULTI_VS_PACKET_LENGTH;
                        }

                        for(u8temp = 0; u8temp < u8PacketLength; u8temp++)
                        {
                            if(Hal_HDMI_get_packet_value(u8HDMIInfoSource, enPacketType, u8temp, &u8PacketData, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]))
                            {
                                 pu8PacketContent[u8temp] = u8PacketData;
                            }
                        }
                    }
                    break;

                default:
                    break;
            };
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bGetContentFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_CreatePath
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_CreatePath(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if((enInputPortType >= INPUT_PORT_DVI0) && (enInputPortType <= INPUT_PORT_DVI_END))
        {
            if(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource] != enInputPortType)
            {
                pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource] = enInputPortType;

                MDRV_HDMIRX_MSG_INFO("** TMDS switch to port %d source %d\r\n", (enInputPortType -INPUT_PORT_DVI0), u8HDMIInfoSource);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetGCPacketInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 mdrv_HDMI_GetGCPacketInfo(void* pInstance, HDMI_GControl_INFO_t enGCControlInfo, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U16 u16GCPacketInfo = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u16GCPacketInfo = Hal_HDMI_gcontrol_info(enGCControlInfo, u8HDMIInfoSource);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u16GCPacketInfo;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_ErrorStatusUpdate
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_ErrorStatusUpdate(void* pInstance, MS_U8 u8HDMIInfoSource, MS_U8 u8Value, MS_BOOL bReadFlag, MS_BOOL bNewPathFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8ErrorStatus = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!bNewPathFlag)
        {
            u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);
        }

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u8ErrorStatus = Hal_HDMI_err_status_update(u8HDMIInfoSource, u8Value, bReadFlag, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);

            if(bNewPathFlag)
            {
                if(u8ErrorStatus > 0)
                {
                    if(Hal_HDMI_err_status_update(u8HDMIInfoSource, u8ErrorStatus, FALSE, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) > 0)
                    {
                        // Do nothing for coverity issue
                    }
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8ErrorStatus;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_AudioContentProtectInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_AudioContentProtectInfo(void* pInstance, MS_U8 u8HDMIInfoSource, MS_BOOL bNewPathFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8ContentProtectInfo = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!bNewPathFlag)
        {
            u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);
        }

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u8ContentProtectInfo = Hal_HDMI_audio_content_protection_info(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8ContentProtectInfo;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetMIDInfoValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_GetMIDInfoValue(void* pInstance, MS_U8 u8HDMIInfoSource, MS_U8* pu8ManufactorData)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8temp = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            Hal_HDMI_packet_ctrl(u8HDMIInfoSource, 0x04, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]); //set SPD

            for(u8temp = 0; u8temp < 25; u8temp++)
            {
                pu8ManufactorData[u8temp] = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, u8temp +2 *(47 -40) +1, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPacketValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetPacketValue(void* pInstance, MS_U8 u8HDMIInfoSource, MS_HDMI_PACKET_STATE_t enPacketType, MS_U8 u8PacketIndex, MS_U8 *pu8PacketValue)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetPacketFlag = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            bGetPacketFlag = Hal_HDMI_get_packet_value(u8HDMIInfoSource, enPacketType, u8PacketIndex, pu8PacketValue, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bGetPacketFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_AudioChannelStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_AudioChannelStatus(void* pInstance, MS_U8 u8HDMIInfoSource, MS_U8 u8ByteIndex)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8ChannelStatus = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AUDIO_SAMPLE_PACKET_RECEIVE_FLAG)
            {
                u8ChannelStatus = Hal_HDMI_audio_channel_status(u8HDMIInfoSource, u8ByteIndex, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8ChannelStatus;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetColorFormat
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDMI_COLOR_FORMAT mdrv_HDMI_GetColorFormat(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_HDMI_COLOR_FORMAT enColorFormat = MS_HDMI_COLOR_UNKNOWN;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            //if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG) // Remove for AT_Bifrost fail
            {
                switch(Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_1, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource])& 0x60)
                {
                    case 0x00:
                        enColorFormat = MS_HDMI_COLOR_RGB;
                        break;

                    case 0x40:
                        enColorFormat = MS_HDMI_COLOR_YUV_444;
                        break;

                    case 0x20:
                        enColorFormat = MS_HDMI_COLOR_YUV_422;
                        break;

                    case 0x60:
                        enColorFormat = MS_HDMI_COLOR_YUV_420;
                        break;

                    default:
                        break;
                };
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enColorFormat;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetColorRange
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
EN_HDMI_COLOR_RANGE mdrv_HDMI_GetColorRange(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    EN_HDMI_COLOR_RANGE enColorRange = E_HDMI_COLOR_RANGE_DEFAULT;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                if((Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_1, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &0x60) == 0x00)  // RGB
                {
                    switch(Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_3, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &0x0C)
                    {
                        case 0x00:
                            //CEA spec VIC=1 640*480, is the only timeing of table 1,
                            //the default color space shall be RGB using Full Range
                            if((Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_4, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &0x7F) == 0x01)
                            {
                                enColorRange = E_HDMI_COLOR_RANGE_FULL;
                            }
                            else if((Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_4, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &0x7F) == 0x00)
                            {
                                enColorRange = E_HDMI_COLOR_RANGE_FULL;
                            }
                            else
                            {
                                enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                            }
                            break;
                        case 0x04:
                            enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                            break;
                        case 0x08:
                            enColorRange = E_HDMI_COLOR_RANGE_FULL;
                            break;
                        case 0x0C:
                            enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                            break;
                        default:
                            break;
                    };
                }
                else // YUV
                {
                    switch(Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_5, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &0xC0)
                    {
                        case 0x00:
                            enColorRange = E_HDMI_COLOR_RANGE_LIMIT;
                            break;
                        case 0x40:
                            enColorRange = E_HDMI_COLOR_RANGE_FULL;
                            break;
                        case 0x80:
                            enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                            break;
                        case 0xC0:
                            enColorRange = E_HDMI_COLOR_RANGE_RESERVED;
                            break;
                        default:
                            break;
                    };
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enColorRange;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetAspectRatio
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDMI_AR_TYPE mdrv_HDMI_GetAspectRatio(void* pInstance, MS_U8 u8HDMIInfoSource, MS_BOOL bEn_PAR, MS_BOOL bEn_AFAR)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8M10Value = 0;
    MS_HDMI_AR_TYPE enAspectRatio = HDMI_Pic_AR_NODATA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                u8M10Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_2, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &HDMI_AR_REG_MASK;

                if(bEn_PAR)
                {
                    switch(u8M10Value &HDMI_AR_MASK)
                    {
                        case 0x00:     // IF0[13..12] 00
                            enAspectRatio = HDMI_Pic_AR_NODATA;
                            break;

                        case 0x10:     // IF0[13..12] 01, 4:3
                            enAspectRatio = HDMI_Pic_AR_4x3;
                            break;

                        case 0x20:     // IF0[13..12] 10, 16:9
                            enAspectRatio = HDMI_Pic_AR_16x9;
                            break;

                        case 0x30:     // IF0[13..12] 11, reserved
                        default:
                            enAspectRatio = HDMI_Pic_AR_RSV;
                            break;
                    };
                }
                else
                {
                    enAspectRatio = HDMI_Pic_AR_RSV;
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enAspectRatio;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetActiveFormatAspectRatio
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDMI_AR_TYPE mdrv_HDMI_GetActiveFormatAspectRatio(void* pInstance, MS_U8 u8HDMIInfoSource, MS_BOOL bEn_AFAR)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8R3210Value = 0;
    MS_HDMI_AR_TYPE enAspectRatio = HDMI_Pic_AR_NODATA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                u8R3210Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_2, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &HDMI_AR_REG_MASK;

                if(bEn_AFAR)
                {
                    switch(u8R3210Value &HDMI_AFAR_MASK)
                    {
                        case 0x02:    // IF0[11..8] 0010, box 16:9 (top)
                            enAspectRatio = HDMI_AF_AR_16x9_Top;
                            break;

                        case 0x03:    // IF0[11..8] 0011, box 14:9 (top)
                            enAspectRatio = HDMI_AF_AR_14x9_Top;
                            break;

                        case 0x04:    // IF0[11..8] 0100, box >16:9 (centre)
                            enAspectRatio = HDMI_AF_AR_GT_16x9;
                            break;

                        case 0x08:    // IF0[11..8] 1000, same as picture
                            enAspectRatio = HDMI_AF_AR_SAME;
                            break;

                        case 0x09:    // IF0[11..8] 1001, 4:3 Center
                            enAspectRatio = HDMI_AF_AR_4x3_C;
                            break;

                        case 0x0A:    // IF0[11..8] 1010, 16:9 Center
                            enAspectRatio = HDMI_AF_AR_16x9_C;
                            break;

                        case 0x0B:    // IF0[11..8] 1011, 14:9 Center
                            enAspectRatio = HDMI_AF_AR_14x9_C;
                            break;

                        case 0x0D:    // IF0[11..8] 1101, 4:3 with shoot and protect 14:9 centre
                            enAspectRatio = HDMI_AF_AR_4x3_with_14x9_C;
                            break;

                        case 0x0E:    // IF0[11..8] 1110, 16:9 with shoot and protect 14:9 centre
                            enAspectRatio = HDMI_AF_AR_16x9_with_14x9_C;
                            break;

                        case 0x0F:    // IF0[11..8] 1111, 16:9 with shoot and protect 4:3 centre.
                            enAspectRatio = HDMI_AF_AR_16x9_with_4x3_C;
                            break;

                        default:
                            enAspectRatio = HDMI_AF_AR_SAME;
                            break;
                    }
                }
                else
                {
                    enAspectRatio = HDMI_AF_AR_SAME;
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enAspectRatio;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetContentType
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDMI_CONTENT_TYPE mdrv_HDMI_GetContentType(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8ITCValue = 0;
    MS_U8 u8CN10Value = 0;
    MS_HDMI_CONTENT_TYPE enContentType = MS_HDMI_CONTENT_NoData;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                u8ITCValue = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_3, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BIT(7);
                u8CN10Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_5, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BMASK(5:4);

                switch(u8CN10Value)
                {
                    case 0x00:
                        if(u8ITCValue > 0)
                        {
                            enContentType = MS_HDMI_CONTENT_Graphics;
                        }
                        else
                        {
                            enContentType = MS_HDMI_CONTENT_NoData;
                        }
                        break;

                    case 0x10:
                        enContentType = MS_HDMI_CONTENT_Photo;
                        break;

                    case 0x20:
                        enContentType = MS_HDMI_CONTENT_Cinema;
                        break;

                    case 0x30:
                        enContentType = MS_HDMI_CONTENT_Game;
                        break;

                    default:
                        enContentType = MS_HDMI_CONTENT_NoData;
                        break;
                };
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enContentType;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPixelRepetition
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetPixelRepetition(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8PixelRepetition = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                u8PixelRepetition = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_5, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BMASK(3:0);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8PixelRepetition;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetAVIInfoFrameVersion
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
EN_AVI_INFOFRAME_VERSION mdrv_HDMI_GetAVIInfoFrameVersion(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    EN_AVI_INFOFRAME_VERSION enInfoFrameVersion = E_AVI_INFOFRAME_VERSION_NON;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                enInfoFrameVersion = Hal_HDMI_avi_infoframe_info_ver(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enInfoFrameVersion;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetActiveInformationPresent
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetActiveInformationPresent(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bActiveInformationPresent = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_1, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BIT(4))
                {
                    bActiveInformationPresent = TRUE;
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bActiveInformationPresent;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetColormetry
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_HDMI_EXT_COLORIMETRY_FORMAT mdrv_HDMI_GetColormetry(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8C10Value = 0;
    MS_U8 u8EC210Value = 0;
    MS_U8 u8ACE3210Value = 0;
    static MS_U16 usColorimetryTemp[HDMI_INFO_SOURCE_MAX] = {0xFF};
    MS_HDMI_EXT_COLORIMETRY_FORMAT enColormetry = MS_HDMI_EXT_COLOR_UNKNOWN;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
            {
                u8C10Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_2, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BMASK(7:6);
                u8EC210Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_3, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BMASK(6:4);

                if(u8C10Value == 0xC0)
                {
                    switch(u8EC210Value)
                    {
                        case 0x00:
                            enColormetry = MS_HDMI_EXT_COLOR_XVYCC601;
                            break;

                        case 0x10:
                            enColormetry = MS_HDMI_EXT_COLOR_XVYCC709;
                            break;

                        case 0x20:
                            enColormetry = MS_HDMI_EXT_COLOR_SYCC601;
                            break;

                        case 0x30:
                            enColormetry = MS_HDMI_EXT_COLOR_ADOBEYCC601;
                            break;

                        case 0x40:
                            enColormetry = MS_HDMI_EXT_COLOR_ADOBERGB;
                            break;

                        case 0x50:
                            enColormetry = MS_HDMI_EXT_COLOR_BT2020YcCbcCrc;
                            break;

                        case 0x60:
                            enColormetry = MS_HDMI_EXT_COLOR_BT2020RGBYCbCr;
                            break;

                        case 0x70:
                            u8ACE3210Value = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, _BYTE_14, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) &BMASK(7:4);

                            if(u8ACE3210Value == 0x00)
                            {
                                enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_D65;

                                //for LG legacy log
                                if(usColorimetryTemp[u8HDMIInfoSource] != 0x70)
                                {
                                    MDRV_HDMIRX_MSG_INFO("** HDMI extended colorimetry format is ADDITIONAL_DCI_P3_D65 on port %d\r\n",u8HDMIInfoSource);
                                    usColorimetryTemp[u8HDMIInfoSource] = 0x70;
                                }
                            }
                            else if(u8ACE3210Value == 0x10)
                            {
                                enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_THEATER;

                                //for LG legacy log
                                if(usColorimetryTemp[u8HDMIInfoSource] != 0x71)
                                {
                                    MDRV_HDMIRX_MSG_INFO("** HDMI extended colorimetry format is ADDITIONAL_DCI_P3_THEATER on port %d\r\n",u8HDMIInfoSource);
                                    usColorimetryTemp[u8HDMIInfoSource] = 0x71;
                                }
                            }
                            break;

                        default:
                            enColormetry = MS_HDMI_EXT_COLOR_UNKNOWN;
                            break;
                    }
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enColormetry;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_CheckVSPacket4K2K
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_CheckVSPacket4K2K(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bCheckFlag = FALSE;
    //E_HDMI_ADDITIONAL_VIDEO_FORMAT enAdditionalFormat = E_HDMI_NA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                //enAdditionalFormat = Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource);

                if(Hal_HDMI_Is_Input_Large_166MHz(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]))
                {
                    bCheckFlag = TRUE;

                    //if(enAdditionalFormat == E_HDMI_4Kx2K_FORMAT)
                    //{
                        // Do nothing
                    //}
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bCheckFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_3Dor4K2KProcess
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_3Dor4K2KProcess(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    //E_HDMI_ADDITIONAL_VIDEO_FORMAT enAdditionalFormat = E_HDMI_NA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                //enAdditionalFormat = Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource);

                if(Hal_HDMI_Is_Input_Large_166MHz(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]))
                {
                    Hal_HDMI_AVG_ScaleringDown(u8HDMIInfoSource, TRUE);
                }
                else
                {
                    Hal_HDMI_AVG_ScaleringDown(u8HDMIInfoSource, FALSE);
                }

                //if(enAdditionalFormat == E_HDMI_4Kx2K_FORMAT)
                //{
                    // Do nothing
                //}
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVSAdditionalFormat
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_ADDITIONAL_VIDEO_FORMAT mdrv_HDMI_GetVSAdditionalFormat(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    E_HDMI_ADDITIONAL_VIDEO_FORMAT enAdditionalFormat = E_HDMI_NA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                enAdditionalFormat = Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enAdditionalFormat;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVS3DStructure
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_XC_3D_INPUT_MODE mdrv_HDMI_GetVS3DStructure(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    E_XC_3D_INPUT_MODE en3DStructure = E_XC_3D_INPUT_MODE_NONE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) == E_HDMI_3D_FORMAT)
                {
                    en3DStructure = Hal_HDMI_Get_3D_Structure(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return en3DStructure;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVS3DExtendData
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_3D_EXT_DATA_T mdrv_HDMI_GetVS3DExtendData(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    E_HDMI_3D_EXT_DATA_T en3DExtendData = E_3D_EXT_DATA_MODE_MAX;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) == E_HDMI_3D_FORMAT)
                {
                    en3DExtendData = Hal_HDMI_Get_3D_Ext_Data(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return en3DExtendData;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVS3DMetaField
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_GetVS3DMetaField(void* pInstance, MS_U8 u8HDMIInfoSource, sHDMI_3D_META_FIELD *pst3DMetaField)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    sHDMI_3D_META_FIELD st3DMetaField = {0};
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) == E_HDMI_3D_FORMAT)
                {
                    Hal_HDMI_Get_3D_Meta_Field(u8HDMIInfoSource, &st3DMetaField, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                }
            }
        }
    }

    memcpy(pst3DMetaField, &st3DMetaField, sizeof(sHDMI_3D_META_FIELD));

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVSVICCode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetVSVICCode(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8VICCode = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) == E_HDMI_4Kx2K_FORMAT)
                {
                    u8VICCode = Hal_HDMI_Get_VIC_Code(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8VICCode;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetVS4K2KVICCode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_VIC_4Kx2K_CODE mdrv_HDMI_GetVS4K2KVICCode(void* pInstance, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    E_HDMI_VIC_4Kx2K_CODE en4K2KVICCode = E_VIC_NA;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            if(pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
            {
                if(Hal_HDMI_Check_Additional_Format(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]) == E_HDMI_4Kx2K_FORMAT)
                {
                    en4K2KVICCode = Hal_HDMI_Get_4Kx2K_VIC_Code(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return en4K2KVICCode;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_DVIMuxSwitch
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_DVIMuxSwitch(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(enInputPortType);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            Hal_DVI_SwitchSrc(enInputPortType, u8HDMIInfoSource);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_AverageScalingDown
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_AverageScalingDown(void* pInstance, MS_U8 u8HDMIInfoSource, MS_BOOL bEnableFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            Hal_HDMI_AVG_ScaleringDown(u8HDMIInfoSource, bEnableFlag);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetTimingInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 mdrv_HDMI_GetTimingInfo(void* pInstance, E_HDMI_GET_DATA_INFO enInfoType, MS_U8 u8HDMIInfoSource, MS_BOOL bNewPathFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U16 u16TimingInfo = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!bNewPathFlag)
        {
            u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);
        }

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u16TimingInfo = Hal_HDMI_GetDataInfo(enInfoType, u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u16TimingInfo;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_PacketReset
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDMI_PacketReset(void* pInstance, HDMI_REST_t enResetType, MS_U8 u8HDMIInfoSource)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            Hal_HDMI_pkt_reset(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource], u8HDMIInfoSource, enResetType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetHDCPAuthVersion
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_HDCP_STATE mdrv_HDCP_GetAuthVersion(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    E_HDMI_HDCP_STATE enHDCPAuthVersion = E_HDCP_NO_ENCRYPTION;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        enHDCPAuthVersion = Hal_HDCP_GetAuthVersion(enInputPortType, TRUE);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return enHDCPAuthVersion;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetISRC1HeaderInfo
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_GetISRC1HeaderInfo(void* pInstance, MS_U8 u8HDMIInfoSource, MS_BOOL bNewPathFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U32 ulPacketStatus = 0;
    MS_U8 u8ContentProtectInfo = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(!bNewPathFlag)
        {
            u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(pHDMIRxResourcePrivate->enMuxInputPort[u8HDMIInfoSource]);
        }

        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            ulPacketStatus = pHDMIRxResourcePrivate->ulPacketStatus[u8HDMIInfoSource];

            if(ulPacketStatus &HDMI_STATUS_ISRC1_PACKET_RECEIVE_FLAG)
            {
                u8ContentProtectInfo = Hal_HDMI_GetISRC1HeaderInfo(u8HDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8ContentProtectInfo;
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_WriteX74
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  ucOffset                      \b IN: x74 offset
/// @param   ucData                       \b IN: x74 data
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_WriteX74(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset, MS_U8 ucData)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_HDCP_WriteX74(enInputPortType, ucOffset, ucData);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_ReadX74
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  ucOffset                      \b IN: x74 offset
/// return x74 data
//-------------------------------------------------------------------------------------------------
MS_U8 mdrv_HDCP_ReadX74(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 ucOffset)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucRetData = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucRetData = Hal_HDCP_ReadX74(enInputPortType, ucOffset);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucRetData;
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_SetRepeater
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  bIsRepeater                 \b IN: Set REAPEATER or not
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_SetRepeater(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bIsRepeater)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucBcaps = 0x00;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bIsRepeater = bIsRepeater;

        ucBcaps = Hal_HDCP_ReadX74(enInputPortType, 0x40);//Read Bcaps

        if(bIsRepeater)
        {
            ucBcaps = ucBcaps | 0x40;
        }
        else
        {
            ucBcaps = ucBcaps & 0xBF;
        }

        Hal_HDCP_WriteX74(enInputPortType, 0x40, ucBcaps);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
///mdrv_HDCP_SetBStatus
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  usBStatus                    \b IN: Set Bstatus
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_SetBstatus(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U16 usBstatus)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucBStatus = 0x00;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucBStatus = (MS_U8)(usBstatus & BMASK(7:0)); // [6:0]: Device cnt
        Hal_HDCP_WriteX74(enInputPortType, 0x41, ucBStatus);

        ucBStatus = (MS_U8)((usBstatus & BMASK(15:8)) >> 8);//[10:8]: Depth and [12]: HDMI mode
        Hal_HDCP_WriteX74(enInputPortType, 0x42, ucBStatus);

        MDRV_HDMIRX_MSG_INFO("SN Set Bstatus=%x\r\n", ucBStatus);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_SetHDMIMode
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  bHDMIMode                  \b IN: Set HDMI mode or not
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_SetHDMIMode(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bHDMIMode)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucBstatusHighByte = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucBstatusHighByte = Hal_HDCP_ReadX74(enInputPortType, 0x42);
        ucBstatusHighByte = bHDMIMode? (ucBstatusHighByte|0x10): (ucBstatusHighByte&0xEF);

        Hal_HDCP_WriteX74(enInputPortType, 0x42, ucBstatusHighByte);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_GetInterruptStatus
/// @param  enInputPortType           \b IN: HDMI Port number
/// @return  interrupt status
//-------------------------------------------------------------------------------------------------
MS_U8 mdrv_HDCP_GetInterruptStatus(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucRetIntStatus = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucRetIntStatus = (Hal_HDCP_getstatus(enInputPortType) & BMASK(15:8)) >> 8;

        pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].ucHDCPInt = ucRetIntStatus;

        if(ucRetIntStatus & BIT(0)) // [8]: Aksv
        {
            MDRV_HDMIRX_MSG_INFO("[%s@%d]Clear Ready", __FUNCTION__, __LINE__);
            Hal_HDCP_SetReady(enInputPortType, FALSE); // Clear Ready
        }

        Hal_HDCP_ClearStatus(enInputPortType, (ucRetIntStatus & BMASK(4:0))); // Clear [0] Aksv, [1] An, [2] Km, [3] Bksv, [4] R0 interrupt status
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucRetIntStatus;
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_WriteKSVList
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  bHDMIMode                  \b IN: Set HDMI mode or not
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_WriteKSVList(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8* pucKSV, MS_U32 ulDataLen)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_HDCP_WriteKSVList(enInputPortType, pucKSV, ulDataLen);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//-------------------------------------------------------------------------------------------------
/// mdrv_HDCP_SetVPrime
/// @param  enInputPortType           \b IN: HDMI Port number
/// @param  ubVPrime                     \b IN: Set V prime
//-------------------------------------------------------------------------------------------------
void mdrv_HDCP_SetVPrime(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8* pucVPrime)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 ucCnt = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        for(ucCnt = 0; ucCnt < HDMI_HDCP_VPRIME_LENGTH; ucCnt++)
        {
            Hal_HDCP_WriteX74(enInputPortType, 0x20 + ucCnt, *(pucVPrime + ucCnt));
        }

        MDRV_HDMIRX_MSG_INFO("[%s@%d]Set Ready", __FUNCTION__, __LINE__);
        Hal_HDCP_SetReady(enInputPortType, TRUE); // Set Ready
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_FillCipherKey
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_FillCipherKey(void* pInstance, MS_U8 ucPortIdx, MS_U8* pucRiv, MS_U8* pucSessionKey)
{
    Hal_HDCP22_FillCipherKey(ucPortIdx, pucRiv, pucSessionKey);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_PortInit
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_PortInit(void* pInstance, MS_U8 ucPortIdx)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_HDCP22_PortInit(ucPortIdx);

        Hal_HDCP_WriteDoneInterruptEnable((E_MUX_INPUTPORT)(INPUT_PORT_DVI0 +ucPortIdx), TRUE);

        pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[ucPortIdx] = FALSE;
        pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[ucPortIdx] = TRUE;
        pHDMIRxResourcePrivate->bHDCP22EnableFlag[ucPortIdx] = TRUE;
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_PollingReadDone
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDCP22_PollingReadDone(void* pInstance, MS_U8 ucPortIdx)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bReadDoneFlag = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(Hal_HDCP22_PollingReadDone(ucPortIdx))
        {
            bReadDoneFlag = TRUE;
        }
        else if(GET_HDMI_FLAG(pHDMIRxResourcePrivate->ucHDCPReadDoneIndex, BIT(ucPortIdx)))
        {
            CLR_HDMI_FLAG(pHDMIRxResourcePrivate->ucHDCPReadDoneIndex, BIT(ucPortIdx));

            bReadDoneFlag = TRUE;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bReadDoneFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_EnableCipher
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_EnableCipher(void* pInstance, MS_U8 ucPortType, MS_U8 ucPortIdx, MS_BOOL bIsEnable)
{
    Hal_HDCP22_EnableCipher(ucPortType, ucPortIdx, bIsEnable);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_SendMessage
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_SendMessage(void* pInstance, MS_U8 ucPortType, MS_U8 ucPortIdx, MS_U8* pucData, MS_U32 dwDataLen, void* pDummy)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 ubRecIDListSetDone = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    Hal_HDCP22_SendMsg(ucPortType, ucPortIdx, pucData, dwDataLen, pDummy, &ubRecIDListSetDone);

    if (*(MS_U8*)(pucData) == 0xC)
    {
        UtopiaInstanceGetModule(pInstance, &pModule);

        if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
        }
        else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
        {
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
        }
        else
        {
            if (*(MS_U8*)(pucData) == 0xC)
            {
                pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[ucPortIdx] = ubRecIDListSetDone;
            }
        }

        UtopiaResourceRelease(pHDMIRxResource);
    }
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_Handler
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDCP22_Handler(void* pInstance, MS_U8 ucPortIdx, MS_U8* ucPortType, MS_U8* pucMsgData, MS_U32* wMsgLen, MS_BOOL bIRQModeFlag)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bCallbackFlag = FALSE;
    MS_U8 ucMsgData[HDMI_HDCP22_MESSAGE_LENGTH +1] = {0};
    MS_U16 ustemp = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(bIRQModeFlag)
        {
            if(Hal_HDCP22_RecvMsg(ucPortIdx, ucMsgData))
            {
                *ucPortType = E_HDCP22_IF_HDMI;

                for(ustemp = 0; ustemp < HDMI_HDCP22_MESSAGE_LENGTH; ustemp++)
                {
                    pucMsgData[ustemp] = ucMsgData[ustemp];
                }

                *wMsgLen = ucMsgData[HDMI_HDCP22_MESSAGE_LENGTH];

                bCallbackFlag = TRUE;
            }
        }
        else
        {
            if(pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[ucPortIdx])
            {
                Hal_HDCP_WriteDoneInterruptEnable((E_MUX_INPUTPORT)(INPUT_PORT_DVI0 +ucPortIdx), FALSE);

                pHDMIRxResourcePrivate->bHDCP22IRQMaskFlag[ucPortIdx] = FALSE;
            }

            if(Hal_HDCP22_PollingWriteDone(ucPortIdx)) //polling msg
            {
                if(Hal_HDCP22_RecvMsg(ucPortIdx, ucMsgData))
                {
                    *ucPortType = E_HDCP22_IF_HDMI;

                    for(ustemp = 0; ustemp < HDMI_HDCP22_MESSAGE_LENGTH; ustemp++)
                    {
                        pucMsgData[ustemp] = ucMsgData[ustemp];
                    }

                    *wMsgLen = ucMsgData[HDMI_HDCP22_MESSAGE_LENGTH];

                    bCallbackFlag = TRUE;
                }
            }
            else
            {
                *wMsgLen = 0;
            }

            if(ucMsgData[0] == 0x00)
            {
                if(pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[ucPortIdx])
                {
                    if(Hal_HDCP22_PollingReadDone(ucPortIdx))
                    {
                        pHDMIRxResourcePrivate->ucHDCP22RecvIDListSend[ucPortIdx] = FALSE;

                        Hal_HDCP22_SetReadyBit(ucPortIdx, FALSE); //clear ready bit after source read re
                    }
                }
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bCallbackFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_WaitEvent
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDCP22_WaitEvent(void* pInstance)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bGetEventIDFlag = FALSE;
    MS_U8 ucHDCPWriteDoneIndex = 0;
    MS_U32 ulEvent = 0;
    MS_S32 slHDMIHDCPEventID = -1;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        slHDMIHDCPEventID = pHDMIRxResourcePrivate->slHDMIHDCPEventID;
        bGetEventIDFlag = TRUE;
    }

    UtopiaResourceRelease(pHDMIRxResource);
#ifndef MSOS_TYPE_NUTTX
    if(bGetEventIDFlag)
    {
        MsOS_WaitEvent_Interrupt(slHDMIHDCPEventID, HDMI_HDCP_ALL_EVENT, &ulEvent, E_OR_CLEAR, MSOS_WAIT_FOREVER);
    }
#endif
    //MDRV_HDMIRX_MSG_INFO("** TMDS wait event happen kernel space !!!!!!\r\n");

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        ucHDCPWriteDoneIndex = pHDMIRxResourcePrivate->ucHDCPWriteDoneIndex;
        pHDMIRxResourcePrivate->ucHDCPWriteDoneIndex = 0;
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return ucHDCPWriteDoneIndex;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_SetEvent
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_SetEvent(void* pInstance, MS_U16 usHDCPEventIndex)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    //MS_S32 slHDMIHDCPEventID = -1;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        //slHDMIHDCPEventID = pHDMIRxResourcePrivate->slHDMIHDCPEventID;

        if(usHDCPEventIndex == HDMI_HDCP_EVENT_STR_SUSPEND)
        {
            pHDMIRxResourcePrivate->bSTRSuspendFlag = TRUE;
        }

        MsOS_SetEvent(pHDMIRxResourcePrivate->slHDMIHDCPEventID, usHDCPEventIndex);
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP14_InitResumeCBFunc
//  [Description]:
//                  register STR resume HDCP14 resume function
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP14_InitResumeCBFunc(void* pInstance, HDCP14_Resume_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp14RxResumeCBFunc = pCBFunc;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}
//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_InitResumeCBFunc
//  [Description]:
//                  register STR resume HDCP22 resume function
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_InitResumeCBFunc(void* pInstance, HDCP22_Resume_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp22RxResumeCBFunc = pCBFunc;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP14_InitSuspendCBFunc
//  [Description]:
//                  register STR suspend HDCP14 suspend function
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP14_InitSuspendCBFunc(void* pInstance, HDCP14_Suspend_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp14RxSuspendCBFunc = pCBFunc;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP14_InitAccessWriteProtectedCBFunc
//  [Description]:
//                  register access write protected reg
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP14_InitAccessWriteProtectedCBFunc(void* pInstance, HDCP14_AccessWriteProtectedReg_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp14_AccessWriteProtectedReg = pCBFunc;
            MDRV_HDMIRX_MSG_ERROR("[%s,%5d] ** HDMI  fpHdcp14_AccessWriteProtectedReg != NULL \n", __FUNCTION__, __LINE__);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}


//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP22_InitSuspendCBFunc
//  [Description]:
//                  register STR suspend HDCP22 suspend function
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP22_InitSuspendCBFunc(void* pInstance, HDCP22_Suspend_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp22RxSuspendCBFunc = pCBFunc;
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_ClockStableStatus
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_ClockStableStatus(void* pInstance, E_MUX_INPUTPORT enInputPortType)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_BOOL bClockStableFlag = FALSE;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else if(!pHDMIRxResourcePrivate->stHDMIPollingInfo[enInputPortType - INPUT_PORT_DVI0].bNoInputFlag)
    {
        bClockStableFlag = TRUE;
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bClockStableFlag;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetPipeLineMode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
E_HDMI_PIPELINE_MODE mdrv_HDMI_GetPipeLineMode(void* pInstance)
{
    return Hal_HDMI_GetPipeLineMode();
}


//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_Get_HW_Ability
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_Get_HW_Ability(void* pInstance, E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_WHICH_HW_ABILITY enWhichAbility)
{
    return Hal_HDMI_Get_HW_Ability(enInputPortType, enWhichAbility);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDCP14_InitSVPLockCBFunc
//  [Description]:
//                  register HDCP14 SVP Lock Callback Func
//  [Arguments]:
//                  bIsRx:1 means Rx, 0 means Tx
//  [Return]:
//
//**************************************************************************
void mdrv_HDCP14_InitSVPLockCBFunc(void* pInstance, HDCP14_SVPLock_CBF pCBFunc, void* pContext)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pCBFunc != NULL)
        {
            pHDMIRxResourcePrivate->fpHdcp14SVPLockCBFunc = pCBFunc;

            for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
            {
                Hal_HDCP_ReadR0InterruptEnable(enInputPortType, TRUE);
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}


//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_InitialSetting
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Init(void* pInstance)
{
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        pHDMIEARCTxResourcePrivate->bSelfCreateTaskFlag = FALSE;
        pHDMIEARCTxResourcePrivate->bEARCTaskProcFlag = FALSE;
        pHDMIEARCTxResourcePrivate->slEARCPollingTaskID = -1;
        pHDMIEARCTxResourcePrivate->u8EARCTXConnectState = EARC_CONNECTION_LOST_5V;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeTmpFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeTmpFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bSetLatencyFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeUpdateDoneFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeUpdateDoneFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeCapBusyFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeStatBusyFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCommonModeSetLatencyBusyFlag = FALSE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactioinRetryCnt = 0;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactionResult = HDMI_EARCTX_FREE;
        pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8TransactionState = HDMI_EARCTX_TRANSATION_NONE;

        Hal_EARC_TX_InitialSetting();

        if(!pHDMIEARCTxResourcePrivate->bSelfCreateTaskFlag)
        {
            if(_mdrv_EARC_CreateTask(pHDMIEARCTxResourcePrivate))
            {
                pHDMIEARCTxResourcePrivate->bSelfCreateTaskFlag = TRUE;
            }
        }

        MDRV_HDMIRX_MSG_INFO("[%s] EARC Initial Setting Done\n", __FUNCTION__);

        bReValue = TRUE;
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Get_Connect_Status
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_EARC_Get_Connect_Status(void* pInstance)
{
    MS_U8 u8ReValue = EARC_CONNECTION_LOST_5V;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8ReValue = pHDMIEARCTxResourcePrivate->u8EARCTXConnectState;
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return u8ReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Get_Capability_Info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Get_Capability_Info(void* pInstance, stEARC_CAPABILITY_INFO *pstEARCCapabilityInfo)
{
    MS_U16 u16temp = 0;
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bCapChangeUpdateDoneFlag)
        {
            pstEARCCapabilityInfo->bCapabilityChangeFlag = TRUE;

            for(u16temp = 0; u16temp < EARC_GET_CAPABILITY_CONTENT_LENGTH; u16temp++)
            {
                pstEARCCapabilityInfo->u8CapabilityValue[u16temp] = pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8CapabilityContent[u16temp];
            }
        }
        else
        {
            pstEARCCapabilityInfo->bCapabilityChangeFlag = FALSE;

            for(u16temp = 0; u16temp < EARC_GET_CAPABILITY_CONTENT_LENGTH; u16temp++)
            {
                pstEARCCapabilityInfo->u8CapabilityValue[u16temp] = 0;
            }
        }

        bReValue = TRUE;
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Get_Latency_Info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Get_Latency_Info(void* pInstance, stEARC_LATENCY_INFO *pstEARCLatencyInfo)
{
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.bStatChangeUpdateDoneFlag)
        {
            pstEARCLatencyInfo->bLatencyChangeFlag = TRUE;
            pstEARCLatencyInfo->u8LatencyValue = pHDMIEARCTxResourcePrivate->stHDMIEARCTXCommonModePollinfInfo.u8LatencyContent;
        }
        else
        {
            pstEARCLatencyInfo->bLatencyChangeFlag = FALSE;
            pstEARCLatencyInfo->u8LatencyValue = 0;
        }

        bReValue = TRUE;
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Set_ATOPSetting_AUPLL
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Set_ATOPSetting_AUPLL(void* pInstance, stEARC_AUDIO_INFO *pstEARCAudioInfo)
{
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(Hal_EARC_TX_SetATOPSetting_AUPLL(HDMI_EARCTX_SELECT_ATOP_MODE, pstEARCAudioInfo->enAudioFrequency, pstEARCAudioInfo->enAudioChannelLayout))
        {
            bReValue = TRUE;
        }
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Set_AudioOutputStream
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Set_AudioOutputStream(void* pInstance, MS_BOOL bEnableFlag)
{
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_EARC_TX_SetAudioOutputStream(bEnableFlag);

        bReValue = TRUE;
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Set_SupportMode
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_EARC_Set_SupportMode(void* pInstance, EN_EARC_SUPPORT_MODE enSupportMode)
{
    MS_BOOL bReValue = FALSE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(Hal_EARC_TX_SetSupportMode(enSupportMode))
        {
            bReValue = TRUE;
        }
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return bReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_Get_ChipCapability
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_EARC_Get_ChipCapability(void* pInstance)
{
    MS_U8 u8ReValue = SUPPORT_NONE;
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8ReValue = Hal_EARC_TX_GetChipCapability();
    }

    UtopiaResourceRelease(pHDMIEARCResource);

    return u8ReValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_EARC_OutputPortSelect
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
void mdrv_EARC_OutputPortSelect(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_BOOL bEnable5VDetectInvert, MS_BOOL bEnableHPDInvert)
{
    void* pModule = NULL;
    void* pHDMIEARCResource = NULL;
    HDMI_EARCTX_RESOURCE_PRIVATE *pHDMIEARCTxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_EARC_POOL, &pHDMIEARCResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIEARCResource, (void*)&pHDMIEARCTxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI EARC resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        Hal_EARC_TX_OutputPortSelect(enInputPortType, bEnable5VDetectInvert, bEnableHPDInvert);

        pHDMIEARCTxResourcePrivate->u8EARCOutputPort = enInputPortType;
        pHDMIEARCTxResourcePrivate->bEnable5VDetectInvert = bEnable5VDetectInvert;
        pHDMIEARCTxResourcePrivate->bEnableHPDInvert = bEnableHPDInvert;
    }

    UtopiaResourceRelease(pHDMIEARCResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_avi_infoframe_info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_avi_infoframe_info(void* pInstance, MS_U8 u8byte)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    MS_U8 u8Return = 0;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        u8Return = Hal_HDMI_avi_infoframe_info(HDMI_INFO_SOURCE0, u8byte, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_INFO_SOURCE0]);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8Return;
}


//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_AVIInfoframePort
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_AVIInfoframePort(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 u8byte)
{
    MS_U8 u8AVIInfoframeValue = 0;
    MS_U8 u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(enInputPortType);

    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u8AVIInfoframeValue = Hal_HDMI_avi_infoframe_info(u8HDMIInfoSource, u8byte, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8AVIInfoframeValue;
}


//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_audio_cp_data_info
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U8 mdrv_HDMI_audio_cp_data_info(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_U8 u8byte)
{
    MS_U8 u8AVIInfoframeValue = 0;
    MS_U8 u8HDMIInfoSource = Hal_HDMI_GetPortToSourceMapping(enInputPortType);

    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        if(u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
        {
            u8AVIInfoframeValue = Hal_HDMI_audio_payload_info(u8HDMIInfoSource, u8byte, &pHDMIRxResourcePrivate->stHDMIPollingInfo[u8HDMIInfoSource]);
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u8AVIInfoframeValue;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetCRCValue
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_BOOL mdrv_HDMI_GetCRCValue(void* pInstance, E_MUX_INPUTPORT enInputPortType, MS_DVI_CHANNEL_TYPE u8Channel, MS_U16* pu16CRCVal)
{
    MS_BOOL bRetValue = FALSE;
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        bRetValue = Hal_HDMI_GetCRCValue(enInputPortType, u8Channel, pu16CRCVal, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)]);
    }

    UtopiaResourceRelease(pHDMIRxResource);

    return bRetValue;
}

void mdrv_HDMI_SetHotplugDelay(void* pInstance, MS_U8 u8HotplugDelay)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        pHDMIRxResourcePrivate->u8HotplugDelay = u8HotplugDelay;
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetFrameRate
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U32 mdrv_HDMI_GetFrameRate(E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_DATA_INFO enInfoType)
{
    MS_U16 u32retFrameRate = 0;
    stHDMI_GET_SOURCE_SELECT stHDMIRxArgs = {enInputPortType, HDMI_INFO_SOURCE_NONE};
    MS_U32 u32VFrequency = 0;
    MS_U16 u16HTTValue = 0;
    MS_U16 u16VTTValue = 0;

    //MS_U16 usInfoData = 0;

    //get raw clock
    stHDMI_GET_TMDS_RATES_KHZ stHDMIRxArgs1 = {enInputPortType, E_HDMI_GET_TMDS_CHARACTER_RATES, 0};

    if(pulHDMIRxInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TMDS_RATES_KHZ, (void*)&stHDMIRxArgs1) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
    }
    u32VFrequency = stHDMIRxArgs1.ulRetRates;
    //printf("u32VFrequency: %d\r\n", u32VFrequency);

    //get depth
    MS_U8 ucPacketContent[HDMI_GET_PACKET_LENGTH] = {0};
    stGC_PARSING_INFO stGCParsingInfo;
    memset(&stGCParsingInfo, 0, sizeof(stGC_PARSING_INFO));

    if(MDrv_HDMI_GetPacketContent(enInputPortType, PKT_GC, HDMI_GET_PACKET_LENGTH, ucPacketContent))
    {
        _mdrv_HDMI_ParseGCInfoFrame(ucPacketContent, &stGCParsingInfo);
    }

    switch(stGCParsingInfo.enColorDepth)
    {
        case MS_HDMI_COLOR_DEPTH_10_BIT:
            u32VFrequency = u32VFrequency *8 /10;
            printf("MS_HDMI_COLOR_DEPTH_10_BIT\r\n");
            break;
        case MS_HDMI_COLOR_DEPTH_12_BIT:
            u32VFrequency = u32VFrequency *8 /12;
            printf("MS_HDMI_COLOR_DEPTH_12_BIT\r\n");
            break;
        case MS_HDMI_COLOR_DEPTH_16_BIT:
            u32VFrequency = u32VFrequency *8 /16;
            printf("MS_HDMI_COLOR_DEPTH_16_BIT\r\n");
            break;
        default:
            break;
    };

    if(pulHDMIRxInst == NULL)
    {
        MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
    }
    else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_SOURCE_SELECT, (void*)&stHDMIRxArgs) != 0)
    {
        MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
    }
    else if(stHDMIRxArgs.u8HDMIInfoSource < HDMI_INFO_SOURCE_MAX)
    {
        //get HTT
        stHDMI_GET_DATA_INFO stHDMIRxArgs2 = {0};
        stHDMIRxArgs2.usReturnValue = 0;
        stHDMIRxArgs2.enInfo = E_HDMI_GET_HTT;
        stHDMIRxArgs2.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
        //stHDMIRxArgs2.bNewPathFlag = TRUE;

        if(pulHDMIRxInst == NULL)
        {
            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
        }
        else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TIMING_INFO, (void*)&stHDMIRxArgs2) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        u16HTTValue = stHDMIRxArgs2.usReturnValue;
        printf("u16HTTValue: %d\r\n", u16HTTValue);

        //get VTT
        stHDMI_GET_DATA_INFO stHDMIRxArgs3 = {0};
        stHDMIRxArgs3.usReturnValue = 0;
        stHDMIRxArgs3.enInfo = E_HDMI_GET_VTT;
        stHDMIRxArgs3.ucHDMIInfoSource = stHDMIRxArgs.u8HDMIInfoSource;
        //stHDMIRxArgs3.bNewPathFlag = TRUE;

        if(pulHDMIRxInst == NULL)
        {
            MDRV_HDMIRX_MSG_WARNING("[%s,%5d]No instance existed, please get an instance by calling MDrv_HDMI_init() first\n",__FUNCTION__,__LINE__);
        }
        else if(UtopiaIoctl(pulHDMIRxInst, MAPI_CMD_HDMIRX_GET_TIMING_INFO, (void*)&stHDMIRxArgs3) != 0)
        {
            MDRV_HDMIRX_MSG_ERROR("Obtain HDMI Rx engine fail\n");
        }
        u16VTTValue = stHDMIRxArgs3.usReturnValue + 1; //because value read from reg always minus 1
        printf("11u16VTTValue: %d\r\n", u16VTTValue);
    }

    //count frame rate
    u32VFrequency = (MS_U32)u32VFrequency *1000 /u16HTTValue;
    //printf("u32VFrequency: %d\r\n", u32VFrequency);
    if(u16VTTValue > 0)
    {
        u32retFrameRate = (u32VFrequency * 100 /u16VTTValue);
    }
    printf("u16retFrameRate: %d\r\n", u32retFrameRate);
    return u32retFrameRate;
}

//**************************************************************************
//  [Function Name]:
//                  mdrv_HDMI_GetHDMI_Status
//  [Description]:
//
//  [Arguments]:
//
//  [Return]:
//
//**************************************************************************
MS_U16 mdrv_HDMI_GetHDMI_Status(void* pInstance, E_MUX_INPUTPORT enInputPortType, E_HDMI_GET_HDMI_STATUS_IDX enInfoType, MS_U8 u8Param)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 u8ErrorStatus = 0;
    MS_U16 u16Ret = 0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        switch(enInfoType)
        {
            case E_HDMI_GET_HDMI_SIGNAL_STABLE:
                {
                    if(HDMI_GET_PORT_SELECT(enInputPortType) < HDMI_INFO_SOURCE_MAX)
                    {
                        u8ErrorStatus = Hal_HDMI_err_status_update(HDMI_GET_PORT_SELECT(enInputPortType), 0, TRUE, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)]);

                        if(u8ErrorStatus > 0)
                        {
                            if(Hal_HDMI_err_status_update(HDMI_GET_PORT_SELECT(enInputPortType), u8ErrorStatus, FALSE, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)]) > 0)
                            {
                                // Do nothing for coverity issue
                            }
                        }
                    }

                    if((Hal_DVI_clklose_det(enInputPortType)) || (u8ErrorStatus & HDMI_BCH_ERROR) || (!Hal_HDMI_GetPHYDone(enInputPortType, &pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)])))
                    {
                        u16Ret = 0;
                    }
                    else
                    {
                        u16Ret = 1;
                    }
                }
                break;

            default:
                break;
        }

    }

    UtopiaResourceRelease(pHDMIRxResource);

    return u16Ret;
}

#ifdef CONFIG_UTOPIA_PROC_DBG_SUPPORT
//-------------------------------------------------------------------------------------------------
///mdrv_HDMI_MDCMDEchoCommand
/// @param  u64ReqHdl               \b IN: Module index
/// @param  pcCmdLine               \b IN: Command line info
//-------------------------------------------------------------------------------------------------
void mdrv_HDMI_MDCMDEchoCommand(void* pInstance, MS_U64* u64ReqHdl, char* pcCmdLine)
{
    //char pch[] = "=,";
    char space[] = " ";
    char equal[] = "=";
    char* pString[7] = {NULL};
    MS_U8 uctemp = 0;
    MS_U8 ucEQValue = 0xFF;
    E_MUX_INPUTPORT enInputPortType = (E_MUX_INPUTPORT)(INPUT_PORT_DVI_END +1);

    MdbPrint(u64ReqHdl, "LINE:%d, MDBCMD_CMDLINE\n", __LINE__);
    MdbPrint(u64ReqHdl, "pcCmdLine: %s\n", pcCmdLine);

    //psep = strsep(&pcCmdLine, space);
    //MdbPrint(u64ReqHdl, "%s\n", psep);

    for(uctemp = 0; uctemp < 7; uctemp++)
    {
        pString[uctemp] = strsep(&pcCmdLine, space);

        if(pString[uctemp] == NULL)
        {
            break;
        }
        else
        {
            //MdbPrint(u64ReqHdl, "%s\n", pString[uctemp -1]);
        }
    }

    if(pString[0] != NULL)
    {
        if(strncmp("help", pString[0], 4) == 0)
        {
            MdbPrint(u64ReqHdl, "----------------  Mstar HDMI Rx help -----------------\n");
            MdbPrint(u64ReqHdl, "Support fixed EQ echo function\n");
            MdbPrint(u64ReqHdl, "ex: echo fixedeq port0 eq=8\n");
            MdbPrint(u64ReqHdl, "or  echo fixedeq port3 eq=31\n");
            MdbPrint(u64ReqHdl, "port0~3, eq=0~31\n");
            MdbPrint(u64ReqHdl, "------------------------------------------------------\n");
        }
        else if(strncmp("fixedeq", pString[0], 7) == 0)
        {
            if(pString[1] != NULL) // "port0"
            {
                if(strncmp("port", pString[1], 4) == 0)
                {
                    if(*(pString[1] +4) == 48)
                    {
                        enInputPortType = INPUT_PORT_DVI0;
                    }
                    else if(*(pString[1] +4) == 49)
                    {
                        enInputPortType = INPUT_PORT_DVI1;
                    }
                    else if(*(pString[1] +4) == 50)
                    {
                        enInputPortType = INPUT_PORT_DVI2;
                    }
                    else if(*(pString[1] +4) == 51)
                    {
                        enInputPortType = INPUT_PORT_DVI3;
                    }

                    if(enInputPortType <= INPUT_PORT_DVI_END)
                    {
                        if(pString[2] != NULL) // "eq=1"
                        {
                            if(strncmp("eq=", pString[2], 3) == 0)
                            {
                                pString[3] = strsep(&pString[2], equal);

                                if(pString[3] != NULL)
                                {
                                    pString[3] = strsep(&pString[2], equal);
                                    ucEQValue = atoi(pString[3]);
                                }

                                if(ucEQValue < 32)
                                {
                                    Hal_HDMI_Set_EQ(enInputPortType, MS_HDMI_EQ_USER_DEFINE, ucEQValue);

                                    MdbPrint(u64ReqHdl, "Setup fixed EQ port%d EQ=%d ready!!\n", HDMI_GET_PORT_SELECT(enInputPortType), ucEQValue);
                                }
                                else
                                {
                                    MdbPrint(u64ReqHdl, "EQ value is invalid\n");
                                }
                            }
                        }
                    }
                    else
                    {
                        MdbPrint(u64ReqHdl, "Port number is invalid\n");
                    }
                }
            }
        }
    }
}

//-------------------------------------------------------------------------------------------------
///mdrv_HDMI_MDCMDGetInfo
/// @param  u64ReqHdl               \b IN: Module index
//-------------------------------------------------------------------------------------------------
void mdrv_HDMI_MDCMDGetInfo(void* pInstance, MS_U64* u64ReqHdl)
{
    void* pModule = NULL;
    void* pHDMIRxResource = NULL;
    MS_U8 uctemp = 0;
    MS_U8 ucHDMIInfoSource = HDMI_INFO_SOURCE0;
    MS_U8 ucPacketInfo[32] = {0};
    MS_U8 u8PktCounter = 0;
    MS_U8 u8C10Value = 0;
    MS_U8 u8EC210Value = 0;
    MS_U8 u8ACE3210Value = 0;
    MS_U16 usHDEValue = 0;
    MS_U16 usVDEValue = 0;
    MS_U16 usDeepColorInfo = HDMI_GC_INFO_CD_NOT_INDICATED;
    E_MUX_INPUTPORT enInputPortType = INPUT_PORT_DVI0;
    HDMI_RX_RESOURCE_PRIVATE *pHDMIRxResourcePrivate = NULL;
    stHDMI_AUDIO_INFO stAudioInfo;
    memset(&stAudioInfo, 0 ,sizeof(stHDMI_AUDIO_INFO));
    stHDMI_HDCP_INFO stHDCPInfo;
    memset(&stHDCPInfo, 0 ,sizeof(stHDMI_HDCP_INFO));

    UtopiaInstanceGetModule(pInstance, &pModule);

    if(UtopiaResourceObtain(pModule, HDMI_RX_POOL, &pHDMIRxResource) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] UtopiaResourceObtainToInstant failed\n", __FUNCTION__, __LINE__);
    }
    else if(UtopiaResourceGetPrivate(pHDMIRxResource, (void*)&pHDMIRxResourcePrivate) != UTOPIA_STATUS_SUCCESS)
    {
        MDRV_HDMIRX_MSG_ERROR("[%s,%5d] Get HDMI Rx resource failed\n", __FUNCTION__, __LINE__);
    }
    else
    {
        MdbPrint(u64ReqHdl, "LINE:%d, MDBCMD_GETINFO\n", __LINE__);

        for(enInputPortType = INPUT_PORT_DVI0; enInputPortType <= INPUT_PORT_DVI_END; enInputPortType++)
        {
            MdbPrint(u64ReqHdl, "----------------  Mstar HDMI port%d Info -----------------\n", HDMI_GET_PORT_SELECT(enInputPortType));

            if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bNoInputFlag)
            {
                MdbPrint(u64ReqHdl, "Input clock unstable, 5V cable detected: %d\n", Hal_HDMI_GetSCDCCableDetectFlag(enInputPortType));
            }
            else
            {
                MdbPrint(u64ReqHdl, "Input clock stable, 5V cable detected: %d \n", Hal_HDMI_GetSCDCCableDetectFlag(enInputPortType));

                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMI20Flag)
                {
                    MdbPrint(u64ReqHdl, "HDMI 2.0 timing\n");
                }
                else
                {
                    MdbPrint(u64ReqHdl, "HDMI 1.4 timing\n");
                }

                MdbPrint(u64ReqHdl, "TMDS clock rate:%dM\n", pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].usClockCount);

                if(pHDMIRxResourcePrivate->stHDMIPollingInfo[HDMI_GET_PORT_SELECT(enInputPortType)].bHDMIModeFlag)
                {
                    MdbPrint(u64ReqHdl, "HDMI mode\n");
                }
                else
                {
                    MdbPrint(u64ReqHdl, "DVI mode\n");
                }
            }
        }

        for(ucHDMIInfoSource = HDMI_INFO_SOURCE0; ucHDMIInfoSource < HDMI_INFO_SOURCE_MAX; ucHDMIInfoSource++)
        {
            MdbPrint(u64ReqHdl, "----------------  Mstar HDMI IP%d Info -----------------\n", ucHDMIInfoSource);

            usHDEValue = Hal_HDMI_GetDataInfo(E_HDMI_GET_HDE, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]);
            usVDEValue = Hal_HDMI_GetDataInfo(E_HDMI_GET_VDE, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]);

            if((usHDEValue > 0) && (usVDEValue > 0) && (!pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource].bNoInputFlag))
            {
                Hal_HDMI_err_status_update(ucHDMIInfoSource, 0xFF, FALSE, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]);	//BCH error clear

                MdbPrint(u64ReqHdl, "HDE: %d\n", usHDEValue);
                MdbPrint(u64ReqHdl, "VDE: %d\n", usVDEValue);
                MdbPrint(u64ReqHdl, "HTT: %d\n", Hal_HDMI_GetDataInfo(E_HDMI_GET_HTT, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]));
                MdbPrint(u64ReqHdl, "VTT: %d\n", Hal_HDMI_GetDataInfo(E_HDMI_GET_VTT, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]));
                MdbPrint(u64ReqHdl, "Interlace mode: %d \n", Hal_HDMI_GetDataInfo(E_HDMI_GET_INTERLACE, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]));

                switch(Hal_HDMI_avi_infoframe_info(ucHDMIInfoSource, _BYTE_1, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource])& 0x60)
                {
                    case 0x00:
                        MdbPrint(u64ReqHdl, "Color space RGB444\n");
                        break;

                    case 0x40:
                        MdbPrint(u64ReqHdl, "Color space YUV444\n");
                        break;

                    case 0x20:
                        MdbPrint(u64ReqHdl, "Color space YUV422\n");
                        break;

                    case 0x60:
                        MdbPrint(u64ReqHdl, "Color space YUV420\n");
                        break;

                    default:

                        break;
                };

                usDeepColorInfo = Hal_HDMI_gcontrol_info(G_Ctrl_CD_VAL, ucHDMIInfoSource) >> 8;

                switch(usDeepColorInfo)
                {
                    case HDMI_GC_INFO_CD_24BITS:
                        MdbPrint(u64ReqHdl, "Color depth 8bit\n");
                        break;

                    case HDMI_GC_INFO_CD_30BITS:
                        MdbPrint(u64ReqHdl, "Color depth 10bit\n");
                        break;

                    case HDMI_GC_INFO_CD_36BITS:
                        MdbPrint(u64ReqHdl, "Color depth 12bit\n");
                        break;

                    case HDMI_GC_INFO_CD_48BITS:
                        MdbPrint(u64ReqHdl, "Color depth 16bit\n");
                        break;

                    default:
                        MdbPrint(u64ReqHdl, "Color depth not indicated\n");
                        break;
                };

                MdbPrint(u64ReqHdl, "BCH error:%x\n", Hal_HDMI_err_status_update(ucHDMIInfoSource, 0x0, TRUE, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]));	//BCH error read, 0: no error; else: error happen
                MdbPrint(u64ReqHdl, "AVMute:%d\n", Hal_HDMI_gcontrol_info(G_CTRL_AVMUTE, ucHDMIInfoSource));
                MdbPrint(u64ReqHdl, "Packet receive status:%x\n", pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource]);

                if(pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_AVI_PACKET_RECEIVE_FLAG)
                {
                    MdbPrint(u64ReqHdl, "Receive AVI packet\n");
                    for(u8PktCounter = 0; u8PktCounter < 13; u8PktCounter ++)
                    {
                        if(Hal_HDMI_get_packet_value(ucHDMIInfoSource, PKT_AVI, u8PktCounter, ucPacketInfo, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]))
                        {
                            MdbPrint(u64ReqHdl, "AVI packet Byte %01d: 0x%x \n",u8PktCounter+1, ucPacketInfo[uctemp]);
                        }
                    }

                    u8C10Value = Hal_HDMI_avi_infoframe_info(ucHDMIInfoSource, _BYTE_2, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]) &BMASK(7:6);
                    u8EC210Value = Hal_HDMI_avi_infoframe_info(ucHDMIInfoSource, _BYTE_3, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]) &BMASK(6:4);

                    MdbPrint(u64ReqHdl, "Color space u8C10Value:%x, u8EC210Value:%x\n", u8C10Value, u8EC210Value);

                    if(u8C10Value == 0x00)
                    {
                        MdbPrint(u64ReqHdl, "Color space No Data\n");
                    }
                    else if(u8C10Value == 0x40)
                    {
                        MdbPrint(u64ReqHdl, "Color space SMPTE170M\n");
                    }
                    else if(u8C10Value == 0x80)
                    {
                        MdbPrint(u64ReqHdl, "Color space ITU-R BT.709\n");
                    }
                    if(u8C10Value == 0xC0)
                    {
                        switch(u8EC210Value)
                        {
                            case 0x00:
                                //enColormetry = MS_HDMI_EXT_COLOR_XVYCC601;
                                MdbPrint(u64ReqHdl, "Color space XVYCC601\n");
                                break;

                            case 0x10:
                                //enColormetry = MS_HDMI_EXT_COLOR_XVYCC709;
                                MdbPrint(u64ReqHdl, "Color space XVYCC709\n");
                                break;

                            case 0x20:
                                //enColormetry = MS_HDMI_EXT_COLOR_SYCC601;
                                MdbPrint(u64ReqHdl, "Color space SYCC601\n");
                                break;

                            case 0x30:
                                //enColormetry = MS_HDMI_EXT_COLOR_ADOBEYCC601;
                                MdbPrint(u64ReqHdl, "Color space ADOBEYCC601\n");
                                break;

                            case 0x40:
                                //enColormetry = MS_HDMI_EXT_COLOR_ADOBERGB;
                                MdbPrint(u64ReqHdl, "Color space ADOBERGB\n");
                                break;

                            case 0x50:
                                //enColormetry = MS_HDMI_EXT_COLOR_BT2020YcCbcCrc;
                                MdbPrint(u64ReqHdl, "Color space BT2020YcCbcCrc\n");
                                break;

                            case 0x60:
                                //enColormetry = MS_HDMI_EXT_COLOR_BT2020RGBYCbCr;
                                MdbPrint(u64ReqHdl, "Color space BT2020RGBYCbCr\n");
                                break;

                            case 0x70:
                                u8ACE3210Value = Hal_HDMI_avi_infoframe_info(ucHDMIInfoSource, _BYTE_14, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]) &BMASK(7:4);

                                if(u8ACE3210Value == 0x00)
                                {
                                    //enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_D65;
                                    MdbPrint(u64ReqHdl, "Color space ADDITIONAL_DCI_P3_RGB_D65\n");
                                }
                                else if(u8ACE3210Value == 0x10)
                                {
                                    //enColormetry = MS_HDMI_EXT_COLOR_ADDITIONAL_DCI_P3_RGB_THEATER;
                                    MdbPrint(u64ReqHdl, "Color space ADDITIONAL_DCI_P3_RGB_THEATER\n");
                                }
                                break;

                            default:
                                //enColormetry = MS_HDMI_EXT_COLOR_UNKNOWN;
                                MdbPrint(u64ReqHdl, "Color space UNKNOWN\n");
                                break;
                        }
                    }
                    MdbPrint(u64ReqHdl, "\n");
                }

                if(pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_VS_PACKET_RECEIVE_FLAG)
                {
                    MdbPrint(u64ReqHdl, "Receive VS packet\n");

                    for(u8PktCounter = 0; u8PktCounter < 28; u8PktCounter ++)
                    {
                        if(Hal_HDMI_get_packet_value(ucHDMIInfoSource, PKT_VS, u8PktCounter, ucPacketInfo, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]))
                        {
                            MdbPrint(u64ReqHdl, "VS packet Byte %d: 0x%x ",u8PktCounter, ucPacketInfo[uctemp]);
                        }
                    }
                    MdbPrint(u64ReqHdl, "\n");
                }

                if(pHDMIRxResourcePrivate->ulPacketStatus[ucHDMIInfoSource] &HDMI_STATUS_HDR_PACKET_RECEIVE_FLAG)
                {
                    MdbPrint(u64ReqHdl, "Receive HDR packet\n");

                    for(u8PktCounter = 0; u8PktCounter < 28; u8PktCounter ++)
                    {
                        if(Hal_HDMI_get_packet_value(ucHDMIInfoSource, PKT_HDR, u8PktCounter, ucPacketInfo, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]))
                        {
                            MdbPrint(u64ReqHdl, "HDR packet Byte %d: 0x%x ",u8PktCounter, ucPacketInfo[uctemp]);
                        }
                    }
                    MdbPrint(u64ReqHdl, "\n");
                }

                // ------ HDMI AUDIO BEGIN -----
                Hal_HDMI_Get_AudioInfo(&stAudioInfo, ucHDMIInfoSource, &pHDMIRxResourcePrivate->stHDMIPollingInfo[ucHDMIInfoSource]);
                MdbPrint(u64ReqHdl, "CTS:%d, N:%d, TMDS Clock:%d, FS: %d \n",
                    stAudioInfo.u32AudioInfo_CTS, stAudioInfo.u32AudioInfo_N, stAudioInfo.u32ClockRates, stAudioInfo.u32AudioInfo_FS);
                MdbPrint(u64ReqHdl, "Audio Mux:%d, Audio Status Enable:0x%x,  Audio Mute Enable:0x%x \n",
                    stAudioInfo.u8AudioMux, stAudioInfo.u8AudioStatusEnable, stAudioInfo.u8AudioMuteEnable);
                // ------ HDMI AUDIO END -----

                // ------ HDMI HDCP BEGIN -----
                Hal_HDCP_clearflag((E_MUX_INPUTPORT)(INPUT_PORT_DVI0 +ucHDMIInfoSource));
                stHDCPInfo.u16HDCP14IrqStatus = Hal_HDCP_getstatus((E_MUX_INPUTPORT)(INPUT_PORT_DVI0 +ucHDMIInfoSource));
                stHDCPInfo.u16HDCP22HwControlStatus = Hal_HDCP_Read22HwControl((E_MUX_INPUTPORT)(INPUT_PORT_DVI0 +ucHDMIInfoSource));
                MdbPrint(u64ReqHdl, "HDCP14 IRQ Status:0x%x, HDCP22 HW Control Status:0x%x \n",
                    stHDCPInfo.u16HDCP14IrqStatus, stHDCPInfo.u16HDCP22HwControlStatus);
                // ------ HDMI HDCP END -----


            }
            else
            {
                MdbPrint(u64ReqHdl, "Signal invalid\n");
            }
        }
    }

    UtopiaResourceRelease(pHDMIRxResource);
}

#endif
#endif
