//<MStar Software>
//******************************************************************************
// MStar Software
// Copyright (c) 2010 - 2012 MStar Semiconductor, Inc. All rights reserved.
// All software, firmware and related documentation herein ("MStar Software") are
// intellectual property of MStar Semiconductor, Inc. ("MStar") and protected by
// law, including, but not limited to, copyright law and international treaties.
// Any use, modification, reproduction, retransmission, or republication of all
// or part of MStar Software is expressly prohibited, unless prior written
// permission has been granted by MStar.
//
// By accessing, browsing and/or using MStar Software, you acknowledge that you
// have read, understood, and agree, to be bound by below terms ("Terms") and to
// comply with all applicable laws and regulations:
//
// 1. MStar shall retain any and all right, ownership and interest to MStar
//    Software and any modification/derivatives thereof.
//    No right, ownership, or interest to MStar Software and any
//    modification/derivatives thereof is transferred to you under Terms.
//
// 2. You understand that MStar Software might include, incorporate or be
//    supplied together with third party`s software and the use of MStar
//    Software may require additional licenses from third parties.
//    Therefore, you hereby agree it is your sole responsibility to separately
//    obtain any and all third party right and license necessary for your use of
//    such third party`s software.
//
// 3. MStar Software and any modification/derivatives thereof shall be deemed as
//    MStar`s confidential information and you agree to keep MStar`s
//    confidential information in strictest confidence and not disclose to any
//    third party.
//
// 4. MStar Software is provided on an "AS IS" basis without warranties of any
//    kind. Any warranties are hereby expressly disclaimed by MStar, including
//    without limitation, any warranties of merchantability, non-infringement of
//    intellectual property rights, fitness for a particular purpose, error free
//    and in conformity with any international standard.  You agree to waive any
//    claim against MStar for any loss, damage, cost or expense that you may
//    incur related to your use of MStar Software.
//    In no event shall MStar be liable for any direct, indirect, incidental or
//    consequential damages, including without limitation, lost of profit or
//    revenues, lost or damage of data, and unauthorized system use.
//    You agree that this Section 4 shall still apply without being affected
//    even if MStar Software has been modified by MStar in accordance with your
//    request or instruction for your use, except otherwise agreed by both
//    parties in writing.
//
// 5. If requested, MStar may from time to time provide technical supports or
//    services in relation with MStar Software to you for your use of
//    MStar Software in conjunction with your or your customer`s product
//    ("Services").
//    You understand and agree that, except otherwise agreed by both parties in
//    writing, Services are provided on an "AS IS" basis and the warranty
//    disclaimer set forth in Section 4 above shall apply.
//
// 6. Nothing contained herein shall be construed as by implication, estoppels
//    or otherwise:
//    (a) conferring any license or right to use MStar name, trademark, service
//        mark, symbol or any other identification;
//    (b) obligating MStar or any of its affiliates to furnish any person,
//        including without limitation, you and your customers, any assistance
//        of any kind whatsoever, or any information; or
//    (c) conferring any license or right under any intellectual property right.
//
// 7. These terms shall be governed by and construed in accordance with the laws
//    of Taiwan, R.O.C., excluding its conflict of law rules.
//    Any and all dispute arising out hereof or related hereto shall be finally
//    settled by arbitration referred to the Chinese Arbitration Association,
//    Taipei in accordance with the ROC Arbitration Law and the Arbitration
//    Rules of the Association by three (3) arbitrators appointed in accordance
//    with the said Rules.
//    The place of arbitration shall be in Taipei, Taiwan and the language shall
//    be English.
//    The arbitration award shall be final and binding to both parties.
//
//******************************************************************************
//<MStar Software>
#ifndef _HAL_PNL_C_
#define _HAL_PNL_C_

//-------------------------------------------------------------------------------------------------
//  Include Files
//-------------------------------------------------------------------------------------------------

#include "MsCommon.h"
#include "MsTypes.h"
#include "utopia.h"
#include "utopia_dapi.h"
#include "apiPNL.h"
#include "apiPNL_v2.h"
#include "apiXC.h"
#include "drvPNL.h"
#include "halPNL.h"
#include "PNL_private.h"
#include "pnl_hwreg_utility2.h"
#include "M7332_864M_pnl_lpll_tbl.h"
#include "drvPM.h"
#include "halPNL_test_curve.h"

#ifdef MSOS_TYPE_LINUX_KERNEL
#include <linux/string.h>
#include <linux/delay.h>
#include <asm/div64.h>
#else
#include "string.h"
#define do_div(x,y) ((x)/=(y))
#endif

//-------------------------------------------------------------------------------------------------
//  Driver Compiler Options
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Defines
//-------------------------------------------------------------------------------------------------
#ifdef MSOS_TYPE_LINUX_KERNEL
#define PNL_TEST_LOG(_fmt, _args...) //printk(_fmt, ##_args);
#else
#define PNL_TEST_LOG(_fmt, _args...) //printf(_fmt, ##_args);
#endif

#define UNUSED(x)       (void)(x)
#if 1
#define HAL_PNL_DBG(_dbgSwitch_, _fmt, _args...)      { if((_dbgSwitch_ & _u16PnlDbgSwitch) != 0) printf("PNL:"_fmt, ##_args); }
#define HAL_MOD_CAL_DBG(x)    //x
#else
#define HAL_PNL_DBG(_dbgSwitch_, _fmt, _args...)      { }
#endif
//if mboot read eFuse and fill the register, then add this define to mark utopia efuse code flow
#define MOD_EFUSE_IN_MBOOT

#define DAC_LPLL_ICTRL     0x0002
#define LVDS_LPLL_ICTRL    0x0001

//Get MOD calibration time
#define MOD_CAL_TIMER   FALSE

#define FRC_PIPE_DELAY_VCNT_FSC_FHD     0x0A
#define FRC_PIPE_DELAY_HCNT_FSC_FHD     0x140

//FRC miu mask address
#define MIU0_G4_REQUEST_Addr 0x310506
#define MIU0_G5_REQUEST_Addr 0x310606

#define PQ_GAMMA_TABLE_SIZE_MAX (256*2*3)    //256 entry * sizeof(MS_U16) * 3 channel

#define VSTART_OFFSET           (2) //same setting as in mhal_frc.h
#define SC0_PATH_DLY_FSC2X_V    (6) //same setting as in mhal_xc_chip_config.h
#define ENABLE_FIX_PP_DLY   (TRUE)  //same setting as in mhal_xc_chip_config.h

#define HW_RINT_THRESHOLD   (0x400) //HW rint detect threshold, if xtal = 12M, uses 0x400,   if xtal = 24M , uses 0x200.

#ifdef CONFIG_UTOPIA_PROC_DBG_SUPPORT
#ifdef MSOS_TYPE_LINUX_KERNEL
#define atoi(str) simple_strtoul(((str != NULL) ? str : ""), NULL, 0)
#endif
#endif    //CONFIG_UTOPIA_PROC_DBG_SUPPORT
//-------------------------------------------------------------------------------------------------
//  Local Structurs
//-------------------------------------------------------------------------------------------------
typedef enum
{
    E_HFRC_FHDtoFHD,
    E_HFRC_4KtoFHD,
    E_HFRC_MAX,
}EN_HFRC_TIMING;

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
//old od table
#define OVERDRIVER_TABLE_SIZE_33X32    1056
//new od table
#define OVERDRIVER_TABLE_SIZE_33X33    1089

#define OVERDRIVER_SRAM1_SIZE (289)
#define OVERDRIVER_SRAM2_SIZE (272)
#define OVERDRIVER_SRAM3_SIZE (272)
#define OVERDRIVER_SRAM4_SIZE (256)
#define OD_DECRYPTION_IDX_1 (9)
#define OD_DECRYPTION_IDX_2 (10)
#define XC_AUTODOWNLOAD_OVERDRIVER_DEPTH  (296)
ST_PNL_OD_NEW_MODE_DRAM_FORMAT* g_pstOD_ADL_CMD = NULL;

#define LANE_NUM_EACH_PINMAPPING_GROUP1 4
#define LANE_NUM_EACH_PINMAPPING_GROUP2 4
#define LANE_NUM_EACH_PINMAPPING_GROUP3 4
#define LANE_NUM_EACH_PINMAPPING_GROUP4 2

#define PINMAPPING_EXP 16

#define MHZ (1000000UL)


extern MS_BOOL MApi_XC_MLoad_WriteCmd_NonXC(void *pInstance,MS_U32 u32Bank,MS_U32 u32Addr, MS_U16 u16Data, MS_U16 u16Mask);
extern MLOAD_TYPE MApi_XC_MLoad_GetStatus_U2(void *pInstance);
extern MS_BOOL MApi_XC_MLoad_WriteCmd_And_Fire(MS_U32 u32Addr, MS_U16 u16Data, MS_U16 u16Mask);

//-------------------------------------------------------------------------------------------------
//  Local Variables
//-------------------------------------------------------------------------------------------------
MS_U8 LANE_AND_CLK_TBL[VBY1_CLK_TBL_ROW][3]=
{ //lane(from)  lane(to) bit(mask)
 { 0, 3, 0x02, },
 { 4, 6, 0x04, },
 { 7, 9, 0x08, },
 { 10, 13, 0x10, }
};
//-------------------------------------------------------------------------------------------------
//  Debug Functions
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Local Functions
//-------------------------------------------------------------------------------------------------
static void _MHal_PNL_RegSettingAfterLpllPowerOn(void *pInstance);

static void _MHal_PNL_Auto_Set_Config(void *pInstance,
                                      MS_U16 u16OutputOrder0_3,
                                      MS_U16 u16OutputOrder4_7,
                                      MS_U16 u16OutputOrder8_11,
                                      MS_U16 u16OutputOrder12_13);
#if 0
static void _MHal_PNL_Set_Clk(void *pInstance,
                              MS_U8 u8LaneNum,
                              MS_U16 u16OutputOrder0_3,
                              MS_U16 u16OutputOrder4_7,
                              MS_U16 u16OutputOrder8_11,
                              MS_U16 u16OutputOrder12_13);
#endif

static MS_U8 _MHal_PNL_Get_LaneNum(void *pInstance);

static MS_BOOL _Hal_MOD_External_eFuse(void);
extern ST_PNL_TCON_Config stTCONPanelConfig;
static MS_BOOL _MHal_PNL_CheckMODCalibrationEn(void *pInstance);

static MS_BOOL _MHal_PNL_DumpBiasConFromEfuse(void *pInstance);

static MS_BOOL _MHal_PNL_openYMYTestCase(void);
//-------------------------------------------------------------------------------------------------
//  Global Function
//-------------------------------------------------------------------------------------------------
/**
*   @brief: Power On MOD. but not mutex protected
*
*/
MS_U8 MHal_MOD_PowerOn(void *pInstance, MS_BOOL bEn, MS_U8 u8LPLL_Type,MS_U8 DualModeType, MS_U16 u16OutputCFG0_7, MS_U16 u16OutputCFG8_15, MS_U16 u16OutputCFG16_21)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if( bEn )
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, 0x00, BIT(8));

        //analog MOD power down. 1: power down, 0: power up
        // For Mod2 no output signel
        ///////////////////////////////////////////////////

        //2. Power on MOD (current and regulator)
        if(u8LPLL_Type == E_PNL_TYPE_TTL)
        {
            if(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_3A_L,BIT(0))!=BIT(0))
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, BIT(0) , BIT(0));
            }
        }
        else
        {
            if(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_3A_L,BIT(0))!=0)
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0x00 , BIT(0));
            }
        }
        if(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_3A_L,BIT(8))!=BIT(8))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, BIT(8) , BIT(8));
        }

        // Power on LPLL
        if(R2BYTEMSK(L_BK_LPLL(0x03),BIT(5))!= 0)
        {
            MHal_PNL_PowerDownLPLL(pInstance, FALSE);
        }

        //enable ib, enable ck
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, (BIT(1) | BIT(0)), (BIT(1) | BIT(0)));

        // clock gen of dot-mini
        if(u8LPLL_Type == E_PNL_TYPE_MINILVDS)
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x4400);
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x4400);
        }
        else if((u8LPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE)||
                (u8LPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE)||
                (u8LPLL_Type == E_PNL_LPLL_VBY1_8BIT_4LANE) ||
                (u8LPLL_Type == E_PNL_LPLL_VBY1_8BIT_2LANE))

        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0400);  // [11:8]reg_ckg_dot_mini_pre2_osd
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0044);  // [3:0]reg_ckg_dot_mini_osd
                                                    // [7:4]reg_ckg_dot_mini_pre_osd
        }
        //// for osd dedicated output port, 1 port for video and 1 port for osd
        else if((u8LPLL_Type == E_PNL_TYPE_HS_LVDS)&&
                (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Mode == E_PNL_MODE_SINGLE))
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0400);  // [11:8]reg_ckg_dot_mini_pre2_osd
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0044);  // [3:0]reg_ckg_dot_mini_osd
                                                    // [7:4]reg_ckg_dot_mini_pre_osd
        }
        #if 0 // this only for 8V4O case,  no need this for Maxim 23x23 8V case
        else if(u8LPLL_Type == E_PNL_LPLL_VBY1_10BIT_8LANE)
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0440);  // [11:8]reg_ckg_dot_mini_pre2_osd
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0044);  // [3:0]reg_ckg_dot_mini_osd
        }
        #endif
        //Vby1, LVDS case
        else
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0000);
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0000);
        }

        //8V4O case
        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Ext_LpllType == E_PNL_LPLL_VBY1_10BIT_4LANE)
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0400);  // [11:8]reg_ckg_dot_mini_pre2_osd
            MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0044);  // [3:0]reg_ckg_dot_mini_osd, // [7:4]reg_ckg_dot_mini_pre_osd
        }

        //After setting of clk gen, need to dump Bias con value from Efuse.
        _MHal_PNL_DumpBiasConFromEfuse(pInstance);

        //VBY1 training en.
        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.bVideo_HW_Training_En)
        {
            MHal_PNL_VBY1_Hardware_TrainingMode_En(pInstance, TRUE, ENABLE);
        }

        MOD_W2BYTEMSK(REG_MOD_BK00_7D_L, 0x0080, 0x0080); // reg_proc_st_clr
        MOD_W2BYTEMSK(REG_MOD_BK00_7D_L, 0x0000, 0x0080);

        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0x0000, 0x8000);
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0x8000, 0x8000);

        /* RD recommand code flow
         * MOD related setting -> force fix read pointer -> output config
         * NO more any other mod setting after output config
         */
        // this function for mod register setting after power on
        _MHal_PNL_RegSettingAfterLpllPowerOn(pInstance);

        MOD_W2BYTEMSK(REG_MOD_BK00_5B_L, 0x0008, 0x0008); // reg_fix_cnt_clr
        MOD_W2BYTEMSK(REG_MOD_BK00_5B_L, 0x0000, 0x0008);

        // 3. 4. 5.
        if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
        {
            MHal_Output_LVDS_Pair_Setting(pInstance, DualModeType, u16OutputCFG0_7, u16OutputCFG8_15, u16OutputCFG16_21);
        }

        MsOS_DelayTask(10); //Wait vb1 training stable.
        MOD_W2BYTEMSK(REG_MOD_BK00_7E_L, 0x0080, 0x0080); // reg_unlock_cnt_clr
        MOD_W2BYTEMSK(REG_MOD_BK00_7E_L, 0x0000, 0x0080);
    }
    else
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, BIT(8), BIT(8));
        if(u8LPLL_Type !=E_PNL_TYPE_MINILVDS)
        {
            if(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_3A_L,BIT(0))!=BIT(0))
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, BIT(0), BIT(0));                              //analog MOD power down. 1: power down, 0: power up
            }
            if(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_3A_L,BIT(8))!=0)
            {
               MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0x00, BIT(8));
            }
        }

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0, (BIT(1) | BIT(0) ));                           //enable ib, enable ck

        // clock gen of dot-mini
        MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x1100);
        MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x1111);

        if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0020,0x0020);
            MsOS_DelayTask(1);
            MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0000);
            MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0000);
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0000,0x0020);
        }
    }
    return 1;
}

/**
*   @brief: Setup the PVDD power 1:2.5V, 0:3.3V
*
*/
void MHal_MOD_PVDD_Power_Setting(void *pInstance, MS_BOOL bIs2p5)
{
    //MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, ((bIs2p5)? BIT(6):0), BIT(6));    //MOD PVDD=1: 0.9
}

void MHal_PNL_TCON_Init(void *pInstance)
{

}

void MHal_Shift_LVDS_Pair(void *pInstance, MS_U8 Type)
{
    if(Type == 1)
        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, (BIT(7) | BIT(6)), (BIT(7) | BIT(6))); // shift_lvds_pair, set LVDS Mode3
    else
        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, 0x0000, (BIT(7) | BIT(6)));

}

void MHal_Output_LVDS_Pair_Setting(void *pInstance, MS_U8 Type, MS_U16 u16OutputCFG0_7, MS_U16 u16OutputCFG8_15, MS_U16 u16OutputCFG16_21)
{

    if(Type == LVDS_DUAL_OUTPUT_SPECIAL )
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0555);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x1554);
    }
    else if(Type == LVDS_SINGLE_OUTPUT_A)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0x5550, 0xFFF0);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0555);
    }
    else if( Type == LVDS_SINGLE_OUTPUT_B)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0x5550, 0xFFF0);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0555);
    }
    else if( Type == LVDS_OUTPUT_User)
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, u16OutputCFG0_7);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, u16OutputCFG8_15);
    }
    else
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0x5550, 0xFFF0);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0555);
    }

    MsOS_DelayTask(2);


}

static MS_U8 _MHal_PNL_Get_LaneNum(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    MS_U8 u8LaneNum = 0;
    //check lane num
    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE)
     ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
    {
        u8LaneNum = 8;
    }
    else if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_4LANE)
          ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_4LANE))
    {
        u8LaneNum = 4;
    }
    else if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_2LANE)
          ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_2LANE))
    {
        u8LaneNum = 2;
    }
    else
    {
        u8LaneNum = 0;
    }
    return u8LaneNum;
}


static void _MHal_PNL_Auto_Set_Config(void *pInstance,
                                      MS_U16 u16OutputOrder0_3,
                                      MS_U16 u16OutputOrder4_7,
                                      MS_U16 u16OutputOrder8_11,
                                      MS_U16 u16OutputOrder12_13)
{
    //attention : This function just support vby1 now.
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    MS_U8   u8OutputConfigCount = 0;
    MS_U16  u16Config =0;
    MS_U8   u8Count = 0;
    MS_U8   u8LaneNum = 0;
    MS_BOOL bSkip = TRUE;
#if 0
    //check lane num

    if(u8LaneNum!=0)
    {
        bSkip = FALSE;
    }
    else
    {
        bSkip = TRUE;

        //use default config
        MHal_Output_LVDS_Pair_Setting(pInstance,
                                      pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type,
                                      pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7,
                                      pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15,
                                      pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);
    }
#endif
    if(IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext))
    {
        u8LaneNum = _MHal_PNL_Get_LaneNum(pInstance);
        bSkip = FALSE;
    }

    if(!bSkip)
    {
        //set output config
        u16Config = 0;
        u8OutputConfigCount = 0;
        for( u8Count = 0 ; u8Count < LANE_NUM_EACH_PINMAPPING_GROUP1 ; u8Count++ )
        {
            if( ( u16OutputOrder0_3 % PINMAPPING_EXP ) < u8LaneNum)
            {
                u16Config += CONFIG_FOR_VBY1_DATA<<u8OutputConfigCount;
            }
            u16OutputOrder0_3 = u16OutputOrder0_3 / PINMAPPING_EXP;
            u8OutputConfigCount += CONFIG_FOR_VBY1_DATA_BIT_NUM;
        }
        for( u8Count = 0 ; u8Count < LANE_NUM_EACH_PINMAPPING_GROUP2 ; u8Count++ )
        {
            if( (u16OutputOrder4_7 % PINMAPPING_EXP ) < u8LaneNum)
            {
                u16Config += CONFIG_FOR_VBY1_DATA<<u8OutputConfigCount;
            }
            u16OutputOrder4_7 = u16OutputOrder4_7 / PINMAPPING_EXP;
            u8OutputConfigCount += CONFIG_FOR_VBY1_DATA_BIT_NUM;
        }
        pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7 = u16Config;

        u16Config =0;
        u8OutputConfigCount = 0;
        for( u8Count = 0 ; u8Count < LANE_NUM_EACH_PINMAPPING_GROUP3 ; u8Count++ )
        {
            if( (u16OutputOrder8_11 % PINMAPPING_EXP ) < u8LaneNum)
            {
                u16Config += CONFIG_FOR_VBY1_DATA<<u8OutputConfigCount;
            }
            u16OutputOrder8_11 = u16OutputOrder8_11 / PINMAPPING_EXP;
            u8OutputConfigCount += CONFIG_FOR_VBY1_DATA_BIT_NUM;
        }
        for( u8Count = 0 ; u8Count < LANE_NUM_EACH_PINMAPPING_GROUP4 ; u8Count++ )
        {
            if( (u16OutputOrder12_13 % PINMAPPING_EXP ) < u8LaneNum)
            {
                u16Config += CONFIG_FOR_VBY1_DATA<<u8OutputConfigCount;
            }
            u16OutputOrder12_13 = u16OutputOrder12_13 / PINMAPPING_EXP;
            u8OutputConfigCount += CONFIG_FOR_VBY1_DATA_BIT_NUM;
        }
        pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15 = u16Config;
    }
}

#if 0
static void _MHal_PNL_Set_Clk(void *pInstance,
                              MS_U8 Type,
                              MS_U16 u16OutputOrder0_3,
                              MS_U16 u16OutputOrder4_7,
                              MS_U16 u16OutputOrder8_11,
                              MS_U16 u16OutputOrder12_13)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if(Type == APIPNL_OUTPUT_CHANNEL_ORDER_USER )
    {
        MS_U8 u8Clk = 0;
        MS_U8   u8LaneNum = 0;
        MS_BOOL bSkip = TRUE;
        MS_U8   u8Count = 0;
        MS_U8   u8Count1 = 0;
        MS_U8   u8StartLane = 0;

        //check lane num
        u8LaneNum = _MHal_PNL_Get_LaneNum(pInstance);
        if(u8LaneNum!=0)
        {
            bSkip = FALSE;
        }
        else
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x1F, 0x1F); //open all clk
            bSkip = TRUE;
        }

        //count clk
        if(!bSkip)
        {
            u8Clk = 0;
            u8StartLane = 0;
            for( u8Count = u8StartLane ; u8Count < (u8StartLane+LANE_NUM_EACH_PINMAPPING_GROUP1) ; u8Count++ )
            {//lane 0 - lane 3
                if( ( u16OutputOrder0_3 % PINMAPPING_EXP ) < u8LaneNum)
                {
                    u8Count1 = 0;
                    do
                    {
                        if(u8Count>=LANE_AND_CLK_TBL[u8Count1][0] && u8Count<=LANE_AND_CLK_TBL[u8Count1][1])
                        {
                            u8Clk |= LANE_AND_CLK_TBL[u8Count1][2];
                            u8Clk |= 0x01;//mod_a_reg38 [1]:clk enable
                            break;
                        }
                        u8Count1 ++;
                    }
                    while(u8Count1<VBY1_CLK_TBL_ROW);
                }
                u16OutputOrder0_3 /= PINMAPPING_EXP;
            }

            u8StartLane = 4;
            for( u8Count = u8StartLane ; u8Count < (u8StartLane+LANE_NUM_EACH_PINMAPPING_GROUP2) ; u8Count++ )
            {//lane 4 - lane 7
                if( ( u16OutputOrder4_7 % PINMAPPING_EXP ) < u8LaneNum)
                {
                    u8Count1 = 0;
                    do
                    {
                        if(u8Count>=LANE_AND_CLK_TBL[u8Count1][0] && u8Count<=LANE_AND_CLK_TBL[u8Count1][1])
                        {
                            u8Clk |= LANE_AND_CLK_TBL[u8Count1][2];
                            u8Clk |= 0x01;//mod_a_reg38 [1]:clk enable
                            break;
                        }
                        u8Count1 ++;
                    }
                    while(u8Count1<VBY1_CLK_TBL_ROW);
                }
                u16OutputOrder4_7 /= PINMAPPING_EXP;
            }

            u8StartLane = 8;
            for( u8Count = u8StartLane ; u8Count < (u8StartLane+LANE_NUM_EACH_PINMAPPING_GROUP3) ; u8Count++ )
            {//lane 8 - lane 11
                if( ( u16OutputOrder8_11 % PINMAPPING_EXP ) < u8LaneNum)
                {
                    u8Count1 = 0;
                    do
                    {
                        if(u8Count>=LANE_AND_CLK_TBL[u8Count1][0] && u8Count<=LANE_AND_CLK_TBL[u8Count1][1])
                        {
                            u8Clk |= LANE_AND_CLK_TBL[u8Count1][2];
                            u8Clk |= 0x01;//mod_a_reg38 [1]:clk enable
                            break;
                        }
                        u8Count1 ++;
                    }
                    while(u8Count1<VBY1_CLK_TBL_ROW);
                }
                u16OutputOrder8_11 /= PINMAPPING_EXP;

            }

            u8StartLane = 12;
            for( u8Count = u8StartLane ; u8Count < (u8StartLane+LANE_NUM_EACH_PINMAPPING_GROUP4) ; u8Count++ )
            {//lane 12 - lane 13
                if( ( u16OutputOrder12_13 % PINMAPPING_EXP ) < u8LaneNum)
                {
                    u8Count1 = 0;
                    do
                    {
                        if(u8Count>=LANE_AND_CLK_TBL[u8Count1][0] && u8Count<=LANE_AND_CLK_TBL[u8Count1][1])
                        {
                            u8Clk |= LANE_AND_CLK_TBL[u8Count1][2];
                            u8Clk |= 0x01;//mod_a_reg38 [1]:clk enable
                            break;
                        }
                        u8Count1 ++;
                    }
                    while(u8Count1<VBY1_CLK_TBL_ROW);
                }
                u16OutputOrder12_13 /= PINMAPPING_EXP;
            }

            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, u8Clk, 0x1F);
        }
    }
    else
    {
        if(   (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE)
            ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
        {
            if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16MOD_CTRLA & BIT(1))
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x0F, 0x1F);
            }
            else
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x0F, 0x1F);
            }
        }
        else if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)
        {//LVDS
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x1F, 0x1F);
        }
        else if(   (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_4LANE)
                 ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_4LANE))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x03, 0x1F);
        }
        else if(   (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_2LANE)
                 ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_2LANE))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x03, 0x1F);
        }
        else
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x1F, 0x1F);
        }
    }
}
#endif

void MHal_Output_Channel_Order(void *pInstance,
                               MS_U8 Type,
                               MS_U16 u16OutputOrder0_3,
                               MS_U16 u16OutputOrder4_7,
                               MS_U16 u16OutputOrder8_11,
                               MS_U16 u16OutputOrder12_13)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if(Type == APIPNL_OUTPUT_CHANNEL_ORDER_USER )
    {
        MOD_W2BYTE(REG_MOD_BK00_08_L, u16OutputOrder0_3);
        MOD_W2BYTE(REG_MOD_BK00_09_L, u16OutputOrder4_7);
        MOD_W2BYTE(REG_MOD_BK00_0A_L, u16OutputOrder8_11);
        MOD_W2BYTE(REG_MOD_BK00_0B_L, u16OutputOrder12_13);
    }
    else
    {
        if(   (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE)
            ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
        {
            if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16MOD_CTRLA & BIT(1)) // 2 Divisoin
            {
                MOD_W2BYTE(REG_MOD_BK00_08_L, 0x6420);
                MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7531);
                MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
                MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
            }
            else
            {
                MOD_W2BYTE(REG_MOD_BK00_08_L, 0x3210);
                MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7654);
                MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
                MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
            }
        }
        else if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)
        {//LVDS
            MOD_W2BYTE(REG_MOD_BK00_08_L, 0x10DC);
            MOD_W2BYTE(REG_MOD_BK00_09_L, 0x5432);
            MOD_W2BYTE(REG_MOD_BK00_0A_L, 0x9876);
            MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x00BA);
        }
        else
        {// default is VB1 4lane/2lane
            MOD_W2BYTE(REG_MOD_BK00_08_L, 0x3210);
            MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7654);
            MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
            MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
        }
    }

}

void MHal_PQ_Clock_Gen_For_Gamma(void *pInstance)
{
    W2BYTEMSK(REG_CLKGEN0_52_L, 0x00, 0x07);
}

void MHal_VOP_SetGammaMappingMode(void *pInstance, MS_U8 u8Mapping)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if(u8Mapping & GAMMA_MAPPING)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_74_L, (u8Mapping & GAMMA_10BIT_MAPPING)? BIT(15):0, BIT(15));
    }
    else
    {
        PNL_ASSERT(0, "Invalid eSupportGammaMapMode [%d] Passed to [%s], please make sure the u8Mapping[%d] is valid\n.",
                       u8Mapping, __FUNCTION__, u8Mapping);
    }
}

MS_BOOL Hal_VOP_Is_GammaMappingMode_enable(void *pInstance)
{
    // Only support 1024 entry
    return TRUE;
}

// After A5, 8 bit mode only support burst write!!!
MS_BOOL Hal_VOP_Is_GammaSupportSignalWrite(void *pInstance, DRVPNL_GAMMA_MAPPEING_MODE u8Mapping)
{
    if( u8Mapping == E_DRVPNL_GAMMA_10BIT_MAPPING )
        return TRUE;
    else
        return FALSE;
}


void hal_PNL_WriteGamma12Bit(void *pInstance, MS_U8 u8Channel, MS_BOOL bBurstWrite, MS_U16 u16Addr, MS_U16 u16GammaValue)
{
    MS_U16 u16Delay = 0xFFFF;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    PNL_DBG(PNL_DBGLEVEL_GAMMA, "Write SC%tu [ch %d][addr 0x%x]: 0x%x \n", (ptrdiff_t)pPNLInstancePrivate->u32DeviceID, u8Channel, u16Addr, u16GammaValue);

    if (!bBurstWrite )
    {
        while (SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L, 0xE0) && (--u16Delay));          // Check whether the Write chanel is ready
        PNL_ASSERT(u16Delay > 0, "%s\n", "WriteGamma timeout");

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6C_L, u16Addr, 0x3FF);                          // set address port
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, (REG_SC_BK10_6E_L + u8Channel *2), u16GammaValue, 0xFFF);      // Set channel data

        // kick off write
        switch(u8Channel)
        {
            case 0:  // Red
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L, BIT(5), BIT(5));
                break;

            case 1:  // Green
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L, BIT(6), BIT(6));
                break;

            case 2:  // Blue
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L, BIT(7), BIT(7));
                break;
        }

        while (SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L, 0xE0) && (--u16Delay));          // Check whether the Write chanel is ready
    }
    else
    {

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_72_L, u16GammaValue << 2, 0x3FFC);
        SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK00_7F_L, 0x00); // make little time delay
    }


    PNL_ASSERT(u16Delay > 0, "%s\n", "WriteGamma timeout");

}


void hal_PNL_SetMaxGammaValue(void *pInstance, MS_U8 u8Channel, MS_U16 u16MaxGammaValue)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    PNL_DBG(PNL_DBGLEVEL_GAMMA, "Max gamma of SC%tu %d is 0x%x\n", (ptrdiff_t)pPNLInstancePrivate->u32DeviceID, u8Channel, u16MaxGammaValue);
#ifdef MONACO_SC2
    if(pPNLInstancePrivate->u32DeviceID == 0)
    {
#endif
           switch(u8Channel)
        {
            case 0:  // max. Red
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_7D_L , u16MaxGammaValue << 2, 0x3FFC);           // max. base 0
                break;

            case 1:  // max. Green
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_7E_L , u16MaxGammaValue << 2, 0x3FFC);           // max. base 1
                break;

            case 2:  //max.  Blue
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_7F_L , u16MaxGammaValue << 2, 0x3FFC);           // max. base 1
                break;
           }
#ifdef MONACO_SC2
    }else    //Nike
    {
    switch(u8Channel)
    {
        case 0:  // max. Red
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7A_L , u16MaxGammaValue, 0xFFF);           // max. base 0
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7B_L , u16MaxGammaValue, 0xFFF);           // max. base 1
            break;

        case 1:  // max. Green
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7C_L , u16MaxGammaValue, 0xFFF);           // max. base 0
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7D_L , u16MaxGammaValue, 0xFFF);           // max. base 1
            break;

        case 2:  //max.  Blue
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7E_L , u16MaxGammaValue, 0xFFF);           // max. base 0
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_7F_L , u16MaxGammaValue, 0xFFF);           // max. base 1
            break;
     }

    }
#endif
}

/////////////////////////////////////////////////////////////////////////////
// Gamma format (12 bit LUT)
//      0, 1, 2, 3, ..., NumOfLevel, totally N Sets of tNormalGammaR/G/B[],
//      1 set uses 2 bytes of memory.
//
// [T2 and before ] N = 256
// [T3]             N = 256 or 1024
// ______________________________________________________________________________
//  Byte | 0         1           2               n-1        n
//    [G1|G0]       [G0]       [G1] . ...... .  [Gmax]    [Gmax]
//    3:0  3:0      11:4       11:4              3:0       11:4
//

void Hal_PNL_Set12BitGammaPerChannel(void *pInstance, MS_U8 u8Channel, MS_U8 * u8Tab, DRVPNL_GAMMA_MAPPEING_MODE GammaMapMode)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 u16Addr            = 0;
    MS_U16 u16CodeTableIndex  = u16Addr/2*3;
    MS_U16 u16GammaValue      = 0;
    MS_U16 u16MaxGammaValue   = 0;
    MS_U16 u16NumOfLevel = GammaMapMode == E_DRVPNL_GAMMA_8BIT_MAPPING ? 256 : 1024;
    MS_BOOL bUsingBurstWrite = !Hal_VOP_Is_GammaSupportSignalWrite(pInstance,GammaMapMode);

    // Go to burst write if not support
    if ( bUsingBurstWrite )
    {
        // 1.   initial burst write address, LUT_ADDR[7:0]
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_71_L, 0x00 , 0x3FF);

        // 2.   select burst write channel, REG_LUT_BW_CH_SEL[1:0]
        switch(u8Channel)
        {
            case 0:  // Red
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, 0x00 , BIT(6) | BIT(5) );
                break;

            case 1:  // Green
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(5) , BIT(6) | BIT(5) );
                break;

            case 2:  // Blue
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(6) , BIT(6) | BIT(5) );
                break;
        }

        // 3.   enable burst write mode, REG_LUT_BW_MAIN_EN
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(7) , BIT(7)); // Burst write enable

    }

    //printf("\33[0;31m Gamma Mapping mode %d \n \33[m",GammaMapMode );
    // write gamma table per one channel
    for(; u16Addr < u16NumOfLevel; u16CodeTableIndex += 3)
    {
        // gamma x
        u16GammaValue = u8Tab[u16CodeTableIndex] & 0x0F;
        u16GammaValue |= u8Tab[u16CodeTableIndex+1] << 4;

        PNL_DBG(PNL_DBGLEVEL_GAMMA,"Gamma x: SrcGTbl[%d] = 0x%x, [%d] = 0x%x, Gamma = 0x%x, GammaLvl=%d\n",
                                    u16CodeTableIndex, u8Tab[u16CodeTableIndex], u16CodeTableIndex+1, u8Tab[u16CodeTableIndex+1], u16GammaValue, u16NumOfLevel);

        if(u16MaxGammaValue < u16GammaValue)
        {
            u16MaxGammaValue = u16GammaValue;
        }

        // write gamma value
        hal_PNL_WriteGamma12Bit(pInstance,u8Channel,bUsingBurstWrite, u16Addr, u16GammaValue);
        u16Addr++;

        // gamma x+1
        u16GammaValue = (u8Tab[u16CodeTableIndex] & 0xF0) >> 4;
        u16GammaValue |= u8Tab[u16CodeTableIndex+2] << 4;

        PNL_DBG(PNL_DBGLEVEL_GAMMA, "Gamma x+1: SrcGTbl[%d] = 0x%x, [%d] = 0x%x, Gamma = 0x%x\n", u16CodeTableIndex, u8Tab[u16CodeTableIndex], u16CodeTableIndex+2, u8Tab[u16CodeTableIndex+2], u16GammaValue);

        if(u16MaxGammaValue < u16GammaValue)
        {
            u16MaxGammaValue = u16GammaValue;
        }

        // write gamma value
        hal_PNL_WriteGamma12Bit(pInstance,u8Channel,bUsingBurstWrite, u16Addr, u16GammaValue);
        u16Addr++;
    }

    if ( bUsingBurstWrite )
    {
        // 5.   after finish burst write data of one channel, disable burst write mode
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, 0x00 , BIT(7));
    }

    hal_PNL_SetMaxGammaValue(pInstance,u8Channel, u16MaxGammaValue);
}
/////////////////////////////////////////////////////////////////////////////
// Gamma format (12 bit LUT)
//      0, 1, 2, 3, ..., NumOfLevel, totally N Sets of tNormalGammaR/G/B[],
//      1 set uses 2 bytes of memory.
//
// [T2 and before ] N = 256
// [T3]             N = 256 or 1024
// ______________________________________________________________________________
//  Byte | 0         1           2               n-1        n
//    [G1|G0]       [G0]       [G1] . ...... .  [Gmax]    [Gmax]
//    3:0  3:0      11:4       11:4              3:0       11:4
//
#ifdef MONACO_SC2
void Hal_PNL_Set12BitGammaPerChannel_SC2(void *pInstance, MS_U8 u8Channel, MS_U8 * u8Tab, DRVPNL_GAMMA_MAPPEING_MODE GammaMapMode )
{
    MS_U16 u16Addr             = 0;
    MS_U16 u16CodeTableIndex  = u16Addr/2*3;
    MS_U16 u16GammaValue      = 0;
    MS_U16 u16MaxGammaValue   = 0;
    MS_U16 u16NumOfLevel = GammaMapMode == E_DRVPNL_GAMMA_8BIT_MAPPING ? 256 : 1024;
    MS_BOOL bUsingBurstWrite = !Hal_VOP_Is_GammaSupportSignalWrite(pInstance,GammaMapMode);
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    // Go to burst write if not support
    if ( bUsingBurstWrite )
    {
        // 1.   initial burst write address, LUT_ADDR[7:0]
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6C_L, 0x00 , 0x3FF);

        // 2.   select burst write channel, REG_LUT_BW_CH_SEL[1:0]
        switch(u8Channel)
        {
            case 0:  // Red
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6D_L, 0x00 , BIT(3) | BIT(2) );
                break;

            case 1:  // Green
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6D_L, BIT(2) , BIT(3) | BIT(2) );
                break;

            case 2:  // Blue
                SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6D_L, BIT(3) , BIT(3) | BIT(2) );
                break;
        }

        // 3.   enable burst write mode, REG_LUT_BW_MAIN_EN
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6D_L, BIT(0) , BIT(0)); // Burst write enable

    }

    //printf("\33[0;31m Gamma Mapping mode %d \n \33[m",GammaMapMode );
    // write gamma table per one channel
    for(; u16Addr < u16NumOfLevel; u16CodeTableIndex += 3)
    {
        // gamma x
        u16GammaValue = u8Tab[u16CodeTableIndex] & 0x0F;
        u16GammaValue |= u8Tab[u16CodeTableIndex+1] << 4;

        PNL_DBG(PNL_DBGLEVEL_GAMMA,"Gamma x: SrcGTbl[%d] = 0x%x, [%d] = 0x%x, Gamma = 0x%x, GammaLvl=%d\n",
                                    u16CodeTableIndex, u8Tab[u16CodeTableIndex], u16CodeTableIndex+1, u8Tab[u16CodeTableIndex+1], u16GammaValue, u16NumOfLevel);

        if(u16MaxGammaValue < u16GammaValue)
        {
            u16MaxGammaValue = u16GammaValue;
        }

        // write gamma value
        hal_PNL_WriteGamma12Bit(pInstance,u8Channel,bUsingBurstWrite, u16Addr, u16GammaValue);
        u16Addr++;

        // gamma x+1
        u16GammaValue = (u8Tab[u16CodeTableIndex] & 0xF0) >> 4;
        u16GammaValue |= u8Tab[u16CodeTableIndex+2] << 4;

        PNL_DBG(PNL_DBGLEVEL_GAMMA, "Gamma x+1: SrcGTbl[%d] = 0x%x, [%d] = 0x%x, Gamma = 0x%x\n", u16CodeTableIndex, u8Tab[u16CodeTableIndex], u16CodeTableIndex+2, u8Tab[u16CodeTableIndex+2], u16GammaValue);

        if(u16MaxGammaValue < u16GammaValue)
            {
            u16MaxGammaValue = u16GammaValue;
            }

        // write gamma value
        hal_PNL_WriteGamma12Bit(pInstance,u8Channel,bUsingBurstWrite, u16Addr, u16GammaValue);
        u16Addr++;
    }

    if ( bUsingBurstWrite )
    {
        // 5.   after finish burst write data of one channel, disable burst write mode
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_6D_L, 0x00 , BIT(0));
    }

    hal_PNL_SetMaxGammaValue(pInstance,u8Channel, u16MaxGammaValue);
}
#endif

/**************************************************************************/
/*       THIS IS CHIP-SPECIFIC GAMMA LOADING METHOD!                      */
/*       Before you copy to another chip's Hal layer                      */
/*       Please make sure that the HW design is the same.                 */
/**************************************************************************/
void Hal_PNL_Get12BitGammaPerChannel(void *pInstance, MS_U8 u8Channel, MS_U8 * u8Tab, DRVPNL_GAMMA_MAPPEING_MODE GammaMapMode )
{
    // THERE IS NO BURST READ IN GAMMA.
    if (u8Channel >= 3 )
        return;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 u16NumOfLevel =256;
    MS_U16 u16Addr    = 0;
    MS_U16 u16Timeout = 0;

    //1. set Read channel
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L,u8Channel<<5,BIT(6) | BIT(5));

    for(u16Addr = 0; u16Addr < u16NumOfLevel; u16Addr++)
    {
        // 1. set entry addr
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_71_L, u16Addr, 0xFF);
        // 2. toggle read enable bit(auto toggle)
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(4),BIT(4));
        u16Timeout = 0;
        // after setting toggle bit, we need to make sure the bit is already toggled.
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(4)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTaskUs(10);
            u16Timeout++;
        }
        // 3. read gamma value
        u8Tab[u16Addr] = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_74_L)>>6; // shift 6 bit

    }

}

/**************************************************************************/
/*       THIS IS CHIP-SPECIFIC GAMMA LOADING METHOD!                      */
/*       Before you copy to another chip's Hal layer                      */
/*       Please make sure that the HW design is the same.                 */
/**************************************************************************/
void MHal_PNL_GetGammaPerChannel(void *pInstance, MS_U8 u8Channel, MS_U16 * pu16Tab )
{
    // THERE IS NO BURST READ IN GAMMA.
    if (u8Channel >= 3 )
    {
        return;
    }
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 u16NumOfLevel =256;
    MS_U16 u16Addr    = 0;
    MS_U16 u16Timeout = 0;

    //1. set Read channel
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L,u8Channel<<5,BIT(6) | BIT(5));
    //for(u16Counter = 0,u16Addr = 0; u16Addr < u16NumOfLevel; u16Addr+=2,u16Counter+=3)
    for(u16Addr = 0; u16Addr < u16NumOfLevel; u16Addr++)
    {
        // 1. set entry addr
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_71_L, u16Addr, 0xFF);
        // 2. toggle read enable bit(auto toggle)
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(4),BIT(4));
        u16Timeout = 0;
        // after setting toggle bit, we need to make sure the bit is already toggled.
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_70_L, BIT(4)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTaskUs(10);
            u16Timeout++;
        }
        // 3. read gamma value
        pu16Tab[u16Addr] = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_74_L);

    }

}

// src : 1 (scaler lpll)
// src : 0 (frc lpll)
MS_U8 MHal_PNL_FRC_lpll_src_sel(void *pInstance, MS_U8 u8src)
{
    if (u8src > 1)
    {
        return FALSE;
    }
    else
    {
//Not support two LPLL (frc lpll) for Manhattan
#if 0
        W2BYTEMSK(L_BK_LPLL(0x00), 0x00, 0x0F);
        W2BYTEMSK(L_BK_LPLL(0x7F), u8src?BIT(8):0, BIT(8));

        if(u8src==0)
        {
            W2BYTEMSK(L_BK_LPLL(0x00), 0x01, 0x0F);
            W2BYTEMSK(L_BK_LPLL(0x7F), BIT(8), BIT(8));
            W2BYTEMSK(L_BK_LPLL(0x00), 0x00, 0x0F); // restore to sub bnak 0
        }
#endif
        return TRUE;
    }

}

static MS_U8 _MHal_PNL_GetSupportedLPLLIndex(void *pInstance,
                                                                 PNL_TYPE eLPLL_Type,
                                                                 PNL_MODE eLPLL_Mode,
                                                                 MS_U64 ldHz, PNL_LPLL_TYPE_SEL lpll_type_sel)
{
    MS_U8 u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
#if defined (__aarch64__)
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]eLPLL_Type=%u, eLPLL_Mode=%u, ldHz=%lu, lpll_type_sel=%u\n", __FUNCTION__, __LINE__, eLPLL_Type, eLPLL_Mode, ldHz, lpll_type_sel);
#else
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]eLPLL_Type=%u, eLPLL_Mode=%u, ldHz=%llu, lpll_type_sel=%u\n", __FUNCTION__, __LINE__, eLPLL_Type, eLPLL_Mode, ldHz, lpll_type_sel);
#endif

    MHal_PNL_AdjustDclk(pInstance, &ldHz,eLPLL_Type);
    switch(lpll_type_sel)
    {
        default:
        case E_PNL_LPLL_VIDEO:
        {
            switch (eLPLL_Type)
            {
               case E_PNL_TYPE_DAC_P:
               case E_PNL_TYPE_DAC_I:
                   break;

               case E_PNL_TYPE_TTL:
                   if (ldHz < 750*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_TTL_75to75MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_TTL_75to150MHz;
                   }
               break;

               case E_PNL_TYPE_LVDS:
                   switch (eLPLL_Mode)
                   {
                       case E_PNL_MODE_SINGLE:
                           if (ldHz < 2975*MHZ)
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_1ch_297_5to297_5MHz;
                           }
                           else if ((ldHz >= 2975*MHZ) && (ldHz < 4500*MHZ))
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_1ch_297_5to450MHz;
                           }
                           else
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_1ch_450to595MHz;
                           }
                       break;

                       default:
                       case E_PNL_MODE_DUAL:
                           if (ldHz < 3000*MHZ)
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_2ch_300to300MHz;
                           }
                           else if ((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_2ch_300to450MHz;
                           }
                           else
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_LVDS_2ch_450to600MHz;
                           }
                       break;
                   }
               break;

               case E_PNL_TYPE_HS_LVDS:

                   switch (eLPLL_Mode)
                   {
                       case E_PNL_MODE_SINGLE:
                           if(ldHz < 3000*MHZ)
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_1ch_300to300MHz;
                           }
                           else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_1ch_300to450MHz;
                           }
                           else
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_1ch_450to600MHz;
                           }
                       break;

                       default:
                       case E_PNL_MODE_DUAL:
                           if(ldHz < 3000*MHZ)
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_2ch_300to300MHz;
                           }
                           else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_2ch_300to450MHz;
                           }
                           else
                           {
                               u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_HS_LVDS_2ch_450to600MHz;
                           }
                       break;
                   }
               break;

               case E_PNL_LPLL_MINILVDS_1CH_3P_6BIT:
                   if(ldHz < 2975*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_6bit_297_5to297_5MHz;
                   }
                   else if((ldHz >= 2975*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_6bit_297_5to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_6bit_450to595MHz;
                   }
               break;
               case E_PNL_LPLL_MINILVDS_1CH_6P_6BIT:
                   if(ldHz < 2975*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_6bit_297_5to297_5MHz;
                   }
                   else if((ldHz >= 2975*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_6bit_297_5to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_6bit_450to595MHz;
                   }
               break;
               case E_PNL_LPLL_MINILVDS_2CH_3P_6BIT:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_3pair_6bit_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_3pair_6bit_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_MINILVDS_2CH_6P_6BIT:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_6pair_6bit_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_6pair_6bit_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_MINILVDS_1CH_3P_8BIT:
                   if(ldHz < 2975*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_8bit_297_5to297_5MHz;
                   }
                   else if((ldHz >= 2975*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_8bit_297_5to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_3pair_8bit_450to595MHz;
                   }
               break;

                case E_PNL_LPLL_MINILVDS_1CH_6P_8BIT:
                   if(ldHz < 2975*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_8bit_297_5to297_5MHz;
                   }
                   else if((ldHz >= 2975*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_8bit_297_5to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_1bk_6pair_8bit_450to595MHz;
                   }
               break;
               case E_PNL_LPLL_MINILVDS_2CH_3P_8BIT:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_3pair_8bit_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_3pair_8bit_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_MINILVDS_2CH_6P_8BIT:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_6pair_8bit_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_2bk_6pair_8bit_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_2P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_2pair_FHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_2pair_FHD_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_2pair_FHD_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_4P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_4pair_FHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_4pair_FHD_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_4pair_FHD_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_6P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_6pair_FHD_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_6pair_FHD_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_8P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_FHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_FHD_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_FHD_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI24_16P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_24_10bit_16pair_UHD_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_24_10bit_16pair_UHD_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_16P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_16pair_UHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_16pair_UHD_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_16pair_UHD_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI24_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_24_10bit_12pair_UHD_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_24_10bit_12pair_UHD_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_12pair_UHD_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_12pair_UHD_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_6P_EPI3G:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_6pair_EPI3G_UHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 3500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_6pair_EPI3G_UHD_300to350MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_6pair_EPI3G_UHD_350to600MHz;
                   }
               break;

               case E_PNL_LPLL_EPI28_8P_EPI3G:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_EPI3G_UHD_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_EPI3G_UHD_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_EPI_28_8bit_8pair_EPI3G_UHD_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_CMPI27_8BIT_8P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_27_8bit_8pair_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_27_8bit_8pair_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_27_8bit_8pair_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_CMPI24_10BIT_8P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_24_10bit_8pair_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 3500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_24_10bit_8pair_300to350MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_24_10bit_8pair_350to600MHz;
                   }
               break;

               case E_PNL_LPLL_CMPI27_8BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_27_8bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_27_8bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_CMPI24_10BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_24_10bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_CMPI_24_10bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USI_T_10BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USI_T_10bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USI_T_10bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USI_T_8BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USI_T_8bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USI_T_8bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_10BIT_6P_D:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_6pair_dual_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_6pair_dual_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_8BIT_6P_D:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_6pair_dual_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_6pair_dual_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_8BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_10BIT_12P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_12pair_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_12pair_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_10BIT_8P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_8pair_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4000*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_8pair_300to400MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_10bit_8pair_400to600MHz;
                   }
               break;

               case E_PNL_LPLL_ISP_8BIT_8P:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_8pair_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_8pair_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_ISP_8bit_8pair_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_8BIT_12X1_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_12x1_4k_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_12x1_4k_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_8BIT_6X1_D_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x1_4k_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x1_4k_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_8BIT_8X1_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_8x1_4k_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_8x1_4k_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_8x1_4k_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_8BIT_6X1_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x1_4k_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x1_4k_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_8BIT_6X2_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x2_4k_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_8bit_6x2_4k_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_10BIT_12X1_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_12x1_4k_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 5000*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_12x1_4k_300to500MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_12x1_4k_500to600MHz;
                   }
               break;

               case E_PNL_LPLL_USII_10BIT_8X1_4K:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_8x1_4k_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 3500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_8x1_4k_300to350MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_USII_10bit_8x1_4k_350to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_10BIT_8LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_8_lane_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_8_lane_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_10BIT_4LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_4_lane_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_4_lane_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_10BIT_2LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_2_lane_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_10bit_2_lane_300to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_8BIT_8LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_8_lane_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_8_lane_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_8_lane_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_8BIT_4LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_4_lane_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_4_lane_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_4_lane_450to600MHz;
                   }
               break;

               case E_PNL_LPLL_VBY1_8BIT_2LANE:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_2_lane_300to300MHz;
                   }
                   else if((ldHz >= 3000*MHZ) && (ldHz < 4500*MHZ))
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_2_lane_300to450MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_Vx1_8bit_2_lane_450to600MHz;
                   }
               break;
               case E_PNL_LPLL_MINILVDS_4CH_6P_8BIT:
                   if(ldHz < 3000*MHZ)
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_4bk_6pair_8bit_300to300MHz;
                   }
                   else
                   {
                       u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MINILVDS_4bk_6pair_8bit_300to600MHz;
                   }
               break;
               default:
                   u8SupportedLPLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
               break;
           }
        }
        break;

        case E_PNL_LPLL_OSD:
        break;
    }
    return u8SupportedLPLLIndex;
}

static void _MHal_PNL_DumpLPLLTable(void *pInstance, MS_U8 LPLLTblIndex, PNL_LPLL_TYPE_SEL lpll_type_sel)
{
    if(lpll_type_sel == E_PNL_LPLL_VIDEO)
    {
        if (LPLLTblIndex == E_PNL_SUPPORTED_LPLL_MAX)
        {
            printf("[%s,%5d] Unspported LPLL Type, skip LPLL setting\n",__FUNCTION__,__LINE__);
            return;
        }

        int indexCounter = 0;

        for(indexCounter = 0 ; indexCounter<LPLL_REG_NUM; indexCounter++)
        {
            if (LPLLSettingTBL[LPLLTblIndex][indexCounter].address == 0xFF) //delay in micro second
            {
                MsOS_DelayTaskUs(LPLLSettingTBL[LPLLTblIndex][indexCounter].value);
                continue; // step forward to next register setting.
            }

            W2BYTEMSK(L_BK_LPLL(LPLLSettingTBL[LPLLTblIndex][indexCounter].address),
                      LPLLSettingTBL[LPLLTblIndex][indexCounter].value,
                      LPLLSettingTBL[LPLLTblIndex][indexCounter].mask);
        }
        PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]LPLLTblIndex=%u\n", __FUNCTION__, __LINE__, LPLLTblIndex);
    }
    else
    {
        PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]OSD combine to video table=%u\n", __FUNCTION__, __LINE__, LPLLTblIndex);
    }
}

void MHal_PNL_Init_LPLL(void *pInstance, PNL_TYPE eLPLL_Type,PNL_MODE eLPLL_Mode,MS_U64 ldHz)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]\n", __FUNCTION__, __LINE__);

    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type,eLPLL_Mode,ldHz, E_PNL_LPLL_VIDEO);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        printf("Not Supported LPLL Type, skip LPLL Init\n");
        return;
    }

    _MHal_PNL_DumpLPLLTable(pInstance, u8SupportedLPLLLIndex, E_PNL_LPLL_VIDEO);


    MHal_MOD_PVDD_Power_Setting(pInstance, pPNLResourcePrivate->sthalPNL._bPVDD_2V5); // Einstein is always use 3.3V PVDD Power.
}

MS_U8 MHal_PNL_Get_Loop_DIV(void *pInstance, MS_U8 u8LPLL_Mode, MS_U8 eLPLL_Type, MS_U64 ldHz)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    MS_U16 u16loop_div = 0;
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
#if defined (__aarch64__)
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]E_PNL_LPLL_VIDEO : eLPLL_Type=%u, u8LPLL_Mode=%u, ldHz=%lu\n", __FUNCTION__, __LINE__, eLPLL_Type, u8LPLL_Mode, ldHz);
#else
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]E_PNL_LPLL_VIDEO : eLPLL_Type=%u, u8LPLL_Mode=%u, ldHz=%llu\n", __FUNCTION__, __LINE__, eLPLL_Type, u8LPLL_Mode, ldHz);
#endif
    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type,u8LPLL_Mode,ldHz,E_PNL_LPLL_VIDEO);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        printf("[%s,%5d] Error LPLL type\n",__FUNCTION__,__LINE__);
        u16loop_div = 0 ;
    }
    else
    {
        u16loop_div = u16LoopDiv[u8SupportedLPLLLIndex];
    }

    // For Maxim HDMI Tx case, loop div follow HDMI SPEC
    if ((E_PNL_TYPE_DAC_P == eLPLL_Type) || (E_PNL_TYPE_DAC_I == eLPLL_Type))
    {
        if (ldHz < 500000000UL)
        {
            u16loop_div = 16;
        }
        else if ((ldHz >= 500000000UL) && (ldHz < 1200000000UL))
        {
            u16loop_div = 8;
        }
        else if ((ldHz >= 1200000000UL) && (ldHz < 2500000000UL))
        {
            u16loop_div = 4;
        }
        else if ((ldHz >= 2500000000UL) && (ldHz < 3000000000UL))
        {
            if ((pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16HTotal < 3000) &&
                (pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16VTotal < 2000))
            {
                u16loop_div = 2; // 1080px12bits, 297MHz
            }
            else
            {
                u16loop_div = 4; // 4K@30, 297MHz
            }
        }
        else if((ldHz >= 3000000000UL) && (ldHz < 6500000000UL))
        {
            u16loop_div = 2;
        }
        else
        {
            u16loop_div = 1;
        }
    }

    PNL_DBG(PNL_DBGLEVEL_INIT, "E_PNL_LPLL_VIDEO : u16loop_div=%u\n", u16loop_div);

    u16loop_div *= 2;
    return u16loop_div;
}

MS_U16 MHal_PNL_Get_LPLL_LoopGain(void *pInstance, MS_U8 eLPLL_Mode, MS_U8 eLPLL_Type, MS_U64 ldHz)
{
    MS_U16 u16loop_gain = 0;
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
#if defined (__aarch64__)
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]E_PNL_LPLL_VIDEO : eLPLL_Type=%u, eLPLL_Mode=%u, ldHz=%lu\n", __FUNCTION__, __LINE__, eLPLL_Type, eLPLL_Mode, ldHz);
#else
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]E_PNL_LPLL_VIDEO : eLPLL_Type=%u, eLPLL_Mode=%u, ldHz=%llu\n", __FUNCTION__, __LINE__, eLPLL_Type, eLPLL_Mode, ldHz);
#endif
    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type,eLPLL_Mode,ldHz,E_PNL_LPLL_VIDEO);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        printf("[%s,%5d] Error LPLL type\n",__FUNCTION__,__LINE__);
        u16loop_gain = 0 ;
    }
    else
    {
        u16loop_gain = u16LoopGain[u8SupportedLPLLLIndex];
    }

    // For Maxim HDMI Tx case, loop gain follow HDMI SPEC
    if ((E_PNL_TYPE_DAC_P == eLPLL_Type) || (E_PNL_TYPE_DAC_I == eLPLL_Type))
    {
        u16loop_gain = 4;
    }

    PNL_DBG(PNL_DBGLEVEL_INIT, "E_PNL_LPLL_VIDEO : u16loop_gain=%u\n", u16loop_gain);
    return u16loop_gain;
}

#define SKIP_TIMING_CHANGE_CAP  TRUE
MS_BOOL Hal_PNL_SkipTimingChange_GetCaps(void *pInstance)
{
    #if (SKIP_TIMING_CHANGE_CAP)
        return TRUE;
    #else
        return FALSE;
    #endif
}

void MHal_PNL_PreSetModeOn(void *pInstance, MS_BOOL bSetMode)
{
    if (bSetMode == TRUE)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, 0, BIT(15));
    }
    else
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, BIT(15), BIT(15));
    }
}

void MHal_PNL_HWLVDSReservedtoLRFlag(void *pInstance, PNL_DrvHW_LVDSResInfo lvdsresinfo)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if (lvdsresinfo.bEnable)
    {
        if (lvdsresinfo.u16channel & BIT(0))  // Channel A
        {
            if (lvdsresinfo.u32pair & BIT(3))  // pair 3
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_5A_L, BIT(15), BIT(15));
            }
            if (lvdsresinfo.u32pair & BIT(4))  // pair 4
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_5A_L, BIT(14), BIT(14));
            }
        }
        if (lvdsresinfo.u16channel & BIT(1))  // Channel B
        {
            if (lvdsresinfo.u32pair & BIT(3))  // pair 3
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_5A_L, BIT(13), BIT(13));
            }
            if (lvdsresinfo.u32pair & BIT(4))  // pair 4
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_5A_L, BIT(12), BIT(12));
            }
        }

        if(  IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type) )
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_30_L, BIT(14), BIT(14)); //reg_sel_ext_bit: sel extend bit, 0: osd_de 1: three_d_flag
        }
    }
    else
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_5A_L, 0x0000, (BIT(15) | BIT(14) | BIT(13) | BIT(12)));

        if(  IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type) )
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_30_L, 0x00, BIT(14)); //reg_sel_ext_bit: sel extend bit, 0: osd_de 1: three_d_flag
        }
    }
}

////////////////////////////////////////////////////////////////////////
// Turn OD function
////////////////////////////////////////////////////////////////////////
void MHal_PNL_OverDriver_Init(void *pInstance, MS_PHY u32OD_MSB_Addr, MS_PHY u32OD_MSB_limit, MS_PHY u32OD_LSB_Addr, MS_PHY u32OD_LSB_limit, MS_U8 u8MIUSel)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    MS_PHY phypg_adde_base = 0;
    MS_PHY phypg_addr_limit = 0;
    MS_PHY phydram_addr_offset = 0;
    MS_U32 u32panel_pixels = 0;
    MS_U16 u16hsize = 0;
    MS_U16 u16vsize = 0;
    MS_U32 u32ipm_offset = 0;
    MS_U16 u16Timeout = 0;

    if((OVERDRIVE_ENABLE_RGBW == TRUE)&& (MHal_PNL_OverDriver_IsEnableRGBW(pInstance)))
    {
        //If LPLL is powerdown, Double buffer can't update register value.
        if(R2BYTEMSK(L_BK_LPLL(0x03),BIT(5))!= 0)
        {
            MHal_PNL_PowerDownLPLL(pInstance, FALSE);
        }
        //Disable Overdriver RGBW to prevent miu hit.
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L,0,BIT(15));
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L, BIT(15))&&(u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTaskUs(10);
            u16Timeout++;
        }
    }

    u16hsize = ALIGN_8(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width);//panel H, need align 8
    u16vsize = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height;//panel V

    if((OVERDRIVE_ENABLE_RGBW == TRUE)&& (MHal_PNL_OverDriver_IsEnableRGBW(pInstance)))
    {
        u16hsize = (u16hsize * OVERDRIVE_RGB_CHANNEL) / OVERDRIVE_RGBW_CHANNEL;// RGBW mode h*3/4
    }

    u32panel_pixels =u16hsize*u16vsize;

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_61_L,u8MIUSel<<8,BIT(8)|BIT(9)); // OD MIU select
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_15_L, (MS_U16)(u32OD_MSB_Addr & 0xFFFF)); // OD MSB request base address
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_16_L, (MS_U16)((u32OD_MSB_Addr >> 16) & 0xFFFF), 0xFFFF); // OD MSB request base address

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_39_L, ((OVERDRIVE_MODE*u32panel_pixels+4096)/32)&0xFFFF); //low byte
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3A_L, (((OVERDRIVE_MODE*u32panel_pixels+4096)/32)>>16), BMASK(7:0)); //hightbyte

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3B_L, (u32panel_pixels)&0xFFFF); //low byte
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3C_L, (u32panel_pixels)>>16, BMASK(7:0)); //high byte

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3B_L, (u32panel_pixels)&0xFFFF); //low byte
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3C_L, (u32panel_pixels)>>16, BMASK(7:0)); //high byte


    if(OVERDRIVE_ENABLE_HSCALING == TRUE)
    {
        if(OVERDRIVE_ENABLE_HSD_4X == TRUE)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L,BIT(7),BIT(7)); // HScaling_4X
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_68_L,0xE60D);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_69_L,0xD);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_34_L,0x1804);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_35_L,0x503C);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_36_L,0x183C);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_37_L,0x4);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_38_L,0x8819);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_39_L,0xFA65);
            u16hsize=u16hsize/4;
        }
        else
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L,0x0,BIT(7)); // HScaling_2X
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_66_L,0x843E);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_67_L,0x3E);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_68_L,0xE60D);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_69_L,0xD);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6A_L,0x95EB);
            SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6B_L,0xEB95);
            u16hsize=u16hsize/2;
        }
    }

    if(OVERDRIVE_ENABLE_VSCALING == TRUE)
    {
        SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0F_L,u16vsize);//V_size
        SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_58_L,u16vsize);//vertical size of a frame
        //vertical scaling 2x
        SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_3A_L,0x8040);
        SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_3B_L,0x40);
        u16vsize=u16vsize/2;
    }
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_11_L,OVERDRIVE_WEIGHT,0xFF); //OD Weight
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_12_L,OVERDRIVE_ACTIVE_THRESHOLD,0xFF); //OD Active threshold


    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_2F_L, u16hsize, BMASK(11:0)); // OD horizontal size of a frame

    if((OVERDRIVE_ENABLE_RGBW == TRUE)&& (MHal_PNL_OverDriver_IsEnableRGBW(pInstance)))
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, ((pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width*OVERDRIVE_RGB_CHANNEL)/OVERDRIVE_RGBW_CHANNEL), BMASK(11:0)); //set reg_htotal*3/4
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width, BMASK(11:0)); //set reg_htotal
    }

    phypg_addr_limit = ((u16hsize*u16vsize)*OVERDRIVE_MODE*OVERDRIVE_EXTRA_MEM_SIZE_FACTOR)/(10*OVERDRIVE_MIU_BUS*OVERDRIVE_PG_ENTRY_PRE_REQ*OVERDRIVE_PG_LENGTH);//254,128,64

    phydram_addr_offset = (phypg_addr_limit+1)*OVERDRIVE_PG_ENTRY_PRE_REQ*OVERDRIVE_PG_LENGTH;

    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%5d]pg_addr_limit=%td dram_addr_offset=%td \n", __FUNCTION__, __LINE__,(ptrdiff_t)phypg_addr_limit,(ptrdiff_t)phydram_addr_offset);

    u32OD_MSB_limit = u32OD_MSB_Addr + phydram_addr_offset + phypg_addr_limit;

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_17_L, (MS_U16)(u32OD_MSB_limit & 0xFFFF)); // OD MSB request address limit
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_18_L, (MS_U16)((u32OD_MSB_limit >> 16) & 0xFFFF), 0xFFFF); // OD MSB request address limit

    phypg_adde_base = u32OD_MSB_Addr + phydram_addr_offset;


    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_47_L, (MS_U16)(phypg_adde_base & 0xFFFF)); // OD PG Base request base address
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_48_L, (MS_U16)((phypg_adde_base >> 16) & 0xFFFF), 0xFFFF); //OD PG Base request base address
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_27_L,phypg_addr_limit,0x1FF); //OD PG Address Limit
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_27_L,BIT(9),BIT(9)); //OD PG Address Limit Enable

    //SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6F_L,0x0F00);//for merlin definition different

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_29_L,0x0F);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_29_L,BIT(8)|BIT(9)|BIT(10)|BIT(11),BIT(8)|BIT(9)|BIT(10)|BIT(11)); //OD one index containts

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_19_L,0x10);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_1A_L,0x3F);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_1A_L,(0x00 << 8),(0xFF << 8));
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_1B_L,0x0A);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_1C_L,0x3F);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_1C_L,(0x00 << 8),(0xFF << 8));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_28_L,0x3F,0xFF);

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_19_L,BIT(13),BIT(13));

#ifdef STELLAR
    if((SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BKF2_73_L,0xF0) >> 4) == 5) //M+
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_55_L,BIT(15),BIT(15));
    }
    else if((SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BKF2_73_L,0xF0) >> 4) == 8) //SiW RGBW
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_55_L,BIT(14),BIT(14));
    }
    else
#endif
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_55_L,FALSE,BIT(14)|BIT(15));
    }

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_32_L,FALSE,BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_32_L,FALSE,BIT(1));

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3A_L,0x2EFF);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_36_L,0x0007);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_39_L,0xFFFF);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3C_L,0xFF,0xFF);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_3B_L,0xFFFF);

    //OD setting
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_20_L,BIT(15),BIT(15));//od request vsync reset wait all request end enable,RD suggest for prevent miu lock problem

    //random_active_threshold
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_08_L,0x0302);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_09_L,0x0905);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0A_L,0x0302);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0B_L,0x0905);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0C_L,0x0302);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0D_L,0x0905);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0E_L,BIT(0),BIT(0));

    //compression setting
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_21_L,0x1218);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_22_L,0x060C);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_23_L,0x1417);
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_24_L,0x0E11);

    //compression decode round enable
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6E_L,BIT(8),BIT(8));

    if((OVERDRIVE_ENABLE_RGBW == TRUE)&& (MHal_PNL_OverDriver_IsEnableRGBW(pInstance)))
    {
#if defined(SUPPORT_SCALER_EXT_REG) && (SUPPORT_SCALER_EXT_REG == 1)
        if(OVERDRIVE_ODW_BITMODE ==OVERDRIVE_ODW_6_BITMODE)
        {
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_01_L,0x0,BIT(0));
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_10_L,0x0,BIT(0));
        }
        else if(OVERDRIVE_ODW_BITMODE ==OVERDRIVE_ODW_8_BITMODE)
        {
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_01_L,0x0,BIT(0));
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_10_L,0x1,BIT(0));
        }
        else if(OVERDRIVE_ODW_BITMODE ==OVERDRIVE_ODW_10_BITMODE)
        {
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_01_L,0x1,BIT(0));
        }
        else
        {
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_01_L,0x0,BIT(0));
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_10_L,0x0,BIT(0));
        }

        MS_U16 u16hsize_align_up = ((u16hsize) + ((OVERDRIVE_MIU_BUS)-1)) & ~((OVERDRIVE_MIU_BUS)-1);// h need align up to 256
        MS_U32 u32odw_limit_size = u16hsize_align_up*u16vsize*OVERDRIVE_ODW_BITMODE/OVERDRIVE_MIU_BUS;
        SC_EXT_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_20_L,(MS_U16)(u32odw_limit_size & 0xFFFF));

        SC_EXT_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_1D_L,(MS_U16)(u32OD_MSB_limit+1));//ODw_base_addr LSB
        SC_EXT_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_EXT_BK27_1E_L,(MS_U16)(((u32OD_MSB_limit+1) >> 16) & 0xFFFF));//ODw_base_addr MSB

#endif
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L,0xE4,0x0FFF);//ch_loc_w,ch_loc_b,ch_loc_g,ch_loc_r
    }

}

void MHal_PNL_OverDriver_Enable(void *pInstance, MS_BOOL bEnable)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_U16 u16Taget_BR =0; //Target bit rate
    // When OD enable, disable OD/RGBW SRAM PowerDown.
#ifdef CONFIG_MSTAR_SRAMPD
    if(pPNLInstancePrivate->u32DeviceID == 0)
    {
        if(bEnable)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, 0, BIT(0));   //OD SRAM PD Enable  : SC_SPD_BK3F_10[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, BIT(8), BIT(8));        //OD Clock gate : ~SC_SPD_BK3F_50[8]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(15)); //OD Bypass Enable :SC_OD_BK16_6F[15]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, 0, BIT(1));   // RGBW SRAM PD Enable : SC_SPD_BK3F_10[1]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, BIT(9), BIT(9));        // RGBW Clock Gate : ~SC_SPD_BK3F_50[9]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(12)); // RGBW bypass enable :SC_OD_BK16_6F[12]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, BIT(13), BIT(13));       // RGBW bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_17_L, 0, BIT(0));   //M+ SRAM PD Enable  : SC_SPD_BK3F_17[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, BIT(10), BIT(10));        //M+ Clock gate : ~SC_SPD_BK3F_50[10]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L,0 , BIT(12)); // M+ bypass enable :SC_OD_BK16_6F[12]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(13));       // M+ bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, 0, BIT(4));   //Demura SRAM PD Enable  : SC_SPD_BK3F_0B[4]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, 0, BIT(14));   //Demura Clock Gate  : SC_BK77_26[14]
            W2BYTEMSK(REG_RVD_45_L, 0, BIT(0));                                              //Demura Clock gate : BK100A_CLKGEN2_45[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, 0, BIT(15));   //Demura Clock Gate  : SC_BK77_26[15]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, 0, BIT(5));   //DGA_GAMMA SRAM PD Enable  : SC_SPD_BK3F_0B[5]
        }
        else
        {
            #if 0 // Skip power down for bring up
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, BIT(0), BIT(0));   //OD SRAM PD Enable  : SC_SPD_BK3F_10[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(8));        //OD Clock gate : ~SC_SPD_BK3F_50[8]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, BIT(15), BIT(15)); //OD Bypass Enable :SC_OD_BK16_6F[15]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, BIT(1), BIT(1));   // RGBW SRAM PD Enable : SC_SPD_BK3F_10[1]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(9));        // RGBW Clock Gate : ~SC_SPD_BK3F_50[9]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, BIT(12), BIT(12)); // RGBW bypass enable :SC_OD_BK16_6F[12]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(13));       // RGBW bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_17_L, BIT(0), BIT(0));   //M+ SRAM PD Enable  : SC_SPD_BK3F_17[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(10));        //M+ Clock gate : ~SC_SPD_BK3F_50[10]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L,0 , BIT(12)); // M+ bypass enable :SC_OD_BK16_6F[12]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(13));       // M+ bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, BIT(4), BIT(4));   //Demura SRAM PD Enable  : SC_SPD_BK3F_0B[4]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, BIT(14), BIT(14));   //Demura Clock Gate  : SC_BK77_26[14]
            W2BYTEMSK(REG_RVD_45_L, BIT(0), BIT(0));                                              //Demura Clock gate : BK100A_CLKGEN2_45[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, BIT(15), BIT(15));   //Demura Clock Gate  : SC_BK77_26[15]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, BIT(5), BIT(5));   //DGA_GAMMA SRAM PD Enable  : SC_SPD_BK3F_0B[5]
            #endif
        }
    }
#endif

    // OD mode
    // OD used user weight to output blending directly
    // OD Enable
    if (bEnable)
    {
        u16Taget_BR = (OVERDRIVE_MODE*64)+3;//Formula = 64*bits_per_pixel + 3
        //Target bit rate of compression engine output for 4 bit compression
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_2D_L, u16Taget_BR,0x0F);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, 0xB,0x0F); //compression mode

        if((OVERDRIVE_ENABLE_RGBW == TRUE)&& (MHal_PNL_OverDriver_IsEnableRGBW(pInstance)))
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L,BIT(15),BIT(15));//RGBW input Enable
        }

        if(OVERDRIVE_ENABLE_HSCALING == TRUE)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L,BIT(0),BIT(0));//Enable HScaling

            //Enable HScaling 0x1316[0] with double buffer. sometimes can't update register value. Need retry mechanism.
            if((SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L, BIT(0)) != BIT(0)))
            {
                MsOS_DelayTask(16);
                if((SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L, BIT(0)) != BIT(0)))
                {
                    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]Enable HScaling fail\n", __FUNCTION__, __LINE__);
                }
            }
        }

        if(OVERDRIVE_ENABLE_VSCALING == TRUE)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L,BIT(1),BIT(1));//Enable VScaling
        }
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, 0x0,  BIT(0));//RGB 444mode
    }


}

void MHal_PNL_OverDriver_TBL(void *pInstance, MS_U8 u8ODTbl[1056])
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 i;
    MS_U8 u8target;
    MS_BOOL bEnable;
    MS_U16 u16Timeout = 0;

    bEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, 0x00, BIT(0)); // OD enable
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, 0x0E, 0x0E); // OD table SRAM enable, RGB channel

    u8target= u8ODTbl[9];
    for (i=0; i<272; i++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_03_L, (i == 9)?u8target:(u8target ^ u8ODTbl[i]), 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, (i|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, BIT(15)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTask(1);
            u16Timeout++;
        }
        u16Timeout = 0;
    }

    u8target= u8ODTbl[(272+19)];
    for (i=0; i<272; i++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_06_L, (i == 19)?u8target:(u8target ^ u8ODTbl[(272+i)]), 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, (i|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, BIT(15)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTask(1);
            u16Timeout++;
        }
        u16Timeout = 0;
    }

    u8target= u8ODTbl[(272*2+29)];
    for (i=0; i<256; i++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_09_L, (i == 29)?u8target:(u8target ^ u8ODTbl[(272*2+i)]), 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, (i|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, BIT(15)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTask(1);
            u16Timeout++;
        }
        u16Timeout = 0;
    }

    u8target= u8ODTbl[(272*2+256+39)];
    for (i=0; i<256; i++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0C_L, (i == 39)?u8target:(u8target ^ u8ODTbl[(272*2+256+i)]), 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, (i|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, BIT(15)) && (u16Timeout < MAX_DELAY_TIME))
        {
            MsOS_DelayTask(1);
            u16Timeout++;
        }
        u16Timeout = 0;
    }

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, 0x00, 0x0E); // OD table SRAM enable, RGB channel
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, bEnable, BIT(0)); // OD enable
}

#ifdef ROLLS_ROYCE
void MHal_PNL_OverDriver_Setting_TBL(void *pInstance, MS_U8* pu8ODTbl,MS_U32 u32ODTabSize,MS_U16 u16SepatateMode)
{
    if(!pu8ODTbl)
    {
        return;
    }
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 u16CodeTableIndex = 0;
    MS_U16 u16CodeCurrentTableIndex = 0;
    MS_U16 u16TargetIndex = 0;
    MS_U8 u8Target = 0;
    MS_BOOL bEnable = FALSE;
    MS_U8 u8OD_SRAM1[OVERDRIVER_SRAM1_SIZE] = {0};
    MS_U8 u8OD_SRAM2[OVERDRIVER_SRAM2_SIZE] = {0};
    MS_U8 u8OD_SRAM3[OVERDRIVER_SRAM3_SIZE] = {0};
    MS_U8 u8OD_SRAM4[OVERDRIVER_SRAM4_SIZE] = {0};

    //Decrypt OD table.
    u16TargetIndex = OD_DECRYPTION_IDX_1;
    u8Target = pu8ODTbl[u16TargetIndex];
    for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM1_SIZE; u16CodeTableIndex++)
    {
        if(u16CodeTableIndex == u16TargetIndex)
        {
            u8OD_SRAM1[u16CodeTableIndex] = (MS_U8)u8Target;
        }
        else
        {
            u8OD_SRAM1[u16CodeTableIndex] = (MS_U8)(pu8ODTbl[u16CodeTableIndex]^u8Target);
        }
    }

    u16CodeCurrentTableIndex += OVERDRIVER_SRAM1_SIZE;
    u16TargetIndex += OD_DECRYPTION_IDX_2;
    u8Target = pu8ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
    for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM2_SIZE; u16CodeTableIndex++)
    {
        if(u16CodeTableIndex == u16TargetIndex)
        {
            u8OD_SRAM2[u16CodeTableIndex] = (MS_U8)u8Target;
        }
        else
        {
            u8OD_SRAM2[u16CodeTableIndex] = (MS_U8)(pu8ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u8Target);
        }
    }

    u16CodeCurrentTableIndex += OVERDRIVER_SRAM2_SIZE;
    u16TargetIndex += OD_DECRYPTION_IDX_2;
    u8Target = pu8ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
    for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM3_SIZE; u16CodeTableIndex++)
    {
        if(u16CodeTableIndex == u16TargetIndex)
        {
            u8OD_SRAM3[u16CodeTableIndex] = (MS_U8)u8Target;
        }
        else
        {
            u8OD_SRAM3[u16CodeTableIndex] = (MS_U8)(pu8ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u8Target);
        }
    }

    u16CodeCurrentTableIndex += OVERDRIVER_SRAM3_SIZE;
    u16TargetIndex += OD_DECRYPTION_IDX_2;
    u8Target = pu8ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
    for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM4_SIZE; u16CodeTableIndex++)
    {
        if(u16CodeTableIndex == u16TargetIndex)
        {
            u8OD_SRAM4[u16CodeTableIndex] = (MS_U8)u8Target;
        }
        else
        {
            u8OD_SRAM4[u16CodeTableIndex] = (MS_U8)(pu8ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u8Target);
        }
    }

    bEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, 0x00, BIT(0)); // OD enable

#if  (ENABLE_OD_AUTODOWNLOAD == TRUE)

    //Always use new mode, support write independent R/G/B/W tables
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK29_43_L, BIT(13), BIT(13));
    if(g_pstOD_ADL_CMD == NULL)
    {
        g_pstOD_ADL_CMD = (MS_U8*)vmalloc(sizeof(ST_PNL_OD_NEW_MODE_DRAM_FORMAT)*XC_AUTODOWNLOAD_OVERDRIVER_DEPTH);
        if(g_pstOD_ADL_CMD == NULL)
        {
            printf("[%s][%d] OD Autodownload allocate mem fail\n",__FUNCTION__,__LINE__);
            return;
        }
    }
    memset(g_pstOD_ADL_CMD, 0, sizeof(ST_PNL_OD_NEW_MODE_DRAM_FORMAT)*XC_AUTODOWNLOAD_OVERDRIVER_DEPTH);

    for(u16CodeTableIndex = 0; u16CodeTableIndex < XC_AUTODOWNLOAD_OVERDRIVER_DEPTH; u16CodeTableIndex++)
    {
        if(u16CodeTableIndex < OVERDRIVER_SRAM1_SIZE)
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B1 = u8OD_SRAM1[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G1 = u8OD_SRAM1[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R1 = u8OD_SRAM1[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W1 = u8OD_SRAM1[u16CodeTableIndex];

            g_pstOD_ADL_CMD[u16CodeTableIndex].u32Table_WriteAddr = u16CodeTableIndex;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_B_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_B_ENABLE)>>1;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_G_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_G_ENABLE)>>2;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_R_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_R_ENABLE)>>3;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_W_WriteEn = u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_W_ENABLE;
        }
        else
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B1 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G1 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R1 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W1 = 0;
        }

        if(u16CodeTableIndex < OVERDRIVER_SRAM2_SIZE)
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B2 =  u8OD_SRAM2[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G2 =  u8OD_SRAM2[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R2 =  u8OD_SRAM2[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W2 =  u8OD_SRAM2[u16CodeTableIndex];
        }
        else
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B2 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G2 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R2 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W2 = 0;
        }

        if(u16CodeTableIndex < OVERDRIVER_SRAM3_SIZE)
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B3 =  u8OD_SRAM3[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G3 =  u8OD_SRAM3[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R3 =  u8OD_SRAM3[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W3 =  u8OD_SRAM3[u16CodeTableIndex];
        }
        else
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B3 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G3 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R3 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W3 = 0;
        }

        if(u16CodeTableIndex < OVERDRIVER_SRAM4_SIZE)
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B4 =  u8OD_SRAM4[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G4 =  u8OD_SRAM4[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R4 =  u8OD_SRAM4[u16CodeTableIndex];
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W4 =  u8OD_SRAM4[u16CodeTableIndex];
        }
        else
        {
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B4 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G4 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R4 = 0;
            g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W4 = 0;
        }
    }

    XC_AUTODOWNLOAD_FORMAT_INFO stAdlFormatInfo;
    memset(&stAdlFormatInfo, 0, sizeof(XC_AUTODOWNLOAD_FORMAT_INFO));
    stAdlFormatInfo.u32FormatInfo_Version = AUTODOWNLOAD_FORMAT_INFO_VERSION;
    stAdlFormatInfo.u16FormatInfo_Length=sizeof(XC_AUTODOWNLOAD_FORMAT_INFO);

    XC_AUTODOWNLOAD_DATA_INFO stAdlDataInfo;
    memset(&stAdlDataInfo, 0, sizeof(XC_AUTODOWNLOAD_DATA_INFO));
    stAdlDataInfo.u32DataInfo_Version = AUTODOWNLOAD_DATA_INFO_VERSION;
    stAdlDataInfo.u16DataInfo_Length =  sizeof(XC_AUTODOWNLOAD_DATA_INFO);
    stAdlDataInfo.pu8Data = g_pstOD_ADL_CMD;
    stAdlDataInfo.enClient = E_XC_AUTODOWNLOAD_CLIENT_OD;
    stAdlDataInfo.pParam = &stAdlFormatInfo;
    stAdlDataInfo.u32Size =  XC_AUTODOWNLOAD_OVERDRIVER_DEPTH*BYTE_PER_WORD;

    if(MApi_XC_AutoDownload_Write(&stAdlDataInfo) != E_APIXC_RET_OK )
    {
        printk("\033[1;31m OD,Write ADL Fail!!!!!!!!!!\033[m\n");
    }

    if(MApi_XC_AutoDownload_Fire(E_XC_AUTODOWNLOAD_CLIENT_OD)!=E_APIXC_RET_OK)
    {
        printk("\033[1;31m OD, Fire ADL Fail!!!!!!!!!!\033[m\n");
    }

    if(g_pstOD_ADL_CMD != NULL)
    {
        vfree(g_pstOD_ADL_CMD);
        g_pstOD_ADL_CMD = NULL;
    }
#else
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, u16SepatateMode, 0x0F); // OD table SRAM enable, RGBW channel

    for(u16CodeTableIndex=0;u16CodeTableIndex<OVERDRIVER_SRAM1_SIZE;u16CodeTableIndex++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_03_L, u8OD_SRAM1[u16CodeTableIndex], 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, (u16CodeTableIndex|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, BIT(15)));
    }

    for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM2_SIZE; u16CodeTableIndex++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_06_L, u8OD_SRAM2[u16CodeTableIndex], 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, (u16CodeTableIndex|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, BIT(15)));
    }

    for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM3_SIZE; u16CodeTableIndex++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_09_L, u8OD_SRAM3[u16CodeTableIndex], 0x00FF);

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, (u16CodeTableIndex|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, BIT(15)));
    }

    for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM4_SIZE; u16CodeTableIndex++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0C_L, u8OD_SRAM4[u16CodeTableIndex], 0x00FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, (u16CodeTableIndex|0x8000), 0x81FF);
        while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, BIT(15)));
    }

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, 0x00, 0x0F); // OD table SRAM enable, RGBW channel
#endif

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, bEnable, BIT(0)); // OD enable
}

#else
void MHal_PNL_OverDriver_Setting_TBL(void *pInstance, MS_U16* pu16ODTbl,MS_U32 u32ODTabSize,MS_U16 u16SepatateMode)
{
    if(!pu16ODTbl)
    {
        return;
    }
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U16 u16CodeTableIndex = 0;
    MS_U16 u16CodeCurrentTableIndex = 0;
    MS_U16 u16TargetIndex = 0;
    MS_U16 u16Target = 0;
    MS_BOOL bEnable = FALSE;
    MS_U8* pu8OD_SRAM1 = NULL;
    MS_U8* pu8OD_SRAM2 = NULL;
    MS_U8* pu8OD_SRAM3 = NULL;
    MS_U8* pu8OD_SRAM4 = NULL;
    MS_BOOL bRet = TRUE;

    PNL_MALLOC_MEM(pu8OD_SRAM1, OVERDRIVER_SRAM1_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM2, OVERDRIVER_SRAM2_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM3, OVERDRIVER_SRAM3_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM4, OVERDRIVER_SRAM4_SIZE, bRet);

    if(pu8OD_SRAM1 != NULL && pu8OD_SRAM2 != NULL && pu8OD_SRAM3 != NULL && pu8OD_SRAM4 != NULL)
    {
        //Decrypt OD table.
        u16TargetIndex = OD_DECRYPTION_IDX_1;
        u16Target = pu16ODTbl[u16TargetIndex];
        for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM1_SIZE; u16CodeTableIndex++)
        {
            if(u16CodeTableIndex == u16TargetIndex)
            {
                pu8OD_SRAM1[u16CodeTableIndex] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM1[u16CodeTableIndex] = (MS_U8)(pu16ODTbl[u16CodeTableIndex]^u16Target);
            }
        }

        u16CodeCurrentTableIndex += OVERDRIVER_SRAM1_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = pu16ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM2_SIZE; u16CodeTableIndex++)
        {
            if(u16CodeTableIndex == u16TargetIndex)
            {
                pu8OD_SRAM2[u16CodeTableIndex] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM2[u16CodeTableIndex] = (MS_U8)(pu16ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u16Target);
            }
        }

        u16CodeCurrentTableIndex += OVERDRIVER_SRAM2_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = pu16ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM3_SIZE; u16CodeTableIndex++)
        {
            if(u16CodeTableIndex == u16TargetIndex)
            {
                pu8OD_SRAM3[u16CodeTableIndex] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM3[u16CodeTableIndex] = (MS_U8)(pu16ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u16Target);
            }
        }

        u16CodeCurrentTableIndex += OVERDRIVER_SRAM3_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = pu16ODTbl[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16CodeTableIndex = 0; u16CodeTableIndex < OVERDRIVER_SRAM4_SIZE; u16CodeTableIndex++)
        {
            if(u16CodeTableIndex == u16TargetIndex)
            {
                pu8OD_SRAM4[u16CodeTableIndex] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM4[u16CodeTableIndex] = (MS_U8)(pu16ODTbl[(u16CodeCurrentTableIndex+u16CodeTableIndex)]^u16Target);
            }
        }

        bEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, BIT(0));
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, 0x00, BIT(0)); // OD enable

        if(MHal_PNL_OverDriver_IsEnableRGBW(pInstance))
        {
            if(R2BYTEMSK(L_BK_LPLL(0x03),BIT(5))!= 0)
            {
                MHal_PNL_PowerDownLPLL(pInstance, FALSE);
            }
            //Disable Overdriver RGBW to prevent write od table fail
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L,0x00,BIT(15));
        }

#if  (ENABLE_OD_AUTODOWNLOAD == TRUE)

        //Always use new mode, support write independent R/G/B/W tables
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK29_43_L, BIT(13), BIT(13));
        if(g_pstOD_ADL_CMD == NULL)
        {
            g_pstOD_ADL_CMD = (MS_U8*)vmalloc(sizeof(ST_PNL_OD_NEW_MODE_DRAM_FORMAT)*XC_AUTODOWNLOAD_OVERDRIVER_DEPTH);
            if(g_pstOD_ADL_CMD == NULL)
            {
                printf("[%s][%d] OD Autodownload allocate mem fail\n",__FUNCTION__,__LINE__);
                return;
            }
        }
        memset(g_pstOD_ADL_CMD, 0, sizeof(ST_PNL_OD_NEW_MODE_DRAM_FORMAT)*XC_AUTODOWNLOAD_OVERDRIVER_DEPTH);

        for(u16CodeTableIndex = 0; u16CodeTableIndex < XC_AUTODOWNLOAD_OVERDRIVER_DEPTH; u16CodeTableIndex++)
        {
            if(u16CodeTableIndex < OVERDRIVER_SRAM1_SIZE)
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B1 = pu8OD_SRAM1[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G1 = pu8OD_SRAM1[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R1 = pu8OD_SRAM1[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W1 = pu8OD_SRAM1[u16CodeTableIndex];

                g_pstOD_ADL_CMD[u16CodeTableIndex].u32Table_WriteAddr = u16CodeTableIndex;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_B_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_B_ENABLE)>>1;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_G_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_G_ENABLE)>>2;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_R_WriteEn = (u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_R_ENABLE)>>3;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32OD_W_WriteEn = u16SepatateMode & E_PNL_OVERDRIVER_SEPARATE_W_ENABLE;
            }
            else
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B1 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G1 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R1 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W1 = 0;
            }

            if(u16CodeTableIndex < OVERDRIVER_SRAM2_SIZE)
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B2 =  pu8OD_SRAM2[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G2 =  pu8OD_SRAM2[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R2 =  pu8OD_SRAM2[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W2 =  pu8OD_SRAM2[u16CodeTableIndex];
            }
            else
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B2 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G2 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R2 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W2 = 0;
            }

            if(u16CodeTableIndex < OVERDRIVER_SRAM3_SIZE)
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B3 =  pu8OD_SRAM3[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G3 =  pu8OD_SRAM3[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R3 =  pu8OD_SRAM3[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W3 =  pu8OD_SRAM3[u16CodeTableIndex];
            }
            else
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B3 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G3 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R3 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W3 = 0;
            }

            if(u16CodeTableIndex < OVERDRIVER_SRAM4_SIZE)
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B4 =  pu8OD_SRAM4[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G4 =  pu8OD_SRAM4[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R4 =  pu8OD_SRAM4[u16CodeTableIndex];
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W4 =  pu8OD_SRAM4[u16CodeTableIndex];
            }
            else
            {
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_B4 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_G4 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_R4 = 0;
                g_pstOD_ADL_CMD[u16CodeTableIndex].u32SRAM_W4 = 0;
            }
        }

        XC_AUTODOWNLOAD_FORMAT_INFO stAdlFormatInfo;
        memset(&stAdlFormatInfo, 0, sizeof(XC_AUTODOWNLOAD_FORMAT_INFO));
        stAdlFormatInfo.u32FormatInfo_Version = AUTODOWNLOAD_FORMAT_INFO_VERSION;
        stAdlFormatInfo.u16FormatInfo_Length=sizeof(XC_AUTODOWNLOAD_FORMAT_INFO);

        XC_AUTODOWNLOAD_DATA_INFO stAdlDataInfo;
        memset(&stAdlDataInfo, 0, sizeof(XC_AUTODOWNLOAD_DATA_INFO));
        stAdlDataInfo.u32DataInfo_Version = AUTODOWNLOAD_DATA_INFO_VERSION;
        stAdlDataInfo.u16DataInfo_Length =  sizeof(XC_AUTODOWNLOAD_DATA_INFO);
        stAdlDataInfo.pu8Data = g_pstOD_ADL_CMD;
        stAdlDataInfo.enClient = E_XC_AUTODOWNLOAD_CLIENT_OD;
        stAdlDataInfo.pParam = &stAdlFormatInfo;
        stAdlDataInfo.u32Size =  XC_AUTODOWNLOAD_OVERDRIVER_DEPTH*BYTE_PER_WORD;

        if(MApi_XC_AutoDownload_Write(&stAdlDataInfo) != E_APIXC_RET_OK )
        {
            printk("\033[1;31m OD,Write ADL Fail!!!!!!!!!!\033[m\n");
        }

        if(MApi_XC_AutoDownload_Fire(E_XC_AUTODOWNLOAD_CLIENT_OD)!=E_APIXC_RET_OK)
        {
            printk("\033[1;31m OD, Fire ADL Fail!!!!!!!!!!\033[m\n");
        }

        if(g_pstOD_ADL_CMD != NULL)
        {
            vfree(g_pstOD_ADL_CMD);
            g_pstOD_ADL_CMD = NULL;
        }
#else
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, u16SepatateMode, 0x0F); // OD table SRAM enable, RGBW channel

        for(u16CodeTableIndex=0;u16CodeTableIndex<OVERDRIVER_SRAM1_SIZE;u16CodeTableIndex++)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_03_L, pu8OD_SRAM1[u16CodeTableIndex], 0x00FF);
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, (u16CodeTableIndex|0x8000), 0x81FF);
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, BIT(15)));
        }

        for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM2_SIZE; u16CodeTableIndex++)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_06_L, pu8OD_SRAM2[u16CodeTableIndex], 0x00FF);
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, (u16CodeTableIndex|0x8000), 0x81FF);
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, BIT(15)));
        }

        for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM3_SIZE; u16CodeTableIndex++)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_09_L, pu8OD_SRAM3[u16CodeTableIndex], 0x00FF);
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, (u16CodeTableIndex|0x8000), 0x81FF);
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, BIT(15)));
        }

        for (u16CodeTableIndex=0; u16CodeTableIndex<OVERDRIVER_SRAM4_SIZE; u16CodeTableIndex++)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0C_L, pu8OD_SRAM4[u16CodeTableIndex], 0x00FF);
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, (u16CodeTableIndex|0x8000), 0x81FF);
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, BIT(15)));
        }

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, 0x00, 0x0F); // OD table SRAM enable, RGBW channel
#endif


        if(MHal_PNL_OverDriver_IsEnableRGBW(pInstance))
        {
            if(R2BYTEMSK(L_BK_LPLL(0x03),BIT(5))!= 0)
            {
                MHal_PNL_PowerDownLPLL(pInstance, FALSE);
            }
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_77_L,bEnable,BIT(15));
        }
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_10_L, bEnable, BIT(0)); // OD enable
    }

    PNL_FREE_MEM(pu8OD_SRAM1, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM2, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM3, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM4, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
}

#endif

MS_U16 _MHal_PNL_MOD_Swing_Refactor_AfterCAL(void *pInstance, MS_U16 u16Swing_Level)
{
    MS_U8 u8ibcal = 0x00;
    MS_U16 u16AfterCal_value = 0;
    MS_U16 u16Cus_value = 0;

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    // =========
    // GCR_CAL_LEVEL[1:0] : REG_MOD_A_BK00_70_L =>
    // 2'b00 250mV ' GCR_ICON_CHx[5:0]=2'h15 (decimal 21)
    // 2'b01 350mV ' GCR_ICON_CHx[5:0]=2'h1F (decimal 31)
    // 2'b10 300mV ' GCR_ICON_CHx[5:0]=2'h1A (decimal 26)
    // 2'b11 200mV ' GCR_ICON_CHx[5:0]=2'h10 (decimal 16)
    // =========
    switch(pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET)
    {
        default:
        case 0:
            u8ibcal = 0x15;
        break;
        case 1:
            u8ibcal = 0x1F;
        break;
        case 2:
            u8ibcal = 0x1A;
        break;
        case 3:
            u8ibcal = 0x10;
        break;
    }
    u16Cus_value = (u16Swing_Level) * (pPNLResourcePrivate->sthalPNL._u8MOD_CALI_VALUE + 4)/(u8ibcal + 4);

    u16AfterCal_value = (u16Cus_value-40)/10+2;

    HAL_MOD_CAL_DBG(printf("\r\n--Swing value after refactor = %d\n", u16AfterCal_value));

    return u16AfterCal_value;
}

MS_U16 MHal_PNL_MODSwingRegToRealLevelValue(void *pInstance, MS_U16 u16SwingRegValue)
{
    MS_U8 u8ibcal = 0x00;
    MS_U16 u16SwingRealLevelValue = 0;
    MS_U16 u16CusValue = 0;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    // =========
    // GCR_CAL_LEVEL[1:0] : REG_MOD_A_BK00_70_L =>
    // 2'b00 250mV ' GCR_ICON_CHx[5:0]=2'h15 (decimal 21)
    // 2'b01 350mV ' GCR_ICON_CHx[5:0]=2'h1F (decimal 31)
    // 2'b10 300mV ' GCR_ICON_CHx[5:0]=2'h1A (decimal 26)
    // 2'b11 200mV ' GCR_ICON_CHx[5:0]=2'h10 (decimal 16)
    // =========
    switch(pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET)
    {
        default:
        case 0:
            u8ibcal = 0x15;
        break;
        case 1:
            u8ibcal = 0x1F;
        break;
        case 2:
            u8ibcal = 0x1A;
        break;
        case 3:
            u8ibcal = 0x10;
        break;
    }

    u16CusValue =  ((u16SwingRegValue-2)*10)+40;
    u16SwingRealLevelValue=(u16CusValue*(u8ibcal + 4))/(pPNLResourcePrivate->sthalPNL._u8MOD_CALI_VALUE + 4);

    HAL_MOD_CAL_DBG(printf("\r\n--Swing Real Level Value = %d\n", u16SwingRealLevelValue));

    return u16SwingRealLevelValue;
}

MS_U16 MHal_PNL_GetCurrentSwingLevel(void *pInstance)
{

    MS_U16 u16CurrentSwingLevelValue = 0;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type >= E_PNL_LPLL_VBY1_10BIT_4LANE)&&
       (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type <= E_PNL_LPLL_VBY1_8BIT_8LANE))
    {
        //Vby1 Case
        u16CurrentSwingLevelValue = (MOD_A_R2BYTE(REG_MOD_A_BK00_08_L)>>12);

    }
    else if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)||
      (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_HS_LVDS))
    {
         //LVDS Case
        u16CurrentSwingLevelValue = MHal_PNL_MODSwingRegToRealLevelValue(pInstance,MOD_A_R2BYTE(REG_MOD_A_BK00_08_L)>>8);

    }
    else
    {
        u16CurrentSwingLevelValue = 0;
    }
    return u16CurrentSwingLevelValue;
}

MS_BOOL MHal_PNL_MOD_Control_Out_Swing(void *pInstance, MS_U16 u16Swing_Level)
{
    MS_BOOL bStatus = FALSE;

    MS_U16 u16ValidSwing = 0;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)||
      (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_HS_LVDS))
    {
        if(u16Swing_Level>600)
            u16Swing_Level=600;
        if(u16Swing_Level<40)
            u16Swing_Level=40;

        u16ValidSwing = _MHal_PNL_MOD_Swing_Refactor_AfterCAL(pInstance, u16Swing_Level);
    }
    else
    {
        u16ValidSwing = u16Swing_Level;
    }

    // Disable HW calibration keep mode first, to make SW icon value can write into register.
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, 0, BIT(15));

    //RCON and Bias-con are trim in IC, can't controlled by customer.

    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)||
       (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_HS_LVDS))
    {
        u16ValidSwing &=0xFF;
        // LVDS fill ICON
        // ch0+ch1
        MOD_A_W2BYTE(REG_MOD_A_BK00_08_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch2+ch3
        MOD_A_W2BYTE(REG_MOD_A_BK00_09_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch4+ch5
        MOD_A_W2BYTE(REG_MOD_A_BK00_0A_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch6+ch7
        MOD_A_W2BYTE(REG_MOD_A_BK00_0B_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch8+ch9
        MOD_A_W2BYTE(REG_MOD_A_BK00_0C_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch10+ch11
        MOD_A_W2BYTE(REG_MOD_A_BK00_0D_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch12+ch13
        MOD_A_W2BYTE(REG_MOD_A_BK00_0E_L, (u16ValidSwing << 8 | u16ValidSwing));
        // ch14+ch15
        MOD_A_W2BYTE(REG_MOD_A_BK00_0F_L, (u16ValidSwing << 8 | u16ValidSwing));
    }
    bStatus = TRUE;

    MHal_PNL_StoreSwingCfg(pInstance);

    return bStatus;
}

////////////////////////////////////////////////////////////////////////
// Turn Pre-Emphasis Current function
////////////////////////////////////////////////////////////////////////
MS_BOOL MHal_PNL_MOD_Control_Out_PE_Current (void *pInstance, MS_U16 u16Current_Level)
{
    MS_BOOL bStatus = FALSE;
    MS_U16 u16ValidCurrent = u16Current_Level & 0x1F;

    MOD_A_W2BYTE(REG_MOD_A_BK00_18_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_19_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_1A_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_1B_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_1C_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_1D_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    MOD_A_W2BYTE(REG_MOD_A_BK00_1E_L, ((u16ValidCurrent) |(u16ValidCurrent << 8)));

    if (MOD_A_R2BYTE(REG_MOD_A_BK00_30_L)!=0x3FFF)
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_30_L, 0x3fff); // PE ch0~ch13 enable.
    }

    //Store PE config
    MHal_PNL_StorePECfg(pInstance);

    bStatus = TRUE;

    return bStatus;
}

void MHal_PNL_MOD_PECurrent_Setting(void *pInstance, MS_U16 u16Current_Level, MS_U16 u16Channel_Select)
{
    MS_U16 u16ValidCurrent = u16Current_Level & 0x1F;
    MS_U16 u16Ch00_01_mask = 0,u16Ch02_03_mask = 0,u16Ch04_05_mask = 0,u16Ch06_07_mask = 0,u16Ch08_09_mask = 0;
    MS_U16 u16Ch10_11_mask = 0,u16Ch12_13_mask = 0,u16Ch14_15_mask = 0;

    u16Ch00_01_mask = (((u16Channel_Select & BIT(0))?  0x001F:0x00)|((u16Channel_Select & BIT(1))?  0x1F00:0x00));
    u16Ch02_03_mask = (((u16Channel_Select & BIT(2))?  0x001F:0x00)|((u16Channel_Select & BIT(3))?  0x1F00:0x00));
    u16Ch04_05_mask = (((u16Channel_Select & BIT(4))?  0x001F:0x00)|((u16Channel_Select & BIT(5))?  0x1F00:0x00));
    u16Ch06_07_mask = (((u16Channel_Select & BIT(6))?  0x001F:0x00)|((u16Channel_Select & BIT(7))?  0x1F00:0x00));
    u16Ch08_09_mask = (((u16Channel_Select & BIT(8))?  0x001F:0x00)|((u16Channel_Select & BIT(9))?  0x1F00:0x00));
    u16Ch10_11_mask = (((u16Channel_Select & BIT(10))? 0x001F:0x00)|((u16Channel_Select & BIT(11))? 0x1F00:0x00));
    u16Ch12_13_mask = (((u16Channel_Select & BIT(12))? 0x001F:0x00)|((u16Channel_Select & BIT(13))? 0x1F00:0x00));
    u16Ch14_15_mask = (((u16Channel_Select & BIT(14))? 0x001F:0x00)|((u16Channel_Select & BIT(15))? 0x1F00:0x00));

    if(u16Ch00_01_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_18_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch00_01_mask);
    }

    if(u16Ch02_03_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_19_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch02_03_mask);
    }

    if(u16Ch04_05_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1A_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch04_05_mask);
    }

    if(u16Ch06_07_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1B_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch06_07_mask);
    }

    if(u16Ch08_09_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1C_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch08_09_mask);
    }

    if(u16Ch10_11_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1D_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch10_11_mask);
    }

    if(u16Ch12_13_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1E_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch12_13_mask);
    }

    if(u16Ch14_15_mask)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_1F_L,
            ((u16ValidCurrent)|(u16ValidCurrent << 8)), u16Ch14_15_mask);
    }

    if (MOD_A_R2BYTE(REG_MOD_A_BK00_30_L)!=0x3FFF)
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_30_L, 0x3fff); // PE ch0~ch13 enable.
    }

    //Store PE config
    MHal_PNL_StorePECfg(pInstance);
}

////////////////////////////////////////////////////////////////////////
// 1.Turn TTL low-power mode function
// 2.Turn internal termination function
// 3.Turn DRIVER BIAS OP function
////////////////////////////////////////////////////////////////////////
MS_BOOL MHal_PNL_MOD_Control_Out_TTL_Resistor_OP (void *pInstance, MS_BOOL bEnble)
{
    MS_BOOL bStatus = FALSE;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    if(bEnble)
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_34_L, 0xFFFF, 0xFFFF); //Enable TTL low-power mode
//        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, 0x001E, 0x001E);

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_32_L, 0xFFFF, 0xFFFF); //GCR_EN_RINT (internal termination open)
        MOD_W2BYTEMSK(REG_MOD_BK00_76_L, 0x003F, 0x003F);
        if(IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3C_L, 0xFFFF, 0xFFFF); //Disable DRIVER BIAS OP
        }
        MOD_W2BYTEMSK(REG_MOD_BK00_7A_L, 0x003F, 0x003F);
    }
    else
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_34_L, 0x0000, 0xFFFF); //Disable TTL low-power mode
//        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, 0x0000, 0x001E);

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_32_L, 0x0000, 0xFFFF); //GCR_EN_RINT (internal termination close)
        MOD_W2BYTEMSK(REG_MOD_BK00_76_L, 0x0000, 0x003F);
        if(IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type))
        {
            MOD_A_W2BYTE(REG_MOD_A_BK00_3C_L, 0x0000); //Enable DRIVER BIAS OP
        }
        MOD_W2BYTEMSK(REG_MOD_BK00_7A_L, 0x0000, 0x003F);
    }

    bStatus = TRUE;
    return bStatus;
}

void MHal_PNL_PreInit(void *pInstance, PNL_OUTPUT_MODE eParam)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    pPNLResourcePrivate->sthalPNL._eDrvPnlInitOptions = eParam;
}

PNL_OUTPUT_MODE MHal_PNL_Get_Output_MODE(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    PNL_OUTPUT_MODE eParam = pPNLResourcePrivate->sthalPNL._eDrvPnlInitOptions;

    return eParam;
}

MS_U32 msReadEfuse(void *pInstance, MS_U8 u8Bank, MS_U32 u32Mask)
{
    MS_U32 u32Result = 0;
    MS_U8 u8Count = 0;

    W2BYTEMSK(0x2050, u8Bank<<2, BMASK(8:2));  /// reg28[8:2]Addr 6~0
    W2BYTEMSK(0x2050, BIT(13), BIT(13));       /// Reg28[13] Margin Read
    while(R2BYTEMSK(0x2050, BIT(13)) == BIT(13))
    {
        MsOS_DelayTask(1);
        u8Count ++;

        if (u8Count >10)
            break;
    }

    u32Result = (R4BYTE(0x2058)& u32Mask);    /// reg2C,2D read value
    HAL_MOD_CAL_DBG(printf("[%s][%d]u32Result=%tx, after mask u32Result=%tx\n", __FUNCTION__, __LINE__,(ptrdiff_t) R4BYTE(0x2058), (ptrdiff_t)u32Result));
    return u32Result;

}

void msSetVBY1RconValue(void *pInstance)
{
    MS_U16 u16DefaultICON_Max = 40, u16DefaultICON_Min = 7;
    MS_U16 u16DefaultICON = 19;
    MS_U32 u32Mask = 0x3F;
    MS_BOOL bEfuseMode = FALSE;
    MS_U16 u16SwingOffset = 0;  // by HW RD request
    MS_U16 u16temp = 0;

    if(!(_Hal_MOD_External_eFuse()))
    {
        if (msReadEfuse(pInstance, 0x4E, BIT(6)) == BIT(6))
            bEfuseMode = TRUE;

        // Disable HW calibration keep mode first, to make SW icon value can write into register.
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, 0, BIT(15));

        if (bEfuseMode)
        {
            if(((MS_U16)msReadEfuse(pInstance, 0x4E, u32Mask) + u16SwingOffset) > u16DefaultICON_Max)
                u16temp = u16DefaultICON;
            else if(((MS_U16)msReadEfuse(pInstance, 0x4E, u32Mask) + u16SwingOffset) < u16DefaultICON_Min)
                u16temp = u16DefaultICON;
            else
                u16temp = (MS_U16)msReadEfuse(pInstance, 0x4E, u32Mask) + u16SwingOffset;
        }
        else
        {
            u16temp = u16DefaultICON;
        }

        //ch0~ch13 rcon setting
        u16temp &= (u16temp&(MS_U16)u32Mask);
        printf("[%s][%d]u16temp= %x\n", __FUNCTION__, __LINE__, u16temp);

        MOD_A_W2BYTE(REG_MOD_A_BK00_10_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_11_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_12_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_13_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_14_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_15_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_16_L, (u16temp<<8|u16temp));
        MOD_A_W2BYTE(REG_MOD_A_BK00_17_L, (u16temp<<8|u16temp));
    }
}

void MHal_PNL_SetOutputType(void *pInstance, PNL_OUTPUT_MODE eOutputMode, PNL_TYPE eLPLL_Type)
{
    MS_U16 u16ValidSwing2 = 0;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    if( eLPLL_Type == E_PNL_TYPE_TTL)
    {
        // select pair output to be TTL
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0020,0x0020);
        MsOS_DelayTask(1);
        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0000);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0000);
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0000,0x0020);

        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, 0x0000, (BIT(7) | BIT(6))); // shift_lvds_pair

        // other TTL setting
        MOD_A_W2BYTE(REG_MOD_A_BK00_68_L, 0x0000);     // TTL output enable

        MOD_A_W2BYTE(REG_MOD_A_BK00_40_L, 0x0000);
        MOD_A_W2BYTE(REG_MOD_A_BK00_41_L, 0x0000);
        MOD_W2BYTEMSK(REG_MOD_BK00_7E_L, 0x0000, 0xE000);

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, 0x3FF, 0x3FF);       // TTL skew

        MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0001, 0x0007);//[2:0]reg_mft_mode

        MOD_A_W2BYTE(REG_MOD_A_BK00_38_L, 0x0000); // enable clk_dot_mini_pre_osd & clk_dot_mini_osd

        // GPO gating
        MOD_W2BYTEMSK(REG_MOD_BK00_4A_L, BIT(8), BIT(8));         // GPO gating
    }
    else if(( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_8LANE)||
            ( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE)||
            ( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE))
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_34_L, 0xF000, 0xF000);  //[15:14]datax[13:12]data_format3,2

        // rcon
        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u32PNL_MISC & (MS_U32)E_APIPNL_MISC_SKIP_ICONVALUE)
        {
            HAL_MOD_CAL_DBG(printf("User define Swing Value=%u\n", __FUNCTION__, __LINE__, pPNLResourcePrivate->sthalPNL._u16PnlDefault_SwingLevel));
            MHal_PNL_MOD_Control_Out_Swing(pInstance, pPNLResourcePrivate->sthalPNL._u16PnlDefault_SwingLevel);
        }
        else
        {
            HAL_MOD_CAL_DBG(printf("Use RconValue\n", __FUNCTION__, __LINE__));
            msSetVBY1RconValue(pInstance);
        }

        // rint
        MOD_A_W2BYTE(REG_MOD_A_BK00_32_L, 0x0000);

        MOD_A_W2BYTE(REG_MOD_A_BK00_68_L, 0x003f);

        //-------------------------------------
        //## pe
//        MOD_A_W2BYTE(REG_MOD_A_BK00_30_L, 0x3fff);
//        MOD_W2BYTE(REG_MOD_BK00_23_L, 0x7000);
//        MOD_W2BYTE(REG_MOD_BK00_24_L, 0x7fff);
//        MOD_W2BYTE(REG_MOD_BK00_25_L, 0x003f);

        // VBY1 setting
        MOD_A_W2BYTE(REG_MOD_A_BK00_71_L, 0xFFFF);

        MOD_W2BYTE(REG_MOD_BK00_61_L, 0x8F3F); //[15]all dk scr[13:8]aln_de_cnt [7:0] aln_pix_cnt
//        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0xA040, 0xFDCF); //[15]proc_st[13:12]byte_mode 4 byte mode[6]4ch_vby1[9]swap

        MOD_A_W2BYTE(REG_MOD_A_BK00_40_L, 0xFFFF);
        MOD_A_W2BYTE(REG_MOD_A_BK00_41_L, 0xFFFF);
        MOD_A_W2BYTE(REG_MOD_A_BK00_48_L, 0xFFFF);
        MOD_A_W2BYTE(REG_MOD_A_BK00_49_L, 0xFFFF);
        MOD_A_W2BYTE(REG_MOD_A_BK00_42_L, 0xFFFF);
        MOD_A_W2BYTE(REG_MOD_A_BK00_43_L, 0xFFFF);

        switch(eOutputMode)
        {
            case E_PNL_OUTPUT_NO_OUTPUT:
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0020,0x0020);
                MsOS_DelayTask(1);
                MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0000);
                MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0000);
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0000,0x0020);
                if(1)//( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE)
                {
                    //-------------------------------------
                    //## icon (Swing)
                    MOD_A_W2BYTE(REG_MOD_A_BK00_08_L, 0x0000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_09_L, 0x0000);

                    //-------------------------------------
                    //vby1
                    MOD_W2BYTE(REG_MOD_BK00_61_L, 0x8f3f); //[15]all dk scr[13:8]aln_de_cnt [7:0] aln_pix_cnt
                    //MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0xa040, 0xFDCF); //[15]proc_st[13:12]byte_mode 4 byte mode[6]4ch_vby1[9]swap
                }
                else if( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE)
                {
                    //-------------------------------------
                    //## icon (Swing)
                    MOD_A_W2BYTE(REG_MOD_A_BK00_08_L, 0x0000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_09_L, 0x0000);

                    //vby1
                    MOD_W2BYTE(REG_MOD_BK00_61_L, 0x8f3f);
                    //MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0xa040, 0xFDCF);
                }
                break;

            case E_PNL_OUTPUT_CLK_ONLY:
            case E_PNL_OUTPUT_DATA_ONLY:
            case E_PNL_OUTPUT_CLK_DATA:
            default:
                if(eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_8LANE)
                {
                    //MOD_A_W2BYTE(REG_MOD_A_BK00_3A_L, 0xC100);
                    if( APIPNL_OUTPUT_CHANNEL_ORDER_USER == pPNLResourcePrivate->stdrvPNL._stPnlInitData.u8OutputOrderType )
                    {
                        _MHal_PNL_Auto_Set_Config(pInstance,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder0_3,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder4_7,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder8_11,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder12_13);
                    }
                    else
                    {
                        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x5555); //[15:0]reg_output_conf[15:0]
                    }
                }
                else if( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE)
                {
                    //MOD_A_W2BYTE(REG_MOD_A_BK00_3A_L, 0xC000);
                    MOD_W2BYTE(REG_MOD_BK00_4A_L, 0x0002); //[1]reg_dualmode[0]abswitch
                    MOD_A_W2BYTE(REG_MOD_A_BK00_0A_L, 0x7f7f);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_0B_L, 0x7f7f);

                    if( APIPNL_OUTPUT_CHANNEL_ORDER_USER == pPNLResourcePrivate->stdrvPNL._stPnlInitData.u8OutputOrderType )
                    {
                        _MHal_PNL_Auto_Set_Config(pInstance,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder0_3,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder4_7,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder8_11,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder12_13);
                    }
                    else
                    {
                        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0055); //[15:0]reg_output_conf[15:0]
                    }
                }
                else if( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE)
                {
                    //MOD_A_W2BYTE(REG_MOD_A_BK00_3A_L, 0xC000);
                    MOD_W2BYTE(REG_MOD_BK00_4A_L, 0x0000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_0A_L, 0x7f7f);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_0B_L, 0x0000);

                    if( APIPNL_OUTPUT_CHANNEL_ORDER_USER == pPNLResourcePrivate->stdrvPNL._stPnlInitData.u8OutputOrderType )
                    {
                        _MHal_PNL_Auto_Set_Config(pInstance,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder0_3,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder4_7,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder8_11,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder12_13);
                    }
                    else
                    {
                        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0005);
                    }
                }
                break;
        }
    }
    //// for osd dedicated output port, 1 port for video and 1 port for osd
    else if((eLPLL_Type == E_PNL_TYPE_HS_LVDS)&&
            (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Mode == E_PNL_MODE_SINGLE))
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_38_L, 0xC01F); // enable clk_dot_mini_pre_osd & clk_dot_mini_osd
        MOD_W2BYTE(REG_MOD_BK00_42_L, BIT(3) ); // enable osd lvds path
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, BIT(15), BIT(15)); //[15]sw_rst
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0xC000 , 0xF000); // enable clk_dot_mini_pre_osd & clk_dot_mini_osd
    }
    else
    {
        switch(eOutputMode)
        {
            case E_PNL_OUTPUT_NO_OUTPUT:
                // if MOD_45[5:0] = 0x3F && XC_MOD_EXT_DATA_EN_L = 0x0,
                // then if XC_MOD_OUTPUT_CONF_L = 0x0 ---> output TTL as tri-state
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0020,0x0020);
                MsOS_DelayTask(1);
                if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
                {
                    MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0x0000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0000);
                }
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3B_L, 0x0000,0x0020);

                //----------------------------------
                // Purpose: Set the output to be the GPO, and let it's level to Low
                // 1. External Enable, Pair 0~5
                // 2. GPIO Enable, pair 0~5
                // 3. GPIO Output data : All low, pair 0~5
                // 4. GPIO OEZ: output piar 0~5
                //----------------------------------
                if(!IsTCONLESS(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type))
                {
                    //1.External Enable, Pair 0~5
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_40_L, 0x0FFF, 0x0FFF);
                    //2.GPIO Enable, pair 0~5
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_48_L, 0x0FFF, 0x0FFF);
                    //3.GPIO Output data : All low, pair 0~5
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_44_L, 0x0000, 0x0FFF);
                    //4.GPIO OEZ: output piar 0~5
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_42_L, 0x0000, 0x0FFF);

                    //1.External Enable, Pair 6~15
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_40_L, 0xF000, 0xF000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_41_L, 0xFFFF);
                    //2.GPIO Enable, pair 6~15
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_48_L, 0xF000, 0xF000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_49_L, 0xFFFF);
                    //3.GPIO Output data : All low, pair 6~15
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_44_L, 0x0000, 0xF000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_45_L, 0x0000);
                    //4.GPIO OEZ: output piar 6~15
                    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_42_L, 0x0000, 0xF000);
                    MOD_A_W2BYTE(REG_MOD_A_BK00_43_L, 0x0000);
                }
                //1234.External Enable, Pair 16~17
                MOD_W2BYTE(REG_MOD_BK00_7E_L, 0xFF00);

                //1.External Enable, Pair 18~20, 2.GPIO Enable, pair 18~20
                MOD_W2BYTEMSK(REG_MOD_BK00_7C_L, 0x3F3F, 0x3F3F);
                //3.GPIO Output data : All low, pair 18~20
                MOD_W2BYTEMSK(REG_MOD_BK00_7A_L, 0x0000, 0x3F00);
                //4.GPIO OEZ: output piar 18~20
                MOD_W2BYTEMSK(REG_MOD_BK00_7F_L, 0x0000, 0xFC00);
                break;

            case E_PNL_OUTPUT_CLK_ONLY:
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0, 0xF000);
                MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x4004);
                break;

            case E_PNL_OUTPUT_DATA_ONLY:
            case E_PNL_OUTPUT_CLK_DATA:
            default:
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_48_L, 0x0000, 0xF000);
                MOD_A_W2BYTE(REG_MOD_A_BK00_49_L, 0x0000);
                //1. set GCR_PVDD_2P5=1¡¦b1;           MOD PVDD power:    1: 2.5V
                //MOD_A_W2BYTEMSK(REG_MOD_A_BK00_39_L, 0, BIT(6));
                //2. set PD_IB_MOD=1¡¦b0;
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0x00, BIT(0));
                //  save ch6 init value
                u16ValidSwing2 = (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0B_L, 0x3F00)>>8);
                //3. set Desired Pairs: GCR_ICON[5:0]=6h3f (current all open);
                if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
                {
                    MHal_PNL_MOD_Control_Out_Swing(pInstance, MHal_PNL_MODSwingRegToRealLevelValue(pInstance, 0x3F));
                }
                //4. set Desired Pairs: GCR_PE_ADJ[2:0]=3h7 (pre-emphasis current all open )
                MHal_PNL_MOD_Control_Out_PE_Current (pInstance, 0x07);
                //5. Enable low-power modeinternal termination Open, Enable OP
                MHal_PNL_MOD_Control_Out_TTL_Resistor_OP (pInstance, 1);

                MsOS_DelayTask(1);

                //6. Enable low-power modeinternal termination Open, Enable OP
                if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
                {
                    MHal_Output_LVDS_Pair_Setting(pInstance,
                                                  pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15,
                                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);
                }

                MHal_Shift_LVDS_Pair(pInstance, pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Shift);


                //7. set Desired Pairs: GCR_PE_ADJ[2:0]=3¡¦h0 (pre-emphasis current all Close)
                MHal_PNL_MOD_Control_Out_PE_Current (pInstance, 0x00);
                //8. set Desired Pairs: GCR_ICON[5:0]    (current all init);
                if(!IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
                {
                    MHal_PNL_MOD_Control_Out_Swing(pInstance, MHal_PNL_MODSwingRegToRealLevelValue(pInstance, u16ValidSwing2));
                }
                //9. Disable low-power modeinternal termination Close, Disable OP
                MHal_PNL_MOD_Control_Out_TTL_Resistor_OP (pInstance, 0);

                // other TTL setting
                MOD_A_W2BYTE(REG_MOD_A_BK00_68_L, 0x003F);     // LVDS output enable, [5:4] Output enable: PANEL_LVDS/ PANEL_miniLVDS/ PANEL_RSDS

                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_40_L, 0x0000, 0xF000);
                MOD_A_W2BYTE(REG_MOD_A_BK00_41_L, 0x0000);
                MOD_W2BYTEMSK(REG_MOD_BK00_7E_L, 0x0000, 0x000F);

                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, 0x000, 0x3FF);    // TTL skew

                // GPO gating
                MOD_W2BYTEMSK(REG_MOD_BK00_4A_L, 0x0, BIT(8));     // GPO gating

                break;
        }
    }

//    MHal_PNL_Bringup(pInstance);
}

void Mhal_PNL_Flock_LPLLSet(void *pInstance, MS_U64 ldHz)
{
    UNUSED(ldHz);
}

void MHal_PNL_MISC_Control(void *pInstance, MS_U32 u32PNL_MISC)
{
    if(u32PNL_MISC & E_DRVPNL_MISC_MFC_ENABLE)
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, BIT(7), BIT(7));       // shift LVDS pair
    }
}

void MHal_PNL_Init_XC_Clk(void *pInstance, PNL_InitData *pstPanelInitData)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]\n", __FUNCTION__, __LINE__);

    // setup output dot clock
    if (pstPanelInitData->eLPLL_Type == E_PNL_TYPE_TTL)
    {
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_GATED);                          // [0] disable clock
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_INVERT);                         // [1] invert clock
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_SEL_LPLL, CKG_ODCLK_SEL_SOURCE);          // [2] select source tobe LPLL clock, 0:synthetic clock out, 1:LPLL clock out
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_CLK_DIV_2, CKG_ODCLK_MASK);               // [4..3] LPLL clock div, TTL:DIV2

        W2BYTEMSK(REG_RVD_44_L, 0x0004, 0x001F); //[4:0]reg_ckg_clk_misc
        W2BYTEMSK(L_CLKGEN0(0x58), 0x0001, 0x000F); //[3:0]ckg_tx_mod
    }
    else if ((pstPanelInitData->eLPLL_Type == E_PNL_TYPE_LVDS) && ((pstPanelInitData->eLPLL_Mode == E_PNL_MODE_SINGLE)))
    {
        // 1366x768, 1CH LVDS case
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_GATED);                          // [0] disable clock
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_INVERT);                         // [1] invert clock
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_SEL_LPLL, CKG_ODCLK_SEL_SOURCE);          // [2] select source tobe LPLL clock, 0:synthetic clock out, 1:LPLL clock out
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_CLK_DIV_2, CKG_ODCLK_MASK);               // [4..3] LPLL clock div, TTL:DIV2

        W2BYTEMSK(REG_RVD_44_L, 0x0004, 0x001F); //[4:0]reg_ckg_clk_misc
        W2BYTEMSK(L_CLKGEN0(0x58), 0x0000, 0x000F); //[3:0]ckg_tx_mod

        MOD_W2BYTEMSK(REG_MOD_BK00_2F_L,BIT(0),BIT(0));
    }
    if ((pstPanelInitData->eLPLL_Type == E_PNL_TYPE_DAC_P) || (pstPanelInitData->eLPLL_Type == E_PNL_TYPE_DAC_I))
    {
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_GATED);                          // [0] disable clock
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_INVERT);                         // [1] invert clock
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_SEL_LPLL, CKG_ODCLK_SEL_SOURCE);          // [2] select source tobe LPLL clock, 0:synthetic clock out, 1:LPLL clock out
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_CLK_LPLL, CKG_ODCLK_MASK);               // [4..3] LPLL clock div, TTL:DIV2

        // For mantis 1208252: HDMI Tx frame lock issue, HW RD said 0x100A_44 should be 0x1C for HDMI Tx case
        W2BYTEMSK(REG_RVD_44_L, 0x001C, 0x001F); //[4:0]reg_ckg_clk_misc
    }
    else
    {
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_GATED);                          // [0] disable clock
        W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_INVERT);                         // [1] invert clock
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_SEL_LPLL, CKG_ODCLK_SEL_SOURCE);          // [2] select source tobe LPLL clock, 0:synthetic clock out, 1:LPLL clock out
        W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_XTAL, CKG_ODCLK_MASK);               // [4..3] LPLL clock div

        W2BYTEMSK(REG_RVD_44_L, 0x000C, 0x001F); //[4:0]reg_ckg_clk_misc
        W2BYTEMSK(L_CLKGEN0(0x58), 0x0000, 0x000F); //[3:0]ckg_tx_mod

        MOD_W2BYTEMSK(REG_MOD_BK00_2F_L,BIT(1),BIT(1));
    }

    // reg_ckg_odclk_mft
    W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_MFT_GATED);                          // [8] disable clock
    W2BYTEMSK(L_CLKGEN0(0x53), DISABLE, CKG_ODCLK_MFT_INVERT);                         // [9] invert clock
    W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_MFT_SEL_LPLL, CKG_ODCLK_MFT_SEL_SOURCE);      // [10] select source tobe LPLL clock, 0:synthetic clock out, 1:LPLL clock out                                                            // 0:synthetic clock out, 1:LPLL clock out
    W2BYTEMSK(L_CLKGEN0(0x53), CKG_ODCLK_MFT_XTAL, CKG_ODCLK_MFT_MASK);                // [12:11] LPLL clock div

    W2BYTE(L_CLKGEN1(0x31), 0x0000); //[11:8]ckg_odclk_frc

    MS_U32 u32PanelLinkType = 0;
    if(IsTCONLESS(stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType))
    {
        u32PanelLinkType = stTCONPanelConfig.stPanelInfo.u32PanelLinkExtType;
    }
    else
    {
        u32PanelLinkType = pstPanelInitData->eLPLL_Type;
    }

    if(  IsVBY1(u32PanelLinkType) )
    {
        if((u32PanelLinkType == E_PNL_LPLL_VBY1_8BIT_8LANE) || (u32PanelLinkType == E_PNL_LPLL_VBY1_8BIT_16LANE))  //video 8bit case
        {
            W2BYTEMSK(REG_CLKGEN0_57_L, 0x000C, 0x000F); //[3:0]ckg_fifo
        }
        else if((u32PanelLinkType == E_PNL_LPLL_VBY1_10BIT_4LANE) || (u32PanelLinkType == E_PNL_LPLL_VBY1_10BIT_16LANE)) //video 10bit case
        {
            W2BYTEMSK(REG_CLKGEN0_57_L, 0x0008, 0x000F); //[3:0]ckg_fifo
        }
        W2BYTE(REG_RVD_09_L, 0x0800); //[13:8]ckg_vby1_fifo_osd [3:0]clk_vby1_fifo
        //W2BYTEMSK(L_CLKGEN0(0x55), 0x00, 0xF00); //[11:8] reg_ckg_osdc
        W2BYTEMSK(L_CLKGEN0(0x56),0x0010, 0x00F0); //[7:4]reg_ckg_odclk_a
        W2BYTE(L_CLKGEN0(0x63),0x1001); //[15:12]reg_ckg_nossc_odclk[11:8]ckg_tx_mod [5:0]ckg_osd2mod

        if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE)
            ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
        {
            W2BYTE(L_CLKGEN0(0x5A),0x0800); //[15:12]reg_ckg_vby1_omode [11:8]reg_ckg_vby1_vmode
        }
        else if ((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_4LANE)
                ||(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_4LANE))
        {
            W2BYTE(L_CLKGEN0(0x5A),0x0900); //[15:12]reg_ckg_vby1_omode [11:8]reg_ckg_vby1_vmode
        }
        else
        {
            W2BYTE(L_CLKGEN0(0x5A),0x0A00); //[15:12]reg_ckg_vby1_omode [11:8]reg_ckg_vby1_vmode
        }
        MOD_W2BYTEMSK(REG_MOD_BK00_2F_L,BIT(1),BIT(1));

    }
    else
    {
        if (u32PanelLinkType == E_PNL_TYPE_TTL)
        {
            W2BYTEMSK(REG_CLKGEN0_57_L, 0x0001, 0x000F); //[3:0]ckg_fifo
            W2BYTEMSK(L_CLKGEN0(0x56),0x0040, 0x00F0); //[7:4]reg_ckg_odclk_a
            W2BYTE(L_CLKGEN0(0x63),0x1100); //[15:12]reg_ckg_nossc_odclk[11:8]ckg_tx_mod [5:0]ckg_osd2mod
        }
        else if( IsTCONType_ISP(u32PanelLinkType) || IsTCONType_USIT(u32PanelLinkType) || IsTCONType_USII(u32PanelLinkType) || IsTCONType_MiniLVDS(u32PanelLinkType) )
        {
            W2BYTE(L_CLKGEN0(0x63),0x1001); //[15:12]reg_ckg_nossc_odclk [11:8]reg_ckg_tx_mod_osd [5:0]reg_ckg_osd2mod
            W2BYTEMSK(REG_CLKGEN0_57_L, 0x0008, 0x00FF); //[3:0]reg_ckg_fifo_mini
            W2BYTEMSK(L_CLKGEN0(0x56),0x0010, 0x00FF); //[11:8]reg_ckg_dot_mini_pre[7:4]reg_ckg_odclk_a[3:0]reg_ckg_dot_mini
        }
        else
        {
            W2BYTEMSK(REG_CLKGEN0_57_L, 0x0000, 0x000F); //[3:0]ckg_fifo
            W2BYTE(L_CLKGEN0(0x63),0x1000); //[15:12]reg_ckg_nossc_odclk[11:8]ckg_tx_mod [5:0]ckg_osd2mod
        }

        if((u32PanelLinkType == E_PNL_TYPE_HS_LVDS)&&(pstPanelInitData->eLPLL_Mode == E_PNL_MODE_SINGLE))
        {
            W2BYTE(REG_RVD_09_L, 0x1000); //[12:8]ckg_vby1_fifo_osd [3:0]clk_vby1_fifo+
        }
        else if( IsTCONType_ISP(u32PanelLinkType) || IsTCONType_USIT(u32PanelLinkType) || IsTCONType_USII(u32PanelLinkType) || IsTCONType_MiniLVDS(u32PanelLinkType) )
        {
            W2BYTE(REG_RVD_09_L, 0x0800); //[12:8]ckg_vby1_fifo_osd [3:0]clk_vby1_fifo
        }
        else
        {
            W2BYTE(REG_RVD_09_L, 0x0000); //[12:8]ckg_vby1_fifo_osd [3:0]clk_vby1_fifo
        }
        if(u32PanelLinkType == E_PNL_TYPE_LVDS)
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_2F_L,BIT(0),BIT(0));
        }
    }

    if( (E_PNL_TYPE_TTL == u32PanelLinkType)||
       ((E_PNL_TYPE_LVDS == u32PanelLinkType)&&(E_PNL_MODE_SINGLE==pstPanelInitData->eLPLL_Mode))||
       ((E_PNL_TYPE_HS_LVDS == u32PanelLinkType)&&(E_PNL_MODE_SINGLE==pstPanelInitData->eLPLL_Mode)))
    {
        //set LPLL mux
        W2BYTEMSK(L_CLKGEN0(0x7E), 0x0010, 0x00FC);           //LPLL_ODCLK setting  reg_ckg_odclk = reg_clkgen0_reserved0[7:2]
                                                              //[0] : diable clock
                                                              //[1] : invert clock
                                                              //[4:2] : xx0 : floclk_odclk_synth_out
                                                              // 011 / 111: LPLL output clock
    }
    else if ((E_PNL_TYPE_DAC_P == u32PanelLinkType) ||
            (E_PNL_TYPE_DAC_I == u32PanelLinkType))
    {
        //set LPLL mux
        W2BYTEMSK(L_CLKGEN0(0x7E), 0x00A0, 0x00FC);           //LPLL_ODCLK setting  reg_ckg_odclk = reg_clkgen0_reserved0[7:2]
                                                              //[0] : diable clock
                                                              //[1] : invert clock
                                                              //[4:2] : xx0 : floclk_odclk_synth_out
                                                              // 011 / 111: LPLL output clock
    }
    else
    {
        //set LPLL mux
        W2BYTEMSK(L_CLKGEN0(0x7E), 0x0030, 0x00FC);           //LPLL_ODCLK setting  reg_ckg_odclk = reg_clkgen0_reserved0[7:2]
                                                              //[0] : diable clock
                                                              //[1] : invert clock
                                                              //[4:2] : xx0 : floclk_odclk_synth_out
                                                              // 011 / 111: LPLL output clock
    }

    if(u32PanelLinkType == E_PNL_LPLL_ISP_8BIT_12P)
    {
        W2BYTEMSK(REG_CHIP_50_L, 0x000F, 0x000F);
    }

    //MISC part
    if(IsVBY1(pstPanelInitData->eLPLL_Type_Ext))
    {
        /*********Common Setting bk1523*********/
        //disp misc, bk1523 need to set before bk1032
        W2BYTEMSK(L_BK_DISP_MISC(0x0f), 0x0, 0x1f); //reg_ckg_mod_odclk [4:0]
        W2BYTEMSK(L_BK_DISP_MISC(0x10), 0x404, 0x1f1f); //reg_ckg_mod_odclk [4:0] //reg_ckg_mod_odclk_mft [12:8]
        W2BYTEMSK(L_BK_DISP_MISC(0x11), 0x0, 0x1f1f); //reg_ckg_mod_sr_rclk [4:0]  //reg_ckg_mod_sr_wclk [12:8]
        W2BYTEMSK(L_BK_DISP_MISC(0x12), 0x100, 0x1f00); //reg_ckg_mod_odclk_a [12:8]
        W2BYTE(L_BK_DISP_MISC(0x20), 0x0); //reg_mod_osd_odclk_div_nm [15:0]
        W2BYTEMSK(L_BK_DISP_MISC(0x21), 0x01, 0x3f); //reg_ckg_osd_odclk [5:0]
        W2BYTEMSK(L_BK_DISP_MISC(0x22), 0x0, 0x1f1f); //reg_ckg_osd_sr_rclk [4:0] //reg_ckg_osd_sr_wclk [12:8]
        W2BYTEMSK(L_BK_DISP_MISC(0x30), BIT(0)|BIT(1), BIT(0)|BIT(1)|BIT(2)); //reg_v_root_en //reg_v_scalar_en //reg_o_scalar_en
        W2BYTEMSK(L_BK_DISP_MISC(0x31), 0x0, 0x707); //reg_ckg_v_scalar[2:0] //reg_ckg_0_scalar[10:8]
        W2BYTEMSK(L_BK_DISP_MISC(0x70), 0x101, 0xf07); //reg_ckg_mod_det_od[3:0] //reg_ckg_mod_det_fifo[11:8]

        /*Update Vx1 apn setting*/
        if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
        {
            //update disp_misc top clk ratio, bk1523 need to set before bk1032
            W2BYTE(L_BK_DISP_MISC(0x13), 0x108); //reg_mod_odclk_div_nm, [15:0]
            W2BYTE(L_BK_DISP_MISC(0x32), 0x0); //reg_scalar_v_div_nm, [15:0]
            W2BYTE(L_BK_DISP_MISC(0x33), 0x0); //reg_scalar_o_div_nm, [15:0]
        }
        else if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_4LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_4LANE))
        {
                //update disp_misc top clk ratio, bk1523 need to set before bk1032
                W2BYTE(L_BK_DISP_MISC(0x13), 0x104); //reg_mod_odclk_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x32), 0x102); //reg_scalar_v_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x33), 0x0); //reg_scalar_o_div_nm, [15:0]
        }
        else if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_2LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_2LANE))
        {
                //update disp_misc top clk ratio, bk1523 need to set before bk1032
                W2BYTE(L_BK_DISP_MISC(0x13), 0x102); //reg_mod_odclk_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x32), 0x104); //reg_scalar_v_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x33), 0x0); //reg_scalar_o_div_nm, [15:0]
        }
        else
        {
            PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]do not support type!! eLPLL_Type_Ext = %d\n", __FUNCTION__, __LINE__, pstPanelInitData->eLPLL_Type_Ext);
        }
    }
    else
    {
        //LVDS setting
        if(pstPanelInitData->eLPLL_Type == E_PNL_TYPE_LVDS)
        {
            /*********Common Setting bk1523*********/
            //disp misc, bk1523 need to set before bk1032
            W2BYTEMSK(L_BK_DISP_MISC(0x0f), 0x0, 0x1f); //reg_ckg_mod_odclk [4:0]
            W2BYTEMSK(L_BK_DISP_MISC(0x11), 0x0, 0x1f1f); //reg_ckg_mod_sr_rclk [4:0] //reg_ckg_mod_sr_wclk [12:8]
            W2BYTEMSK(L_BK_DISP_MISC(0x12), 0x100, 0x1f00); //reg_ckg_mod_odclk_a [12:8]
            W2BYTE(L_BK_DISP_MISC(0x20), 0x0); //reg_mod_osd_odclk_div_nm [15:0]
            W2BYTEMSK(L_BK_DISP_MISC(0x22), 0x0, 0x1f1f); //reg_ckg_osd_sr_rclk [4:0]  //reg_ckg_osd_sr_wclk [12:8]
            W2BYTEMSK(L_BK_DISP_MISC(0x30), BIT(0)|BIT(1), BIT(0)|BIT(1)|BIT(2)); //reg_v_root_en  //reg_v_scalar_en //reg_o_scalar_en
            W2BYTEMSK(L_BK_DISP_MISC(0x31), 0x0, 0x707); //reg_ckg_v_scalar[2:0] //reg_ckg_0_scalar[10:8]
            W2BYTEMSK(L_BK_DISP_MISC(0x70), 0x101, 0xf07); //reg_ckg_mod_det_od[3:0] //reg_ckg_mod_det_fifo[11:8]

            if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Mode == E_PNL_MODE_SINGLE)
            {
                W2BYTEMSK(L_BK_DISP_MISC(0x10), 0x4, 0x1f1f); //reg_ckg_mod_odclk [4:0] //reg_ckg_mod_odclk_mft [12:8]
                W2BYTE(L_BK_DISP_MISC(0x13), 0x0); //reg_mod_odclk_div_nm, [15:0]
                W2BYTEMSK(L_BK_DISP_MISC(0x21), 0x01, 0x3f); //reg_ckg_osd_odclk [5:0]
                W2BYTE(L_BK_DISP_MISC(0x32), 0x107); //reg_scalar_v_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x33), 0x0); //reg_scalar_o_div_nm, [15:0]
            }
            else if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Mode == E_PNL_MODE_DUAL)
            {
                W2BYTEMSK(L_BK_DISP_MISC(0x10), 0x404, 0x1f1f);//reg_ckg_mod_odclk [4:0] //reg_ckg_mod_odclk_mft [12:8]
                W2BYTE(L_BK_DISP_MISC(0x13), 0x102); //reg_mod_odclk_div_nm, [15:0]
                W2BYTEMSK(L_BK_DISP_MISC(0x21), 0x24, 0x3f); //reg_ckg_osd_odclk [5:0]
                W2BYTE(L_BK_DISP_MISC(0x32), 0x104); //reg_scalar_v_div_nm, [15:0]
                W2BYTE(L_BK_DISP_MISC(0x33), 0x0); //reg_scalar_o_div_nm, [15:0]
            }
            else
            {
                PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]do not support type!! eLPLL_Type_Ext = %d\n", __FUNCTION__, __LINE__, pstPanelInitData->eLPLL_Type_Ext);
            }
        }
    }
}

static MS_U8 _MHal_PNL_GetLaneNum(void *pInstance,PNL_TYPE eLPLL_Type)
{
    if(eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_16LANE || eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_16LANE)
    {
        return 16;
    }
    else if(eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_8LANE || eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_8LANE)
    {
        return 8;
    }
    else if(eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_4LANE || eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE
          || eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE_BYPASS_MODE || eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_4LANE_BYPASS_MODE )
    {
        return 4;
    }
    else if(eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_2LANE || eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE)
    {
        return 2;
    }
    else if(eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_1LANE || eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_1LANE)
    {
        return 1;
    }
    return 0;
}

#define OUTPUT_CHANNEL_PART_A   0x0055 // CH0~3
#define OUTPUT_CHANNEL_PART_B   0x5500 // CH4~7
#define OUTPUT_CHANNEL_PART_C   0x0015 // CH8~10
#define OUTPUT_CHANNEL_PART_D   0x0540 // CH11~13
#define OUTPUT_CHANNEL_ALL      0x5555

void MHal_PNL_Init_MOD(void *pInstance, PNL_InitData *pstPanelInitData)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]\n", __FUNCTION__, __LINE__);

    //------------------------------------------------------------------------

    PNL_DBG(PNL_DBGLEVEL_INIT, "u16MOD_CTRL0 = %x\n", pstPanelInitData->u16MOD_CTRL0);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u16MOD_CTRL9 = %x\n", pstPanelInitData->u16MOD_CTRL9);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u16MOD_CTRLA = %x\n", pstPanelInitData->u16MOD_CTRLA);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u8MOD_CTRLB  = %x\n", pstPanelInitData->u8MOD_CTRLB);

    //-------------------------------------------------------------------------
    // Set MOD registers
    //-------------------------------------------------------------------------

    MOD_W2BYTEMSK(REG_MOD_BK00_40_L, pstPanelInitData->u16MOD_CTRL0, LBMASK);

    //    GPIO is controlled in drvPadConf.c
    //    MDrv_Write2Byte(L_BK_MOD(0x46), 0x0000);    //EXT GPO disable
    //    MDrv_Write2Byte(L_BK_MOD(0x47), 0x0000);    //EXT GPO disable

    if(IsVBY1(pstPanelInitData->eLPLL_Type_Ext))
    {
        /*Update Vx1 apn setting*/
        if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_8LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_8LANE))
        {
            if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16MOD_CTRLA & BIT(1))  // use Dual port to decide the Vx1 1 or 2 devision config
            {
                // 2 divison just be supported in monet vby1 8 lane
                /****MOD Setting - 8-lane & 2-division****/
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0002, 0x0007);                       //[2:0]reg_mft_mode
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0F00, 0xFFFF);                       //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0780, 0xFFFF);                       //[12:0]reg_div_len
                //MOD_W2BYTEMSK(REG_MOD_BK00_7F_L, 0x0002, 0xFFFF);                      //[2:0]reg_sram_usage
                MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0x300, 0x3F00);                       //[13:8]reg_mft_vld_wrap_fix_htt
                //MOD_W2BYTEMSK(REG_MOD_BK00_2B_L, 0x0002,0x0003);                       //[1:0]reg_mft_read_trig_se
                MOD_W2BYTEMSK(REG_MOD_BK00_62_L, BIT(6), BIT(6));                        //[6]4ch_vby1
                MOD_W2BYTEMSK(REG_MOD_BK00_63_L, BIT(12), BIT(12));                     //[12] enable 8ch vx1 mode : 1
                MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0xff, 0xffff);                         //[15:0]reg_seri_ch_cken_bitmap
                PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]Vx1 2 division\n", __FUNCTION__, __LINE__);
            }
            else
            {
                /****MOD Setting - 8-lane & 1-division****/
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0005, 0x0007);                       //[2:0]reg_mft_mode
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0F00, 0xFFFF);                       //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0F00, 0xFFFF);                       //[12:0]reg_div_len
                //MOD_W2BYTEMSK(REG_MOD_BK00_7F_L, 0x0002, 0xFFFF);                      //[2:0]reg_sram_usage
                MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0x300, 0x3F00);                       //[13:8]reg_mft_vld_wrap_fix_htt
                //MOD_W2BYTEMSK(REG_MOD_BK00_2B_L, 0x0002,0x0003);                       //[1:0]reg_mft_read_trig_se
                MOD_W2BYTEMSK(REG_MOD_BK00_62_L, BIT(6), BIT(6));                        //[6]4ch_vby1
                MOD_W2BYTEMSK(REG_MOD_BK00_63_L, BIT(12), BIT(12));                     //[12] enable 8ch vx1 mode : 1
                MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0xff, 0xffff);                         //[15:0]reg_seri_ch_cken_bitmap
                PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]Vx1 1 division\n", __FUNCTION__, __LINE__);
            }
        }
        else if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_4LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_4LANE))
        {
                /****MOD Setting - 4-lane****/
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0005, 0x0007);                       //[2:0]reg_mft_mode
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0780, 0xFFFF);                       //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0780, 0xFFFF);                       //[12:0]reg_div_len
                //MOD_W2BYTEMSK(REG_MOD_BK00_7F_L, 0x0002, 0xFFFF);                      //[2:0]reg_sram_usage
                MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0x100, 0x3F00);                       //[13:8]reg_mft_vld_wrap_fix_htt
                //MOD_W2BYTEMSK(REG_MOD_BK00_2B_L, 0x0002,0x0003);                       //[1:0]reg_mft_read_trig_se
                MOD_W2BYTEMSK(REG_MOD_BK00_62_L, BIT(6), BIT(6));                        //[6]4ch_vby1
                MOD_W2BYTEMSK(REG_MOD_BK00_63_L, 0x0, BIT(12));                     //[12] enable 8ch vx1 mode : 1
                MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0xf, 0xffff);                         //[15:0]reg_seri_ch_cken_bitmap
        }
        else if((pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_10BIT_2LANE) || (pstPanelInitData->eLPLL_Type_Ext == E_PNL_LPLL_VBY1_8BIT_2LANE))
        {
                /****MOD Setting - 2-lane****/
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0005, 0x0007);                       //[2:0]reg_mft_mode
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0780, 0xFFFF);                       //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0780, 0xFFFF);                       //[12:0]reg_div_len
                //MOD_W2BYTEMSK(REG_MOD_BK00_7F_L, 0x0002, 0xFFFF);                      //[2:0]reg_sram_usage
                MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0x0, 0x3F00);                       //[13:8]reg_mft_vld_wrap_fix_htt
                //MOD_W2BYTEMSK(REG_MOD_BK00_2B_L, 0x0002,0x0003);                       //[1:0]reg_mft_read_trig_se
                MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0x0, BIT(6));                        //[6]4ch_vby1
                MOD_W2BYTEMSK(REG_MOD_BK00_63_L, 0x0, BIT(12));                     //[12] enable 8ch vx1 mode : 1
                MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0x3, 0xffff);                         //[15:0]reg_seri_ch_cken_bitmap
        }
        else
        {
            PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]do not support type!! eLPLL_Type_Ext = %d\n", __FUNCTION__, __LINE__, pstPanelInitData->eLPLL_Type_Ext);
        }

        /*********Common Setting bk1032*********/
        MOD_W2BYTEMSK(REG_MOD_BK00_53_L, BIT(14), BIT(14));                       //[14]reg_vfde_mask
        MOD_W2BYTEMSK(REG_MOD_BK00_1B_L, 0, BIT(6));                        //[6]reg_mft_keep_vb_de_out
        MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0, BIT(15)|BIT(14));                       //[15]reg_mft_fix_htt_disable  //[14]reg_mft_ctrl_ts_fix_disable
        MOD_W2BYTEMSK(REG_MOD_BK00_45_L, (BIT(0)|BIT(1)), (BIT(0)|BIT(1)));//[1]:reg_vby1_vs_inv, [0]:reg_vby1_hs_inv
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, BIT(15)|BIT(13), BIT(15)|BIT(13)|BIT(12));                     //[15]:reg_vby1_new_engine /[13:12]:reg_vby1_byte_mode
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, 0x0, (BIT(6)|BIT(5)));              //[6]:reg_vby1_2ch_osd, [5]reg_vby1_4ch_osd
        MOD_W2BYTEMSK(REG_MOD_BK00_2F_L, BIT(1), BIT(1));                       //[15]reg_mft_ctrl_ts_fix_disable

        ///u16MOD_CTRL9 [6] : 62[9]reg_vby1_pair_swap
        ///u16MOD_CTRL9 [12:11] : 62[5:4]reg_vby1_pair_mirror
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0xAC40, 0xFDCF); //[15]proc_st[13:12]byte_mode 4 byte mode[6]4ch_vby1[9]swap
        MOD_W2BYTEMSK(REG_MOD_BK00_5B_L, BIT(6), BIT(6)|BIT(7));                       //[6]reg_seri_buf16_en //[7]reg_seri_osd_mod
        //M7621 not used.
        //MOD_W2BYTEMSK(REG_MOD_BK00_62_L, (pstPanelInitData->u16MOD_CTRL9 & BIT(6))<<3, BIT(9));
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, (pstPanelInitData->u16MOD_CTRL9 & (BIT(12)|BIT(11)))>>7, BIT(5)|BIT(4));

#ifdef PNL_SUPPORT_H_BRCK_PORCH_MODE
        MS_U16 u16HDEStart = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_04_L);
        if(u16HDEStart > 0 && _MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext) > 0)
        {
            MS_U16 u16TSNum = 0;
            if(_MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext) > 1)
            {
                u16TSNum = _MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext)/MFT_P_NUM;
                if(u16TSNum > 0)
                {
                    MOD_W2BYTEMSK(REG_MOD_BK00_2C_L,(u16HDEStart/(MOD_P_NUM*_MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext)*u16TSNum)-1), 0xFFFF); //[15:0]reg_read_hde_st
                }
            }
            else
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L,((u16HDEStart/MOD_P_NUM)-1), 0xFFFF); //[15:0]reg_read_hde_st
            }
        }
#else
        switch(pstPanelInitData->eLPLL_Type_Ext)
        {
            case E_PNL_LPLL_VBY1_10BIT_8LANE:
            case E_PNL_LPLL_VBY1_8BIT_8LANE:
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L, 0x22,0xFFFF); //[15:0]reg_read_hde_st
            }
            break;
            case E_PNL_LPLL_VBY1_8BIT_4LANE:
            case E_PNL_LPLL_VBY1_10BIT_4LANE:
            case E_PNL_LPLL_VBY1_10BIT_4LANE_BYPASS_MODE:
            case E_PNL_LPLL_VBY1_8BIT_4LANE_BYPASS_MODE:
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L, 0x45,0xFFFF); //[15:0]reg_read_hde_st
            }
            break;
            case E_PNL_LPLL_VBY1_8BIT_2LANE:
            case E_PNL_LPLL_VBY1_10BIT_2LANE:
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L, 0x8B,0xFFFF); //[15:0]reg_read_hde_st
            }
            break;
            default:
                break;
        }
#endif
        ///u16MOD_CTRL9 [7] : 63[13]reg_vby1_pair_swap_osd
        ///u16MOD_CTRL9 [14:13] : 63[11:10]reg_vby1_pair_mirror2
        MOD_W2BYTEMSK(REG_MOD_BK00_63_L, (pstPanelInitData->u16MOD_CTRL9 & BIT(7))<<6, BIT(13));
        MOD_W2BYTEMSK(REG_MOD_BK00_63_L, (pstPanelInitData->u16MOD_CTRL9 & (BIT(14)|BIT(13)))>>3, BIT(11)|BIT(10));

        MHal_Output_Channel_Order(pInstance,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u8OutputOrderType,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder0_3,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder4_7,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder8_11,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder12_13);

        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16MOD_CTRLA & BIT(1))  // use Dual port to decide the Vx1 1 or 2 devision config
        {
            PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]Vx1 2 division\n", __FUNCTION__, __LINE__);
        }
        else
        {
            PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]Vx1 1 division\n", __FUNCTION__, __LINE__);
        }

        ////per RD's suggestion ---Start
        MOD_A_W2BYTE(REG_MOD_A_BK00_30_L, 0xFFFF);   //reg_gcr_pe_en_ch
        MOD_A_W2BYTE(REG_MOD_A_BK00_32_L, 0x0000);   //reg_gcr_en_rint_ch
        MOD_A_W2BYTE(REG_MOD_A_BK00_3C_L, 0xFFFF);   //reg_gcr_test_ch

        ////per RD's suggestion, for VBY1 channel skew
        MOD_A_W2BYTE(REG_MOD_A_BK00_02_L, 0x4444);   //Fine tune skew for channel 0~7
        MOD_A_W2BYTE(REG_MOD_A_BK00_03_L, 0x4444);   //Fine tune skew for channel 8~15

        /// reg_gcr_pe_adj ch0~ch15
        MOD_A_W2BYTE(REG_MOD_A_BK00_18_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_19_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1A_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1B_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1C_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1D_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1E_L,0x0202);
        MOD_A_W2BYTE(REG_MOD_A_BK00_1F_L,0x0202);

        MOD_W2BYTE(REG_MOD_BK00_67_L, 0x80E8); //[0]:reg_vby1_8v4o_mode

        //The threshold value be set too strict ( ori: MOD_5C =0x0 )
        //And this reg should be set before enable serializer function
        //[15]reg_sw_rptr_fix_en: pointer fix by sw mode enable
        //[14:12]reg_sw_wptr_check: sw mode to decision write point check point
        //[10:8]reg_sw_rptr_fix_ini: sw mode to decision read point initial value
        //[6:4]reg_sw_rptr_fix_hi_th: sw mode to decision read pointer hi boundary
        //[2:0]reg_sw_rptr_fix_lo_th: sw mode to decision read pointer low boundary
        MOD_W2BYTEMSK(REG_MOD_BK00_5C_L,0x8142,0xFFFF);
        MOD_W2BYTEMSK(REG_MOD_BK00_5E_L,0x8142,0xFFFF);

        MS_U8 u8TimeSlotValue = 0;
        if(_MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext) > 1)
        {
            u8TimeSlotValue = _MHal_PNL_GetLaneNum(pInstance,pstPanelInitData->eLPLL_Type_Ext) / MFT_P_NUM - 1;
        }
        else
        {
            u8TimeSlotValue = 0;
        }
        MOD_W2BYTEMSK(REG_MOD_BK00_27_L,u8TimeSlotValue << 8,0x3F00);//reg_mft_vld_wrap_fix_htt
    }
    else
    {
        PNL_DBG(PNL_DBGLEVEL_INIT, "\n[%s][%d]do not support type!!\n", __FUNCTION__, __LINE__);
        if(pstPanelInitData->eLPLL_Type == E_PNL_TYPE_LVDS)
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0x0FFF, 0xFFFF);//for power saving

            // LVDS case, set MFT 1 division mode to enable MOD H mirror.
            MOD_W2BYTEMSK(REG_MOD_BK00_53_L, BIT(14), BIT(14));                //[14]reg_vfde_mask
            MOD_W2BYTEMSK(REG_MOD_BK00_1B_L, 0, BIT(6));                       //[6]reg_mft_keep_vb_de_out
            MOD_W2BYTEMSK(REG_MOD_BK00_27_L, 0x0, 0xBF00);                     //[15]reg_mft_fix_htt_disable //[13:8]reg_mft_vld_wrap_fix_htt
            MOD_W2BYTEMSK(REG_MOD_BK00_2B_L, 0x2,0x0003);                      //[1]reg_mft_read_trig_sel [0]reg_mft_ctrl_regen_mode


            if(pstPanelInitData->u16Width == PNL_FHD_WIDTH)
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x0005, 0x0007);                  //[2:0]reg_mft_mode : 1 division
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0780, 0xFFFF);                  //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0780, 0xFFFF);                  //[12:0]reg_div_len
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L, 0x8B,0xFFFF);                     //[15:0] reg_read_hde_st = Fixed H-Back-porch length = H-Back/MOD_P/TS_num - 1  (TS_num = lane/MFT_P)
            }

            if(pstPanelInitData->u16Width == PNL_1366X768_WIDTH)
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0x000D, 0x000D);                  //[3]2p_to_1p mode [2:0]reg_mft_mode : 1 division
                MOD_W2BYTEMSK(REG_MOD_BK00_22_L, 0x0558, 0xFFFF);                  //[12:0]reg_hsize
                MOD_W2BYTEMSK(REG_MOD_BK00_26_L, 0x0558, 0xFFFF);                  //[12:0]reg_div_len
                MOD_W2BYTEMSK(REG_MOD_BK00_2D_L, BIT(15), BIT(15));                //reg_hde_end_truncate_1t
                MOD_W2BYTEMSK(REG_MOD_BK00_2C_L, 0x27,0xFFFF);                     //[15:0] reg_read_hde_st = Fixed H-Back-porch length = H-Back/MOD_P/TS_num - 1  (TS_num = lane/MFT_P)
            }
        }

        // non-VBY1 case
        MOD_W2BYTE(REG_MOD_BK00_49_L, pstPanelInitData->u16MOD_CTRL9); //{L_BK_MOD(0x49), 0x00}, // [7,6] : output formate selction 10: 8bit, 01: 6bit :other 10bit, bit shift
        MHal_Output_Channel_Order(pInstance,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u8OutputOrderType,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder0_3,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder4_7,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder8_11,
                                  pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputOrder12_13);
    }

    MOD_W2BYTE(REG_MOD_BK00_4A_L, pstPanelInitData->u16MOD_CTRLA);
    MOD_W2BYTE(REG_MOD_BK00_4B_L,  pstPanelInitData->u8MOD_CTRLB);  //[1:0]ti_bitmode 10:8bit  11:6bit  0x:10bit

    if(pstPanelInitData->eLPLL_Type == E_PNL_TYPE_TTL)
    {
        if(pstPanelInitData->u16MOD_CTRLA & BIT(4))
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L,BIT(12),BIT(12));
        }
        else
        {
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L,FALSE,BIT(12));
        }
    }

    #if 0 // no need this for Maxim 23x23 8V case
    //MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, (_u8PnlDiffSwingLevel << 1), 0xFE);       //differential output swing level
    if(((pstPanelInitData->eLPLL_Type_Ext>= E_PNL_LPLL_VBY1_10BIT_4LANE)&&
        (pstPanelInitData->eLPLL_Type_Ext<= E_PNL_LPLL_VBY1_8BIT_8LANE)))
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0xC000, 0xF000);       //bank selection for skew clock
    #endif

    //if(!MHal_PNL_MOD_Control_Out_Swing(_u8PnlDiffSwingLevel))
    //    printf(">>Swing Level setting error!!\n");
    if(pstPanelInitData->eLPLL_Type != E_PNL_TYPE_MINILVDS)
    {
        MOD_W2BYTEMSK(REG_MOD_A_BK00_70_L, 0x7, 0x07);
    }

    //// Patch for Vx1 and it should be control by panel ini
    if(  IsVBY1(pstPanelInitData->eLPLL_Type_Ext) )
    {
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_31_L, 0x0FFF, 0x3FFF);
    }
    else
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_31_L, pstPanelInitData->u16LVDSTxSwapValue);
    }

#if 0 // Skip power down for bring up
#ifdef CONFIG_MSTAR_SRAMPD
    // At initial step, if non TCON panel, enable OD/RGBW/M+/Demura/DGA_Gamma SRAM PD
    if((pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI34_8P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI28_8P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI34_6P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI28_6P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI34_2P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI34_4P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI28_2P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI28_4P)&&
       (pstPanelInitData->eLPLL_Type_Ext != E_PNL_LPLL_EPI28_12P))
    {
        if(pPNLInstancePrivate->u32DeviceID == 0)
        {
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, BIT(0), BIT(0));   //OD SRAM PD Enable  : SC_SPD_BK3F_10[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(8));        //OD Clock gate : ~SC_SPD_BK3F_50[8]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, BIT(15), BIT(15)); //OD Bypass Enable :SC_OD_BK16_6F[15]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_10_L, BIT(1), BIT(1));   // RGBW SRAM PD Enable : SC_SPD_BK3F_10[1]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(9));        // RGBW Clock Gate : ~SC_SPD_BK3F_50[9]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, BIT(12), BIT(12)); // RGBW bypass enable :SC_OD_BK16_6F[12]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(13));       // RGBW bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_17_L, BIT(0), BIT(0));   //M+ SRAM PD Enable  : SC_SPD_BK3F_17[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_50_L, 0, BIT(10));        //M+ Clock gate : ~SC_SPD_BK3F_50[10]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L,0 , BIT(12)); // M+ bypass enable :SC_OD_BK16_6F[12]
            //SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_6F_L, 0, BIT(13));       // M+ bypass enable : ~SC_OD_BK16_6F[13]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, BIT(4), BIT(4));   //Demura SRAM PD Enable  : SC_SPD_BK3F_0B[4]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, BIT(14), BIT(14));   //Demura Clock Gate  : SC_BK77_26[14]
            W2BYTEMSK(REG_RVD_45_L, BIT(0), BIT(0));                                              //Demura Clock gate : BK100A_CLKGEN2_45[0]
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_26_L, BIT(15), BIT(15));   //Demura Clock Gate  : SC_BK77_26[15]

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK3F_0B_L, BIT(5), BIT(5));   //DGA_GAMMA SRAM PD Enable  : SC_SPD_BK3F_0B[5]
        }
    }

    // At initial step, SRAM PD for LD
    // To avoid mantis 1082875: boot logo flash issue, remove  SRAM PD for LD from XC_init to PNL_init
    if(pPNLInstancePrivate->u32DeviceID == 0)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BKC9_65_L, (BIT(0)), 0x0001);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BKCE_01_L, (0x0000), 0x1000);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BKCE_37_L, (BIT(15)), 0x8000);
        //non-FO setting
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK3F_12_L, (BIT(0)| BIT(1)), 0x0003);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK3F_50_L, (0x0000), 0x3800);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK2E_37_L, (BIT(15)), 0x8000);
    }
#endif
#endif

    // TODO: move from MDrv_Scaler_Init(), need to double check!
    MOD_W2BYTEMSK(REG_MOD_BK00_53_L, BIT(0), BIT(0));


    //--------------------------------------------------------------
    //Depend On Bitmode to set Dither
    //--------------------------------------------------------------


    // always enable noise dither and disable TAILCUT
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, ((pstPanelInitData->u8PanelNoiseDith ? XC_PAFRC_DITH_NOISEDITH_EN : (1 - XC_PAFRC_DITH_NOISEDITH_EN)) <<3) , BIT(3));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, XC_PAFRC_DITH_TAILCUT_DISABLE, BIT(4));

    switch(pstPanelInitData->u8MOD_CTRLB & 0x03)//[1:0]ti_bitmode b'10:8bit  11:6bit  0x:10bit
    {
        case HAL_TI_6BIT_MODE:
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, BIT(0), BIT(0));
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, BIT(2), BIT(2));
            break;

        case HAL_TI_8BIT_MODE:
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, BIT(0), BIT(0));
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, 0x00, BIT(2));
            break;

        case HAL_TI_10BIT_MODE:
        default:
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, 0x00, BIT(0));
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_3F_L, 0x00, BIT(2));
            break;
    }


    //-----depend on bitmode to set Dither------------------------------
    MHal_PNL_SetOutputType(pInstance, pPNLResourcePrivate->sthalPNL._eDrvPnlInitOptions, pstPanelInitData->eLPLL_Type);     // TTL to Ursa

    //dual port lvds _start_//
    // output configure for 26 pair output 00: TTL, 01: LVDS/RSDS/mini-LVDS data differential pair, 10: mini-LVDS clock output, 11: RSDS clock output
    if (IsVBY1(pstPanelInitData->eLPLL_Type_Ext))
    {
        MS_U16 u16ChannelClk_En = BIT(0); // bit0 is en_clk
        MS_U16 u16OutputCFG0_7 = OUTPUT_CHANNEL_ALL, u16OutputCFG8_15 = OUTPUT_CHANNEL_ALL;
        if(pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type == LVDS_OUTPUT_User)
        {
            u16OutputCFG0_7 = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7;
            u16OutputCFG8_15 = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15;
        }

        if(u16OutputCFG0_7  & OUTPUT_CHANNEL_PART_A)
            u16ChannelClk_En |= BIT(1); //bit1: partA - ch0~3
        if(u16OutputCFG0_7  & OUTPUT_CHANNEL_PART_B)
            u16ChannelClk_En |= BIT(2); //bit2: partB - ch4~7
        if(u16OutputCFG8_15 & OUTPUT_CHANNEL_PART_C)
            u16ChannelClk_En |= BIT(3); //bit3: partC - ch8~10
        if(u16OutputCFG8_15 & OUTPUT_CHANNEL_PART_D)
            u16ChannelClk_En |= BIT(4); //bit4: partD - ch11~13

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, u16ChannelClk_En, 0x1F); //[4]ck_pd[3]ck_pc[2]ck_pb[1]ck_pa[0]en_ck   // original is MDrv_WriteByteMask(L_BK_MOD(0x77), 0x0F, BITMASK(7:2));
    }
    else
    {
        // LVDS
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x1F, 0x1F);    //[4]ck_pd[3]ck_pc[2]ck_pb[1]ck_pa[0]en_ck   // original is MDrv_WriteByteMask(L_BK_MOD(0x77), 0x0F, BITMASK(7:2));
    }
    //dual port lvds _end_//
    //MHal_PNL_Bringup(pInstance);

    MHal_PNL_MISC_Control(pInstance, pstPanelInitData->u32PNL_MISC);

    PNL_DBG(PNL_DBGLEVEL_INIT, "OutputType  = %x, eLPLL_Type = %x\n", pPNLResourcePrivate->sthalPNL._eDrvPnlInitOptions, pstPanelInitData->eLPLL_Type);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u32PNL_MISC  = %tx\n", (ptrdiff_t)pstPanelInitData->u32PNL_MISC);

}

//This function is only for haps
//For Haps Tgen can output 4K but MOD only support LVDS
//HFRC can trasfer 4K to FHD
void MHal_HFRC_Setting(void *pInstance, EN_HFRC_TIMING eHfrcTiming)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    switch(eHfrcTiming)
    {
        case E_HFRC_FHDtoFHD:
        {
            W2BYTE(0x337A02, 0x0898); // tg_htt      = 16'h898
            W2BYTE(0x337A04, 0x0000); // tg_hs_st    = 16'h000
            W2BYTE(0x337A06, 0x000A); // tg_hs_end   = 16'h00a
            W2BYTE(0x337A08, 0x0070); // tg_hfde_st  = 16'h070
            W2BYTE(0x337A0A, 0x07EF); // tg_hfde_end = 16'h7EF
            W2BYTE(0x337A0C, 0x0070); // tg_hde_st   = 16'h070
            W2BYTE(0x337A0E, 0x07EF); // tg_hde_end  = 16'h7EF
            W2BYTE(0x337A10, 0x0465); // tg_vtt      = 16'h465
            W2BYTE(0x337A12, 0x0000); // tg_vs_st    = 16'h000
            W2BYTE(0x337A14, 0x0006); // tg_vs_end   = 16'h006
            W2BYTE(0x337A16, 0x0009); // tg_vfde_st  = 16'h9
            W2BYTE(0x337A18, 0x0440); // tg_vfde_end = 16'h440
            W2BYTE(0x337A1A, 0x0009); // tg_vde_st   = 16'h9
            W2BYTE(0x337A1C, 0x0440); // tg_vde_end  = 16'h440
            W2BYTE(0x337A24, 0x0002); // tg_frc_vs_st   = 16'h002
            W2BYTE(0x337A26, 0x000A); // tg_frc_vs_end  = 16'h00a
            W2BYTE(0x337A64, 0x0800); // scmi_size    20'h 70800 (1280*720/2)
            W2BYTE(0x337A66, 0x0007); // scmi_size    20'h 70800 (1280*720/2)
            W2BYTE(0x337A68, 0x03C0); // lb_size      12'h 3c0   (960)
            W2BYTE(0x337A76, 0x0280); // scmi_hsize   12'h 280   (640)
            W2BYTE(0x337A78, 0x03C0); // ip_hsize     12'h 3c0   (960)
            W2BYTE(0x337A7A, 0x03C0); // op_hsize     12'h 3c0   (960)

            W2BYTE(0x337A32, 0x020B); // ip_r2y_srgb11 = 13'h 020b
            W2BYTE(0x337A34, 0x1E25); // ip_r2y_srgb12 = 13'h 1e25
            W2BYTE(0x337A36, 0x1FD0); // ip_r2y_srgb13 = 13'h 1fd0
            W2BYTE(0x337A38, 0x00DA); // ip_r2y_srgb21 = 13'h 00da
            W2BYTE(0x337A3A, 0x02DC); // ip_r2y_srgb22 = 13'h 02dc
            W2BYTE(0x337A3C, 0x004A); // ip_r2y_srgb23 = 13'h 004a
            W2BYTE(0x337A3E, 0x1F88); // ip_r2y_srgb31 = 13'h 1f88
            W2BYTE(0x337A40, 0x1E6D); // ip_r2y_srgb32 = 13'h 1e6d
            W2BYTE(0x337A42, 0x020B); // ip_r2y_srgb32 = 13'h 1e6d

            W2BYTE(0x337A4E, 0x0629); // ip_r2y_srgb11 = 13'h 0629
            W2BYTE(0x337A50, 0x0400); // ip_r2y_srgb12 = 13'h 0400
            W2BYTE(0x337A52, 0x0000); // ip_r2y_srgb13 = 13'h 0000
            W2BYTE(0x337A54, 0x1E2A); // ip_r2y_srgb21 = 13'h 1e2a
            W2BYTE(0x337A56, 0x0400); // ip_r2y_srgb22 = 13'h 0400
            W2BYTE(0x337A58, 0x1F45); // ip_r2y_srgb23 = 13'h 1f45
            W2BYTE(0x337A5A, 0x0000); // ip_r2y_srgb31 = 13'h 0000
            W2BYTE(0x337A5C, 0x0400); // ip_r2y_srgb32 = 13'h 0400
            W2BYTE(0x337A5E, 0x0744); // ip_r2y_srgb33 = 13'h 0744

            W2BYTE(0x337B02, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337B04, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337B06, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337B08, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337B0E, 0x0000); // reg_scale_factor_ho  = 24'h180000
            W2BYTE(0x337B10, 0x0118); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h180000
            W2BYTE(0x337B12, 0x0000); // reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337B14, 0x0118); // reg_scale_ve_en=1, reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337B16, 0x42C2); // op_mode
            W2BYTE(0x337B50, 0x8780); // reg_h_size_usr_md[15] = 1'd1 , 1'd0, reg_h_size[13:0] = 14'h780(1920)
            W2BYTE(0x337B52, 0x8438); // reg_v_size_usr_md[15] = 1'd1 , 1'd0, reg_v_size[13:0] = 14'h438(1080)
            W2BYTE(0x337B54, 0x0500); // reg_scl_h_size = 13'h0500 (1280)
            W2BYTE(0x337B56, 0x02D0); // reg_scl_h_size = 13'h0500 (1280)
            W2BYTE(0x337BC0, 0x0000); // reg_bypass = 0

            W2BYTE(0x337C02, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337C04, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337C06, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337C08, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337C0E, 0xAAAA); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h0AAAAA
            W2BYTE(0x337C10, 0x010A); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h0AAAAA
            W2BYTE(0x337C12, 0xAAAA); // reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337C14, 0x010A); // reg_scale_ve_en=1, reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337C16, 0x42C2); // op_mode
            W2BYTE(0x337C50, 0x8500); // reg_h_size_usr_md[15] = 1'd1 , 1'd0, reg_h_size[13:0] = 14'h500(1280)
            W2BYTE(0x337C52, 0x82D0); // reg_v_size_usr_md[15] = 1'd1 , 1'd0, reg_v_size[13:0] = 14'h2d0(720)
            W2BYTE(0x337C54, 0x0780); // reg_scl_h_size = 13'h0780 (1920)
            W2BYTE(0x337C56, 0x0438); // reg_scl_v_size = 13'h0438 (1080)
            W2BYTE(0x337CC0, 0x0000); // reg_bypass = 0
            break;
        }
        case E_HFRC_4KtoFHD:
        {
            W2BYTE(0x337A02, 0x0898); // tg_htt      = 16'h898
            W2BYTE(0x337A04, 0x0000); // tg_hs_st    = 16'h000
            W2BYTE(0x337A06, 0x000A); // tg_hs_end   = 16'h00a
            W2BYTE(0x337A08, 0x0070); // tg_hfde_st  = 16'h070
            W2BYTE(0x337A0A, 0x07EF); // tg_hfde_end = 16'h7EF
            W2BYTE(0x337A0C, 0x0070); // tg_hde_st   = 16'h070
            W2BYTE(0x337A0E, 0x07EF); // tg_hde_end  = 16'h7EF
            W2BYTE(0x337A10, 0x0465); // tg_vtt      = 16'h465
            W2BYTE(0x337A12, 0x0000); // tg_vs_st    = 16'h000
            W2BYTE(0x337A14, 0x0006); // tg_vs_end   = 16'h006
            W2BYTE(0x337A16, 0x0009); // tg_vfde_st  = 16'h9
            W2BYTE(0x337A18, 0x0440); // tg_vfde_end = 16'h440
            W2BYTE(0x337A1A, 0x0009); // tg_vde_st   = 16'h9
            W2BYTE(0x337A1C, 0x0440); // tg_vde_end  = 16'h440
            W2BYTE(0x337A24, 0x0002); // tg_frc_vs_st   = 16'h002
            W2BYTE(0x337A26, 0x000A); // tg_frc_vs_end  = 16'h00a
            W2BYTE(0x337A64, 0x0800); // scmi_size    20'h 70800 (1280*720/2)
            W2BYTE(0x337A66, 0x0007); // scmi_size    20'h 70800 (1280*720/2)
            W2BYTE(0x337A68, 0x03C0); // lb_size      12'h 3c0   (960)
            W2BYTE(0x337A76, 0x0280); // scmi_hsize   12'h 280   (640)
            W2BYTE(0x337A78, 0x0780); // ip_hsize     12'h 3c0   (960)
            W2BYTE(0x337A7A, 0x03C0); // op_hsize     12'h 3c0   (960)

            W2BYTE(0x337A32, 0x020B); // ip_r2y_srgb11 = 13'h 020b
            W2BYTE(0x337A34, 0x1E25); // ip_r2y_srgb12 = 13'h 1e25
            W2BYTE(0x337A36, 0x1FD0); // ip_r2y_srgb13 = 13'h 1fd0
            W2BYTE(0x337A38, 0x00DA); // ip_r2y_srgb21 = 13'h 00da
            W2BYTE(0x337A3A, 0x02DC); // ip_r2y_srgb22 = 13'h 02dc
            W2BYTE(0x337A3C, 0x004A); // ip_r2y_srgb23 = 13'h 004a
            W2BYTE(0x337A3E, 0x1F88); // ip_r2y_srgb31 = 13'h 1f88
            W2BYTE(0x337A40, 0x1E6D); // ip_r2y_srgb32 = 13'h 1e6d
            W2BYTE(0x337A42, 0x020B); // ip_r2y_srgb32 = 13'h 1e6d

            W2BYTE(0x337A4E, 0x0629); // ip_r2y_srgb11 = 13'h 0629
            W2BYTE(0x337A50, 0x0400); // ip_r2y_srgb12 = 13'h 0400
            W2BYTE(0x337A52, 0x0000); // ip_r2y_srgb13 = 13'h 0000
            W2BYTE(0x337A54, 0x1E2A); // ip_r2y_srgb21 = 13'h 1e2a
            W2BYTE(0x337A56, 0x0400); // ip_r2y_srgb22 = 13'h 0400
            W2BYTE(0x337A58, 0x1F45); // ip_r2y_srgb23 = 13'h 1f45
            W2BYTE(0x337A5A, 0x0000); // ip_r2y_srgb31 = 13'h 0000
            W2BYTE(0x337A5C, 0x0400); // ip_r2y_srgb32 = 13'h 0400
            W2BYTE(0x337A5E, 0x0744); // ip_r2y_srgb33 = 13'h 0744

            W2BYTE(0x337B02, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337B04, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337B06, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337B08, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337B0E, 0x0000); // reg_scale_factor_ho  = 24'h180000
            W2BYTE(0x337B10, 0x0130); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h180000
            W2BYTE(0x337B12, 0x0000); // reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337B14, 0x0130); // reg_scale_ve_en=1, reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337B16, 0x42C2); // op_mode
            W2BYTE(0x337B50, 0x8F00); // reg_h_size_usr_md[15] = 1'd1 , 1'd0, reg_h_size[13:0] = 14'h780(1920)
            W2BYTE(0x337B52, 0x8870); // reg_v_size_usr_md[15] = 1'd1 , 1'd0, reg_v_size[13:0] = 14'h438(1080)
            W2BYTE(0x337B54, 0x0500); // reg_scl_h_size = 13'h0500 (1280)
            W2BYTE(0x337B56, 0x02D0); // reg_scl_h_size = 13'h0500 (1280)
            W2BYTE(0x337BC0, 0x0000); // reg_bypass = 0


            W2BYTE(0x337C02, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337C04, 0x0000); // reg_ini_factor_ho = 24'h000000
            W2BYTE(0x337C06, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337C08, 0x0000); // reg_ini_factor_ve = 24'h000000
            W2BYTE(0x337C0E, 0xAAAA); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h0AAAAA
            W2BYTE(0x337C10, 0x010A); // reg_scale_ho_en=1, reg_scale_factor_ho  = 24'h0AAAAA
            W2BYTE(0x337C12, 0xAAAA); // reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337C14, 0x010A); // reg_scale_ve_en=1, reg_scale_factor_ve  = 24'h180000
            W2BYTE(0x337C16, 0x42C2); // op_mode
            W2BYTE(0x337C50, 0x8500); // reg_h_size_usr_md[15] = 1'd1 , 1'd0, reg_h_size[13:0] = 14'h500(1280)
            W2BYTE(0x337C52, 0x82D0); // reg_v_size_usr_md[15] = 1'd1 , 1'd0, reg_v_size[13:0] = 14'h2d0(720)
            W2BYTE(0x337C54, 0x0780); // reg_scl_h_size = 13'h0780 (1920)
            W2BYTE(0x337C56, 0x0438); // reg_scl_v_size = 13'h0438 (1080)
            W2BYTE(0x337CC0, 0x0000); // reg_bypass = 0
            break;
        }
        default:
        {
            break;
        }
    }
}

//this function is only for Haps Patch
//For Haps only allow LVDS out, so we need hard code LVDS output to let 4K tgen && FHD LVDS out
void MHal_PNL_Haps_Patch(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    //haps Settings
    MS_U8 u8i = 0;
    //this register need to write repeatly
    for(u8i = 0;u8i < 100 ;u8i++)
    {
         MOD_A_W2BYTE(REG_MOD_A_BK00_78_L, 0x0010);
         MsOS_DelayTask(10);
         MOD_A_W2BYTE(REG_MOD_A_BK00_78_L, 0x0000);
         MsOS_DelayTask(10);
    }
    //MOD setting
    MOD_W2BYTE(REG_MOD_BK00_2F_L, 0x0001);
    MOD_W2BYTE(REG_MOD_BK00_08_L, 0x3210);
    MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7654);
    MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
    MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);

    if  (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)
    {
        MHal_HFRC_Setting(pInstance, E_HFRC_FHDtoFHD);
    }
    else
    {
        MHal_HFRC_Setting(pInstance, E_HFRC_4KtoFHD);
    }
}
void MHal_PNL_DumpMODReg(void *pInstance, MS_U32 u32Addr, MS_U16 u16Value, MS_BOOL bHiByte, MS_U16 u16Mask)
{
    if (bHiByte)
    {
        MOD_W2BYTEMSK(u32Addr, (u16Value << 8), (u16Mask << 8));
    }
    else
    {
        MOD_W2BYTEMSK(u32Addr, u16Value, u16Mask);
    }
}

void MHal_MOD_Calibration_Init(void *pInstance, PNL_ModCali_InitData *pstModCaliInitData)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    // Setup the default swing level
    pPNLResourcePrivate->sthalPNL._u16PnlDefault_SwingLevel = pstModCaliInitData->u16ExpectSwingLevel;   //mv
#if 0
    // Pair setting
    // =========
    // Select calibration source pair, 00: ch2, 01: ch6, 10:ch8, 11:ch12
    //MOD_7D_L[3:2]
    // =========
    //in msModCurrentCalibration, it will transfer to the real data

    switch(pstModCaliInitData->u8ModCaliPairSel)
    {
        default:
        case 0:
        //ch 2
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_PAIR_SEL = 0x00; // ch2
        break;
        case 1:
        //ch 6
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_PAIR_SEL = 0x01; // ch6, calibration initialized value
        break;
        case 2:
        //ch 8
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_PAIR_SEL = 0x02;
        break;
        case 3:
        //ch 12
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_PAIR_SEL = 0x03;
        break;
    }
#endif
    // Target setting
    // =========
    // GCR_CAL_LEVEL[1:0] : REG_MOD_A_BK00_70_L =>
    // =========
    //in msModCurrentCalibration, it will transfer to the real data
    switch(pstModCaliInitData->u8ModCaliTarget)
    {
        default:
        case 0:
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET = 0;
        break;
        case 1:
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET = 1;
        break;
        case 2:
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET = 2;
        break;
        case 3:
            pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET = 3;
        break;
    }
    // Offset setting, for fine tune
    //_usMOD_CALI_OFFSET = pstModCaliInitData->s8ModCaliOffset;
    // _u8MOD_CALI_VALUE is a real value; the _u8MOD_CALI_VALUE is an idea value
    // Target value should be the same with _u8MOD_CALI_VALUE to be a default value
    pPNLResourcePrivate->sthalPNL._u8MOD_CALI_VALUE= pstModCaliInitData->u8ModCaliTarget;
    // PVDD setting
    pPNLResourcePrivate->sthalPNL._bPVDD_2V5 = pstModCaliInitData->bPVDD_2V5;

    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]\n", __FUNCTION__, __LINE__);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u16ExpectSwingLevel = %u\n", pstModCaliInitData->u16ExpectSwingLevel);
    PNL_DBG(PNL_DBGLEVEL_INIT, "u8ModCaliTarget     = %x\n", pstModCaliInitData->u8ModCaliTarget);
    PNL_DBG(PNL_DBGLEVEL_INIT, "_u8MOD_CALI_TARGET  = %x\n", pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET);
    PNL_DBG(PNL_DBGLEVEL_INIT, "_u8MOD_CALI_VALUE   = %x\n", pPNLResourcePrivate->sthalPNL._u8MOD_CALI_VALUE);
    PNL_DBG(PNL_DBGLEVEL_INIT, "bPVDD_2V5           = %x\n", pstModCaliInitData->bPVDD_2V5);

}

void MHal_BD_LVDS_Output_Type(void *pInstance, MS_U16 Type)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    if(Type == LVDS_DUAL_OUTPUT_SPECIAL )
    {
        pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Shift = LVDS_DUAL_OUTPUT_SPECIAL;
        pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type = 1;
    }
    else
    {
        pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type  = Type;
    }
    PNL_DBG(PNL_DBGLEVEL_INIT, "[%s][%d]\n", __FUNCTION__, __LINE__);
    PNL_DBG(PNL_DBGLEVEL_INIT, "_u8MOD_LVDS_Pair_Type = %u\n", pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type);

}

MS_BOOL msModCalDDAOUT(void)
{
   // W2BYTEMSK(BK_MOD(0x7D), ENABLE, 8:8);
   // MsOS_DelayTask(10);  //10ms
    return (MS_BOOL)((MOD_R2BYTEMSK(REG_MOD_A_BK00_74_L, BIT(8))) >> 8);
}

MS_U8 msModCurrentCalibration(void *pInstance)
{
#if MOD_CAL_TIMER
        MS_U32 delay_start_time;
        delay_start_time=MsOS_GetSystemTime();
#endif

#if (!ENABLE_Auto_ModCurrentCalibration)
        return 0x60;
#else
    MS_U8 u8cur_ibcal=0;
    MS_U16 u16reg_32da = 0, u16reg_32dc = 0;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));
    u16reg_32da = MOD_A_R2BYTE(REG_MOD_A_BK00_00_L);
    u16reg_32dc = MOD_A_R2BYTE(REG_MOD_A_BK00_01_L);

    PNL_DBG(PNL_DBGLEVEL_CALIBRATION, "[%s][%d]\n", __FUNCTION__, __LINE__);

    // (1) Set keep mode to auto write calibration result into register.
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, BIT(15), BIT(15));

    // (2) Set calibration step waiting time
    MOD_A_W2BYTE(REG_MOD_A_BK00_71_L, 0x0080); // (about 5us)
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, 0x0000, 0x00FF);

    // (3) Set calibration toggle time
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, 0x0500, 0x0F00);

    // (4) Select calibration level (LVDS is 250mV)
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_70_L, pPNLResourcePrivate->sthalPNL._u8MOD_CALI_TARGET, BIT(2)|BIT(1)|BIT(0));    // Select calibration target voltage, 00: 250mV, 01:350mV, 10: 300mV, 11: 200mV

    // (5) Enable Calibration mode
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_70_L, BIT(7), BIT(7));         // Enable calibration function

    // (6) Store output configuration value and Enable each pair test mode
    MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, 0xFFFF);
    MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0FFF);

    MS_U8 u8CheckTimes = 0;
    while(1)
    {
        // (7) Enable Hardware calibration
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_73_L, BIT(15), BIT(15));

        // (8) Wait 2ms
        MsOS_DelayTask(2);

        // (10) Disable Hardware calibration
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_73_L, 0x00, BIT(15));

        // (9)Check Finish and Fail flag bit
        //BK111E, 0x73[14], Finish flag=1
        //BK111E, 0x73[13], Fail flag=0
        if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_73_L, 0x6000) == 0x4000)
        {
            //printf("\033[0;31m [%s][%d] cal ok, break  \033[0m\n", __FUNCTION__, __LINE__);
            break;
        }
        else
        {
            u8CheckTimes ++;
            //printf("\033[0;31m [%s][%d] cal ng, u8CheckTimes: %d  \033[0m\n", __FUNCTION__, __LINE__, u8CheckTimes);
        }

        if (u8CheckTimes > 3)
        {
            // (13) If 3 times all fail, set all pair to nominal value by disable keep mode
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_72_L, 0x00, BIT(15));
            //printf("\033[0;31m [%s][%d] If 3 times all fail, set all pair to nominal value by disable keep mode  \033[0m\n", __FUNCTION__, __LINE__);
            break;
        }
    }

    if (u8CheckTimes <=3)
    {
         PNL_DBG(PNL_DBGLEVEL_CALIBRATION, "\r\n----- Calibration ok \n");
    }
    else
    {
        PNL_DBG(PNL_DBGLEVEL_CALIBRATION, "\r\n----- Calibration fail: 0x%x \n", MOD_R2BYTEMSK(REG_MOD_BK00_3D_L, 0x6000));
    }

    // Wait 2ms to make sure HW auto write calibration result into register
    MsOS_DelayTask(2);

    // (14) Restore each pair output configuration
    MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, u16reg_32da);
    MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, u16reg_32dc);

    // (15) Disable calibration mode
    MOD_W2BYTEMSK(REG_MOD_A_BK00_70_L, 0x00, BIT(7));         // Disable calibration function

    // With HW calibration mode, HW would cal for each channel, and each channel would get different value
    // Return channel 2 vaule
    u8cur_ibcal = MOD_A_R2BYTEMSK(REG_MOD_A_BK00_09_L, 0x007F); // return ch2 calibration result

#if MOD_CAL_TIMER
    PNL_DBG(PNL_DBGLEVEL_CALIBRATION, "[%s] takes %ld ms\n", __FUNCTION__, (MsOS_GetSystemTime()-delay_start_time));
#endif
    PNL_DBG(PNL_DBGLEVEL_CALIBRATION, "\r\n Calibration result= %x\n", u8cur_ibcal);

    return (u8cur_ibcal&0x7F);//MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0D_L, 0x003F);
#endif
}

PNL_Result MHal_PNL_MOD_Calibration(void *pInstance)
{
    MS_U8 u8Cab;
    MS_U8 u8BackUSBPwrStatus;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    u8BackUSBPwrStatus = R2BYTEMSK(L_BK_UTMI1(0x04), BIT(7));

    W2BYTEMSK(L_BK_UTMI1(0x04), 0x00, BIT(7));

    u8Cab = msModCurrentCalibration(pInstance);

    W2BYTEMSK(L_BK_UTMI1(0x04), u8BackUSBPwrStatus, BIT(7));

    if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type !=E_PNL_TYPE_MINILVDS)
        MOD_W2BYTEMSK(REG_MOD_A_BK00_70_L, u8Cab, 0x07);

    return E_PNL_OK;

}

void MHal_PNL_PowerDownLPLL(void *pInstance, MS_BOOL bEnable)
{
    if(bEnable)
    {
        W2BYTEMSK(L_BK_LPLL(0x03), BIT(5), BIT(5));
    }
    else
    {
        W2BYTEMSK(L_BK_LPLL(0x03), FALSE, BIT(5));
    }
}

PNL_Result MHal_PNL_En(void *pInstance, MS_BOOL bPanelOn)
{
    MS_U8 u8Cab;
    MS_U8 u8BackUSBPwrStatus;
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "[%s][%d]\n", __FUNCTION__, __LINE__);

    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u32PNL_MISC = %tx\n", (ptrdiff_t)pPNLResourcePrivate->stdrvPNL._stPnlInitData.u32PNL_MISC);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "bPanelOn = %x\n", bPanelOn);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "eLPLL_Type            = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "_u8MOD_LVDS_Pair_Type = %x\n", pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG0_7       = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG8_15      = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG16_21     = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);

    MS_U16 u16PortA = MOD_A_R2BYTE(REG_MOD_A_BK00_00_L);
    MS_U16 u16PortB = MOD_A_R2BYTE(REG_MOD_A_BK00_01_L);

    if((u16PortA!=0)||(u16PortB!=0))
    {
        MHal_BD_LVDS_Output_Type(pInstance, LVDS_OUTPUT_User);
    }

    if(u16PortA!=0)
        pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7   = MOD_A_R2BYTE(REG_MOD_A_BK00_00_L);
    if(u16PortB!=0)
        pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15  = MOD_A_R2BYTE(REG_MOD_A_BK00_01_L);

    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "==========================\n\n");
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG0_7       = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG8_15      = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15);
    PNL_DBG(PNL_DBGLEVEL_PANEL_EN, "u16OutputCFG16_21     = %x\n", pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);


    if(bPanelOn)
    {
        // The order is PanelVCC -> delay pnlGetOnTiming1() -> VOP -> MOD
        // VOP
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_46_L, 0x4000, HBMASK);

        // For Napoli compatible
        // need to wait 1ms to wait LDO stable before MOD power on
        MsOS_DelayTask(1);

        // turn on LPLL
        if(R2BYTEMSK(L_BK_LPLL(0x03),BIT(5))!= 0)
        {
            // turn on LPLL
            MHal_PNL_PowerDownLPLL(pInstance, FALSE);
        }

        // for LG panel black video
        if(pPNLResourcePrivate->stapiPNL.ePrevPowerState != E_POWER_RESUME)
            MHal_PNL_VBY1_Hardware_TrainingMode_En(pInstance, TRUE, DISABLE);

        // mod power on
        MHal_MOD_PowerOn(pInstance
                        , ENABLE
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type
                        , pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);

        if(_MHal_PNL_CheckMODCalibrationEn(pInstance) == TRUE)
        {

            u8BackUSBPwrStatus = R2BYTEMSK(L_BK_UTMI1(0x04), BIT(7));

            W2BYTEMSK(L_BK_UTMI1(0x04), 0x00, BIT(7));

            u8Cab = msModCurrentCalibration(pInstance);

            W2BYTEMSK(L_BK_UTMI1(0x04), u8BackUSBPwrStatus, BIT(7));

        }


    }
    else
    {
        // The order is LPLL -> MOD -> VOP -> delay for MOD power off -> turn off VCC

        // LPLL
        // MHal_PNL_PowerDownLPLL(TRUE); //Remove to keep op vsync if panel off

        MHal_MOD_PowerOn(pInstance
                        , DISABLE
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type
                        , pPNLResourcePrivate->sthalPNL._u8MOD_LVDS_Pair_Type
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG0_7
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG8_15
                        , pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16OutputCFG16_21);
        // VOP
        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS ||
            pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_DAC_I ||
            pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_DAC_P)//(bIsLVDS)
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_46_L, 0xFF, LBMASK);
        else
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_46_L, 0x00, 0xFF);
    }

    return E_PNL_OK;
}

void MHal_PNL_SetOutputPattern(void *pInstance, MS_BOOL bEnable, MS_U16 u16Red , MS_U16 u16Green, MS_U16 u16Blue)
{
    if (bEnable)
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_02_L, u16Red , 0x03FF);
        MOD_W2BYTEMSK(REG_MOD_BK00_03_L, u16Green , 0x03FF);
        MOD_W2BYTEMSK(REG_MOD_BK00_04_L, u16Blue , 0x03FF);
        MsOS_DelayTask(10);
        MOD_W2BYTEMSK(REG_MOD_BK00_01_L, BIT(15) , BIT(15));
    }
    else
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_01_L, DISABLE , BIT(15));
    }

}

void MHal_PNL_Switch_LPLL_SubBank(void *pInstance, MS_U16 u16Bank)
{
    UNUSED(u16Bank);
}

void MHal_PNL_Switch_TCON_SubBank(void *pInstance, MS_U16 u16Bank)
{
    W2BYTEMSK(L_BK_TCON(0x00), u16Bank&0xff, 0xFF);
}

MS_U16 MHal_PNL_Read_TCON_SubBank(void *pInstance)
{
    return (MS_U16)R2BYTEMSK(L_BK_TCON(0x00),0xFF);
}

MS_BOOL MHal_PNL_Is_VBY1_Locked(void *pInstance)
{
    if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_47_L, 0x0300) == 0x00)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

MS_BOOL MHal_PNL_Is_VBY1_LockN_Locked(void *pInstance)
{
    if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_47_L, 0x0100) == 0x00)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

MS_BOOL MHal_PNL_VBY1_Handshake(void *pInstance)
{
    MS_BOOL bIsLock = FALSE;

    if (MHal_PNL_Is_VBY1_Locked(pInstance) == FALSE)
    {
        MS_U16 u16CheckTimes = 0;
        //MS_U16 u16DeboundTimes = 0;

        // need to toggle vby1 packer  process start first
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0x00, BIT(11));
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, BIT(11), BIT(11));

        MOD_W2BYTE(REG_MOD_BK00_60_L, 0x0F56); // set reg. initial value
        MOD_W2BYTEMSK(REG_MOD_BK00_60_L, 0xD6, 0x00FF); // after power on go to stand-by
        MOD_W2BYTEMSK(REG_MOD_BK00_60_L, 0x96, 0x00FF); // connection is established, go to Acquisition
        MOD_W2BYTEMSK(REG_MOD_BK00_60_L, 0xB6, 0x00FF); // when internal clock is stable, got to CDR training
        MOD_W2BYTEMSK(REG_MOD_BK00_60_L, 0xBE, 0x00FF); // enable encoder for DC blance

        while(u16CheckTimes < 10)
        {
#if 0
            u16DeboundTimes = 2;
            while ((!MHal_PNL_Is_VBY1_LockN_Locked()) && (u16DeboundTimes --))
            {
                MsOS_DelayTask(1); // can't remove
            }
#endif
            if(MHal_PNL_Is_VBY1_LockN_Locked(pInstance))
            {
                //-------------------------------------------------------------------
                // step1. Toggle clock when training
                MOD_W2BYTE(REG_MOD_BK00_60_L, 0x0FAE);
                //--------------------------------------------------------------------
                bIsLock = TRUE;
                // pass 2 times debound to make sure VBY1 is locked
                break;
            }

            u16CheckTimes++;
            MsOS_DelayTaskUs(40);
        }

        if(bIsLock)
            {
            // step3. Disable HW check when lock done, Enable when loss lock
            //MOD_W2BYTEMSK(REG_MOD_BK00_33_L, 0x00, BIT15);

            /// Add the delay to increase time to send
            //MDrv_TIMER_Delayms(10);
        }
    }
    else
    {
        if(MOD_R2BYTEMSK(REG_MOD_BK00_60_L, 0x0FFF) != 0x0FAE)
        {
            MOD_W2BYTE(REG_MOD_BK00_60_L, 0x0FAE);
        }
        bIsLock = TRUE;
    }

    return bIsLock;
}

MS_BOOL MHal_PNL_Is_VBY1_OC_Locked(void *pInstance)
{
    if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_47_L, 0x0C00) == 0x00)  // MOD_BK00_56_L[11:10]        for OSD
            {
        return TRUE;
    }
    else
    {
        return FALSE;
            }
}

MS_BOOL MHal_PNL_Is_VBY1_OC_LockN_Locked(void *pInstance)
{
    if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_47_L, 0x0400) == 0x00)  // MOD_BK00_56_L[11:10]        for OSD
            {
        return TRUE;
    }
    else
    {
        return FALSE;
            }
}

MS_BOOL MHal_PNL_VBY1_OC_Handshake(void *pInstance)
{
    MS_BOOL bIsLock = FALSE;

    if (MHal_PNL_Is_VBY1_OC_Locked(pInstance) == FALSE)
    {
        MS_U16 u16CheckTimes = 0;
//        MS_U16 u16DeboundTimes = 0;

        // need to toggle vby1 packer  process start first
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, 0x00, BIT(11));
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, BIT(11), BIT(11));


        MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0F56); // set reg. initial value
        MOD_W2BYTEMSK(REG_MOD_BK00_64_L, 0xD6, 0x00FF); // after power on go to stand-by
        MOD_W2BYTEMSK(REG_MOD_BK00_64_L, 0x96, 0x00FF); // connection is established, go to Acquisition
        MOD_W2BYTEMSK(REG_MOD_BK00_64_L, 0xB6, 0x00FF); // when internal clock is stable, got to CDR training
        MOD_W2BYTEMSK(REG_MOD_BK00_64_L, 0xBE, 0x00FF); // enable encoder for DC blance

        while(u16CheckTimes < 10)
        {
        #if 0
            u16DeboundTimes = 2;
            while ((!MHal_PNL_Is_VBY1_OC_LockN_Locked()) && (u16DeboundTimes --))
            {
            MsOS_DelayTask(1);
            }
        #endif
            if(MHal_PNL_Is_VBY1_OC_LockN_Locked(pInstance))
            {
                //-------------------------------------------------------------------
                // step1. Toggle clock when training

                MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0FAE);
                bIsLock = TRUE;
                // pass 2 times debound to make sure VBY1 is locked
                break;
            }

            u16CheckTimes++;
            MsOS_DelayTaskUs(40);
        }

        if(bIsLock)
        {
            // step3. Disable HW check when lock done, Enable when loss lock
//            MOD_W2BYTEMSK(REG_MOD_BK00_33_L, 0x00, BIT15);
        }
    }
    else
    {
        if(MOD_R2BYTEMSK(REG_MOD_BK00_64_L, 0x0FFF) != 0x0FAE)
        {
            MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0FAE);
        }
        bIsLock = TRUE;
    }

    return bIsLock;
}

MS_BOOL MHal_PNL_IsYUVOutput(void *pInstance)
{
   return FALSE;
}

MS_BOOL MHal_PNL_SetOutputInterlaceTiming(void *pInstance, MS_BOOL bEnable)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if (bEnable == TRUE)
    {
        //interlace output vtotal modify
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_21_L, BIT(9), BIT(9));

        // two different interlace information through channel A reserved bit
        MOD_W2BYTEMSK(REG_MOD_BK00_40_L, BIT(4) | BIT(7), BIT(4) | BIT(7));
        // two different interlace information through channel B reserved bit
        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, BIT(10)|BIT(11), BIT(10)|BIT(11));
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_21_L  , 0, BIT(9));
        MOD_W2BYTEMSK(REG_MOD_BK00_40_L, 0, BIT(4) | BIT(7));
        MOD_W2BYTEMSK(REG_MOD_BK00_42_L, 0, BIT(10)|BIT(11));
    }

    return TRUE;
}

MS_BOOL MHal_PNL_GetOutputInterlaceTiming(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_BOOL bIsInterlaceOutput = FALSE;
    //interlace output vtotal modify
    if (SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_21_L, BIT(9)) == BIT(9))
    {
        if ((MOD_R2BYTEMSK(REG_MOD_BK00_40_L, BIT(4) | BIT(7)) == (BIT(4) | BIT(7)))
            || (MOD_R2BYTEMSK(REG_MOD_BK00_42_L, BIT(10) | BIT(11)) == (BIT(10)|BIT(11))))
        {
            bIsInterlaceOutput = TRUE;
        }
    }
    else
    {
        bIsInterlaceOutput = FALSE;
    }
    return bIsInterlaceOutput;
}

////Ext LPLL setting
static void _MHal_PNL_Init_ExtLPLL(void *pInstance, PNL_TYPE eLPLL_Type,PNL_MODE eLPLL_Mode,MS_U64 ldHz)
{
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;

    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type,eLPLL_Mode,ldHz, E_PNL_LPLL_OSD);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        printf("Not Supported LPLL Type, skip LPLL Init\n");
        return;
    }

    _MHal_PNL_DumpLPLLTable(pInstance, u8SupportedLPLLLIndex, E_PNL_LPLL_OSD);
}

static MS_U8 _MHal_PNL_Get_ExtLPLL_LoopDIV(void *pInstance, MS_U8 u8LPLL_Mode, MS_U8 eLPLL_Type, MS_U64 ldHz)
{
    MS_U16 u16loop_div = 0;
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type,u8LPLL_Mode, ldHz, E_PNL_LPLL_OSD);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        u16loop_div = 0 ;
        printf("Invalid LPLL Index \n");
    }
    else
    {
        u16loop_div = u16LoopDiv[u8SupportedLPLLLIndex];
    }

    u16loop_div *= 2;
    return u16loop_div;
}

static MS_U8 _MHal_PNL_Get_ExtLPLL_LoopGain(void *pInstance, MS_U8 eLPLL_Mode, MS_U8 eLPLL_Type, MS_U64 ldHz)
{
    MS_U16 u16loop_gain = 0;
    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, eLPLL_Type, eLPLL_Mode,ldHz, E_PNL_LPLL_OSD);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        u16loop_gain = 0 ;
        printf("Invalid LPLL Index \n");
    }
    else
    {
        u16loop_gain = u16LoopGain[u8SupportedLPLLLIndex];
    }
    return u16loop_gain;
}

// Output Dclk
void MHal_PNL_CalExtLPLLSETbyDClk(void *pInstance, MS_U8 u8LPLL_Mode, MS_U8 u8LPLL_Type, MS_U64 ldHz)
{
//m7332 OSD setting combine to video table
#if 0
    MS_U64 u64LdPllSet = 0;
    MS_U64 u64DclkFactor = 0;
    MS_U32 u32Div = 0;
    // loop div and loop gain use default parameters to avoid dclk floating out of range and getting wrong value
    MS_U32 u32Factor = 10;

    E_PNL_SUPPORTED_LPLL_TYPE u8SupportedLPLLLIndex = E_PNL_SUPPORTED_LPLL_MAX;
    u8SupportedLPLLLIndex = _MHal_PNL_GetSupportedLPLLIndex(pInstance, u8LPLL_Type, u8LPLL_Mode,ldHz, E_PNL_LPLL_OSD);

    _MHal_PNL_Init_ExtLPLL(pInstance, u8LPLL_Type, u8LPLL_Mode, ldHz);

    if (u8SupportedLPLLLIndex == E_PNL_SUPPORTED_LPLL_MAX)
    {
        u64LdPllSet = 0;
        printf("Invalid LPLL Index \n");
    }
    else
    {
        //the first " *2 " is from  the dual mode
        u32Div=(MS_U32)(_MHal_PNL_Get_ExtLPLL_LoopDIV(pInstance, u8LPLL_Mode, u8LPLL_Type, ldHz));
        u64DclkFactor=((MS_U64)LVDS_OSD_MPLL_CLOCK_MHZ * (MS_U64)524288 * (MS_U64)_MHal_PNL_Get_ExtLPLL_LoopGain(pInstance, u8LPLL_Mode, u8LPLL_Type, ldHz));
        u64LdPllSet = (u64DclkFactor * 1000000 * u32Factor *2) + ((ldHz * u32Div) >> 1);
        do_div(u64LdPllSet, ldHz);
        if(u32Div == 0)
        {
            printf("u32Div = 0!!!\n");
        }
        else
        {
            do_div(u64LdPllSet, u32Div);
        }
    }
    W4BYTE(L_BK_LPLL(0x48), (MS_U32)u64LdPllSet);
    //printf("MHal_PNL_CalExtLPLLSETbyDClk u32KHz = %u, u32LpllSet = %x\n", ldHz, (MS_U32)u64LdPllSet);
#endif
}

void MHal_PNL_SetOSDCOutputType(void *pInstance, PNL_TYPE eLPLL_Type, E_PNL_OSDC_OUTPUT_FORMAT eOC_OutputFormat)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    // VBy1 co-registers
    if(  IsVBY1(eLPLL_Type) )
    {
        /*********Common Setting bk1523*********/
        //disp misc, bk1523 need to set before bk1032
        W2BYTEMSK(L_BK_DISP_MISC(0x21), 0x0, 0x3f); //reg_ckg_osd_odclk [5:0]
        W2BYTEMSK(L_BK_DISP_MISC(0x22), 0x800, 0x1f1f); //reg_ckg_osd_sr_rclk [4:0]  //reg_ckg_osd_sr_wclk [12:8]
        W2BYTEMSK(L_BK_DISP_MISC(0x30), BIT(2), BIT(2)); //reg_o_scalar_en
        W2BYTEMSK(L_BK_DISP_MISC(0x31), 0x0, 0x700); //reg_ckg_0_scalar[10:8]

        /*********Common Setting bk1032*********/
        MOD_W2BYTE(REG_MOD_BK00_42_L, 0x1008); //[3]enable osd lvds channel
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_69_L, BIT(15), BIT(15)); //[15]sw_rst
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0x4000, 0xF000);       //bank selection for skew clock

        MOD_W2BYTE(REG_MOD_BK00_65_L, 0x0f3f); //[15]all dk scr[13:8]aln_de_cnt [7:0] aln_pix_cnt
        MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0aae);
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, BIT(13)|BIT(6), (BIT(13)|BIT(12)|BIT(6))); //[13:12]reg_vby1_byte_mode_osd //[6]2ch_vby1_osd
        MOD_W2BYTEMSK(REG_MOD_BK00_2F_L, BIT(11), BIT(11)); //[11] Vby1 OSD
        MOD_W2BYTEMSK(REG_MOD_BK00_62_L, 0x00, BIT(14)); //[14]:reg_lockn_to_acq
        MOD_W2BYTEMSK(REG_MOD_BK00_45_L, 0x0000, 0x0003); //[1]:reg_vby1_vs_inv, [0]:reg_vby1_hs_inv
        MOD_W2BYTEMSK(REG_MOD_BK00_5B_L, BIT(7), BIT(7)); //[7]:reg_seri_osd_mod

        W2BYTE(REG_CLKGEN0_63_L,0x1410); //[11:8] clk_tx_mod_osd, [4:0] osd2mod
        W2BYTE(REG_RVD_09_L, 0x1800); //[12:8]ckg_vby1_fifo_osd [3:0]clk_vby1_fifo

        MOD_W2BYTEMSK(REG_MOD_BK00_67_L, 0x0001, 0x0001); //[0]:reg_vby1_8v4o_mode
    }

    if( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_4LANE)
    {
        //disp misc, bk1523 need to set before bk1032
        W2BYTE(L_BK_DISP_MISC(0x13), 0x108); //reg_mod_odclk_div_nm, [15:0]
        W2BYTE(L_BK_DISP_MISC(0x20), 0x104); //reg_mod_osd_odclk_div_nm [15:0]
        W2BYTE(L_BK_DISP_MISC(0x32), 0x0); //reg_scalar_v_div_nm, [15:0]
        W2BYTE(L_BK_DISP_MISC(0x33), 0x102); //reg_scalar_o_div_nm, [15:0]

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, BIT(14),BIT(14)|BIT(15)); //[15]enskew_path2[14]enclk_path2[4]ck_pd[3]ck_pc[2]ck_pb[1]ck_pa[0]en_ck
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0055); //[7:0]reg_output_conf[27:16]
        W2BYTE(L_CLKGEN0(0x5A),0x8800); //[15:12]reg_ckg_vby1_omode [11:8]reg_ckg_vby1_vmode

        MOD_W2BYTE(REG_MOD_BK00_42_L, 0x1000); //[12]sw_rst, [3]enable osd lvds channel
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x001F, 0x001F); //[4]ck_pd[3]ck_pc[2]ck_pb[1]ck_pa[0]en_ck
        MOD_A_W2BYTE(REG_MOD_A_BK00_58_L, 0x0400); //[3:0] reg_ckg_tx_mod
        MOD_A_W2BYTE(REG_MOD_A_BK00_59_L, 0x0044); //reg_ckg_dot

        MOD_W2BYTE(REG_MOD_BK00_71_L, 0xffff);
        MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0xf00, 0xff00); //[15:0]reg_seri_ch_cken_bitmap

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0xC000, 0xF000); //[15:12] reg_gcr_bank_clk_sel

        //-------------------------------------
        //## icon (Swing)
        MOD_A_W2BYTE(REG_MOD_A_BK00_0C_L, 0x7f7f);
        MOD_A_W2BYTE(REG_MOD_A_BK00_0D_L, 0x7f7f);
        MOD_A_W2BYTE(REG_MOD_A_BK00_0E_L, 0x0000);

        // vby1 osd 4 lane
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, BIT(5), BIT(5)); //[5]4ch_vby1_osd

        if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16MOD_CTRLA & BIT(1)) // 2 Divisoin
        {
            MOD_W2BYTE(REG_MOD_BK00_08_L, 0x6420);
            MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7531);
            MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
            MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
        }
        else
        {
            MOD_W2BYTE(REG_MOD_BK00_08_L, 0x3210);
            MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7654);
            MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
            MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
        }
    }
    else if( eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_2LANE)
    {
        //disp misc, bk1523 need to set before bk1032
        W2BYTE(L_BK_DISP_MISC(0x13), 0x104); //reg_mod_odclk_div_nm, [15:0]
        W2BYTE(L_BK_DISP_MISC(0x20), 0x102); //reg_mod_osd_odclk_div_nm [15:0]
        W2BYTE(L_BK_DISP_MISC(0x32), 0x102); //reg_scalar_v_div_nm, [15:0]
        W2BYTE(L_BK_DISP_MISC(0x33), 0x104); //reg_scalar_o_div_nm, [15:0]

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, BIT(14) ,BIT(14)|BIT(15));
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, 0x0005);
        W2BYTE(L_CLKGEN0(0x5A),0x9900); //[15:12]reg_ckg_vby1_omode [11:8]reg_ckg_vby1_vmode
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_38_L, 0x001B, 0x001F); //[4]ck_pd[3]ck_pc[2]ck_pb[1]ck_pa[0]en_ck

        MOD_W2BYTEMSK(REG_MOD_BK00_59_L, 0x300, 0xff00); //[15:0]reg_seri_ch_cken_bitmap
        //-------------------------------------
        //## icon (Swing)
        MOD_A_W2BYTE(REG_MOD_A_BK00_0C_L, 0x7f7f);
        MOD_A_W2BYTE(REG_MOD_A_BK00_0D_L, 0x0000);
        MOD_A_W2BYTE(REG_MOD_A_BK00_0E_L, 0x0000);

        //vby1 osd 2 lane
        MOD_W2BYTEMSK(REG_MOD_BK00_66_L, 0, BIT(5)); //[5]4ch_vby1_osd

        MOD_W2BYTE(REG_MOD_BK00_08_L, 0x3210);
        MOD_W2BYTE(REG_MOD_BK00_09_L, 0x7654);
        MOD_W2BYTE(REG_MOD_BK00_0A_L, 0xBA98);
        MOD_W2BYTE(REG_MOD_BK00_0B_L, 0x0000);
    }

    // Control VBY1 output format and bit orders
    switch(eOC_OutputFormat)
    {
        case E_PNL_OSDC_OUTPUT_FORMAT_VBY1_ARGB1:
            MOD_W2BYTEMSK(REG_MOD_BK00_6A_L, 0, BIT(1));
            break;

        case E_PNL_OSDC_OUTPUT_FORMAT_VBY1_ARGB2:
            MOD_W2BYTEMSK(REG_MOD_BK00_6A_L, BIT(1), BIT(1));
            break;

        default:
            printf("OSDC output format uses default value\n");
            MOD_W2BYTEMSK(REG_MOD_BK00_6A_L, 0, BIT(1));
            break;
    }


}

MS_BOOL MHal_PNL_SetOSDSSC(void *pInstance, MS_U16 u16Fmodulation, MS_U16 u16Rdeviation, MS_BOOL bEnable)
{
    MS_U16 u16Span;
    MS_U16 u16Step;
    MS_U32 u32PLL_SET;/// = MDrv_Read3Byte(L_BK_LPLL(0x0F));

    MHal_PNL_Switch_LPLL_SubBank(pInstance, 0x00);
    u32PLL_SET = R4BYTE(L_BK_LPLL(0x48));
    // Set SPAN
    if(u16Fmodulation < 200 || u16Fmodulation > 400)
        u16Fmodulation = 300;
    u16Span =( ( (((MS_U32)LVDS_MPLL_CLOCK_MHZ*LVDS_SPAN_FACTOR ) / (u16Fmodulation) ) * 10000) / ((MS_U32)u32PLL_SET) ) ;

    // Set STEP
    if(u16Rdeviation > 300)
        u16Rdeviation = 300;
    u16Step = ((MS_U32)u32PLL_SET*u16Rdeviation) / ((MS_U32)u16Span*10000);

    W2BYTE(L_BK_LPLL(0x4E), u16Step & 0x0FFF);// LPLL_STEP
    W2BYTE(L_BK_LPLL(0x4F), u16Span & 0x3FFF);// LPLL_SPAN
    W2BYTEMSK((L_BK_LPLL(0x4E)), (bEnable << 15), BIT(15)); // Enable ssc


    return TRUE;
}

void MHal_PNL_SetOSDSSC_En(void *pInstance, MS_BOOL bEnable)
{
    //printf("bEnable = %d\n", bEnable);
    MHal_PNL_Switch_LPLL_SubBank(pInstance, 0x00);
    W2BYTEMSK((L_BK_LPLL(0x4E)), (bEnable << 15), BIT(15)); // Enable ssc
}

void MHal_PNL_Set_T3D_Setting(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_55_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width, 0x1FFF);//pixel width
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_66_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height, 0x1FFF);//reg_col_height
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_51_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width, 0x1FFF);//reg_ln_width
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_52_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height, 0x1FFF);//reg_col_height
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK62_61_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width, 0x3FFF);//reg_ln_width
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK62_62_L, pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height, 0x1FFF);//reg_col_height

    //per designer, should always enable t3d, since it will affect osd/video's pipeline
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_61_L, BIT(0), BIT(0));//Enable Depth Render, for osd pipe line adjustment
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK62_08_L, BIT(4), BIT(4));//mtv bypass mode
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK63_57_L,(BIT(0)|BIT(1)),(BIT(0)|BIT(1)));//T3D fix subde enable, fix for T3D/PIP conflict issue (bit 0)     Bug Fix miu eco (bit 1)

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK62_63_L, 0x00, BIT(0)); // default disable T3D SRAM
}

void MHal_PNL_Set_Device_Bank_Offset(void *pInstance)
{
    UNUSED(pInstance);
    memset(u32PNL_XCDeviceBankOffset, 0, sizeof(MS_U32)*E_PNL_DEVICE_ID_NUM);
    u32PNL_XCDeviceBankOffset[E_PNL_DEVICE_ID_0] = E_HALPNL_DEVICE0_XC_BANK_OFFSET; // Set SC0 reg bank offset
    u32PNL_XCDeviceBankOffset[E_PNL_DEVICE_ID_1] = E_HALPNL_DEVICE1_XC_BANK_OFFSET; // Set SC2 reg bank offset
}

void MHal_PNL_Init(void *pInstance)
{
    // Do nothing
    //UNUSED(pInstance);
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    // STGEN reset
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_50_L, (BIT(0)|BIT(3)), (BIT(0)|BIT(3)) );
#if defined(CONFIG_MBOOT)
#if (ENABLE_FIX_PP_DLY == TRUE)
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_52_L, 0, 0x3FFF);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_51_L, (2 * VSTART_OFFSET + SC0_PATH_DLY_FSC2X_V + 1), 0x1FFF);
#else
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_52_L, FRC_PIPE_DELAY_HCNT_FSC_FHD, 0x3FFF);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_51_L, FRC_PIPE_DELAY_VCNT_FSC_FHD, 0x1FFF);
#endif
#endif
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK68_20_L, BIT(5), BIT(5)); // reg_stgen_en: use simple tgen to trigger op/vip

#if(RGB_3D_LUT_BF_XVYCC_DEGAMMA == TRUE)
    //RGB 3D LUT position is before XVYCC degamma.
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK14_52_L, BIT(4), BIT(4));
#endif

    //enable odclk in pnl init
    W2BYTEMSK(REG_SCEX_0D52_L,BIT(4),BIT(4));
}

void MHal_PNL_Bringup(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    ///patch for bring up
    if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_TYPE_LVDS)
    {
    }
    else if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_8LANE)
    {
    }

}

MS_U16 MHal_PNL_GetPanelVStart(void)
{
    return 8;
}

MS_BOOL MHal_PNL_Check_VBY1_Handshake_Status(void *pInstance)
{
    if(MOD_R2BYTEMSK(REG_MOD_BK00_60_L, 0x0FFF) != 0xFAE)
    {
        //printf("VBY1 handshake return because the reg value is 0x%u, not 0xFAE.\n", MOD_R2BYTEMSK(REG_MOD_BK00_60_L, 0x0FFF));
        return FALSE;
    }
    else
    {
        //printf("VBY handshake check success.\n");
        return TRUE;
    }
}

void MHal_PNL_ChannelFIFOPointerADjust(void *pInstance)
{
    //NOT support
}

void MHal_PNL_VBY1_Hardware_TrainingMode_En(void *pInstance, MS_BOOL bIsVideoMode ,MS_BOOL bEnable)
{
    if(bIsVideoMode)
    {
        if(bEnable)
        {
            MOD_W2BYTE(REG_MOD_BK00_60_L, 0x0AAE);
        }
        else
        {
            MOD_W2BYTE(REG_MOD_BK00_60_L, 0x0AA6);
        }
    }
    else
    {
        if(bEnable)
        {
            MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0AAE);
        }
        else
        {
            MOD_W2BYTE(REG_MOD_BK00_64_L, 0x0AA6);
        }
    }
}

MS_BOOL MHal_PNL_VBY1_IsSupport_Hardware_TrainingMode(void *pInstance)
{
    #ifdef SUPPORT_VBY1_HWTRAINING_MODE
        return TRUE;
    #else
        return FALSE;
    #endif
}

void MHal_PNL_TCON_Patch(void)
{
//tcon apply new tcon.bin in maxim
//so change patch
//step 1: enable param.
//step 2: delay one vysnc
//step 3: disable param.

    W2BYTEMSK(REG_SC2_00_L, 0x0008, 0x000F);
    W2BYTEMSK(REG_SC2_03_L, 0x0002, 0x0002);
    MsOS_DelayTask(20);
    W2BYTEMSK(REG_SC2_03_L, 0x0000, 0x0002);
#if 0
    // MOD sw reset
    MOD_W2BYTE(REG_MOD_BK00_42_L,0x0000);
    MOD_W2BYTE(REG_MOD_BK00_42_L,0x1000);

    // Setting TCON signal through MOD PAD
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x1811);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x9811);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x0000);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x1912);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x9912);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x0000);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x1a13);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x9a13);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x0000);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x1b17);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x9b17);
    MOD_A_W2BYTE(REG_MOD_A_BK00_51_L,0x0000);
#endif

}

static MS_BOOL _Hal_MOD_External_eFuse(void)
{
#ifdef MOD_EFUSE_IN_MBOOT
    return TRUE;
#else
    return FALSE;
#endif
}

static void _MHal_PNL_RegSettingAfterLpllPowerOn(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if( IsVBY1(pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type) )
    {
        //Force SW fix trigger
        MOD_W2BYTEMSK(REG_MOD_BK00_5C_L, 0, BIT(14)); //[14]reg_sw_force_fix_trig
        MOD_W2BYTEMSK(REG_MOD_BK00_5C_L, BIT(14), BIT(14)); //[14]reg_sw_force_fix_trig
        MOD_W2BYTEMSK(REG_MOD_BK00_5C_L, 0, BIT(14)); //[14]reg_sw_force_fix_trig

        MsOS_DelayTask(1);
    }
}
void MHal_PNL_TCON_GPIOPatch(void *pInstance)
{
    W2BYTEMSK(L_BK_TCON(0x00),0x0A,0x00FF);
    W1BYTEMSK(L_BK_TCON(0x50),BIT(1),BIT(1));
    MsOS_DelayTask(30);
    W1BYTEMSK(L_BK_TCON(0x50),FALSE,BIT(1));
}

MS_BOOL MHal_PNL_VsyncShift(void)
{
    if((OVERDRIVE_VSCALING_PATCH == TRUE)&& (OVERDRIVE_ENABLE_VSCALING == TRUE))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

void MHal_PNL_TCON_OD_VScaling_Patch(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    MS_U16 u16VsyncStart = 0;
    MS_U16 u16VsyncEnd = 0;
    MS_U16 u16vsize = 0;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK16_6C_L,BIT(1),BIT(1));//Enable VScaling
    u16VsyncStart = stTCONPanelConfig.stPanelInfo.u32VsyncStart + 1;
    u16VsyncEnd= stTCONPanelConfig.stPanelInfo.u32VsyncEnd + 1;
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_02_L, u16VsyncStart, VOP_VSYNC_START_MASK);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK10_03_L, u16VsyncEnd, VOP_VSYNC_END_MASK);
    u16vsize = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height;
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_0F_L,u16vsize);
}

void MHal_MOD_PrePowerOn(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, 0x00 , BIT(0));
    MOD_A_W2BYTEMSK(REG_MOD_A_BK00_3A_L, BIT(8) , BIT(8));
}
MS_BOOL MHal_PNL_DumpPCID_TBL(void *pInstance, MS_U8* pu8RTbl,MS_U8* pu8GTbl,MS_U8* pu8BTbl,PNL_HAL_PCID_LUT_TYPE eLutType)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_BOOL bEnable = FALSE;
    MS_U16 u16CodeTableX = 0;
    MS_U16 u16CodeTableY = 0;
    MS_U16 u16Timeout = 0;

    if(pu8RTbl == NULL || pu8GTbl == NULL || pu8BTbl == NULL)
    {
        PNL_ASSERT(0, "[%s] Table error \n.", __FUNCTION__);
        return FALSE;
    }

    bEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,FALSE, BIT(0)); // PCID Disable
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,eLutType << 15, BIT(15)); //sel lut

    for(u16CodeTableY=0;u16CodeTableY<PCID_TABLE_ROW_9;u16CodeTableY++)
    {
        for(u16CodeTableX=0;u16CodeTableX<PCID_TABLE_COL_9;u16CodeTableX++)
        {
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L,pu8RTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2], 0xFF); //R
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L,pu8GTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2], 0xFF); //G
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L,pu8BTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2], 0xFF); //B
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L,u16CodeTableX+(u16CodeTableY*PCID_TABLE_COL_9), 0x7F); //sram address
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,TRIG_WRITE_SRAM|SELECT_SRAM1|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
           while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
           {
               MsOS_DelayTask(1);
               u16Timeout++;
           }
           u16Timeout = 0;
        }
    }
    u16CodeTableX = 0;
    u16CodeTableY = 0;
    for(u16CodeTableY=0;u16CodeTableY<PCID_TABLE_ROW_9;u16CodeTableY++)
    {
        for(u16CodeTableX=0;u16CodeTableX<PCID_TABLE_COL_8;u16CodeTableX++)
        {
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L,pu8RTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + 1], 0xFF); //R
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L,pu8GTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + 1], 0xFF); //G
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L,pu8BTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + 1], 0xFF); //B
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L,u16CodeTableX+(u16CodeTableY*PCID_TABLE_COL_8), 0x7F); //sram address
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,TRIG_WRITE_SRAM|SELECT_SRAM2|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
           while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
           {
               MsOS_DelayTask(1);
               u16Timeout++;
           }
           u16Timeout = 0;
        }
    }

    u16CodeTableX = 0;
    u16CodeTableY = 0;
    for(u16CodeTableY=0;u16CodeTableY<PCID_TABLE_ROW_8;u16CodeTableY++)
    {
        for(u16CodeTableX=0;u16CodeTableX<PCID_TABLE_COL_9;u16CodeTableX++)
        {
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L,pu8RTbl[u16CodeTableX * 2 + (u16CodeTableY) * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8)], 0xFF); //R
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L,pu8GTbl[u16CodeTableX * 2 + (u16CodeTableY) * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8)], 0xFF); //G
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L,pu8BTbl[u16CodeTableX * 2 + (u16CodeTableY) * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8)], 0xFF); //B
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L,u16CodeTableX+(u16CodeTableY*PCID_TABLE_COL_9), 0x7F); //sram address
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,TRIG_WRITE_SRAM|SELECT_SRAM3|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
           while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
           {
               MsOS_DelayTask(1);
               u16Timeout++;
           }
           u16Timeout = 0;
        }
    }

    u16CodeTableX = 0;
    u16CodeTableY = 0;
    for(u16CodeTableY=0;u16CodeTableY<PCID_TABLE_ROW_8;u16CodeTableY++)
    {
        for(u16CodeTableX=0;u16CodeTableX<PCID_TABLE_COL_8;u16CodeTableX++)
        {
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L,pu8RTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8) +1], 0xFF); //R
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L,pu8GTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8) +1], 0xFF); //G
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L,pu8BTbl[u16CodeTableX * 2 + u16CodeTableY * (PCID_TABLE_COL_9+PCID_TABLE_COL_8) * 2 + (PCID_TABLE_COL_9+PCID_TABLE_COL_8) +1], 0xFF); //B
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L,u16CodeTableX+(u16CodeTableY*PCID_TABLE_COL_8), 0x7F); //sram address
           SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,TRIG_WRITE_SRAM|SELECT_SRAM4|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
           while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
           {
               MsOS_DelayTask(1);
               u16Timeout++;
           }
           u16Timeout = 0;
        }
    }
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,0, BIT(15)|BIT(9)|BIT(8)|BIT(3)|BIT(2));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,bEnable, BIT(0)); // PCID Enable
    return TRUE;
}

void MHal_PNL_PCID_Init(void *pInstance,MS_U8* pu8Tbl,MS_BOOL bPixelOverdriveEn,MS_BOOL bXTREn)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_50_L,bPixelOverdriveEn | bXTREn << 4, BIT(0) | BIT(4));

    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_51_L,(pu8Tbl[0] | (pu8Tbl[1] << 8)));
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_52_L,(pu8Tbl[2] | (pu8Tbl[3] << 8)));
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_53_L,(pu8Tbl[4] | (pu8Tbl[5] << 8)));
    SC_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_54_L,(pu8Tbl[6] | (pu8Tbl[7] << 8)));
}
void MHal_PNL_PCID_Enable(void *pInstance, MS_BOOL bEnable)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    if (bEnable)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(0), BIT(0));
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, FALSE, BIT(0));
    }
}

MS_BOOL MHal_PNL_DumpLineOD_TBL(void *pInstance, MS_U8* pu8EvenRTbl, MS_U8* pu8EvenGTbl, MS_U8* pu8EvenBTbl, MS_U8* pu8OddRTbl, MS_U8* pu8OddGTbl, MS_U8* pu8OddBTbl, MS_U8 u8LutSramSeg)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_BOOL bPcidEnable = FALSE, bLineODEnable = FALSE;
    MS_U16 u16CodeTableX = 0;
    MS_U16 u16CodeTableY = 0;
    MS_U16 u16Timeout = 0, lutIndex = 0;

    if( PNL_RGB_LUT_CHECK(pu8EvenRTbl, pu8EvenGTbl, pu8EvenBTbl) || PNL_RGB_LUT_CHECK(pu8OddRTbl, pu8OddGTbl, pu8OddBTbl) )
    {
        PNL_ASSERT(0, "[%s] Table error \n.", __FUNCTION__);
        return FALSE;
    }

    bPcidEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L,FALSE, BIT(0)); // PCID Disable
    bLineODEnable = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK4A_60_L, BIT(0));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK4A_60_L, FALSE, BIT(0)); // LINE OD Disable
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L, u8LutSramSeg << 12, BIT(14)|BIT(13)|BIT(12)); //sel PCID lut sram segment

    //sram1
    for(u16CodeTableY=0;u16CodeTableY<LINE_OD_TABLE_ROW_10;u16CodeTableY++)
    {
        for(u16CodeTableX=0;u16CodeTableX<LINE_OD_TABLE_COL_10;u16CodeTableX++)
        {
            lutIndex = u16CodeTableX * 2 + u16CodeTableY * (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9) * 2;
            if( lutIndex >= LINE_OD_TABLE_SIZE_19X19 )
            {
                PNL_DBG(PNL_DBGLEVEL_INIT, "lutIndex error: %d\n", lutIndex);
                return FALSE;
            }
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8EvenRTbl[lutIndex], 0xFF); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8EvenGTbl[lutIndex], 0xFF); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8EvenBTbl[lutIndex], 0xFF); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8OddRTbl[lutIndex]<<8, 0xFF00); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8OddGTbl[lutIndex]<<8, 0xFF00); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8OddBTbl[lutIndex]<<8, 0xFF00); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L, u16CodeTableX+(u16CodeTableY*LINE_OD_TABLE_COL_10), 0x7F); //sram address
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, TRIG_WRITE_SRAM|SELECT_SRAM1|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
            {
               MsOS_DelayTask(1);
               u16Timeout++;
            }
            u16Timeout = 0;
        }
    }

    //sram2
    for(u16CodeTableY = 0; u16CodeTableY < LINE_OD_TABLE_ROW_10; u16CodeTableY++)
    {
        for(u16CodeTableX = 0; u16CodeTableX < LINE_OD_TABLE_COL_9; u16CodeTableX++)
        {
            lutIndex = u16CodeTableX * 2 + u16CodeTableY * (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9) * 2 + 1;
            if( lutIndex >= LINE_OD_TABLE_SIZE_19X19 )
            {
                PNL_DBG(PNL_DBGLEVEL_INIT, "lutIndex error: %d\n", lutIndex);
                return FALSE;
            }
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8EvenRTbl[lutIndex], 0xFF); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8EvenGTbl[lutIndex], 0xFF); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8EvenBTbl[lutIndex], 0xFF); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8OddRTbl[lutIndex]<<8, 0xFF00); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8OddGTbl[lutIndex]<<8, 0xFF00); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8OddBTbl[lutIndex]<<8, 0xFF00); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L, u16CodeTableX+(u16CodeTableY*LINE_OD_TABLE_COL_9), 0x7F); //sram address
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, TRIG_WRITE_SRAM|SELECT_SRAM2|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
            {
               MsOS_DelayTask(1);
               u16Timeout++;
            }
            u16Timeout = 0;
        }
    }

    //sram3
    for(u16CodeTableY = 0; u16CodeTableY < LINE_OD_TABLE_ROW_9; u16CodeTableY++)
    {
        for(u16CodeTableX = 0; u16CodeTableX < LINE_OD_TABLE_COL_10; u16CodeTableX++)
        {
            lutIndex = u16CodeTableX * 2 + (u16CodeTableY) * (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9) * 2 + (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9);
            if( lutIndex >= LINE_OD_TABLE_SIZE_19X19 )
            {
                PNL_DBG(PNL_DBGLEVEL_INIT, "lutIndex error: %d\n", lutIndex);
                return FALSE;
            }
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8EvenRTbl[lutIndex], 0xFF); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8EvenGTbl[lutIndex], 0xFF); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8EvenBTbl[lutIndex], 0xFF); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8OddRTbl[lutIndex]<<8, 0xFF00); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8OddGTbl[lutIndex]<<8, 0xFF00); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8OddBTbl[lutIndex]<<8, 0xFF00); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L, u16CodeTableX+(u16CodeTableY*LINE_OD_TABLE_COL_10), 0x7F); //sram address
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, TRIG_WRITE_SRAM|SELECT_SRAM3|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
            {
               MsOS_DelayTask(1);
               u16Timeout++;
            }
            u16Timeout = 0;
        }
    }

    //sram4
    for(u16CodeTableY = 0; u16CodeTableY < LINE_OD_TABLE_ROW_9; u16CodeTableY++)
    {
        for(u16CodeTableX = 0; u16CodeTableX < LINE_OD_TABLE_COL_9; u16CodeTableX++)
        {
            lutIndex = u16CodeTableX * 2 + u16CodeTableY * (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9) * 2 + (LINE_OD_TABLE_COL_10+LINE_OD_TABLE_COL_9) +1;
            if( lutIndex >= LINE_OD_TABLE_SIZE_19X19 )
            {
                PNL_DBG(PNL_DBGLEVEL_INIT, "lutIndex error: %d\n", lutIndex);
                return FALSE;
            }
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8EvenRTbl[lutIndex], 0xFF); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8EvenGTbl[lutIndex], 0xFF); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8EvenBTbl[lutIndex], 0xFF); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_02_L, pu8OddRTbl[lutIndex]<<8, 0xFF00); //R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_03_L, pu8OddGTbl[lutIndex]<<8, 0xFF00); //G
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_04_L, pu8OddBTbl[lutIndex]<<8, 0xFF00); //B

            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_01_L, u16CodeTableX+(u16CodeTableY*LINE_OD_TABLE_COL_9), 0x7F); //sram address
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, TRIG_WRITE_SRAM|SELECT_SRAM4|SELECT_ALL_CHANNEL_RGB, W_RGB_MASK); //sel RGB
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, BIT(13)) && (u16Timeout < MAX_DELAY_TIME))
            {
               MsOS_DelayTask(1);
               u16Timeout++;
            }
            u16Timeout = 0;
        }
    }

    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, 0, BIT(9)|BIT(8)|BIT(3)|BIT(2));
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK77_0F_L, bPcidEnable, BIT(0)); // PCID Enable
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK4A_60_L, bLineODEnable, BIT(0)); // LINE OD Enable

    return TRUE;
}

MS_BOOL MHal_PNL_DumpLineOD_Gain_TBL(void *pInstance, MS_U8* pu8ODGainTbl, MS_U16 u16GainTableSize)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_BOOL bGainEnable = FALSE;
    MS_U16 u16GainTableIndex = 0;
    MS_U16 u16HWGainSize = LINE_OD_HW_GAIN_SIZE_13X9;//hw size

    if( pu8ODGainTbl == NULL )
    {
        PNL_ASSERT(0, "[%s] GainTable error \n.", __FUNCTION__);
        return FALSE;
    }

    //Note:REG_SC_EXT_BK2D/332D/(Bank =104F Sub-Bank=2D)/0x302f00 0x2d

    bGainEnable = SC_EXT_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, BIT(0));

    if(u16GainTableSize==LINE_OD_GAIN_TABLE_SIZE_13X9)
    {

        for(u16GainTableIndex =0; u16GainTableIndex <LINE_OD_GAIN_TABLE_SIZE_13X9; u16GainTableIndex++) //fill the 13x9 data on 13x9 hw address
        {
            SC_EXT_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, 0x00);//gain disable
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_02_L, u16GainTableIndex, BITMASK(6:0));//[6:0]lineOD gain address
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_03_L, pu8ODGainTbl[u16GainTableIndex],BITMASK(7:0));//[7:0]lineOD write data3.5
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, BIT(3), BIT(3));//[3]wr enable
        }
    }
    else if(u16GainTableSize==LINE_OD_GAIN_TABLE_SIZE_4X4)
    {
        MS_U16 u16HWGainIndex = 0;
        for(u16GainTableIndex =0; u16GainTableIndex <LINE_OD_GAIN_TABLE_SIZE_4X4; u16GainTableIndex++)// fill the 4x4 data on 13x9 hw address(other ignore)
        {
            u16HWGainIndex = ((u16GainTableIndex/LINE_OD_GAIN_TABLE_COL_4)*LINE_OD_GAIN_TABLE_COL_13)+(u16GainTableIndex%LINE_OD_GAIN_TABLE_COL_4);
            SC_EXT_W2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, 0x00);//gain disable
            if(u16HWGainIndex<LINE_OD_HW_GAIN_SIZE_13X9)
            {
                SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_02_L, u16HWGainIndex, BITMASK(6:0));//[6:0]lineOD gain address
                SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_03_L, pu8ODGainTbl[u16GainTableIndex],BITMASK(7:0));//[7:0]lineOD write data3.5
            }
            SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, BIT(3), BIT(3));//[3]wr enable
        }
    }
    else
    {
        PNL_DBG(PNL_DBGLEVEL_INIT, "Table size=%d not correct\n",u16GainTableSize);
    }

    SC_EXT_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_EXT_BK2D_01_L, bGainEnable, BIT(0));//gain enable status recovery

    return TRUE;
}

MS_BOOL MHal_PNL_PolInvert(void* pInstance, MS_U8 u8MotionThreshold, MS_U8 u8OsdThreshold, MS_U8 u8VideoThreshold)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    MS_U16 u16PanelWidth, u16PanelHeight;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    u16PanelWidth  = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width;
    u16PanelHeight = pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height;

    //RGBW Panel Check
    if(R2BYTEMSK(0x331020, BIT(0)) != BIT(0))
    {
        return FALSE;
    }

    if((u16PanelWidth == 0) || (u16PanelHeight == 0))
    {
        return FALSE;
    }

    //MAIN WINDOW check DNR is open
    if(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK06_21_L, BIT(0)) != BIT(0))
    {
        return FALSE;
    }

    //OSD Window check threshold
    MS_U32 u32OsdPixel = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6D_L) & 0xfff;
    u32OsdPixel |= (SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_6C_L) & 0xfff) << 12;
    u32OsdPixel = u32OsdPixel*2;//for 2p mode
    MS_U32 u32PanelPixel = u16PanelWidth * u16PanelHeight;
    if((u32OsdPixel * POLINVERT_FULL_THRESHOLD) / u32PanelPixel > u8OsdThreshold)
    {
        return FALSE;
    }

    //Display window check threshold
    MS_U16 x = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_08_L);                   // Display H start
    MS_U16 u16VideoWidth  = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_09_L) - x + 1; // Display H end
    MS_U16 y = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_0A_L);                   // Display V start
    MS_U16 u16VideoHeight = SC_R2BYTE(pPNLInstancePrivate->u32DeviceID, REG_SC_BK10_0B_L) - y + 1; // Display V end
    MS_U32 u32VideoPixel = u16VideoWidth * u16VideoHeight;
    if((u32VideoPixel * POLINVERT_FULL_THRESHOLD) / u32PanelPixel < u8VideoThreshold)
    {
        return FALSE;
    }

    MS_BOOL bFSSEnable = R2BYTEMSK(REG_IPMUX_73_L, BIT(0));
    MS_U16 u16FSSSource = R2BYTEMSK(REG_IPMUX_71_L, BMASK(3:0));
    MS_U16 u16FSSBankOffset = (((FASTSOURCESWITCH_BANK_OFFSET + u16FSSSource - 1)<<8)|HBMASK);
    MS_U16 u16SourceHeight = 0;
    MS_U16 u16SourceWidth = 0;

    //Motion Report check threshold
    if(bFSSEnable)
    {
        u16SourceHeight  = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, (REG_SC_BK01_06_L+u16FSSBankOffset), 0x1FFF);
        u16SourceWidth  = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, (REG_SC_BK01_07_L+u16FSSBankOffset), 0x3FFF);
    }
    else
    {
        u16SourceHeight  = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK01_06_L, 0x1FFF);
        u16SourceWidth  = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK01_07_L, 0x3FFF);
    }
    if(u16SourceHeight == 0 || u16SourceWidth == 0)
    {
        return FALSE;
    }
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK0A_2D_L,0x01,0x0F);//set Motion threshold
    MS_U32 u32MotionPixel = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK0A_2E_L,0x7FFF) << 5;
    MS_U32 u32SourcePixel = u16SourceHeight * u16SourceWidth;
    if((u32MotionPixel * POLINVERT_FULL_THRESHOLD) / u32SourcePixel < u8MotionThreshold)
    {
        return FALSE;
    }

    //Read pol value, change subbank to 09
    W2BYTE(L_BK_TCON(0x00), 0x08);
    MS_U16 u16POLValue = R2BYTE(L_BK_TCON(0x40));
    if(u16POLValue == 0x6000)
    {
        W2BYTE(L_BK_TCON(0x40), 0x5000);
    }
    else
    {
        W2BYTE(L_BK_TCON(0x40), 0x6000);
    }
    W2BYTE(L_BK_TCON(0x00), 0x0A);
    W2BYTE(L_BK_TCON(0x59), 0x10);
    MsOS_DelayTask(20);
    W2BYTE(L_BK_TCON(0x59), 0x00);
    return TRUE;
}

void MHal_PNL_SetMirrorMode(void *pInstance, ST_PNL_MIRRORINFO *pstPnlMirrorInfo)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if (pstPnlMirrorInfo->bEnable == TRUE)
    {
        switch(pstPnlMirrorInfo->eMirrorType)
        {
            case E_PNL_MOD_H_MIRROR:
            {
                MS_U16 u16OutputOrder0_3 = 0x0000;
                MS_U16 u16OutputOrder4_7 = 0x0000;
                MS_U16 u16OutputOrder8_11 = 0x0000;
                MS_U16 u16OutputOrder12_15 = 0x0000;
                EN_PANEL_MFT_MODE  enPnlMftMode = MOD_R2BYTEMSK(REG_MOD_BK00_20_L, 0x0007);

                if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_LPLL_VBY1_10BIT_16LANE) || (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type == E_PNL_LPLL_VBY1_8BIT_16LANE))
                {
                    if((enPnlMftMode >= E_PANEL_MFT_MODE_2DIVISION) && (enPnlMftMode <= E_PANEL_MFT_MODE_8DIVISION)) //div>=2 need do swap
                    {
                        u16OutputOrder0_3 = MOD_R2BYTE(REG_MOD_BK00_08_L);
                        u16OutputOrder4_7 = MOD_R2BYTE(REG_MOD_BK00_09_L);
                        u16OutputOrder8_11 = MOD_R2BYTE(REG_MOD_BK00_0A_L);
                        u16OutputOrder12_15 = MOD_R2BYTE(REG_MOD_BK00_0B_L);

                        //do left and right swap
                        MOD_W2BYTE(REG_MOD_BK00_08_L, u16OutputOrder8_11);
                        MOD_W2BYTE(REG_MOD_BK00_09_L, u16OutputOrder12_15);
                        MOD_W2BYTE(REG_MOD_BK00_0A_L, u16OutputOrder0_3);
                        MOD_W2BYTE(REG_MOD_BK00_0B_L, u16OutputOrder4_7);
                    }
                }

                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, BIT(14), BIT(14));
                break;
            }
            default:
                break;
        }
    }
    else
    {
        switch(pstPnlMirrorInfo->eMirrorType)
        {
            case E_PNL_MOD_H_MIRROR:
            {
                MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0, BIT(14));
                break;
            }
            default:
                break;
        }
    }
}

void MHal_PNL_GetMirrorStatus(void *pInstance, ST_PNL_MIRRORINFO *pstPnlMirrorInfo)
{
    MS_BOOL bEnable = FALSE;
    switch(pstPnlMirrorInfo->eMirrorType)
    {
        case E_PNL_MOD_H_MIRROR:
        {
            if(MOD_R2BYTEMSK(REG_MOD_BK00_20_L,BIT(14)))
            {
                bEnable = TRUE;
            }
            break;
        }
        default:
            break;
    }

    pstPnlMirrorInfo->bEnable = bEnable;
}

void MHal_PNL_SetPanelGammaByPass(void *pInstance, MS_BOOL bEnable)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if (bEnable)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6C_L, BIT(15), BIT(15));
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6C_L, 0, BIT(15));
    }
}

MS_BOOL MHal_PNL_GetPanelGammaByPass(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    return (SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6C_L, BIT(15)) >> 15);
}

void MHal_PNL_PanelGamma_Enable(void *pInstance, MS_BOOL bEnable)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    if (bEnable)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, BIT(12), BIT(12));
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, FALSE, BIT(12));
    }
}

MS_BOOL MHal_PNL_PanelGamma_Set2LUTMode(void *pInstance, MS_PNL_PNLGAMMA_Config *pstPNLGammaConfig)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if (pstPNLGammaConfig == NULL)
    {
        return FALSE;
    }

    //mode select
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6A_L, (pstPNLGammaConfig->u8Gamma2LutMode & 0x03) << 14, BIT(15) | BIT(14));

    //enable
    if (pstPNLGammaConfig->bGamma2LutEnable)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_69_L, BIT(15), BIT(15));
    }
    else
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_69_L, FALSE, BIT(15));
    }

    return TRUE;
}

MS_BOOL MHal_PNL_PanelGamma_Get2LutEnable(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    return (SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_69_L, BIT(15)) >> 15);
}

//--------------------------------------------------------------------------------------------------------------------
// Panel Gamma Auto Download Format
// Lut0
// R/G/B/W cmd format in DRAM
//            |END | START |
//  R_LUT0     63      0
//  G_LUT0     127     64
//  B_LUT0     191     128
//  W_LUT0     255     192
//--------------------------------------------------------------------------------------------------------------------
#ifdef UFO_XC_AUTO_DOWNLOAD
MS_BOOL _MHal_PNL_ADL_SetPanelGammaTbl(void* pInstance, ST_PNL_PNLGAMMATBL *pstSetPNLGammaTbl)
{
    MS_U8 *pu8GammaTbl;
    MS_BOOL bRet = FALSE;
    MS_U32 u32index = 0;
    MS_BOOL b2LutEn = MHal_PNL_PanelGamma_Get2LutEnable(pInstance);
    MS_U32 u32PnlGammaADLCmdSize;
    MS_U8 *pu8R0, *pu8G0, *pu8B0, *pu8W0, *pu8R1, *pu8G1, *pu8B1;
    MS_BOOL bGammaByPass = MHal_PNL_GetPanelGammaByPass(pInstance);

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if( b2LutEn == FALSE )
    {
        u32PnlGammaADLCmdSize = PNLGAMMA_ADL_CMD_256_ENTRY;
    }
    else
    {
        u32PnlGammaADLCmdSize = PNLGAMMA_ADL_CMD_384_ENTRY;
    }

    PNL_MALLOC_MEM(pu8GammaTbl, PNLGAMMA_ADL_CMD_LENGTH*u32PnlGammaADLCmdSize, bRet);

    if( bRet == TRUE )
    {
        pu8R0 = pu8GammaTbl;
        if( b2LutEn == FALSE )
        {
            pu8G0 = pu8R0 + PANEL_GAMMA_CHANNEL_OFFSET;  // 64entry  * 32 byte =  2048
            pu8B0 = pu8G0 + PANEL_GAMMA_CHANNEL_OFFSET ;
            pu8W0 = pu8B0 + PANEL_GAMMA_CHANNEL_OFFSET;
        }
        else
        {
            pu8G0 = pu8R0 + PANEL_GAMMA_CHANNEL_OFFSET;  // 64entry  * 32 byte =  2048
            pu8B0 = pu8G0 + PANEL_GAMMA_CHANNEL_OFFSET ;

            pu8R1 = pu8B0 + PANEL_GAMMA_CHANNEL_OFFSET;
            pu8G1 = pu8R1 + PANEL_GAMMA_CHANNEL_OFFSET;
            pu8B1 = pu8G1 + PANEL_GAMMA_CHANNEL_OFFSET;
        }

        for(u32index = 0 ; u32index < PANEL_GAMMA_ENTRY; u32index++)
        {
            if( b2LutEn == FALSE )
            {
                //LUT 0
                *((MS_U16 *)pu8R0 + u32index) = pstSetPNLGammaTbl->pu16RChannel0[u32index];
                *((MS_U16 *)pu8G0 + u32index) = pstSetPNLGammaTbl->pu16GChannel0[u32index];
                *((MS_U16 *)pu8B0 + u32index) = pstSetPNLGammaTbl->pu16BChannel0[u32index];
                *((MS_U16 *)pu8W0 + u32index) = pstSetPNLGammaTbl->pu16WChannel0[u32index];
            }
            else
            {
                //LUT 0
                *((MS_U16 *)pu8R0 + u32index) = pstSetPNLGammaTbl->pu16RChannel0[u32index];
                *((MS_U16 *)pu8G0 + u32index) = pstSetPNLGammaTbl->pu16GChannel0[u32index];
                *((MS_U16 *)pu8B0 + u32index) = pstSetPNLGammaTbl->pu16BChannel0[u32index];

                //LUT 1
                *((MS_U16 *)pu8R1 + u32index) = pstSetPNLGammaTbl->pu16RChannel1[u32index];
                *((MS_U16 *)pu8G1 + u32index) = pstSetPNLGammaTbl->pu16GChannel1[u32index];
                *((MS_U16 *)pu8B1 + u32index) = pstSetPNLGammaTbl->pu16BChannel1[u32index];
            }
        }

        if (bGammaByPass == TRUE)
        {
            MHal_PNL_SetPanelGammaByPass(pInstance, DISABLE);
        }

        // Write ADL
        XC_AUTODOWNLOAD_FORMAT_INFO stAdlFomatInfo;
        memset(&stAdlFomatInfo, 0, sizeof(XC_AUTODOWNLOAD_FORMAT_INFO));
        stAdlFomatInfo.u32FormatInfo_Version = AUTODOWNLOAD_FORMAT_INFO_VERSION;
        stAdlFomatInfo.u16FormatInfo_Length = sizeof(XC_AUTODOWNLOAD_FORMAT_INFO);

        XC_AUTODOWNLOAD_DATA_INFO stDataInfo;
        memset(&stDataInfo, 0, sizeof(XC_AUTODOWNLOAD_DATA_INFO));
        stDataInfo.u32DataInfo_Version = AUTODOWNLOAD_DATA_INFO_VERSION;
        stDataInfo.u16DataInfo_Length = sizeof(XC_AUTODOWNLOAD_DATA_INFO);
        stDataInfo.pu8Data = pu8GammaTbl;
        stDataInfo.enClient = E_XC_AUTODOWNLOAD_CLIENT_PANEL_GAMMA;
        stDataInfo.pParam = &stAdlFomatInfo;
        stDataInfo.u32Size = PNLGAMMA_ADL_CMD_LENGTH*u32PnlGammaADLCmdSize;

        if(MApi_XC_AutoDownload_Write(&stDataInfo) != E_APIXC_RET_OK)
        {
            printf("\033[1;31mPanelgamma,Write ADL Fail!!!!!!!!!!\033[m\n");
            PNL_FREE_MEM(pu8GammaTbl, PNLGAMMA_ADL_CMD_LENGTH*u32PnlGammaADLCmdSize);
            return FALSE;
        }

        //Fire ADL
        if(MApi_XC_AutoDownload_Fire(E_XC_AUTODOWNLOAD_CLIENT_PANEL_GAMMA) != E_APIXC_RET_OK)
        {
            printf("\033[1;31mPanelgamma,Fire ADL Fail!!!!!!!!!!\033[m\n");
            PNL_FREE_MEM(pu8GammaTbl, PNLGAMMA_ADL_CMD_LENGTH*u32PnlGammaADLCmdSize);
            return FALSE;
        }

        PNL_FREE_MEM(pu8GammaTbl, PNLGAMMA_ADL_CMD_LENGTH*u32PnlGammaADLCmdSize);
    }
    else
    {
        return FALSE;
    }

    return TRUE;
}
#endif

MS_BOOL MHal_PNL_SetPanelGammaTbl(void* pInstance,ST_PNL_PNLGAMMATBL *pstSetPNLGammaTbl)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_U16 u16NumOfLevel =1024;
    MS_U16 u16Addr = 0;
    MS_U8 u8Channel =0;
    MS_U16 u16Timeout = 0;
    MS_U16 u16tmpReg = 0;

#ifdef UFO_XC_AUTO_DOWNLOAD
    E_PANEL_GAMMA_WRITE_TYPE enPnlGammaWType = E_PANEL_GAMMA_SUPPORT_ADL;

    if( enPnlGammaWType == E_PANEL_GAMMA_SUPPORT_ADL )
    {
        if( _MHal_PNL_ADL_SetPanelGammaTbl(pInstance, pstSetPNLGammaTbl) == TRUE )
        {
            //use ADL to set panel gamma table, use ML to set panel gamma enable
            //but ML trigger early ADL so that it shows garbage in AC on
            //so delay 1 vsync for ML
            MsOS_DelayTask(20);
            return TRUE;
        }
    }
#endif

    MS_BOOL b2LutEn = MHal_PNL_PanelGamma_Get2LutEnable(pInstance);
    MS_U8 u8MaxChannel = b2LutEn?PANEL_GAMMA_2LUT_MAX_CHANNEL:PANEL_GAMMA_1LUT_MAX_CHANNEL; // 1: RGBRGB, 0: RGBW
    MS_BOOL bGammaByPass = MHal_PNL_GetPanelGammaByPass(pInstance);

    if (bGammaByPass == TRUE)
    {
        MHal_PNL_SetPanelGammaByPass(pInstance, DISABLE);
    }

    // Store Panel Gamma enable flag
    u16tmpReg = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, BIT(12));
    //Disbale Panel Gamma
    if(u16tmpReg & BIT(12))
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,FALSE,BIT(12));
    }

    //Due to odclk and mcu clock not the same, the set value not stable.
    //Need add 4 NOP for burst wirte to guarantee that set value correct.
    //So use normal write to save time.
    for(u8Channel = 0; u8Channel < u8MaxChannel; u8Channel++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, BIT(0), BIT(0)); //BW enable

        if( b2LutEn == FALSE )
        {
            // set Write channel
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, u8Channel<<2, BIT(3)|BIT(2));

            //reg_gamma_lut_sel, select lut0 load table
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_69_L, 0, BIT(13));
        }
        else
        {
            // set Write channel
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6D_L, (u8Channel%3)<<2, BIT(3)|BIT(2));

            //reg_gamma_lut_sel, select lut0 or lut1 to load table
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_69_L, (u8Channel >= 3)?BIT(13):0,BIT(13));
        }

        for(u16Addr = 0; u16Addr < u16NumOfLevel; u16Addr++)
        {
            //2. set LUT_ADDR
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6C_L,u16Addr,0x3FF);
            //3. set write data
            if( b2LutEn == FALSE )
            {
                if(u8Channel == 0) //R channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6E_L,pstSetPNLGammaTbl->pu16RChannel0[u16Addr],0xFFF);
                }
                if(u8Channel == 1) //G channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6E_L,pstSetPNLGammaTbl->pu16GChannel0[u16Addr],0xFFF);
                }
                if(u8Channel == 2) //B channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6E_L,pstSetPNLGammaTbl->pu16BChannel0[u16Addr],0xFFF);
                }
                if(u8Channel == 3) //W channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6E_L,pstSetPNLGammaTbl->pu16WChannel0[u16Addr],0xFFF);
                }
            }
            else
            {
                //lut0
                if(u8Channel == 0) //R0 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16RChannel0[u16Addr], 0xFFF);
                }
                if(u8Channel == 1) //G0 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16GChannel0[u16Addr], 0xFFF);
                }
                if(u8Channel == 2) //B0 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16BChannel0[u16Addr], 0xFFF);
                }
                //lut1
                if(u8Channel == 3) //R1 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16RChannel1[u16Addr], 0xFFF);
                }
                if(u8Channel == 4) //G1 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16GChannel1[u16Addr], 0xFFF);
                }
                if(u8Channel == 5) //B1 channel
                {
                    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK24_6E_L, pstSetPNLGammaTbl->pu16BChannel1[u16Addr], 0xFFF);
                }
            }

            //4. toggle LUT_W_FLAG_R
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,BIT(5),BIT(5));
            // after setting toggle bit, we need to make sure the bit is already toggled.
            while(SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L, BIT(5)) && (u16Timeout < MAX_DELAY_TIME))
            {
                MsOS_DelayTaskUs(10);
                u16Timeout++;
            }
        }

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,0,BIT(0)); //Afetr writing one channel, disable BW.

    }

    // restore setting
    if(u16tmpReg & BIT(12))
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,BIT(12),BIT(12));
    }

    return TRUE;
}

MS_BOOL MHal_PNL_GetPanelGammaTbl_From_Bin(void* pInstance,ST_PNL_PNLGAMMATBL *pstGetPNLGammaTbl)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    MS_U32 u32index=0;
    MS_PNL_PNLGAMMA_Config stPNLGammaConfig;
    MS_U8 *u8ptr = NULL;
    memset(&stPNLGammaConfig,0,sizeof(MS_PNL_PNLGAMMA_Config));
    memcpy(&stPNLGammaConfig,(pstGetPNLGammaTbl->pu8GammaTbl + PANEL_GAMMA_MAINHEADER_LEN),sizeof(MS_PNL_PNLGAMMA_Config));

    MS_U32 u32StartAddress = stPNLGammaConfig.u32Table2DStartAddress + stPNLGammaConfig.u16PreTableAlignedSize * pstGetPNLGammaTbl->u16Index;

    if (pstGetPNLGammaTbl->pu8GammaTbl == NULL)
    {
        return FALSE;
    }

    if (pstGetPNLGammaTbl->pu16RChannel0 == NULL || pstGetPNLGammaTbl->pu16GChannel0 == NULL || pstGetPNLGammaTbl->pu16BChannel0 == NULL)
    {
        return FALSE;
    }

    if(SUPPORT_PANEL_GAMMA_W_CHANNEL == TRUE)
    {
        if( pstGetPNLGammaTbl->pu16WChannel0 == NULL )
        {
            return FALSE;
        }
    }

    if (stPNLGammaConfig.bGamma2LutEnable == TRUE)
    {
        if (pstGetPNLGammaTbl->pu16RChannel1 == NULL || pstGetPNLGammaTbl->pu16GChannel1 == NULL || pstGetPNLGammaTbl->pu16BChannel1 == NULL)
        {
            return FALSE;
        }

        if( pstGetPNLGammaTbl->pu16WChannel1 == NULL )
        {
            return FALSE;
        }
    }

    u8ptr = u32StartAddress + pstGetPNLGammaTbl->pu8GammaTbl;
    if(stPNLGammaConfig.u8Table2DNumber+stPNLGammaConfig.u8Table3DNumber <= pstGetPNLGammaTbl->u16Index)
    {
        return FALSE;
    }
    pstGetPNLGammaTbl->bSupportChannel1 = TRUE;
    pstGetPNLGammaTbl->u162DTableNum = stPNLGammaConfig.u8Table2DNumber;
    pstGetPNLGammaTbl->u163DTableNum = stPNLGammaConfig.u8Table3DNumber;
    if((stPNLGammaConfig.u8GammaType & BIT(4)))
    {
        pstGetPNLGammaTbl->enGammaType = E_APIPNL_GAMMA_10BIT;
    }
    else if ((stPNLGammaConfig.u8GammaType & BIT(5)))
    {
        pstGetPNLGammaTbl->enGammaType = E_APIPNL_GAMMA_12BIT;
    }

    if((stPNLGammaConfig.u8GammaType & BIT(0)))
    {
        pstGetPNLGammaTbl->enGammaEntries = E_PNL_PNLGAMMA_256_ENTRIES;
    }
    else
    {
        pstGetPNLGammaTbl->enGammaEntries = E_PNL_PNLGAMMA_1024_ENTRIES;
    }
    /* data format
    //each cmd:
    //|  d_15  |   d_14  |   d_13  |  d_12   |   d_11  |   d_10  |   d_9   |   d_8   |   d_7   |   d_6  |  d_5  |  d_4  |  d_3  |  d_2  |  d_1  | d_0  |
    //|251:240 | 235:224 | 219:208 | 203:192 | 187:176 | 171:160 | 155:144 | 139:128 | 123:112 | 107:96 | 91:80 | 75:64 | 59:48 | 43:32 | 27:16 | 11:0 |
    //
    //R/G/B/W cmd order in DRAM
    //            |END | START |
    //  R_LUT0     63      0
    //  G_LUT0     127     64
    //  B_LUT0     191     128
    //  W_LUT0     255     192
    //  R_LUT1     319     256
    //  G_LUT1     383     320
    //  B_LUT1     447     384
    //  W_LUT1     511     448
    */

    MS_U8* pu8ptr_R0 = u8ptr;
    MS_U8* pu8ptr_G0 = pu8ptr_R0 + PANEL_GAMMA_CHANNEL_OFFSET;  // 64entry  * 32 byte =  2048
    MS_U8* pu8ptr_B0 = pu8ptr_G0 + PANEL_GAMMA_CHANNEL_OFFSET ;
    MS_U8* pu8ptr_W0 = pu8ptr_B0 + PANEL_GAMMA_CHANNEL_OFFSET;

    MS_U8* pu8ptr_R1 = pu8ptr_R0 + PANEL_GAMMA_TBL_OFFSET;
    MS_U8* pu8ptr_G1 = pu8ptr_R1 + PANEL_GAMMA_CHANNEL_OFFSET;
    MS_U8* pu8ptr_B1 = pu8ptr_G1 + PANEL_GAMMA_CHANNEL_OFFSET;
    MS_U8* pu8ptr_W1 = pu8ptr_B1 + PANEL_GAMMA_CHANNEL_OFFSET;

    for(u32index=0 ; u32index<PANEL_GAMMA_ENTRY; u32index++)
    {
        //LUT 0
        pstGetPNLGammaTbl->pu16RChannel0[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_R0 + u32index));
        pstGetPNLGammaTbl->pu16GChannel0[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_G0 + u32index));
        pstGetPNLGammaTbl->pu16BChannel0[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_B0 + u32index));
        pstGetPNLGammaTbl->pu16WChannel0[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_W0 + u32index));

        if (stPNLGammaConfig.bGamma2LutEnable == TRUE)
        {
            //LUT 1
            pstGetPNLGammaTbl->pu16RChannel1[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_R1 + u32index));
            pstGetPNLGammaTbl->pu16GChannel1[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_G1 + u32index));
            pstGetPNLGammaTbl->pu16BChannel1[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_B1 + u32index));
            pstGetPNLGammaTbl->pu16WChannel1[u32index] = ((MS_U16)*((MS_U16 *)pu8ptr_W1 + u32index));
        }
    }
    return TRUE;
}

MS_BOOL MHal_PNL_GetPanelGammaTbl_From_Reg(void* pInstance,ST_PNL_PNLGAMMATBL *pstGetPNLGammaTbl)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    MS_U16 u16NumOfLevel =1024;
    MS_U16 u16Addr = 0;
    MS_U8 u8Channel =0;
    MS_BOOL bGammaByPass = MHal_PNL_GetPanelGammaByPass(pInstance);

    if (bGammaByPass == TRUE)
    {
        MHal_PNL_SetPanelGammaByPass(pInstance, DISABLE);
    }

    for(u8Channel = 0; u8Channel <4; u8Channel++)
    {
        //1. set Read channel
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,u8Channel<<2,BIT(3) | BIT(2));
        for(u16Addr = 0; u16Addr < u16NumOfLevel; u16Addr++)
        {
            //2. set LUT_ADDR 0
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6C_L,u16Addr,0x3FF);
            //3. toggle LUT_R_FLAG
            SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_6D_L,BIT(13),BIT(13));

            if(u8Channel == 0) //R channel
            {
                pstGetPNLGammaTbl->pu16RChannel0[u16Addr] = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_71_L,0xFFF);
            }
            if(u8Channel == 1) //G channel
            {
                pstGetPNLGammaTbl->pu16GChannel0[u16Addr] = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_71_L,0xFFF);
            }
            if(u8Channel == 2) //B channel
            {
                pstGetPNLGammaTbl->pu16BChannel0[u16Addr] = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_71_L,0xFFF);
            }
            if(u8Channel == 3) //W channel
            {
                pstGetPNLGammaTbl->pu16WChannel0[u16Addr] = SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK24_71_L,0xFFF);
            }
        }

    }

    if (bGammaByPass == TRUE)
    {
        //restore setting
        MHal_PNL_SetPanelGammaByPass(pInstance, bGammaByPass);
    }

    return TRUE;
}

MS_BOOL MHal_PNL_ADL_SetPQGammaTable(void *pInstance, DRVPNL_GAMMA_TYPE enGammaType, MS_U8* pu8GammaTab[3], DRVPNL_GAMMA_MAPPEING_MODE enGammaMapMode)
{
    MS_U32 u32SizePerChannel = 0;
    MS_BOOL bRet = TRUE;

    XC_AUTODOWNLOAD_FORMAT_INFO stAdlFormatInfo;
    memset(&stAdlFormatInfo, 0, sizeof(XC_AUTODOWNLOAD_FORMAT_INFO));
    stAdlFormatInfo.u32FormatInfo_Version = AUTODOWNLOAD_FORMAT_INFO_VERSION;
    stAdlFormatInfo.u16FormatInfo_Length=sizeof(XC_AUTODOWNLOAD_FORMAT_INFO);

    XC_AUTODOWNLOAD_DATA_INFO stAdlDataInfo;
    memset(&stAdlDataInfo, 0, sizeof(XC_AUTODOWNLOAD_DATA_INFO));
    stAdlDataInfo.u32DataInfo_Version = AUTODOWNLOAD_DATA_INFO_VERSION;
    stAdlDataInfo.u16DataInfo_Length =  sizeof(XC_AUTODOWNLOAD_DATA_INFO);
    stAdlDataInfo.pu8Data = NULL;
    stAdlDataInfo.enClient = E_XC_AUTODOWNLOAD_CLIENT_PQ_GAMMA;
    stAdlDataInfo.pParam = &stAdlFormatInfo;

    PNL_MALLOC_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8), bRet);
    if (bRet == FALSE)
    {
        return FALSE;
    }
    if(enGammaType == E_DRVPNL_GAMMA_12BIT)
    {
        MS_U16 u16GammaEntry = ((enGammaMapMode == E_DRVPNL_GAMMA_8BIT_MAPPING) ? 256 : 1024);
        u32SizePerChannel = (u16GammaEntry / 2) * 3; //two 12bit gamma value in three bytes
        stAdlDataInfo.u32Size =  u32SizePerChannel * 3; //three channel r/g/b
    }
    else
    {
        printf("[%s,%5d] Unspported gamma bit type %d\n", __FUNCTION__, __LINE__, enGammaType);
        PNL_FREE_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8));
        return FALSE;
    }

    if (enGammaMapMode == E_DRVPNL_GAMMA_8BIT_MAPPING)
    {
        memcpy(stAdlDataInfo.pu8Data, pu8GammaTab[0], u32SizePerChannel);
        memcpy(stAdlDataInfo.pu8Data+u32SizePerChannel, pu8GammaTab[1], u32SizePerChannel);
        memcpy(stAdlDataInfo.pu8Data+u32SizePerChannel*2, pu8GammaTab[2], u32SizePerChannel);
    }
    else
    {
        printf("[%s,%5d] Unspported gamma bit type %d\n", __FUNCTION__, __LINE__, enGammaType);
        PNL_FREE_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8));
        return FALSE;
    }

    if(MApi_XC_AutoDownload_Write(&stAdlDataInfo) != E_APIXC_RET_OK )
    {
        printf("\033[1;31m PQ Gamma ,Write ADL Fail!!!!!!!!!!\033[m\n");
        PNL_FREE_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8));
        return FALSE;
    }

    if( MApi_XC_AutoDownload_Fire(E_XC_AUTODOWNLOAD_CLIENT_PQ_GAMMA) != E_APIXC_RET_OK)
    {
        printf("\033[1;31m PQ Gamma ,Fire ADL Fail!!!!!!!!!!\033[m\n");
        PNL_FREE_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8));
        return FALSE;
    }

    PNL_FREE_MEM(stAdlDataInfo.pu8Data, PQ_GAMMA_TABLE_SIZE_MAX * sizeof(MS_U8));
    return TRUE;
}

MS_BOOL MHal_PNL_STR_StoreSetting(void* pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    //Store free swap config
    MDrv_PNL_StoreFreeSwapCfg(pInstance);

    ST_PNL_MIRRORINFO stPNLMirrorInfo;
    stPNLMirrorInfo.eMirrorType = E_PNL_MOD_H_MIRROR;
    stPNLMirrorInfo.bEnable = FALSE;
    MDrv_PNL_GetMirrorStatus(pInstance, &stPNLMirrorInfo);
    //Store H mirror info
    pPNLResourcePrivate->stapiPNL._bMOD_HmirrorEn = stPNLMirrorInfo.bEnable;

    //Store PNL SSC setting.
    pPNLResourcePrivate->stapiPNL._u16SSC_Fmodulation = R2BYTE(L_BK_LPLL(0x18));// LPLL_STEP
    pPNLResourcePrivate->stapiPNL._u16SSC_Rdeviation = R2BYTE(L_BK_LPLL(0x17));// LPLL_STEP
    pPNLResourcePrivate->stapiPNL._bSSC_En = (R2BYTEMSK(L_BK_LPLL(0x0D), BIT(11))>>11) ? TRUE: FALSE;

    //Store Swing config
    MDrv_PNL_StoreSwingCfg(pInstance);

    //Store PE config
    MDrv_PNL_StorePECfg(pInstance);

    return TRUE;
}

MS_BOOL MHal_PNL_STR_RecoverSetting(void* pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    //recover mirror
    if(pPNLResourcePrivate->stapiPNL._bMOD_HmirrorEn)
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_20_L, BIT(14), BIT(14));
    }
    else
    {
        MOD_W2BYTEMSK(REG_MOD_BK00_20_L, 0, BIT(14));
    }

    //recover free swap config
    MDrv_PNL_RecoverFreeSwapCfg(pInstance);

    W2BYTE(L_BK_LPLL(0x18), pPNLResourcePrivate->stapiPNL._u16SSC_Fmodulation & 0x3FFF);// LPLL_SPAN
    W2BYTE(L_BK_LPLL(0x17), pPNLResourcePrivate->stapiPNL._u16SSC_Rdeviation & 0x0FFF);// LPLL_STEP
    W2BYTEMSK((L_BK_LPLL(0x0D)), (pPNLResourcePrivate->stapiPNL._bSSC_En << 11), BIT(11)); // Enable ssc

    //recover Swing config
    MDrv_PNL_RecoverSwingCfg(pInstance);

    //recover PE config
    MDrv_PNL_RecoverPECfg(pInstance);

    return TRUE;
}

MS_BOOL MHal_PNL_OverDriver_IsEnableRGBW(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    if((SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_7E_L,BIT(15))) == (BIT(15)))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

void MHal_PNL_OverDriver_EnableWChannel(void *pInstance,MS_BOOL bEnWChannel)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    //use dummy register to store tcon w channel enable status
    SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID,REG_SC_BK29_7E_L, bEnWChannel? BIT(15): 0,BIT(15));
}

void MHal_PNL_EnablePQGammaCLK(void *pInstance)
{
    if (R2BYTEMSK(L_BK_LPLL(0x03), BIT(5)) != 0)
    {
        MHal_PNL_PowerDownLPLL(pInstance, FALSE);
    }

    if (R2BYTEMSK(REG_CLKGEN0_52_L, 0x07) != 0)
    {
        MHal_PQ_Clock_Gen_For_Gamma(pInstance);
    }
}

PNL_Result MHal_PNL_GetMODSTATUS(void* pInstance,ST_PNL_MODSTATUS *pstGetMODSTATUS)
{
    //debug clk enable
    MOD_W2BYTEMSK(REG_MOD_BK00_2F_L, 0x8000, 0x8000);
    //set the channel
    switch(pstGetMODSTATUS->enMODCRCChannel)
    {
        case E_PNL_MODCRC_R_CHANNEL:
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, 0, 0x0003);
            break;
        }
        case E_PNL_MODCRC_G_CHANNEL:
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, BIT(0), 0x0003);
            break;
        }
        case E_PNL_MODCRC_B_CHANNEL:
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, BIT(1), 0x0003);
            break;
        }
        default:
        {
            //printf("ERROR!! SET WRONG type for read MOD CRC!!!!!\n");
            break;
        }
    }
    // set the CRC pair
    switch(pstGetMODSTATUS->enMODCRCPair)
    {
        case E_PNL_MODCRC_EVEN_PAIR:
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, BIT(2), BIT(2));
            break;
        }
        case E_PNL_MODCRC_ODD_PAIR:
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, 0, BIT(2));
            break;
        }
        default:
        {
            //printf("ERROR!! SET WRONG type for read MOD CRC!!!!!\n");
            break;
        }
    }
    //enable CRC
    MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, BIT(3), BIT(3));
    MsOS_DelayTask(40); //delay 2 frame
    *(pstGetMODSTATUS->pu16CRCValue) = MOD_R2BYTE(REG_MOD_BK00_54_L);
    MOD_W2BYTEMSK(REG_MOD_BK00_4C_L, FALSE, BIT(3));
    //disable debug clk
    MOD_W2BYTEMSK(REG_MOD_BK00_2F_L, 0x0000, 0x8000);
    return E_PNL_OK;
}

void MHal_PNL_StoreSwingCfg(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type >= E_PNL_LPLL_VBY1_10BIT_4LANE)&&
       (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type <= E_PNL_LPLL_VBY1_8BIT_8LANE))
    {
        // vby1 rcon
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch0= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_10_L,0x003F);//bit 0~5
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch1= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_10_L,0x3F00))>>8);//bit 8~13
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch2= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_11_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch3= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_11_L,0x3F00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch4= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_12_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch5= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_12_L,0x3F00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch6= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_13_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch7= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_13_L,0x3F00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch8= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_14_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch9= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_14_L,0x3F00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch10= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_15_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch11= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_15_L,0x3F00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch12= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_16_L,0x003F);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch13= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_16_L,0x3F00))>>8);
    }
    else
    {
        // LVDS icon
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch0= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_08_L,0x00FF);//bit 0~7
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch1= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_08_L,0xFF00))>>8);//bit 8~15
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch2= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_09_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch3= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_09_L,0xFF00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch4= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0A_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch5= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0A_L,0xFF00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch6= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0B_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch7= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0B_L,0xFF00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch8= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0C_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch9= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0C_L,0xFF00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch10= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0D_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch11= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0D_L,0xFF00))>>8);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch12= MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0E_L,0x00FF);
        pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch13= ((MOD_A_R2BYTEMSK(REG_MOD_A_BK00_0E_L,0xFF00))>>8);
    }
}

void MHal_PNL_RecoverSwingCfg(void *pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if((pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type >= E_PNL_LPLL_VBY1_10BIT_4LANE)&&
       (pPNLResourcePrivate->stdrvPNL._stPnlInitData.eLPLL_Type <= E_PNL_LPLL_VBY1_8BIT_8LANE))
    {
        // vby1 rcon
        MOD_A_W2BYTE(REG_MOD_A_BK00_10_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch0)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch1<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_11_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch2)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch3<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_12_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch4)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch5<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_13_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch6)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch7<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_14_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch8)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch9<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_15_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch10)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch11<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_16_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch12)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch13<<8));

    }
    else
    {
        // LVDS icon
        MOD_A_W2BYTE(REG_MOD_A_BK00_08_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch0)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch1<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_09_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch2)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch3<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_0A_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch4)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch5<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_0B_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch6)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch7<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_0C_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch8)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch9<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_0D_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch10)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch11<<8));
        MOD_A_W2BYTE(REG_MOD_A_BK00_0E_L,(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch12)|(pPNLResourcePrivate->stdrvPNL.stPnlSwingCfg.u16ControlSwingch13<<8));
    }


}

void MHal_PNL_StorePECfg(void* pInstance)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch0=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_18_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch1=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_18_L,0xFF00) >> 8);
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch2=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_19_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch3=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_19_L,0xFF00) >> 8);

    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch4=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1A_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch5=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1A_L,0xFF00) >> 8);
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch6=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1B_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch7=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1B_L,0xFF00) >> 8);

    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch8=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1C_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch9=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1C_L,0xFF00) >> 8);
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch10=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1D_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch11=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1D_L,0xFF00) >> 8);

    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch12=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1E_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch13=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1E_L,0xFF00) >> 8);
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch14=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1F_L,0x00FF));
    pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch15=(MOD_A_R2BYTEMSK(REG_MOD_A_BK00_1F_L,0xFF00) >> 8);

}

void MHal_PNL_RecoverPECfg(void* pInstance)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    MS_U16 u16ValidCurrent;

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch0)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch1) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_18_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch2)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch3) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_19_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch4)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch5) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1A_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch6)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch7) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1B_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch8)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch9) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1C_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch10)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch11) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1D_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch12)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch13) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1E_L,u16ValidCurrent);

    u16ValidCurrent=((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch14)|((pPNLResourcePrivate->stdrvPNL.stPnlPECfg.u16ControlPEch15) << 8));
    MOD_A_W2BYTE(REG_MOD_A_BK00_1F_L,u16ValidCurrent);

    if (MOD_A_R2BYTE(REG_MOD_A_BK00_30_L)!=0x3FFF)
    {
        MOD_A_W2BYTE(REG_MOD_A_BK00_30_L, 0x3fff); // PE ch0~ch13 enable.
    }
}

void MHal_PNL_StoreFreeSwapCfg(void* pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder0_3 = MOD_R2BYTE(REG_MOD_BK00_08_L);
    pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder4_7= MOD_R2BYTE(REG_MOD_BK00_09_L);
    pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder8_11= MOD_R2BYTE(REG_MOD_BK00_0A_L);
    pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder12_15= MOD_R2BYTE(REG_MOD_BK00_0B_L);
}
void MHal_PNL_RecoverFreeSwapCfg(void* pInstance)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;

    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    MOD_W2BYTE(REG_MOD_BK00_08_L, pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder0_3);
    MOD_W2BYTE(REG_MOD_BK00_09_L, pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder4_7);
    MOD_W2BYTE(REG_MOD_BK00_0A_L, pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder8_11);
    MOD_W2BYTE(REG_MOD_BK00_0B_L, pPNLResourcePrivate->stdrvPNL.stPnlSwapCfg.u16OutputOrder12_15);
}

MS_BOOL MHal_PNL_CheckOutputConfig(void* pInstance)
{
    MS_U16 u16reg_32da = 0, u16reg_32dc = 0;

    u16reg_32da = MOD_A_R2BYTE(REG_MOD_A_BK00_00_L);
    u16reg_32dc = MOD_A_R2BYTE(REG_MOD_A_BK00_01_L);

    if((u16reg_32da ==0) && (u16reg_32dc==0))
    {
        //outputconfig is off status
        return FALSE;
    }
    else
    {
        //outputconfig is set status
        return TRUE;
    }
}

static MS_BOOL _MHal_PNL_CheckMODCalibrationEn(void *pInstance)
{
    //For some chip (check with designer), NO NEED to do calibration when panel init.
   // Instead, Read and Dump the ICON/RCON settings from Efuse.
    return FALSE;
}

MS_U16 MHal_PNL_Get_LPLL_Cal_Param(void)
{
    return LPLL_CAL_PARAMETER;
}

MS_BOOL MHal_PNL_IsForceRunFSMTimingMode(void)
{
    return ENABLE_FORCE_FSM_TIMING_MODE;
}

void MHal_PNL_AdjustDclk(void *pInstance, MS_U64* pu64InputDclk,PNL_TYPE ePnlType)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    if(pu64InputDclk!=NULL)
    {
        if( E_PNL_TYPE_TTL != ePnlType )
        {
            if( (*pu64InputDclk)>0 )
            {
                MS_U64 u64LpllFactor = 0;

                if (ePnlType == E_PNL_LPLL_VBY1_8BIT_8LANE || ePnlType == E_PNL_LPLL_VBY1_10BIT_8LANE)
                {
                    u64LpllFactor = 1;
                }
                else if (ePnlType == E_PNL_LPLL_VBY1_8BIT_4LANE || ePnlType == E_PNL_LPLL_VBY1_10BIT_4LANE) //pixel rate= 300MHZ
                {
                    u64LpllFactor = 2;
                }
                else if (ePnlType == E_PNL_LPLL_VBY1_8BIT_2LANE || ePnlType == E_PNL_LPLL_VBY1_10BIT_2LANE) //pixel rate = 150MHZ
                {
                    u64LpllFactor = 4;
                }
                else if (ePnlType == E_PNL_TYPE_LVDS || IsTCONType_MiniLVDS(ePnlType))
                {
                    if(pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16DefaultVFreq == 1200) //pixel rate = 150MHZ
                    {
                        u64LpllFactor = 2;
                    }
                    else
                    {
                        if (pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height == PNL_1366X768_HEIGHT
                            && pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width == PNL_1366X768_WIDTH) //pixel rate = 85MHZ
                        {
                            u64LpllFactor = 7;
                        }

                        if (pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Height == PNL_FHD_HEIGHT
                            && pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16Width == PNL_FHD_WIDTH) //pixel rate = 150MHZ
                        {
                            u64LpllFactor = 4;
                        }
                    }
                }
                else
                {
                    u64LpllFactor = 1;
                }

                if(u64LpllFactor>0)
                {
                    (*pu64InputDclk) *=  u64LpllFactor;
                }
            }
        }
    }
}

#ifdef CONFIG_UTOPIA_PROC_DBG_SUPPORT

static MS_BOOL _MHal_PNL_openYMYTestCase(void)
{
    return OPEN_YMY_TEST_CASE;
}

static void _MHal_PNL_Mdbloctl_Cat_MOD(void *pInstance, MDBCMD_GETINFO_PARAMETER *paraGetInfo)
{
    MdbPrint(paraGetInfo->u64ReqHdl,"----------------  MTK MOD Info -----------------\n");
    MdbPrint(paraGetInfo->u64ReqHdl,"Output config                               : 0x%x, 0x%x\n",  MOD_A_R2BYTE(REG_MOD_A_BK00_00_L), MOD_A_R2BYTE(REG_MOD_A_BK00_01_L));
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel clock                                 : %d\n", MOD_A_R2BYTEMSK(REG_MOD_A_BK00_00_L,0x1F));
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel free swap                             : 0x%x, 0x%x, 0x%x, 0x%x\n", MOD_R2BYTE(REG_MOD_BK00_08_L),MOD_R2BYTE(REG_MOD_BK00_09_L),MOD_R2BYTE(REG_MOD_BK00_0A_L),MOD_R2BYTE(REG_MOD_BK00_0B_L) );
    MdbPrint(paraGetInfo->u64ReqHdl,"VB1 Video Lock status                       : %s\n", MOD_R2BYTEMSK(REG_MOD_BK00_61_L, BIT(14))==BIT(14)?"UNLOCK":"LOCK");
    MdbPrint(paraGetInfo->u64ReqHdl,"VB1 unlock count                            : %d\n", MOD_R2BYTEMSK(REG_MOD_BK00_7E_L, 0x7F));
    MdbPrint(paraGetInfo->u64ReqHdl,"VB1 HTPDN status                            : %s\n", MOD_R2BYTEMSK(REG_MOD_BK00_7F_L, BIT(5)) == BIT(5) ? "NOT Connected":"Connected");
    MdbPrint(paraGetInfo->u64ReqHdl,"VB1 LockN status                            : %s\n", MOD_R2BYTEMSK(REG_MOD_BK00_7F_L, BIT(4)) == BIT(4) ? "UNLOCK":"LOCK");
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel PN swap by CH                         : 0x%x\n",  MOD_A_R2BYTEMSK(REG_MOD_A_BK00_31_L, 0x3FFF));
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel swing                                 : 0x%x\n", MOD_A_R2BYTEMSK(REG_MOD_A_BK00_08_L,0xFF));
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel swing (VB1)                           : 0x%x\n", MOD_A_R2BYTEMSK(REG_MOD_A_BK00_10_L,0x3F));
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel pre-emphasis enable                   : %s\n", MOD_A_R2BYTEMSK(REG_MOD_A_BK00_30_L,BIT(0)) ? "Enable":"Disable");
    MdbPrint(paraGetInfo->u64ReqHdl,"Panel pre-emphasis level                    : 0x%x\n", MOD_A_R2BYTEMSK(REG_MOD_A_BK00_18_L,0x1F));
    MdbPrint(paraGetInfo->u64ReqHdl,"Htotal/Vtotal                               : %d, %d\n", SC_R2BYTEMSK(0, REG_SC_BK10_0C_L , 0x3FFF), SC_R2BYTEMSK(0, REG_SC_BK10_0D_L , 0x3FFF));
    MdbPrint(paraGetInfo->u64ReqHdl,"LPLL set                                    : 0x%x\n", R4BYTE(L_BK_LPLL(0x0F)));
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Enable                                  : %s\n", R2BYTEMSK(L_BK_LPLL(0x0D),BIT(11)) ? "Enable" : "Disable");
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Step                                    : 0x%x\n", R2BYTE(L_BK_LPLL(0x17)));
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Span                                    : 0x%x\n", R2BYTE(L_BK_LPLL(0x18)));

    if(_MHal_PNL_openYMYTestCase())
    {
        EN_PNL_TEST_STATUS enPanelGammaResult = E_PNL_TEST_FAIL;
        Hal_PNL_TestAutodownload_PanelGamma(pInstance, &enPanelGammaResult);
        MdbPrint(paraGetInfo->u64ReqHdl,"Panel gamma test                            : %s\n", (enPanelGammaResult == E_PNL_TEST_PASS) ? "PASS":"FAIL");

        EN_PNL_TEST_STATUS enODResult = E_PNL_TEST_FAIL;
        Hal_PNL_TestAutodownload_OD(pInstance, &enODResult);
        MdbPrint(paraGetInfo->u64ReqHdl,"OD test                                     : %s\n", (enODResult == E_PNL_TEST_PASS) ? "PASS":"FAIL");
    }
}

static void _MHal_PNL_Mdbloctl_Cat_PNL(void *pInstance, MDBCMD_GETINFO_PARAMETER *paraGetInfo)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));


    MdbPrint(paraGetInfo->u64ReqHdl,"----------------  MTK PNL Info -----------------\n");
    MdbPrint(paraGetInfo->u64ReqHdl,"PanelOnTiming1(VCC/Data)                    : %d\n",pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16PanelOnTiming1);
    MdbPrint(paraGetInfo->u64ReqHdl,"PanelOnTiming2(Data/BackLight)              : %d\n",pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16PanelOnTiming2);
    MdbPrint(paraGetInfo->u64ReqHdl,"PanelOffTiming1(Backlight/Data)             : %d\n",pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16PanelOffTiming1);
    MdbPrint(paraGetInfo->u64ReqHdl,"PanelOffTiming2(Data/VCC)                   : %d\n",pPNLResourcePrivate->stdrvPNL._stPnlInitData.u16PanelOffTiming2);
    if (pPNLResourcePrivate->stapiPNL._eSTRMode == E_PNL_RFSTR_MODE)
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"STR mode                                    :RFSTR\n");
        MdbPrint(paraGetInfo->u64ReqHdl,"-->Panel STR sequence only on panel drvier\n");

        MdbPrint(paraGetInfo->u64ReqHdl,"Backlight Control PAD                       :%d\n",pPNLResourcePrivate->stapiPNL._u16GPIO_INV_CTL_PAD_NO);
        MdbPrint(paraGetInfo->u64ReqHdl,"Backlight INVERTOR                          :%d\n",pPNLResourcePrivate->stapiPNL._bGPIO_INV_CTL_INVERTOR);
        MdbPrint(paraGetInfo->u64ReqHdl,"Panel Control PAD                           :%d\n",pPNLResourcePrivate->stapiPNL._u16GPIO_PNL_CTL_PAD_NO);
        MdbPrint(paraGetInfo->u64ReqHdl,"Panel INVERTOR                              :%d\n",pPNLResourcePrivate->stapiPNL._bGPIO_PNL_CTL_INVERTOR);

        MdbPrint(paraGetInfo->u64ReqHdl,"Wakeup Source(PM_PowerOnMode)               :%d\n",MDrv_PM_GetWakeupSource());
    }
    else if (pPNLResourcePrivate->stapiPNL._eSTRMode == E_PNL_STR_MODE)
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"STR mode                                    :STR\n");
        MdbPrint(paraGetInfo->u64ReqHdl,"-->Panel STR sequence on mboot\n");
    }
    else if (pPNLResourcePrivate->stapiPNL._eSTRMode == E_PNL_FSTR_MODE)
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"STR mode                                    :FSTR\n");
        MdbPrint(paraGetInfo->u64ReqHdl,"-->Panel STR sequence on MI/SN\n");
    }
    else
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"!!!STR mode is not expected!!!\n");
    }

    if (stTCONPanelConfig.bUsingTCON == TRUE)
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"UsingTCON                                   :TRUE\n");

        if(strlen(pPNLResourcePrivate->stapiPNL._pm_TCON_FILE_PATH) != 0)
        {
            MdbPrint(paraGetInfo->u64ReqHdl,"TCON path                                   :%s\n",pPNLResourcePrivate->stapiPNL._pm_TCON_FILE_PATH);
        }
        else
        {
            MdbPrint(paraGetInfo->u64ReqHdl,"TCON path error\n");
        }
    }
    else
    {
        MdbPrint(paraGetInfo->u64ReqHdl,"UsingTCON                                   :FALSE\n");
    }
    MdbPrint(paraGetInfo->u64ReqHdl,"PanelDCLK(MHz)                              : %d\n", pPNLResourcePrivate->stapiPNL._u16PanelDCLK);
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Enable                                  : %s\n", pPNLResourcePrivate->stapiPNL._bSSC_En ? "Enable" : "Disable");
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Modulation                              : %d\n", pPNLResourcePrivate->stapiPNL._u16SSC_Fmodulation);
    MdbPrint(paraGetInfo->u64ReqHdl,"SSC Deviation                               : %d\n", pPNLResourcePrivate->stapiPNL._u16SSC_Rdeviation);
}

static void _MHal_PNL_MdbIoctl_Echo_Help(MDBCMD_CMDLINE_PARAMETER *paraCmdLine)
{
    MdbPrint(paraCmdLine->u64ReqHdl, "------------------- MTK PNL help --------------------\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================MOD test mode pattern======================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo mod_test=1 HVmode=0 Color=0 Depth=0x3FF inv=0 start=0 end=0xFFF > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "mod_test=0/1:   0 means disable, 1 means enable\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "HVmode=0/1:     0 means H line mode, 1 means V line mode\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Color=0/1/2/3: '0' means Gray, '1' means Red, '2' means Blue, '3' means Green\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Depth=0~0x3FF:  10 bit color depth\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "[Option] inv=0/1:       invert disable/enable\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "[Option] start=0~0xFFF: test start point\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "[Option] end=0~0xFFF:   test end point\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================VOP display window border==================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo vop_border=1 color=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "vop_border=0~255: the value means border width\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "color=0/1/2/3:    '0' means Gray, '1' means Red, '2' means Blue, '3' means Green\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LPLL set===================================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo lpll_set=Var1 freerun=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "lpll_set==0x22c12f1:  It will change DCLK\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "freerun==0/1:         '1' means force free run\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Output Htotal/Vtotal set===================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo vop_set htt=Var1 vtt=Var2 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "htt==0~0x3FFF, vtt=0~0x3FFF\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Free swap =================================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo free_swap CH3210=Var1 CH7654=Var2 CHba98=Var3 CHfedc=Var4 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "CH3210: bit[15:12] for CH3, bit[11:8] for CH2, bit[7:4] for CH2, bit[3:0] for CH0\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "CH7654: bit[15:12] for CH7, bit[11:8] for CH6, bit[7:4] for CH5, bit[3:0] for CH4\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "CHba98: bit[15:12] for CH11, bit[11:8] for CH10, bit[7:4] for CH9, bit[3:0] for CH8\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "CHfedc: bit[7:4]   for CH13, bit[3:0] for CH12\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================PN swap for channel =======================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo pn_swap_ch=Var1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Var1: Bit[13:0] is for Channel[13:0] PN swap \n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LVDS PN swap ==============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo pn_swap=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "pn_swap==0/1: LVDS Porlarity swap\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LVDS Port swap ============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo port_swap=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "port_swap==0/1: LVDS Odd/Even port swap\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LVDS Channel swap =========================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo ch_swap=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ch_swap==0/1: LVDS channel pair swap\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Dual port enable ==========================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo dual_port=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "dual_port==0/1: Dual pixel output enable\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LVDS TI mode  =============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo ti_mode=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ti_mode==0/1: '0' is JEIDA mode, '1' is VESA mode\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Output Config  ============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo outconf CH7to0=Var1 CHfto8=Var2 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Output mode config for each channel:\n");
    MdbPrint(paraCmdLine->u64ReqHdl, " 2'b00: TTL mode\n 2'b01: LVDS data mode\n 2'b10: skew clock mode\n 2'b11: test clock mode\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================MOD GPIO control  =========================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo mod_gpio pad=27 oen=0 out=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "pad==0~27: '0' is CH0P, '27' is CH13N\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "oen==0/1:  '0' is output, '1' is input. If input mode, log will print the GPI value\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "out==0/1:  '0' is output low, '1' is output high\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Swing control  ============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo swing=Var1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Var1 == 40~600 (For LVDS/TCON-less):  swing level\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Var1 == 0~0x3F (For VB1):             swing level, default is 0x13\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Pre-Emphasis control  =====================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo preem en=1 value=Var1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "en==0/1:        Disable/Enable pre-emphasis\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "Var1==0~0xF:    pre-emphasis level\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Panel Enable ==============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo pnl_en=1 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "en==0/1:    Disable/Enable panel\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================SSC control================================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo ssc en=1 mod=200 dev=100 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "en==0/1:    Disable/Enable SSC\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "mod==xx(0.1KHz):     SSC modulation\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "dev==xx(0.01pct):    SSC deviation\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================LPLL table dump============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo lpll_dump > /proc/utopia_mdb/pnl\n");

    MdbPrint(paraCmdLine->u64ReqHdl, "==================Debug Level set============================\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "ex: echo DbgLevel=0x0003 > /proc/utopia_mdb/pnl\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "DbgLevel==0~0xFFFF, debug level mask\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_OFF:             (0x0000)\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_INIT:            (0x0001)\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_PANEL_EN:        (0x0002)\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_SSC:             (0x0004)\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_GAMMA:           (0x0008)\n");
    MdbPrint(paraCmdLine->u64ReqHdl, "PNL_DBGLEVEL_CALIBRATION:     (0x0010)\n");
}

static void _MHal_PNL_MdbIoctl_EchoCmd(void *pInstance, MDBCMD_CMDLINE_PARAMETER *paraCmdLine)
{
    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    PNL_RESOURCE_PRIVATE* pPNLResourcePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);
    UtopiaResourceGetPrivate(g_pPNLResource[PNL_GET_INTERNAL_POOL_ID(pPNLInstancePrivate->u32DeviceID)],(void**)(&pPNLResourcePrivate));

    char pch[] = " =\n\r";
    char *pcCmdLine = paraCmdLine->pcCmdLine;
    char* psep;
    psep = strsep(&pcCmdLine,pch);

    // echo mod_test=1 HVmode=0 Color=0 Depth=0x3FF > /proc/utopia_mdb/pnl
    if(strcmp("mod_test", psep) == 0)
    {
        MS_BOOL bEnable  = FALSE;
        MS_BOOL bInv     = FALSE;
        MS_BOOL bHVmode  = FALSE;    //"FALSE" means H mode;  "TRUE" means V mode
        MS_U8   u8Color  = 0;       //"0" means Gray, "1" means Red, "2" means Blue, "3" means Green
        MS_U16  u16Depth = 0;
        MS_U16  u16Start = 0;
        MS_U16  u16End   = 0;

        psep = strsep(&pcCmdLine,pch);    //mod_test value
        if(psep == NULL)
        {
            return;
        }
        bEnable = (MS_BOOL)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "HVmode"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //HVmode value
        if(psep == NULL)
        {
            return;
        }
        bHVmode = (MS_BOOL)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "Color"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //Color value
        if(psep == NULL)
        {
            return;
        }
        u8Color = (MS_U8)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "Depth"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //Depth value
        if(psep == NULL)
        {
            return;
        }
        u16Depth = (MS_U16)atoi(psep);

        //clear RGB color depth
        MOD_W2BYTEMSK(REG_MOD_BK00_02_L, 0x0 , 0x03FF);
        MOD_W2BYTEMSK(REG_MOD_BK00_03_L, 0x0 , 0x03FF);
        MOD_W2BYTEMSK(REG_MOD_BK00_04_L, 0x0 , 0x03FF);
        //disable H/V inv
        MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, 0x0, BIT(14));
        MOD_W2BYTEMSK(REG_MOD_BK00_51_L, 0x0, BIT(14));
        if(bHVmode == FALSE)        // H mode
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, 0x0 , 0x1FFF);        // reg_test_h_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_50_L, 0x1FFF, 0x1FFF);    // reg_test_h_line_end
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, 0x0 , 0x0FFF);        // reg_test_v_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0x0, 0x0FFF);        // reg_test_v_line_end
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, (bEnable==TRUE)?BIT(15):0, BIT(15));        // reg_test_h_line_mode
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, 0 , BIT(15));        // reg_test_v_line_mode
        }
        else
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, 0x0 , 0x1FFF);        // reg_test_h_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_50_L, 0x0, 0x1FFF);        // reg_test_h_line_end
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, 0x0 , 0x0FFF);        // reg_test_v_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0x0FFF, 0x0FFF);    // reg_test_v_line_end
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, 0 , BIT(1));        // reg_test_h_line_mode
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, (bEnable==TRUE)?BIT(15):0, BIT(15));        // reg_test_v_line_mode
        }
        switch(u8Color)
        {
            case 0:    //Gray
                MOD_W2BYTEMSK(REG_MOD_BK00_02_L, u16Depth , 0x03FF);    //reg_pat_r
                MOD_W2BYTEMSK(REG_MOD_BK00_03_L, u16Depth , 0x03FF);    //reg_pat_g
                MOD_W2BYTEMSK(REG_MOD_BK00_04_L, u16Depth , 0x03FF);    //reg_pat_b
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, BIT(14)|BIT(13)|BIT(12) , BIT(14)|BIT(13)|BIT(12));    //reg_test_gray_en
                break;
            case 1:    //Red
                MOD_W2BYTEMSK(REG_MOD_BK00_02_L, u16Depth , 0x03FF);    //reg_pat_r
                MOD_W2BYTEMSK(REG_MOD_BK00_03_L, 0 , 0x03FF);    //reg_pat_g
                MOD_W2BYTEMSK(REG_MOD_BK00_04_L, 0 , 0x03FF);    //reg_pat_b
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0 , BIT(14)|BIT(13)|BIT(12));    //reg_test_gray_en
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, BIT(12) , BIT(12));    //reg_test_gray_r_en
                break;
            case 2:    //Green
                MOD_W2BYTEMSK(REG_MOD_BK00_02_L, 0 , 0x03FF);    //reg_pat_r
                MOD_W2BYTEMSK(REG_MOD_BK00_03_L, u16Depth , 0x03FF);    //reg_pat_g
                MOD_W2BYTEMSK(REG_MOD_BK00_04_L, 0 , 0x03FF);    //reg_pat_b
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0 , BIT(14)|BIT(13)|BIT(12));    //reg_test_gray_en
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, BIT(13) , BIT(13));    //reg_test_gray_g_en
                break;
            case 3:    //Blue
                MOD_W2BYTEMSK(REG_MOD_BK00_02_L, 0 , 0x03FF);    //reg_pat_r
                MOD_W2BYTEMSK(REG_MOD_BK00_03_L, 0 , 0x03FF);    //reg_pat_g
                MOD_W2BYTEMSK(REG_MOD_BK00_04_L, u16Depth , 0x03FF);    //reg_pat_b
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0 , BIT(14)|BIT(13)|BIT(12));    //reg_test_gray_en
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, BIT(14) , BIT(14));    //reg_test_gray_b_en
                break;
            default: //Black
                MOD_W2BYTEMSK(REG_MOD_BK00_02_L, 0 , 0x03FF);    //reg_pat_r
                MOD_W2BYTEMSK(REG_MOD_BK00_03_L, 0 , 0x03FF);    //reg_pat_g
                MOD_W2BYTEMSK(REG_MOD_BK00_04_L, 0 , 0x03FF);    //reg_pat_b
                MOD_W2BYTEMSK(REG_MOD_BK00_52_L, 0 , BIT(14)|BIT(13)|BIT(12));    //reg_test_gray_en
                break;
        }

        psep = strsep(&pcCmdLine,pch);    //find keyword "inv"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //inv value
        if(psep == NULL)
        {
            return;
        }
        bInv = (MS_BOOL)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "start"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //start value
        if(psep == NULL)
        {
            return;
        }
        u16Start = (MS_U16)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "end"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //end value
        if(psep == NULL)
        {
            return;
        }
        u16End = (MS_U16)atoi(psep);

        if(bHVmode == FALSE)        // H mode
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, (bInv==TRUE)?BIT(14):0, BIT(14));        // reg_test_h_inv
            MOD_W2BYTEMSK(REG_MOD_BK00_4F_L, u16Start , 0x1FFF);        // reg_test_h_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_50_L, u16End, 0x1FFF);        // reg_test_h_line_end
        }
        else
        {
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, (bInv==TRUE)?BIT(14):0, BIT(14));        // reg_test_v_inv
            MOD_W2BYTEMSK(REG_MOD_BK00_51_L, u16Start , 0x0FFF);        // reg_test_v_line_st
            MOD_W2BYTEMSK(REG_MOD_BK00_52_L, u16End, 0x0FFF);        // reg_test_v_line_end
        }
    }
    // echo vop_border=1 color=1> /proc/utopia_mdb/pnl
    else if(strcmp("vop_border", psep) == 0)
    {
        MS_U8  u8Pixel       = 0;
        MS_U8  u8Color       = 0;
        MS_U16 u16HfDE_Start = 0;
        MS_U16 u16HfDE_End   = 0;
        MS_U16 u16VfDE_Start = 0;
        MS_U16 u16VfDE_End   = 0;
        MS_U16 u16HDE_Start  = 0;
        MS_U16 u16HDE_End    = 0;
        MS_U16 u16VDE_Start  = 0;
        MS_U16 u16VDE_End    = 0;
        psep = strsep(&pcCmdLine,pch);    //vop_border value
        if(psep == NULL)
        {
            return;
        }
        u8Pixel = (MS_U8)atoi(psep);

        u16HfDE_Start = SC_R2BYTEMSK(0, REG_SC_BK10_04_L, 0x3FFF);
        u16HfDE_End = SC_R2BYTEMSK(0, REG_SC_BK10_05_L, 0x3FFF);
        u16VfDE_Start = SC_R2BYTEMSK(0, REG_SC_BK10_06_L, 0x3FFF);
        u16VfDE_End = SC_R2BYTEMSK(0, REG_SC_BK10_07_L, 0x3FFF);

        u16HDE_Start = u16HfDE_Start + u8Pixel;
        u16HDE_End = u16HfDE_End - u8Pixel;
        u16VDE_Start = u16VfDE_Start + u8Pixel;
        u16VDE_End = u16VfDE_End - u8Pixel;

        SC_W2BYTEMSK(0, REG_SC_BK10_08_L, u16HDE_Start , 0x3FFF);  // Display H start
        SC_W2BYTEMSK(0, REG_SC_BK10_09_L, u16HDE_End   , 0x3FFF);  // Display H end
        SC_W2BYTEMSK(0, REG_SC_BK10_0A_L, u16VDE_Start , 0x3FFF);  // Display V start
        SC_W2BYTEMSK(0, REG_SC_BK10_0B_L, u16VDE_End   , 0x3FFF);  // Display V end

        psep = strsep(&pcCmdLine,pch);    //find keyword "Color"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //Color value
        if(psep == NULL)
        {
            return;
        }
        u8Color = (MS_U8)atoi(psep);
        switch(u8Color)
        {
            case 0:    //White
                SC_W2BYTEMSK(0, REG_SC_BK10_19_L, 0xFF00 , 0xFF00);  // Red
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x00FF , 0x00FF);  // Green
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0xFF00 , 0xFF00);  // Blue
                break;
            case 1:    //Red
                SC_W2BYTEMSK(0, REG_SC_BK10_19_L, 0xFF00 , 0xFF00);  // Red
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x0000 , 0x00FF);  // Green
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x0000 , 0xFF00);  // Blue
                break;
            case 2:    //Green
                SC_W2BYTEMSK(0, REG_SC_BK10_19_L, 0x0000 , 0xFF00);  // Red
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x00FF , 0x00FF);  // Green
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x0000 , 0xFF00);  // Blue
                break;
            case 3:    //Blue
                SC_W2BYTEMSK(0, REG_SC_BK10_19_L, 0x0000 , 0xFF00);  // Red
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0x0000 , 0x00FF);  // Green
                SC_W2BYTEMSK(0, REG_SC_BK10_1A_L, 0xFF00 , 0xFF00);  // Blue
                break;
            default:
                break;
        }
        MdbPrint(paraCmdLine->u64ReqHdl,"vop_border width=%d, color=%d\n", u8Pixel, u8Color);
    }
    // echo lpll_set=[value] freerun=1 > /proc/utopia_mdb/pnl
    else if(strcmp("lpll_set", psep) == 0)
    {
        MS_U32  u32Set     = 0;
        MS_U32  u32Set_pre = 0;
        MS_BOOL bFreeRun   = FALSE;
        psep = strsep(&pcCmdLine,pch);    //lpll_set value
        if(psep == NULL)
        {
            return;
        }
        u32Set = (MS_U32)atoi(psep);

        psep = strsep(&pcCmdLine,pch);    //find keyword "freerun"
        if(psep == NULL)
        {
            return;
        }

        psep = strsep(&pcCmdLine,pch);    //freerun value
        if(psep == NULL)
        {
            return;
        }
        bFreeRun = (MS_BOOL)atoi(psep);

        u32Set_pre = R4BYTE(L_BK_LPLL(0x0F));
        if(bFreeRun==TRUE)
        {
            W2BYTEMSK(L_BK_LPLL(0x0C), 0, BIT(3));
        }
        W4BYTE(L_BK_LPLL(0x0F), u32Set);
        MdbPrint(paraCmdLine->u64ReqHdl,"LPLL set from 0x%x to 0x%x\n", u32Set_pre, u32Set);
    }
    // echo vop_set htt=[value] vtt=[value] > /proc/utopia_mdb/pnl
    else if(strcmp("vop_set", psep) == 0)
    {
        MS_U16 u16Htt     = 0;
        MS_U16 u16Vtt     = 0;
        MS_U16 u16Htt_pre = 0;
        MS_U16 u16Vtt_pre = 0;
        psep = strsep(&pcCmdLine,pch);    //keyword "htt"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //htt value
        if(psep == NULL)
        {
            return;
        }
        u16Htt = (MS_U16)atoi(psep);
        psep = strsep(&pcCmdLine,pch);    //keyword "vtt"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //vtt value
        if(psep == NULL)
        {
            return;
        }
        u16Vtt = (MS_U16)atoi(psep);

        u16Htt_pre = SC_R2BYTEMSK(0, REG_SC_BK10_0C_L , 0x3FFF);
        u16Vtt_pre = SC_R2BYTEMSK(0, REG_SC_BK10_0D_L , 0x3FFF);
        SC_W2BYTEMSK(0, REG_SC_BK10_0C_L, u16Htt , 0x3FFF);  // Htotal
        SC_W2BYTEMSK(0, REG_SC_BK10_0D_L, u16Vtt , 0x3FFF);  // Vtotal
        MdbPrint(paraCmdLine->u64ReqHdl,"Htotal set from %d to %d\n", u16Htt_pre, u16Htt);
        MdbPrint(paraCmdLine->u64ReqHdl,"Vtotal set from %d to %d\n", u16Vtt_pre, u16Vtt);
    }
    //echo free_swap CH3210=[value] CH7654=[value] CHba98=[value] CHfedc=[value]> /proc/utopia_mdb/pnl
    else if(strcmp("free_swap", psep) == 0)
    {
        MS_U16 u16CH3210     = 0;
        MS_U16 u16CH7654     = 0;
        MS_U16 u16CHba98     = 0;
        MS_U16 u16CHfedc     = 0;
        MS_U16 u16CH3210_pre = 0;
        MS_U16 u16CH7654_pre = 0;
        MS_U16 u16CHba98_pre = 0;
        MS_U16 u16CHfedc_pre = 0;
        psep = strsep(&pcCmdLine,pch);    //find keyword "CH3210"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CH3210 value
        if(psep == NULL)
        {
            return;
        }
        u16CH3210 = (MS_U16)atoi(psep);
        u16CH3210_pre = MOD_R2BYTEMSK(REG_MOD_BK00_08_L, 0xFFFF);
        MOD_W2BYTEMSK(REG_MOD_BK00_08_L, u16CH3210, 0xFFFF);
        MdbPrint(paraCmdLine->u64ReqHdl,"Free swap CH3210 set from 0x%x to 0x%x\n", u16CH3210_pre, u16CH3210);

        psep = strsep(&pcCmdLine,pch);    //find keyword "CH7654"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CH7654 value
        if(psep == NULL)
        {
            return;
        }
        u16CH7654 = (MS_U16)atoi(psep);
        u16CH7654_pre = MOD_R2BYTEMSK(REG_MOD_BK00_09_L, 0xFFFF);
        MOD_W2BYTEMSK(REG_MOD_BK00_09_L, u16CH7654, 0xFFFF);
        MdbPrint(paraCmdLine->u64ReqHdl,"Free swap CH7654 set from 0x%x to 0x%x\n", u16CH7654_pre, u16CH7654);

        psep = strsep(&pcCmdLine,pch);    //find keyword "CHba98"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CHba98 value
        if(psep == NULL)
        {
            return;
        }
        u16CHba98 = (MS_U16)atoi(psep);
        u16CHba98_pre = MOD_R2BYTEMSK(REG_MOD_BK00_0A_L, 0xFFFF);
        MOD_W2BYTEMSK(REG_MOD_BK00_0A_L, u16CHba98, 0xFFFF);
        MdbPrint(paraCmdLine->u64ReqHdl,"Free swap CHba98 set from 0x%x to 0x%x\n", u16CHba98_pre, u16CHba98);

        psep = strsep(&pcCmdLine,pch);    //find keyword "CHfedc"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CHfedc value
        if(psep == NULL)
        {
            return;
        }
        u16CHfedc = (MS_U16)atoi(psep);
        u16CHfedc_pre = MOD_R2BYTEMSK(REG_MOD_BK00_0B_L, 0xFFFF);
        MOD_W2BYTEMSK(REG_MOD_BK00_0B_L, u16CHfedc, 0x00FF);
        MdbPrint(paraCmdLine->u64ReqHdl,"Free swap CHfedc set from 0x%x to 0x%x\n", u16CHfedc_pre, u16CHfedc);
    }
    // echo pn_swap_ch=Var1 > /proc/utopia_mdb/pnl
    else if(strcmp("pn_swap_ch", psep) == 0)
    {
        MS_U16 u16PNswap_ch  = 0;
        MS_U16 u16PNswap_pre = 0;
        psep = strsep(&pcCmdLine,pch);    //pn_swap_ch value
        if(psep == NULL)
        {
            return;
        }
        u16PNswap_ch = (MS_U16)atoi(psep);
        u16PNswap_pre = MOD_A_R2BYTEMSK(REG_MOD_A_BK00_31_L, 0x3FFF);
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_31_L, u16PNswap_ch, 0x3FFF);
        MdbPrint(paraCmdLine->u64ReqHdl,"PN swap for CH set from 0x%x to 0x%x\n", u16PNswap_pre, u16PNswap_ch);
    }
    // echo pn_swap=1 > /proc/utopia_mdb/pnl
    else if(strcmp("pn_swap", psep) == 0)
    {
        MS_BOOL bPnSwap = FALSE;
        psep = strsep(&pcCmdLine,pch);    //pn_swap value
        if(psep == NULL)
        {
            return;
        }
        bPnSwap = (MS_BOOL)atoi(psep);
        MOD_W2BYTEMSK(REG_MOD_BK00_40_L, (bPnSwap==TRUE)?BIT(5):0, BIT(5));
    }
    else if(strcmp("port_swap", psep) == 0)
    {
        MS_BOOL bPortSwap = FALSE;
        psep = strsep(&pcCmdLine,pch);    //port_swap value
        if(psep == NULL)
        {
            return;
        }
        bPortSwap = (MS_BOOL)atoi(psep);
        MOD_W2BYTEMSK(REG_MOD_BK00_4A_L, (bPortSwap==TRUE)?BIT(0):0, BIT(0));
    }
    else if(strcmp("ch_swap", psep) == 0)
    {
        MS_BOOL bChSwap = FALSE;
        psep = strsep(&pcCmdLine,pch);    //ch_swap value
        if(psep == NULL)
        {
            return;
        }
        bChSwap = (MS_BOOL)atoi(psep);
        MOD_W2BYTEMSK(REG_MOD_BK00_40_L, (bChSwap==TRUE)?BIT(6):0, BIT(6));
    }
    else if(strcmp("dual_port", psep) == 0)
    {
        MS_BOOL bDualPort = FALSE;
        psep = strsep(&pcCmdLine,pch);    //bDualPort value
        if(psep == NULL)
        {
            return;
        }
        bDualPort = (MS_BOOL)atoi(psep);
        SC_W2BYTEMSK(0,REG_SC_BK10_21_L, (bDualPort==TRUE)?BIT(8):0, BIT(8));
    }
    else if(strcmp("ti_mode", psep) == 0)
    {
        MS_BOOL bTiMode = FALSE;
        psep = strsep(&pcCmdLine,pch);    //ti_mode value
        if(psep == NULL)
        {
            return;
        }
        bTiMode = (MS_BOOL)atoi(psep);
        MOD_W2BYTEMSK(REG_MOD_BK00_40_L, (bTiMode==TRUE)?BIT(2):0, BIT(2));
    }
    //echo outconf CH7to0=[value] CHfto8=[value] > /proc/utopia_mdb/pnl
    else if(strcmp("outconf", psep) == 0)
    {
        MS_U16 u16CH7to0 = 0;
        MS_U16 u16CHfto8 = 0;
        psep = strsep(&pcCmdLine,pch);    //find keyword "CH7to0"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CH7to0 value
        if(psep == NULL)
        {
            return;
        }
        u16CH7to0 = (MS_U16)atoi(psep);
        MOD_A_W2BYTE(REG_MOD_A_BK00_00_L, u16CH7to0);

        psep = strsep(&pcCmdLine,pch);    //find keyword "CHfto8"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    // CHfto8 value
        if(psep == NULL)
        {
            return;
        }
        u16CHfto8 = (MS_U16)atoi(psep);
        MOD_A_W2BYTE(REG_MOD_A_BK00_01_L, u16CHfto8);
    }
    //echo mod_gpio pad=27 oen=0 out=1 > /proc/utopia_mdb/pnl
    else if(strcmp("mod_gpio", psep) == 0)
    {
        //GPIO num
        //Outconf
        //OEN
        //GPO Sel/Detain
        MS_BOOL  bOen       = FALSE;
        MS_BOOL  bOutValue  = FALSE;
        MS_BOOL  bInValue   = FALSE;
        MS_U8    u8PadNum   = 0;
        MS_U8    u8Reg      = 0;
        MS_U8    u8PadShift = 0;

        psep = strsep(&pcCmdLine,pch);    //find keyword "pad"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //pad number
        if(psep == NULL)
        {
            return;
        }
        u8PadNum = (MS_U8)atoi(psep);
        if(u8PadNum < 16)
        {
            u8Reg = 0;
            u8PadShift = u8PadNum;
        }
        else if(u8PadNum < 32)
        {
            u8Reg = 1;
            u8PadShift = u8PadNum - 16;
        }
        else
        {
            return;
        }
        MdbPrint(paraCmdLine->u64ReqHdl,"MOD u8Reg=0x%x, u8PadShift=%d\n", u8Reg, u8PadShift);

        psep = strsep(&pcCmdLine,pch);    //find keyword "oen"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //oen value
        if(psep == NULL)
        {
            return;
        }
        bOen = (MS_BOOL)atoi(psep);

        if(bOen == TRUE)        //Input
        {
            if(u8Reg == 0)
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0, BIT((u8PadShift>>1)*2 +1)|BIT((u8PadShift>>1)*2));    //outputconf
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_40_L,BIT(u8PadShift), BIT(u8PadShift));    // ext_data_en=1, for GPIO function
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_42_L,BIT(u8PadShift), BIT(u8PadShift));    // oez
                bInValue = (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_46_L,BIT(u8PadShift)))>>u8PadShift;
            }
            else
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_01_L, 0, BIT((u8PadShift>>1)*2 +1)|BIT((u8PadShift>>1)*2));    //outputconf
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_41_L,BIT(u8PadShift), BIT(u8PadShift));    // ext_data_en=1, for GPIO function
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_43_L,BIT(u8PadShift), BIT(u8PadShift));    // oez
                bInValue = (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_47_L,BIT(u8PadShift)))>>u8PadShift;
            }
            MdbPrint(paraCmdLine->u64ReqHdl,"MOD pad[%d] GPI = %d\n", u8PadNum, bInValue);
        }
        else                //Output
        {
            psep = strsep(&pcCmdLine,pch);    //find keyword "out"
            if(psep == NULL)
            {
                return;
            }
            psep = strsep(&pcCmdLine,pch);    //out value
            if(psep == NULL)
            {
                return;
            }
            bOutValue = (MS_BOOL)atoi(psep);

            if(u8Reg == 0)
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_00_L, 0, BIT((u8PadShift>>1)*2 +1)|BIT((u8PadShift>>1)*2));    //outputconf
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_40_L,BIT(u8PadShift), BIT(u8PadShift));    // ext_data_en=1, for GPIO function
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_42_L,0, BIT(u8PadShift));    // oez
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_44_L,(bOutValue==TRUE)?BIT(u8PadShift):0, BIT(u8PadShift));    // gpo_detain
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_48_L,BIT(u8PadShift), BIT(u8PadShift));    // gpo_sel
            }
            else
            {
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_01_L, 0, BIT((u8PadShift>>1)*2 +1)|BIT((u8PadShift>>1)*2));
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_41_L,BIT(u8PadShift), BIT(u8PadShift));    // ext_data_en=1, for GPIO function
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_43_L,0, BIT(u8PadShift));    // oez
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_45_L,(bOutValue==TRUE)?BIT(u8PadShift):0, BIT(u8PadShift));    // gpo_detain
                MOD_A_W2BYTEMSK(REG_MOD_A_BK00_49_L,BIT(u8PadShift), BIT(u8PadShift));    // gpo_sel
            }
            MdbPrint(paraCmdLine->u64ReqHdl,"MOD pad[%d] GPO = %d\n", u8PadNum, bOutValue);
        }
    }
    //echo swing=[value] > /proc/utopia_mdb/pnl
    else if(strcmp("swing", psep) == 0)
    {
        MS_U16 u16Swing = 0;
        psep = strsep(&pcCmdLine,pch);    //swing value
        if(psep == NULL)
        {
            return;
        }
        u16Swing = (MS_U16)atoi(psep);
        MHal_PNL_MOD_Control_Out_Swing(pInstance,u16Swing);
        MdbPrint(paraCmdLine->u64ReqHdl,"Swing control : 0x%x\n", u16Swing);
    }
    //echo preem en=1 value=[value] > /proc/utopia_mdb/pnl
    else if(strcmp("preem", psep) == 0)
    {
        MS_BOOL bEnable  = FALSE;
        MS_U16  u16Value = 0;
        psep = strsep(&pcCmdLine,pch);    //find keyword "en"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //en value
        if(psep == NULL)
        {
            return;
        }
        bEnable = (MS_BOOL)atoi(psep);
        if(bEnable)
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_30_L,0x3FF, 0x3FF);
        else
            MOD_A_W2BYTEMSK(REG_MOD_A_BK00_30_L,0, 0x3FF);

        psep = strsep(&pcCmdLine,pch);    //find keyword "value"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //value
        if(psep == NULL)
        {
            return;
        }
        u16Value = (MS_U16)atoi(psep);

        MHal_PNL_MOD_Control_Out_PE_Current(pInstance,u16Value);
    }
    //echo pnl_en=1 > /proc/utopia_mdb/pnl
    else if(strcmp("pnl_en", psep) == 0)
    {
        MS_BOOL bEnable = TRUE;
        psep = strsep(&pcCmdLine,pch);    //pnl_en value
        if(psep == NULL)
        {
            return;
        }
        bEnable = (MS_BOOL)atoi(psep);
        MApi_PNL_EnablePanel(bEnable);
        MdbPrint(paraCmdLine->u64ReqHdl,"MApi_PNL_EnablePanel : %d\n", bEnable);
    }
    //echo ssc en=1 dev=100 mod=200 > /proc/utopia_mdb/pnl
    else if(strcmp("ssc", psep) == 0)
    {
        MS_BOOL bEnable = FALSE;
        MS_U16  u16Mod  = 0;
        MS_U16  u16Dev  = 0;
        psep = strsep(&pcCmdLine,pch);    //find keyword "en"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //en value
        if(psep == NULL)
        {
            return;
        }
        bEnable = (MS_BOOL)atoi(psep);
        MApi_PNL_SetSSC_En_U2(pInstance,bEnable);
        MdbPrint(paraCmdLine->u64ReqHdl,"MApi_PNL_SetSSC_En : %d\n", bEnable );

        psep = strsep(&pcCmdLine,pch);    //find keyword "mod"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //mod value
        if(psep == NULL)
        {
            return;
        }
        u16Mod = (MS_U16)atoi(psep);
        MApi_PNL_SetSSC_Fmodulation_U2(pInstance,u16Mod);
        MdbPrint(paraCmdLine->u64ReqHdl,"MApi_PNL_SetSSC_Fmodulation : %d\n", u16Mod );

        psep = strsep(&pcCmdLine,pch);    //find keyword "dev"
        if(psep == NULL)
        {
            return;
        }
        psep = strsep(&pcCmdLine,pch);    //dev value
        if(psep == NULL)
        {
            return;
        }
        u16Dev = (MS_U16)atoi(psep);
        MApi_PNL_SetSSC_Rdeviation_U2(pInstance,u16Dev);
        MdbPrint(paraCmdLine->u64ReqHdl,"MApi_PNL_SetSSC_Rdeviation : %d\n", u16Dev );
    }
    //echo lpll_dump > /proc/utopia_mdb/pnl
    else if(strcmp("lpll_dump", psep) == 0)
    {
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_mpll_loop_div_first            : 2'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x6D),(BIT(7)|BIT(6))))>>6);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_mpll_test[5]                   : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x6E),BIT(5)))>>5);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_ibias_ictrl              : 3'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x03),(BIT(4)|BIT(3)|BIT(2))))>>2);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_input_div_first          : 2'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x15),(BIT(1)|BIT(0)))));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_loop_div_first           : 2'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x01),(BIT(1)|BIT(0)))));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_loop_div_second          : 5'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x01),0x1F00))>>8);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_output_div_second[13:12] : 2'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x02),(BIT(13)|BIT(12))))>>12);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_output_div_second[11:8]  : 4'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x02),0x0F00))>>8);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_skew_div                 : 3'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x35),(BIT(14)|BIT(13)|BIT(12))))>>12);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_fifo_div                 : 3'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),(BIT(2)|BIT(1)|BIT(0)))));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_fifo_div5_en             : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x03),BIT(11)))>>11);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_en_fix_clk               : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),BIT(12)))>>12);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll_2ndpll_clk_sel            : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x35),BIT(5)))>>5);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[19]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x37),BIT(3)))>>3);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[20]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x37),BIT(4)))>>4);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_dual_lp_en               : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),BIT(15)))>>15);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_sdiv3p5_en               : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x04),BIT(13)))>>13);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_sdiv2p5_en               : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x03),BIT(10)))>>10);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_en_mini                  : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),BIT(14)))>>14);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_en_fifo                  : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),BIT(6)))>>6);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll_en_scalar                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x2E),BIT(4)))>>4);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[18]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x37),BIT(2)))>>2);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[15]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x36),BIT(15)))>>15);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[16]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x37),BIT(0))));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_test[22]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x37),BIT(6)))>>6);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_pd                       : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x33),BIT(5)))>>5);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_ibias_ictrl              : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x33),BIT(2)))>>2);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_input_div_first          : 5'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x30),0x001F)));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_loop_div_first           : 2'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x31),(BIT(1)|BIT(0)))));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_loop_div_second          : 5'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x31),0x1F00)>>8));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_output_div_first         : 4'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x32),0x000F)));
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_test[11]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x39),BIT(11)))>>11);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_test[12]                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x39),BIT(12)))>>12);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_test[8]                  : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x39),BIT(8)))>>8);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll2_div14_en                 : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x33),BIT(7)))>>7);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_scalar2fifo_en           : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x38),BIT(9)))>>9);
        MdbPrint(paraCmdLine->u64ReqHdl,"reg_lpll1_scalar2fifo_div2         : 1'd%d\n", (R2BYTEMSK(L_BK_LPLL(0x38),BIT(8)))>>8);
    }
    //echo DbgLevel=0x0003 > /proc/utopia_mdb/pnl
    else if(strcmp("DbgLevel", psep) == 0)
    {
        MS_U16 u16DbgLvl = 0;
        psep = strsep(&pcCmdLine,pch);    //DbgLevel value
        if(psep == NULL)
        {
            return;
        }
        u16DbgLvl = (MS_U16)atoi(psep);
        MApi_PNL_SetDbgLevel_U2(pInstance,u16DbgLvl);
        MdbPrint(paraCmdLine->u64ReqHdl,"MApi_PNL_SetDbgLevel : 0x%x\n", u16DbgLvl );
    }
    else if(strcmp("help", psep) == 0) //help
    {
        _MHal_PNL_MdbIoctl_Echo_Help(paraCmdLine);
    }
}

void SYMBOL_WEAK MHal_PNL_MdbIoctl_EchoCmd(MDBCMD_CMDLINE_PARAMETER *paraCmdLine)
{
    void *pInstance=NULL;
    pInstance = UtopiaModuleGetLocalInstantList(MODULE_PNL, pInstance);

    _MHal_PNL_MdbIoctl_EchoCmd(pInstance,paraCmdLine);
}
void SYMBOL_WEAK MHal_PNL_MdbIoctl_Cat(MDBCMD_GETINFO_PARAMETER *paraGetInfo)
{
    void *pInstance=NULL;
    pInstance = UtopiaModuleGetLocalInstantList(MODULE_PNL, pInstance);
    _MHal_PNL_Mdbloctl_Cat_MOD(pInstance, paraGetInfo);
    _MHal_PNL_Mdbloctl_Cat_PNL(pInstance, paraGetInfo);
}
#endif    //CONFIG_UTOPIA_PROC_DBG_SUPPORT

static MS_BOOL _MHal_PNL_DumpBiasConFromEfuse(void *pInstance)
{
    MS_U8 u8BiasCon = 0;
    MS_BOOL bReadSingleTerm = FALSE;
    MS_U32 u32EfuseData = 0;

    if (MOD_A_R2BYTEMSK(REG_MOD_A_BK00_76_L, BIT(15)) == BIT(15))
    {
        MsOS_DelayTaskUs(11); //wait 11 us for detecting.
        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_76_L, HW_RINT_THRESHOLD, 0x7FFF);

        if (MOD_A_R2BYTE(REG_MOD_A_BK00_77_L) > HW_RINT_THRESHOLD)
        {
            bReadSingleTerm = FALSE;
        }
        else
        {
            bReadSingleTerm = TRUE;
        }

        u32EfuseData = msReadEfuse(pInstance, EFUSE_MOD_ATOP_SUBBANK, 0xFFFFFFFF);

        if (bReadSingleTerm == TRUE)
        {
            if ((u32EfuseData & BIT(14)) == BIT(14)) //Check activate flag for Single term.
            {
                u8BiasCon = (u32EfuseData >> 7)&0x7F; // BIT [13:7] =>Bias con
            }
            else
            {
                return FALSE;
            }
        }
        else
        {
            if ((u32EfuseData & BIT(22)) == BIT(22)) //Check activate flag for Double term.
            {
                u8BiasCon = (u32EfuseData >> 15)&0x7F; // BIT [21:15] =>Bias con
            }
            else
            {
                return FALSE;
            }
        }

        MOD_A_W2BYTEMSK(REG_MOD_A_BK00_37_L, u8BiasCon,0x7F); //[6:0]Control swing of bias con
    }

    return TRUE;
}

void MHal_PNL_EfuseMPLL(MS_BOOL bEnable)
{
    if(bEnable == TRUE) //old mode
    {
        W2BYTEMSK(L_BK_LPLL(0x6E), BIT(8), BIT(8));//TEST_MPLL[8] = 1, 0x1031_6e[8]
        W2BYTEMSK(L_BK_LPLL(0x6D), BIT(6), (BIT(7)|BIT(6)));//MPLL_LOOPDIV_FIRST[7:6] = 2'b01, 0x1031_6d[7:6]
    }
    else
    {
        W2BYTEMSK(L_BK_LPLL(0x6E), 0x0, BIT(8));//TEST_MPLL[8] = 0, 0x1031_6e[8]
        W2BYTEMSK(L_BK_LPLL(0x6D), BIT(7), (BIT(7)|BIT(6)));//MPLL_LOOPDIV_FIRST[7:6] = 2'b10, 0x1031_6d[7:6]
    }
}

MS_BOOL Hal_PNL_TestAutodownload_OD(void* pInstance, EN_PNL_TEST_STATUS* penTestReport)
{

    PNL_INSTANCE_PRIVATE *pPNLInstancePrivate = NULL;
    UtopiaInstanceGetPrivate(pInstance, (void**)&pPNLInstancePrivate);

    //od init
    //parse curve, write ADL, and ADL fire
    MHal_PNL_OverDriver_Setting_TBL(pInstance,u16ODCurve,OD_CURVE_GOLDEN_ENTRY_NUM,E_PNL_OVERDRIVER_SEPARATE_W_ENABLE);
    MHal_PNL_OverDriver_Setting_TBL(pInstance,u16ODCurve,OD_CURVE_GOLDEN_ENTRY_NUM,E_PNL_OVERDRIVER_SEPARATE_B_ENABLE);
    MHal_PNL_OverDriver_Setting_TBL(pInstance,u16ODCurve,OD_CURVE_GOLDEN_ENTRY_NUM,E_PNL_OVERDRIVER_SEPARATE_G_ENABLE);
    MHal_PNL_OverDriver_Setting_TBL(pInstance,u16ODCurve,OD_CURVE_GOLDEN_ENTRY_NUM,E_PNL_OVERDRIVER_SEPARATE_R_ENABLE);

    //compare golden
    MsOS_DelayTask(50);
    MS_U16 u16Count = 0;
    MS_U16 u16TableCount = 0;
    MS_U8 u8ReadData = 0;
    //SRAM1
    MS_U16 u16CodeCurrentTableIndex = 0;
    MS_U16 u16TargetIndex = 0;
    MS_U16 u16Target = 0;
    MS_U8* pu8OD_SRAM1 = NULL;
    MS_U8* pu8OD_SRAM2 = NULL;
    MS_U8* pu8OD_SRAM3 = NULL;
    MS_U8* pu8OD_SRAM4 = NULL;
    MS_BOOL bRet = TRUE;

    PNL_MALLOC_MEM(pu8OD_SRAM1, OVERDRIVER_SRAM1_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM2, OVERDRIVER_SRAM2_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM3, OVERDRIVER_SRAM3_SIZE, bRet);
    PNL_MALLOC_MEM(pu8OD_SRAM4, OVERDRIVER_SRAM4_SIZE, bRet);

    if(pu8OD_SRAM1 != NULL && pu8OD_SRAM2 != NULL && pu8OD_SRAM3 != NULL && pu8OD_SRAM4 != NULL)
    {
        //Decrypt OD table.
        u16TargetIndex = OD_DECRYPTION_IDX_1;
        u16Target = u16ODCurve[u16TargetIndex];
        for(u16Count = 0; u16Count < OVERDRIVER_SRAM1_SIZE; u16Count++)
        {
            if(u16Count == u16TargetIndex)
            {
                pu8OD_SRAM1[u16Count] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM1[u16Count] = (MS_U8)(u16ODCurve[u16Count]^u16Target);
            }
        }


        u16CodeCurrentTableIndex += OVERDRIVER_SRAM1_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = u16ODCurve[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16Count = 0; u16Count < OVERDRIVER_SRAM2_SIZE; u16Count++)
        {
            if(u16Count == u16TargetIndex)
            {
                pu8OD_SRAM2[u16Count] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM2[u16Count] = (MS_U8)(u16ODCurve[(u16CodeCurrentTableIndex+u16Count)]^u16Target);
            }
        }

        u16CodeCurrentTableIndex += OVERDRIVER_SRAM2_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = u16ODCurve[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16Count = 0; u16Count < OVERDRIVER_SRAM3_SIZE; u16Count++)
        {
            if(u16Count == u16TargetIndex)
            {
                pu8OD_SRAM3[u16Count] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM3[u16Count] = (MS_U8)(u16ODCurve[(u16CodeCurrentTableIndex+u16Count)]^u16Target);
            }
        }

        u16CodeCurrentTableIndex += OVERDRIVER_SRAM3_SIZE;
        u16TargetIndex += OD_DECRYPTION_IDX_2;
        u16Target = u16ODCurve[(u16CodeCurrentTableIndex+u16TargetIndex)];
        for(u16Count = 0; u16Count < OVERDRIVER_SRAM4_SIZE; u16Count++)
        {
            if(u16Count == u16TargetIndex)
            {
                pu8OD_SRAM4[u16Count] = (MS_U8)u16Target;
            }
            else
            {
                pu8OD_SRAM4[u16Count] = (MS_U8)(u16ODCurve[(u16CodeCurrentTableIndex+u16Count)]^u16Target);
            }
        }
     }



    *penTestReport = E_PNL_TEST_PASS;
    for(u16Count=0;u16Count<OVERDRIVER_SRAM1_SIZE;u16Count++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, E_PNL_OVERDRIVER_SEPARATE_R_ENABLE, 0x0F);

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, u16Count, 0x01FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_02_L, BIT(14), BIT(14));
        u8ReadData = (MS_U8)SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_04_L, 0xFF);

        if(u16Count>=OD_CURVE_GOLDEN_ENTRY_NUM)
        {
            *penTestReport = E_PNL_TEST_FAIL;
            PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM1 out of golden array  [%s][%d]\n", __func__, __LINE__);
            break;
        }
        else
        {
            if( u8ReadData !=  pu8OD_SRAM1[u16Count] )
            {
                *penTestReport = E_PNL_TEST_FAIL;
                PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM1 entry=%d  Curve=0x%x RIU=0x%x  [%s][%d]\n",u16Count,pu8OD_SRAM1[u16Count],u8ReadData , __func__, __LINE__);
            }
        }

    }


    for(u16Count=0;u16Count<OVERDRIVER_SRAM2_SIZE;u16Count++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, E_PNL_OVERDRIVER_SEPARATE_G_ENABLE, 0x0F);

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, u16Count, 0x01FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_05_L, BIT(14), BIT(14));
        u8ReadData = (MS_U8)SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_07_L, 0xFF);

        if(u16Count>=OD_CURVE_GOLDEN_ENTRY_NUM)
        {
            *penTestReport = E_PNL_TEST_FAIL;
            PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM2 out of golden array  [%s][%d]\n", __func__, __LINE__);
            break;
        }
        else
        {
            if( u8ReadData !=  pu8OD_SRAM2[u16Count] )
            {
                *penTestReport = E_PNL_TEST_FAIL;
                PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM2 entry=%d  Curve=0x%x RIU=0x%x  [%s][%d]\n",u16Count,pu8OD_SRAM2[u16Count],u8ReadData , __func__, __LINE__);
            }
        }

    }

    for(u16Count=0;u16Count<OVERDRIVER_SRAM3_SIZE;u16Count++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, E_PNL_OVERDRIVER_SEPARATE_B_ENABLE, 0x0F);

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, u16Count, 0x01FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_08_L, BIT(14), BIT(14));
        u8ReadData = (MS_U8)SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0A_L, 0xFF);

        if(u16Count>=OD_CURVE_GOLDEN_ENTRY_NUM)
        {
            *penTestReport = E_PNL_TEST_FAIL;
            PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM3 out of golden array  [%s][%d]\n", __func__, __LINE__);
            break;
        }
        else
        {
            if( u8ReadData !=  pu8OD_SRAM3[u16Count] )
            {
                *penTestReport = E_PNL_TEST_FAIL;
                PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM3 entry=%d  Curve=0x%x RIU=0x%x  [%s][%d]\n",u16Count,pu8OD_SRAM3[u16Count],u8ReadData , __func__, __LINE__);
            }
        }

    }

    for(u16Count=0;u16Count<OVERDRIVER_SRAM4_SIZE;u16Count++)
    {
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_01_L, E_PNL_OVERDRIVER_SEPARATE_W_ENABLE, 0x0F);

        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, u16Count, 0x01FF);
        SC_W2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0B_L, BIT(14), BIT(14));
        u8ReadData = (MS_U8)SC_R2BYTEMSK(pPNLInstancePrivate->u32DeviceID, REG_SC_BK16_0D_L, 0xFF);

        if(u16Count>=OD_CURVE_GOLDEN_ENTRY_NUM)
        {
            *penTestReport = E_PNL_TEST_FAIL;
            PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM4 out of golden array  [%s][%d]\n", __func__, __LINE__);
            break;
        }
        else
        {
            if( u8ReadData !=  pu8OD_SRAM4[u16Count] )
            {
                *penTestReport = E_PNL_TEST_FAIL;
                PNL_TEST_LOG("PNL_TEST_LOG: OD SRAM4 entry=%d  Curve=0x%x RIU=0x%x  [%s][%d]\n",u16Count,pu8OD_SRAM4[u16Count],u8ReadData , __func__, __LINE__);
            }
        }

    }


    PNL_FREE_MEM(pu8OD_SRAM1, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM2, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM3, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));
    PNL_FREE_MEM(pu8OD_SRAM4, OVERDRIVER_SRAM1_SIZE * sizeof(MS_U8));


    return TRUE;


}

MS_BOOL Hal_PNL_TestAutodownload_PanelGamma(void* pInstance, EN_PNL_TEST_STATUS* penTestReport)
{
    if(penTestReport == NULL)
    {
        return FALSE;
    }
    *penTestReport = E_PNL_TEST_PASS;

    //parse the golden curve
    //write autodownload
    ST_PNL_PNLGAMMATBL stSetPNLGammaTbl;
    stSetPNLGammaTbl.u32TblSize = PANEL_GAMMA_ENTRY;
    stSetPNLGammaTbl.pu16RChannel0 = u16PanelGamma_R0;
    stSetPNLGammaTbl.pu16GChannel0 = u16PanelGamma_G0;
    stSetPNLGammaTbl.pu16BChannel0 = u16PanelGamma_B0;
    stSetPNLGammaTbl.pu16WChannel0 = u16PanelGamma_W0;

    if( _MHal_PNL_ADL_SetPanelGammaTbl(pInstance, &stSetPNLGammaTbl) == TRUE )
    {
        //use ADL to set panel gamma table, use ML to set panel gamma enable
        //but ML trigger early ADL so that it shows garbage in AC on
        //so delay 1 vsync for ML
        MsOS_DelayTask(20);
    }

    //get curve from SRAM
    static MS_U16 u16RCurve[PANEL_GAMMA_ENTRY]={0};
    static MS_U16 u16GCurve[PANEL_GAMMA_ENTRY]={0};
    static MS_U16 u16BCurve[PANEL_GAMMA_ENTRY]={0};
    static MS_U16 u16WCurve[PANEL_GAMMA_ENTRY]={0};
    static MS_U16 u16Channel1[PANEL_GAMMA_ENTRY]={0};
    ST_PNL_PNLGAMMATBL stGetPNLGammaTbl;
    stGetPNLGammaTbl.pu16RChannel0 = u16RCurve;
    stGetPNLGammaTbl.pu16GChannel0 = u16GCurve;
    stGetPNLGammaTbl.pu16BChannel0 = u16BCurve;
    stGetPNLGammaTbl.pu16WChannel0 = u16WCurve;

    stGetPNLGammaTbl.pu16RChannel1 = u16Channel1;
    stGetPNLGammaTbl.pu16GChannel1 = u16Channel1;
    stGetPNLGammaTbl.pu16BChannel1 = u16Channel1;
    stGetPNLGammaTbl.pu16WChannel1 = u16Channel1;

    MS_U8 u8CurveNum = 0;
    MS_U16 u16EntryNum = 0;
    for(u8CurveNum=0;u8CurveNum<4;u8CurveNum++)
    {
         if( MHal_PNL_GetPanelGammaTbl_From_Reg(pInstance,&stGetPNLGammaTbl) )
         {
             MS_U16 *pu16Golden = NULL;
             MS_U16 *pu16RIUData = NULL;
             if(u8CurveNum == 0)
             {
                 pu16Golden = u16PanelGamma_R0;
                 pu16RIUData = stGetPNLGammaTbl.pu16RChannel0;
             }
             else if(u8CurveNum == 1)
             {
                 pu16Golden = u16PanelGamma_G0;
                 pu16RIUData = stGetPNLGammaTbl.pu16GChannel0;
             }
             else if(u8CurveNum == 2)
             {
                 pu16Golden = u16PanelGamma_B0;
                 pu16RIUData = stGetPNLGammaTbl.pu16BChannel0;
             }
             else
             {
                 pu16Golden = u16PanelGamma_W0;
                 pu16RIUData = stGetPNLGammaTbl.pu16WChannel0;
             }

             //compare
             for(u16EntryNum=0;u16EntryNum<1024;u16EntryNum++)
             {
                 if( *(pu16Golden+u16EntryNum) != *(pu16RIUData+u16EntryNum) )
                 {
                     *penTestReport = E_PNL_TEST_FAIL;
                     PNL_TEST_LOG("PNL_TEST_LOG: PanelGamma=%d entry=%d  Curve=0x%x RIU=0x%x  [%s][%d]\n",u8CurveNum,u16EntryNum,*(pu16Golden+u16EntryNum),*(pu16RIUData+u16EntryNum) , __func__, __LINE__);
                 }
             }
         }
         else
         {
             //get gamma fail
             *penTestReport = E_PNL_TEST_FAIL;
             return FALSE;
         }
    }

    return TRUE;
}

#endif
